#!@PYTHON@
# -*- coding: utf-8 -*-
#
# sysex.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

import os
import select
import signal
import sys
import time
import midi
from multiprocessing import Process,Queue
from Queue import Empty
from modeparam import modeparam_dict
from table1 import ProgramParameters
import sysex

class _Getch:
	"""Gets a single character from standard input.  Does not echo to the screen."""
	def __init__(self):
		try:
			self.impl = _GetchWindows()
		except ImportError:
			self.impl = _GetchUnix()

	def __call__(self):
		return self.impl()

class _GetchUnix:
	def __init__(self):
		import tty

	def __call__(self):
		import tty, termios
		fd = sys.stdin.fileno()
		print 'fd',fd
		old_settings = termios.tcgetattr(fd)
		try:
			tty.setraw(sys.stdin.fileno())
			ch = sys.stdin.read(1)
		finally:
			termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
		return ch

class _GetchWindows:
	def __init__(self):
		import msvcrt

	def __call__(self):
		import msvcrt
		return msvcrt.getch()


getch = _Getch()

test_data = (
		 75, 82, 79, 77, 69, 32, 71,114, 97,110,100, 32,
		 80,105, 97,110,111, 32, 32, 32, 32, 32, 32, 32,
		200,  0, 64,119,194,  0, 17,211, 97, 96, 48,102,
		 10,192,220,129,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  7, 16,  2,  0, 64,  0,127,127,
		  0,  0,  0,  0,200, 18,153,152, 12,  0,184, 72,
		 23,  0,  0,  0,  0, 64,240,  1,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0, 16,  3,  0,
		 64,  0,127,127,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0, 16,  4,  0, 64,  0,127,127,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0, 16,  5,  0, 64,  0,127,127,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0, 16,  0,  0,
		 64,  0,127,127,  0,  0,  0,  0,  0, 38, 25, 50,
		  9,  0, 70,202,240,249,176, 64,174,237,  4,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		 29, 80,  0,  0, 20, 20, 69, 80,226,  1,  0,  0,
		 60,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0, 84, 80,  0,  0,
		 90, 90,  0,127,200,  0,  0, 57,248,254, 97, 68,
		  1, 96,  0,120,  5,128, 98,225,  1,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  5, 80,  0,  0,
		127,  0,  0,  0, 25,  0,  0, 16,159, 15,  0,  4,
		 25,  0,  0, 16,159, 15,  0,  4,  9,  0,  0,  0,
		 41,  0,  0, 11,  0,127,  1,127,  0, 16, 16,  0,
		 60,117,  0,  0, 63,117,  0,  0, 67,117,  0,  0,
		 70,117,  0,  0,255,117,  0,  0,255,117,  0,  0,
		255,117,  0,  0,255,117,  0,  0,  0, 16,  0,  0,
		 62,117,  0,  0, 65,117,  0,  0, 69,117,  0,  0,
		 72,117,  0,  0,255,117,  0,  0,255,117,  0,  0,
		255,117,  0,  0,255,117,  0,  0,  0, 16,  0,  0,
		 65,117,  0,  0, 68,117,  0,  0, 72,117,  0,  0,
		 75,117,  0,  0,255,117,  0,  0,255,117,  0,  0,
		255,117,  0,  0,255,117,  0,  0,  0, 16,  0,  0,
		 67,117,  0,  0, 70,117,  0,  0, 74,117,  0,  0,
		 77,117,  0,  0,255,117,  0,  0,255,117,  0,  0,
		255,117,  0,  0,255,117,  0,  0,  0, 16,  0,  0,
		148, 42,  0,  0,  0,  0,  0,  0, 93,  0,  5, 24,
		101, 87,  0,127,  0,127,  1,  0, 99,  0, 97,  0,
		  6,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,
		  0,  0, 17,  0, 90,  0, 10,  0, 65, 72,  2,  0,
		  0, 92,127,  0,  0,  0,  0,  0,  0,  0,  3,  4,
		 16,  2,  3,  4, 99,  0, 83, 16,  4,  2,  0,  0,
		 48,  0,  0,  0, 65,  0,  0,  0, 63,  0,  0,  0,
		 58,  0,  0,  0,  5,  0,  0,  0,  6,  0,  0,  0,
		  7,  0,  0,  0,  8,  0,  0,  0, 52,  0, 12,  0,
		180,  0,  7,  0, 51,  0,  0,  0,179,  0, 10,  0,
		 53,  0,255,255,  1,  0, 10,  0,  2,  0, 40,  0,
		  3,  0, 10,  0,  0,127,  0,127,127,127,  1,  2,
		157, 30,  0, 30,  0, 30, 39,  0,  0,  0,  0,  0,
		 38,  0,  0,  0, 14,  0,  0,  0, 14,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,132,  1,  0,  0,
		 19,  0,  0,  0,  0,  0,  0,  0,  3, 39,  0,  0,
		  0,  0,  0,  0, 41, 55, 67, 45, 40,  0,  0,  0,
		 41, 60, 84,206,206, 50, 50,  0,  1,  3,  7,  0,
		127,  0,  0,  0,  1,  3,  6,  0,120,  0,  0,  0,
		  1,  3,  5,  0,102,  0,  0,  0,  1,  3,  4,  0,
		 87,  0,  0,  0,  1,  3,  3,  0, 76,  0,  0,  0,
		  1,  3,  2,  0, 70,  0,  0,  0,  1,  3,  1,  0,
		 67,  0,  0,  0,  1,  3,  0,  0, 67,  0,  0,  0,
		114,  0,  0,  0,100,  0,  0,  0, 86,  0,  0,  0,
		 72,  0,  0,  0, 58,  0,  0,  0, 44,  0,  0,  0,
		 30,  0,  0,  0,  0,  1,127,  0,131, 60,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,132,  1,  0,  0,
		 13,  0,  0,  0,  0,  0,  0,  0, 19,  0,  0,  0,
		  0, 68,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  6,  1,  0,  0, 13,  0,  0,  0,  0,  0,  0,  0,
		 19,  0,  0,  0, 17,  0,  0,  0, 20,  0,  0,  0,
		  8,  0,  0,  0, 17,  0,  0,  0,  0,  0,  0,  0,
		 10,  0,  0,  0,  0, 23,  0,  0,  0,  0,  0,  0,
		  0, 24,  8, 17,  6,  0,  0,  0, 14,  0,  0,  0,
		 20,  0,  0,  0, 20,254,  0,  0, 17,  0,  0,  0,
		  0, 99, 99,  2,  0, 50, 35, 70, 12,  0,  0,  0,
		  0, 35,  0,  0,  0,  0,  0,  0, 14,  0,  0,  0,
		 14,  0,  0,  0, 18,  0,  0,  0, 17,  0,  0,  0,
		  0, 99, 99, 99,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  4,  4,  4,  4,
		 99, 43, 99, 71, 82, 99,  1, 90,255, 14,  0,  0,
		  0, 14, 14,  3,  0, 70, 13,  0,239,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0, 41, 55, 91, 45,
		 40,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,127, 70,  0,  0, 17,  0,  0,  0,
		  0,  0,  0,  0, 19, 73,  0,  0, 20,  0,  0,  0,
		  0,  0, 99, 40, 99, 40, 99, 42,  0,  6,  6, 10,
		 14,  0,  0,  0,  9,  0,  0,  0, 40, 15,  0,  0,
		  0,  9,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		 41, 60, 84, 16,  8,255,254,  0, 13,  8,  0,  0,
		  0, 64,127,127,  2, 14,  0, 27, 99,  2,132,  0,
		172,255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  2, 14, 99, 26, 99,  2,132,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  1,  3,  8,  0, 24,  0,  0,  0,
		  0,  1,  1,  0,119,  0,  0,  0,  0,  1,  0,  0,
		119,  0,  0,  0,  0,  0,  0,  0,127,  0,  0,  0,
		  0,  0,  0,  0,127,  0,  0,  0,  0,  0,  0,  0,
		127,  0,  0,  0,  0,  0,  0,  0,127,  0,  0,  0,
		  0,  0,  0,  0,127,  0,  0,  0,  1,  0,  0,  0,
		  1,  0,  0,  0,  1,  0,  0,  0,  1,  0,  0,  0,
		  1,  0,  0,  0,  1,  0,  0,  0,  1,  0,  0,  0,
		128,  1,127,  0,131, 60,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,132,  1,  0,  0, 13,  0,  0,  0,
		  0,  0,  0,  0, 19,  0,  0,  0,  0, 68,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  6,  1,  0,  0,
		 13,  0,  0,  0,  0,  0,  0,  0, 19,  0,  0,  0,
		 17,  0,  0,  0, 20,  0,  0,  0,  8,  0,  0,  0,
		 17,  0,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0,
		  0, 23,  0,  0,  0,  0,  0,  0,  0, 24,  8, 17,
		  6,  0,  0,  0, 14,  0,  0,  0, 20,  0,  0,  0,
		 20,254,  0,  0, 17,  0,  0,  0,  0, 99, 99,  2,
		  0, 50, 35, 70, 12,  0,  0,  0,  0, 35,  0,  0,
		  0,  0,  0,  0, 14,  0,  0,  0, 14,  0,  0,  0,
		 18,  0,  0,  0, 17,  0,  0,  0,  0, 99, 99, 99,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  4,  4,  4,  4, 99, 43, 99, 71,
		 82, 99,  1, 90,255, 14,  0,  0,  0, 14, 14,  3,
		  0, 70, 13,  0,239,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0, 41, 55, 91, 45, 40,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		127, 70,  0,  0, 17,  0,  0,  0,  0,  0,  0,  0,
		 19, 73,  0,  0, 20,  0,  0,  0,  0,  0, 99, 40,
		 99, 40, 99, 40,  0,  6,  6, 10, 14,  0,  0,  0,
		  9,  0,  0,  0, 40, 15,  0,  0,  0,  9,  1,  0,
		  0,  0,  0,  0,  0,  0,  0,  0, 41, 60, 84, 16,
		  8,255,254,  0, 13,  8,  0,  0,  0, 64,127,127,
		  2, 14,  0, 27, 99,  2,132,  0,172,255,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  2, 14, 99, 26, 99,  2,132,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
)

def transfer(fio,qin,qout):
	bytes = []
	while True:
		try:
			data = qin.get_nowait()
			if data is None:
				qout.put(None)
				return
			os.write(fio,data)
#			print data
#			qout.put(tuple([ord(d) for d in data]))
		except Empty:
#			print type(exc)
			pass
		try:
			data = [ord(d) for d in os.read(fio,128)]
#			if len(data)> 1:
#				print len(data),data
			for d in data:
#				if d != 0xF8:
#					print '0x%x' % d,
				if d & 0x80:
					if d == 0xF7:
						bytes.append(d)
					if bytes:
						qout.put(tuple(bytes))
						bytes = []
#				if d == 0xF7 or d == 0xF8:
				if d == 0xF7:
					pass
				else:
					bytes.append(d)
		except OSError:
			pass
		time.sleep(0.01)
		
def monitor(qin,qout):
	csts_dump = sysex.CurrentSongTimbreSetDataDumpRequest(globchnl=0).pack()
	clock_count = 0
	bytes = []
	orig_data = None
	while True:
		try:
			data = qout.get_nowait()
			if data is None: return
			status = data[0]
			if status < 0x90:
				chnl = (status & 0x0F)+1
				data = midi.NoteOff(clock_count,chnl,data[1],data[2])
			elif status < 0xA0:
				chnl = (status & 0x0F)+1
				data = midi.NoteOn(clock_count,chnl,data[1],data[2])
			elif status < 0xB0:
				chnl = (status & 0x0F)+1
				data = midi.PolyKeyPressure(clock_count,chnl,data[1],data[2])
			elif status < 0xC0:
				chnl = (status & 0x0F)+1
				data = midi.ControlChange(clock_count,chnl,data[1],data[2])
			elif status < 0xD0:
				chnl = (status & 0x0F)+1
				data = midi.ProgramChange(clock_count,chnl,data[1])
			elif status < 0xE0:
				chnl = (status & 0x0F)+1
				data = midi.ChannelKeyPressure(clock_count,chnl,data[1])
			elif status < 0xF0:
				chnl = (status & 0x0F)+1
				data = midi.PitchBendChange(clock_count,chnl,data[1]|(data[2]<<7))
			elif status == 0xF2:
				data = midi.SongPositionPointer(data[1]|(data[2]<<7))
			elif status == 0xF3:
				data = midi.SongSelect(data[1])
			elif status == 0xF8:
#				data = midi.TimingClock()
				clock_count += 1
				data = None
			elif status == 0xFA:
				data = midi.Start()
				clock_count = 0
			elif status == 0xFB:
				data = midi.Continue()
			elif status == 0xFC:
				data = midi.Stop()
			elif status == 0xFE:
				data = midi.ActiveSensing()
			elif status == 0xF0:
				category = data[1]
				if category == 0x42:
					try:
						function = data[6]
#						print 'function 0x%X' % function
						if function == 0x41:
							value = (data[12]<<28)|(data[13]<<21)|(data[14]<<14)|(data[15]<<7)|data[16]
							try:
								mp = modeparam_dict[data[7]][data[9]][data[11]]
#								print mp
								data = sysex.ParameterChange(data[2] & 0x0F,data[7],data[9],data[11],value,mp['offset'],mp['bit'],mp['bits'],parameter=mp['parameter'],rng=mp['range'])
#								data = None
							except:
								data = sysex.ParameterChange(data[2] & 0x0F,data[7],data[9],data[11],value)
							qin.put(csts_dump)
						elif function == 0x4E:
							data = sysex.ModeChange(data[2] & 0x0F,data[7])
						elif function == 0x53:
							index = data[7] & 0x7F
							bank = data[8] & 0x7F
							kit = data[9] & 0x7F
							param = data[11] & 0x7F
							value = ((data[12] & 0x0F) << 28) | ((data[13] & 0x7F) << 21) | ((data[14] & 0x7F) << 14) | ((data[15] & 0x7F) << 7) | (data[16] & 0x7F)
							data = sysex.DrumKitParameterChange(index=index,bank=bank,kit=kit,param=param,value=value)
						elif function == 0x72:
							data = ''
						elif function == 0x73:
							obj = data[7]
							print '\nobject',obj
							if obj == 0:
								bank = data[8]
								prog = data[10]
								data = sysex.convert_7_bit_to_8_bit(list(data[12:-1]))
#								print 'object %d bank %s program %d' % (obj,bank,prog)
#								fo = open('test_data/o%d_b_%d_p%d.py' % (obj,bank,prog),'wb')
#								fo.write('test_data = (')
#								td = [ord(c) for c in data]
#								for i,c in enumerate(td):
#									if (i % 12) == 0:
#										fo.write('\n\t\t')
#									fo.write('%3d,' % c)
#								fo.write(')\n\n')
#								fo.close()
#								data = ProgramParameters(data)
#IFX1								data = [ord(d) for d in data[88:100]]
#								data = [ord(d) for d in data[812:836]]
#								data = [ord(d) for d in data[932:936]]
#								data = [ord(d) for d in data[1196:1232]]
#								data = None
#								data = [(i,ord(d)) for i,d in enumerate(data) if ord(d) != test_data[i]]
#								data = [ord(d) for d in data[896:904]]
								data = [ord(d) for d in data[252:273]]
						elif function == 0x74:
							data = ''
						elif function == 0x75:
							obj = data[7]
#							print '\nobject',obj
							data = sysex.convert_7_bit_to_8_bit(list(data[9:-1]))
							if obj == 0:  # Program TABLE 1
#								data = [ord(d) for d in data]
#								data = [(i,ord(d)) for i,d in enumerate(data) if ord(d) != test_data[i]]
								data = [ord(d) for d in data[896:898]]
							elif obj == 1:  # Combination TABLE 2
								data = None
							elif obj == 2:  # Song Timbre Set TABLE 10
								if orig_data is None:
									orig_data = [ord(d) for d in data]
									data = None
								else:
									data = [(i,ord(d)) for i,d in enumerate(data) if ord(d) != orig_data[i]]
#								data = [ord(d) for d in data[632:636]]
#								data = [ord(d) for d in data[824:836]]
					except IndexError: pass
				elif category == 0x7E:
					try:
						sub_id1 = data[3]
						sub_id2 = data[4]
						if sub_id1 == 0x6:
							if sub_id2 == 0x02:
								data = sysex.InquiryReply(data)
					except IndexError: pass
#			else:
			if data is not None:
				print data
#				print test_data[896:904]
		except Empty:
			pass
		time.sleep(0.01)
		
def keyboard(qin):
	csts_dump = sysex.CurrentSongTimbreSetDataDumpRequest(globchnl=0).pack()
#	rlist = (sys.stdin,)
#	wlist = ()
#	xlist = ()
#	while True:
	for i in xrange(20):
#		dr,dw,de = select.select([sys.stdin], [], [], 0)
#		if dr:S
		print 'keyboard',sys.stdin.read(1)
#		try:
#			raw = raw_input()
#		except EOFError:
#			raw = None
#		print 'keyboard',raw
		time.sleep(1.0)
#		print len(sys.stdin.read())
#		print 'keyboard',s[0][0].read()

if __name__ == '__main__':
	import struct
#	from sysex import *
	fio = os.open('/dev/dmmidi2',os.O_RDWR | os.O_APPEND | os.O_NONBLOCK)
	qin = Queue()
	qout = Queue()
	def signal_handler(signal, frame):
		qin.put(None)
	signal.signal(signal.SIGINT, signal_handler)
	p1 = Process(target=transfer,args=(fio,qin,qout,))
	p2 = Process(target=monitor,args=(qin,qout,))
#	p3 = Process(target=keyboard,args=(qin,))
#	p3.start()
	p2.start()
	p1.start()
	qin.put(sysex.DeviceInquiry().pack())
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=1,subid=0,value=39).pack())
#	for i in xrange(8):
#		qin.put(struct.pack('>2B',0xF3,i))
#		print i
#		time.sleep(5.0)
	time.sleep(1.0)
#	for b in xrange(6):
#		for p in xrange(128):
#			if not os.path.exists('test_data/o0_b_%d_p%d.py' % (b,p)):
#				qin.put(ProgramParameterDumpRequest(globchnl=0,bank=b,program=p).pack())
#				time.sleep(1.5)
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x00,subid=0x12,value=64).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ProgramParameterDumpRequest(globchnl=0,bank=0,program=19).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x00,subid=0x12,value=64).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ProgramParameterDumpRequest(globchnl=0,bank=0,program=0).pack())
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x0C,subid=0x15,value=1).pack())
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x0D,subid=0x23,value=48).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ProgramParameterDumpRequest(globchnl=0,bank=0,program=0).pack())
#	print 'sleeping'
#	time.sleep(120.0)
#	qin.put(None)
#	qin.put(sysex.CurrentProgramParameterDumpRequest(globchnl=0).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x00,subid=0x16,value=1).pack())
#	time.sleep(0.5)
#	for v in xrange(78):
#		print v
#		qin.put(sysex.ParameterChange(globchnl=0,mode=2,id=0x01,subid=0x00,value=v).pack())
#		time.sleep(0.5)
#		qin.put(sysex.CurrentProgramParameterDumpRequest(globchnl=0).pack())
#		time.sleep(0.5)
#	qin.put(sysex.CurrentSongTimbreSetDataDumpRequest(globchnl=0).pack())
#	time.sleep(0.5)
#	qin.put(sysex.ParameterChange(globchnl=0,mode=4,id=0x00,subid=0x0F,value=0).pack())
#	time.sleep(0.5)
	qin.put(sysex.CurrentSongTimbreSetDataDumpRequest(globchnl=0).pack())
#	print sys.stdin.tell()
	p1.join()
	p2.join()
#	p3.join()
	os.close(fio)
		
		

