#!@PYTHON@
# -*- coding: utf-8 -*-
#
# table10.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

import struct

#===============================================================================
class MasterBarEvent(object):
	beat_dict = {0:'specified',1:'/4',2:'/8',3:'/16'}
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'MasterBar 0x%x 0x%x 0x%x %d%s %d %d' % (self.kind,self.u1,self.u2,self.beats+1,MasterBarEvent.beat_dict[self.beat],self.size,self.meas+1)

#-------------------------------------------------------------------------------
	def __repr__(self):
		return 'MasterBarEvent(meas=%d,size=%d,beats=%d,beat=%d)' % (self.meas,self.size,self.beats,self.beat)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,meter,self.size,self.meas = struct.unpack('<BBBBHH',raw)
		self.beats = meter & 0xF
		self.beat = (meter >> 4) & 0x3

#-------------------------------------------------------------------------------
	def pack(self):
		meter = (self.beats & 0xF) | ((self.beat & 0x3) << 4)
		v = [self.kind,0,0,meter,self.size,self.meas]
		return struct.pack('<BBBBHH',*v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('beats',self.beats))
		json.append(('beat',self.beat))
		json.append(('meter','%s%s' % (self.beats,MasterBarEvent.beat_dict[self.beat])))
		json.append(('size',self.size))
		json.append(('meas',self.meas))
#		json.append(('u1','0x%02X' % self.u1))
#		json.append(('u2','0x%02X' % self.u2))
#		json.append(('u1',chr(self.u1)))
#		json.append(('u2',chr(self.u2)))
#		json.append(('u1_2',chr(self.u1)+chr(self.u2)))
		return json

#===============================================================================
class BarEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'Bar 0x%x 0x%x 0x%x 0x%x %d %d' % (self.kind,self.u1,self.u2,self.u3,self.size,self.meas)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,self.u3,self.size,self.meas = struct.unpack('<BBBBHH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a BarEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('size',self.size))
		json.append(('meas',self.meas))
		json.append(('u1',self.u1))
		json.append(('u2',self.u2))
		json.append(('u3',self.u3))
		return json

#===============================================================================
class PatternBarEvent(BarEvent):
	pass

#===============================================================================
class PatEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'Pat 0x%x 0x%x 0x%x %d %d %d' % (self.kind,self.u1,self.u2,self.pattern_meas,self.pattern_number,self.meas)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,self.pattern_meas,self.pattern_number,self.meas = struct.unpack('<BBBBHH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a PatEvent event'''
		json = []
		json.append(('name','PatEvent'))
		json.append(('kind',self.kind))
		json.append(('pattern_meas',self.pattern_meas))
		json.append(('pattern_number',self.pattern_number))
		json.append(('meas',self.meas))
		json.append(('u1',self.u1))
		json.append(('u2',self.u2))
		return json

#===============================================================================
class TrkEndEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,meas=0,raw=None):
		if raw is None:
			self.kind = 3
			self.meas = meas
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'TrkEnd 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x %d' % (self.kind,self.u1,self.u2,self.u3,self.u4,self.u5,self.meas)

#-------------------------------------------------------------------------------
	def __repr__(self):
		return 'TrkEndEvent(meas=%d)' % (self.meas)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,self.u3,self.u4,self.u5,self.meas = struct.unpack('<BBBBBBH',raw)

#-------------------------------------------------------------------------------
	def pack(self):
		v = [self.kind,0,0,0,0,0,self.meas]
		return struct.pack('<BBBBBBH',*v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('meas',self.meas))
#		json.append(('u1',self.u1))
#		json.append(('u2',self.u2))
#		json.append(('u3',self.u3))
#		json.append(('u4',self.u4))
#		json.append(('u5',self.u5))
		return json

#===============================================================================
class NoteEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		if self.length == 0x7FFF:
			if self.tick == 0x7FFF:
				return 'Note 0x%02x 0x%02x %3d %3d ~~~~ ~~~~' % (self.kind,self.u1,self.key,self.velocity) 
			else:
				return 'Note 0x%02x 0x%02x %3d %3d ~~~~ %4d' % (self.kind,self.u1,self.key,self.velocity,self.tick) 
		else:
			if self.tick == 0x7FFF:
				return 'Note 0x%02x 0x%02x %3d %3d %4d ~~~~' % (self.kind,self.u1,self.key,self.velocity,self.length) 
			else:
				return 'Note 0x%02x 0x%02x %3d %3d %4d %4d' % (self.kind,self.u1,self.key,self.velocity,self.length,self.tick) 

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.key,self.velocity,self.length,self.tick = struct.unpack('<BBBBHH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a NoteEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('key',self.key))
		json.append(('velocity',self.velocity))
		json.append(('length',self.length))
		json.append(('tick',self.tick))
		json.append(('u1',self.u1))
		return json

#===============================================================================
class PolyPressEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'PolyPress 0x%x 0x%x 0x%x %d %d %d %d' % (self.kind,self.u1,self.u2,self.u3,self.value,self.key,self.tick)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,self.u3,self.value,self.key,self.tick = struct.unpack('<BBBBBBH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a PolyPressEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('value',self.value))
		json.append(('key',self.key))
		json.append(('tick',self.tick))
		json.append(('u1',self.u1))
		json.append(('u2',self.u2))
		json.append(('u3',self.u3))
		return json

#===============================================================================
class ControlChgEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'ControlChg 0x%x 0x%x 0x%x %d %d %d %d' % (self.kind,self.u1,self.unfixed,self.last,self.value,self.control,self.tick)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.unfixed,self.last,self.value,self.control,self.tick = struct.unpack('<BBBBBBH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ControlChgEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('unfixed',self.unfixed))
		json.append(('last',self.last))
		json.append(('value',self.value))
		json.append(('control',self.control))
		json.append(('tick',self.tick))
		json.append(('u1',self.u1))
		return json

#===============================================================================
class TempoChgEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,tick=0,tempo=12000,last_tempo=0,last_tempo_unfixed=1,raw=None):
		if raw is None:
			self.kind = 11
			self.number = 0x6B
			self.tick = tick
			self.tempo = tempo
			self.last_tempo = last_tempo
			self.last_tempo_unfixed = last_tempo_unfixed
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'TempoChgEvent kind:0x%x number:0x%x unfixed:%d last:0x%x tempo:%d tick:%d' % (self.kind,self.number,self.last_tempo_unfixed,self.last_tempo,self.tempo,self.tick)

#-------------------------------------------------------------------------------
	def __repr__(self):
		return 'TempoChgEvent(tick=%d,tempo=%d,last_tempo=%d,last_tempo_unfixed=%d)' % (self.tick,self.tempo,self.last_tempo,self.last_tempo_unfixed)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.number,last_tempo,self.tempo,self.tick = struct.unpack('<BBHHH',raw)
		self.last_tempo = last_tempo & 0x7FFF
		self.last_tempo_unfixed = last_tempo >> 15

#-------------------------------------------------------------------------------
	def pack(self):
		last_tempo = (self.last_tempo & 0x7FFF) | (self.last_tempo_unfixed << 15)
		v = [self.kind,self.number,last_tempo,self.tempo,self.tick]
		return struct.pack('<BBHHH',*v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a TempoChgEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
#		json.append(('number',self.number))
		json.append(('last_tempo_unfixed',self.last_tempo_unfixed))
		json.append(('last_tempo',self.last_tempo))
		json.append(('tempo',self.tempo))
		json.append(('tick',self.tick))
		return json

#===============================================================================
class ProgramChgEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'ProgramChg 0x%x 0x%x 0x%x %d 0x%x %d %d' % (self.kind,self.last_bank,self.last_unfixed,self.last_program,self.bank,self.program,self.tick)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.last_bank,self.last_unfixed,self.last_program,self.bank,self.program,self.tick = struct.unpack('<BBBBBBH',raw)
#		print '\tProgramChg 0x%x 0x%x 0x%x %d 0x%x %d %d' % (self.kind,self.last_bank,self.last_unfixed,self.last_program,self.bank,self.program,self.tick) 

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ProgramChgEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('last_bank',self.last_bank))
		json.append(('last_unfixed',self.last_unfixed))
		json.append(('last_program',self.last_program))
		json.append(('bank',self.bank))
		json.append(('program',self.program))
		json.append(('tick',self.tick))
		return json

#===============================================================================
class ChPressEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'ChPress 0x%x 0x%x 0x%x %d %d %d %d' % (self.kind,self.u1,self.u2,self.last_unfixed,self.last,self.value,self.tick)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.u1,self.u2,self.last_unfixed,self.last,self.value,self.tick = struct.unpack('<BBBBBBH',raw)
#		print '\tChPress 0x%x 0x%x 0x%x %d %d %d %d' % (self.kind,self.u1,self.u2,self.last_unfixed,self.last,self.value,self.tick) 

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ChPressEvent event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('last_unfixed',self.last_unfixed))
		json.append(('last',self.last))
		json.append(('value',self.value))
		json.append(('tick',self.tick))
		json.append(('u1',self.u1))
		json.append(('u2',self.u2))
		return json

#===============================================================================
class PitchBendEvent(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'PitchBend 0x%x 0x%x 0x%x %d %d %d %d' % (self.kind,self.last_unfixed,self.last_hi,self.last_lo,self.hi,self.lo,self.tick) 

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		self.raw = raw
		self.kind,self.last_unfixed,self.last_hi,self.last_lo,self.hi,self.lo,self.tick = struct.unpack('<BBBBBBH',raw)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a PitchBend event'''
		json = []
		json.append(('name',self.__class__.__name__))
		json.append(('kind',self.kind))
		json.append(('last_unfixed',self.last_unfixed))
		json.append(('last_hi',self.last_hi))
		json.append(('last_lo',self.last_lo))
		json.append(('hi',self.hi))
		json.append(('lo',self.lo))
		json.append(('tick',self.tick))
		return json


