#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx003.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX003(object):
	category = '''Dynamic'''
	description = '''Stereo Limiter'''
	short_description = '''Stereo Limiter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX003 Stereo Limiter object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.envelope_source = (bytes[3] >> 5) & 0x3
		self.fx_control_trim = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x3F) << 1)
		self.envelope_select = (bytes[4] >> 6) & 0x3
		self.ratio = bytes[5] & 0xFF
		data = bytes[6] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.threshold = data
		self.attack = ((bytes[6] & 0x80) >> 7) | ((bytes[7] & 0x3F) << 1)
		self.release = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x1F) << 2)
		self.side_peq_insert = (bytes[8] >> 5) & 0x1
		self.trigger_monitor = (bytes[8] >> 6) & 0x1
		self.side_peq_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		self.side_peq_q = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.side_peq_gain = data
		data = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.gain_adjust = data
		self.gain_adjust_mod_source = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x3) << 4)
		data = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.gain_adjust_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX003 Stereo Limiter object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.envelope_source & 0x3) << 5) | ((self.fx_control_trim << 7) & 0x80)
		bytes[4] = ((self.fx_control_trim >> 1) & 0x3F) | ((self.envelope_select & 0x3) << 6)
		bytes[5] = self.ratio & 0xFF
		bytes[6] = (self.threshold & 0x7F) | ((self.attack << 7) & 0x80)
		bytes[7] = ((self.attack >> 1) & 0x3F) | ((self.release << 6) & 0xC0)
		bytes[8] = ((self.release >> 2) & 0x1F) | ((self.side_peq_insert & 0x1) << 5) | ((self.trigger_monitor & 0x1) << 6) | ((self.side_peq_cutoff << 7) & 0x80)
		bytes[9] = ((self.side_peq_cutoff >> 1) & 0x7F) | ((self.side_peq_q << 7) & 0x80)
		bytes[10] = ((self.side_peq_q >> 1) & 0x3F) | ((self.side_peq_gain << 6) & 0xC0)
		bytes[11] = ((self.side_peq_gain >> 2) & 0x1F) | ((self.gain_adjust << 5) & 0xE0)
		bytes[12] = ((self.gain_adjust >> 3) & 0xF) | ((self.gain_adjust_mod_source << 4) & 0xF0)
		bytes[13] = ((self.gain_adjust_mod_source >> 4) & 0x3) | ((self.gain_adjust_mod_intensity << 2) & 0xFC)
		bytes[14] = (self.gain_adjust_mod_intensity >> 6) & 0x1
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX003 Stereo Limiter object'''
		json = []
		json.append(('name','FX003'))
		json.append(('description','Stereo Limiter'))
		json.append(['envelope_select',self.envelope_select])
		json.append(['ratio',self.ratio])
		json.append(['threshold',self.threshold])
		json.append(['attack',self.attack])
		json.append(['release',self.release])
		json.append(['gain_adjust',self.gain_adjust])
		json.append(['gain_adjust_mod_source',self.gain_adjust_mod_source])
		json.append(['gain_adjust_mod_intensity',self.gain_adjust_mod_intensity])
		json.append(['side_peq_insert',self.side_peq_insert])
		json.append(['trigger_monitor',self.trigger_monitor])
		json.append(['side_peq_cutoff',self.side_peq_cutoff])
		json.append(['side_peq_q',self.side_peq_q])
		json.append(['side_peq_gain',self.side_peq_gain])
		json.append(['side_peq_gain_display',self.side_peq_gain_display])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['envelope_source',self.envelope_source])
		json.append(['fx_control_trim',self.fx_control_trim])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_source(self):
		'''Envelope Source'''
		return self._envelope_source

	@envelope_source.setter
	def envelope_source(self,value):
		if 0 <= value <= 2:
			self._envelope_source = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def fx_control_trim(self):
		'''Fx Control Trim'''
		return self._fx_control_trim

	@fx_control_trim.setter
	def fx_control_trim(self,value):
		if 0 <= value <= 100:
			self._fx_control_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select(self):
		'''Envelope Select'''
		return self._envelope_select

	@envelope_select.setter
	def envelope_select(self,value):
		if 0 <= value <= 3:
			self._envelope_select = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def ratio(self):
		'''Ratio'''
		return self._ratio

	@ratio.setter
	def ratio(self,value):
		if 0 <= value <= 131:
			self._ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if -40 <= value <= 0:
			self._threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 1 <= value <= 100:
			self._attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def release(self):
		'''Release'''
		return self._release

	@release.setter
	def release(self,value):
		if 1 <= value <= 100:
			self._release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_insert(self):
		'''Side PEQ Insert'''
		return self._side_peq_insert

	@side_peq_insert.setter
	def side_peq_insert(self,value):
		if 0 <= value <= 1:
			self._side_peq_insert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def trigger_monitor(self):
		'''Trigger Monitor'''
		return self._trigger_monitor

	@trigger_monitor.setter
	def trigger_monitor(self,value):
		if 0 <= value <= 1:
			self._trigger_monitor = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_cutoff(self):
		'''Side PEQ Cutoff'''
		return self._side_peq_cutoff

	@side_peq_cutoff.setter
	def side_peq_cutoff(self,value):
		if 1 <= value <= 200:
			self._side_peq_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_q(self):
		'''Side PEQ Q'''
		return self._side_peq_q

	@side_peq_q.setter
	def side_peq_q(self,value):
		if 0 <= value <= 95:
			self._side_peq_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_gain(self):
		'''Side PEQ Gain'''
		return self._side_peq_gain

	@side_peq_gain.setter
	def side_peq_gain(self,value):
		if -36 <= value <= 36:
			self._side_peq_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def side_peq_gain_display(self):
		'''Side PEQ Gain display value'''
		dv = self._side_peq_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def gain_adjust(self):
		'''Gain Adjust'''
		return self._gain_adjust

	@gain_adjust.setter
	def gain_adjust(self,value):
		if -39 <= value <= 24:
			self._gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def gain_adjust_mod_source(self):
		'''Gain Adjust Mod.Source'''
		return self._gain_adjust_mod_source

	@gain_adjust_mod_source.setter
	def gain_adjust_mod_source(self,value):
		if 0 <= value <= 37:
			self._gain_adjust_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def gain_adjust_mod_intensity(self):
		'''Gain Adjust Mod.Intensity'''
		return self._gain_adjust_mod_intensity

	@gain_adjust_mod_intensity.setter
	def gain_adjust_mod_intensity(self,value):
		if -63 <= value <= 63:
			self._gain_adjust_mod_intensity = value
		else:
			raise ValueError('%s not -63 <= value <= 63' % value)

