#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx009.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX009(object):
	category = '''Eq/Filter'''
	description = '''Stereo Exciter/Enhancer'''
	short_description = '''St.Exciter/Enhncr'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX009 Stereo Exciter/Enhancer object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.blend = data
		self.blend_mod_source = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.blend_mod_intensity = data
		self.emphasis_frequency = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.emphasis_frequency_mod_source = (bytes[7] >> 2) & 0x3F
		data = bytes[8] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.emphasis_frequency_mod_intensity = data
		self.l_delay = bytes[9] & 0xFF
		self.r_delay = bytes[10] & 0xFF
		self.depth = bytes[11] & 0x7F
		self.depth_mod_source = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x1F) << 1)
		data = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		data = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.leq_gain = data
		data = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.heq_gain = data
		self.leq_fc = (bytes[15] >> 1) & 0x1
		self.heq_fc = (bytes[15] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX009 Stereo Exciter/Enhancer object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.blend << 5) & 0xE0)
		bytes[4] = ((self.blend >> 3) & 0x1F) | ((self.blend_mod_source << 5) & 0xE0)
		bytes[5] = ((self.blend_mod_source >> 3) & 0x7) | ((self.blend_mod_intensity << 3) & 0xF8)
		bytes[6] = ((self.blend_mod_intensity >> 5) & 0x7) | ((self.emphasis_frequency << 3) & 0xF8)
		bytes[7] = ((self.emphasis_frequency >> 5) & 0x3) | ((self.emphasis_frequency_mod_source & 0x3F) << 2)
		bytes[8] = self.emphasis_frequency_mod_intensity & 0xFF
		bytes[9] = self.l_delay & 0xFF
		bytes[10] = self.r_delay & 0xFF
		bytes[11] = (self.depth & 0x7F) | ((self.depth_mod_source << 7) & 0x80)
		bytes[12] = ((self.depth_mod_source >> 1) & 0x1F) | ((self.depth_mod_intensity << 5) & 0xE0)
		bytes[13] = ((self.depth_mod_intensity >> 3) & 0x1F) | ((self.leq_gain << 5) & 0xE0)
		bytes[14] = ((self.leq_gain >> 3) & 0x7) | ((self.heq_gain << 3) & 0xF8)
		bytes[15] = ((self.heq_gain >> 5) & 0x1) | ((self.leq_fc & 0x1) << 1) | ((self.heq_fc & 0x1) << 2)
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX009 Stereo Exciter/Enhancer object'''
		json = []
		json.append(('name','FX009'))
		json.append(('description','Stereo Exciter/Enhancer'))
		json.append(['blend',self.blend])
		json.append(['blend_mod_source',self.blend_mod_source])
		json.append(['blend_mod_intensity',self.blend_mod_intensity])
		json.append(['emphasis_frequency',self.emphasis_frequency])
		json.append(['emphasis_frequency_mod_source',self.emphasis_frequency_mod_source])
		json.append(['emphasis_frequency_mod_intensity',self.emphasis_frequency_mod_intensity])
		json.append(['l_delay',self.l_delay])
		json.append(['r_delay',self.r_delay])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['input_trim',self.input_trim])
		json.append(['leq_gain',self.leq_gain])
		json.append(['heq_gain',self.heq_gain])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['leq_fc',self.leq_fc])
		json.append(['heq_fc',self.heq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def blend(self):
		'''Blend'''
		return self._blend

	@blend.setter
	def blend(self,value):
		if -100 <= value <= 100:
			self._blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def blend_mod_source(self):
		'''Blend Mod.Source'''
		return self._blend_mod_source

	@blend_mod_source.setter
	def blend_mod_source(self,value):
		if 0 <= value <= 37:
			self._blend_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def blend_mod_intensity(self):
		'''Blend Mod.Intensity'''
		return self._blend_mod_intensity

	@blend_mod_intensity.setter
	def blend_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._blend_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def emphasis_frequency(self):
		'''Emphasis Frequency'''
		return self._emphasis_frequency

	@emphasis_frequency.setter
	def emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def emphasis_frequency_mod_source(self):
		'''Emphasis Frequency Mod.Source'''
		return self._emphasis_frequency_mod_source

	@emphasis_frequency_mod_source.setter
	def emphasis_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._emphasis_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def emphasis_frequency_mod_intensity(self):
		'''Emphasis Frequency Mod.Intensity'''
		return self._emphasis_frequency_mod_intensity

	@emphasis_frequency_mod_intensity.setter
	def emphasis_frequency_mod_intensity(self,value):
		if -70 <= value <= 70:
			self._emphasis_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -70 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay(self):
		'''L Delay'''
		return self._l_delay

	@l_delay.setter
	def l_delay(self,value):
		if 0 <= value <= 140:
			self._l_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay(self):
		'''R Delay'''
		return self._r_delay

	@r_delay.setter
	def r_delay(self,value):
		if 0 <= value <= 140:
			self._r_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def leq_gain(self):
		'''LEQ Gain'''
		return self._leq_gain

	@leq_gain.setter
	def leq_gain(self,value):
		if -30 <= value <= 30:
			self._leq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def heq_gain(self):
		'''HEQ Gain'''
		return self._heq_gain

	@heq_gain.setter
	def heq_gain(self,value):
		if -30 <= value <= 30:
			self._heq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def leq_fc(self):
		'''LEQ Fc'''
		return self._leq_fc

	@leq_fc.setter
	def leq_fc(self,value):
		if 0 <= value <= 1:
			self._leq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def heq_fc(self):
		'''HEQ Fc'''
		return self._heq_fc

	@heq_fc.setter
	def heq_fc(self,value):
		if 0 <= value <= 1:
			self._heq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

