#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx019.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX019(object):
	category = '''Eq/Filter'''
	description = '''Stereo Analog Record'''
	short_description = '''St. Analog Record'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX019 Stereo Analog Record object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.speed = (bytes[3] >> 5) & 0x3
		self.flutter = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x3F) << 1)
		self.noise_density = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x1F) << 2)
		self.noise_tone = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0xF) << 3)
		self.noise_level = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0x7) << 4)
		self.noise_level_mod_source = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x1) << 5)
		data = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.noise_level_mod_intensity = data
		self.click_level = (bytes[9] >> 1) & 0x7F
		self.click_level_mod_source = bytes[10] & 0x3F
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.click_level_mod_intensity = data
		self.pre_eq_cutoff = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0x1F) << 2)
		self.pre_eq_q = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.pre_eq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX019 Stereo Analog Record object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.speed & 0x3) << 5) | ((self.flutter << 7) & 0x80)
		bytes[4] = ((self.flutter >> 1) & 0x3F) | ((self.noise_density << 6) & 0xC0)
		bytes[5] = ((self.noise_density >> 2) & 0x1F) | ((self.noise_tone << 5) & 0xE0)
		bytes[6] = ((self.noise_tone >> 3) & 0xF) | ((self.noise_level << 4) & 0xF0)
		bytes[7] = ((self.noise_level >> 4) & 0x7) | ((self.noise_level_mod_source << 3) & 0xF8)
		bytes[8] = ((self.noise_level_mod_source >> 5) & 0x1) | ((self.noise_level_mod_intensity << 1) & 0xFE)
		bytes[9] = ((self.noise_level_mod_intensity >> 7) & 0x1) | ((self.click_level & 0x7F) << 1)
		bytes[10] = (self.click_level_mod_source & 0x3F) | ((self.click_level_mod_intensity << 6) & 0xC0)
		bytes[11] = ((self.click_level_mod_intensity >> 2) & 0x3F) | ((self.pre_eq_cutoff << 6) & 0xC0)
		bytes[12] = ((self.pre_eq_cutoff >> 2) & 0x1F) | ((self.pre_eq_q << 5) & 0xE0)
		bytes[13] = ((self.pre_eq_q >> 3) & 0xF) | ((self.pre_eq_gain << 4) & 0xF0)
		bytes[14] = (self.pre_eq_gain >> 4) & 0x7
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX019 Stereo Analog Record object'''
		json = []
		json.append(('name','FX019'))
		json.append(('description','Stereo Analog Record'))
		json.append(['speed',self.speed])
		json.append(['flutter',self.flutter])
		json.append(['noise_density',self.noise_density])
		json.append(['noise_tone',self.noise_tone])
		json.append(['noise_level',self.noise_level])
		json.append(['noise_level_mod_source',self.noise_level_mod_source])
		json.append(['noise_level_mod_source_display',self.noise_level_mod_source_display])
		json.append(['noise_level_mod_intensity',self.noise_level_mod_intensity])
		json.append(['click_level',self.click_level])
		json.append(['click_level_mod_source',self.click_level_mod_source])
		json.append(['click_level_mod_intensity',self.click_level_mod_intensity])
		json.append(['input_trim',self.input_trim])
		json.append(['pre_eq_cutoff',self.pre_eq_cutoff])
		json.append(['pre_eq_q',self.pre_eq_q])
		json.append(['pre_eq_gain',self.pre_eq_gain])
		json.append(['pre_eq_gain_display',self.pre_eq_gain_display])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_source_display',self.wet_dry_mod_source_display])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def wet_dry_mod_source_display(self):
		'''Wet/Dry Mod.Source display value'''
		return module_sources[self._wet_dry_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 0 <= value <= 2:
			self._speed = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def flutter(self):
		'''Flutter'''
		return self._flutter

	@flutter.setter
	def flutter(self,value):
		if 0 <= value <= 100:
			self._flutter = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_density(self):
		'''Noise Density'''
		return self._noise_density

	@noise_density.setter
	def noise_density(self,value):
		if 0 <= value <= 100:
			self._noise_density = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_tone(self):
		'''Noise Tone'''
		return self._noise_tone

	@noise_tone.setter
	def noise_tone(self,value):
		if 0 <= value <= 100:
			self._noise_tone = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_level(self):
		'''Noise Level'''
		return self._noise_level

	@noise_level.setter
	def noise_level(self,value):
		if 0 <= value <= 100:
			self._noise_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_level_mod_source(self):
		'''Noise Level Mod.Source'''
		return self._noise_level_mod_source

	@noise_level_mod_source.setter
	def noise_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._noise_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def noise_level_mod_source_display(self):
		'''Noise Level Mod.Source display value'''
		return module_sources[self._noise_level_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def noise_level_mod_intensity(self):
		'''Noise Level Mod.Intensity'''
		return self._noise_level_mod_intensity

	@noise_level_mod_intensity.setter
	def noise_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._noise_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def click_level(self):
		'''Click Level'''
		return self._click_level

	@click_level.setter
	def click_level(self,value):
		if 0 <= value <= 100:
			self._click_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def click_level_mod_source(self):
		'''Click Level Mod.Source'''
		return self._click_level_mod_source

	@click_level_mod_source.setter
	def click_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._click_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def click_level_mod_intensity(self):
		'''Click Level Mod.Intensity'''
		return self._click_level_mod_intensity

	@click_level_mod_intensity.setter
	def click_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._click_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_eq_cutoff(self):
		'''Pre EQ Cutoff'''
		return self._pre_eq_cutoff

	@pre_eq_cutoff.setter
	def pre_eq_cutoff(self,value):
		if 0 <= value <= 97:
			self._pre_eq_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_eq_q(self):
		'''Pre EQ Q'''
		return self._pre_eq_q

	@pre_eq_q.setter
	def pre_eq_q(self,value):
		if 0 <= value <= 95:
			self._pre_eq_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_eq_gain(self):
		'''Pre EQ Gain'''
		return self._pre_eq_gain

	@pre_eq_gain.setter
	def pre_eq_gain(self,value):
		if -36 <= value <= 36:
			self._pre_eq_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def pre_eq_gain_display(self):
		'''Pre EQ Gain display value'''
		dv = self._pre_eq_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

