#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx020.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX020(object):
	category = '''OD Amp Mic'''
	description = '''Overdrive/Hi-Gain Wah'''
	short_description = '''OD/Hi.Gain Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX020 Overdrive/Hi-Gain Wah object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.wah_on = (bytes[3] >> 5) & 0x1
		self.wah_on_mod_source = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xF) << 2)
		self.wah_on_momentary = (bytes[4] >> 4) & 0x1
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x3) << 3)
		if data & 0x10:
			data -= 0x20
		self.wah_sweep_range = data
		self.sweep_range_mod_source = (bytes[5] >> 2) & 0x3F
		self.hi_gain_on = bytes[6] & 0x1
		self.drive = (bytes[6] >> 1) & 0x7F
		self.pre_low_cut = bytes[7] & 0xF
		self.output_level = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x3) << 4)
		self.output_level_mod_source = (bytes[8] >> 2) & 0x3F
		data = bytes[9] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.output_level_mod_intensity = data
		self.eq_low_cutoff = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x1F) << 1)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.eq_low_gain = data
		self.eq_mid_1_cutoff = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x1) << 5)
		self.eq_mid_1_q = (bytes[12] >> 1) & 0x3F
		data = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.eq_mid_1_gain = data
		self.eq_mid_2_cutoff = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x7) << 3)
		self.eq_mid_2_q = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x1) << 5)
		data = (bytes[15] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.eq_mid_2_gain = data
		self.direct_mix = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x1F) << 1)
		self.speaker_simulator = (bytes[16] >> 5) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX020 Overdrive/Hi-Gain Wah object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.wah_on & 0x1) << 5) | ((self.wah_on_mod_source << 6) & 0xC0)
		bytes[4] = ((self.wah_on_mod_source >> 2) & 0xF) | ((self.wah_on_momentary & 0x1) << 4) | ((self.wah_sweep_range << 5) & 0xE0)
		bytes[5] = ((self.wah_sweep_range >> 3) & 0x3) | ((self.sweep_range_mod_source & 0x3F) << 2)
		bytes[6] = (self.hi_gain_on & 0x1) | ((self.drive & 0x7F) << 1)
		bytes[7] = (self.pre_low_cut & 0xF) | ((self.output_level << 4) & 0xF0)
		bytes[8] = ((self.output_level >> 4) & 0x3) | ((self.output_level_mod_source & 0x3F) << 2)
		bytes[9] = (self.output_level_mod_intensity & 0x7F) | ((self.eq_low_cutoff << 7) & 0x80)
		bytes[10] = ((self.eq_low_cutoff >> 1) & 0x1F) | ((self.eq_low_gain << 5) & 0xE0)
		bytes[11] = ((self.eq_low_gain >> 3) & 0x7) | ((self.eq_mid_1_cutoff << 3) & 0xF8)
		bytes[12] = ((self.eq_mid_1_cutoff >> 5) & 0x1) | ((self.eq_mid_1_q & 0x3F) << 1) | ((self.eq_mid_1_gain << 7) & 0x80)
		bytes[13] = ((self.eq_mid_1_gain >> 1) & 0x1F) | ((self.eq_mid_2_cutoff << 5) & 0xE0)
		bytes[14] = ((self.eq_mid_2_cutoff >> 3) & 0x7) | ((self.eq_mid_2_q << 3) & 0xF8)
		bytes[15] = ((self.eq_mid_2_q >> 5) & 0x1) | ((self.eq_mid_2_gain & 0x3F) << 1) | ((self.direct_mix << 7) & 0x80)
		bytes[16] = ((self.direct_mix >> 1) & 0x1F) | ((self.speaker_simulator & 0x1) << 5)
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX020 Overdrive/Hi-Gain Wah object'''
		json = []
		json.append(('name','FX020'))
		json.append(('description','Overdrive/Hi-Gain Wah'))
		json.append(['wah_on',self.wah_on])
		json.append(['wah_on_mod_source',self.wah_on_mod_source])
		json.append(['wah_on_momentary',self.wah_on_momentary])
		json.append(['wah_sweep_range',self.wah_sweep_range])
		json.append(['sweep_range_mod_source',self.sweep_range_mod_source])
		json.append(['hi_gain_on',self.hi_gain_on])
		json.append(['drive',self.drive])
		json.append(['pre_low_cut',self.pre_low_cut])
		json.append(['output_level',self.output_level])
		json.append(['output_level_mod_source',self.output_level_mod_source])
		json.append(['output_level_mod_intensity',self.output_level_mod_intensity])
		json.append(['eq_low_cutoff',self.eq_low_cutoff])
		json.append(['eq_low_gain',self.eq_low_gain])
		json.append(['eq_mid_1_cutoff',self.eq_mid_1_cutoff])
		json.append(['eq_mid_1_q',self.eq_mid_1_q])
		json.append(['eq_mid_1_gain',self.eq_mid_1_gain])
		json.append(['eq_mid_2_cutoff',self.eq_mid_2_cutoff])
		json.append(['eq_mid_2_q',self.eq_mid_2_q])
		json.append(['eq_mid_2_gain',self.eq_mid_2_gain])
		json.append(['direct_mix',self.direct_mix])
		json.append(['speaker_simulator',self.speaker_simulator])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_on(self):
		'''Wah On'''
		return self._wah_on

	@wah_on.setter
	def wah_on(self,value):
		if 0 <= value <= 1:
			self._wah_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_on_mod_source(self):
		'''Wah On Mod.Source'''
		return self._wah_on_mod_source

	@wah_on_mod_source.setter
	def wah_on_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_on_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_on_momentary(self):
		'''Wah On Momentary'''
		return self._wah_on_momentary

	@wah_on_momentary.setter
	def wah_on_momentary(self,value):
		if 0 <= value <= 1:
			self._wah_on_momentary = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_range(self):
		'''Wah Sweep Range'''
		return self._wah_sweep_range

	@wah_sweep_range.setter
	def wah_sweep_range(self,value):
		if -10 <= value <= 10:
			self._wah_sweep_range = value
		else:
			raise ValueError('%s not -10 <= value <= 10' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_range_mod_source(self):
		'''Sweep Range Mod.Source'''
		return self._sweep_range_mod_source

	@sweep_range_mod_source.setter
	def sweep_range_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_range_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def hi_gain_on(self):
		'''Hi-Gain On'''
		return self._hi_gain_on

	@hi_gain_on.setter
	def hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def drive(self):
		'''Drive'''
		return self._drive

	@drive.setter
	def drive(self,value):
		if 1 <= value <= 100:
			self._drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_low_cut(self):
		'''Pre Low Cut'''
		return self._pre_low_cut

	@pre_low_cut.setter
	def pre_low_cut(self,value):
		if 0 <= value <= 10:
			self._pre_low_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 10' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 50:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_source(self):
		'''Output Level Mod.Source'''
		return self._output_level_mod_source

	@output_level_mod_source.setter
	def output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_intensity(self):
		'''Output Level Mod.Intensity'''
		return self._output_level_mod_intensity

	@output_level_mod_intensity.setter
	def output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_low_cutoff(self):
		'''EQ Low Cutoff'''
		return self._eq_low_cutoff

	@eq_low_cutoff.setter
	def eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_low_gain(self):
		'''EQ Low Gain'''
		return self._eq_low_gain

	@eq_low_gain.setter
	def eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_1_cutoff(self):
		'''EQ Mid 1 Cutoff'''
		return self._eq_mid_1_cutoff

	@eq_mid_1_cutoff.setter
	def eq_mid_1_cutoff(self,value):
		if 1 <= value <= 63:
			self._eq_mid_1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_1_q(self):
		'''EQ Mid 1 Q'''
		return self._eq_mid_1_q

	@eq_mid_1_q.setter
	def eq_mid_1_q(self,value):
		if 0 <= value <= 60:
			self._eq_mid_1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_1_gain(self):
		'''EQ Mid 1 Gain'''
		return self._eq_mid_1_gain

	@eq_mid_1_gain.setter
	def eq_mid_1_gain(self,value):
		if -18 <= value <= 18:
			self._eq_mid_1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_2_cutoff(self):
		'''EQ Mid 2 Cutoff'''
		return self._eq_mid_2_cutoff

	@eq_mid_2_cutoff.setter
	def eq_mid_2_cutoff(self,value):
		if 0 <= value <= 55:
			self._eq_mid_2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_2_q(self):
		'''EQ Mid 2 Q'''
		return self._eq_mid_2_q

	@eq_mid_2_q.setter
	def eq_mid_2_q(self,value):
		if 0 <= value <= 60:
			self._eq_mid_2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def eq_mid_2_gain(self):
		'''EQ Mid 2 Gain'''
		return self._eq_mid_2_gain

	@eq_mid_2_gain.setter
	def eq_mid_2_gain(self,value):
		if -18 <= value <= 18:
			self._eq_mid_2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def direct_mix(self):
		'''Direct Mix'''
		return self._direct_mix

	@direct_mix.setter
	def direct_mix(self,value):
		if 0 <= value <= 50:
			self._direct_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def speaker_simulator(self):
		'''Speaker Simulator'''
		return self._speaker_simulator

	@speaker_simulator.setter
	def speaker_simulator(self,value):
		if 0 <= value <= 1:
			self._speaker_simulator = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

