#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx025.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX025(object):
	category = '''OD Amp Mic'''
	description = '''Treble Booster'''
	short_description = '''Treble Booster'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX025 Treble Booster object'''
		bytes = struct.unpack('<7B',raw[:7])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.drive = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.level = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.tone = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX025 Treble Booster object into raw data'''
		bytes = [0] * 7
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.drive << 5) & 0xE0
		bytes[4] = ((self.drive >> 3) & 0xF) | ((self.level << 4) & 0xF0)
		bytes[5] = ((self.level >> 4) & 0x7) | ((self.tone << 3) & 0xF8)
		bytes[6] = (self.tone >> 5) & 0x3
		return struct.pack('<7B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX025 Treble Booster object'''
		json = []
		json.append(('name','FX025'))
		json.append(('description','Treble Booster'))
		json.append(['drive',self.drive])
		json.append(['level',self.level])
		json.append(['tone',self.tone])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive(self):
		'''Drive'''
		return self._drive

	@drive.setter
	def drive(self,value):
		if 0 <= value <= 99:
			self._drive = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def level(self):
		'''Level'''
		return self._level

	@level.setter
	def level(self,value):
		if 0 <= value <= 100:
			self._level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tone(self):
		'''Tone'''
		return self._tone

	@tone.setter
	def tone(self,value):
		if 0 <= value <= 99:
			self._tone = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

