#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx029.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX029(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Stereo Chorus'''
	short_description = '''Stereo Chorus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX029 Stereo Chorus object'''
		bytes = struct.unpack('<18B',raw[:18])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wet_dry = data
		self.wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0x80) >> 7) | ((bytes[3] & 0x3F) << 1)
		self.lfo_waveform = (bytes[3] >> 6) & 0x1
		data = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.right_lfo_phase = data
		self.lfo_type = (bytes[4] >> 5) & 0x3
		data = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x1F) << 3)
		self.lfo_frequency_mod_source = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x7) << 3)
		data = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[8] >> 3) & 0x1
		self.lfo_bpm = ((bytes[8] & 0xF0) >> 4) | (bytes[9] << 4) | ((bytes[10] & 0x7) << 12)
		self.lfo_bpm_base_note = (bytes[10] >> 3) & 0xF
		self.lfo_bpm_times = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x1F) << 1)
		self.left_pre_delay = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x1F) << 3)
		self.right_pre_delay = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0x1F) << 3)
		self.depth = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		self.depth_mod_source = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x3) << 4)
		data = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		data = (bytes[16] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pre_low_eq_gain = data
		data = bytes[17] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pre_high_eq_gain = data
		self.pre_low_eq_fc = (bytes[17] >> 6) & 0x1
		self.pre_high_eq_fc = (bytes[17] >> 7) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX029 Stereo Chorus object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry << 1) & 0xFE
		bytes[1] = ((self.wet_dry >> 7) & 0x1) | ((self.wet_dry_mod_source & 0x3F) << 1) | ((self.wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = ((self.wet_dry_mod_intensity >> 1) & 0x7F) | ((self.input_trim << 7) & 0x80)
		bytes[3] = ((self.input_trim >> 1) & 0x3F) | ((self.lfo_waveform & 0x1) << 6) | ((self.right_lfo_phase << 7) & 0x80)
		bytes[4] = ((self.right_lfo_phase >> 1) & 0x1F) | ((self.lfo_type & 0x3) << 5) | ((self.common_lfo_phase_offset << 7) & 0x80)
		bytes[5] = ((self.common_lfo_phase_offset >> 1) & 0x1F) | ((self.lfo_frequency << 5) & 0xE0)
		bytes[6] = ((self.lfo_frequency >> 3) & 0x1F) | ((self.lfo_frequency_mod_source << 5) & 0xE0)
		bytes[7] = ((self.lfo_frequency_mod_source >> 3) & 0x7) | ((self.lfo_frequency_mod_intensity << 3) & 0xF8)
		bytes[8] = ((self.lfo_frequency_mod_intensity >> 5) & 0x7) | ((self.lfo_bpm_midi_sync & 0x1) << 3) | ((self.lfo_bpm << 4) & 0xF0)
		bytes[9] = (self.lfo_bpm >> 4) & 0xFF
		bytes[10] = ((self.lfo_bpm >> 7) & 0xC) | ((self.lfo_bpm_base_note & 0xF) << 3) | ((self.lfo_bpm_times << 7) & 0x80)
		bytes[11] = ((self.lfo_bpm_times >> 1) & 0x1F) | ((self.left_pre_delay << 5) & 0xE0)
		bytes[12] = ((self.left_pre_delay >> 3) & 0x1F) | ((self.right_pre_delay << 5) & 0xE0)
		bytes[13] = ((self.right_pre_delay >> 3) & 0x1F) | ((self.depth << 5) & 0xE0)
		bytes[14] = ((self.depth >> 3) & 0xF) | ((self.depth_mod_source << 4) & 0xF0)
		bytes[15] = ((self.depth_mod_source >> 4) & 0x3) | ((self.depth_mod_intensity << 2) & 0xFC)
		bytes[16] = ((self.depth_mod_intensity >> 6) & 0x3) | ((self.pre_low_eq_gain & 0x3F) << 2)
		bytes[17] = (self.pre_high_eq_gain & 0x3F) | ((self.pre_low_eq_fc & 0x1) << 6) | ((self.pre_high_eq_fc & 0x1) << 7)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX029 Stereo Chorus object'''
		json = []
		json.append(('name','FX029'))
		json.append(('description','Stereo Chorus'))
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['right_lfo_phase',self.right_lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['left_pre_delay',self.left_pre_delay])
		json.append(['right_pre_delay',self.right_pre_delay])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['input_trim',self.input_trim])
		json.append(['pre_low_eq_gain',self.pre_low_eq_gain])
		json.append(['pre_high_eq_gain',self.pre_high_eq_gain])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['pre_low_eq_fc',self.pre_low_eq_fc])
		json.append(['pre_high_eq_fc',self.pre_high_eq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if -100 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def right_lfo_phase(self):
		'''Right LFO Phase'''
		return self._right_lfo_phase

	@right_lfo_phase.setter
	def right_lfo_phase(self,value):
		if -18 <= value <= 18:
			self._right_lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def left_pre_delay(self):
		'''Left Pre Delay'''
		return self._left_pre_delay

	@left_pre_delay.setter
	def left_pre_delay(self,value):
		if 0 <= value <= 135:
			self._left_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 135' % value)

#-------------------------------------------------------------------------------
	@property
	def right_pre_delay(self):
		'''Right Pre Delay'''
		return self._right_pre_delay

	@right_pre_delay.setter
	def right_pre_delay(self,value):
		if 0 <= value <= 135:
			self._right_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 135' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_low_eq_gain(self):
		'''Pre Low EQ Gain'''
		return self._pre_low_eq_gain

	@pre_low_eq_gain.setter
	def pre_low_eq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_low_eq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_high_eq_gain(self):
		'''Pre High EQ Gain'''
		return self._pre_high_eq_gain

	@pre_high_eq_gain.setter
	def pre_high_eq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_high_eq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_low_eq_fc(self):
		'''Pre Low EQ Fc'''
		return self._pre_low_eq_fc

	@pre_low_eq_fc.setter
	def pre_low_eq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_low_eq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_high_eq_fc(self):
		'''Pre High EQ Fc'''
		return self._pre_high_eq_fc

	@pre_high_eq_fc.setter
	def pre_high_eq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_high_eq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

