#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx035.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX035(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Multitap Chorus/Delay 4Taps'''
	short_description = '''Multitap Cho/Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX035 Multitap Chorus/Delay 4Taps object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.lfo_type = (bytes[3] >> 5) & 0x3
		data = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		self.tap1_time = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		self.tap1_depth = (bytes[6] >> 3) & 0x1F
		self.tap1_level = bytes[7] & 0x1F
		data = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0x1) << 3)
		if data & 0x8:
			data -= 0x10
		self.tap1_pan = data
		self.tap2_time = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1) << 7)
		self.tap2_depth = (bytes[9] >> 1) & 0x1F
		self.tap2_level = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x7) << 2)
		data = (bytes[10] >> 3) & 0xF
		if data & 0x8:
			data -= 0x10
		self.tap2_pan = data
		self.tap3_time = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x7F) << 1)
		self.tap3_depth = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0xF) << 1)
		self.tap3_level = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x1) << 4)
		data = (bytes[13] >> 1) & 0xF
		if data & 0x8:
			data -= 0x10
		self.tap3_pan = data
		self.tap4_time = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x1F) << 3)
		self.tap4_depth = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x3) << 3)
		self.tap4_level = (bytes[15] >> 2) & 0x1F
		data = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x7) << 1)
		if data & 0x8:
			data -= 0x10
		self.tap4_pan = data
		data = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.tap1_feedback = data
		self.tap1_feedback_mod_source = ((bytes[17] & 0xF8) >> 3) | ((bytes[18] & 0x1) << 5)
		data = ((bytes[18] & 0xFE) >> 1) | ((bytes[19] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.tap1_feedback_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX035 Multitap Chorus/Delay 4Taps object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.lfo_type & 0x3) << 5) | ((self.common_lfo_phase_offset << 7) & 0x80)
		bytes[4] = ((self.common_lfo_phase_offset >> 1) & 0x1F) | ((self.lfo_frequency << 5) & 0xE0)
		bytes[5] = ((self.lfo_frequency >> 3) & 0x7) | ((self.tap1_time << 3) & 0xF8)
		bytes[6] = ((self.tap1_time >> 5) & 0x7) | ((self.tap1_depth & 0x1F) << 3)
		bytes[7] = (self.tap1_level & 0x1F) | ((self.tap1_pan << 5) & 0xE0)
		bytes[8] = ((self.tap1_pan >> 3) & 0x1) | ((self.tap2_time << 1) & 0xFE)
		bytes[9] = ((self.tap2_time >> 7) & 0x1) | ((self.tap2_depth & 0x1F) << 1) | ((self.tap2_level << 6) & 0xC0)
		bytes[10] = ((self.tap2_level >> 2) & 0x7) | ((self.tap2_pan & 0xF) << 3) | ((self.tap3_time << 7) & 0x80)
		bytes[11] = ((self.tap3_time >> 1) & 0x7F) | ((self.tap3_depth << 7) & 0x80)
		bytes[12] = ((self.tap3_depth >> 1) & 0xF) | ((self.tap3_level << 4) & 0xF0)
		bytes[13] = ((self.tap3_level >> 4) & 0x1) | ((self.tap3_pan & 0xF) << 1) | ((self.tap4_time << 5) & 0xE0)
		bytes[14] = ((self.tap4_time >> 3) & 0x1F) | ((self.tap4_depth << 5) & 0xE0)
		bytes[15] = ((self.tap4_depth >> 3) & 0x3) | ((self.tap4_level & 0x1F) << 2) | ((self.tap4_pan << 7) & 0x80)
		bytes[16] = ((self.tap4_pan >> 1) & 0x7) | ((self.tap1_feedback << 3) & 0xF8)
		bytes[17] = ((self.tap1_feedback >> 5) & 0x7) | ((self.tap1_feedback_mod_source << 3) & 0xF8)
		bytes[18] = ((self.tap1_feedback_mod_source >> 5) & 0x1) | ((self.tap1_feedback_mod_intensity << 1) & 0xFE)
		bytes[19] = (self.tap1_feedback_mod_intensity >> 7) & 0x1
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX035 Multitap Chorus/Delay 4Taps object'''
		json = []
		json.append(('name','FX035'))
		json.append(('description','Multitap Chorus/Delay 4Taps'))
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['tap1_time',self.tap1_time])
		json.append(['tap1_depth',self.tap1_depth])
		json.append(['tap1_level',self.tap1_level])
		json.append(['tap1_pan',self.tap1_pan])
		json.append(['tap2_time',self.tap2_time])
		json.append(['tap2_depth',self.tap2_depth])
		json.append(['tap2_level',self.tap2_level])
		json.append(['tap2_pan',self.tap2_pan])
		json.append(['tap3_time',self.tap3_time])
		json.append(['tap3_depth',self.tap3_depth])
		json.append(['tap3_level',self.tap3_level])
		json.append(['tap3_pan',self.tap3_pan])
		json.append(['tap4_time',self.tap4_time])
		json.append(['tap4_depth',self.tap4_depth])
		json.append(['tap4_level',self.tap4_level])
		json.append(['tap4_pan',self.tap4_pan])
		json.append(['tap1_feedback',self.tap1_feedback])
		json.append(['tap1_feedback_mod_intensity',self.tap1_feedback_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['tap1_feedback_mod_source',self.tap1_feedback_mod_source])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 63:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_time(self):
		'''Tap1 Time'''
		return self._tap1_time

	@tap1_time.setter
	def tap1_time(self,value):
		if 0 <= value <= 190:
			self._tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 190' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_depth(self):
		'''Tap1 Depth'''
		return self._tap1_depth

	@tap1_depth.setter
	def tap1_depth(self,value):
		if 0 <= value <= 30:
			self._tap1_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_level(self):
		'''Tap1 Level'''
		return self._tap1_level

	@tap1_level.setter
	def tap1_level(self,value):
		if 0 <= value <= 30:
			self._tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_pan(self):
		'''Tap1 Pan'''
		return self._tap1_pan

	@tap1_pan.setter
	def tap1_pan(self,value):
		if -6 <= value <= 6:
			self._tap1_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_time(self):
		'''Tap2 Time'''
		return self._tap2_time

	@tap2_time.setter
	def tap2_time(self,value):
		if 0 <= value <= 190:
			self._tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 190' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_depth(self):
		'''Tap2 Depth'''
		return self._tap2_depth

	@tap2_depth.setter
	def tap2_depth(self,value):
		if 0 <= value <= 30:
			self._tap2_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_level(self):
		'''Tap2 Level'''
		return self._tap2_level

	@tap2_level.setter
	def tap2_level(self,value):
		if 0 <= value <= 30:
			self._tap2_level = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_pan(self):
		'''Tap2 Pan'''
		return self._tap2_pan

	@tap2_pan.setter
	def tap2_pan(self,value):
		if -6 <= value <= 6:
			self._tap2_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_time(self):
		'''Tap3 Time'''
		return self._tap3_time

	@tap3_time.setter
	def tap3_time(self,value):
		if 0 <= value <= 190:
			self._tap3_time = value
		else:
			raise ValueError('%s not 0 <= value <= 190' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_depth(self):
		'''Tap3 Depth'''
		return self._tap3_depth

	@tap3_depth.setter
	def tap3_depth(self,value):
		if 0 <= value <= 30:
			self._tap3_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_level(self):
		'''Tap3 Level'''
		return self._tap3_level

	@tap3_level.setter
	def tap3_level(self,value):
		if 0 <= value <= 30:
			self._tap3_level = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_pan(self):
		'''Tap3 Pan'''
		return self._tap3_pan

	@tap3_pan.setter
	def tap3_pan(self,value):
		if -6 <= value <= 6:
			self._tap3_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_time(self):
		'''Tap4 Time'''
		return self._tap4_time

	@tap4_time.setter
	def tap4_time(self,value):
		if 0 <= value <= 190:
			self._tap4_time = value
		else:
			raise ValueError('%s not 0 <= value <= 190' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_depth(self):
		'''Tap4 Depth'''
		return self._tap4_depth

	@tap4_depth.setter
	def tap4_depth(self,value):
		if 0 <= value <= 30:
			self._tap4_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_level(self):
		'''Tap4 Level'''
		return self._tap4_level

	@tap4_level.setter
	def tap4_level(self,value):
		if 0 <= value <= 30:
			self._tap4_level = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_pan(self):
		'''Tap4 Pan'''
		return self._tap4_pan

	@tap4_pan.setter
	def tap4_pan(self,value):
		if -6 <= value <= 6:
			self._tap4_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback(self):
		'''Tap1 Feedback'''
		return self._tap1_feedback

	@tap1_feedback.setter
	def tap1_feedback(self,value):
		if -100 <= value <= 100:
			self._tap1_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback_mod_source(self):
		'''Tap1 Feedback Mod.Source'''
		return self._tap1_feedback_mod_source

	@tap1_feedback_mod_source.setter
	def tap1_feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._tap1_feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback_mod_intensity(self):
		'''Tap1 Feedback Mod.Intensity'''
		return self._tap1_feedback_mod_intensity

	@tap1_feedback_mod_intensity.setter
	def tap1_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._tap1_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

