#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx037.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX037(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Polysix Ensemble'''
	short_description = '''Polysix Ensemble'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX037 Polysix Ensemble object'''
		bytes = struct.unpack('<7B',raw[:7])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.depth = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.depth_mod_source = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x3) << 4)
		data = ((bytes[5] & 0xFC) >> 2) | ((bytes[6] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX037 Polysix Ensemble object into raw data'''
		bytes = [0] * 7
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.depth << 5) & 0xE0
		bytes[4] = ((self.depth >> 3) & 0xF) | ((self.depth_mod_source << 4) & 0xF0)
		bytes[5] = ((self.depth_mod_source >> 4) & 0x3) | ((self.depth_mod_intensity << 2) & 0xFC)
		bytes[6] = (self.depth_mod_intensity >> 6) & 0x3
		return struct.pack('<7B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX037 Polysix Ensemble object'''
		json = []
		json.append(('name','FX037'))
		json.append(('description','Polysix Ensemble'))
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

