#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx043.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX043(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Small Phaser'''
	short_description = '''Small Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX043 Small Phaser object'''
		bytes = struct.unpack('<7B',raw[:7])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.speed = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		self.color = (bytes[4] >> 5) & 0x1
		self.speed_mod_source = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0xF) << 2)
		data = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX043 Small Phaser object into raw data'''
		bytes = [0] * 7
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.speed << 5) & 0xE0
		bytes[4] = ((self.speed >> 3) & 0x1F) | ((self.color & 0x1) << 5) | ((self.speed_mod_source << 6) & 0xC0)
		bytes[5] = ((self.speed_mod_source >> 2) & 0xF) | ((self.speed_mod_intensity << 4) & 0xF0)
		bytes[6] = (self.speed_mod_intensity >> 4) & 0xF
		return struct.pack('<7B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX043 Small Phaser object'''
		json = []
		json.append(('name','FX043'))
		json.append(('description','Small Phaser'))
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['color',self.color])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 5 <= value <= 210:
			self._speed = value
		else:
			raise ValueError('%s not 5 <= value <= 210' % value)

#-------------------------------------------------------------------------------
	@property
	def color(self):
		'''Color'''
		return self._color

	@color.setter
	def color(self,value):
		if 0 <= value <= 1:
			self._color = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -105 <= value <= 105:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -105 <= value <= 105' % value)

