#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx050.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

#===============================================================================
class FX050(object):
	category = '''Mod./P.Shift'''
	description = '''Stereo Auto Fade Modulation'''
	short_description = '''St. Auto Fade Mod.'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX050 Stereo Auto Fade Modulation object'''
		bytes = struct.unpack('<19B',raw[:19])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wet_dry = data
		self.wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.auto_fade_mod_source = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xF) << 2)
		self.fade_in_rate = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.fade_in_delay = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		self.lfo_type = (bytes[6] >> 3) & 0x3
		data = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_waveform = (bytes[7] >> 3) & 0x1
		data = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.lfo_shape = data
		data = (bytes[8] >> 4) & 0xF
		if data & 0x8:
			data -= 0x10
		self.lfo_phase = data
		self.lfo_mod_ = bytes[9] & 0x1
		self.lfo_frequency = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0x1) << 7)
		self.lfo_frequency_mod_source = (bytes[10] >> 1) & 0x3F
		data = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.l_delay_time = ((bytes[11] & 0x80) >> 7) | (bytes[12] << 1) | ((bytes[13] & 0xF) << 9)
		self.r_delay_time = ((bytes[13] & 0xF0) >> 4) | (bytes[14] << 4) | ((bytes[15] & 0x1) << 12)
		self.depth = ((bytes[15] & 0xFE) >> 1) | ((bytes[16] & 0x1) << 7)
		data = ((bytes[16] & 0xFE) >> 1) | ((bytes[17] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = (bytes[17] >> 1) & 0x7F
		self.wet_dry_mod_ = bytes[18] & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX050 Stereo Auto Fade Modulation object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.wet_dry << 1) & 0xFE
		bytes[1] = ((self.wet_dry >> 7) & 0x1) | ((self.wet_dry_mod_source & 0x3F) << 1) | ((self.wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = (self.auto_fade_mod_source << 6) & 0xC0
		bytes[4] = ((self.auto_fade_mod_source >> 2) & 0xF) | ((self.fade_in_rate << 4) & 0xF0)
		bytes[5] = ((self.fade_in_rate >> 4) & 0x7) | ((self.fade_in_delay << 3) & 0xF8)
		bytes[6] = ((self.fade_in_delay >> 5) & 0x7) | ((self.lfo_type & 0x3) << 3) | ((self.common_lfo_phase_offset << 5) & 0xE0)
		bytes[7] = ((self.common_lfo_phase_offset >> 3) & 0x7) | ((self.lfo_waveform & 0x1) << 3) | ((self.lfo_shape << 4) & 0xF0)
		bytes[8] = ((self.lfo_shape >> 4) & 0xF) | ((self.lfo_phase & 0xF) << 4)
		bytes[9] = (self.lfo_mod_ & 0x1) | ((self.lfo_frequency << 1) & 0xFE)
		bytes[10] = ((self.lfo_frequency >> 7) & 0x1) | ((self.lfo_frequency_mod_source & 0x3F) << 1) | ((self.lfo_frequency_mod_intensity << 7) & 0x80)
		bytes[11] = ((self.lfo_frequency_mod_intensity >> 1) & 0x7F) | ((self.l_delay_time << 7) & 0x80)
		bytes[12] = (self.l_delay_time >> 1) & 0xFF
		bytes[13] = ((self.l_delay_time >> 15) & 0x9) | ((self.r_delay_time << 4) & 0xF0)
		bytes[14] = (self.r_delay_time >> 4) & 0xFF
		bytes[15] = ((self.r_delay_time >> 1) & 0xC) | ((self.depth << 1) & 0xFE)
		bytes[16] = ((self.depth >> 7) & 0x1) | ((self.feedback << 1) & 0xFE)
		bytes[17] = ((self.feedback >> 7) & 0x1) | ((self.high_damp & 0x7F) << 1)
		bytes[18] = self.wet_dry_mod_ & 0x1
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX050 Stereo Auto Fade Modulation object'''
		json = []
		json.append(('name','FX050'))
		json.append(('description','Stereo Auto Fade Modulation'))
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_shape',self.lfo_shape])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_mod_',self.lfo_mod_])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['depth',self.depth])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['wet_dry_mod_',self.wet_dry_mod_])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['auto_fade_mod_source',self.auto_fade_mod_source])
		json.append(['fade_in_rate',self.fade_in_rate])
		json.append(['fade_in_delay',self.fade_in_delay])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if -100 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def auto_fade_mod_source(self):
		'''Auto Fade Mod.Source'''
		return self._auto_fade_mod_source

	@auto_fade_mod_source.setter
	def auto_fade_mod_source(self,value):
		if 0 <= value <= 37:
			self._auto_fade_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def fade_in_rate(self):
		'''Fade-In Rate'''
		return self._fade_in_rate

	@fade_in_rate.setter
	def fade_in_rate(self,value):
		if 1 <= value <= 100:
			self._fade_in_rate = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def fade_in_delay(self):
		'''Fade-In Delay'''
		return self._fade_in_delay

	@fade_in_delay.setter
	def fade_in_delay(self,value):
		if 0 <= value <= 200:
			self._fade_in_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_shape(self):
		'''LFO Shape'''
		return self._lfo_shape

	@lfo_shape.setter
	def lfo_shape(self,value):
		if -100 <= value <= 100:
			self._lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -4 <= value <= 4:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -4 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_mod_(self):
		'''LFO Mod.'''
		return self._lfo_mod_

	@lfo_mod_.setter
	def lfo_mod_(self,value):
		if 0 <= value <= 1:
			self._lfo_mod_ = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 5000:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5000' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 5000:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5000' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_(self):
		'''Wet/Dry Mod.'''
		return self._wet_dry_mod_

	@wet_dry_mod_.setter
	def wet_dry_mod_(self,value):
		if 0 <= value <= 1:
			self._wet_dry_mod_ = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

