#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx053.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX053(object):
	category = '''Mod./P.Shift'''
	description = '''Scratch'''
	short_description = '''Scratch'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX053 Scratch object'''
		bytes = struct.unpack('<9B',raw[:9])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.scratch_mod_source = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x7) << 3)
		self.scratch_response = ((bytes[4] & 0xF8) >> 3) | ((bytes[5] & 0x3) << 5)
		self.envelope_select = (bytes[5] >> 2) & 0x1
		self.envelope_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		self.envelope_threshold = (bytes[6] >> 1) & 0x7F
		self.envelope_response = bytes[7] & 0x7F
		self.direct_mix = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x1) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX053 Scratch object into raw data'''
		bytes = [0] * 9
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.scratch_mod_source << 5) & 0xE0
		bytes[4] = ((self.scratch_mod_source >> 3) & 0x7) | ((self.scratch_response << 3) & 0xF8)
		bytes[5] = ((self.scratch_response >> 5) & 0x3) | ((self.envelope_select & 0x1) << 2) | ((self.envelope_mod_source << 3) & 0xF8)
		bytes[6] = ((self.envelope_mod_source >> 5) & 0x1) | ((self.envelope_threshold & 0x7F) << 1)
		bytes[7] = (self.envelope_response & 0x7F) | ((self.direct_mix << 7) & 0x80)
		bytes[8] = (self.direct_mix >> 1) & 0x1
		return struct.pack('<9B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX053 Scratch object'''
		json = []
		json.append(('name','FX053'))
		json.append(('description','Scratch'))
		json.append(['scratch_mod_source',self.scratch_mod_source])
		json.append(['scratch_response',self.scratch_response])
		json.append(['envelope_select',self.envelope_select])
		json.append(['envelope_mod_source',self.envelope_mod_source])
		json.append(['envelope_threshold',self.envelope_threshold])
		json.append(['envelope_response',self.envelope_response])
		json.append(['direct_mix',self.direct_mix])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def scratch_mod_source(self):
		'''Scratch Mod.Source'''
		return self._scratch_mod_source

	@scratch_mod_source.setter
	def scratch_mod_source(self,value):
		if 0 <= value <= 37:
			self._scratch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def scratch_response(self):
		'''Scratch Response'''
		return self._scratch_response

	@scratch_response.setter
	def scratch_response(self,value):
		if 0 <= value <= 100:
			self._scratch_response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select(self):
		'''Envelope Select'''
		return self._envelope_select

	@envelope_select.setter
	def envelope_select(self,value):
		if 0 <= value <= 1:
			self._envelope_select = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_mod_source(self):
		'''Envelope Mod.Source'''
		return self._envelope_mod_source

	@envelope_mod_source.setter
	def envelope_mod_source(self,value):
		if 0 <= value <= 37:
			self._envelope_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_threshold(self):
		'''Envelope Threshold'''
		return self._envelope_threshold

	@envelope_threshold.setter
	def envelope_threshold(self,value):
		if 0 <= value <= 100:
			self._envelope_threshold = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_response(self):
		'''Envelope Response'''
		return self._envelope_response

	@envelope_response.setter
	def envelope_response(self,value):
		if 0 <= value <= 100:
			self._envelope_response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def direct_mix(self):
		'''Direct Mix'''
		return self._direct_mix

	@direct_mix.setter
	def direct_mix(self,value):
		if 0 <= value <= 2:
			self._direct_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

