#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx065.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

#===============================================================================
class FX065(object):
	category = '''Mod./P.Shift'''
	description = '''Organ Vibrato/Chorus'''
	short_description = '''Organ Vib/Chorus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX065 Organ Vibrato/Chorus object'''
		bytes = struct.unpack('<14B',raw[:14])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.mode = (bytes[3] >> 5) & 0x1
		self.type = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1) << 2)
		self.type_mod_source = (bytes[4] >> 1) & 0x3F
		data = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x7) << 1)
		if data & 0x8:
			data -= 0x10
		self.type_mod_intensity = data
		self.mix = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.mix_mod_source = (bytes[6] >> 2) & 0x3F
		data = bytes[7] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.mix_mod_intensity = data
		self.depth = bytes[8] & 0x7F
		self.depth_mod_source = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x1F) << 1)
		data = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		self.speed = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1F) << 3)
		self.speed_mod_source = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x7) << 3)
		data = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX065 Organ Vibrato/Chorus object into raw data'''
		bytes = [0] * 14
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.mode & 0x1) << 5) | ((self.type << 6) & 0xC0)
		bytes[4] = ((self.type >> 2) & 0x1) | ((self.type_mod_source & 0x3F) << 1) | ((self.type_mod_intensity << 7) & 0x80)
		bytes[5] = ((self.type_mod_intensity >> 1) & 0x7) | ((self.mix << 3) & 0xF8)
		bytes[6] = ((self.mix >> 5) & 0x3) | ((self.mix_mod_source & 0x3F) << 2)
		bytes[7] = self.mix_mod_intensity & 0xFF
		bytes[8] = (self.depth & 0x7F) | ((self.depth_mod_source << 7) & 0x80)
		bytes[9] = ((self.depth_mod_source >> 1) & 0x1F) | ((self.depth_mod_intensity << 5) & 0xE0)
		bytes[10] = ((self.depth_mod_intensity >> 3) & 0x1F) | ((self.speed << 5) & 0xE0)
		bytes[11] = ((self.speed >> 3) & 0x1F) | ((self.speed_mod_source << 5) & 0xE0)
		bytes[12] = ((self.speed_mod_source >> 3) & 0x7) | ((self.speed_mod_intensity << 3) & 0xF8)
		bytes[13] = (self.speed_mod_intensity >> 5) & 0x7
		return struct.pack('<14B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX065 Organ Vibrato/Chorus object'''
		json = []
		json.append(('name','FX065'))
		json.append(('description','Organ Vibrato/Chorus'))
		json.append(['input_trim',self.input_trim])
		json.append(['mode',self.mode])
		json.append(['type',self.type])
		json.append(['type_mod_source',self.type_mod_source])
		json.append(['type_mod_intensity',self.type_mod_intensity])
		json.append(['mix',self.mix])
		json.append(['mix_mod_source',self.mix_mod_source])
		json.append(['mix_mod_intensity',self.mix_mod_intensity])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['speed_mod_intensity_display',self.speed_mod_intensity_display])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 1:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 5:
			self._type = value
		else:
			raise ValueError('%s not 0 <= value <= 5' % value)

#-------------------------------------------------------------------------------
	@property
	def type_mod_source(self):
		'''Type Mod.Source'''
		return self._type_mod_source

	@type_mod_source.setter
	def type_mod_source(self,value):
		if 0 <= value <= 37:
			self._type_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def type_mod_intensity(self):
		'''Type Mod.Intensity'''
		return self._type_mod_intensity

	@type_mod_intensity.setter
	def type_mod_intensity(self,value):
		if -5 <= value <= 5:
			self._type_mod_intensity = value
		else:
			raise ValueError('%s not -5 <= value <= 5' % value)

#-------------------------------------------------------------------------------
	@property
	def mix(self):
		'''Mix'''
		return self._mix

	@mix.setter
	def mix(self,value):
		if 0 <= value <= 100:
			self._mix = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mix_mod_source(self):
		'''Mix Mod.Source'''
		return self._mix_mod_source

	@mix_mod_source.setter
	def mix_mod_source(self,value):
		if 0 <= value <= 37:
			self._mix_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def mix_mod_intensity(self):
		'''Mix Mod.Intensity'''
		return self._mix_mod_intensity

	@mix_mod_intensity.setter
	def mix_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._mix_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 1 <= value <= 230:
			self._speed = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def speed_mod_intensity_display(self):
		'''Speed Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._speed_mod_intensity)

