#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx066.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX066(object):
	category = '''Mod./P.Shift'''
	description = '''Rotary Speaker'''
	short_description = '''Rotary Speaker'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX066 Rotary Speaker object'''
		bytes = struct.unpack('<13B',raw[:13])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode_switch = (bytes[3] >> 5) & 0x1
		self.mode_switch_mod_source = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xF) << 2)
		self.mode_switch_mode = (bytes[4] >> 4) & 0x1
		self.speed_switch = (bytes[4] >> 5) & 0x1
		self.speed_switch_mod_source = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0xF) << 2)
		self.speed_switch_mode = (bytes[5] >> 4) & 0x1
		self.manual_speed_control_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		self.horn_acceleration = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.horn_ratio = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		self.rotor_acceleration = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.rotor_ratio = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0x1) << 7)
		self.horn_rotor_balance = (bytes[10] >> 1) & 0x7F
		self.mic_distance = bytes[11] & 0x7F
		self.mic_spread = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x3F) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX066 Rotary Speaker object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode_switch & 0x1) << 5) | ((self.mode_switch_mod_source << 6) & 0xC0)
		bytes[4] = ((self.mode_switch_mod_source >> 2) & 0xF) | ((self.mode_switch_mode & 0x1) << 4) | ((self.speed_switch & 0x1) << 5) | ((self.speed_switch_mod_source << 6) & 0xC0)
		bytes[5] = ((self.speed_switch_mod_source >> 2) & 0xF) | ((self.speed_switch_mode & 0x1) << 4) | ((self.manual_speed_control_mod_source << 5) & 0xE0)
		bytes[6] = ((self.manual_speed_control_mod_source >> 3) & 0x7) | ((self.horn_acceleration << 3) & 0xF8)
		bytes[7] = ((self.horn_acceleration >> 5) & 0x3) | ((self.horn_ratio << 2) & 0xFC)
		bytes[8] = ((self.horn_ratio >> 6) & 0x3) | ((self.rotor_acceleration << 2) & 0xFC)
		bytes[9] = ((self.rotor_acceleration >> 6) & 0x1) | ((self.rotor_ratio << 1) & 0xFE)
		bytes[10] = ((self.rotor_ratio >> 7) & 0x1) | ((self.horn_rotor_balance & 0x7F) << 1)
		bytes[11] = (self.mic_distance & 0x7F) | ((self.mic_spread << 7) & 0x80)
		bytes[12] = (self.mic_spread >> 1) & 0x3F
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX066 Rotary Speaker object'''
		json = []
		json.append(('name','FX066'))
		json.append(('description','Rotary Speaker'))
		json.append(['mode_switch',self.mode_switch])
		json.append(['mode_switch_mod_source',self.mode_switch_mod_source])
		json.append(['mode_switch_mode',self.mode_switch_mode])
		json.append(['speed_switch',self.speed_switch])
		json.append(['speed_switch_mod_source',self.speed_switch_mod_source])
		json.append(['speed_switch_mode',self.speed_switch_mode])
		json.append(['manual_speed_control_mod_source',self.manual_speed_control_mod_source])
		json.append(['rotor_acceleration',self.rotor_acceleration])
		json.append(['rotor_ratio',self.rotor_ratio])
		json.append(['horn_acceleration',self.horn_acceleration])
		json.append(['horn_ratio',self.horn_ratio])
		json.append(['horn_rotor_balance',self.horn_rotor_balance])
		json.append(['mic_distance',self.mic_distance])
		json.append(['mic_spread',self.mic_spread])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch(self):
		'''Mode Switch'''
		return self._mode_switch

	@mode_switch.setter
	def mode_switch(self,value):
		if 0 <= value <= 1:
			self._mode_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch_mod_source(self):
		'''Mode Switch Mod.Source'''
		return self._mode_switch_mod_source

	@mode_switch_mod_source.setter
	def mode_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._mode_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch_mode(self):
		'''Mode Switch Mode'''
		return self._mode_switch_mode

	@mode_switch_mode.setter
	def mode_switch_mode(self,value):
		if 0 <= value <= 1:
			self._mode_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch(self):
		'''Speed Switch'''
		return self._speed_switch

	@speed_switch.setter
	def speed_switch(self,value):
		if 0 <= value <= 1:
			self._speed_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch_mod_source(self):
		'''Speed Switch Mod.Source'''
		return self._speed_switch_mod_source

	@speed_switch_mod_source.setter
	def speed_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch_mode(self):
		'''Speed Switch Mode'''
		return self._speed_switch_mode

	@speed_switch_mode.setter
	def speed_switch_mode(self,value):
		if 0 <= value <= 1:
			self._speed_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def manual_speed_control_mod_source(self):
		'''Manual Speed Control Mod.Source'''
		return self._manual_speed_control_mod_source

	@manual_speed_control_mod_source.setter
	def manual_speed_control_mod_source(self,value):
		if 0 <= value <= 37:
			self._manual_speed_control_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_acceleration(self):
		'''Horn Acceleration'''
		return self._horn_acceleration

	@horn_acceleration.setter
	def horn_acceleration(self,value):
		if 0 <= value <= 100:
			self._horn_acceleration = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_ratio(self):
		'''Horn Ratio'''
		return self._horn_ratio

	@horn_ratio.setter
	def horn_ratio(self,value):
		if 49 <= value <= 200:
			self._horn_ratio = value
		else:
			raise ValueError('%s not 49 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def rotor_acceleration(self):
		'''Rotor Acceleration'''
		return self._rotor_acceleration

	@rotor_acceleration.setter
	def rotor_acceleration(self,value):
		if 0 <= value <= 100:
			self._rotor_acceleration = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def rotor_ratio(self):
		'''Rotor Ratio'''
		return self._rotor_ratio

	@rotor_ratio.setter
	def rotor_ratio(self,value):
		if 49 <= value <= 200:
			self._rotor_ratio = value
		else:
			raise ValueError('%s not 49 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_rotor_balance(self):
		'''Horn/Rotor Balance'''
		return self._horn_rotor_balance

	@horn_rotor_balance.setter
	def horn_rotor_balance(self,value):
		if 0 <= value <= 100:
			self._horn_rotor_balance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_distance(self):
		'''Mic Distance'''
		return self._mic_distance

	@mic_distance.setter
	def mic_distance(self,value):
		if 0 <= value <= 100:
			self._mic_distance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_spread(self):
		'''Mic Spread'''
		return self._mic_spread

	@mic_spread.setter
	def mic_spread(self,value):
		if 0 <= value <= 100:
			self._mic_spread = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

