#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx067.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX067(object):
	category = '''Mod./P.Shift'''
	description = '''Stereo EP Tone+Tremolo'''
	short_description = '''St.EP Tone+Trml'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX067 Stereo EP Tone+Tremolo object'''
		bytes = struct.unpack('<13B',raw[:13])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.volume = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.treble = data
		data = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.bass = data
		self.vibrato_switch = (bytes[6] >> 4) & 0x1
		self.vibrato_switch_mod_source = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x7) << 3)
		self.vibrato_switch_mode = (bytes[7] >> 3) & 0x1
		self.vibrato_intensity = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x7) << 4)
		self.vibrato_intensity_mod_source = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x1) << 5)
		data = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.vibrato_intensity_mod_intensity = data
		self.vibrato_speed = (bytes[10] >> 1) & 0x7F
		self.vibrato_speed_mod_source = bytes[11] & 0x3F
		data = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.vibrato_speed_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX067 Stereo EP Tone+Tremolo object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.volume << 5) & 0xE0
		bytes[4] = ((self.volume >> 3) & 0xF) | ((self.treble << 4) & 0xF0)
		bytes[5] = ((self.treble >> 4) & 0xF) | ((self.bass << 4) & 0xF0)
		bytes[6] = ((self.bass >> 4) & 0xF) | ((self.vibrato_switch & 0x1) << 4) | ((self.vibrato_switch_mod_source << 5) & 0xE0)
		bytes[7] = ((self.vibrato_switch_mod_source >> 3) & 0x7) | ((self.vibrato_switch_mode & 0x1) << 3) | ((self.vibrato_intensity << 4) & 0xF0)
		bytes[8] = ((self.vibrato_intensity >> 4) & 0x7) | ((self.vibrato_intensity_mod_source << 3) & 0xF8)
		bytes[9] = ((self.vibrato_intensity_mod_source >> 5) & 0x1) | ((self.vibrato_intensity_mod_intensity << 1) & 0xFE)
		bytes[10] = ((self.vibrato_intensity_mod_intensity >> 7) & 0x1) | ((self.vibrato_speed & 0x7F) << 1)
		bytes[11] = (self.vibrato_speed_mod_source & 0x3F) | ((self.vibrato_speed_mod_intensity << 6) & 0xC0)
		bytes[12] = (self.vibrato_speed_mod_intensity >> 2) & 0x3F
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX067 Stereo EP Tone+Tremolo object'''
		json = []
		json.append(('name','FX067'))
		json.append(('description','Stereo EP Tone+Tremolo'))
		json.append(['volume',self.volume])
		json.append(['treble',self.treble])
		json.append(['bass',self.bass])
		json.append(['vibrato_switch',self.vibrato_switch])
		json.append(['vibrato_switch_mod_source',self.vibrato_switch_mod_source])
		json.append(['vibrato_switch_mode',self.vibrato_switch_mode])
		json.append(['vibrato_intensity',self.vibrato_intensity])
		json.append(['vibrato_intensity_mod_source',self.vibrato_intensity_mod_source])
		json.append(['vibrato_intensity_mod_intensity',self.vibrato_intensity_mod_intensity])
		json.append(['vibrato_speed',self.vibrato_speed])
		json.append(['vibrato_speed_mod_source',self.vibrato_speed_mod_source])
		json.append(['vibrato_speed_mod_intensity',self.vibrato_speed_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 100:
			self._volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def treble(self):
		'''Treble'''
		return self._treble

	@treble.setter
	def treble(self,value):
		if -100 <= value <= 100:
			self._treble = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass(self):
		'''Bass'''
		return self._bass

	@bass.setter
	def bass(self,value):
		if -100 <= value <= 100:
			self._bass = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_switch(self):
		'''Vibrato Switch'''
		return self._vibrato_switch

	@vibrato_switch.setter
	def vibrato_switch(self,value):
		if 0 <= value <= 1:
			self._vibrato_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_switch_mod_source(self):
		'''Vibrato Switch Mod.Source'''
		return self._vibrato_switch_mod_source

	@vibrato_switch_mod_source.setter
	def vibrato_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._vibrato_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_switch_mode(self):
		'''Vibrato Switch Mode'''
		return self._vibrato_switch_mode

	@vibrato_switch_mode.setter
	def vibrato_switch_mode(self,value):
		if 0 <= value <= 1:
			self._vibrato_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_intensity(self):
		'''Vibrato Intensity'''
		return self._vibrato_intensity

	@vibrato_intensity.setter
	def vibrato_intensity(self,value):
		if 0 <= value <= 100:
			self._vibrato_intensity = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_intensity_mod_source(self):
		'''Vibrato Intensity Mod.Source'''
		return self._vibrato_intensity_mod_source

	@vibrato_intensity_mod_source.setter
	def vibrato_intensity_mod_source(self,value):
		if 0 <= value <= 37:
			self._vibrato_intensity_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_intensity_mod_intensity(self):
		'''Vibrato Intensity Mod.Intensity'''
		return self._vibrato_intensity_mod_intensity

	@vibrato_intensity_mod_intensity.setter
	def vibrato_intensity_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._vibrato_intensity_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_speed(self):
		'''Vibrato Speed'''
		return self._vibrato_speed

	@vibrato_speed.setter
	def vibrato_speed(self,value):
		if 0 <= value <= 100:
			self._vibrato_speed = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_speed_mod_source(self):
		'''Vibrato Speed Mod.Source'''
		return self._vibrato_speed_mod_source

	@vibrato_speed_mod_source.setter
	def vibrato_speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._vibrato_speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def vibrato_speed_mod_intensity(self):
		'''Vibrato Speed Mod.Intensity'''
		return self._vibrato_speed_mod_intensity

	@vibrato_speed_mod_intensity.setter
	def vibrato_speed_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._vibrato_speed_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

