#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx068.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX068(object):
	category = '''Delay'''
	description = '''Left, Center, Right Delay'''
	short_description = '''L/C/R Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX068 Left, Center, Right Delay object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.l_delay_time = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0x1) << 11)
		self.l_delay_level = (bytes[5] >> 1) & 0x3F
		self.c_delay_time = ((bytes[5] & 0x80) >> 7) | (bytes[6] << 1) | ((bytes[7] & 0x7) << 9)
		self.c_delay_level = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x1) << 5)
		self.r_delay_time = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1F) << 7)
		self.r_delay_level = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x7) << 3)
		data = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.feedback_mod_source = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x1) << 5)
		data = ((bytes[12] & 0xFE) >> 1) | ((bytes[13] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data
		self.high_damp = (bytes[13] >> 1) & 0x7F
		self.low_damp = bytes[14] & 0x7F
		self.input_trim_mod_source = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x1F) << 1)
		data = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		self.spread = ((bytes[16] & 0xE0) >> 5) | ((bytes[17] & 0x7) << 3)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX068 Left, Center, Right Delay object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.l_delay_time << 5) & 0xE0
		bytes[4] = (self.l_delay_time >> 3) & 0xFF
		bytes[5] = ((self.l_delay_time >> 1) & 0xB) | ((self.l_delay_level & 0x3F) << 1) | ((self.c_delay_time << 7) & 0x80)
		bytes[6] = (self.c_delay_time >> 1) & 0xFF
		bytes[7] = ((self.c_delay_time >> 7) & 0x9) | ((self.c_delay_level << 3) & 0xF8)
		bytes[8] = ((self.c_delay_level >> 5) & 0x1) | ((self.r_delay_time << 1) & 0xFE)
		bytes[9] = ((self.r_delay_time >> 7) & 0x1F) | ((self.r_delay_level << 5) & 0xE0)
		bytes[10] = ((self.r_delay_level >> 3) & 0x7) | ((self.feedback << 3) & 0xF8)
		bytes[11] = ((self.feedback >> 5) & 0x7) | ((self.feedback_mod_source << 3) & 0xF8)
		bytes[12] = ((self.feedback_mod_source >> 5) & 0x1) | ((self.feedback_mod_intensity << 1) & 0xFE)
		bytes[13] = ((self.feedback_mod_intensity >> 7) & 0x1) | ((self.high_damp & 0x7F) << 1)
		bytes[14] = (self.low_damp & 0x7F) | ((self.input_trim_mod_source << 7) & 0x80)
		bytes[15] = ((self.input_trim_mod_source >> 1) & 0x1F) | ((self.input_trim_mod_intensity << 5) & 0xE0)
		bytes[16] = ((self.input_trim_mod_intensity >> 3) & 0x1F) | ((self.spread << 5) & 0xE0)
		bytes[17] = (self.spread >> 3) & 0x7
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX068 Left, Center, Right Delay object'''
		json = []
		json.append(('name','FX068'))
		json.append(('description','Left, Center, Right Delay'))
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['l_delay_level',self.l_delay_level])
		json.append(['c_delay_time',self.c_delay_time])
		json.append(['c_delay_level',self.c_delay_level])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['r_delay_level',self.r_delay_level])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 2730:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 2730' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_level(self):
		'''L Delay Level'''
		return self._l_delay_level

	@l_delay_level.setter
	def l_delay_level(self,value):
		if 0 <= value <= 50:
			self._l_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def c_delay_time(self):
		'''C Delay Time'''
		return self._c_delay_time

	@c_delay_time.setter
	def c_delay_time(self,value):
		if 0 <= value <= 2730:
			self._c_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 2730' % value)

#-------------------------------------------------------------------------------
	@property
	def c_delay_level(self):
		'''C Delay Level'''
		return self._c_delay_level

	@c_delay_level.setter
	def c_delay_level(self,value):
		if 0 <= value <= 50:
			self._c_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 2730:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 2730' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_level(self):
		'''R Delay Level'''
		return self._r_delay_level

	@r_delay_level.setter
	def r_delay_level(self,value):
		if 0 <= value <= 50:
			self._r_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if 0 <= value <= 50:
			self._spread = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

