#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx070.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX070(object):
	category = '''Delay'''
	description = '''Stereo Multitap Delay'''
	short_description = '''St. Multitap Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX070 Stereo Multitap Delay object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode = (bytes[3] >> 5) & 0x3
		self.tap1_time = ((bytes[3] & 0x80) >> 7) | (bytes[4] << 1) | ((bytes[5] & 0x1F) << 9)
		self.tap2_time = ((bytes[5] & 0xE0) >> 5) | (bytes[6] << 3) | ((bytes[7] & 0x7) << 11)
		self.tap1_level = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		data = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.feedback_mod_source = (bytes[9] >> 2) & 0x3F
		data = bytes[10] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data
		self.high_damp = bytes[11] & 0x7F
		self.low_damp = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x3F) << 1)
		self.input_trim_mod_source = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0xF) << 2)
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		data = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.spread = data
		self.spread_mod_source = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x3) << 4)
		data = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.spread_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX070 Stereo Multitap Delay object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode & 0x3) << 5) | ((self.tap1_time << 7) & 0x80)
		bytes[4] = (self.tap1_time >> 1) & 0xFF
		bytes[5] = ((self.tap1_time >> 31) & 0x9) | ((self.tap2_time << 5) & 0xE0)
		bytes[6] = (self.tap2_time >> 3) & 0xFF
		bytes[7] = ((self.tap2_time >> 7) & 0xB) | ((self.tap1_level << 3) & 0xF8)
		bytes[8] = ((self.tap1_level >> 5) & 0x3) | ((self.feedback << 2) & 0xFC)
		bytes[9] = ((self.feedback >> 6) & 0x3) | ((self.feedback_mod_source & 0x3F) << 2)
		bytes[10] = self.feedback_mod_intensity & 0xFF
		bytes[11] = (self.high_damp & 0x7F) | ((self.low_damp << 7) & 0x80)
		bytes[12] = ((self.low_damp >> 1) & 0x3F) | ((self.input_trim_mod_source << 6) & 0xC0)
		bytes[13] = ((self.input_trim_mod_source >> 2) & 0xF) | ((self.input_trim_mod_intensity << 4) & 0xF0)
		bytes[14] = ((self.input_trim_mod_intensity >> 4) & 0xF) | ((self.spread << 4) & 0xF0)
		bytes[15] = ((self.spread >> 4) & 0xF) | ((self.spread_mod_source << 4) & 0xF0)
		bytes[16] = ((self.spread_mod_source >> 4) & 0x3) | ((self.spread_mod_intensity << 2) & 0xFC)
		bytes[17] = (self.spread_mod_intensity >> 6) & 0x3
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX070 Stereo Multitap Delay object'''
		json = []
		json.append(('name','FX070'))
		json.append(('description','Stereo Multitap Delay'))
		json.append(['tap1_time',self.tap1_time])
		json.append(['tap2_time',self.tap2_time])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['spread_mod_source',self.spread_mod_source])
		json.append(['spread_mod_intensity',self.spread_mod_intensity])
		json.append(['mode',self.mode])
		json.append(['tap1_level',self.tap1_level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 3:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_time(self):
		'''Tap1 Time'''
		return self._tap1_time

	@tap1_time.setter
	def tap1_time(self,value):
		if 0 <= value <= 13600:
			self._tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_time(self):
		'''Tap2 Time'''
		return self._tap2_time

	@tap2_time.setter
	def tap2_time(self,value):
		if 0 <= value <= 13600:
			self._tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_level(self):
		'''Tap1 Level'''
		return self._tap1_level

	@tap1_level.setter
	def tap1_level(self,value):
		if 0 <= value <= 100:
			self._tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -100 <= value <= 100:
			self._spread = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread_mod_source(self):
		'''Spread Mod.Source'''
		return self._spread_mod_source

	@spread_mod_source.setter
	def spread_mod_source(self,value):
		if 0 <= value <= 37:
			self._spread_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def spread_mod_intensity(self):
		'''Spread Mod.Intensity'''
		return self._spread_mod_intensity

	@spread_mod_intensity.setter
	def spread_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._spread_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

