#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx073.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX073(object):
	category = '''Delay'''
	description = '''Stereo Auto Panning Delay'''
	short_description = '''St. AutoPanningDly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX073 Stereo Auto Panning Delay object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.l_delay_time = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0x7) << 11)
		self.r_delay_time = ((bytes[5] & 0xF8) >> 3) | (bytes[6] << 5) | ((bytes[7] & 0x1) << 13)
		data = ((bytes[7] & 0xFE) >> 1) | ((bytes[8] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.l_feedback = data
		data = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.r_feedback = data
		self.high_damp = (bytes[9] >> 1) & 0x7F
		self.low_damp = bytes[10] & 0x7F
		self.lfo_waveform = (bytes[10] >> 7) & 0x1
		data = bytes[11] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.lfo_shape = data
		data = bytes[12] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.lfo_phase = data
		self.lfo_type = (bytes[12] >> 6) & 0x3
		data = bytes[13] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x3F) << 2)
		self.lfo_bpm_midi_sync = (bytes[14] >> 6) & 0x1
		self.lfo_bpm = ((bytes[14] & 0x80) >> 7) | (bytes[15] << 1) | ((bytes[16] & 0x3F) << 9)
		self.lfo_bpm_base_note = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x3) << 2)
		self.lfo_bpm_times = (bytes[17] >> 2) & 0x3F
		self.depth = bytes[18] & 0x7F
		self.depth_mod_source = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x1F) << 1)
		data = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX073 Stereo Auto Panning Delay object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.l_delay_time << 5) & 0xE0
		bytes[4] = (self.l_delay_time >> 3) & 0xFF
		bytes[5] = ((self.l_delay_time >> 7) & 0xB) | ((self.r_delay_time << 3) & 0xF8)
		bytes[6] = (self.r_delay_time >> 5) & 0xFF
		bytes[7] = ((self.r_delay_time >> 1) & 0xD) | ((self.l_feedback << 1) & 0xFE)
		bytes[8] = ((self.l_feedback >> 7) & 0x1) | ((self.r_feedback << 1) & 0xFE)
		bytes[9] = ((self.r_feedback >> 7) & 0x1) | ((self.high_damp & 0x7F) << 1)
		bytes[10] = (self.low_damp & 0x7F) | ((self.lfo_waveform & 0x1) << 7)
		bytes[11] = self.lfo_shape & 0xFF
		bytes[12] = (self.lfo_phase & 0x3F) | ((self.lfo_type & 0x3) << 6)
		bytes[13] = (self.common_lfo_phase_offset & 0x3F) | ((self.lfo_frequency << 6) & 0xC0)
		bytes[14] = ((self.lfo_frequency >> 2) & 0x3F) | ((self.lfo_bpm_midi_sync & 0x1) << 6) | ((self.lfo_bpm << 7) & 0x80)
		bytes[15] = (self.lfo_bpm >> 1) & 0xFF
		bytes[16] = ((self.lfo_bpm >> 63) & 0x9) | ((self.lfo_bpm_base_note << 6) & 0xC0)
		bytes[17] = ((self.lfo_bpm_base_note >> 2) & 0x3) | ((self.lfo_bpm_times & 0x3F) << 2)
		bytes[18] = (self.depth & 0x7F) | ((self.depth_mod_source << 7) & 0x80)
		bytes[19] = ((self.depth_mod_source >> 1) & 0x1F) | ((self.depth_mod_intensity << 5) & 0xE0)
		bytes[20] = (self.depth_mod_intensity >> 3) & 0x1F
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX073 Stereo Auto Panning Delay object'''
		json = []
		json.append(('name','FX073'))
		json.append(('description','Stereo Auto Panning Delay'))
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['r_delay_time_display',self.r_delay_time_display])
		json.append(['l_feedback',self.l_feedback])
		json.append(['r_feedback',self.r_feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_shape',self.lfo_shape])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 13600:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 13600:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

	@property
	def r_delay_time_display(self):
		'''R Delay Time display value'''
		dv = self._r_delay_time * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def l_feedback(self):
		'''L Feedback'''
		return self._l_feedback

	@l_feedback.setter
	def l_feedback(self,value):
		if -100 <= value <= 100:
			self._l_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_feedback(self):
		'''R Feedback'''
		return self._r_feedback

	@r_feedback.setter
	def r_feedback(self,value):
		if -100 <= value <= 100:
			self._r_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_shape(self):
		'''LFO Shape'''
		return self._lfo_shape

	@lfo_shape.setter
	def lfo_shape(self,value):
		if -100 <= value <= 100:
			self._lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

