#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx074.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX074(object):
	category = '''Delay'''
	description = '''Echo Plus'''
	short_description = '''Echo Plus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX074 Echo Plus object'''
		bytes = struct.unpack('<14B',raw[:14])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.time = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0x1) << 11)
		self.feedback = (bytes[5] >> 1) & 0x7F
		self.tone = bytes[6] & 0x7F
		self.low_damp = ((bytes[6] & 0x80) >> 7) | ((bytes[7] & 0x3F) << 1)
		self.input_trim_mod_source = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0xF) << 2)
		data = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		self.time_mod_source = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x3) << 4)
		self.time_mod_intensity = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x7F) << 6)
		self.feedback_mod_source = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x1F) << 1)
		data = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX074 Echo Plus object into raw data'''
		bytes = [0] * 14
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.time << 5) & 0xE0
		bytes[4] = (self.time >> 3) & 0xFF
		bytes[5] = ((self.time >> 1) & 0xB) | ((self.feedback & 0x7F) << 1)
		bytes[6] = (self.tone & 0x7F) | ((self.low_damp << 7) & 0x80)
		bytes[7] = ((self.low_damp >> 1) & 0x3F) | ((self.input_trim_mod_source << 6) & 0xC0)
		bytes[8] = ((self.input_trim_mod_source >> 2) & 0xF) | ((self.input_trim_mod_intensity << 4) & 0xF0)
		bytes[9] = ((self.input_trim_mod_intensity >> 4) & 0xF) | ((self.time_mod_source << 4) & 0xF0)
		bytes[10] = ((self.time_mod_source >> 4) & 0x3) | ((self.time_mod_intensity << 2) & 0xFC)
		bytes[11] = ((self.time_mod_intensity >> 6) & 0x7F) | ((self.feedback_mod_source << 7) & 0x80)
		bytes[12] = ((self.feedback_mod_source >> 1) & 0x1F) | ((self.feedback_mod_intensity << 5) & 0xE0)
		bytes[13] = (self.feedback_mod_intensity >> 3) & 0x1F
		return struct.pack('<14B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX074 Echo Plus object'''
		json = []
		json.append(('name','FX074'))
		json.append(('description','Echo Plus'))
		json.append(['time',self.time])
		json.append(['time_mod_source',self.time_mod_source])
		json.append(['time_mod_intensity',self.time_mod_intensity])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['tone',self.tone])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def time(self):
		'''Time'''
		return self._time

	@time.setter
	def time(self,value):
		if 0 <= value <= 523:
			self._time = value
		else:
			raise ValueError('%s not 0 <= value <= 523' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if 0 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tone(self):
		'''Tone'''
		return self._tone

	@tone.setter
	def tone(self,value):
		if 0 <= value <= 99:
			self._tone = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def time_mod_source(self):
		'''Time Mod.Source'''
		return self._time_mod_source

	@time_mod_source.setter
	def time_mod_source(self,value):
		if 0 <= value <= 37:
			self._time_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def time_mod_intensity(self):
		'''Time Mod.Intensity'''
		return self._time_mod_intensity

	@time_mod_intensity.setter
	def time_mod_intensity(self,value):
		if 62580 <= value <= 2700:
			self._time_mod_intensity = value
		else:
			raise ValueError('%s not 62580 <= value <= 2700' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

