#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx076.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX076(object):
	category = '''Delay'''
	description = '''Auto Reverse'''
	short_description = '''Auto Reverse'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX076 Auto Reverse object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.rec_mode = (bytes[3] >> 5) & 0x1
		self.reverse_time = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x7F) << 2)
		self.envelope_select = (bytes[4] >> 7) & 0x1
		self.envelope_mod_source = bytes[5] & 0x3F
		self.threshold = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x1F) << 2)
		self.response = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0xF) << 3)
		self.direct_mix = (bytes[7] >> 4) & 0x3

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX076 Auto Reverse object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.rec_mode & 0x1) << 5) | ((self.reverse_time << 6) & 0xC0)
		bytes[4] = ((self.reverse_time >> 2) & 0x7F) | ((self.envelope_select & 0x1) << 7)
		bytes[5] = (self.envelope_mod_source & 0x3F) | ((self.threshold << 6) & 0xC0)
		bytes[6] = ((self.threshold >> 2) & 0x1F) | ((self.response << 5) & 0xE0)
		bytes[7] = ((self.response >> 3) & 0xF) | ((self.direct_mix & 0x3) << 4)
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX076 Auto Reverse object'''
		json = []
		json.append(('name','FX076'))
		json.append(('description','Auto Reverse'))
		json.append(['rec_mode',self.rec_mode])
		json.append(['envelope_select',self.envelope_select])
		json.append(['envelope_mod_source',self.envelope_mod_source])
		json.append(['threshold',self.threshold])
		json.append(['direct_mix',self.direct_mix])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['reverse_time',self.reverse_time])
		json.append(['response',self.response])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def rec_mode(self):
		'''Rec Mode'''
		return self._rec_mode

	@rec_mode.setter
	def rec_mode(self,value):
		if 0 <= value <= 1:
			self._rec_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def reverse_time(self):
		'''Reverse Time'''
		return self._reverse_time

	@reverse_time.setter
	def reverse_time(self,value):
		if 20 <= value <= 139:
			self._reverse_time = value
		else:
			raise ValueError('%s not 20 <= value <= 139' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select(self):
		'''Envelope Select'''
		return self._envelope_select

	@envelope_select.setter
	def envelope_select(self,value):
		if 0 <= value <= 1:
			self._envelope_select = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_mod_source(self):
		'''Envelope Mod.Source'''
		return self._envelope_mod_source

	@envelope_mod_source.setter
	def envelope_mod_source(self,value):
		if 0 <= value <= 37:
			self._envelope_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if 0 <= value <= 100:
			self._threshold = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def response(self):
		'''Response'''
		return self._response

	@response.setter
	def response(self,value):
		if 0 <= value <= 100:
			self._response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def direct_mix(self):
		'''Direct Mix'''
		return self._direct_mix

	@direct_mix.setter
	def direct_mix(self,value):
		if 0 <= value <= 2:
			self._direct_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

