#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx080.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX080(object):
	category = '''Delay'''
	description = '''Stereo Beats per Minute Multitap Delay'''
	short_description = '''St. BPM Mtap Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX080 Stereo Beats per Minute Multitap Delay object'''
		bytes = struct.unpack('<19B',raw[:19])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode = (bytes[3] >> 5) & 0x3
		self.bpm = ((bytes[3] & 0x80) >> 7) | (bytes[4] << 1) | ((bytes[5] & 0x3F) << 9)
		self.tap1_base_note = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x3) << 2)
		self.tap1_times = (bytes[6] >> 2) & 0x3F
		self.tap2_base_note = bytes[7] & 0xF
		self.tap2_times = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x3) << 4)
		self.tap1_level = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		data = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.feedback_mod_source = (bytes[10] >> 1) & 0x3F
		data = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data
		self.high_damp = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x3F) << 1)
		self.low_damp = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		self.input_trim_mod_source = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x7) << 3)
		data = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		data = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.spread = data
		self.spread_mod_source = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x1) << 5)
		data = ((bytes[17] & 0xFE) >> 1) | ((bytes[18] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.spread_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX080 Stereo Beats per Minute Multitap Delay object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode & 0x3) << 5) | ((self.bpm << 7) & 0x80)
		bytes[4] = (self.bpm >> 1) & 0xFF
		bytes[5] = ((self.bpm >> 63) & 0x9) | ((self.tap1_base_note << 6) & 0xC0)
		bytes[6] = ((self.tap1_base_note >> 2) & 0x3) | ((self.tap1_times & 0x3F) << 2)
		bytes[7] = (self.tap2_base_note & 0xF) | ((self.tap2_times << 4) & 0xF0)
		bytes[8] = ((self.tap2_times >> 4) & 0x3) | ((self.tap1_level << 2) & 0xFC)
		bytes[9] = ((self.tap1_level >> 6) & 0x1) | ((self.feedback << 1) & 0xFE)
		bytes[10] = ((self.feedback >> 7) & 0x1) | ((self.feedback_mod_source & 0x3F) << 1) | ((self.feedback_mod_intensity << 7) & 0x80)
		bytes[11] = ((self.feedback_mod_intensity >> 1) & 0x7F) | ((self.high_damp << 7) & 0x80)
		bytes[12] = ((self.high_damp >> 1) & 0x3F) | ((self.low_damp << 6) & 0xC0)
		bytes[13] = ((self.low_damp >> 2) & 0x1F) | ((self.input_trim_mod_source << 5) & 0xE0)
		bytes[14] = ((self.input_trim_mod_source >> 3) & 0x7) | ((self.input_trim_mod_intensity << 3) & 0xF8)
		bytes[15] = ((self.input_trim_mod_intensity >> 5) & 0x7) | ((self.spread << 3) & 0xF8)
		bytes[16] = ((self.spread >> 5) & 0x7) | ((self.spread_mod_source << 3) & 0xF8)
		bytes[17] = ((self.spread_mod_source >> 5) & 0x1) | ((self.spread_mod_intensity << 1) & 0xFE)
		bytes[18] = (self.spread_mod_intensity >> 7) & 0x1
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX080 Stereo Beats per Minute Multitap Delay object'''
		json = []
		json.append(('name','FX080'))
		json.append(('description','Stereo Beats per Minute Multitap Delay'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['spread_mod_source',self.spread_mod_source])
		json.append(['spread_mod_intensity',self.spread_mod_intensity])
		json.append(['mode',self.mode])
		json.append(['tap1_level',self.tap1_level])
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['tap1_base_note',self.tap1_base_note])
		json.append(['tap1_base_note_display',self.tap1_base_note_display])
		json.append(['tap2_base_note',self.tap2_base_note])
		json.append(['tap2_base_note_display',self.tap2_base_note_display])
		json.append(['tap1_times',self.tap1_times])
		json.append(['tap2_times',self.tap2_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 3:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def tap1_base_note(self):
		'''Tap1 Base Note'''
		return self._tap1_base_note

	@tap1_base_note.setter
	def tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def tap1_base_note_display(self):
		'''Tap1 Base Note display value'''
		return base_note_dict[self._tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def tap1_times(self):
		'''Tap1 Times'''
		return self._tap1_times

	@tap1_times.setter
	def tap1_times(self,value):
		if 1 <= value <= 8:
			self._tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 8' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_base_note(self):
		'''Tap2 Base Note'''
		return self._tap2_base_note

	@tap2_base_note.setter
	def tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def tap2_base_note_display(self):
		'''Tap2 Base Note display value'''
		return base_note_dict[self._tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def tap2_times(self):
		'''Tap2 Times'''
		return self._tap2_times

	@tap2_times.setter
	def tap2_times(self,value):
		if 1 <= value <= 8:
			self._tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 8' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_level(self):
		'''Tap1 Level'''
		return self._tap1_level

	@tap1_level.setter
	def tap1_level(self,value):
		if 0 <= value <= 100:
			self._tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -100 <= value <= 100:
			self._spread = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread_mod_source(self):
		'''Spread Mod.Source'''
		return self._spread_mod_source

	@spread_mod_source.setter
	def spread_mod_source(self,value):
		if 0 <= value <= 37:
			self._spread_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def spread_mod_intensity(self):
		'''Spread Mod.Intensity'''
		return self._spread_mod_intensity

	@spread_mod_intensity.setter
	def spread_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._spread_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

