#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx082.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX082(object):
	category = '''Delay'''
	description = '''Stereo Beats per Minute Auto Panning Delay'''
	short_description = '''St. BPMAutoPanDly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX082 Stereo Beats per Minute Auto Panning Delay object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.delay_bpm = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0xF) << 11)
		self.l_delay_base_note = (bytes[5] >> 4) & 0xF
		self.r_delay_base_note = bytes[6] & 0xF
		self.l_delay_times = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0x3) << 4)
		self.r_delay_times = (bytes[7] >> 2) & 0x3F
		data = bytes[8] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.l_feedback = data
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.r_feedback = data
		self.high_damp = bytes[10] & 0x7F
		self.low_damp = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x3F) << 1)
		self.lfo_waveform = (bytes[11] >> 6) & 0x1
		data = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.lfo_shape = data
		data = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.lfo_phase = data
		self.lfo_type = (bytes[13] >> 5) & 0x3
		data = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x1F) << 3)
		self.lfo_bpm_midi_sync = (bytes[15] >> 5) & 0x1
		self.lfo_bpm = ((bytes[15] & 0xC0) >> 6) | (bytes[16] << 2) | ((bytes[17] & 0x1F) << 10)
		self.lfo_bpm_base_note = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x1) << 3)
		self.lfo_bpm_times = (bytes[18] >> 1) & 0x3F
		self.depth = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		self.depth_mod_source = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0xF) << 2)
		data = ((bytes[20] & 0xF0) >> 4) | ((bytes[21] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX082 Stereo Beats per Minute Auto Panning Delay object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.delay_bpm << 5) & 0xE0
		bytes[4] = (self.delay_bpm >> 3) & 0xFF
		bytes[5] = ((self.delay_bpm >> 15) & 0xB) | ((self.l_delay_base_note & 0xF) << 4)
		bytes[6] = (self.r_delay_base_note & 0xF) | ((self.l_delay_times << 4) & 0xF0)
		bytes[7] = ((self.l_delay_times >> 4) & 0x3) | ((self.r_delay_times & 0x3F) << 2)
		bytes[8] = self.l_feedback & 0xFF
		bytes[9] = self.r_feedback & 0xFF
		bytes[10] = (self.high_damp & 0x7F) | ((self.low_damp << 7) & 0x80)
		bytes[11] = ((self.low_damp >> 1) & 0x3F) | ((self.lfo_waveform & 0x1) << 6) | ((self.lfo_shape << 7) & 0x80)
		bytes[12] = ((self.lfo_shape >> 1) & 0x7F) | ((self.lfo_phase << 7) & 0x80)
		bytes[13] = ((self.lfo_phase >> 1) & 0x1F) | ((self.lfo_type & 0x3) << 5) | ((self.common_lfo_phase_offset << 7) & 0x80)
		bytes[14] = ((self.common_lfo_phase_offset >> 1) & 0x1F) | ((self.lfo_frequency << 5) & 0xE0)
		bytes[15] = ((self.lfo_frequency >> 3) & 0x1F) | ((self.lfo_bpm_midi_sync & 0x1) << 5) | ((self.lfo_bpm << 6) & 0xC0)
		bytes[16] = (self.lfo_bpm >> 2) & 0xFF
		bytes[17] = ((self.lfo_bpm >> 31) & 0xA) | ((self.lfo_bpm_base_note << 5) & 0xE0)
		bytes[18] = ((self.lfo_bpm_base_note >> 3) & 0x1) | ((self.lfo_bpm_times & 0x3F) << 1) | ((self.depth << 7) & 0x80)
		bytes[19] = ((self.depth >> 1) & 0x3F) | ((self.depth_mod_source << 6) & 0xC0)
		bytes[20] = ((self.depth_mod_source >> 2) & 0xF) | ((self.depth_mod_intensity << 4) & 0xF0)
		bytes[21] = (self.depth_mod_intensity >> 4) & 0xF
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX082 Stereo Beats per Minute Auto Panning Delay object'''
		json = []
		json.append(('name','FX082'))
		json.append(('description','Stereo Beats per Minute Auto Panning Delay'))
		json.append(['l_feedback',self.l_feedback])
		json.append(['r_feedback',self.r_feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_shape',self.lfo_shape])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['delay_bpm',self.delay_bpm])
		json.append(['delay_bpm_display',self.delay_bpm_display])
		json.append(['l_delay_base_note',self.l_delay_base_note])
		json.append(['l_delay_base_note_display',self.l_delay_base_note_display])
		json.append(['r_delay_base_note',self.r_delay_base_note])
		json.append(['r_delay_base_note_display',self.r_delay_base_note_display])
		json.append(['l_delay_times',self.l_delay_times])
		json.append(['r_delay_times',self.r_delay_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_bpm(self):
		'''Delay BPM'''
		return self._delay_bpm

	@delay_bpm.setter
	def delay_bpm(self,value):
		if 3999 <= value <= 30000:
			self._delay_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def delay_bpm_display(self):
		'''Delay BPM display value'''
		return bpm_decode(self._delay_bpm)

#-------------------------------------------------------------------------------
	@property
	def l_delay_base_note(self):
		'''L Delay Base Note'''
		return self._l_delay_base_note

	@l_delay_base_note.setter
	def l_delay_base_note(self,value):
		if 0 <= value <= 9:
			self._l_delay_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def l_delay_base_note_display(self):
		'''L Delay Base Note display value'''
		return base_note_dict[self._l_delay_base_note]

#-------------------------------------------------------------------------------
	@property
	def r_delay_base_note(self):
		'''R Delay Base Note'''
		return self._r_delay_base_note

	@r_delay_base_note.setter
	def r_delay_base_note(self,value):
		if 0 <= value <= 9:
			self._r_delay_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def r_delay_base_note_display(self):
		'''R Delay Base Note display value'''
		return base_note_dict[self._r_delay_base_note]

#-------------------------------------------------------------------------------
	@property
	def l_delay_times(self):
		'''L Delay Times'''
		return self._l_delay_times

	@l_delay_times.setter
	def l_delay_times(self,value):
		if 1 <= value <= 8:
			self._l_delay_times = value
		else:
			raise ValueError('%s not 1 <= value <= 8' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_times(self):
		'''R Delay Times'''
		return self._r_delay_times

	@r_delay_times.setter
	def r_delay_times(self,value):
		if 1 <= value <= 8:
			self._r_delay_times = value
		else:
			raise ValueError('%s not 1 <= value <= 8' % value)

#-------------------------------------------------------------------------------
	@property
	def l_feedback(self):
		'''L Feedback'''
		return self._l_feedback

	@l_feedback.setter
	def l_feedback(self,value):
		if -100 <= value <= 100:
			self._l_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_feedback(self):
		'''R Feedback'''
		return self._r_feedback

	@r_feedback.setter
	def r_feedback(self,value):
		if -100 <= value <= 100:
			self._r_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_shape(self):
		'''LFO Shape'''
		return self._lfo_shape

	@lfo_shape.setter
	def lfo_shape(self,value):
		if -100 <= value <= 100:
			self._lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

