#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx092.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX092(object):
	category = '''Reverb/ER'''
	description = '''Reverb2 Plate'''
	short_description = '''Reverb2 Plate'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX092 Reverb2 Plate object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.time = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.high_damp = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.low_damp = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.pre_delay = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX092 Reverb2 Plate object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.time << 5) & 0xE0
		bytes[4] = ((self.time >> 3) & 0xF) | ((self.high_damp << 4) & 0xF0)
		bytes[5] = ((self.high_damp >> 4) & 0x7) | ((self.low_damp << 3) & 0xF8)
		bytes[6] = ((self.low_damp >> 5) & 0x3) | ((self.pre_delay << 2) & 0xFC)
		bytes[7] = (self.pre_delay >> 6) & 0x1
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX092 Reverb2 Plate object'''
		json = []
		json.append(('name','FX092'))
		json.append(('description','Reverb2 Plate'))
		json.append(['time',self.time])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['pre_delay',self.pre_delay])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def time(self):
		'''Time'''
		return self._time

	@time.setter
	def time(self,value):
		if 0 <= value <= 99:
			self._time = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_delay(self):
		'''Pre Delay'''
		return self._pre_delay

	@pre_delay.setter
	def pre_delay(self,value):
		if 0 <= value <= 70:
			self._pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

