#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx095.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX095(object):
	category = '''Mono-Mono'''
	description = '''P4EQ - Exciter (Parametric 4-Band EQ - Exciter)'''
	short_description = '''P4EQ - Exciter (Parametric 4-Band EQ - Exciter)'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX095 P4EQ - Exciter (Parametric 4-Band EQ - Exciter) object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.p4eq_trim = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.p4eq_band1_cutoff = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.p4eq_band1_q = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		data = (bytes[6] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = (bytes[7] & 0xFF) | ((bytes[8] & 0x1) << 8)
		self.p4eq_band2_q = (bytes[8] >> 1) & 0x7F
		data = bytes[9] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.p4eq_band3_q = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x3) << 6)
		self.p4eq_band4_q = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		data = (bytes[14] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band4_gain = data
		data = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x3F) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX095 P4EQ - Exciter (Parametric 4-Band EQ - Exciter) object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.p4eq_trim << 5) & 0xE0
		bytes[4] = ((self.p4eq_trim >> 3) & 0xF) | ((self.p4eq_band1_cutoff << 4) & 0xF0)
		bytes[5] = ((self.p4eq_band1_cutoff >> 4) & 0x7) | ((self.p4eq_band1_q << 3) & 0xF8)
		bytes[6] = ((self.p4eq_band1_q >> 5) & 0x3) | ((self.p4eq_band1_gain & 0x3F) << 2)
		bytes[7] = self.p4eq_band2_cutoff & 0xFF
		bytes[8] = ((self.p4eq_band2_cutoff >> 8) & 0x1) | ((self.p4eq_band2_q & 0x7F) << 1)
		bytes[9] = (self.p4eq_band2_gain & 0x3F) | ((self.p4eq_band3_cutoff << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band3_cutoff >> 2) & 0x1F) | ((self.p4eq_band3_q << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band3_q >> 3) & 0xF) | ((self.p4eq_band3_gain << 4) & 0xF0)
		bytes[12] = ((self.p4eq_band3_gain >> 4) & 0x3) | ((self.p4eq_band4_cutoff << 2) & 0xFC)
		bytes[13] = ((self.p4eq_band4_cutoff >> 6) & 0x3) | ((self.p4eq_band4_q << 2) & 0xFC)
		bytes[14] = ((self.p4eq_band4_q >> 6) & 0x1) | ((self.p4eq_band4_gain & 0x3F) << 1) | ((self.exciter_blend << 7) & 0x80)
		bytes[15] = ((self.exciter_blend >> 1) & 0x7F) | ((self.exciter_emphasis_frequency << 7) & 0x80)
		bytes[16] = (self.exciter_emphasis_frequency >> 1) & 0x3F
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX095 P4EQ - Exciter (Parametric 4-Band EQ - Exciter) object'''
		json = []
		json.append(('name','FX095'))
		json.append(('description','P4EQ - Exciter (Parametric 4-Band EQ - Exciter)'))
		json.append(['p4eq_trim',self.p4eq_trim])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_trim(self):
		'''(P4EQ) Trim'''
		return self._p4eq_trim

	@p4eq_trim.setter
	def p4eq_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

