#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx096.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX096(object):
	category = '''Mono-Mono'''
	description = '''Parametric 4-Band EQ - Wah/Auto Wah'''
	short_description = '''P4EQ - Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX096 Parametric 4-Band EQ - Wah/Auto Wah object'''
		bytes = struct.unpack('<23B',raw[:23])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.p4eq_trim = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.p4eq_band1_cutoff = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.p4eq_band1_q = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		data = (bytes[6] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = (bytes[7] & 0xFF) | ((bytes[8] & 0x1) << 8)
		self.p4eq_band2_q = (bytes[8] >> 1) & 0x7F
		data = bytes[9] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.p4eq_band3_q = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x3) << 6)
		self.p4eq_band4_q = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		data = (bytes[14] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band4_gain = data
		self.wah_frequency_bottom = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x3F) << 1)
		self.wah_frequency_top = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0x1F) << 2)
		self.wah_sweep_mode = (bytes[16] >> 5) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[16] & 0x80) >> 7) | ((bytes[17] & 0x1F) << 1)
		self.wah_lfo_frequency = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x1F) << 3)
		self.wah_resonance = ((bytes[18] & 0xE0) >> 5) | ((bytes[19] & 0xF) << 3)
		self.wah_low_pass_filter = (bytes[19] >> 4) & 0x1
		self.wah_wet_dry = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0xF) << 3)
		self.wah_wet_dry_mod_source = ((bytes[20] & 0xF0) >> 4) | ((bytes[21] & 0x3) << 4)
		data = ((bytes[21] & 0xFC) >> 2) | ((bytes[22] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.routing = (bytes[22] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX096 Parametric 4-Band EQ - Wah/Auto Wah object into raw data'''
		bytes = [0] * 23
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.p4eq_trim << 5) & 0xE0
		bytes[4] = ((self.p4eq_trim >> 3) & 0xF) | ((self.p4eq_band1_cutoff << 4) & 0xF0)
		bytes[5] = ((self.p4eq_band1_cutoff >> 4) & 0x7) | ((self.p4eq_band1_q << 3) & 0xF8)
		bytes[6] = ((self.p4eq_band1_q >> 5) & 0x3) | ((self.p4eq_band1_gain & 0x3F) << 2)
		bytes[7] = self.p4eq_band2_cutoff & 0xFF
		bytes[8] = ((self.p4eq_band2_cutoff >> 8) & 0x1) | ((self.p4eq_band2_q & 0x7F) << 1)
		bytes[9] = (self.p4eq_band2_gain & 0x3F) | ((self.p4eq_band3_cutoff << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band3_cutoff >> 2) & 0x1F) | ((self.p4eq_band3_q << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band3_q >> 3) & 0xF) | ((self.p4eq_band3_gain << 4) & 0xF0)
		bytes[12] = ((self.p4eq_band3_gain >> 4) & 0x3) | ((self.p4eq_band4_cutoff << 2) & 0xFC)
		bytes[13] = ((self.p4eq_band4_cutoff >> 6) & 0x3) | ((self.p4eq_band4_q << 2) & 0xFC)
		bytes[14] = ((self.p4eq_band4_q >> 6) & 0x1) | ((self.p4eq_band4_gain & 0x3F) << 1) | ((self.wah_frequency_bottom << 7) & 0x80)
		bytes[15] = ((self.wah_frequency_bottom >> 1) & 0x3F) | ((self.wah_frequency_top << 6) & 0xC0)
		bytes[16] = ((self.wah_frequency_top >> 2) & 0x1F) | ((self.wah_sweep_mode & 0x3) << 5) | ((self.wah_sweep_mode_mod_source << 7) & 0x80)
		bytes[17] = ((self.wah_sweep_mode_mod_source >> 1) & 0x1F) | ((self.wah_lfo_frequency << 5) & 0xE0)
		bytes[18] = ((self.wah_lfo_frequency >> 3) & 0x1F) | ((self.wah_resonance << 5) & 0xE0)
		bytes[19] = ((self.wah_resonance >> 3) & 0xF) | ((self.wah_low_pass_filter & 0x1) << 4) | ((self.wah_wet_dry << 5) & 0xE0)
		bytes[20] = ((self.wah_wet_dry >> 3) & 0xF) | ((self.wah_wet_dry_mod_source << 4) & 0xF0)
		bytes[21] = ((self.wah_wet_dry_mod_source >> 4) & 0x3) | ((self.wah_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[22] = ((self.wah_wet_dry_mod_intensity >> 6) & 0x3) | ((self.routing & 0x1) << 2)
		return struct.pack('<23B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX096 Parametric 4-Band EQ - Wah/Auto Wah object'''
		json = []
		json.append(('name','FX096'))
		json.append(('description','Parametric 4-Band EQ - Wah/Auto Wah'))
		json.append(['p4eq_trim',self.p4eq_trim])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_trim(self):
		'''(P4EQ) Trim'''
		return self._p4eq_trim

	@p4eq_trim.setter
	def p4eq_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

