#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx097.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX097(object):
	category = '''Mono-Mono'''
	description = '''Parametric 4-Band EQ - Chorus/Flanger'''
	short_description = '''P4EQ - Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX097 Parametric 4-Band EQ - Chorus/Flanger object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_band1_cutoff = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.p4eq_band1_q = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x3) << 7)
		self.p4eq_band2_q = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		data = (bytes[8] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.p4eq_band3_q = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x7) << 5)
		self.p4eq_band4_q = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x3) << 5)
		data = (bytes[13] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band4_gain = data
		self.chorus_flanger_pre_delay = (bytes[14] & 0xFF) | ((bytes[15] & 0x7) << 8)
		self.chorus_flanger_lfo_waveform = (bytes[15] >> 3) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0xF) << 4)
		self.chorus_flanger_depth = ((bytes[16] & 0xF0) >> 4) | ((bytes[17] & 0x7) << 4)
		data = ((bytes[17] & 0xF8) >> 3) | ((bytes[18] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = ((bytes[18] & 0xF8) >> 3) | ((bytes[19] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[19] & 0xF8) >> 3) | ((bytes[20] & 0x1) << 5)
		data = ((bytes[20] & 0xFE) >> 1) | ((bytes[21] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_wet_invert = (bytes[21] >> 1) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX097 Parametric 4-Band EQ - Chorus/Flanger object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.p4eq_band1_cutoff << 5) & 0xE0)
		bytes[4] = ((self.p4eq_band1_cutoff >> 3) & 0xF) | ((self.p4eq_band1_q << 4) & 0xF0)
		bytes[5] = ((self.p4eq_band1_q >> 4) & 0x7) | ((self.p4eq_band1_gain << 3) & 0xF8)
		bytes[6] = ((self.p4eq_band1_gain >> 5) & 0x1) | ((self.p4eq_band2_cutoff << 1) & 0xFE)
		bytes[7] = ((self.p4eq_band2_cutoff >> 7) & 0x3) | ((self.p4eq_band2_q << 2) & 0xFC)
		bytes[8] = ((self.p4eq_band2_q >> 6) & 0x1) | ((self.p4eq_band2_gain & 0x3F) << 1) | ((self.p4eq_band3_cutoff << 7) & 0x80)
		bytes[9] = ((self.p4eq_band3_cutoff >> 1) & 0x3F) | ((self.p4eq_band3_q << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band3_q >> 2) & 0x1F) | ((self.p4eq_band3_gain << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band3_gain >> 3) & 0x7) | ((self.p4eq_band4_cutoff << 3) & 0xF8)
		bytes[12] = ((self.p4eq_band4_cutoff >> 5) & 0x7) | ((self.p4eq_band4_q << 3) & 0xF8)
		bytes[13] = ((self.p4eq_band4_q >> 5) & 0x3) | ((self.p4eq_band4_gain & 0x3F) << 2)
		bytes[14] = self.chorus_flanger_pre_delay & 0xFF
		bytes[15] = ((self.chorus_flanger_pre_delay >> 8) & 0x7) | ((self.chorus_flanger_lfo_waveform & 0x1) << 3) | ((self.chorus_flanger_lfo_frequency << 4) & 0xF0)
		bytes[16] = ((self.chorus_flanger_lfo_frequency >> 4) & 0xF) | ((self.chorus_flanger_depth << 4) & 0xF0)
		bytes[17] = ((self.chorus_flanger_depth >> 4) & 0x7) | ((self.chorus_flanger_feedback << 3) & 0xF8)
		bytes[18] = ((self.chorus_flanger_feedback >> 5) & 0x7) | ((self.chorus_flanger_wet_dry << 3) & 0xF8)
		bytes[19] = ((self.chorus_flanger_wet_dry >> 5) & 0x7) | ((self.chorus_flanger_wet_dry_mod_source << 3) & 0xF8)
		bytes[20] = ((self.chorus_flanger_wet_dry_mod_source >> 5) & 0x1) | ((self.chorus_flanger_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[21] = ((self.chorus_flanger_wet_dry_mod_intensity >> 7) & 0x1) | ((self.chorus_flanger_wet_invert & 0x1) << 1)
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX097 Parametric 4-Band EQ - Chorus/Flanger object'''
		json = []
		json.append(('name','FX097'))
		json.append(('description','Parametric 4-Band EQ - Chorus/Flanger'))
		json.append(['input_trim',self.input_trim])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_invert',self.chorus_flanger_wet_invert])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 175:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 175' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_invert(self):
		'''(Chorus/Flanger) Wet Invert'''
		return self._chorus_flanger_wet_invert

	@chorus_flanger_wet_invert.setter
	def chorus_flanger_wet_invert(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

