#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx099.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX099(object):
	category = '''Mono-Mono'''
	description = '''Parametric 4-Band EQ - Multitap Delay'''
	short_description = '''P4EQ - Mt. Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX099 Parametric 4-Band EQ - Multitap Delay object'''
		bytes = struct.unpack('<23B',raw[:23])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_band1_cutoff = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.p4eq_band1_q = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x3) << 7)
		self.p4eq_band2_q = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		data = (bytes[8] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.p4eq_band3_q = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x7) << 5)
		self.p4eq_band4_q = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x3) << 5)
		data = (bytes[13] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band4_gain = data
		self.multitap_delay_tap1_time = (bytes[14] & 0xFF) | ((bytes[15] & 0x3F) << 8)
		self.multitap_delay_tap2_time = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0xFF) << 2)
		self.multitap_delay_tap1_level = ((bytes[17] & 0xF0) >> 4) | ((bytes[18] & 0x7) << 4)
		data = ((bytes[18] & 0xF8) >> 3) | ((bytes[19] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_feedback = data
		self.multitap_delay_high_damp = ((bytes[19] & 0xF8) >> 3) | ((bytes[20] & 0x3) << 5)
		self.multitap_delay_wet_dry = ((bytes[20] & 0xFC) >> 2) | ((bytes[21] & 0x1) << 6)
		self.multitap_delay_wet_dry_mod_source = (bytes[21] >> 1) & 0x3F
		data = ((bytes[21] & 0x80) >> 7) | ((bytes[22] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_wet_dry_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX099 Parametric 4-Band EQ - Multitap Delay object into raw data'''
		bytes = [0] * 23
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.p4eq_band1_cutoff << 5) & 0xE0)
		bytes[4] = ((self.p4eq_band1_cutoff >> 3) & 0xF) | ((self.p4eq_band1_q << 4) & 0xF0)
		bytes[5] = ((self.p4eq_band1_q >> 4) & 0x7) | ((self.p4eq_band1_gain << 3) & 0xF8)
		bytes[6] = ((self.p4eq_band1_gain >> 5) & 0x1) | ((self.p4eq_band2_cutoff << 1) & 0xFE)
		bytes[7] = ((self.p4eq_band2_cutoff >> 7) & 0x3) | ((self.p4eq_band2_q << 2) & 0xFC)
		bytes[8] = ((self.p4eq_band2_q >> 6) & 0x1) | ((self.p4eq_band2_gain & 0x3F) << 1) | ((self.p4eq_band3_cutoff << 7) & 0x80)
		bytes[9] = ((self.p4eq_band3_cutoff >> 1) & 0x3F) | ((self.p4eq_band3_q << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band3_q >> 2) & 0x1F) | ((self.p4eq_band3_gain << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band3_gain >> 3) & 0x7) | ((self.p4eq_band4_cutoff << 3) & 0xF8)
		bytes[12] = ((self.p4eq_band4_cutoff >> 5) & 0x7) | ((self.p4eq_band4_q << 3) & 0xF8)
		bytes[13] = ((self.p4eq_band4_q >> 5) & 0x3) | ((self.p4eq_band4_gain & 0x3F) << 2)
		bytes[14] = self.multitap_delay_tap1_time & 0xFF
		bytes[15] = ((self.multitap_delay_tap1_time >> 8) & 0x3F) | ((self.multitap_delay_tap2_time << 6) & 0xC0)
		bytes[16] = (self.multitap_delay_tap2_time >> 2) & 0xFF
		bytes[17] = (self.multitap_delay_tap1_level << 4) & 0xF0
		bytes[18] = ((self.multitap_delay_tap1_level >> 4) & 0x7) | ((self.multitap_delay_feedback << 3) & 0xF8)
		bytes[19] = ((self.multitap_delay_feedback >> 5) & 0x7) | ((self.multitap_delay_high_damp << 3) & 0xF8)
		bytes[20] = ((self.multitap_delay_high_damp >> 5) & 0x3) | ((self.multitap_delay_wet_dry << 2) & 0xFC)
		bytes[21] = ((self.multitap_delay_wet_dry >> 6) & 0x1) | ((self.multitap_delay_wet_dry_mod_source & 0x3F) << 1) | ((self.multitap_delay_wet_dry_mod_intensity << 7) & 0x80)
		bytes[22] = (self.multitap_delay_wet_dry_mod_intensity >> 1) & 0x7F
		return struct.pack('<23B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX099 Parametric 4-Band EQ - Multitap Delay object'''
		json = []
		json.append(('name','FX099'))
		json.append(('description','Parametric 4-Band EQ - Multitap Delay'))
		json.append(['input_trim',self.input_trim])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['multitap_delay_tap1_time',self.multitap_delay_tap1_time])
		json.append(['multitap_delay_tap2_time',self.multitap_delay_tap2_time])
		json.append(['multitap_delay_tap1_level',self.multitap_delay_tap1_level])
		json.append(['multitap_delay_feedback',self.multitap_delay_feedback])
		json.append(['multitap_delay_high_damp',self.multitap_delay_high_damp])
		json.append(['multitap_delay_wet_dry',self.multitap_delay_wet_dry])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['multitap_delay_wet_dry_mod_source',self.multitap_delay_wet_dry_mod_source])
		json.append(['multitap_delay_wet_dry_mod_intensity',self.multitap_delay_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_time(self):
		'''(Multitap Delay) Tap1 Time'''
		return self._multitap_delay_tap1_time

	@multitap_delay_tap1_time.setter
	def multitap_delay_tap1_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap2_time(self):
		'''(Multitap Delay) Tap2 Time'''
		return self._multitap_delay_tap2_time

	@multitap_delay_tap2_time.setter
	def multitap_delay_tap2_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_level(self):
		'''(Multitap Delay) Tap1 Level'''
		return self._multitap_delay_tap1_level

	@multitap_delay_tap1_level.setter
	def multitap_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_feedback(self):
		'''(Multitap Delay) Feedback'''
		return self._multitap_delay_feedback

	@multitap_delay_feedback.setter
	def multitap_delay_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_high_damp(self):
		'''(Multitap Delay) High Damp'''
		return self._multitap_delay_high_damp

	@multitap_delay_high_damp.setter
	def multitap_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry(self):
		'''(Multitap Delay) Wet/Dry'''
		return self._multitap_delay_wet_dry

	@multitap_delay_wet_dry.setter
	def multitap_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_source(self):
		'''(Multitap Delay) Wet/Dry Mod.Source'''
		return self._multitap_delay_wet_dry_mod_source

	@multitap_delay_wet_dry_mod_source.setter
	def multitap_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_intensity(self):
		'''(Multitap Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_delay_wet_dry_mod_intensity

	@multitap_delay_wet_dry_mod_intensity.setter
	def multitap_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

