#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx100.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX100(object):
	category = '''Mono-Mono'''
	description = '''Compressor - Wah/Auto Wah'''
	short_description = '''Comp - Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX100 Compressor - Wah/Auto Wah object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.compressor_sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.compressor_attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.compressor_output_level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.compressor_eq_trim = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		data = (bytes[7] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		self.wah_frequency_bottom = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x3) << 5)
		self.wah_frequency_top = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0x1) << 6)
		self.wah_sweep_mode = (bytes[10] >> 1) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x1) << 5)
		self.wah_lfo_frequency = ((bytes[11] & 0xFE) >> 1) | ((bytes[12] & 0x1) << 7)
		self.wah_resonance = (bytes[12] >> 1) & 0x7F
		self.wah_low_pass_filter = bytes[13] & 0x1
		self.wah_wet_dry = (bytes[13] >> 1) & 0x7F
		self.wah_wet_dry_mod_source = bytes[14] & 0x3F
		data = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.routing = (bytes[15] >> 6) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX100 Compressor - Wah/Auto Wah object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.compressor_sensitivity << 5) & 0xE0
		bytes[4] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[5] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[6] = ((self.compressor_output_level >> 5) & 0x3) | ((self.compressor_eq_trim << 2) & 0xFC)
		bytes[7] = ((self.compressor_eq_trim >> 6) & 0x1) | ((self.compressor_pre_leq_gain & 0x1F) << 1) | ((self.compressor_pre_heq_gain << 6) & 0xC0)
		bytes[8] = ((self.compressor_pre_heq_gain >> 2) & 0x7) | ((self.wah_frequency_bottom << 3) & 0xF8)
		bytes[9] = ((self.wah_frequency_bottom >> 5) & 0x3) | ((self.wah_frequency_top << 2) & 0xFC)
		bytes[10] = ((self.wah_frequency_top >> 6) & 0x1) | ((self.wah_sweep_mode & 0x3) << 1) | ((self.wah_sweep_mode_mod_source << 3) & 0xF8)
		bytes[11] = ((self.wah_sweep_mode_mod_source >> 5) & 0x1) | ((self.wah_lfo_frequency << 1) & 0xFE)
		bytes[12] = ((self.wah_lfo_frequency >> 7) & 0x1) | ((self.wah_resonance & 0x7F) << 1)
		bytes[13] = (self.wah_low_pass_filter & 0x1) | ((self.wah_wet_dry & 0x7F) << 1)
		bytes[14] = (self.wah_wet_dry_mod_source & 0x3F) | ((self.wah_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[15] = ((self.wah_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.routing & 0x1) << 6)
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX100 Compressor - Wah/Auto Wah object'''
		json = []
		json.append(('name','FX100'))
		json.append(('description','Compressor - Wah/Auto Wah'))
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['compressor_eq_trim',self.compressor_eq_trim])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_eq_trim(self):
		'''(Compressor) EQ Trim'''
		return self._compressor_eq_trim

	@compressor_eq_trim.setter
	def compressor_eq_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

