#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx102.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX102(object):
	category = '''Mono-Mono'''
	description = '''Compressor - Overdrive/Hi.Gain'''
	short_description = '''Comp - OD/HiGain'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX102 Compressor - Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.compressor_sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.compressor_attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.compressor_output_level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.od_hi_gain_hi_gain_on = (bytes[6] >> 2) & 0x1
		self.od_hi_gain_drive = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.od_hi_gain_output_level = (bytes[7] >> 2) & 0x3F
		self.od_hi_gain_output_level_mod_source = bytes[8] & 0x3F
		data = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x7) << 3)
		data = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = (bytes[11] >> 1) & 0x3F
		self.od_hi_gain_eq_mid1_q = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7) << 1)
		data = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = (bytes[13] >> 1) & 0x3F
		self.od_hi_gain_eq_mid2_q = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x7) << 1)
		data = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data
		self.od_hi_gain_wet_dry = (bytes[15] >> 1) & 0x7F
		self.od_hi_gain_wet_dry_mod_source = bytes[16] & 0x3F
		data = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.routing = (bytes[17] >> 6) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX102 Compressor - Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.compressor_sensitivity << 5) & 0xE0
		bytes[4] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[5] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[6] = ((self.compressor_output_level >> 5) & 0x3) | ((self.od_hi_gain_hi_gain_on & 0x1) << 2) | ((self.od_hi_gain_drive << 3) & 0xF8)
		bytes[7] = ((self.od_hi_gain_drive >> 5) & 0x3) | ((self.od_hi_gain_output_level & 0x3F) << 2)
		bytes[8] = (self.od_hi_gain_output_level_mod_source & 0x3F) | ((self.od_hi_gain_output_level_mod_intensity << 6) & 0xC0)
		bytes[9] = ((self.od_hi_gain_output_level_mod_intensity >> 2) & 0x1F) | ((self.od_hi_gain_eq_low_cutoff << 5) & 0xE0)
		bytes[10] = ((self.od_hi_gain_eq_low_cutoff >> 3) & 0x7) | ((self.od_hi_gain_eq_low_gain << 3) & 0xF8)
		bytes[11] = ((self.od_hi_gain_eq_low_gain >> 5) & 0x1) | ((self.od_hi_gain_eq_mid1_cutoff & 0x3F) << 1) | ((self.od_hi_gain_eq_mid1_q << 7) & 0x80)
		bytes[12] = ((self.od_hi_gain_eq_mid1_q >> 1) & 0x7) | ((self.od_hi_gain_eq_mid1_gain << 3) & 0xF8)
		bytes[13] = ((self.od_hi_gain_eq_mid1_gain >> 5) & 0x1) | ((self.od_hi_gain_eq_mid2_cutoff & 0x3F) << 1) | ((self.od_hi_gain_eq_mid2_q << 7) & 0x80)
		bytes[14] = ((self.od_hi_gain_eq_mid2_q >> 1) & 0x7) | ((self.od_hi_gain_eq_mid2_gain << 3) & 0xF8)
		bytes[15] = ((self.od_hi_gain_eq_mid2_gain >> 5) & 0x1) | ((self.od_hi_gain_wet_dry & 0x7F) << 1)
		bytes[16] = (self.od_hi_gain_wet_dry_mod_source & 0x3F) | ((self.od_hi_gain_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[17] = ((self.od_hi_gain_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.routing & 0x1) << 6)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX102 Compressor - Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX102'))
		json.append(('description','Compressor - Overdrive/Hi.Gain'))
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

