#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx103.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX103(object):
	category = '''Mono-Mono'''
	description = '''Compressor - Parametric 4-Band EQ'''
	short_description = '''Comp - P4EQ'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX103 Compressor - Parametric 4-Band EQ object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.compressor_sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.compressor_attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.compressor_output_level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.p4eq_trim = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		self.p4eq_band1_cutoff = (bytes[7] >> 1) & 0x7F
		self.p4eq_band1_q = bytes[8] & 0x7F
		data = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x3F) << 3)
		self.p4eq_band2_q = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		data = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x3) << 5)
		self.p4eq_band3_q = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		data = (bytes[14] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x7F) << 1)
		self.p4eq_band4_q = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x3F) << 1)
		data = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.p4eq_band4_gain = data
		self.routing = (bytes[17] >> 4) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX103 Compressor - Parametric 4-Band EQ object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.compressor_sensitivity << 5) & 0xE0
		bytes[4] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[5] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[6] = ((self.compressor_output_level >> 5) & 0x3) | ((self.p4eq_trim << 2) & 0xFC)
		bytes[7] = ((self.p4eq_trim >> 6) & 0x1) | ((self.p4eq_band1_cutoff & 0x7F) << 1)
		bytes[8] = (self.p4eq_band1_q & 0x7F) | ((self.p4eq_band1_gain << 7) & 0x80)
		bytes[9] = ((self.p4eq_band1_gain >> 1) & 0x1F) | ((self.p4eq_band2_cutoff << 5) & 0xE0)
		bytes[10] = ((self.p4eq_band2_cutoff >> 3) & 0x3F) | ((self.p4eq_band2_q << 6) & 0xC0)
		bytes[11] = ((self.p4eq_band2_q >> 2) & 0x1F) | ((self.p4eq_band2_gain << 5) & 0xE0)
		bytes[12] = ((self.p4eq_band2_gain >> 3) & 0x7) | ((self.p4eq_band3_cutoff << 3) & 0xF8)
		bytes[13] = ((self.p4eq_band3_cutoff >> 5) & 0x3) | ((self.p4eq_band3_q << 2) & 0xFC)
		bytes[14] = ((self.p4eq_band3_q >> 6) & 0x1) | ((self.p4eq_band3_gain & 0x3F) << 1) | ((self.p4eq_band4_cutoff << 7) & 0x80)
		bytes[15] = ((self.p4eq_band4_cutoff >> 1) & 0x7F) | ((self.p4eq_band4_q << 7) & 0x80)
		bytes[16] = ((self.p4eq_band4_q >> 1) & 0x3F) | ((self.p4eq_band4_gain << 6) & 0xC0)
		bytes[17] = ((self.p4eq_band4_gain >> 2) & 0xF) | ((self.routing & 0x1) << 4)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX103 Compressor - Parametric 4-Band EQ object'''
		json = []
		json.append(('name','FX103'))
		json.append(('description','Compressor - Parametric 4-Band EQ'))
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['p4eq_trim',self.p4eq_trim])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_trim(self):
		'''(P4EQ) Trim'''
		return self._p4eq_trim

	@p4eq_trim.setter
	def p4eq_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

