#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx104.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX104(object):
	category = '''Mono-Mono'''
	description = '''Compressor - Chorus/Flanger'''
	short_description = '''Comp - Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX104 Compressor - Chorus/Flanger object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.compressor_sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.compressor_attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.compressor_output_level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.compressor_eq_trim = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		data = (bytes[7] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		self.chorus_flanger_pre_delay = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x3F) << 5)
		self.chorus_flanger_lfo_waveform = (bytes[9] >> 6) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x7F) << 1)
		self.chorus_flanger_depth = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x3F) << 1)
		data = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xF) << 2)
		data = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_wet_invert = (bytes[15] >> 4) & 0x1
		self.routing = (bytes[15] >> 5) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX104 Compressor - Chorus/Flanger object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.compressor_sensitivity << 5) & 0xE0
		bytes[4] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[5] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[6] = ((self.compressor_output_level >> 5) & 0x3) | ((self.compressor_eq_trim << 2) & 0xFC)
		bytes[7] = ((self.compressor_eq_trim >> 6) & 0x1) | ((self.compressor_pre_leq_gain & 0x1F) << 1) | ((self.compressor_pre_heq_gain << 6) & 0xC0)
		bytes[8] = ((self.compressor_pre_heq_gain >> 2) & 0x7) | ((self.chorus_flanger_pre_delay << 3) & 0xF8)
		bytes[9] = ((self.chorus_flanger_pre_delay >> 5) & 0x3F) | ((self.chorus_flanger_lfo_waveform & 0x1) << 6) | ((self.chorus_flanger_lfo_frequency << 7) & 0x80)
		bytes[10] = ((self.chorus_flanger_lfo_frequency >> 1) & 0x7F) | ((self.chorus_flanger_depth << 7) & 0x80)
		bytes[11] = ((self.chorus_flanger_depth >> 1) & 0x3F) | ((self.chorus_flanger_feedback << 6) & 0xC0)
		bytes[12] = ((self.chorus_flanger_feedback >> 2) & 0x3F) | ((self.chorus_flanger_wet_dry << 6) & 0xC0)
		bytes[13] = ((self.chorus_flanger_wet_dry >> 2) & 0x3F) | ((self.chorus_flanger_wet_dry_mod_source << 6) & 0xC0)
		bytes[14] = ((self.chorus_flanger_wet_dry_mod_source >> 2) & 0xF) | ((self.chorus_flanger_wet_dry_mod_intensity << 4) & 0xF0)
		bytes[15] = ((self.chorus_flanger_wet_dry_mod_intensity >> 4) & 0xF) | ((self.chorus_flanger_wet_invert & 0x1) << 4) | ((self.routing & 0x1) << 5)
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX104 Compressor - Chorus/Flanger object'''
		json = []
		json.append(('name','FX104'))
		json.append(('description','Compressor - Chorus/Flanger'))
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_invert',self.chorus_flanger_wet_invert])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['compressor_eq_trim',self.compressor_eq_trim])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_eq_trim(self):
		'''(Compressor) EQ Trim'''
		return self._compressor_eq_trim

	@compressor_eq_trim.setter
	def compressor_eq_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 174:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 174' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_invert(self):
		'''(Chorus/Flanger) Wet Invert'''
		return self._chorus_flanger_wet_invert

	@chorus_flanger_wet_invert.setter
	def chorus_flanger_wet_invert(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

