#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx105.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX105(object):
	category = '''Mono-Mono'''
	description = '''Compressor - Phaser'''
	short_description = '''Comp - Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX105 Compressor - Phaser object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.compressor_sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.compressor_attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.compressor_output_level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.compressor_eq_trim = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		data = (bytes[7] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		self.phaser_lfo_waveform = (bytes[8] >> 3) & 0x1
		self.phaser_lfo_frequency = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0xF) << 4)
		self.phaser_manual = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x7) << 4)
		self.phaser_depth = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x3) << 5)
		data = ((bytes[11] & 0xFC) >> 2) | ((bytes[12] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		data = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = (bytes[13] >> 2) & 0x3F
		data = bytes[14] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_wet_invert = bytes[15] & 0x1
		self.routing = (bytes[15] >> 1) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX105 Compressor - Phaser object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.compressor_sensitivity << 5) & 0xE0
		bytes[4] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[5] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[6] = ((self.compressor_output_level >> 5) & 0x3) | ((self.compressor_eq_trim << 2) & 0xFC)
		bytes[7] = ((self.compressor_eq_trim >> 6) & 0x1) | ((self.compressor_pre_leq_gain & 0x1F) << 1) | ((self.compressor_pre_heq_gain << 6) & 0xC0)
		bytes[8] = ((self.compressor_pre_heq_gain >> 2) & 0x7) | ((self.phaser_lfo_waveform & 0x1) << 3) | ((self.phaser_lfo_frequency << 4) & 0xF0)
		bytes[9] = ((self.phaser_lfo_frequency >> 4) & 0xF) | ((self.phaser_manual << 4) & 0xF0)
		bytes[10] = ((self.phaser_manual >> 4) & 0x7) | ((self.phaser_depth << 3) & 0xF8)
		bytes[11] = ((self.phaser_depth >> 5) & 0x3) | ((self.phaser_resonance << 2) & 0xFC)
		bytes[12] = ((self.phaser_resonance >> 6) & 0x3) | ((self.phaser_wet_dry << 2) & 0xFC)
		bytes[13] = ((self.phaser_wet_dry >> 6) & 0x3) | ((self.phaser_wet_dry_mod_source & 0x3F) << 2)
		bytes[14] = self.phaser_wet_dry_mod_intensity & 0xFF
		bytes[15] = (self.phaser_wet_invert & 0x1) | ((self.routing & 0x1) << 1)
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX105 Compressor - Phaser object'''
		json = []
		json.append(('name','FX105'))
		json.append(('description','Compressor - Phaser'))
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_invert',self.phaser_wet_invert])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['compressor_eq_trim',self.compressor_eq_trim])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_eq_trim(self):
		'''(Compressor) EQ Trim'''
		return self._compressor_eq_trim

	@compressor_eq_trim.setter
	def compressor_eq_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_invert(self):
		'''(Phaser) Wet Invert'''
		return self._phaser_wet_invert

	@phaser_wet_invert.setter
	def phaser_wet_invert(self,value):
		if 0 <= value <= 1:
			self._phaser_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

