#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx108.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX108(object):
	category = '''Mono-Mono'''
	description = '''Limiter - Chorus/Flanger'''
	short_description = '''Limiter - Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX108 Limiter - Chorus/Flanger object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.limiter_ratio = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.limiter_release = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		self.chorus_flanger_pre_delay = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0xF) << 7)
		self.chorus_flanger_lfo_waveform = (bytes[9] >> 4) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x1F) << 3)
		self.chorus_flanger_depth = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		self.chorus_flanger_eq_trim = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		data = (bytes[13] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = bytes[14] & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data
		data = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0x7) << 3)
		data = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_wet_invert = (bytes[17] >> 3) & 0x1
		self.routing = (bytes[17] >> 4) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX108 Limiter - Chorus/Flanger object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_ratio << 5) & 0xE0
		bytes[4] = ((self.limiter_ratio >> 3) & 0x1F) | ((self.limiter_threshold << 5) & 0xE0)
		bytes[5] = ((self.limiter_threshold >> 3) & 0xF) | ((self.limiter_attack << 4) & 0xF0)
		bytes[6] = ((self.limiter_attack >> 4) & 0x7) | ((self.limiter_release << 3) & 0xF8)
		bytes[7] = ((self.limiter_release >> 5) & 0x3) | ((self.limiter_gain_adjust << 2) & 0xFC)
		bytes[8] = ((self.limiter_gain_adjust >> 6) & 0x1) | ((self.chorus_flanger_pre_delay << 1) & 0xFE)
		bytes[9] = ((self.chorus_flanger_pre_delay >> 7) & 0xF) | ((self.chorus_flanger_lfo_waveform & 0x1) << 4) | ((self.chorus_flanger_lfo_frequency << 5) & 0xE0)
		bytes[10] = ((self.chorus_flanger_lfo_frequency >> 3) & 0x1F) | ((self.chorus_flanger_depth << 5) & 0xE0)
		bytes[11] = ((self.chorus_flanger_depth >> 3) & 0xF) | ((self.chorus_flanger_feedback << 4) & 0xF0)
		bytes[12] = ((self.chorus_flanger_feedback >> 4) & 0xF) | ((self.chorus_flanger_eq_trim << 4) & 0xF0)
		bytes[13] = ((self.chorus_flanger_eq_trim >> 4) & 0x7) | ((self.chorus_flanger_pre_leq_gain & 0x1F) << 3)
		bytes[14] = (self.chorus_flanger_pre_heq_gain & 0x1F) | ((self.chorus_flanger_wet_dry << 5) & 0xE0)
		bytes[15] = ((self.chorus_flanger_wet_dry >> 3) & 0x1F) | ((self.chorus_flanger_wet_dry_mod_source << 5) & 0xE0)
		bytes[16] = ((self.chorus_flanger_wet_dry_mod_source >> 3) & 0x7) | ((self.chorus_flanger_wet_dry_mod_intensity << 3) & 0xF8)
		bytes[17] = ((self.chorus_flanger_wet_dry_mod_intensity >> 5) & 0x7) | ((self.chorus_flanger_wet_invert & 0x1) << 3) | ((self.routing & 0x1) << 4)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX108 Limiter - Chorus/Flanger object'''
		json = []
		json.append(('name','FX108'))
		json.append(('description','Limiter - Chorus/Flanger'))
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_invert',self.chorus_flanger_wet_invert])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['chorus_flanger_eq_trim',self.chorus_flanger_eq_trim])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 175:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 175' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_eq_trim(self):
		'''(Chorus/Flanger) EQ Trim'''
		return self._chorus_flanger_eq_trim

	@chorus_flanger_eq_trim.setter
	def chorus_flanger_eq_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_invert(self):
		'''(Chorus/Flanger) Wet Invert'''
		return self._chorus_flanger_wet_invert

	@chorus_flanger_wet_invert.setter
	def chorus_flanger_wet_invert(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

