#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx109.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX109(object):
	category = '''Mono-Mono'''
	description = '''Limiter - Phaser'''
	short_description = '''Limiter - Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX109 Limiter - Phaser object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.limiter_ratio = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.limiter_release = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		self.phaser_lfo_waveform = (bytes[8] >> 1) & 0x1
		self.phaser_lfo_frequency = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x3) << 6)
		self.phaser_manual = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0x1) << 6)
		self.phaser_depth = (bytes[10] >> 1) & 0x7F
		data = bytes[11] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		data = bytes[12] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = bytes[13] & 0x3F
		data = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_wet_invert = (bytes[14] >> 6) & 0x1
		self.routing = (bytes[14] >> 7) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX109 Limiter - Phaser object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_ratio << 5) & 0xE0
		bytes[4] = ((self.limiter_ratio >> 3) & 0x1F) | ((self.limiter_threshold << 5) & 0xE0)
		bytes[5] = ((self.limiter_threshold >> 3) & 0xF) | ((self.limiter_attack << 4) & 0xF0)
		bytes[6] = ((self.limiter_attack >> 4) & 0x7) | ((self.limiter_release << 3) & 0xF8)
		bytes[7] = ((self.limiter_release >> 5) & 0x3) | ((self.limiter_gain_adjust << 2) & 0xFC)
		bytes[8] = ((self.limiter_gain_adjust >> 6) & 0x1) | ((self.phaser_lfo_waveform & 0x1) << 1) | ((self.phaser_lfo_frequency << 2) & 0xFC)
		bytes[9] = ((self.phaser_lfo_frequency >> 6) & 0x3) | ((self.phaser_manual << 2) & 0xFC)
		bytes[10] = ((self.phaser_manual >> 6) & 0x1) | ((self.phaser_depth & 0x7F) << 1)
		bytes[11] = self.phaser_resonance & 0xFF
		bytes[12] = self.phaser_wet_dry & 0xFF
		bytes[13] = (self.phaser_wet_dry_mod_source & 0x3F) | ((self.phaser_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[14] = ((self.phaser_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.phaser_wet_invert & 0x1) << 6) | ((self.routing & 0x1) << 7)
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX109 Limiter - Phaser object'''
		json = []
		json.append(('name','FX109'))
		json.append(('description','Limiter - Phaser'))
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_invert',self.phaser_wet_invert])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_invert(self):
		'''(Phaser) Wet Invert'''
		return self._phaser_wet_invert

	@phaser_wet_invert.setter
	def phaser_wet_invert(self,value):
		if 0 <= value <= 1:
			self._phaser_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

