#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx110.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX110(object):
	category = '''Mono-Mono'''
	description = '''Limiter - Multitap Delay'''
	short_description = '''Limiter - Mt.Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX110 Limiter - Multitap Delay object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.limiter_ratio = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.limiter_release = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		self.multitap_delay_tap1_time = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x7F) << 7)
		self.multitap_delay_tap2_time = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0xFF) << 1)
		self.multitap_delay_tap1_level = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_feedback = data
		self.multitap_delay_high_damp = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		self.multitap_delay_wet_dry = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		self.multitap_delay_wet_dry_mod_source = (bytes[15] >> 2) & 0x3F
		data = bytes[16] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_wet_dry_mod_intensity = data
		self.routing = bytes[17] & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX110 Limiter - Multitap Delay object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_ratio << 5) & 0xE0
		bytes[4] = ((self.limiter_ratio >> 3) & 0x1F) | ((self.limiter_threshold << 5) & 0xE0)
		bytes[5] = ((self.limiter_threshold >> 3) & 0xF) | ((self.limiter_attack << 4) & 0xF0)
		bytes[6] = ((self.limiter_attack >> 4) & 0x7) | ((self.limiter_release << 3) & 0xF8)
		bytes[7] = ((self.limiter_release >> 5) & 0x3) | ((self.limiter_gain_adjust << 2) & 0xFC)
		bytes[8] = ((self.limiter_gain_adjust >> 6) & 0x1) | ((self.multitap_delay_tap1_time << 1) & 0xFE)
		bytes[9] = ((self.multitap_delay_tap1_time >> 7) & 0x7F) | ((self.multitap_delay_tap2_time << 7) & 0x80)
		bytes[10] = (self.multitap_delay_tap2_time >> 1) & 0xFF
		bytes[11] = (self.multitap_delay_tap1_level << 5) & 0xE0
		bytes[12] = ((self.multitap_delay_tap1_level >> 3) & 0xF) | ((self.multitap_delay_feedback << 4) & 0xF0)
		bytes[13] = ((self.multitap_delay_feedback >> 4) & 0xF) | ((self.multitap_delay_high_damp << 4) & 0xF0)
		bytes[14] = ((self.multitap_delay_high_damp >> 4) & 0x7) | ((self.multitap_delay_wet_dry << 3) & 0xF8)
		bytes[15] = ((self.multitap_delay_wet_dry >> 5) & 0x3) | ((self.multitap_delay_wet_dry_mod_source & 0x3F) << 2)
		bytes[16] = self.multitap_delay_wet_dry_mod_intensity & 0xFF
		bytes[17] = self.routing & 0x1
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX110 Limiter - Multitap Delay object'''
		json = []
		json.append(('name','FX110'))
		json.append(('description','Limiter - Multitap Delay'))
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['multitap_delay_tap1_time',self.multitap_delay_tap1_time])
		json.append(['multitap_delay_tap2_time',self.multitap_delay_tap2_time])
		json.append(['multitap_delay_tap1_level',self.multitap_delay_tap1_level])
		json.append(['multitap_delay_feedback',self.multitap_delay_feedback])
		json.append(['multitap_delay_high_damp',self.multitap_delay_high_damp])
		json.append(['multitap_delay_wet_dry',self.multitap_delay_wet_dry])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['multitap_delay_wet_dry_mod_source',self.multitap_delay_wet_dry_mod_source])
		json.append(['multitap_delay_wet_dry_mod_intensity',self.multitap_delay_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_time(self):
		'''(Multitap Delay) Tap1 Time'''
		return self._multitap_delay_tap1_time

	@multitap_delay_tap1_time.setter
	def multitap_delay_tap1_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap2_time(self):
		'''(Multitap Delay) Tap2 Time'''
		return self._multitap_delay_tap2_time

	@multitap_delay_tap2_time.setter
	def multitap_delay_tap2_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_level(self):
		'''(Multitap Delay) Tap1 Level'''
		return self._multitap_delay_tap1_level

	@multitap_delay_tap1_level.setter
	def multitap_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_feedback(self):
		'''(Multitap Delay) Feedback'''
		return self._multitap_delay_feedback

	@multitap_delay_feedback.setter
	def multitap_delay_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_high_damp(self):
		'''(Multitap Delay) High Damp'''
		return self._multitap_delay_high_damp

	@multitap_delay_high_damp.setter
	def multitap_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry(self):
		'''(Multitap Delay) Wet/Dry'''
		return self._multitap_delay_wet_dry

	@multitap_delay_wet_dry.setter
	def multitap_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_source(self):
		'''(Multitap Delay) Wet/Dry Mod.Source'''
		return self._multitap_delay_wet_dry_mod_source

	@multitap_delay_wet_dry_mod_source.setter
	def multitap_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_intensity(self):
		'''(Multitap Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_delay_wet_dry_mod_intensity

	@multitap_delay_wet_dry_mod_intensity.setter
	def multitap_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

