#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx111.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX111(object):
	category = '''Mono-Mono'''
	description = '''Exciter - Compressor'''
	short_description = '''Exciter - Comp'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX111 Exciter - Compressor object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.exciter_eq_trim = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		data = (bytes[6] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = bytes[7] & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.compressor_sensitivity = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0xF) << 3)
		self.compressor_attack = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x7) << 4)
		self.compressor_output_level = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0x3) << 5)
		self.routing = (bytes[10] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX111 Exciter - Compressor object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.exciter_blend << 5) & 0xE0
		bytes[4] = ((self.exciter_blend >> 3) & 0x1F) | ((self.exciter_emphasis_frequency << 5) & 0xE0)
		bytes[5] = ((self.exciter_emphasis_frequency >> 3) & 0xF) | ((self.exciter_eq_trim << 4) & 0xF0)
		bytes[6] = ((self.exciter_eq_trim >> 4) & 0x7) | ((self.exciter_pre_leq_gain & 0x1F) << 3)
		bytes[7] = (self.exciter_pre_heq_gain & 0x1F) | ((self.compressor_sensitivity << 5) & 0xE0)
		bytes[8] = ((self.compressor_sensitivity >> 3) & 0xF) | ((self.compressor_attack << 4) & 0xF0)
		bytes[9] = ((self.compressor_attack >> 4) & 0x7) | ((self.compressor_output_level << 3) & 0xF8)
		bytes[10] = ((self.compressor_output_level >> 5) & 0x3) | ((self.routing & 0x1) << 2)
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX111 Exciter - Compressor object'''
		json = []
		json.append(('name','FX111'))
		json.append(('description','Exciter - Compressor'))
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['exciter_eq_trim',self.exciter_eq_trim])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_eq_trim(self):
		'''(Exciter) EQ Trim'''
		return self._exciter_eq_trim

	@exciter_eq_trim.setter
	def exciter_eq_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

