#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx114.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX114(object):
	category = '''Mono-Mono'''
	description = '''Exciter - Phaser'''
	short_description = '''Exciter - Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX114 Exciter - Phaser object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.exciter_eq_trim = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		data = (bytes[6] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = bytes[7] & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.phaser_lfo_waveform = (bytes[7] >> 5) & 0x1
		self.phaser_lfo_frequency = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x3F) << 2)
		self.phaser_manual = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x1F) << 2)
		self.phaser_depth = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0xF) << 3)
		data = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x3) << 4)
		data = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_wet_invert = (bytes[14] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX114 Exciter - Phaser object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.exciter_blend << 5) & 0xE0
		bytes[4] = ((self.exciter_blend >> 3) & 0x1F) | ((self.exciter_emphasis_frequency << 5) & 0xE0)
		bytes[5] = ((self.exciter_emphasis_frequency >> 3) & 0xF) | ((self.exciter_eq_trim << 4) & 0xF0)
		bytes[6] = ((self.exciter_eq_trim >> 4) & 0x7) | ((self.exciter_pre_leq_gain & 0x1F) << 3)
		bytes[7] = (self.exciter_pre_heq_gain & 0x1F) | ((self.phaser_lfo_waveform & 0x1) << 5) | ((self.phaser_lfo_frequency << 6) & 0xC0)
		bytes[8] = ((self.phaser_lfo_frequency >> 2) & 0x3F) | ((self.phaser_manual << 6) & 0xC0)
		bytes[9] = ((self.phaser_manual >> 2) & 0x1F) | ((self.phaser_depth << 5) & 0xE0)
		bytes[10] = ((self.phaser_depth >> 3) & 0xF) | ((self.phaser_resonance << 4) & 0xF0)
		bytes[11] = ((self.phaser_resonance >> 4) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[12] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.phaser_wet_dry_mod_source << 4) & 0xF0)
		bytes[13] = ((self.phaser_wet_dry_mod_source >> 4) & 0x3) | ((self.phaser_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[14] = ((self.phaser_wet_dry_mod_intensity >> 6) & 0x3) | ((self.phaser_wet_invert & 0x1) << 2)
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX114 Exciter - Phaser object'''
		json = []
		json.append(('name','FX114'))
		json.append(('description','Exciter - Phaser'))
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_invert',self.phaser_wet_invert])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['exciter_eq_trim',self.exciter_eq_trim])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_eq_trim(self):
		'''(Exciter) EQ Trim'''
		return self._exciter_eq_trim

	@exciter_eq_trim.setter
	def exciter_eq_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_invert(self):
		'''(Phaser) Wet Invert'''
		return self._phaser_wet_invert

	@phaser_wet_invert.setter
	def phaser_wet_invert(self,value):
		if 0 <= value <= 1:
			self._phaser_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

