#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx115.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX115(object):
	category = '''Mono-Mono'''
	description = '''Exciter - Multitap Delay'''
	short_description = '''Exciter - Mt.Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX115 Exciter - Multitap Delay object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.exciter_eq_trim = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		data = (bytes[6] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = bytes[7] & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.multitap_delay_tap1_time = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0xFF) << 3)
		self.multitap_delay_tap2_time = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0xFF) << 5)
		self.multitap_delay_tap1_level = (bytes[11] >> 1) & 0x7F
		data = bytes[12] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_feedback = data
		self.multitap_delay_high_damp = bytes[13] & 0x7F
		self.multitap_delay_wet_dry = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x3F) << 1)
		self.multitap_delay_wet_dry_mod_source = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0xF) << 2)
		data = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_wet_dry_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX115 Exciter - Multitap Delay object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.exciter_blend << 5) & 0xE0
		bytes[4] = ((self.exciter_blend >> 3) & 0x1F) | ((self.exciter_emphasis_frequency << 5) & 0xE0)
		bytes[5] = ((self.exciter_emphasis_frequency >> 3) & 0xF) | ((self.exciter_eq_trim << 4) & 0xF0)
		bytes[6] = ((self.exciter_eq_trim >> 4) & 0x7) | ((self.exciter_pre_leq_gain & 0x1F) << 3)
		bytes[7] = (self.exciter_pre_heq_gain & 0x1F) | ((self.multitap_delay_tap1_time << 5) & 0xE0)
		bytes[8] = (self.multitap_delay_tap1_time >> 3) & 0xFF
		bytes[9] = (self.multitap_delay_tap2_time << 3) & 0xF8
		bytes[10] = (self.multitap_delay_tap2_time >> 5) & 0xFF
		bytes[11] = (self.multitap_delay_tap1_level & 0x7F) << 1
		bytes[12] = self.multitap_delay_feedback & 0xFF
		bytes[13] = (self.multitap_delay_high_damp & 0x7F) | ((self.multitap_delay_wet_dry << 7) & 0x80)
		bytes[14] = ((self.multitap_delay_wet_dry >> 1) & 0x3F) | ((self.multitap_delay_wet_dry_mod_source << 6) & 0xC0)
		bytes[15] = ((self.multitap_delay_wet_dry_mod_source >> 2) & 0xF) | ((self.multitap_delay_wet_dry_mod_intensity << 4) & 0xF0)
		bytes[16] = (self.multitap_delay_wet_dry_mod_intensity >> 4) & 0xF
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX115 Exciter - Multitap Delay object'''
		json = []
		json.append(('name','FX115'))
		json.append(('description','Exciter - Multitap Delay'))
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['multitap_delay_tap1_time',self.multitap_delay_tap1_time])
		json.append(['multitap_delay_tap2_time',self.multitap_delay_tap2_time])
		json.append(['multitap_delay_tap1_level',self.multitap_delay_tap1_level])
		json.append(['multitap_delay_feedback',self.multitap_delay_feedback])
		json.append(['multitap_delay_high_damp',self.multitap_delay_high_damp])
		json.append(['multitap_delay_wet_dry',self.multitap_delay_wet_dry])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['exciter_eq_trim',self.exciter_eq_trim])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['multitap_delay_wet_dry_mod_source',self.multitap_delay_wet_dry_mod_source])
		json.append(['multitap_delay_wet_dry_mod_intensity',self.multitap_delay_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_eq_trim(self):
		'''(Exciter) EQ Trim'''
		return self._exciter_eq_trim

	@exciter_eq_trim.setter
	def exciter_eq_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_time(self):
		'''(Multitap Delay) Tap1 Time'''
		return self._multitap_delay_tap1_time

	@multitap_delay_tap1_time.setter
	def multitap_delay_tap1_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap2_time(self):
		'''(Multitap Delay) Tap2 Time'''
		return self._multitap_delay_tap2_time

	@multitap_delay_tap2_time.setter
	def multitap_delay_tap2_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_level(self):
		'''(Multitap Delay) Tap1 Level'''
		return self._multitap_delay_tap1_level

	@multitap_delay_tap1_level.setter
	def multitap_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_feedback(self):
		'''(Multitap Delay) Feedback'''
		return self._multitap_delay_feedback

	@multitap_delay_feedback.setter
	def multitap_delay_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_high_damp(self):
		'''(Multitap Delay) High Damp'''
		return self._multitap_delay_high_damp

	@multitap_delay_high_damp.setter
	def multitap_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry(self):
		'''(Multitap Delay) Wet/Dry'''
		return self._multitap_delay_wet_dry

	@multitap_delay_wet_dry.setter
	def multitap_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_source(self):
		'''(Multitap Delay) Wet/Dry Mod.Source'''
		return self._multitap_delay_wet_dry_mod_source

	@multitap_delay_wet_dry_mod_source.setter
	def multitap_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_intensity(self):
		'''(Multitap Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_delay_wet_dry_mod_intensity

	@multitap_delay_wet_dry_mod_intensity.setter
	def multitap_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

