#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx118.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX118(object):
	category = '''Mono-Mono'''
	description = '''Overdrive/Hi.Gain - Phaser'''
	short_description = '''OD/HG - Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX118 Overdrive/Hi.Gain - Phaser object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.od_hi_gain_hi_gain_on = (bytes[3] >> 5) & 0x1
		self.od_hi_gain_drive = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.od_hi_gain_output_level = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		self.od_hi_gain_output_level_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = (bytes[6] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = bytes[7] & 0x3F
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x3) << 4)
		self.od_hi_gain_eq_mid1_q = (bytes[9] >> 2) & 0xF
		data = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0x3) << 4)
		self.od_hi_gain_eq_mid2_q = (bytes[11] >> 2) & 0xF
		data = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data
		self.phaser_lfo_waveform = (bytes[12] >> 4) & 0x1
		self.phaser_lfo_frequency = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0x1F) << 3)
		self.phaser_manual = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		self.phaser_depth = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		data = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		data = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[17] & 0xF8) >> 3) | ((bytes[18] & 0x1) << 5)
		data = ((bytes[18] & 0xFE) >> 1) | ((bytes[19] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_wet_invert = (bytes[19] >> 1) & 0x1
		self.routing = (bytes[19] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX118 Overdrive/Hi.Gain - Phaser object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.od_hi_gain_hi_gain_on & 0x1) << 5) | ((self.od_hi_gain_drive << 6) & 0xC0)
		bytes[4] = ((self.od_hi_gain_drive >> 2) & 0x1F) | ((self.od_hi_gain_output_level << 5) & 0xE0)
		bytes[5] = ((self.od_hi_gain_output_level >> 3) & 0x7) | ((self.od_hi_gain_output_level_mod_source << 3) & 0xF8)
		bytes[6] = ((self.od_hi_gain_output_level_mod_source >> 5) & 0x1) | ((self.od_hi_gain_output_level_mod_intensity & 0x7F) << 1)
		bytes[7] = (self.od_hi_gain_eq_low_cutoff & 0x3F) | ((self.od_hi_gain_eq_low_gain << 6) & 0xC0)
		bytes[8] = ((self.od_hi_gain_eq_low_gain >> 2) & 0xF) | ((self.od_hi_gain_eq_mid1_cutoff << 4) & 0xF0)
		bytes[9] = ((self.od_hi_gain_eq_mid1_cutoff >> 4) & 0x3) | ((self.od_hi_gain_eq_mid1_q & 0xF) << 2) | ((self.od_hi_gain_eq_mid1_gain << 6) & 0xC0)
		bytes[10] = ((self.od_hi_gain_eq_mid1_gain >> 2) & 0xF) | ((self.od_hi_gain_eq_mid2_cutoff << 4) & 0xF0)
		bytes[11] = ((self.od_hi_gain_eq_mid2_cutoff >> 4) & 0x3) | ((self.od_hi_gain_eq_mid2_q & 0xF) << 2) | ((self.od_hi_gain_eq_mid2_gain << 6) & 0xC0)
		bytes[12] = ((self.od_hi_gain_eq_mid2_gain >> 2) & 0xF) | ((self.phaser_lfo_waveform & 0x1) << 4) | ((self.phaser_lfo_frequency << 5) & 0xE0)
		bytes[13] = ((self.phaser_lfo_frequency >> 3) & 0x1F) | ((self.phaser_manual << 5) & 0xE0)
		bytes[14] = ((self.phaser_manual >> 3) & 0xF) | ((self.phaser_depth << 4) & 0xF0)
		bytes[15] = ((self.phaser_depth >> 4) & 0x7) | ((self.phaser_resonance << 3) & 0xF8)
		bytes[16] = ((self.phaser_resonance >> 5) & 0x7) | ((self.phaser_wet_dry << 3) & 0xF8)
		bytes[17] = ((self.phaser_wet_dry >> 5) & 0x7) | ((self.phaser_wet_dry_mod_source << 3) & 0xF8)
		bytes[18] = ((self.phaser_wet_dry_mod_source >> 5) & 0x1) | ((self.phaser_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[19] = ((self.phaser_wet_dry_mod_intensity >> 7) & 0x1) | ((self.phaser_wet_invert & 0x1) << 1) | ((self.routing & 0x1) << 2)
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX118 Overdrive/Hi.Gain - Phaser object'''
		json = []
		json.append(('name','FX118'))
		json.append(('description','Overdrive/Hi.Gain - Phaser'))
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_invert',self.phaser_wet_invert])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_invert(self):
		'''(Phaser) Wet Invert'''
		return self._phaser_wet_invert

	@phaser_wet_invert.setter
	def phaser_wet_invert(self,value):
		if 0 <= value <= 1:
			self._phaser_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

