#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx119.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX119(object):
	category = '''Mono-Mono'''
	description = '''Overdrive/Hi.Gain - Multitap Delay'''
	short_description = '''OD/HG - Mt.Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX119 Overdrive/Hi.Gain - Multitap Delay object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.od_hi_gain_hi_gainon = (bytes[3] >> 5) & 0x1
		self.od_hi_gain_drive = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.od_hi_gain_output_level = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		self.od_hi_gain_output_level_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = (bytes[6] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = bytes[7] & 0x3F
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x3) << 4)
		self.od_hi_gain_eq_mid1_q = (bytes[9] >> 2) & 0xF
		data = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0x3) << 4)
		self.od_hi_gain_eq_mid2_q = (bytes[11] >> 2) & 0xF
		data = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data
		self.multitap_delay_tap1_time = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xFF) << 4)
		self.multitap_delay_tap2_time = ((bytes[14] & 0xFC) >> 2) | ((bytes[15] & 0xFF) << 6)
		self.multitap_delay_tap1_level = bytes[16] & 0x7F
		data = ((bytes[16] & 0x80) >> 7) | ((bytes[17] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_feedback = data
		self.multitap_delay_high_damp = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x3F) << 1)
		self.multitap_delay_wet_dry = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0x1F) << 2)
		self.multitap_delay_wet_dry_mod_source = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x7) << 3)
		data = ((bytes[20] & 0xF8) >> 3) | ((bytes[21] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_wet_dry_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX119 Overdrive/Hi.Gain - Multitap Delay object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.od_hi_gain_hi_gainon & 0x1) << 5) | ((self.od_hi_gain_drive << 6) & 0xC0)
		bytes[4] = ((self.od_hi_gain_drive >> 2) & 0x1F) | ((self.od_hi_gain_output_level << 5) & 0xE0)
		bytes[5] = ((self.od_hi_gain_output_level >> 3) & 0x7) | ((self.od_hi_gain_output_level_mod_source << 3) & 0xF8)
		bytes[6] = ((self.od_hi_gain_output_level_mod_source >> 5) & 0x1) | ((self.od_hi_gain_output_level_mod_intensity & 0x7F) << 1)
		bytes[7] = (self.od_hi_gain_eq_low_cutoff & 0x3F) | ((self.od_hi_gain_eq_low_gain << 6) & 0xC0)
		bytes[8] = ((self.od_hi_gain_eq_low_gain >> 2) & 0xF) | ((self.od_hi_gain_eq_mid1_cutoff << 4) & 0xF0)
		bytes[9] = ((self.od_hi_gain_eq_mid1_cutoff >> 4) & 0x3) | ((self.od_hi_gain_eq_mid1_q & 0xF) << 2) | ((self.od_hi_gain_eq_mid1_gain << 6) & 0xC0)
		bytes[10] = ((self.od_hi_gain_eq_mid1_gain >> 2) & 0xF) | ((self.od_hi_gain_eq_mid2_cutoff << 4) & 0xF0)
		bytes[11] = ((self.od_hi_gain_eq_mid2_cutoff >> 4) & 0x3) | ((self.od_hi_gain_eq_mid2_q & 0xF) << 2) | ((self.od_hi_gain_eq_mid2_gain << 6) & 0xC0)
		bytes[12] = ((self.od_hi_gain_eq_mid2_gain >> 2) & 0xF) | ((self.multitap_delay_tap1_time << 4) & 0xF0)
		bytes[13] = (self.multitap_delay_tap1_time >> 4) & 0xFF
		bytes[14] = (self.multitap_delay_tap2_time << 2) & 0xFC
		bytes[15] = (self.multitap_delay_tap2_time >> 6) & 0xFF
		bytes[16] = (self.multitap_delay_tap1_level & 0x7F) | ((self.multitap_delay_feedback << 7) & 0x80)
		bytes[17] = ((self.multitap_delay_feedback >> 1) & 0x7F) | ((self.multitap_delay_high_damp << 7) & 0x80)
		bytes[18] = ((self.multitap_delay_high_damp >> 1) & 0x3F) | ((self.multitap_delay_wet_dry << 6) & 0xC0)
		bytes[19] = ((self.multitap_delay_wet_dry >> 2) & 0x1F) | ((self.multitap_delay_wet_dry_mod_source << 5) & 0xE0)
		bytes[20] = ((self.multitap_delay_wet_dry_mod_source >> 3) & 0x7) | ((self.multitap_delay_wet_dry_mod_intensity << 3) & 0xF8)
		bytes[21] = (self.multitap_delay_wet_dry_mod_intensity >> 5) & 0x7
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX119 Overdrive/Hi.Gain - Multitap Delay object'''
		json = []
		json.append(('name','FX119'))
		json.append(('description','Overdrive/Hi.Gain - Multitap Delay'))
		json.append(['od_hi_gain_hi_gainon',self.od_hi_gain_hi_gainon])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		json.append(['multitap_delay_tap1_time',self.multitap_delay_tap1_time])
		json.append(['multitap_delay_tap2_time',self.multitap_delay_tap2_time])
		json.append(['multitap_delay_tap1_level',self.multitap_delay_tap1_level])
		json.append(['multitap_delay_feedback',self.multitap_delay_feedback])
		json.append(['multitap_delay_high_damp',self.multitap_delay_high_damp])
		json.append(['multitap_delay_wet_dry',self.multitap_delay_wet_dry])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['multitap_delay_wet_dry_mod_source',self.multitap_delay_wet_dry_mod_source])
		json.append(['multitap_delay_wet_dry_mod_intensity',self.multitap_delay_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gainon(self):
		'''(OD/Hi-Gain) Hi-GainOn'''
		return self._od_hi_gain_hi_gainon

	@od_hi_gain_hi_gainon.setter
	def od_hi_gain_hi_gainon(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gainon = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_time(self):
		'''(Multitap Delay) Tap1 Time'''
		return self._multitap_delay_tap1_time

	@multitap_delay_tap1_time.setter
	def multitap_delay_tap1_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap2_time(self):
		'''(Multitap Delay) Tap2 Time'''
		return self._multitap_delay_tap2_time

	@multitap_delay_tap2_time.setter
	def multitap_delay_tap2_time(self,value):
		if 0 <= value <= 5490:
			self._multitap_delay_tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 5490' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_level(self):
		'''(Multitap Delay) Tap1 Level'''
		return self._multitap_delay_tap1_level

	@multitap_delay_tap1_level.setter
	def multitap_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_feedback(self):
		'''(Multitap Delay) Feedback'''
		return self._multitap_delay_feedback

	@multitap_delay_feedback.setter
	def multitap_delay_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_high_damp(self):
		'''(Multitap Delay) High Damp'''
		return self._multitap_delay_high_damp

	@multitap_delay_high_damp.setter
	def multitap_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry(self):
		'''(Multitap Delay) Wet/Dry'''
		return self._multitap_delay_wet_dry

	@multitap_delay_wet_dry.setter
	def multitap_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_source(self):
		'''(Multitap Delay) Wet/Dry Mod.Source'''
		return self._multitap_delay_wet_dry_mod_source

	@multitap_delay_wet_dry_mod_source.setter
	def multitap_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_intensity(self):
		'''(Multitap Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_delay_wet_dry_mod_intensity

	@multitap_delay_wet_dry_mod_intensity.setter
	def multitap_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

