#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx120.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX120(object):
	category = '''Mono-Mono'''
	description = '''Wah - Amp Simulation'''
	short_description = '''Wah - Amp Sim'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX120 Wah - Amp Simulation object'''
		bytes = struct.unpack('<12B',raw[:12])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.wah_frequency_bottom = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.wah_frequency_top = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.wah_sweep_mode = (bytes[5] >> 3) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		self.wah_lfo_frequency = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		self.wah_resonance = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		self.wah_low_pass_filter = (bytes[8] >> 2) & 0x1
		self.wah_wet_dry = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x3) << 5)
		self.wah_wet_dry_mod_source = (bytes[9] >> 2) & 0x3F
		data = bytes[10] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.amp_simulation_amplifier_type = bytes[11] & 0x3
		self.routing = (bytes[11] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX120 Wah - Amp Simulation object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.wah_frequency_bottom << 5) & 0xE0
		bytes[4] = ((self.wah_frequency_bottom >> 3) & 0xF) | ((self.wah_frequency_top << 4) & 0xF0)
		bytes[5] = ((self.wah_frequency_top >> 4) & 0x7) | ((self.wah_sweep_mode & 0x3) << 3) | ((self.wah_sweep_mode_mod_source << 5) & 0xE0)
		bytes[6] = ((self.wah_sweep_mode_mod_source >> 3) & 0x7) | ((self.wah_lfo_frequency << 3) & 0xF8)
		bytes[7] = ((self.wah_lfo_frequency >> 5) & 0x7) | ((self.wah_resonance << 3) & 0xF8)
		bytes[8] = ((self.wah_resonance >> 5) & 0x3) | ((self.wah_low_pass_filter & 0x1) << 2) | ((self.wah_wet_dry << 3) & 0xF8)
		bytes[9] = ((self.wah_wet_dry >> 5) & 0x3) | ((self.wah_wet_dry_mod_source & 0x3F) << 2)
		bytes[10] = self.wah_wet_dry_mod_intensity & 0xFF
		bytes[11] = (self.amp_simulation_amplifier_type & 0x3) | ((self.routing & 0x1) << 2)
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX120 Wah - Amp Simulation object'''
		json = []
		json.append(('name','FX120'))
		json.append(('description','Wah - Amp Simulation'))
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['amp_simulation_amplifier_type',self.amp_simulation_amplifier_type])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def amp_simulation_amplifier_type(self):
		'''(Amp Simulation) Amplifier Type'''
		return self._amp_simulation_amplifier_type

	@amp_simulation_amplifier_type.setter
	def amp_simulation_amplifier_type(self,value):
		if 0 <= value <= 2:
			self._amp_simulation_amplifier_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

