#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx124.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX124(object):
	category = '''Mono-Mono'''
	description = '''Chorus/Flanger - Multitap Delay'''
	short_description = '''Cho/Flng - Mt.Dly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX124 Chorus/Flanger - Multitap Delay object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.chorus_flanger_pre_delay = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xFF) << 3)
		self.chorus_flanger_lfo_waveform = bytes[5] & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[5] & 0xFE) >> 1) | ((bytes[6] & 0x1) << 7)
		self.chorus_flanger_depth = (bytes[6] >> 1) & 0x7F
		data = bytes[7] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		self.chorus_flanger_eq_trim = bytes[8] & 0x7F
		data = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data
		data = ((bytes[10] & 0xFE) >> 1) | ((bytes[11] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.multitap_delay_tap1_time = ((bytes[11] & 0xFE) >> 1) | ((bytes[12] & 0x7F) << 7)
		self.multitap_delay_tap2_time = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0xFF) << 1)
		self.multitap_delay_tap1_level = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0xF) << 3)
		data = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_feedback = data
		self.multitap_delay_high_damp = ((bytes[16] & 0xF0) >> 4) | ((bytes[17] & 0x7) << 4)
		self.multitap_delay_wet_dry = ((bytes[17] & 0xF8) >> 3) | ((bytes[18] & 0x3) << 5)
		self.multitap_delay_wet_dry_mod_source = (bytes[18] >> 2) & 0x3F
		data = bytes[19] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.multitap_delay_wet_dry_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX124 Chorus/Flanger - Multitap Delay object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.chorus_flanger_pre_delay << 5) & 0xE0
		bytes[4] = (self.chorus_flanger_pre_delay >> 3) & 0xFF
		bytes[5] = (self.chorus_flanger_lfo_waveform & 0x1) | ((self.chorus_flanger_lfo_frequency << 1) & 0xFE)
		bytes[6] = ((self.chorus_flanger_lfo_frequency >> 7) & 0x1) | ((self.chorus_flanger_depth & 0x7F) << 1)
		bytes[7] = self.chorus_flanger_feedback & 0xFF
		bytes[8] = (self.chorus_flanger_eq_trim & 0x7F) | ((self.chorus_flanger_pre_leq_gain << 7) & 0x80)
		bytes[9] = ((self.chorus_flanger_pre_leq_gain >> 1) & 0xF) | ((self.chorus_flanger_pre_heq_gain << 4) & 0xF0)
		bytes[10] = ((self.chorus_flanger_pre_heq_gain >> 4) & 0x1) | ((self.chorus_flanger_wet_dry << 1) & 0xFE)
		bytes[11] = ((self.chorus_flanger_wet_dry >> 7) & 0x1) | ((self.multitap_delay_tap1_time << 1) & 0xFE)
		bytes[12] = ((self.multitap_delay_tap1_time >> 7) & 0x7F) | ((self.multitap_delay_tap2_time << 7) & 0x80)
		bytes[13] = (self.multitap_delay_tap2_time >> 1) & 0xFF
		bytes[14] = (self.multitap_delay_tap1_level << 5) & 0xE0
		bytes[15] = ((self.multitap_delay_tap1_level >> 3) & 0xF) | ((self.multitap_delay_feedback << 4) & 0xF0)
		bytes[16] = ((self.multitap_delay_feedback >> 4) & 0xF) | ((self.multitap_delay_high_damp << 4) & 0xF0)
		bytes[17] = ((self.multitap_delay_high_damp >> 4) & 0x7) | ((self.multitap_delay_wet_dry << 3) & 0xF8)
		bytes[18] = ((self.multitap_delay_wet_dry >> 5) & 0x3) | ((self.multitap_delay_wet_dry_mod_source & 0x3F) << 2)
		bytes[19] = self.multitap_delay_wet_dry_mod_intensity & 0xFF
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX124 Chorus/Flanger - Multitap Delay object'''
		json = []
		json.append(('name','FX124'))
		json.append(('description','Chorus/Flanger - Multitap Delay'))
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['multitap_delay_tap1_time',self.multitap_delay_tap1_time])
		json.append(['multitap_delay_tap2_time',self.multitap_delay_tap2_time])
		json.append(['multitap_delay_tap1_level',self.multitap_delay_tap1_level])
		json.append(['multitap_delay_feedback',self.multitap_delay_feedback])
		json.append(['multitap_delay_high_damp',self.multitap_delay_high_damp])
		json.append(['multitap_delay_wet_dry',self.multitap_delay_wet_dry])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['chorus_flanger_eq_trim',self.chorus_flanger_eq_trim])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		json.append(['multitap_delay_wet_dry_mod_source',self.multitap_delay_wet_dry_mod_source])
		json.append(['multitap_delay_wet_dry_mod_intensity',self.multitap_delay_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 115:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 115' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_eq_trim(self):
		'''(Chorus/Flanger) EQ Trim'''
		return self._chorus_flanger_eq_trim

	@chorus_flanger_eq_trim.setter
	def chorus_flanger_eq_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_time(self):
		'''(Multitap Delay) Tap1 Time'''
		return self._multitap_delay_tap1_time

	@multitap_delay_tap1_time.setter
	def multitap_delay_tap1_time(self,value):
		if 0 <= value <= 4890:
			self._multitap_delay_tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 4890' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap2_time(self):
		'''(Multitap Delay) Tap2 Time'''
		return self._multitap_delay_tap2_time

	@multitap_delay_tap2_time.setter
	def multitap_delay_tap2_time(self,value):
		if 0 <= value <= 4890:
			self._multitap_delay_tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 4890' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_tap1_level(self):
		'''(Multitap Delay) Tap1 Level'''
		return self._multitap_delay_tap1_level

	@multitap_delay_tap1_level.setter
	def multitap_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_feedback(self):
		'''(Multitap Delay) Feedback'''
		return self._multitap_delay_feedback

	@multitap_delay_feedback.setter
	def multitap_delay_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_high_damp(self):
		'''(Multitap Delay) High Damp'''
		return self._multitap_delay_high_damp

	@multitap_delay_high_damp.setter
	def multitap_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry(self):
		'''(Multitap Delay) Wet/Dry'''
		return self._multitap_delay_wet_dry

	@multitap_delay_wet_dry.setter
	def multitap_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_source(self):
		'''(Multitap Delay) Wet/Dry Mod.Source'''
		return self._multitap_delay_wet_dry_mod_source

	@multitap_delay_wet_dry_mod_source.setter
	def multitap_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_delay_wet_dry_mod_intensity(self):
		'''(Multitap Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_delay_wet_dry_mod_intensity

	@multitap_delay_wet_dry_mod_intensity.setter
	def multitap_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

