#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx125.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX125(object):
	category = '''Mono-Mono'''
	description = '''Phaser - Chorus/Flanger'''
	short_description = '''Phaser - Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX125 Phaser - Chorus/Flanger object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.phaser_lfo_waveform = (bytes[3] >> 5) & 0x1
		self.phaser_lfo_frequency = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x3F) << 2)
		self.phaser_manual = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x1F) << 2)
		self.phaser_depth = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0xF) << 3)
		data = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		data = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.chorus_flanger_pre_delay = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x7F) << 4)
		self.chorus_flanger_lfo_waveform = (bytes[9] >> 7) & 0x1
		self.chorus_flanger_lfo_frequency = bytes[10] & 0xFF
		self.chorus_flanger_depth = bytes[11] & 0x7F
		data = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		self.chorus_flanger_eq_trim = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x3F) << 1)
		data = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = (bytes[14] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data
		data = bytes[15] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = bytes[16] & 0x3F
		data = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_wet_invert = (bytes[17] >> 6) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX125 Phaser - Chorus/Flanger object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.phaser_lfo_waveform & 0x1) << 5) | ((self.phaser_lfo_frequency << 6) & 0xC0)
		bytes[4] = ((self.phaser_lfo_frequency >> 2) & 0x3F) | ((self.phaser_manual << 6) & 0xC0)
		bytes[5] = ((self.phaser_manual >> 2) & 0x1F) | ((self.phaser_depth << 5) & 0xE0)
		bytes[6] = ((self.phaser_depth >> 3) & 0xF) | ((self.phaser_resonance << 4) & 0xF0)
		bytes[7] = ((self.phaser_resonance >> 4) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[8] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.chorus_flanger_pre_delay << 4) & 0xF0)
		bytes[9] = ((self.chorus_flanger_pre_delay >> 4) & 0x7F) | ((self.chorus_flanger_lfo_waveform & 0x1) << 7)
		bytes[10] = self.chorus_flanger_lfo_frequency & 0xFF
		bytes[11] = (self.chorus_flanger_depth & 0x7F) | ((self.chorus_flanger_feedback << 7) & 0x80)
		bytes[12] = ((self.chorus_flanger_feedback >> 1) & 0x7F) | ((self.chorus_flanger_eq_trim << 7) & 0x80)
		bytes[13] = ((self.chorus_flanger_eq_trim >> 1) & 0x3F) | ((self.chorus_flanger_pre_leq_gain << 6) & 0xC0)
		bytes[14] = ((self.chorus_flanger_pre_leq_gain >> 2) & 0x7) | ((self.chorus_flanger_pre_heq_gain & 0x1F) << 3)
		bytes[15] = self.chorus_flanger_wet_dry & 0xFF
		bytes[16] = (self.chorus_flanger_wet_dry_mod_source & 0x3F) | ((self.chorus_flanger_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[17] = ((self.chorus_flanger_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.chorus_flanger_wet_invert & 0x1) << 6)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX125 Phaser - Chorus/Flanger object'''
		json = []
		json.append(('name','FX125'))
		json.append(('description','Phaser - Chorus/Flanger'))
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_invert',self.chorus_flanger_wet_invert])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['chorus_flanger_eq_trim',self.chorus_flanger_eq_trim])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 175:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 175' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_eq_trim(self):
		'''(Chorus/Flanger) EQ Trim'''
		return self._chorus_flanger_eq_trim

	@chorus_flanger_eq_trim.setter
	def chorus_flanger_eq_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_invert(self):
		'''(Chorus/Flanger) Wet Invert'''
		return self._chorus_flanger_wet_invert

	@chorus_flanger_wet_invert.setter
	def chorus_flanger_wet_invert(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_wet_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

