#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx126.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX126(object):
	category = '''Mono-Mono'''
	description = '''Reverb - Gate'''
	short_description = '''Reverb - Gate'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX126 Reverb - Gate object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.reverb_reverb_time = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.reverb_high_damp = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.reverb_pre_delay = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		self.reverb_eq_trim = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = (bytes[7] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.reverb_pre_leq_gain = data
		data = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.reverb_pre_heq_gain = data
		self.reverb_reverb_balance = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x7) << 4)
		self.gate_envelope_select = (bytes[9] >> 3) & 0x1
		self.gate_envelope_select_mod_source = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x3) << 4)
		self.gate_input_reverb_mix = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x1) << 6)
		self.gate_polarity = (bytes[11] >> 1) & 0x1
		self.gate_threshold = ((bytes[11] & 0xFC) >> 2) | ((bytes[12] & 0x1) << 6)
		self.gate_attack = (bytes[12] >> 1) & 0x7F
		self.gate_release = bytes[13] & 0x7F
		self.reverb_pre_leq_fc = (bytes[13] >> 7) & 0x1
		self.reverb_pre_heq_fc = bytes[14] & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX126 Reverb - Gate object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.reverb_reverb_time << 5) & 0xE0
		bytes[4] = ((self.reverb_reverb_time >> 3) & 0xF) | ((self.reverb_high_damp << 4) & 0xF0)
		bytes[5] = ((self.reverb_high_damp >> 4) & 0x7) | ((self.reverb_pre_delay << 3) & 0xF8)
		bytes[6] = ((self.reverb_pre_delay >> 5) & 0x7) | ((self.reverb_eq_trim << 3) & 0xF8)
		bytes[7] = ((self.reverb_eq_trim >> 5) & 0x3) | ((self.reverb_pre_leq_gain & 0x1F) << 2) | ((self.reverb_pre_heq_gain << 7) & 0x80)
		bytes[8] = ((self.reverb_pre_heq_gain >> 1) & 0xF) | ((self.reverb_reverb_balance << 4) & 0xF0)
		bytes[9] = ((self.reverb_reverb_balance >> 4) & 0x7) | ((self.gate_envelope_select & 0x1) << 3) | ((self.gate_envelope_select_mod_source << 4) & 0xF0)
		bytes[10] = ((self.gate_envelope_select_mod_source >> 4) & 0x3) | ((self.gate_input_reverb_mix << 2) & 0xFC)
		bytes[11] = ((self.gate_input_reverb_mix >> 6) & 0x1) | ((self.gate_polarity & 0x1) << 1) | ((self.gate_threshold << 2) & 0xFC)
		bytes[12] = ((self.gate_threshold >> 6) & 0x1) | ((self.gate_attack & 0x7F) << 1)
		bytes[13] = (self.gate_release & 0x7F) | ((self.reverb_pre_leq_fc & 0x1) << 7)
		bytes[14] = self.reverb_pre_heq_fc & 0x1
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX126 Reverb - Gate object'''
		json = []
		json.append(('name','FX126'))
		json.append(('description','Reverb - Gate'))
		json.append(['reverb_reverb_time',self.reverb_reverb_time])
		json.append(['reverb_high_damp',self.reverb_high_damp])
		json.append(['reverb_pre_delay',self.reverb_pre_delay])
		json.append(['reverb_reverb_balance',self.reverb_reverb_balance])
		json.append(['gate_envelope_select',self.gate_envelope_select])
		json.append(['gate_envelope_select_mod_source',self.gate_envelope_select_mod_source])
		json.append(['gate_input_reverb_mix',self.gate_input_reverb_mix])
		json.append(['gate_polarity',self.gate_polarity])
		json.append(['gate_threshold',self.gate_threshold])
		json.append(['gate_attack',self.gate_attack])
		json.append(['gate_release',self.gate_release])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['reverb_eq_trim',self.reverb_eq_trim])
		json.append(['reverb_pre_leq_gain',self.reverb_pre_leq_gain])
		json.append(['reverb_pre_heq_gain',self.reverb_pre_heq_gain])
		json.append(['reverb_pre_leq_fc',self.reverb_pre_leq_fc])
		json.append(['reverb_pre_heq_fc',self.reverb_pre_heq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_reverb_time(self):
		'''(Reverb) Reverb Time'''
		return self._reverb_reverb_time

	@reverb_reverb_time.setter
	def reverb_reverb_time(self,value):
		if 1 <= value <= 100:
			self._reverb_reverb_time = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_high_damp(self):
		'''(Reverb) High Damp'''
		return self._reverb_high_damp

	@reverb_high_damp.setter
	def reverb_high_damp(self,value):
		if 0 <= value <= 100:
			self._reverb_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_pre_delay(self):
		'''(Reverb) Pre Delay'''
		return self._reverb_pre_delay

	@reverb_pre_delay.setter
	def reverb_pre_delay(self,value):
		if 0 <= value <= 200:
			self._reverb_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_eq_trim(self):
		'''(Reverb) EQ Trim'''
		return self._reverb_eq_trim

	@reverb_eq_trim.setter
	def reverb_eq_trim(self,value):
		if 0 <= value <= 100:
			self._reverb_eq_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_pre_leq_gain(self):
		'''(Reverb) Pre LEQ Gain'''
		return self._reverb_pre_leq_gain

	@reverb_pre_leq_gain.setter
	def reverb_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._reverb_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_pre_heq_gain(self):
		'''(Reverb) Pre HEQ Gain'''
		return self._reverb_pre_heq_gain

	@reverb_pre_heq_gain.setter
	def reverb_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._reverb_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_reverb_balance(self):
		'''(Reverb) Reverb Balance'''
		return self._reverb_reverb_balance

	@reverb_reverb_balance.setter
	def reverb_reverb_balance(self,value):
		if 0 <= value <= 100:
			self._reverb_reverb_balance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_envelope_select(self):
		'''(Gate) Envelope Select'''
		return self._gate_envelope_select

	@gate_envelope_select.setter
	def gate_envelope_select(self,value):
		if 0 <= value <= 1:
			self._gate_envelope_select = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_envelope_select_mod_source(self):
		'''(Gate) Envelope Select Mod.Source'''
		return self._gate_envelope_select_mod_source

	@gate_envelope_select_mod_source.setter
	def gate_envelope_select_mod_source(self,value):
		if 0 <= value <= 37:
			self._gate_envelope_select_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_input_reverb_mix(self):
		'''(Gate) Input Reverb Mix'''
		return self._gate_input_reverb_mix

	@gate_input_reverb_mix.setter
	def gate_input_reverb_mix(self,value):
		if 0 <= value <= 100:
			self._gate_input_reverb_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_polarity(self):
		'''(Gate) Polarity'''
		return self._gate_polarity

	@gate_polarity.setter
	def gate_polarity(self,value):
		if 0 <= value <= 1:
			self._gate_polarity = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_threshold(self):
		'''(Gate) Threshold'''
		return self._gate_threshold

	@gate_threshold.setter
	def gate_threshold(self,value):
		if 0 <= value <= 100:
			self._gate_threshold = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_attack(self):
		'''(Gate) Attack'''
		return self._gate_attack

	@gate_attack.setter
	def gate_attack(self,value):
		if 1 <= value <= 100:
			self._gate_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def gate_release(self):
		'''(Gate) Release'''
		return self._gate_release

	@gate_release.setter
	def gate_release(self,value):
		if 1 <= value <= 100:
			self._gate_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_pre_leq_fc(self):
		'''(Reverb) Pre LEQ Fc'''
		return self._reverb_pre_leq_fc

	@reverb_pre_leq_fc.setter
	def reverb_pre_leq_fc(self,value):
		if 0 <= value <= 1:
			self._reverb_pre_leq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_pre_heq_fc(self):
		'''(Reverb) Pre HEQ Fc'''
		return self._reverb_pre_heq_fc

	@reverb_pre_heq_fc.setter
	def reverb_pre_heq_fc(self,value):
		if 0 <= value <= 1:
			self._reverb_pre_heq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

