#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx127.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX127(object):
	category = '''Mono//Mono'''
	description = '''Parametric - 4-Band EQ // Parametric 4-Band EQ'''
	short_description = '''P4EQ // P4EQ'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX127 Parametric - 4-Band EQ // Parametric 4-Band EQ object'''
		bytes = struct.unpack('<31B',raw[:31])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_input_trim = (bytes[7] >> 1) & 0x7F
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_band1_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.effect1_band1_q = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.effect1_band1_gain = data
		self.effect1_band2_cutoff = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1F) << 4)
		self.effect1_band2_q = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.effect1_band2_gain = data
		self.effect1_band3_cutoff = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		self.effect1_band3_q = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x1) << 6)
		data = (bytes[16] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.effect1_band3_gain = data
		self.effect1_band4_cutoff = bytes[17] & 0xFF
		self.effect1_band4_q = bytes[18] & 0x7F
		data = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.effect1_band4_gain = data
		self.effect2_band1_cutoff = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0x1F) << 2)
		self.effect2_band1_q = ((bytes[20] & 0xE0) >> 5) | ((bytes[21] & 0xF) << 3)
		data = ((bytes[21] & 0xF0) >> 4) | ((bytes[22] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.effect2_band1_gain = data
		self.effect2_band2_cutoff = ((bytes[22] & 0xF8) >> 3) | ((bytes[23] & 0xF) << 5)
		self.effect2_band2_q = ((bytes[23] & 0xF0) >> 4) | ((bytes[24] & 0x7) << 4)
		data = ((bytes[24] & 0xF8) >> 3) | ((bytes[25] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.effect2_band2_gain = data
		self.effect2_band3_cutoff = ((bytes[25] & 0xFC) >> 2) | ((bytes[26] & 0x1) << 6)
		self.effect2_band3_q = (bytes[26] >> 1) & 0x7F
		data = bytes[27] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.effect2_band3_gain = data
		self.effect2_band4_cutoff = ((bytes[27] & 0x80) >> 7) | ((bytes[28] & 0x7F) << 1)
		self.effect2_band4_q = ((bytes[28] & 0x80) >> 7) | ((bytes[29] & 0x3F) << 1)
		data = ((bytes[29] & 0xC0) >> 6) | ((bytes[30] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.effect2_band4_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX127 Parametric - 4-Band EQ // Parametric 4-Band EQ object into raw data'''
		bytes = [0] * 31
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.effect1_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.effect1_input_trim << 6) & 0xC0)
		bytes[3] = ((self.effect1_input_trim >> 2) & 0x1F) | ((self.effect1_pan << 5) & 0xE0)
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = ((self.effect2_wet_dry_mod_intensity >> 7) & 0x1) | ((self.effect2_input_trim & 0x7F) << 1)
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_band1_cutoff << 7) & 0x80)
		bytes[9] = ((self.effect1_band1_cutoff >> 1) & 0x3F) | ((self.effect1_band1_q << 6) & 0xC0)
		bytes[10] = ((self.effect1_band1_q >> 2) & 0x1F) | ((self.effect1_band1_gain << 5) & 0xE0)
		bytes[11] = ((self.effect1_band1_gain >> 3) & 0xF) | ((self.effect1_band2_cutoff << 4) & 0xF0)
		bytes[12] = ((self.effect1_band2_cutoff >> 4) & 0x1F) | ((self.effect1_band2_q << 5) & 0xE0)
		bytes[13] = ((self.effect1_band2_q >> 3) & 0xF) | ((self.effect1_band2_gain << 4) & 0xF0)
		bytes[14] = ((self.effect1_band2_gain >> 4) & 0x7) | ((self.effect1_band3_cutoff << 3) & 0xF8)
		bytes[15] = ((self.effect1_band3_cutoff >> 5) & 0x3) | ((self.effect1_band3_q << 2) & 0xFC)
		bytes[16] = ((self.effect1_band3_q >> 6) & 0x1) | ((self.effect1_band3_gain & 0x7F) << 1)
		bytes[17] = self.effect1_band4_cutoff & 0xFF
		bytes[18] = (self.effect1_band4_q & 0x7F) | ((self.effect1_band4_gain << 7) & 0x80)
		bytes[19] = ((self.effect1_band4_gain >> 1) & 0x3F) | ((self.effect2_band1_cutoff << 6) & 0xC0)
		bytes[20] = ((self.effect2_band1_cutoff >> 2) & 0x1F) | ((self.effect2_band1_q << 5) & 0xE0)
		bytes[21] = ((self.effect2_band1_q >> 3) & 0xF) | ((self.effect2_band1_gain << 4) & 0xF0)
		bytes[22] = ((self.effect2_band1_gain >> 4) & 0x7) | ((self.effect2_band2_cutoff << 3) & 0xF8)
		bytes[23] = ((self.effect2_band2_cutoff >> 5) & 0xF) | ((self.effect2_band2_q << 4) & 0xF0)
		bytes[24] = ((self.effect2_band2_q >> 4) & 0x7) | ((self.effect2_band2_gain << 3) & 0xF8)
		bytes[25] = ((self.effect2_band2_gain >> 5) & 0x3) | ((self.effect2_band3_cutoff << 2) & 0xFC)
		bytes[26] = ((self.effect2_band3_cutoff >> 6) & 0x1) | ((self.effect2_band3_q & 0x7F) << 1)
		bytes[27] = (self.effect2_band3_gain & 0x7F) | ((self.effect2_band4_cutoff << 7) & 0x80)
		bytes[28] = ((self.effect2_band4_cutoff >> 1) & 0x7F) | ((self.effect2_band4_q << 7) & 0x80)
		bytes[29] = ((self.effect2_band4_q >> 1) & 0x3F) | ((self.effect2_band4_gain << 6) & 0xC0)
		bytes[30] = (self.effect2_band4_gain >> 2) & 0x1F
		return struct.pack('<31B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX127 Parametric - 4-Band EQ // Parametric 4-Band EQ object'''
		json = []
		json.append(('name','FX127'))
		json.append(('description','Parametric - 4-Band EQ // Parametric 4-Band EQ'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_input_trim',self.effect1_input_trim])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_input_trim',self.effect2_input_trim])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_band1_cutoff',self.effect1_band1_cutoff])
		json.append(['effect1_band1_cutoff_display',self.effect1_band1_cutoff_display])
		json.append(['effect1_band1_q',self.effect1_band1_q])
		json.append(['effect1_band1_gain',self.effect1_band1_gain])
		json.append(['effect1_band1_gain_display',self.effect1_band1_gain_display])
		json.append(['effect1_band2_cutoff',self.effect1_band2_cutoff])
		json.append(['effect1_band2_q',self.effect1_band2_q])
		json.append(['effect1_band2_gain',self.effect1_band2_gain])
		json.append(['effect1_band2_gain_display',self.effect1_band2_gain_display])
		json.append(['effect1_band3_cutoff',self.effect1_band3_cutoff])
		json.append(['effect1_band3_q',self.effect1_band3_q])
		json.append(['effect1_band3_gain',self.effect1_band3_gain])
		json.append(['effect1_band3_gain_display',self.effect1_band3_gain_display])
		json.append(['effect1_band4_q',self.effect1_band4_q])
		json.append(['effect1_band4_gain',self.effect1_band4_gain])
		json.append(['effect1_band4_gain_display',self.effect1_band4_gain_display])
		json.append(['effect2_band1_cutoff',self.effect2_band1_cutoff])
		json.append(['effect2_band1_cutoff_display',self.effect2_band1_cutoff_display])
		json.append(['effect2_band1_q',self.effect2_band1_q])
		json.append(['effect2_band1_gain',self.effect2_band1_gain])
		json.append(['effect2_band1_gain_display',self.effect2_band1_gain_display])
		json.append(['effect2_band2_cutoff',self.effect2_band2_cutoff])
		json.append(['effect2_band2_q',self.effect2_band2_q])
		json.append(['effect2_band2_gain',self.effect2_band2_gain])
		json.append(['effect2_band2_gain_display',self.effect2_band2_gain_display])
		json.append(['effect2_band3_cutoff',self.effect2_band3_cutoff])
		json.append(['effect1_band4_cutoff',self.effect1_band4_cutoff])
		json.append(['effect2_band3_q',self.effect2_band3_q])
		json.append(['effect2_band3_gain',self.effect2_band3_gain])
		json.append(['effect2_band3_gain_display',self.effect2_band3_gain_display])
		json.append(['effect2_band4_cutoff',self.effect2_band4_cutoff])
		json.append(['effect2_band4_q',self.effect2_band4_q])
		json.append(['effect2_band4_gain',self.effect2_band4_gain])
		json.append(['effect2_band4_gain_display',self.effect2_band4_gain_display])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_input_trim(self):
		'''(Effect1) Input Trim'''
		return self._effect1_input_trim

	@effect1_input_trim.setter
	def effect1_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect1_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_input_trim(self):
		'''(Effect2) Input Trim'''
		return self._effect2_input_trim

	@effect2_input_trim.setter
	def effect2_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect2_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band1_cutoff(self):
		'''(Effect1) Band1 Cutoff'''
		return self._effect1_band1_cutoff

	@effect1_band1_cutoff.setter
	def effect1_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._effect1_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def effect1_band1_cutoff_display(self):
		'''(Effect1) Band1 Cutoff display value'''
		dv = 20 + self._effect1_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect1_band1_q(self):
		'''(Effect1) Band1 Q'''
		return self._effect1_band1_q

	@effect1_band1_q.setter
	def effect1_band1_q(self,value):
		if 0 <= value <= 95:
			self._effect1_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band1_gain(self):
		'''(Effect1) Band1 Gain'''
		return self._effect1_band1_gain

	@effect1_band1_gain.setter
	def effect1_band1_gain(self,value):
		if -36 <= value <= 36:
			self._effect1_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect1_band1_gain_display(self):
		'''(Effect1) Band1 Gain display value'''
		dv = self._effect1_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect1_band2_cutoff(self):
		'''(Effect1) Band2 Cutoff'''
		return self._effect1_band2_cutoff

	@effect1_band2_cutoff.setter
	def effect1_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._effect1_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band2_q(self):
		'''(Effect1) Band2 Q'''
		return self._effect1_band2_q

	@effect1_band2_q.setter
	def effect1_band2_q(self,value):
		if 0 <= value <= 95:
			self._effect1_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band2_gain(self):
		'''(Effect1) Band2 Gain'''
		return self._effect1_band2_gain

	@effect1_band2_gain.setter
	def effect1_band2_gain(self,value):
		if -36 <= value <= 36:
			self._effect1_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect1_band2_gain_display(self):
		'''(Effect1) Band2 Gain display value'''
		dv = self._effect1_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect1_band3_cutoff(self):
		'''(Effect1) Band3 Cutoff'''
		return self._effect1_band3_cutoff

	@effect1_band3_cutoff.setter
	def effect1_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._effect1_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band3_q(self):
		'''(Effect1) Band3 Q'''
		return self._effect1_band3_q

	@effect1_band3_q.setter
	def effect1_band3_q(self,value):
		if 0 <= value <= 95:
			self._effect1_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band3_gain(self):
		'''(Effect1) Band3 Gain'''
		return self._effect1_band3_gain

	@effect1_band3_gain.setter
	def effect1_band3_gain(self,value):
		if -36 <= value <= 36:
			self._effect1_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect1_band3_gain_display(self):
		'''(Effect1) Band3 Gain display value'''
		dv = self._effect1_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect1_band4_cutoff(self):
		'''(Effect1) Band4 Cutoff'''
		return self._effect1_band4_cutoff

	@effect1_band4_cutoff.setter
	def effect1_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._effect1_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band4_q(self):
		'''(Effect1) Band4 Q'''
		return self._effect1_band4_q

	@effect1_band4_q.setter
	def effect1_band4_q(self,value):
		if 0 <= value <= 95:
			self._effect1_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_band4_gain(self):
		'''(Effect1) Band4 Gain'''
		return self._effect1_band4_gain

	@effect1_band4_gain.setter
	def effect1_band4_gain(self,value):
		if -36 <= value <= 36:
			self._effect1_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect1_band4_gain_display(self):
		'''(Effect1) Band4 Gain display value'''
		dv = self._effect1_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect2_band1_cutoff(self):
		'''(Effect2) Band1 Cutoff'''
		return self._effect2_band1_cutoff

	@effect2_band1_cutoff.setter
	def effect2_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._effect2_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def effect2_band1_cutoff_display(self):
		'''(Effect2) Band1 Cutoff display value'''
		dv = 20 + self._effect2_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect2_band1_q(self):
		'''(Effect2) Band1 Q'''
		return self._effect2_band1_q

	@effect2_band1_q.setter
	def effect2_band1_q(self,value):
		if 0 <= value <= 95:
			self._effect2_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band1_gain(self):
		'''(Effect2) Band1 Gain'''
		return self._effect2_band1_gain

	@effect2_band1_gain.setter
	def effect2_band1_gain(self,value):
		if -36 <= value <= 36:
			self._effect2_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect2_band1_gain_display(self):
		'''(Effect2) Band1 Gain display value'''
		dv = self._effect2_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect2_band2_cutoff(self):
		'''(Effect2) Band2 Cutoff'''
		return self._effect2_band2_cutoff

	@effect2_band2_cutoff.setter
	def effect2_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._effect2_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band2_q(self):
		'''(Effect2) Band2 Q'''
		return self._effect2_band2_q

	@effect2_band2_q.setter
	def effect2_band2_q(self,value):
		if 0 <= value <= 95:
			self._effect2_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band2_gain(self):
		'''(Effect2) Band2 Gain'''
		return self._effect2_band2_gain

	@effect2_band2_gain.setter
	def effect2_band2_gain(self,value):
		if -36 <= value <= 36:
			self._effect2_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect2_band2_gain_display(self):
		'''(Effect2) Band2 Gain display value'''
		dv = self._effect2_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect2_band3_cutoff(self):
		'''(Effect2) Band3 Cutoff'''
		return self._effect2_band3_cutoff

	@effect2_band3_cutoff.setter
	def effect2_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._effect2_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band3_q(self):
		'''(Effect2) Band3 Q'''
		return self._effect2_band3_q

	@effect2_band3_q.setter
	def effect2_band3_q(self,value):
		if 0 <= value <= 95:
			self._effect2_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band3_gain(self):
		'''(Effect2) Band3 Gain'''
		return self._effect2_band3_gain

	@effect2_band3_gain.setter
	def effect2_band3_gain(self,value):
		if -36 <= value <= 36:
			self._effect2_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect2_band3_gain_display(self):
		'''(Effect2) Band3 Gain display value'''
		dv = self._effect2_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def effect2_band4_cutoff(self):
		'''(Effect2) Band4 Cutoff'''
		return self._effect2_band4_cutoff

	@effect2_band4_cutoff.setter
	def effect2_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._effect2_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band4_q(self):
		'''(Effect2) Band4 Q'''
		return self._effect2_band4_q

	@effect2_band4_q.setter
	def effect2_band4_q(self,value):
		if 0 <= value <= 95:
			self._effect2_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_band4_gain(self):
		'''(Effect2) Band4 Gain'''
		return self._effect2_band4_gain

	@effect2_band4_gain.setter
	def effect2_band4_gain(self,value):
		if -36 <= value <= 36:
			self._effect2_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def effect2_band4_gain_display(self):
		'''(Effect2) Band4 Gain display value'''
		dv = self._effect2_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

