#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx131.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX131(object):
	category = '''Mono//Mono'''
	description = '''Parametric 4-Band EQ // Overdrive/Hi.Gain'''
	short_description = '''P4EQ // OD/HG'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX131 Parametric 4-Band EQ // Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<29B',raw[:29])
		self.p4eq_wet_dry = (bytes[0] >> 1) & 0x7F
		self.p4eq_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.p4eq_wet_dry_mod_intensity = data
		self.p4eq_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.od_hi_gain_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.od_hi_gain_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.od_hi_gain_pan = bytes[8] & 0x7F
		self.p4eq_band1_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.p4eq_band1_q = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1F) << 4)
		self.p4eq_band2_q = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		self.p4eq_band3_q = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x1) << 6)
		data = (bytes[16] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = bytes[17] & 0xFF
		self.p4eq_band4_q = bytes[18] & 0x7F
		data = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band4_gain = data
		self.od_hi_gain_hi_gain_on = (bytes[19] >> 6) & 0x1
		self.od_hi_gain_drive = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0x3F) << 1)
		self.od_hi_gain_output_level = ((bytes[20] & 0xC0) >> 6) | ((bytes[21] & 0xF) << 2)
		self.od_hi_gain_output_level_mod_source = ((bytes[21] & 0xF0) >> 4) | ((bytes[22] & 0x3) << 4)
		data = ((bytes[22] & 0xFC) >> 2) | ((bytes[23] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = (bytes[23] >> 1) & 0x3F
		data = ((bytes[23] & 0x80) >> 7) | ((bytes[24] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[24] & 0xE0) >> 5) | ((bytes[25] & 0x7) << 3)
		self.od_hi_gain_eq_mid1_q = (bytes[25] >> 3) & 0xF
		data = ((bytes[25] & 0x80) >> 7) | ((bytes[26] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[26] & 0xE0) >> 5) | ((bytes[27] & 0x7) << 3)
		self.od_hi_gain_eq_mid2_q = (bytes[27] >> 3) & 0xF
		data = ((bytes[27] & 0x80) >> 7) | ((bytes[28] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX131 Parametric 4-Band EQ // Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 29
		bytes[0] = (self.p4eq_wet_dry & 0x7F) << 1
		bytes[1] = (self.p4eq_wet_dry_mod_source & 0x3F) | ((self.p4eq_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.p4eq_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.p4eq_input_trim << 6) & 0xC0)
		bytes[3] = ((self.p4eq_input_trim >> 2) & 0x1F) | ((self.p4eq_pan << 5) & 0xE0)
		bytes[4] = ((self.p4eq_pan >> 3) & 0xF) | ((self.od_hi_gain_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.od_hi_gain_wet_dry >> 4) & 0x7) | ((self.od_hi_gain_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.od_hi_gain_wet_dry_mod_source >> 5) & 0x1) | ((self.od_hi_gain_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.od_hi_gain_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.od_hi_gain_pan & 0x7F) | ((self.p4eq_band1_cutoff << 7) & 0x80)
		bytes[9] = ((self.p4eq_band1_cutoff >> 1) & 0x3F) | ((self.p4eq_band1_q << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band1_q >> 2) & 0x1F) | ((self.p4eq_band1_gain << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band1_gain >> 3) & 0xF) | ((self.p4eq_band2_cutoff << 4) & 0xF0)
		bytes[12] = ((self.p4eq_band2_cutoff >> 4) & 0x1F) | ((self.p4eq_band2_q << 5) & 0xE0)
		bytes[13] = ((self.p4eq_band2_q >> 3) & 0xF) | ((self.p4eq_band2_gain << 4) & 0xF0)
		bytes[14] = ((self.p4eq_band2_gain >> 4) & 0x7) | ((self.p4eq_band3_cutoff << 3) & 0xF8)
		bytes[15] = ((self.p4eq_band3_cutoff >> 5) & 0x3) | ((self.p4eq_band3_q << 2) & 0xFC)
		bytes[16] = ((self.p4eq_band3_q >> 6) & 0x1) | ((self.p4eq_band3_gain & 0x7F) << 1)
		bytes[17] = self.p4eq_band4_cutoff & 0xFF
		bytes[18] = (self.p4eq_band4_q & 0x7F) | ((self.p4eq_band4_gain << 7) & 0x80)
		bytes[19] = ((self.p4eq_band4_gain >> 1) & 0x3F) | ((self.od_hi_gain_hi_gain_on & 0x1) << 6) | ((self.od_hi_gain_drive << 7) & 0x80)
		bytes[20] = ((self.od_hi_gain_drive >> 1) & 0x3F) | ((self.od_hi_gain_output_level << 6) & 0xC0)
		bytes[21] = ((self.od_hi_gain_output_level >> 2) & 0xF) | ((self.od_hi_gain_output_level_mod_source << 4) & 0xF0)
		bytes[22] = ((self.od_hi_gain_output_level_mod_source >> 4) & 0x3) | ((self.od_hi_gain_output_level_mod_intensity << 2) & 0xFC)
		bytes[23] = ((self.od_hi_gain_output_level_mod_intensity >> 6) & 0x1) | ((self.od_hi_gain_eq_low_cutoff & 0x3F) << 1) | ((self.od_hi_gain_eq_low_gain << 7) & 0x80)
		bytes[24] = ((self.od_hi_gain_eq_low_gain >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid1_cutoff << 5) & 0xE0)
		bytes[25] = ((self.od_hi_gain_eq_mid1_cutoff >> 3) & 0x7) | ((self.od_hi_gain_eq_mid1_q & 0xF) << 3) | ((self.od_hi_gain_eq_mid1_gain << 7) & 0x80)
		bytes[26] = ((self.od_hi_gain_eq_mid1_gain >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid2_cutoff << 5) & 0xE0)
		bytes[27] = ((self.od_hi_gain_eq_mid2_cutoff >> 3) & 0x7) | ((self.od_hi_gain_eq_mid2_q & 0xF) << 3) | ((self.od_hi_gain_eq_mid2_gain << 7) & 0x80)
		bytes[28] = (self.od_hi_gain_eq_mid2_gain >> 1) & 0x1F
		return struct.pack('<29B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX131 Parametric 4-Band EQ // Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX131'))
		json.append(('description','Parametric 4-Band EQ // Overdrive/Hi.Gain'))
		json.append(['p4eq_wet_dry',self.p4eq_wet_dry])
		json.append(['p4eq_wet_dry_mod_source',self.p4eq_wet_dry_mod_source])
		json.append(['p4eq_wet_dry_mod_intensity',self.p4eq_wet_dry_mod_intensity])
		json.append(['p4eq_input_trim',self.p4eq_input_trim])
		json.append(['p4eq_pan',self.p4eq_pan])
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		json.append(['od_hi_gain_pan',self.od_hi_gain_pan])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band1_gain_display',self.p4eq_band1_gain_display])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band2_gain_display',self.p4eq_band2_gain_display])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band3_gain_display',self.p4eq_band3_gain_display])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['p4eq_band4_gain_display',self.p4eq_band4_gain_display])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry(self):
		'''(P4EQ) Wet/Dry'''
		return self._p4eq_wet_dry

	@p4eq_wet_dry.setter
	def p4eq_wet_dry(self,value):
		if 0 <= value <= 100:
			self._p4eq_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_source(self):
		'''(P4EQ) Wet/Dry Mod.Source'''
		return self._p4eq_wet_dry_mod_source

	@p4eq_wet_dry_mod_source.setter
	def p4eq_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._p4eq_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_intensity(self):
		'''(P4EQ) Wet/Dry Mod.Intensity'''
		return self._p4eq_wet_dry_mod_intensity

	@p4eq_wet_dry_mod_intensity.setter
	def p4eq_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._p4eq_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_input_trim(self):
		'''(P4EQ) Input Trim'''
		return self._p4eq_input_trim

	@p4eq_input_trim.setter
	def p4eq_input_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_pan(self):
		'''(P4EQ) Pan'''
		return self._p4eq_pan

	@p4eq_pan.setter
	def p4eq_pan(self,value):
		if 0 <= value <= 127:
			self._p4eq_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_pan(self):
		'''(OD/Hi-Gain) Pan'''
		return self._od_hi_gain_pan

	@od_hi_gain_pan.setter
	def od_hi_gain_pan(self,value):
		if 0 <= value <= 127:
			self._od_hi_gain_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band1_gain_display(self):
		'''(P4EQ) Band1 Gain display value'''
		dv = self._p4eq_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band2_gain_display(self):
		'''(P4EQ) Band2 Gain display value'''
		dv = self._p4eq_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band3_gain_display(self):
		'''(P4EQ) Band3 Gain display value'''
		dv = self._p4eq_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band4_gain_display(self):
		'''(P4EQ) Band4 Gain display value'''
		dv = self._p4eq_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

