#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx132.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX132(object):
	category = '''Mono//Mono'''
	description = '''Parametric 4-Band EQ // Wah'''
	short_description = '''P4EQ // Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX132 Parametric 4-Band EQ // Wah object'''
		bytes = struct.unpack('<25B',raw[:25])
		self.p4eq_wet_dry = (bytes[0] >> 1) & 0x7F
		self.p4eq_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.p4eq_wet_dry_mod_intensity = data
		self.p4eq_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.wah_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.wah_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.wah_pan = bytes[8] & 0x7F
		self.p4eq_band1_cutoff = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.p4eq_band1_q = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1F) << 4)
		self.p4eq_band2_q = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		self.p4eq_band3_q = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x1) << 6)
		data = (bytes[16] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = bytes[17] & 0xFF
		self.p4eq_band4_q = bytes[18] & 0x7F
		data = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band4_gain = data
		self.wah_frequency_bottom = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0x1F) << 2)
		self.wah_frequency_top = ((bytes[20] & 0xE0) >> 5) | ((bytes[21] & 0xF) << 3)
		self.wah_sweep_mode = (bytes[21] >> 4) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[21] & 0xC0) >> 6) | ((bytes[22] & 0xF) << 2)
		self.wah_lfo_frequency = ((bytes[22] & 0xF0) >> 4) | ((bytes[23] & 0xF) << 4)
		self.wah_resonance = ((bytes[23] & 0xF0) >> 4) | ((bytes[24] & 0x7) << 4)
		self.wah_low_pass_filter = (bytes[24] >> 3) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX132 Parametric 4-Band EQ // Wah object into raw data'''
		bytes = [0] * 25
		bytes[0] = (self.p4eq_wet_dry & 0x7F) << 1
		bytes[1] = (self.p4eq_wet_dry_mod_source & 0x3F) | ((self.p4eq_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.p4eq_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.p4eq_input_trim << 6) & 0xC0)
		bytes[3] = ((self.p4eq_input_trim >> 2) & 0x1F) | ((self.p4eq_pan << 5) & 0xE0)
		bytes[4] = ((self.p4eq_pan >> 3) & 0xF) | ((self.wah_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.wah_wet_dry >> 4) & 0x7) | ((self.wah_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.wah_wet_dry_mod_source >> 5) & 0x1) | ((self.wah_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.wah_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.wah_pan & 0x7F) | ((self.p4eq_band1_cutoff << 7) & 0x80)
		bytes[9] = ((self.p4eq_band1_cutoff >> 1) & 0x3F) | ((self.p4eq_band1_q << 6) & 0xC0)
		bytes[10] = ((self.p4eq_band1_q >> 2) & 0x1F) | ((self.p4eq_band1_gain << 5) & 0xE0)
		bytes[11] = ((self.p4eq_band1_gain >> 3) & 0xF) | ((self.p4eq_band2_cutoff << 4) & 0xF0)
		bytes[12] = ((self.p4eq_band2_cutoff >> 4) & 0x1F) | ((self.p4eq_band2_q << 5) & 0xE0)
		bytes[13] = ((self.p4eq_band2_q >> 3) & 0xF) | ((self.p4eq_band2_gain << 4) & 0xF0)
		bytes[14] = ((self.p4eq_band2_gain >> 4) & 0x7) | ((self.p4eq_band3_cutoff << 3) & 0xF8)
		bytes[15] = ((self.p4eq_band3_cutoff >> 5) & 0x3) | ((self.p4eq_band3_q << 2) & 0xFC)
		bytes[16] = ((self.p4eq_band3_q >> 6) & 0x1) | ((self.p4eq_band3_gain & 0x7F) << 1)
		bytes[17] = self.p4eq_band4_cutoff & 0xFF
		bytes[18] = (self.p4eq_band4_q & 0x7F) | ((self.p4eq_band4_gain << 7) & 0x80)
		bytes[19] = ((self.p4eq_band4_gain >> 1) & 0x3F) | ((self.wah_frequency_bottom << 6) & 0xC0)
		bytes[20] = ((self.wah_frequency_bottom >> 2) & 0x1F) | ((self.wah_frequency_top << 5) & 0xE0)
		bytes[21] = ((self.wah_frequency_top >> 3) & 0xF) | ((self.wah_sweep_mode & 0x3) << 4) | ((self.wah_sweep_mode_mod_source << 6) & 0xC0)
		bytes[22] = ((self.wah_sweep_mode_mod_source >> 2) & 0xF) | ((self.wah_lfo_frequency << 4) & 0xF0)
		bytes[23] = ((self.wah_lfo_frequency >> 4) & 0xF) | ((self.wah_resonance << 4) & 0xF0)
		bytes[24] = ((self.wah_resonance >> 4) & 0x7) | ((self.wah_low_pass_filter & 0x1) << 3)
		return struct.pack('<25B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX132 Parametric 4-Band EQ // Wah object'''
		json = []
		json.append(('name','FX132'))
		json.append(('description','Parametric 4-Band EQ // Wah'))
		json.append(['p4eq_wet_dry',self.p4eq_wet_dry])
		json.append(['p4eq_wet_dry_mod_source',self.p4eq_wet_dry_mod_source])
		json.append(['p4eq_wet_dry_mod_intensity',self.p4eq_wet_dry_mod_intensity])
		json.append(['p4eq_input_trim',self.p4eq_input_trim])
		json.append(['p4eq_pan',self.p4eq_pan])
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		json.append(['wah_pan',self.wah_pan])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band1_gain_display',self.p4eq_band1_gain_display])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band2_gain_display',self.p4eq_band2_gain_display])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band3_gain_display',self.p4eq_band3_gain_display])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['p4eq_band4_gain_display',self.p4eq_band4_gain_display])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry(self):
		'''(P4EQ) Wet/Dry'''
		return self._p4eq_wet_dry

	@p4eq_wet_dry.setter
	def p4eq_wet_dry(self,value):
		if 0 <= value <= 100:
			self._p4eq_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_source(self):
		'''(P4EQ) Wet/Dry Mod.Source'''
		return self._p4eq_wet_dry_mod_source

	@p4eq_wet_dry_mod_source.setter
	def p4eq_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._p4eq_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_intensity(self):
		'''(P4EQ) Wet/Dry Mod.Intensity'''
		return self._p4eq_wet_dry_mod_intensity

	@p4eq_wet_dry_mod_intensity.setter
	def p4eq_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._p4eq_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_input_trim(self):
		'''(P4EQ) Input Trim'''
		return self._p4eq_input_trim

	@p4eq_input_trim.setter
	def p4eq_input_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_pan(self):
		'''(P4EQ) Pan'''
		return self._p4eq_pan

	@p4eq_pan.setter
	def p4eq_pan(self,value):
		if 0 <= value <= 127:
			self._p4eq_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_pan(self):
		'''(Wah) Pan'''
		return self._wah_pan

	@wah_pan.setter
	def wah_pan(self,value):
		if 0 <= value <= 127:
			self._wah_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band1_gain_display(self):
		'''(P4EQ) Band1 Gain display value'''
		dv = self._p4eq_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band2_gain_display(self):
		'''(P4EQ) Band2 Gain display value'''
		dv = self._p4eq_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band3_gain_display(self):
		'''(P4EQ) Band3 Gain display value'''
		dv = self._p4eq_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band4_gain_display(self):
		'''(P4EQ) Band4 Gain display value'''
		dv = self._p4eq_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

