#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx133.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX133(object):
	category = '''Mono//Mono'''
	description = '''Parametric 4-Band EQ // Chorus/Flanger'''
	short_description = '''P4EQ // Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX133 Parametric 4-Band EQ // Chorus/Flanger object'''
		bytes = struct.unpack('<26B',raw[:26])
		self.p4eq_wet_dry = (bytes[0] >> 1) & 0x7F
		self.p4eq_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.p4eq_wet_dry_mod_intensity = data
		self.p4eq_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_input_trim = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		self.chorus_flanger_pan = (bytes[8] >> 1) & 0x7F
		self.p4eq_band1_cutoff = bytes[9] & 0x7F
		self.p4eq_band1_q = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x3F) << 3)
		self.p4eq_band2_q = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		data = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		self.p4eq_band3_q = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x3) << 5)
		data = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[17] & 0xFE) >> 1) | ((bytes[18] & 0x1) << 7)
		self.p4eq_band4_q = (bytes[18] >> 1) & 0x7F
		data = bytes[19] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.p4eq_band4_gain = data
		self.chorus_flanger_pre_delay = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0xFF) << 1)
		self.chorus_flanger_lfo_waveform = (bytes[21] >> 2) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[21] & 0xF8) >> 3) | ((bytes[22] & 0x7) << 5)
		self.chorus_flanger_depth = ((bytes[22] & 0xF8) >> 3) | ((bytes[23] & 0x3) << 5)
		data = ((bytes[23] & 0xFC) >> 2) | ((bytes[24] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = (bytes[24] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = ((bytes[24] & 0x80) >> 7) | ((bytes[25] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX133 Parametric 4-Band EQ // Chorus/Flanger object into raw data'''
		bytes = [0] * 26
		bytes[0] = (self.p4eq_wet_dry & 0x7F) << 1
		bytes[1] = (self.p4eq_wet_dry_mod_source & 0x3F) | ((self.p4eq_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.p4eq_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.p4eq_input_trim << 6) & 0xC0)
		bytes[3] = ((self.p4eq_input_trim >> 2) & 0x1F) | ((self.p4eq_pan << 5) & 0xE0)
		bytes[4] = ((self.p4eq_pan >> 3) & 0xF) | ((self.chorus_flanger_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.chorus_flanger_wet_dry >> 4) & 0xF) | ((self.chorus_flanger_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.chorus_flanger_wet_dry_mod_source >> 4) & 0x3) | ((self.chorus_flanger_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = ((self.chorus_flanger_wet_dry_mod_intensity >> 6) & 0x3) | ((self.chorus_flanger_input_trim << 2) & 0xFC)
		bytes[8] = ((self.chorus_flanger_input_trim >> 6) & 0x1) | ((self.chorus_flanger_pan & 0x7F) << 1)
		bytes[9] = (self.p4eq_band1_cutoff & 0x7F) | ((self.p4eq_band1_q << 7) & 0x80)
		bytes[10] = ((self.p4eq_band1_q >> 1) & 0x3F) | ((self.p4eq_band1_gain << 6) & 0xC0)
		bytes[11] = ((self.p4eq_band1_gain >> 2) & 0x1F) | ((self.p4eq_band2_cutoff << 5) & 0xE0)
		bytes[12] = ((self.p4eq_band2_cutoff >> 3) & 0x3F) | ((self.p4eq_band2_q << 6) & 0xC0)
		bytes[13] = ((self.p4eq_band2_q >> 2) & 0x1F) | ((self.p4eq_band2_gain << 5) & 0xE0)
		bytes[14] = ((self.p4eq_band2_gain >> 3) & 0xF) | ((self.p4eq_band3_cutoff << 4) & 0xF0)
		bytes[15] = ((self.p4eq_band3_cutoff >> 4) & 0x7) | ((self.p4eq_band3_q << 3) & 0xF8)
		bytes[16] = ((self.p4eq_band3_q >> 5) & 0x3) | ((self.p4eq_band3_gain << 2) & 0xFC)
		bytes[17] = ((self.p4eq_band3_gain >> 6) & 0x1) | ((self.p4eq_band4_cutoff << 1) & 0xFE)
		bytes[18] = ((self.p4eq_band4_cutoff >> 7) & 0x1) | ((self.p4eq_band4_q & 0x7F) << 1)
		bytes[19] = (self.p4eq_band4_gain & 0x7F) | ((self.chorus_flanger_pre_delay << 7) & 0x80)
		bytes[20] = (self.chorus_flanger_pre_delay >> 1) & 0xFF
		bytes[21] = ((self.chorus_flanger_lfo_waveform & 0x1) << 2) | ((self.chorus_flanger_lfo_frequency << 3) & 0xF8)
		bytes[22] = ((self.chorus_flanger_lfo_frequency >> 5) & 0x7) | ((self.chorus_flanger_depth << 3) & 0xF8)
		bytes[23] = ((self.chorus_flanger_depth >> 5) & 0x3) | ((self.chorus_flanger_feedback << 2) & 0xFC)
		bytes[24] = ((self.chorus_flanger_feedback >> 6) & 0x3) | ((self.chorus_flanger_pre_leq_gain & 0x1F) << 2) | ((self.chorus_flanger_pre_heq_gain << 7) & 0x80)
		bytes[25] = (self.chorus_flanger_pre_heq_gain >> 1) & 0xF
		return struct.pack('<26B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX133 Parametric 4-Band EQ // Chorus/Flanger object'''
		json = []
		json.append(('name','FX133'))
		json.append(('description','Parametric 4-Band EQ // Chorus/Flanger'))
		json.append(['p4eq_wet_dry',self.p4eq_wet_dry])
		json.append(['p4eq_wet_dry_mod_source',self.p4eq_wet_dry_mod_source])
		json.append(['p4eq_wet_dry_mod_intensity',self.p4eq_wet_dry_mod_intensity])
		json.append(['p4eq_input_trim',self.p4eq_input_trim])
		json.append(['p4eq_pan',self.p4eq_pan])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		json.append(['chorus_flanger_input_trim',self.chorus_flanger_input_trim])
		json.append(['chorus_flanger_pan',self.chorus_flanger_pan])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band1_gain_display',self.p4eq_band1_gain_display])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band2_gain_display',self.p4eq_band2_gain_display])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band3_gain_display',self.p4eq_band3_gain_display])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['p4eq_band4_gain_display',self.p4eq_band4_gain_display])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry(self):
		'''(P4EQ) Wet/Dry'''
		return self._p4eq_wet_dry

	@p4eq_wet_dry.setter
	def p4eq_wet_dry(self,value):
		if 0 <= value <= 100:
			self._p4eq_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_source(self):
		'''(P4EQ) Wet/Dry Mod.Source'''
		return self._p4eq_wet_dry_mod_source

	@p4eq_wet_dry_mod_source.setter
	def p4eq_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._p4eq_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_intensity(self):
		'''(P4EQ) Wet/Dry Mod.Intensity'''
		return self._p4eq_wet_dry_mod_intensity

	@p4eq_wet_dry_mod_intensity.setter
	def p4eq_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._p4eq_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_input_trim(self):
		'''(P4EQ) Input Trim'''
		return self._p4eq_input_trim

	@p4eq_input_trim.setter
	def p4eq_input_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_pan(self):
		'''(P4EQ) Pan'''
		return self._p4eq_pan

	@p4eq_pan.setter
	def p4eq_pan(self,value):
		if 0 <= value <= 127:
			self._p4eq_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_input_trim(self):
		'''(Chorus/Flanger) Input Trim'''
		return self._chorus_flanger_input_trim

	@chorus_flanger_input_trim.setter
	def chorus_flanger_input_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pan(self):
		'''(Chorus/Flanger) Pan'''
		return self._chorus_flanger_pan

	@chorus_flanger_pan.setter
	def chorus_flanger_pan(self,value):
		if 0 <= value <= 127:
			self._chorus_flanger_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band1_gain_display(self):
		'''(P4EQ) Band1 Gain display value'''
		dv = self._p4eq_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band2_gain_display(self):
		'''(P4EQ) Band2 Gain display value'''
		dv = self._p4eq_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band3_gain_display(self):
		'''(P4EQ) Band3 Gain display value'''
		dv = self._p4eq_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band4_gain_display(self):
		'''(P4EQ) Band4 Gain display value'''
		dv = self._p4eq_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

