#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx134.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX134(object):
	category = '''Mono//Mono'''
	description = '''Parametric 4-Band EQ // Phaser'''
	short_description = '''P4EQ // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX134 Parametric 4-Band EQ // Phaser object'''
		bytes = struct.unpack('<24B',raw[:24])
		self.p4eq_wet_dry = (bytes[0] >> 1) & 0x7F
		self.p4eq_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.p4eq_wet_dry_mod_intensity = data
		self.p4eq_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.p4eq_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = (bytes[8] >> 1) & 0x7F
		self.p4eq_band1_cutoff = bytes[9] & 0x7F
		self.p4eq_band1_q = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band1_gain = data
		self.p4eq_band2_cutoff = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x3F) << 3)
		self.p4eq_band2_q = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		data = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band2_gain = data
		self.p4eq_band3_cutoff = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		self.p4eq_band3_q = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x3) << 5)
		data = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.p4eq_band3_gain = data
		self.p4eq_band4_cutoff = ((bytes[17] & 0xFE) >> 1) | ((bytes[18] & 0x1) << 7)
		self.p4eq_band4_q = (bytes[18] >> 1) & 0x7F
		data = bytes[19] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.p4eq_band4_gain = data
		self.phaser_lfo_waveform = (bytes[19] >> 7) & 0x1
		self.phaser_lfo_frequency = bytes[20] & 0xFF
		self.phaser_manual = bytes[21] & 0x7F
		self.phaser_depth = ((bytes[21] & 0x80) >> 7) | ((bytes[22] & 0x3F) << 1)
		data = ((bytes[22] & 0xC0) >> 6) | ((bytes[23] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX134 Parametric 4-Band EQ // Phaser object into raw data'''
		bytes = [0] * 24
		bytes[0] = (self.p4eq_wet_dry & 0x7F) << 1
		bytes[1] = (self.p4eq_wet_dry_mod_source & 0x3F) | ((self.p4eq_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.p4eq_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.p4eq_input_trim << 6) & 0xC0)
		bytes[3] = ((self.p4eq_input_trim >> 2) & 0x1F) | ((self.p4eq_pan << 5) & 0xE0)
		bytes[4] = ((self.p4eq_pan >> 3) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.phaser_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.phaser_wet_dry_mod_source >> 4) & 0x3) | ((self.phaser_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = (self.phaser_wet_dry_mod_intensity >> 6) & 0x3
		bytes[8] = (self.phaser_pan & 0x7F) << 1
		bytes[9] = (self.p4eq_band1_cutoff & 0x7F) | ((self.p4eq_band1_q << 7) & 0x80)
		bytes[10] = ((self.p4eq_band1_q >> 1) & 0x3F) | ((self.p4eq_band1_gain << 6) & 0xC0)
		bytes[11] = ((self.p4eq_band1_gain >> 2) & 0x1F) | ((self.p4eq_band2_cutoff << 5) & 0xE0)
		bytes[12] = ((self.p4eq_band2_cutoff >> 3) & 0x3F) | ((self.p4eq_band2_q << 6) & 0xC0)
		bytes[13] = ((self.p4eq_band2_q >> 2) & 0x1F) | ((self.p4eq_band2_gain << 5) & 0xE0)
		bytes[14] = ((self.p4eq_band2_gain >> 3) & 0xF) | ((self.p4eq_band3_cutoff << 4) & 0xF0)
		bytes[15] = ((self.p4eq_band3_cutoff >> 4) & 0x7) | ((self.p4eq_band3_q << 3) & 0xF8)
		bytes[16] = ((self.p4eq_band3_q >> 5) & 0x3) | ((self.p4eq_band3_gain << 2) & 0xFC)
		bytes[17] = ((self.p4eq_band3_gain >> 6) & 0x1) | ((self.p4eq_band4_cutoff << 1) & 0xFE)
		bytes[18] = ((self.p4eq_band4_cutoff >> 7) & 0x1) | ((self.p4eq_band4_q & 0x7F) << 1)
		bytes[19] = (self.p4eq_band4_gain & 0x7F) | ((self.phaser_lfo_waveform & 0x1) << 7)
		bytes[20] = self.phaser_lfo_frequency & 0xFF
		bytes[21] = (self.phaser_manual & 0x7F) | ((self.phaser_depth << 7) & 0x80)
		bytes[22] = ((self.phaser_depth >> 1) & 0x3F) | ((self.phaser_resonance << 6) & 0xC0)
		bytes[23] = (self.phaser_resonance >> 2) & 0x3F
		return struct.pack('<24B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX134 Parametric 4-Band EQ // Phaser object'''
		json = []
		json.append(('name','FX134'))
		json.append(('description','Parametric 4-Band EQ // Phaser'))
		json.append(['p4eq_wet_dry',self.p4eq_wet_dry])
		json.append(['p4eq_wet_dry_mod_source',self.p4eq_wet_dry_mod_source])
		json.append(['p4eq_wet_dry_mod_intensity',self.p4eq_wet_dry_mod_intensity])
		json.append(['p4eq_input_trim',self.p4eq_input_trim])
		json.append(['p4eq_pan',self.p4eq_pan])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['p4eq_band1_cutoff',self.p4eq_band1_cutoff])
		json.append(['p4eq_band1_cutoff_display',self.p4eq_band1_cutoff_display])
		json.append(['p4eq_band1_q',self.p4eq_band1_q])
		json.append(['p4eq_band1_gain',self.p4eq_band1_gain])
		json.append(['p4eq_band1_gain_display',self.p4eq_band1_gain_display])
		json.append(['p4eq_band2_cutoff',self.p4eq_band2_cutoff])
		json.append(['p4eq_band2_q',self.p4eq_band2_q])
		json.append(['p4eq_band2_gain',self.p4eq_band2_gain])
		json.append(['p4eq_band2_gain_display',self.p4eq_band2_gain_display])
		json.append(['p4eq_band3_cutoff',self.p4eq_band3_cutoff])
		json.append(['p4eq_band3_q',self.p4eq_band3_q])
		json.append(['p4eq_band3_gain',self.p4eq_band3_gain])
		json.append(['p4eq_band3_gain_display',self.p4eq_band3_gain_display])
		json.append(['p4eq_band4_cutoff',self.p4eq_band4_cutoff])
		json.append(['p4eq_band4_q',self.p4eq_band4_q])
		json.append(['p4eq_band4_gain',self.p4eq_band4_gain])
		json.append(['p4eq_band4_gain_display',self.p4eq_band4_gain_display])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry(self):
		'''(P4EQ) Wet/Dry'''
		return self._p4eq_wet_dry

	@p4eq_wet_dry.setter
	def p4eq_wet_dry(self,value):
		if 0 <= value <= 100:
			self._p4eq_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_source(self):
		'''(P4EQ) Wet/Dry Mod.Source'''
		return self._p4eq_wet_dry_mod_source

	@p4eq_wet_dry_mod_source.setter
	def p4eq_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._p4eq_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_wet_dry_mod_intensity(self):
		'''(P4EQ) Wet/Dry Mod.Intensity'''
		return self._p4eq_wet_dry_mod_intensity

	@p4eq_wet_dry_mod_intensity.setter
	def p4eq_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._p4eq_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_input_trim(self):
		'''(P4EQ) Input Trim'''
		return self._p4eq_input_trim

	@p4eq_input_trim.setter
	def p4eq_input_trim(self,value):
		if 0 <= value <= 100:
			self._p4eq_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_pan(self):
		'''(P4EQ) Pan'''
		return self._p4eq_pan

	@p4eq_pan.setter
	def p4eq_pan(self,value):
		if 0 <= value <= 127:
			self._p4eq_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_cutoff(self):
		'''(P4EQ) Band1 Cutoff'''
		return self._p4eq_band1_cutoff

	@p4eq_band1_cutoff.setter
	def p4eq_band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_band1_cutoff_display(self):
		'''(P4EQ) Band1 Cutoff display value'''
		dv = 20 + self._p4eq_band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_q(self):
		'''(P4EQ) Band1 Q'''
		return self._p4eq_band1_q

	@p4eq_band1_q.setter
	def p4eq_band1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band1_gain(self):
		'''(P4EQ) Band1 Gain'''
		return self._p4eq_band1_gain

	@p4eq_band1_gain.setter
	def p4eq_band1_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band1_gain_display(self):
		'''(P4EQ) Band1 Gain display value'''
		dv = self._p4eq_band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_cutoff(self):
		'''(P4EQ) Band2 Cutoff'''
		return self._p4eq_band2_cutoff

	@p4eq_band2_cutoff.setter
	def p4eq_band2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_q(self):
		'''(P4EQ) Band2 Q'''
		return self._p4eq_band2_q

	@p4eq_band2_q.setter
	def p4eq_band2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band2_gain(self):
		'''(P4EQ) Band2 Gain'''
		return self._p4eq_band2_gain

	@p4eq_band2_gain.setter
	def p4eq_band2_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band2_gain_display(self):
		'''(P4EQ) Band2 Gain display value'''
		dv = self._p4eq_band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_cutoff(self):
		'''(P4EQ) Band3 Cutoff'''
		return self._p4eq_band3_cutoff

	@p4eq_band3_cutoff.setter
	def p4eq_band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_q(self):
		'''(P4EQ) Band3 Q'''
		return self._p4eq_band3_q

	@p4eq_band3_q.setter
	def p4eq_band3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band3_gain(self):
		'''(P4EQ) Band3 Gain'''
		return self._p4eq_band3_gain

	@p4eq_band3_gain.setter
	def p4eq_band3_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band3_gain_display(self):
		'''(P4EQ) Band3 Gain display value'''
		dv = self._p4eq_band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_cutoff(self):
		'''(P4EQ) Band4 Cutoff'''
		return self._p4eq_band4_cutoff

	@p4eq_band4_cutoff.setter
	def p4eq_band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_q(self):
		'''(P4EQ) Band4 Q'''
		return self._p4eq_band4_q

	@p4eq_band4_q.setter
	def p4eq_band4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_band4_gain(self):
		'''(P4EQ) Band4 Gain'''
		return self._p4eq_band4_gain

	@p4eq_band4_gain.setter
	def p4eq_band4_gain(self,value):
		if -36 <= value <= 36:
			self._p4eq_band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def p4eq_band4_gain_display(self):
		'''(P4EQ) Band4 Gain display value'''
		dv = self._p4eq_band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

