#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx136.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX136(object):
	category = '''Mono//Mono'''
	description = '''Compressor // Compressor'''
	short_description = '''Comp // Comp'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX136 Compressor // Compressor object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_input_trim = (bytes[7] >> 1) & 0x7F
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_sensitivity = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.effect1_attack = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.effect1_output_level = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.effect1_pre_leq_gain = data
		data = (bytes[12] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.effect1_pre_heq_gain = data
		self.effect2_sensitivity = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		self.effect2_attack = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		self.effect2_output_level = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		data = (bytes[15] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.effect2_pre_leq_gain = data
		data = bytes[16] & 0x1F
		if data & 0x10:
			data -= 0x20
		self.effect2_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX136 Compressor // Compressor object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.effect1_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.effect1_input_trim << 6) & 0xC0)
		bytes[3] = ((self.effect1_input_trim >> 2) & 0x1F) | ((self.effect1_pan << 5) & 0xE0)
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = ((self.effect2_wet_dry_mod_intensity >> 7) & 0x1) | ((self.effect2_input_trim & 0x7F) << 1)
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_sensitivity << 7) & 0x80)
		bytes[9] = ((self.effect1_sensitivity >> 1) & 0x3F) | ((self.effect1_attack << 6) & 0xC0)
		bytes[10] = ((self.effect1_attack >> 2) & 0x1F) | ((self.effect1_output_level << 5) & 0xE0)
		bytes[11] = ((self.effect1_output_level >> 3) & 0xF) | ((self.effect1_pre_leq_gain << 4) & 0xF0)
		bytes[12] = ((self.effect1_pre_leq_gain >> 4) & 0x1) | ((self.effect1_pre_heq_gain & 0x1F) << 1) | ((self.effect2_sensitivity << 6) & 0xC0)
		bytes[13] = ((self.effect2_sensitivity >> 2) & 0x1F) | ((self.effect2_attack << 5) & 0xE0)
		bytes[14] = ((self.effect2_attack >> 3) & 0xF) | ((self.effect2_output_level << 4) & 0xF0)
		bytes[15] = ((self.effect2_output_level >> 4) & 0x7) | ((self.effect2_pre_leq_gain & 0x1F) << 3)
		bytes[16] = self.effect2_pre_heq_gain & 0x1F
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX136 Compressor // Compressor object'''
		json = []
		json.append(('name','FX136'))
		json.append(('description','Compressor // Compressor'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_input_trim',self.effect1_input_trim])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_input_trim',self.effect2_input_trim])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_sensitivity',self.effect1_sensitivity])
		json.append(['effect1_attack',self.effect1_attack])
		json.append(['effect1_output_level',self.effect1_output_level])
		json.append(['effect1_pre_leq_gain',self.effect1_pre_leq_gain])
		json.append(['effect1_pre_heq_gain',self.effect1_pre_heq_gain])
		json.append(['effect2_sensitivity',self.effect2_sensitivity])
		json.append(['effect2_attack',self.effect2_attack])
		json.append(['effect2_output_level',self.effect2_output_level])
		json.append(['effect2_pre_leq_gain',self.effect2_pre_leq_gain])
		json.append(['effect2_pre_heq_gain',self.effect2_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_input_trim(self):
		'''(Effect1) Input Trim'''
		return self._effect1_input_trim

	@effect1_input_trim.setter
	def effect1_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect1_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_input_trim(self):
		'''(Effect2) Input Trim'''
		return self._effect2_input_trim

	@effect2_input_trim.setter
	def effect2_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect2_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_sensitivity(self):
		'''(Effect1) Sensitivity'''
		return self._effect1_sensitivity

	@effect1_sensitivity.setter
	def effect1_sensitivity(self,value):
		if 1 <= value <= 100:
			self._effect1_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_attack(self):
		'''(Effect1) Attack'''
		return self._effect1_attack

	@effect1_attack.setter
	def effect1_attack(self,value):
		if 1 <= value <= 100:
			self._effect1_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_output_level(self):
		'''(Effect1) Output Level'''
		return self._effect1_output_level

	@effect1_output_level.setter
	def effect1_output_level(self,value):
		if 0 <= value <= 100:
			self._effect1_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pre_leq_gain(self):
		'''(Effect1) Pre LEQ Gain'''
		return self._effect1_pre_leq_gain

	@effect1_pre_leq_gain.setter
	def effect1_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._effect1_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pre_heq_gain(self):
		'''(Effect1) Pre HEQ Gain'''
		return self._effect1_pre_heq_gain

	@effect1_pre_heq_gain.setter
	def effect1_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._effect1_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_sensitivity(self):
		'''(Effect2) Sensitivity'''
		return self._effect2_sensitivity

	@effect2_sensitivity.setter
	def effect2_sensitivity(self,value):
		if 1 <= value <= 100:
			self._effect2_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_attack(self):
		'''(Effect2) Attack'''
		return self._effect2_attack

	@effect2_attack.setter
	def effect2_attack(self,value):
		if 1 <= value <= 100:
			self._effect2_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_output_level(self):
		'''(Effect2) Output Level'''
		return self._effect2_output_level

	@effect2_output_level.setter
	def effect2_output_level(self,value):
		if 0 <= value <= 100:
			self._effect2_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pre_leq_gain(self):
		'''(Effect2) Pre LEQ Gain'''
		return self._effect2_pre_leq_gain

	@effect2_pre_leq_gain.setter
	def effect2_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._effect2_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pre_heq_gain(self):
		'''(Effect2) Pre HEQ Gain'''
		return self._effect2_pre_heq_gain

	@effect2_pre_heq_gain.setter
	def effect2_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._effect2_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

