#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx137.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX137(object):
	category = '''Mono//Mono'''
	description = '''Compressor // Limiter'''
	short_description = '''Comp // Limiter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX137 Compressor // Limiter object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.compressor_wet_dry = (bytes[0] >> 1) & 0x7F
		self.compressor_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.compressor_wet_dry_mod_intensity = data
		self.compressor_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.compressor_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.limiter_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.limiter_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.limiter_wet_dry_mod_intensity = data
		self.limiter_pan = bytes[8] & 0x7F
		self.compressor_sensitivity = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.compressor_attack = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.compressor_output_level = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = (bytes[12] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		self.limiter_ratio = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x3F) << 2)
		data = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0xF) << 3)
		self.limiter_release = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x7) << 4)
		data = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX137 Compressor // Limiter object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.compressor_wet_dry & 0x7F) << 1
		bytes[1] = (self.compressor_wet_dry_mod_source & 0x3F) | ((self.compressor_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.compressor_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.compressor_input_trim << 6) & 0xC0)
		bytes[3] = ((self.compressor_input_trim >> 2) & 0x1F) | ((self.compressor_pan << 5) & 0xE0)
		bytes[4] = ((self.compressor_pan >> 3) & 0xF) | ((self.limiter_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.limiter_wet_dry >> 4) & 0x7) | ((self.limiter_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.limiter_wet_dry_mod_source >> 5) & 0x1) | ((self.limiter_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.limiter_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.limiter_pan & 0x7F) | ((self.compressor_sensitivity << 7) & 0x80)
		bytes[9] = ((self.compressor_sensitivity >> 1) & 0x3F) | ((self.compressor_attack << 6) & 0xC0)
		bytes[10] = ((self.compressor_attack >> 2) & 0x1F) | ((self.compressor_output_level << 5) & 0xE0)
		bytes[11] = ((self.compressor_output_level >> 3) & 0xF) | ((self.compressor_pre_leq_gain << 4) & 0xF0)
		bytes[12] = ((self.compressor_pre_leq_gain >> 4) & 0x1) | ((self.compressor_pre_heq_gain & 0x1F) << 1) | ((self.limiter_ratio << 6) & 0xC0)
		bytes[13] = ((self.limiter_ratio >> 2) & 0x3F) | ((self.limiter_threshold << 6) & 0xC0)
		bytes[14] = ((self.limiter_threshold >> 2) & 0x1F) | ((self.limiter_attack << 5) & 0xE0)
		bytes[15] = ((self.limiter_attack >> 3) & 0xF) | ((self.limiter_release << 4) & 0xF0)
		bytes[16] = ((self.limiter_release >> 4) & 0x7) | ((self.limiter_gain_adjust << 3) & 0xF8)
		bytes[17] = (self.limiter_gain_adjust >> 5) & 0x3
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX137 Compressor // Limiter object'''
		json = []
		json.append(('name','FX137'))
		json.append(('description','Compressor // Limiter'))
		json.append(['compressor_wet_dry',self.compressor_wet_dry])
		json.append(['compressor_wet_dry_mod_source',self.compressor_wet_dry_mod_source])
		json.append(['compressor_wet_dry_mod_intensity',self.compressor_wet_dry_mod_intensity])
		json.append(['compressor_input_trim',self.compressor_input_trim])
		json.append(['compressor_pan',self.compressor_pan])
		json.append(['limiter_wet_dry',self.limiter_wet_dry])
		json.append(['limiter_wet_dry_mod_source',self.limiter_wet_dry_mod_source])
		json.append(['limiter_wet_dry_mod_intensity',self.limiter_wet_dry_mod_intensity])
		json.append(['limiter_pan',self.limiter_pan])
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry(self):
		'''(Compressor) Wet/Dry'''
		return self._compressor_wet_dry

	@compressor_wet_dry.setter
	def compressor_wet_dry(self,value):
		if 0 <= value <= 100:
			self._compressor_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_source(self):
		'''(Compressor) Wet/Dry Mod.Source'''
		return self._compressor_wet_dry_mod_source

	@compressor_wet_dry_mod_source.setter
	def compressor_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._compressor_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_intensity(self):
		'''(Compressor) Wet/Dry Mod.Intensity'''
		return self._compressor_wet_dry_mod_intensity

	@compressor_wet_dry_mod_intensity.setter
	def compressor_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._compressor_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_input_trim(self):
		'''(Compressor) Input Trim'''
		return self._compressor_input_trim

	@compressor_input_trim.setter
	def compressor_input_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pan(self):
		'''(Compressor) Pan'''
		return self._compressor_pan

	@compressor_pan.setter
	def compressor_pan(self,value):
		if 0 <= value <= 127:
			self._compressor_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry(self):
		'''(Limiter) Wet/Dry'''
		return self._limiter_wet_dry

	@limiter_wet_dry.setter
	def limiter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._limiter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_source(self):
		'''(Limiter) Wet/Dry Mod.Source'''
		return self._limiter_wet_dry_mod_source

	@limiter_wet_dry_mod_source.setter
	def limiter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._limiter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_intensity(self):
		'''(Limiter) Wet/Dry Mod.Intensity'''
		return self._limiter_wet_dry_mod_intensity

	@limiter_wet_dry_mod_intensity.setter
	def limiter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._limiter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_pan(self):
		'''(Limiter) Pan'''
		return self._limiter_pan

	@limiter_pan.setter
	def limiter_pan(self,value):
		if 0 <= value <= 127:
			self._limiter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

