#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx138.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX138(object):
	category = '''Mono//Mono'''
	description = '''Compressor // Exciter'''
	short_description = '''Comp // Exciter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX138 Compressor // Exciter object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.compressor_wet_dry = (bytes[0] >> 1) & 0x7F
		self.compressor_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.compressor_wet_dry_mod_intensity = data
		self.compressor_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.compressor_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.exciter_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.exciter_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = (bytes[7] >> 1) & 0x7F
		self.exciter_pan = bytes[8] & 0x7F
		self.compressor_sensitivity = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.compressor_attack = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.compressor_output_level = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0xF) << 3)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = (bytes[12] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		data = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x1F) << 2)
		data = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x3) << 3)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = (bytes[15] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX138 Compressor // Exciter object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.compressor_wet_dry & 0x7F) << 1
		bytes[1] = (self.compressor_wet_dry_mod_source & 0x3F) | ((self.compressor_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.compressor_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.compressor_input_trim << 6) & 0xC0)
		bytes[3] = ((self.compressor_input_trim >> 2) & 0x1F) | ((self.compressor_pan << 5) & 0xE0)
		bytes[4] = ((self.compressor_pan >> 3) & 0xF) | ((self.exciter_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.exciter_wet_dry >> 4) & 0x7) | ((self.exciter_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.exciter_wet_dry_mod_source >> 5) & 0x1) | ((self.exciter_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = ((self.exciter_wet_dry_mod_intensity >> 7) & 0x1) | ((self.exciter_input_trim & 0x7F) << 1)
		bytes[8] = (self.exciter_pan & 0x7F) | ((self.compressor_sensitivity << 7) & 0x80)
		bytes[9] = ((self.compressor_sensitivity >> 1) & 0x3F) | ((self.compressor_attack << 6) & 0xC0)
		bytes[10] = ((self.compressor_attack >> 2) & 0x1F) | ((self.compressor_output_level << 5) & 0xE0)
		bytes[11] = ((self.compressor_output_level >> 3) & 0xF) | ((self.compressor_pre_leq_gain << 4) & 0xF0)
		bytes[12] = ((self.compressor_pre_leq_gain >> 4) & 0x1) | ((self.compressor_pre_heq_gain & 0x1F) << 1) | ((self.exciter_blend << 6) & 0xC0)
		bytes[13] = ((self.exciter_blend >> 2) & 0x3F) | ((self.exciter_emphasis_frequency << 6) & 0xC0)
		bytes[14] = ((self.exciter_emphasis_frequency >> 2) & 0x1F) | ((self.exciter_pre_leq_gain << 5) & 0xE0)
		bytes[15] = ((self.exciter_pre_leq_gain >> 3) & 0x3) | ((self.exciter_pre_heq_gain & 0x1F) << 2)
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX138 Compressor // Exciter object'''
		json = []
		json.append(('name','FX138'))
		json.append(('description','Compressor // Exciter'))
		json.append(['compressor_wet_dry',self.compressor_wet_dry])
		json.append(['compressor_wet_dry_mod_source',self.compressor_wet_dry_mod_source])
		json.append(['compressor_wet_dry_mod_intensity',self.compressor_wet_dry_mod_intensity])
		json.append(['compressor_input_trim',self.compressor_input_trim])
		json.append(['compressor_pan',self.compressor_pan])
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry(self):
		'''(Compressor) Wet/Dry'''
		return self._compressor_wet_dry

	@compressor_wet_dry.setter
	def compressor_wet_dry(self,value):
		if 0 <= value <= 100:
			self._compressor_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_source(self):
		'''(Compressor) Wet/Dry Mod.Source'''
		return self._compressor_wet_dry_mod_source

	@compressor_wet_dry_mod_source.setter
	def compressor_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._compressor_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_intensity(self):
		'''(Compressor) Wet/Dry Mod.Intensity'''
		return self._compressor_wet_dry_mod_intensity

	@compressor_wet_dry_mod_intensity.setter
	def compressor_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._compressor_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_input_trim(self):
		'''(Compressor) Input Trim'''
		return self._compressor_input_trim

	@compressor_input_trim.setter
	def compressor_input_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pan(self):
		'''(Compressor) Pan'''
		return self._compressor_pan

	@compressor_pan.setter
	def compressor_pan(self,value):
		if 0 <= value <= 127:
			self._compressor_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

