#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx141.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX141(object):
	category = '''Mono//Mono'''
	description = '''Compressor // Chorus/Flanger'''
	short_description = '''Comp // Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX141 Compressor // Chorus/Flanger object'''
		bytes = struct.unpack('<19B',raw[:19])
		self.compressor_wet_dry = (bytes[0] >> 1) & 0x7F
		self.compressor_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.compressor_wet_dry_mod_intensity = data
		self.compressor_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.compressor_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_input_trim = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		self.chorus_flanger_pan = (bytes[8] >> 1) & 0x7F
		self.compressor_sensitivity = bytes[9] & 0x7F
		self.compressor_attack = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		self.compressor_output_level = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		data = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x3) << 3)
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_leq_gain = data
		data = (bytes[12] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.compressor_pre_heq_gain = data
		self.chorus_flanger_pre_delay = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0xFF) << 1)
		self.chorus_flanger_lfo_waveform = (bytes[14] >> 2) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x7) << 5)
		self.chorus_flanger_depth = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x3) << 5)
		data = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = (bytes[17] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX141 Compressor // Chorus/Flanger object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.compressor_wet_dry & 0x7F) << 1
		bytes[1] = (self.compressor_wet_dry_mod_source & 0x3F) | ((self.compressor_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.compressor_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.compressor_input_trim << 6) & 0xC0)
		bytes[3] = ((self.compressor_input_trim >> 2) & 0x1F) | ((self.compressor_pan << 5) & 0xE0)
		bytes[4] = ((self.compressor_pan >> 3) & 0xF) | ((self.chorus_flanger_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.chorus_flanger_wet_dry >> 4) & 0xF) | ((self.chorus_flanger_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.chorus_flanger_wet_dry_mod_source >> 4) & 0x3) | ((self.chorus_flanger_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = ((self.chorus_flanger_wet_dry_mod_intensity >> 6) & 0x3) | ((self.chorus_flanger_input_trim << 2) & 0xFC)
		bytes[8] = ((self.chorus_flanger_input_trim >> 6) & 0x1) | ((self.chorus_flanger_pan & 0x7F) << 1)
		bytes[9] = (self.compressor_sensitivity & 0x7F) | ((self.compressor_attack << 7) & 0x80)
		bytes[10] = ((self.compressor_attack >> 1) & 0x3F) | ((self.compressor_output_level << 6) & 0xC0)
		bytes[11] = ((self.compressor_output_level >> 2) & 0x1F) | ((self.compressor_pre_leq_gain << 5) & 0xE0)
		bytes[12] = ((self.compressor_pre_leq_gain >> 3) & 0x3) | ((self.compressor_pre_heq_gain & 0x1F) << 2) | ((self.chorus_flanger_pre_delay << 7) & 0x80)
		bytes[13] = (self.chorus_flanger_pre_delay >> 1) & 0xFF
		bytes[14] = ((self.chorus_flanger_lfo_waveform & 0x1) << 2) | ((self.chorus_flanger_lfo_frequency << 3) & 0xF8)
		bytes[15] = ((self.chorus_flanger_lfo_frequency >> 5) & 0x7) | ((self.chorus_flanger_depth << 3) & 0xF8)
		bytes[16] = ((self.chorus_flanger_depth >> 5) & 0x3) | ((self.chorus_flanger_feedback << 2) & 0xFC)
		bytes[17] = ((self.chorus_flanger_feedback >> 6) & 0x3) | ((self.chorus_flanger_pre_leq_gain & 0x1F) << 2) | ((self.chorus_flanger_pre_heq_gain << 7) & 0x80)
		bytes[18] = (self.chorus_flanger_pre_heq_gain >> 1) & 0xF
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX141 Compressor // Chorus/Flanger object'''
		json = []
		json.append(('name','FX141'))
		json.append(('description','Compressor // Chorus/Flanger'))
		json.append(['compressor_wet_dry',self.compressor_wet_dry])
		json.append(['compressor_wet_dry_mod_source',self.compressor_wet_dry_mod_source])
		json.append(['compressor_wet_dry_mod_intensity',self.compressor_wet_dry_mod_intensity])
		json.append(['compressor_input_trim',self.compressor_input_trim])
		json.append(['compressor_pan',self.compressor_pan])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		json.append(['chorus_flanger_input_trim',self.chorus_flanger_input_trim])
		json.append(['chorus_flanger_pan',self.chorus_flanger_pan])
		json.append(['compressor_sensitivity',self.compressor_sensitivity])
		json.append(['compressor_attack',self.compressor_attack])
		json.append(['compressor_output_level',self.compressor_output_level])
		json.append(['compressor_pre_leq_gain',self.compressor_pre_leq_gain])
		json.append(['compressor_pre_heq_gain',self.compressor_pre_heq_gain])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry(self):
		'''(Compressor) Wet/Dry'''
		return self._compressor_wet_dry

	@compressor_wet_dry.setter
	def compressor_wet_dry(self,value):
		if 0 <= value <= 100:
			self._compressor_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_source(self):
		'''(Compressor) Wet/Dry Mod.Source'''
		return self._compressor_wet_dry_mod_source

	@compressor_wet_dry_mod_source.setter
	def compressor_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._compressor_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_wet_dry_mod_intensity(self):
		'''(Compressor) Wet/Dry Mod.Intensity'''
		return self._compressor_wet_dry_mod_intensity

	@compressor_wet_dry_mod_intensity.setter
	def compressor_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._compressor_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_input_trim(self):
		'''(Compressor) Input Trim'''
		return self._compressor_input_trim

	@compressor_input_trim.setter
	def compressor_input_trim(self,value):
		if 0 <= value <= 100:
			self._compressor_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pan(self):
		'''(Compressor) Pan'''
		return self._compressor_pan

	@compressor_pan.setter
	def compressor_pan(self,value):
		if 0 <= value <= 127:
			self._compressor_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_input_trim(self):
		'''(Chorus/Flanger) Input Trim'''
		return self._chorus_flanger_input_trim

	@chorus_flanger_input_trim.setter
	def chorus_flanger_input_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pan(self):
		'''(Chorus/Flanger) Pan'''
		return self._chorus_flanger_pan

	@chorus_flanger_pan.setter
	def chorus_flanger_pan(self,value):
		if 0 <= value <= 127:
			self._chorus_flanger_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_sensitivity(self):
		'''(Compressor) Sensitivity'''
		return self._compressor_sensitivity

	@compressor_sensitivity.setter
	def compressor_sensitivity(self,value):
		if 1 <= value <= 100:
			self._compressor_sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_attack(self):
		'''(Compressor) Attack'''
		return self._compressor_attack

	@compressor_attack.setter
	def compressor_attack(self,value):
		if 1 <= value <= 100:
			self._compressor_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_output_level(self):
		'''(Compressor) Output Level'''
		return self._compressor_output_level

	@compressor_output_level.setter
	def compressor_output_level(self,value):
		if 0 <= value <= 100:
			self._compressor_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_leq_gain(self):
		'''(Compressor) Pre LEQ Gain'''
		return self._compressor_pre_leq_gain

	@compressor_pre_leq_gain.setter
	def compressor_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def compressor_pre_heq_gain(self):
		'''(Compressor) Pre HEQ Gain'''
		return self._compressor_pre_heq_gain

	@compressor_pre_heq_gain.setter
	def compressor_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._compressor_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

