#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx144.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX144(object):
	category = '''Mono//Mono'''
	description = '''Limiter // Limiter'''
	short_description = '''Limiter // Limiter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX144 Limiter // Limiter object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_ratio = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.effect1_threshold = data
		self.effect1_attack = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		self.effect1_release = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.effect1_gain_adjust = data
		self.effect2_ratio = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x7) << 5)
		data = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.effect2_threshold = data
		self.effect2_attack = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x1) << 6)
		self.effect2_release = (bytes[16] >> 1) & 0x7F
		data = bytes[17] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.effect2_gain_adjust = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX144 Limiter // Limiter object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.effect1_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.effect1_pan << 5) & 0xE0
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.effect2_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_ratio << 7) & 0x80)
		bytes[9] = ((self.effect1_ratio >> 1) & 0x7F) | ((self.effect1_threshold << 7) & 0x80)
		bytes[10] = ((self.effect1_threshold >> 1) & 0x3F) | ((self.effect1_attack << 6) & 0xC0)
		bytes[11] = ((self.effect1_attack >> 2) & 0x1F) | ((self.effect1_release << 5) & 0xE0)
		bytes[12] = ((self.effect1_release >> 3) & 0xF) | ((self.effect1_gain_adjust << 4) & 0xF0)
		bytes[13] = ((self.effect1_gain_adjust >> 4) & 0x7) | ((self.effect2_ratio << 3) & 0xF8)
		bytes[14] = ((self.effect2_ratio >> 5) & 0x7) | ((self.effect2_threshold << 3) & 0xF8)
		bytes[15] = ((self.effect2_threshold >> 5) & 0x3) | ((self.effect2_attack << 2) & 0xFC)
		bytes[16] = ((self.effect2_attack >> 6) & 0x1) | ((self.effect2_release & 0x7F) << 1)
		bytes[17] = self.effect2_gain_adjust & 0x7F
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX144 Limiter // Limiter object'''
		json = []
		json.append(('name','FX144'))
		json.append(('description','Limiter // Limiter'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_ratio',self.effect1_ratio])
		json.append(['effect1_threshold',self.effect1_threshold])
		json.append(['effect1_attack',self.effect1_attack])
		json.append(['effect1_release',self.effect1_release])
		json.append(['effect1_gain_adjust',self.effect1_gain_adjust])
		json.append(['effect2_ratio',self.effect2_ratio])
		json.append(['effect2_threshold',self.effect2_threshold])
		json.append(['effect2_attack',self.effect2_attack])
		json.append(['effect2_release',self.effect2_release])
		json.append(['effect2_gain_adjust',self.effect2_gain_adjust])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_ratio(self):
		'''(Effect1) Ratio'''
		return self._effect1_ratio

	@effect1_ratio.setter
	def effect1_ratio(self,value):
		if 0 <= value <= 131:
			self._effect1_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_threshold(self):
		'''(Effect1) Threshold'''
		return self._effect1_threshold

	@effect1_threshold.setter
	def effect1_threshold(self,value):
		if -40 <= value <= 0:
			self._effect1_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_attack(self):
		'''(Effect1) Attack'''
		return self._effect1_attack

	@effect1_attack.setter
	def effect1_attack(self,value):
		if 1 <= value <= 100:
			self._effect1_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_release(self):
		'''(Effect1) Release'''
		return self._effect1_release

	@effect1_release.setter
	def effect1_release(self,value):
		if 1 <= value <= 100:
			self._effect1_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_gain_adjust(self):
		'''(Effect1) Gain Adjust'''
		return self._effect1_gain_adjust

	@effect1_gain_adjust.setter
	def effect1_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._effect1_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_ratio(self):
		'''(Effect2) Ratio'''
		return self._effect2_ratio

	@effect2_ratio.setter
	def effect2_ratio(self,value):
		if 0 <= value <= 131:
			self._effect2_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_threshold(self):
		'''(Effect2) Threshold'''
		return self._effect2_threshold

	@effect2_threshold.setter
	def effect2_threshold(self,value):
		if -40 <= value <= 0:
			self._effect2_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_attack(self):
		'''(Effect2) Attack'''
		return self._effect2_attack

	@effect2_attack.setter
	def effect2_attack(self,value):
		if 1 <= value <= 100:
			self._effect2_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_release(self):
		'''(Effect2) Release'''
		return self._effect2_release

	@effect2_release.setter
	def effect2_release(self,value):
		if 1 <= value <= 100:
			self._effect2_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_gain_adjust(self):
		'''(Effect2) Gain Adjust'''
		return self._effect2_gain_adjust

	@effect2_gain_adjust.setter
	def effect2_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._effect2_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

