#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx145.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX145(object):
	category = '''Mono//Mono'''
	description = '''Limiter // Exciter'''
	short_description = '''Limiter // Exciter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX145 Limiter // Exciter object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.limiter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.limiter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.limiter_wet_dry_mod_intensity = data
		self.limiter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.exciter_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.exciter_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = (bytes[7] >> 1) & 0x7F
		self.exciter_pan = bytes[8] & 0x7F
		self.limiter_ratio = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		self.limiter_release = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		data = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		data = (bytes[15] >> 2) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX145 Limiter // Exciter object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.limiter_wet_dry & 0x7F) << 1
		bytes[1] = (self.limiter_wet_dry_mod_source & 0x3F) | ((self.limiter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.limiter_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_pan << 5) & 0xE0
		bytes[4] = ((self.limiter_pan >> 3) & 0xF) | ((self.exciter_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.exciter_wet_dry >> 4) & 0x7) | ((self.exciter_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.exciter_wet_dry_mod_source >> 5) & 0x1) | ((self.exciter_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = ((self.exciter_wet_dry_mod_intensity >> 7) & 0x1) | ((self.exciter_input_trim & 0x7F) << 1)
		bytes[8] = (self.exciter_pan & 0x7F) | ((self.limiter_ratio << 7) & 0x80)
		bytes[9] = ((self.limiter_ratio >> 1) & 0x7F) | ((self.limiter_threshold << 7) & 0x80)
		bytes[10] = ((self.limiter_threshold >> 1) & 0x3F) | ((self.limiter_attack << 6) & 0xC0)
		bytes[11] = ((self.limiter_attack >> 2) & 0x1F) | ((self.limiter_release << 5) & 0xE0)
		bytes[12] = ((self.limiter_release >> 3) & 0xF) | ((self.limiter_gain_adjust << 4) & 0xF0)
		bytes[13] = ((self.limiter_gain_adjust >> 4) & 0x7) | ((self.exciter_blend << 3) & 0xF8)
		bytes[14] = ((self.exciter_blend >> 5) & 0x7) | ((self.exciter_emphasis_frequency << 3) & 0xF8)
		bytes[15] = ((self.exciter_emphasis_frequency >> 5) & 0x3) | ((self.exciter_pre_leq_gain & 0x1F) << 2) | ((self.exciter_pre_heq_gain << 7) & 0x80)
		bytes[16] = (self.exciter_pre_heq_gain >> 1) & 0xF
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX145 Limiter // Exciter object'''
		json = []
		json.append(('name','FX145'))
		json.append(('description','Limiter // Exciter'))
		json.append(['limiter_wet_dry',self.limiter_wet_dry])
		json.append(['limiter_wet_dry_mod_source',self.limiter_wet_dry_mod_source])
		json.append(['limiter_wet_dry_mod_intensity',self.limiter_wet_dry_mod_intensity])
		json.append(['limiter_pan',self.limiter_pan])
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry(self):
		'''(Limiter) Wet/Dry'''
		return self._limiter_wet_dry

	@limiter_wet_dry.setter
	def limiter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._limiter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_source(self):
		'''(Limiter) Wet/Dry Mod.Source'''
		return self._limiter_wet_dry_mod_source

	@limiter_wet_dry_mod_source.setter
	def limiter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._limiter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_intensity(self):
		'''(Limiter) Wet/Dry Mod.Intensity'''
		return self._limiter_wet_dry_mod_intensity

	@limiter_wet_dry_mod_intensity.setter
	def limiter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._limiter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_pan(self):
		'''(Limiter) Pan'''
		return self._limiter_pan

	@limiter_pan.setter
	def limiter_pan(self,value):
		if 0 <= value <= 127:
			self._limiter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

