#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx146.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX146(object):
	category = '''Mono//Mono'''
	description = '''Limiter // Overdrive/Hi.Gain'''
	short_description = '''Limiter // OD/HG'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX146 Limiter // Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<23B',raw[:23])
		self.limiter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.limiter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.limiter_wet_dry_mod_intensity = data
		self.limiter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.od_hi_gain_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.od_hi_gain_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.od_hi_gain_pan = bytes[8] & 0x7F
		self.limiter_ratio = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		self.limiter_release = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		self.od_hi_gain_hi_gain_on = (bytes[13] >> 3) & 0x1
		self.od_hi_gain_drive = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		self.od_hi_gain_output_level = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x1) << 5)
		self.od_hi_gain_output_level_mod_source = (bytes[15] >> 1) & 0x3F
		data = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0xF) << 2)
		data = ((bytes[17] & 0xF0) >> 4) | ((bytes[18] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = (bytes[18] >> 2) & 0x3F
		self.od_hi_gain_eq_mid1_q = bytes[19] & 0xF
		data = ((bytes[19] & 0xF0) >> 4) | ((bytes[20] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = (bytes[20] >> 2) & 0x3F
		self.od_hi_gain_eq_mid2_q = bytes[21] & 0xF
		data = ((bytes[21] & 0xF0) >> 4) | ((bytes[22] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX146 Limiter // Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 23
		bytes[0] = (self.limiter_wet_dry & 0x7F) << 1
		bytes[1] = (self.limiter_wet_dry_mod_source & 0x3F) | ((self.limiter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.limiter_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_pan << 5) & 0xE0
		bytes[4] = ((self.limiter_pan >> 3) & 0xF) | ((self.od_hi_gain_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.od_hi_gain_wet_dry >> 4) & 0x7) | ((self.od_hi_gain_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.od_hi_gain_wet_dry_mod_source >> 5) & 0x1) | ((self.od_hi_gain_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.od_hi_gain_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.od_hi_gain_pan & 0x7F) | ((self.limiter_ratio << 7) & 0x80)
		bytes[9] = ((self.limiter_ratio >> 1) & 0x7F) | ((self.limiter_threshold << 7) & 0x80)
		bytes[10] = ((self.limiter_threshold >> 1) & 0x3F) | ((self.limiter_attack << 6) & 0xC0)
		bytes[11] = ((self.limiter_attack >> 2) & 0x1F) | ((self.limiter_release << 5) & 0xE0)
		bytes[12] = ((self.limiter_release >> 3) & 0xF) | ((self.limiter_gain_adjust << 4) & 0xF0)
		bytes[13] = ((self.limiter_gain_adjust >> 4) & 0x7) | ((self.od_hi_gain_hi_gain_on & 0x1) << 3) | ((self.od_hi_gain_drive << 4) & 0xF0)
		bytes[14] = ((self.od_hi_gain_drive >> 4) & 0x7) | ((self.od_hi_gain_output_level << 3) & 0xF8)
		bytes[15] = ((self.od_hi_gain_output_level >> 5) & 0x1) | ((self.od_hi_gain_output_level_mod_source & 0x3F) << 1) | ((self.od_hi_gain_output_level_mod_intensity << 7) & 0x80)
		bytes[16] = ((self.od_hi_gain_output_level_mod_intensity >> 1) & 0x3F) | ((self.od_hi_gain_eq_low_cutoff << 6) & 0xC0)
		bytes[17] = ((self.od_hi_gain_eq_low_cutoff >> 2) & 0xF) | ((self.od_hi_gain_eq_low_gain << 4) & 0xF0)
		bytes[18] = ((self.od_hi_gain_eq_low_gain >> 4) & 0x3) | ((self.od_hi_gain_eq_mid1_cutoff & 0x3F) << 2)
		bytes[19] = (self.od_hi_gain_eq_mid1_q & 0xF) | ((self.od_hi_gain_eq_mid1_gain << 4) & 0xF0)
		bytes[20] = ((self.od_hi_gain_eq_mid1_gain >> 4) & 0x3) | ((self.od_hi_gain_eq_mid2_cutoff & 0x3F) << 2)
		bytes[21] = (self.od_hi_gain_eq_mid2_q & 0xF) | ((self.od_hi_gain_eq_mid2_gain << 4) & 0xF0)
		bytes[22] = (self.od_hi_gain_eq_mid2_gain >> 4) & 0x3
		return struct.pack('<23B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX146 Limiter // Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX146'))
		json.append(('description','Limiter // Overdrive/Hi.Gain'))
		json.append(['limiter_wet_dry',self.limiter_wet_dry])
		json.append(['limiter_wet_dry_mod_source',self.limiter_wet_dry_mod_source])
		json.append(['limiter_wet_dry_mod_intensity',self.limiter_wet_dry_mod_intensity])
		json.append(['limiter_pan',self.limiter_pan])
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		json.append(['od_hi_gain_pan',self.od_hi_gain_pan])
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry(self):
		'''(Limiter) Wet/Dry'''
		return self._limiter_wet_dry

	@limiter_wet_dry.setter
	def limiter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._limiter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_source(self):
		'''(Limiter) Wet/Dry Mod.Source'''
		return self._limiter_wet_dry_mod_source

	@limiter_wet_dry_mod_source.setter
	def limiter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._limiter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_intensity(self):
		'''(Limiter) Wet/Dry Mod.Intensity'''
		return self._limiter_wet_dry_mod_intensity

	@limiter_wet_dry_mod_intensity.setter
	def limiter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._limiter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_pan(self):
		'''(Limiter) Pan'''
		return self._limiter_pan

	@limiter_pan.setter
	def limiter_pan(self,value):
		if 0 <= value <= 127:
			self._limiter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_pan(self):
		'''(OD/Hi-Gain) Pan'''
		return self._od_hi_gain_pan

	@od_hi_gain_pan.setter
	def od_hi_gain_pan(self,value):
		if 0 <= value <= 127:
			self._od_hi_gain_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

