#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx147.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX147(object):
	category = '''Mono//Mono'''
	description = '''Overdrive/Hi.Gain // Overdrive/Hi.Gain'''
	short_description = '''OD/HG // OD/HG'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX147 Overdrive/Hi.Gain // Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<19B',raw[:19])
		self.limiter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.limiter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.limiter_wet_dry_mod_intensity = data
		self.limiter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.wah_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.wah_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.wah_pan = bytes[8] & 0x7F
		self.limiter_ratio = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.limiter_threshold = data
		self.limiter_attack = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		self.limiter_release = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.limiter_gain_adjust = data
		self.wah_frequency_bottom = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x3) << 5)
		self.wah_frequency_top = ((bytes[14] & 0xFC) >> 2) | ((bytes[15] & 0x1) << 6)
		self.wah_sweep_mode = (bytes[15] >> 1) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x1) << 5)
		self.wah_lfo_frequency = ((bytes[16] & 0xFE) >> 1) | ((bytes[17] & 0x1) << 7)
		self.wah_resonance = (bytes[17] >> 1) & 0x7F
		self.wah_low_pass_filter = bytes[18] & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX147 Overdrive/Hi.Gain // Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.limiter_wet_dry & 0x7F) << 1
		bytes[1] = (self.limiter_wet_dry_mod_source & 0x3F) | ((self.limiter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.limiter_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.limiter_pan << 5) & 0xE0
		bytes[4] = ((self.limiter_pan >> 3) & 0xF) | ((self.wah_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.wah_wet_dry >> 4) & 0x7) | ((self.wah_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.wah_wet_dry_mod_source >> 5) & 0x1) | ((self.wah_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.wah_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.wah_pan & 0x7F) | ((self.limiter_ratio << 7) & 0x80)
		bytes[9] = ((self.limiter_ratio >> 1) & 0x7F) | ((self.limiter_threshold << 7) & 0x80)
		bytes[10] = ((self.limiter_threshold >> 1) & 0x3F) | ((self.limiter_attack << 6) & 0xC0)
		bytes[11] = ((self.limiter_attack >> 2) & 0x1F) | ((self.limiter_release << 5) & 0xE0)
		bytes[12] = ((self.limiter_release >> 3) & 0xF) | ((self.limiter_gain_adjust << 4) & 0xF0)
		bytes[13] = ((self.limiter_gain_adjust >> 4) & 0x7) | ((self.wah_frequency_bottom << 3) & 0xF8)
		bytes[14] = ((self.wah_frequency_bottom >> 5) & 0x3) | ((self.wah_frequency_top << 2) & 0xFC)
		bytes[15] = ((self.wah_frequency_top >> 6) & 0x1) | ((self.wah_sweep_mode & 0x3) << 1) | ((self.wah_sweep_mode_mod_source << 3) & 0xF8)
		bytes[16] = ((self.wah_sweep_mode_mod_source >> 5) & 0x1) | ((self.wah_lfo_frequency << 1) & 0xFE)
		bytes[17] = ((self.wah_lfo_frequency >> 7) & 0x1) | ((self.wah_resonance & 0x7F) << 1)
		bytes[18] = self.wah_low_pass_filter & 0x1
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX147 Overdrive/Hi.Gain // Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX147'))
		json.append(('description','Overdrive/Hi.Gain // Overdrive/Hi.Gain'))
		json.append(['limiter_wet_dry',self.limiter_wet_dry])
		json.append(['limiter_wet_dry_mod_source',self.limiter_wet_dry_mod_source])
		json.append(['limiter_wet_dry_mod_intensity',self.limiter_wet_dry_mod_intensity])
		json.append(['limiter_pan',self.limiter_pan])
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		json.append(['wah_pan',self.wah_pan])
		json.append(['limiter_ratio',self.limiter_ratio])
		json.append(['limiter_threshold',self.limiter_threshold])
		json.append(['limiter_attack',self.limiter_attack])
		json.append(['limiter_release',self.limiter_release])
		json.append(['limiter_gain_adjust',self.limiter_gain_adjust])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry(self):
		'''(Limiter) Wet/Dry'''
		return self._limiter_wet_dry

	@limiter_wet_dry.setter
	def limiter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._limiter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_source(self):
		'''(Limiter) Wet/Dry Mod.Source'''
		return self._limiter_wet_dry_mod_source

	@limiter_wet_dry_mod_source.setter
	def limiter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._limiter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_wet_dry_mod_intensity(self):
		'''(Limiter) Wet/Dry Mod.Intensity'''
		return self._limiter_wet_dry_mod_intensity

	@limiter_wet_dry_mod_intensity.setter
	def limiter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._limiter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_pan(self):
		'''(Limiter) Pan'''
		return self._limiter_pan

	@limiter_pan.setter
	def limiter_pan(self,value):
		if 0 <= value <= 127:
			self._limiter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_pan(self):
		'''(Wah) Pan'''
		return self._wah_pan

	@wah_pan.setter
	def wah_pan(self,value):
		if 0 <= value <= 127:
			self._wah_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_ratio(self):
		'''(Limiter) Ratio'''
		return self._limiter_ratio

	@limiter_ratio.setter
	def limiter_ratio(self,value):
		if 0 <= value <= 131:
			self._limiter_ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_threshold(self):
		'''(Limiter) Threshold'''
		return self._limiter_threshold

	@limiter_threshold.setter
	def limiter_threshold(self,value):
		if -40 <= value <= 0:
			self._limiter_threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_attack(self):
		'''(Limiter) Attack'''
		return self._limiter_attack

	@limiter_attack.setter
	def limiter_attack(self,value):
		if 1 <= value <= 100:
			self._limiter_attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_release(self):
		'''(Limiter) Release'''
		return self._limiter_release

	@limiter_release.setter
	def limiter_release(self,value):
		if 1 <= value <= 100:
			self._limiter_release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def limiter_gain_adjust(self):
		'''(Limiter) Gain Adjust'''
		return self._limiter_gain_adjust

	@limiter_gain_adjust.setter
	def limiter_gain_adjust(self,value):
		if -39 <= value <= 24:
			self._limiter_gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

