#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx152.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX152(object):
	category = '''Mono//Mono'''
	description = '''Exciter // Overdrive/Hi.Gain'''
	short_description = '''Exciter // OD/HG'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX152 Exciter // Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.exciter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.exciter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.exciter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.od_hi_gain_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.od_hi_gain_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.od_hi_gain_pan = bytes[8] & 0x7F
		data = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = (bytes[11] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.od_hi_gain_hi_gain_on = bytes[12] & 0x1
		self.od_hi_gain_drive = (bytes[12] >> 1) & 0x7F
		self.od_hi_gain_output_level = bytes[13] & 0x3F
		self.od_hi_gain_output_level_mod_source = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xF) << 2)
		data = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x1) << 5)
		data = (bytes[16] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[16] & 0x80) >> 7) | ((bytes[17] & 0x1F) << 1)
		self.od_hi_gain_eq_mid1_q = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x1) << 3)
		data = (bytes[18] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x1F) << 1)
		self.od_hi_gain_eq_mid2_q = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x1) << 3)
		data = (bytes[20] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX152 Exciter // Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.exciter_wet_dry & 0x7F) << 1
		bytes[1] = (self.exciter_wet_dry_mod_source & 0x3F) | ((self.exciter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.exciter_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.exciter_input_trim << 6) & 0xC0)
		bytes[3] = ((self.exciter_input_trim >> 2) & 0x1F) | ((self.exciter_pan << 5) & 0xE0)
		bytes[4] = ((self.exciter_pan >> 3) & 0xF) | ((self.od_hi_gain_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.od_hi_gain_wet_dry >> 4) & 0x7) | ((self.od_hi_gain_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.od_hi_gain_wet_dry_mod_source >> 5) & 0x1) | ((self.od_hi_gain_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.od_hi_gain_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.od_hi_gain_pan & 0x7F) | ((self.exciter_blend << 7) & 0x80)
		bytes[9] = ((self.exciter_blend >> 1) & 0x7F) | ((self.exciter_emphasis_frequency << 7) & 0x80)
		bytes[10] = ((self.exciter_emphasis_frequency >> 1) & 0x3F) | ((self.exciter_pre_leq_gain << 6) & 0xC0)
		bytes[11] = ((self.exciter_pre_leq_gain >> 2) & 0x7) | ((self.exciter_pre_heq_gain & 0x1F) << 3)
		bytes[12] = (self.od_hi_gain_hi_gain_on & 0x1) | ((self.od_hi_gain_drive & 0x7F) << 1)
		bytes[13] = (self.od_hi_gain_output_level & 0x3F) | ((self.od_hi_gain_output_level_mod_source << 6) & 0xC0)
		bytes[14] = ((self.od_hi_gain_output_level_mod_source >> 2) & 0xF) | ((self.od_hi_gain_output_level_mod_intensity << 4) & 0xF0)
		bytes[15] = ((self.od_hi_gain_output_level_mod_intensity >> 4) & 0x7) | ((self.od_hi_gain_eq_low_cutoff << 3) & 0xF8)
		bytes[16] = ((self.od_hi_gain_eq_low_cutoff >> 5) & 0x1) | ((self.od_hi_gain_eq_low_gain & 0x3F) << 1) | ((self.od_hi_gain_eq_mid1_cutoff << 7) & 0x80)
		bytes[17] = ((self.od_hi_gain_eq_mid1_cutoff >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid1_q << 5) & 0xE0)
		bytes[18] = ((self.od_hi_gain_eq_mid1_q >> 3) & 0x1) | ((self.od_hi_gain_eq_mid1_gain & 0x3F) << 1) | ((self.od_hi_gain_eq_mid2_cutoff << 7) & 0x80)
		bytes[19] = ((self.od_hi_gain_eq_mid2_cutoff >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid2_q << 5) & 0xE0)
		bytes[20] = ((self.od_hi_gain_eq_mid2_q >> 3) & 0x1) | ((self.od_hi_gain_eq_mid2_gain & 0x3F) << 1)
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX152 Exciter // Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX152'))
		json.append(('description','Exciter // Overdrive/Hi.Gain'))
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		json.append(['od_hi_gain_pan',self.od_hi_gain_pan])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_pan(self):
		'''(OD/Hi-Gain) Pan'''
		return self._od_hi_gain_pan

	@od_hi_gain_pan.setter
	def od_hi_gain_pan(self,value):
		if 0 <= value <= 127:
			self._od_hi_gain_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

