#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx154.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX154(object):
	category = '''Mono//Mono'''
	description = '''Exciter // Chorus/Flanger'''
	short_description = '''Exciter // Cho/Fl'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX154 Exciter // Chorus/Flanger object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.exciter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.exciter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.exciter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_input_trim = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		self.chorus_flanger_pan = (bytes[8] >> 1) & 0x7F
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = bytes[10] & 0x7F
		data = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.chorus_flanger_pre_delay = ((bytes[12] & 0xFE) >> 1) | ((bytes[13] & 0xF) << 7)
		self.chorus_flanger_lfo_waveform = (bytes[13] >> 4) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x1F) << 3)
		self.chorus_flanger_depth = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0xF) << 3)
		data = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = ((bytes[16] & 0xF0) >> 4) | ((bytes[17] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = (bytes[17] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX154 Exciter // Chorus/Flanger object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.exciter_wet_dry & 0x7F) << 1
		bytes[1] = (self.exciter_wet_dry_mod_source & 0x3F) | ((self.exciter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.exciter_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.exciter_input_trim << 6) & 0xC0)
		bytes[3] = ((self.exciter_input_trim >> 2) & 0x1F) | ((self.exciter_pan << 5) & 0xE0)
		bytes[4] = ((self.exciter_pan >> 3) & 0xF) | ((self.chorus_flanger_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.chorus_flanger_wet_dry >> 4) & 0xF) | ((self.chorus_flanger_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.chorus_flanger_wet_dry_mod_source >> 4) & 0x3) | ((self.chorus_flanger_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = ((self.chorus_flanger_wet_dry_mod_intensity >> 6) & 0x3) | ((self.chorus_flanger_input_trim << 2) & 0xFC)
		bytes[8] = ((self.chorus_flanger_input_trim >> 6) & 0x1) | ((self.chorus_flanger_pan & 0x7F) << 1)
		bytes[9] = self.exciter_blend & 0xFF
		bytes[10] = (self.exciter_emphasis_frequency & 0x7F) | ((self.exciter_pre_leq_gain << 7) & 0x80)
		bytes[11] = ((self.exciter_pre_leq_gain >> 1) & 0xF) | ((self.exciter_pre_heq_gain << 4) & 0xF0)
		bytes[12] = ((self.exciter_pre_heq_gain >> 4) & 0x1) | ((self.chorus_flanger_pre_delay << 1) & 0xFE)
		bytes[13] = ((self.chorus_flanger_pre_delay >> 7) & 0xF) | ((self.chorus_flanger_lfo_waveform & 0x1) << 4) | ((self.chorus_flanger_lfo_frequency << 5) & 0xE0)
		bytes[14] = ((self.chorus_flanger_lfo_frequency >> 3) & 0x1F) | ((self.chorus_flanger_depth << 5) & 0xE0)
		bytes[15] = ((self.chorus_flanger_depth >> 3) & 0xF) | ((self.chorus_flanger_feedback << 4) & 0xF0)
		bytes[16] = ((self.chorus_flanger_feedback >> 4) & 0xF) | ((self.chorus_flanger_pre_leq_gain << 4) & 0xF0)
		bytes[17] = ((self.chorus_flanger_pre_leq_gain >> 4) & 0x1) | ((self.chorus_flanger_pre_heq_gain & 0x1F) << 1)
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX154 Exciter // Chorus/Flanger object'''
		json = []
		json.append(('name','FX154'))
		json.append(('description','Exciter // Chorus/Flanger'))
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		json.append(['chorus_flanger_input_trim',self.chorus_flanger_input_trim])
		json.append(['chorus_flanger_pan',self.chorus_flanger_pan])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_input_trim(self):
		'''(Chorus/Flanger) Input Trim'''
		return self._chorus_flanger_input_trim

	@chorus_flanger_input_trim.setter
	def chorus_flanger_input_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pan(self):
		'''(Chorus/Flanger) Pan'''
		return self._chorus_flanger_pan

	@chorus_flanger_pan.setter
	def chorus_flanger_pan(self,value):
		if 0 <= value <= 127:
			self._chorus_flanger_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

