#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx155.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX155(object):
	category = '''Mono//Mono'''
	description = '''Exciter // Phaser'''
	short_description = '''Exciter // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX155 Exciter // Phaser object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.exciter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.exciter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.exciter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = (bytes[8] >> 1) & 0x7F
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = bytes[10] & 0x7F
		data = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.phaser_lfo_waveform = (bytes[12] >> 1) & 0x1
		self.phaser_lfo_frequency = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x3) << 6)
		self.phaser_manual = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		self.phaser_depth = (bytes[14] >> 1) & 0x7F
		data = bytes[15] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX155 Exciter // Phaser object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.exciter_wet_dry & 0x7F) << 1
		bytes[1] = (self.exciter_wet_dry_mod_source & 0x3F) | ((self.exciter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.exciter_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.exciter_input_trim << 6) & 0xC0)
		bytes[3] = ((self.exciter_input_trim >> 2) & 0x1F) | ((self.exciter_pan << 5) & 0xE0)
		bytes[4] = ((self.exciter_pan >> 3) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.phaser_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.phaser_wet_dry_mod_source >> 4) & 0x3) | ((self.phaser_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = (self.phaser_wet_dry_mod_intensity >> 6) & 0x3
		bytes[8] = (self.phaser_pan & 0x7F) << 1
		bytes[9] = self.exciter_blend & 0xFF
		bytes[10] = (self.exciter_emphasis_frequency & 0x7F) | ((self.exciter_pre_leq_gain << 7) & 0x80)
		bytes[11] = ((self.exciter_pre_leq_gain >> 1) & 0xF) | ((self.exciter_pre_heq_gain << 4) & 0xF0)
		bytes[12] = ((self.exciter_pre_heq_gain >> 4) & 0x1) | ((self.phaser_lfo_waveform & 0x1) << 1) | ((self.phaser_lfo_frequency << 2) & 0xFC)
		bytes[13] = ((self.phaser_lfo_frequency >> 6) & 0x3) | ((self.phaser_manual << 2) & 0xFC)
		bytes[14] = ((self.phaser_manual >> 6) & 0x1) | ((self.phaser_depth & 0x7F) << 1)
		bytes[15] = self.phaser_resonance & 0xFF
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX155 Exciter // Phaser object'''
		json = []
		json.append(('name','FX155'))
		json.append(('description','Exciter // Phaser'))
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

