#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx156.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX156(object):
	category = '''Mono//Mono'''
	description = '''Exciter // Multitap BPM Delay'''
	short_description = '''Exciter // BPM Dl'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX156 Exciter // Multitap BPM Delay object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.exciter_wet_dry = (bytes[0] >> 1) & 0x7F
		self.exciter_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.exciter_wet_dry_mod_intensity = data
		self.exciter_input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.exciter_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.multitap_bpm_delay_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.multitap_bpm_delay_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_wet_dry_mod_intensity = data
		self.multitap_bpm_delay_pan = bytes[8] & 0x7F
		data = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.exciter_blend = data
		self.exciter_emphasis_frequency = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		data = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_leq_gain = data
		data = (bytes[11] >> 3) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.exciter_pre_heq_gain = data
		self.multitap_bpm_delay_bpm = (bytes[12] & 0xFF) | ((bytes[13] & 0x7F) << 8)
		self.multitap_bpm_delay_tap1_base_note = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x7) << 1)
		self.multitap_bpm_delay_tap1_times = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x1) << 5)
		self.multitap_bpm_delay_tap2_base_note = (bytes[15] >> 1) & 0xF
		self.multitap_bpm_delay_tap2_times = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0x7) << 3)
		self.multitap_bpm_delay_tap1_level = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x3) << 5)
		data = ((bytes[17] & 0xFC) >> 2) | ((bytes[18] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_tap2_feedback = data
		self.multitap_bpm_delay_high_damp = ((bytes[18] & 0xFC) >> 2) | ((bytes[19] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX156 Exciter // Multitap BPM Delay object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.exciter_wet_dry & 0x7F) << 1
		bytes[1] = (self.exciter_wet_dry_mod_source & 0x3F) | ((self.exciter_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.exciter_wet_dry_mod_intensity >> 2) & 0x3F) | ((self.exciter_input_trim << 6) & 0xC0)
		bytes[3] = ((self.exciter_input_trim >> 2) & 0x1F) | ((self.exciter_pan << 5) & 0xE0)
		bytes[4] = ((self.exciter_pan >> 3) & 0xF) | ((self.multitap_bpm_delay_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.multitap_bpm_delay_wet_dry >> 4) & 0x7) | ((self.multitap_bpm_delay_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.multitap_bpm_delay_wet_dry_mod_source >> 5) & 0x1) | ((self.multitap_bpm_delay_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.multitap_bpm_delay_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.multitap_bpm_delay_pan & 0x7F) | ((self.exciter_blend << 7) & 0x80)
		bytes[9] = ((self.exciter_blend >> 1) & 0x7F) | ((self.exciter_emphasis_frequency << 7) & 0x80)
		bytes[10] = ((self.exciter_emphasis_frequency >> 1) & 0x3F) | ((self.exciter_pre_leq_gain << 6) & 0xC0)
		bytes[11] = ((self.exciter_pre_leq_gain >> 2) & 0x7) | ((self.exciter_pre_heq_gain & 0x1F) << 3)
		bytes[12] = self.multitap_bpm_delay_bpm & 0xFF
		bytes[13] = ((self.multitap_bpm_delay_bpm >> 8) & 0x7F) | ((self.multitap_bpm_delay_tap1_base_note << 7) & 0x80)
		bytes[14] = ((self.multitap_bpm_delay_tap1_base_note >> 1) & 0x7) | ((self.multitap_bpm_delay_tap1_times << 3) & 0xF8)
		bytes[15] = ((self.multitap_bpm_delay_tap1_times >> 5) & 0x1) | ((self.multitap_bpm_delay_tap2_base_note & 0xF) << 1) | ((self.multitap_bpm_delay_tap2_times << 5) & 0xE0)
		bytes[16] = ((self.multitap_bpm_delay_tap2_times >> 3) & 0x7) | ((self.multitap_bpm_delay_tap1_level << 3) & 0xF8)
		bytes[17] = ((self.multitap_bpm_delay_tap1_level >> 5) & 0x3) | ((self.multitap_bpm_delay_tap2_feedback << 2) & 0xFC)
		bytes[18] = ((self.multitap_bpm_delay_tap2_feedback >> 6) & 0x3) | ((self.multitap_bpm_delay_high_damp << 2) & 0xFC)
		bytes[19] = (self.multitap_bpm_delay_high_damp >> 6) & 0x1
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX156 Exciter // Multitap BPM Delay object'''
		json = []
		json.append(('name','FX156'))
		json.append(('description','Exciter // Multitap BPM Delay'))
		json.append(['exciter_wet_dry',self.exciter_wet_dry])
		json.append(['exciter_wet_dry_mod_source',self.exciter_wet_dry_mod_source])
		json.append(['exciter_wet_dry_mod_intensity',self.exciter_wet_dry_mod_intensity])
		json.append(['exciter_input_trim',self.exciter_input_trim])
		json.append(['exciter_pan',self.exciter_pan])
		json.append(['multitap_bpm_delay_wet_dry',self.multitap_bpm_delay_wet_dry])
		json.append(['multitap_bpm_delay_wet_dry_mod_source',self.multitap_bpm_delay_wet_dry_mod_source])
		json.append(['multitap_bpm_delay_wet_dry_mod_intensity',self.multitap_bpm_delay_wet_dry_mod_intensity])
		json.append(['multitap_bpm_delay_pan',self.multitap_bpm_delay_pan])
		json.append(['exciter_blend',self.exciter_blend])
		json.append(['exciter_emphasis_frequency',self.exciter_emphasis_frequency])
		json.append(['exciter_pre_leq_gain',self.exciter_pre_leq_gain])
		json.append(['exciter_pre_heq_gain',self.exciter_pre_heq_gain])
		json.append(['multitap_bpm_delay_bpm',self.multitap_bpm_delay_bpm])
		json.append(['multitap_bpm_delay_bpm_display',self.multitap_bpm_delay_bpm_display])
		json.append(['multitap_bpm_delay_tap1_base_note',self.multitap_bpm_delay_tap1_base_note])
		json.append(['multitap_bpm_delay_tap1_base_note_display',self.multitap_bpm_delay_tap1_base_note_display])
		json.append(['multitap_bpm_delay_tap1_times',self.multitap_bpm_delay_tap1_times])
		json.append(['multitap_bpm_delay_tap2_base_note',self.multitap_bpm_delay_tap2_base_note])
		json.append(['multitap_bpm_delay_tap2_base_note_display',self.multitap_bpm_delay_tap2_base_note_display])
		json.append(['multitap_bpm_delay_tap2_times',self.multitap_bpm_delay_tap2_times])
		json.append(['multitap_bpm_delay_tap1_level',self.multitap_bpm_delay_tap1_level])
		json.append(['multitap_bpm_delay_tap2_feedback',self.multitap_bpm_delay_tap2_feedback])
		json.append(['multitap_bpm_delay_high_damp',self.multitap_bpm_delay_high_damp])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry(self):
		'''(Exciter) Wet/Dry'''
		return self._exciter_wet_dry

	@exciter_wet_dry.setter
	def exciter_wet_dry(self,value):
		if 0 <= value <= 100:
			self._exciter_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_source(self):
		'''(Exciter) Wet/Dry Mod.Source'''
		return self._exciter_wet_dry_mod_source

	@exciter_wet_dry_mod_source.setter
	def exciter_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._exciter_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_wet_dry_mod_intensity(self):
		'''(Exciter) Wet/Dry Mod.Intensity'''
		return self._exciter_wet_dry_mod_intensity

	@exciter_wet_dry_mod_intensity.setter
	def exciter_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._exciter_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_input_trim(self):
		'''(Exciter) Input Trim'''
		return self._exciter_input_trim

	@exciter_input_trim.setter
	def exciter_input_trim(self,value):
		if 0 <= value <= 100:
			self._exciter_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pan(self):
		'''(Exciter) Pan'''
		return self._exciter_pan

	@exciter_pan.setter
	def exciter_pan(self,value):
		if 0 <= value <= 127:
			self._exciter_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry(self):
		'''(Multitap BPM Delay) Wet/Dry'''
		return self._multitap_bpm_delay_wet_dry

	@multitap_bpm_delay_wet_dry.setter
	def multitap_bpm_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_source(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Source'''
		return self._multitap_bpm_delay_wet_dry_mod_source

	@multitap_bpm_delay_wet_dry_mod_source.setter
	def multitap_bpm_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_bpm_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_intensity(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_bpm_delay_wet_dry_mod_intensity

	@multitap_bpm_delay_wet_dry_mod_intensity.setter
	def multitap_bpm_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_pan(self):
		'''(Multitap BPM Delay) Pan'''
		return self._multitap_bpm_delay_pan

	@multitap_bpm_delay_pan.setter
	def multitap_bpm_delay_pan(self,value):
		if 0 <= value <= 127:
			self._multitap_bpm_delay_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_blend(self):
		'''(Exciter) Blend'''
		return self._exciter_blend

	@exciter_blend.setter
	def exciter_blend(self,value):
		if -100 <= value <= 100:
			self._exciter_blend = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_emphasis_frequency(self):
		'''(Exciter) Emphasis Frequency'''
		return self._exciter_emphasis_frequency

	@exciter_emphasis_frequency.setter
	def exciter_emphasis_frequency(self,value):
		if 0 <= value <= 70:
			self._exciter_emphasis_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 70' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_leq_gain(self):
		'''(Exciter) Pre LEQ Gain'''
		return self._exciter_pre_leq_gain

	@exciter_pre_leq_gain.setter
	def exciter_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def exciter_pre_heq_gain(self):
		'''(Exciter) Pre HEQ Gain'''
		return self._exciter_pre_heq_gain

	@exciter_pre_heq_gain.setter
	def exciter_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._exciter_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_bpm(self):
		'''(Multitap BPM Delay) BPM'''
		return self._multitap_bpm_delay_bpm

	@multitap_bpm_delay_bpm.setter
	def multitap_bpm_delay_bpm(self,value):
		if 3999 <= value <= 30000:
			self._multitap_bpm_delay_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def multitap_bpm_delay_bpm_display(self):
		'''(Multitap BPM Delay) BPM display value'''
		return bpm_decode(self._multitap_bpm_delay_bpm)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_base_note(self):
		'''(Multitap BPM Delay) Tap1 Base Note'''
		return self._multitap_bpm_delay_tap1_base_note

	@multitap_bpm_delay_tap1_base_note.setter
	def multitap_bpm_delay_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap1_base_note_display(self):
		'''(Multitap BPM Delay) Tap1 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_times(self):
		'''(Multitap BPM Delay) Tap1 Times'''
		return self._multitap_bpm_delay_tap1_times

	@multitap_bpm_delay_tap1_times.setter
	def multitap_bpm_delay_tap1_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_base_note(self):
		'''(Multitap BPM Delay) Tap2 Base Note'''
		return self._multitap_bpm_delay_tap2_base_note

	@multitap_bpm_delay_tap2_base_note.setter
	def multitap_bpm_delay_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap2_base_note_display(self):
		'''(Multitap BPM Delay) Tap2 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_times(self):
		'''(Multitap BPM Delay) Tap2 Times'''
		return self._multitap_bpm_delay_tap2_times

	@multitap_bpm_delay_tap2_times.setter
	def multitap_bpm_delay_tap2_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_level(self):
		'''(Multitap BPM Delay) Tap1 Level'''
		return self._multitap_bpm_delay_tap1_level

	@multitap_bpm_delay_tap1_level.setter
	def multitap_bpm_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_feedback(self):
		'''(Multitap BPM Delay) Tap2 Feedback'''
		return self._multitap_bpm_delay_tap2_feedback

	@multitap_bpm_delay_tap2_feedback.setter
	def multitap_bpm_delay_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_high_damp(self):
		'''(Multitap BPM Delay) High Damp'''
		return self._multitap_bpm_delay_high_damp

	@multitap_bpm_delay_high_damp.setter
	def multitap_bpm_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

