#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx157.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX157(object):
	category = '''Mono//Mono'''
	description = '''Overdrive/Hi.Gain // Overdrive/Hi.Gain'''
	short_description = '''OD/HG // OD/HG'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX157 Overdrive/Hi.Gain // Overdrive/Hi.Gain object'''
		bytes = struct.unpack('<27B',raw[:27])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_hi_gain_on = (bytes[8] >> 7) & 0x1
		self.effect1_drive = bytes[9] & 0x7F
		self.effect1_output_level = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x1F) << 1)
		self.effect1_output_level_mod_source = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		data = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.effect1_output_level_mod_intensity = data
		self.effect1_eq_low_cutoff = (bytes[12] >> 2) & 0x3F
		data = bytes[13] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.effect1_eq_low_gain = data
		self.effect1_eq_mid1_cutoff = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xF) << 2)
		self.effect1_eq_mid1_q = (bytes[14] >> 4) & 0xF
		data = bytes[15] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.effect1_eq_mid1_gain = data
		self.effect1_eq_mid2_cutoff = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0xF) << 2)
		self.effect1_eq_mid2_q = (bytes[16] >> 4) & 0xF
		data = bytes[17] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.effect1_eq_mid2_gain = data
		self.effect2_hi_gain_on = (bytes[17] >> 6) & 0x1
		self.effect2_drive = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x3F) << 1)
		self.effect2_output_level = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0xF) << 2)
		self.effect2_output_level_mod_source = ((bytes[19] & 0xF0) >> 4) | ((bytes[20] & 0x3) << 4)
		data = ((bytes[20] & 0xFC) >> 2) | ((bytes[21] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.effect2_output_level_mod_intensity = data
		self.effect2_eq_low_cutoff = (bytes[21] >> 1) & 0x3F
		data = ((bytes[21] & 0x80) >> 7) | ((bytes[22] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.effect2_eq_low_gain = data
		self.effect2_eq_mid1_cutoff = ((bytes[22] & 0xE0) >> 5) | ((bytes[23] & 0x7) << 3)
		self.effect2_eq_mid1_q = (bytes[23] >> 3) & 0xF
		data = ((bytes[23] & 0x80) >> 7) | ((bytes[24] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.effect2_eq_mid1_gain = data
		self.effect2_eq_mid2_cutoff = ((bytes[24] & 0xE0) >> 5) | ((bytes[25] & 0x7) << 3)
		self.effect2_eq_mid2_q = (bytes[25] >> 3) & 0xF
		data = ((bytes[25] & 0x80) >> 7) | ((bytes[26] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.effect2_eq_mid2_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX157 Overdrive/Hi.Gain // Overdrive/Hi.Gain object into raw data'''
		bytes = [0] * 27
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.effect1_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.effect1_pan << 5) & 0xE0
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.effect2_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_hi_gain_on & 0x1) << 7)
		bytes[9] = (self.effect1_drive & 0x7F) | ((self.effect1_output_level << 7) & 0x80)
		bytes[10] = ((self.effect1_output_level >> 1) & 0x1F) | ((self.effect1_output_level_mod_source << 5) & 0xE0)
		bytes[11] = ((self.effect1_output_level_mod_source >> 3) & 0x7) | ((self.effect1_output_level_mod_intensity << 3) & 0xF8)
		bytes[12] = ((self.effect1_output_level_mod_intensity >> 5) & 0x3) | ((self.effect1_eq_low_cutoff & 0x3F) << 2)
		bytes[13] = (self.effect1_eq_low_gain & 0x3F) | ((self.effect1_eq_mid1_cutoff << 6) & 0xC0)
		bytes[14] = ((self.effect1_eq_mid1_cutoff >> 2) & 0xF) | ((self.effect1_eq_mid1_q & 0xF) << 4)
		bytes[15] = (self.effect1_eq_mid1_gain & 0x3F) | ((self.effect1_eq_mid2_cutoff << 6) & 0xC0)
		bytes[16] = ((self.effect1_eq_mid2_cutoff >> 2) & 0xF) | ((self.effect1_eq_mid2_q & 0xF) << 4)
		bytes[17] = (self.effect1_eq_mid2_gain & 0x3F) | ((self.effect2_hi_gain_on & 0x1) << 6) | ((self.effect2_drive << 7) & 0x80)
		bytes[18] = ((self.effect2_drive >> 1) & 0x3F) | ((self.effect2_output_level << 6) & 0xC0)
		bytes[19] = ((self.effect2_output_level >> 2) & 0xF) | ((self.effect2_output_level_mod_source << 4) & 0xF0)
		bytes[20] = ((self.effect2_output_level_mod_source >> 4) & 0x3) | ((self.effect2_output_level_mod_intensity << 2) & 0xFC)
		bytes[21] = ((self.effect2_output_level_mod_intensity >> 6) & 0x1) | ((self.effect2_eq_low_cutoff & 0x3F) << 1) | ((self.effect2_eq_low_gain << 7) & 0x80)
		bytes[22] = ((self.effect2_eq_low_gain >> 1) & 0x1F) | ((self.effect2_eq_mid1_cutoff << 5) & 0xE0)
		bytes[23] = ((self.effect2_eq_mid1_cutoff >> 3) & 0x7) | ((self.effect2_eq_mid1_q & 0xF) << 3) | ((self.effect2_eq_mid1_gain << 7) & 0x80)
		bytes[24] = ((self.effect2_eq_mid1_gain >> 1) & 0x1F) | ((self.effect2_eq_mid2_cutoff << 5) & 0xE0)
		bytes[25] = ((self.effect2_eq_mid2_cutoff >> 3) & 0x7) | ((self.effect2_eq_mid2_q & 0xF) << 3) | ((self.effect2_eq_mid2_gain << 7) & 0x80)
		bytes[26] = (self.effect2_eq_mid2_gain >> 1) & 0x1F
		return struct.pack('<27B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX157 Overdrive/Hi.Gain // Overdrive/Hi.Gain object'''
		json = []
		json.append(('name','FX157'))
		json.append(('description','Overdrive/Hi.Gain // Overdrive/Hi.Gain'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_hi_gain_on',self.effect1_hi_gain_on])
		json.append(['effect1_drive',self.effect1_drive])
		json.append(['effect1_output_level',self.effect1_output_level])
		json.append(['effect1_output_level_mod_source',self.effect1_output_level_mod_source])
		json.append(['effect1_output_level_mod_intensity',self.effect1_output_level_mod_intensity])
		json.append(['effect1_eq_low_cutoff',self.effect1_eq_low_cutoff])
		json.append(['effect1_eq_low_gain',self.effect1_eq_low_gain])
		json.append(['effect1_eq_mid1_cutoff',self.effect1_eq_mid1_cutoff])
		json.append(['effect1_eq_mid1_q',self.effect1_eq_mid1_q])
		json.append(['effect1_eq_mid1_gain',self.effect1_eq_mid1_gain])
		json.append(['effect1_eq_mid2_cutoff',self.effect1_eq_mid2_cutoff])
		json.append(['effect1_eq_mid2_q',self.effect1_eq_mid2_q])
		json.append(['effect1_eq_mid2_gain',self.effect1_eq_mid2_gain])
		json.append(['effect2_hi_gain_on',self.effect2_hi_gain_on])
		json.append(['effect2_drive',self.effect2_drive])
		json.append(['effect2_output_level',self.effect2_output_level])
		json.append(['effect2_output_level_mod_source',self.effect2_output_level_mod_source])
		json.append(['effect2_output_level_mod_intensity',self.effect2_output_level_mod_intensity])
		json.append(['effect2_eq_low_cutoff',self.effect2_eq_low_cutoff])
		json.append(['effect2_eq_low_gain',self.effect2_eq_low_gain])
		json.append(['effect2_eq_mid1_cutoff',self.effect2_eq_mid1_cutoff])
		json.append(['effect2_eq_mid1_q',self.effect2_eq_mid1_q])
		json.append(['effect2_eq_mid1_gain',self.effect2_eq_mid1_gain])
		json.append(['effect2_eq_mid2_cutoff',self.effect2_eq_mid2_cutoff])
		json.append(['effect2_eq_mid2_q',self.effect2_eq_mid2_q])
		json.append(['effect2_eq_mid2_gain',self.effect2_eq_mid2_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_hi_gain_on(self):
		'''(Effect1) Hi-Gain On'''
		return self._effect1_hi_gain_on

	@effect1_hi_gain_on.setter
	def effect1_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._effect1_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_drive(self):
		'''(Effect1) Drive'''
		return self._effect1_drive

	@effect1_drive.setter
	def effect1_drive(self,value):
		if 1 <= value <= 100:
			self._effect1_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_output_level(self):
		'''(Effect1) Output Level'''
		return self._effect1_output_level

	@effect1_output_level.setter
	def effect1_output_level(self,value):
		if 0 <= value <= 50:
			self._effect1_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_output_level_mod_source(self):
		'''(Effect1) Output Level Mod.Source'''
		return self._effect1_output_level_mod_source

	@effect1_output_level_mod_source.setter
	def effect1_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_output_level_mod_intensity(self):
		'''(Effect1) Output Level Mod.Intensity'''
		return self._effect1_output_level_mod_intensity

	@effect1_output_level_mod_intensity.setter
	def effect1_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._effect1_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_low_cutoff(self):
		'''(Effect1) EQ Low Cutoff'''
		return self._effect1_eq_low_cutoff

	@effect1_eq_low_cutoff.setter
	def effect1_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._effect1_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_low_gain(self):
		'''(Effect1) EQ Low Gain'''
		return self._effect1_eq_low_gain

	@effect1_eq_low_gain.setter
	def effect1_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._effect1_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid1_cutoff(self):
		'''(Effect1) EQ Mid1 Cutoff'''
		return self._effect1_eq_mid1_cutoff

	@effect1_eq_mid1_cutoff.setter
	def effect1_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._effect1_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid1_q(self):
		'''(Effect1) EQ Mid1 Q'''
		return self._effect1_eq_mid1_q

	@effect1_eq_mid1_q.setter
	def effect1_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._effect1_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid1_gain(self):
		'''(Effect1) EQ Mid1 Gain'''
		return self._effect1_eq_mid1_gain

	@effect1_eq_mid1_gain.setter
	def effect1_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._effect1_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid2_cutoff(self):
		'''(Effect1) EQ Mid2 Cutoff'''
		return self._effect1_eq_mid2_cutoff

	@effect1_eq_mid2_cutoff.setter
	def effect1_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._effect1_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid2_q(self):
		'''(Effect1) EQ Mid2 Q'''
		return self._effect1_eq_mid2_q

	@effect1_eq_mid2_q.setter
	def effect1_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._effect1_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_eq_mid2_gain(self):
		'''(Effect1) EQ Mid2 Gain'''
		return self._effect1_eq_mid2_gain

	@effect1_eq_mid2_gain.setter
	def effect1_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._effect1_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_hi_gain_on(self):
		'''(Effect2) Hi-Gain On'''
		return self._effect2_hi_gain_on

	@effect2_hi_gain_on.setter
	def effect2_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._effect2_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_drive(self):
		'''(Effect2) Drive'''
		return self._effect2_drive

	@effect2_drive.setter
	def effect2_drive(self,value):
		if 1 <= value <= 100:
			self._effect2_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_output_level(self):
		'''(Effect2) Output Level'''
		return self._effect2_output_level

	@effect2_output_level.setter
	def effect2_output_level(self,value):
		if 0 <= value <= 50:
			self._effect2_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_output_level_mod_source(self):
		'''(Effect2) Output Level Mod.Source'''
		return self._effect2_output_level_mod_source

	@effect2_output_level_mod_source.setter
	def effect2_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_output_level_mod_intensity(self):
		'''(Effect2) Output Level Mod.Intensity'''
		return self._effect2_output_level_mod_intensity

	@effect2_output_level_mod_intensity.setter
	def effect2_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._effect2_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_low_cutoff(self):
		'''(Effect2) EQ Low Cutoff'''
		return self._effect2_eq_low_cutoff

	@effect2_eq_low_cutoff.setter
	def effect2_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._effect2_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_low_gain(self):
		'''(Effect2) EQ Low Gain'''
		return self._effect2_eq_low_gain

	@effect2_eq_low_gain.setter
	def effect2_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._effect2_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid1_cutoff(self):
		'''(Effect2) EQ Mid1 Cutoff'''
		return self._effect2_eq_mid1_cutoff

	@effect2_eq_mid1_cutoff.setter
	def effect2_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._effect2_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid1_q(self):
		'''(Effect2) EQ Mid1 Q'''
		return self._effect2_eq_mid1_q

	@effect2_eq_mid1_q.setter
	def effect2_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._effect2_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid1_gain(self):
		'''(Effect2) EQ Mid1 Gain'''
		return self._effect2_eq_mid1_gain

	@effect2_eq_mid1_gain.setter
	def effect2_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._effect2_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid2_cutoff(self):
		'''(Effect2) EQ Mid2 Cutoff'''
		return self._effect2_eq_mid2_cutoff

	@effect2_eq_mid2_cutoff.setter
	def effect2_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._effect2_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid2_q(self):
		'''(Effect2) EQ Mid2 Q'''
		return self._effect2_eq_mid2_q

	@effect2_eq_mid2_q.setter
	def effect2_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._effect2_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_eq_mid2_gain(self):
		'''(Effect2) EQ Mid2 Gain'''
		return self._effect2_eq_mid2_gain

	@effect2_eq_mid2_gain.setter
	def effect2_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._effect2_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

