#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx160.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX160(object):
	category = '''Mono//Mono'''
	description = '''Overdrive/Hi.Gain // Phaser'''
	short_description = '''OD/HG // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX160 Overdrive/Hi.Gain // Phaser object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.od_hi_gain_wet_dry = (bytes[0] >> 1) & 0x7F
		self.od_hi_gain_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.od_hi_gain_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = (bytes[8] >> 1) & 0x7F
		self.od_hi_gain_hi_gain_on = bytes[9] & 0x1
		self.od_hi_gain_drive = (bytes[9] >> 1) & 0x7F
		self.od_hi_gain_output_level = bytes[10] & 0x3F
		self.od_hi_gain_output_level_mod_source = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0xF) << 2)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x1) << 5)
		data = (bytes[13] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x1F) << 1)
		self.od_hi_gain_eq_mid1_q = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x1) << 3)
		data = (bytes[15] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x1F) << 1)
		self.od_hi_gain_eq_mid2_q = ((bytes[16] & 0xE0) >> 5) | ((bytes[17] & 0x1) << 3)
		data = (bytes[17] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data
		self.phaser_lfo_waveform = (bytes[17] >> 7) & 0x1
		self.phaser_lfo_frequency = bytes[18] & 0xFF
		self.phaser_manual = bytes[19] & 0x7F
		self.phaser_depth = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0x3F) << 1)
		data = ((bytes[20] & 0xC0) >> 6) | ((bytes[21] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX160 Overdrive/Hi.Gain // Phaser object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.od_hi_gain_wet_dry & 0x7F) << 1
		bytes[1] = (self.od_hi_gain_wet_dry_mod_source & 0x3F) | ((self.od_hi_gain_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.od_hi_gain_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.od_hi_gain_pan << 5) & 0xE0
		bytes[4] = ((self.od_hi_gain_pan >> 3) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.phaser_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.phaser_wet_dry_mod_source >> 4) & 0x3) | ((self.phaser_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = (self.phaser_wet_dry_mod_intensity >> 6) & 0x3
		bytes[8] = (self.phaser_pan & 0x7F) << 1
		bytes[9] = (self.od_hi_gain_hi_gain_on & 0x1) | ((self.od_hi_gain_drive & 0x7F) << 1)
		bytes[10] = (self.od_hi_gain_output_level & 0x3F) | ((self.od_hi_gain_output_level_mod_source << 6) & 0xC0)
		bytes[11] = ((self.od_hi_gain_output_level_mod_source >> 2) & 0xF) | ((self.od_hi_gain_output_level_mod_intensity << 4) & 0xF0)
		bytes[12] = ((self.od_hi_gain_output_level_mod_intensity >> 4) & 0x7) | ((self.od_hi_gain_eq_low_cutoff << 3) & 0xF8)
		bytes[13] = ((self.od_hi_gain_eq_low_cutoff >> 5) & 0x1) | ((self.od_hi_gain_eq_low_gain & 0x3F) << 1) | ((self.od_hi_gain_eq_mid1_cutoff << 7) & 0x80)
		bytes[14] = ((self.od_hi_gain_eq_mid1_cutoff >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid1_q << 5) & 0xE0)
		bytes[15] = ((self.od_hi_gain_eq_mid1_q >> 3) & 0x1) | ((self.od_hi_gain_eq_mid1_gain & 0x3F) << 1) | ((self.od_hi_gain_eq_mid2_cutoff << 7) & 0x80)
		bytes[16] = ((self.od_hi_gain_eq_mid2_cutoff >> 1) & 0x1F) | ((self.od_hi_gain_eq_mid2_q << 5) & 0xE0)
		bytes[17] = ((self.od_hi_gain_eq_mid2_q >> 3) & 0x1) | ((self.od_hi_gain_eq_mid2_gain & 0x3F) << 1) | ((self.phaser_lfo_waveform & 0x1) << 7)
		bytes[18] = self.phaser_lfo_frequency & 0xFF
		bytes[19] = (self.phaser_manual & 0x7F) | ((self.phaser_depth << 7) & 0x80)
		bytes[20] = ((self.phaser_depth >> 1) & 0x3F) | ((self.phaser_resonance << 6) & 0xC0)
		bytes[21] = (self.phaser_resonance >> 2) & 0x3F
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX160 Overdrive/Hi.Gain // Phaser object'''
		json = []
		json.append(('name','FX160'))
		json.append(('description','Overdrive/Hi.Gain // Phaser'))
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		json.append(['od_hi_gain_pan',self.od_hi_gain_pan])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_pan(self):
		'''(OD/Hi-Gain) Pan'''
		return self._od_hi_gain_pan

	@od_hi_gain_pan.setter
	def od_hi_gain_pan(self,value):
		if 0 <= value <= 127:
			self._od_hi_gain_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

