#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx162.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX162(object):
	category = '''Mono//Mono'''
	description = '''Wah // Wah'''
	short_description = '''Wah // Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX162 Wah // Wah object'''
		bytes = struct.unpack('<19B',raw[:19])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_frequency_bottom = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.effect1_frequency_top = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.effect1_sweep_mode = (bytes[10] >> 5) & 0x3
		self.effect1_sweep_mode_mod_source = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x1F) << 1)
		self.effect1_lfo_frequency = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x1F) << 3)
		self.effect1_resonance = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		self.effect1_low_pass_filter = (bytes[13] >> 4) & 0x1
		self.effect2_frequency_bottom = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0xF) << 3)
		self.effect2_frequency_top = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)
		self.effect2_sweep_mode = (bytes[15] >> 3) & 0x3
		self.effect2_sweep_mode_mod_source = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0x7) << 3)
		self.effect2_lfo_frequency = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x7) << 5)
		self.effect2_resonance = ((bytes[17] & 0xF8) >> 3) | ((bytes[18] & 0x3) << 5)
		self.effect2_low_pass_filter = (bytes[18] >> 2) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX162 Wah // Wah object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.effect1_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.effect1_pan << 5) & 0xE0
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.effect2_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_frequency_bottom << 7) & 0x80)
		bytes[9] = ((self.effect1_frequency_bottom >> 1) & 0x3F) | ((self.effect1_frequency_top << 6) & 0xC0)
		bytes[10] = ((self.effect1_frequency_top >> 2) & 0x1F) | ((self.effect1_sweep_mode & 0x3) << 5) | ((self.effect1_sweep_mode_mod_source << 7) & 0x80)
		bytes[11] = ((self.effect1_sweep_mode_mod_source >> 1) & 0x1F) | ((self.effect1_lfo_frequency << 5) & 0xE0)
		bytes[12] = ((self.effect1_lfo_frequency >> 3) & 0x1F) | ((self.effect1_resonance << 5) & 0xE0)
		bytes[13] = ((self.effect1_resonance >> 3) & 0xF) | ((self.effect1_low_pass_filter & 0x1) << 4) | ((self.effect2_frequency_bottom << 5) & 0xE0)
		bytes[14] = ((self.effect2_frequency_bottom >> 3) & 0xF) | ((self.effect2_frequency_top << 4) & 0xF0)
		bytes[15] = ((self.effect2_frequency_top >> 4) & 0x7) | ((self.effect2_sweep_mode & 0x3) << 3) | ((self.effect2_sweep_mode_mod_source << 5) & 0xE0)
		bytes[16] = ((self.effect2_sweep_mode_mod_source >> 3) & 0x7) | ((self.effect2_lfo_frequency << 3) & 0xF8)
		bytes[17] = ((self.effect2_lfo_frequency >> 5) & 0x7) | ((self.effect2_resonance << 3) & 0xF8)
		bytes[18] = ((self.effect2_resonance >> 5) & 0x3) | ((self.effect2_low_pass_filter & 0x1) << 2)
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX162 Wah // Wah object'''
		json = []
		json.append(('name','FX162'))
		json.append(('description','Wah // Wah'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_frequency_bottom',self.effect1_frequency_bottom])
		json.append(['effect1_frequency_top',self.effect1_frequency_top])
		json.append(['effect1_sweep_mode',self.effect1_sweep_mode])
		json.append(['effect1_sweep_mode_mod_source',self.effect1_sweep_mode_mod_source])
		json.append(['effect1_lfo_frequency',self.effect1_lfo_frequency])
		json.append(['effect1_resonance',self.effect1_resonance])
		json.append(['effect1_low_pass_filter',self.effect1_low_pass_filter])
		json.append(['effect2_frequency_bottom',self.effect2_frequency_bottom])
		json.append(['effect2_frequency_top',self.effect2_frequency_top])
		json.append(['effect2_sweep_mode',self.effect2_sweep_mode])
		json.append(['effect2_sweep_mode_mod_source',self.effect2_sweep_mode_mod_source])
		json.append(['effect2_lfo_frequency',self.effect2_lfo_frequency])
		json.append(['effect2_resonance',self.effect2_resonance])
		json.append(['effect2_low_pass_filter',self.effect2_low_pass_filter])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_frequency_bottom(self):
		'''(Effect1) Frequency Bottom'''
		return self._effect1_frequency_bottom

	@effect1_frequency_bottom.setter
	def effect1_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._effect1_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_frequency_top(self):
		'''(Effect1) Frequency Top'''
		return self._effect1_frequency_top

	@effect1_frequency_top.setter
	def effect1_frequency_top(self,value):
		if 0 <= value <= 100:
			self._effect1_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_sweep_mode(self):
		'''(Effect1) Sweep Mode'''
		return self._effect1_sweep_mode

	@effect1_sweep_mode.setter
	def effect1_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._effect1_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_sweep_mode_mod_source(self):
		'''(Effect1) Sweep Mode Mod.Source'''
		return self._effect1_sweep_mode_mod_source

	@effect1_sweep_mode_mod_source.setter
	def effect1_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_lfo_frequency(self):
		'''(Effect1) LFO Frequency'''
		return self._effect1_lfo_frequency

	@effect1_lfo_frequency.setter
	def effect1_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect1_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_resonance(self):
		'''(Effect1) Resonance'''
		return self._effect1_resonance

	@effect1_resonance.setter
	def effect1_resonance(self,value):
		if 0 <= value <= 100:
			self._effect1_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_low_pass_filter(self):
		'''(Effect1) Low Pass Filter'''
		return self._effect1_low_pass_filter

	@effect1_low_pass_filter.setter
	def effect1_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._effect1_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_frequency_bottom(self):
		'''(Effect2) Frequency Bottom'''
		return self._effect2_frequency_bottom

	@effect2_frequency_bottom.setter
	def effect2_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._effect2_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_frequency_top(self):
		'''(Effect2) Frequency Top'''
		return self._effect2_frequency_top

	@effect2_frequency_top.setter
	def effect2_frequency_top(self,value):
		if 0 <= value <= 100:
			self._effect2_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_sweep_mode(self):
		'''(Effect2) Sweep Mode'''
		return self._effect2_sweep_mode

	@effect2_sweep_mode.setter
	def effect2_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._effect2_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_sweep_mode_mod_source(self):
		'''(Effect2) Sweep Mode Mod.Source'''
		return self._effect2_sweep_mode_mod_source

	@effect2_sweep_mode_mod_source.setter
	def effect2_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_lfo_frequency(self):
		'''(Effect2) LFO Frequency'''
		return self._effect2_lfo_frequency

	@effect2_lfo_frequency.setter
	def effect2_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect2_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_resonance(self):
		'''(Effect2) Resonance'''
		return self._effect2_resonance

	@effect2_resonance.setter
	def effect2_resonance(self,value):
		if 0 <= value <= 100:
			self._effect2_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_low_pass_filter(self):
		'''(Effect2) Low Pass Filter'''
		return self._effect2_low_pass_filter

	@effect2_low_pass_filter.setter
	def effect2_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._effect2_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

