#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx163.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX163(object):
	category = '''Mono//Mono'''
	description = '''Wah // Chorus/Flanger'''
	short_description = '''Wah // Cho/Flng'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX163 Wah // Chorus/Flanger object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wah_wet_dry = (bytes[0] >> 1) & 0x7F
		self.wah_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.wah_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_input_trim = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		self.chorus_flanger_pan = (bytes[8] >> 1) & 0x7F
		self.wah_frequency_bottom = bytes[9] & 0x7F
		self.wah_frequency_top = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		self.wah_sweep_mode = (bytes[10] >> 6) & 0x3
		self.wah_sweep_mode_mod_source = bytes[11] & 0x3F
		self.wah_lfo_frequency = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0x3F) << 2)
		self.wah_resonance = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		self.wah_low_pass_filter = (bytes[13] >> 5) & 0x1
		self.chorus_flanger_pre_delay = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xFF) << 2)
		self.chorus_flanger_lfo_waveform = (bytes[15] >> 1) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x3) << 6)
		self.chorus_flanger_depth = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x1) << 6)
		data = ((bytes[17] & 0xFE) >> 1) | ((bytes[18] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = (bytes[18] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX163 Wah // Chorus/Flanger object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wah_wet_dry & 0x7F) << 1
		bytes[1] = (self.wah_wet_dry_mod_source & 0x3F) | ((self.wah_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wah_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.wah_pan << 5) & 0xE0
		bytes[4] = ((self.wah_pan >> 3) & 0xF) | ((self.chorus_flanger_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.chorus_flanger_wet_dry >> 4) & 0xF) | ((self.chorus_flanger_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.chorus_flanger_wet_dry_mod_source >> 4) & 0x3) | ((self.chorus_flanger_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = ((self.chorus_flanger_wet_dry_mod_intensity >> 6) & 0x3) | ((self.chorus_flanger_input_trim << 2) & 0xFC)
		bytes[8] = ((self.chorus_flanger_input_trim >> 6) & 0x1) | ((self.chorus_flanger_pan & 0x7F) << 1)
		bytes[9] = (self.wah_frequency_bottom & 0x7F) | ((self.wah_frequency_top << 7) & 0x80)
		bytes[10] = ((self.wah_frequency_top >> 1) & 0x3F) | ((self.wah_sweep_mode & 0x3) << 6)
		bytes[11] = (self.wah_sweep_mode_mod_source & 0x3F) | ((self.wah_lfo_frequency << 6) & 0xC0)
		bytes[12] = ((self.wah_lfo_frequency >> 2) & 0x3F) | ((self.wah_resonance << 6) & 0xC0)
		bytes[13] = ((self.wah_resonance >> 2) & 0x1F) | ((self.wah_low_pass_filter & 0x1) << 5) | ((self.chorus_flanger_pre_delay << 6) & 0xC0)
		bytes[14] = (self.chorus_flanger_pre_delay >> 2) & 0xFF
		bytes[15] = ((self.chorus_flanger_lfo_waveform & 0x1) << 1) | ((self.chorus_flanger_lfo_frequency << 2) & 0xFC)
		bytes[16] = ((self.chorus_flanger_lfo_frequency >> 6) & 0x3) | ((self.chorus_flanger_depth << 2) & 0xFC)
		bytes[17] = ((self.chorus_flanger_depth >> 6) & 0x1) | ((self.chorus_flanger_feedback << 1) & 0xFE)
		bytes[18] = ((self.chorus_flanger_feedback >> 7) & 0x1) | ((self.chorus_flanger_pre_leq_gain & 0x1F) << 1) | ((self.chorus_flanger_pre_heq_gain << 6) & 0xC0)
		bytes[19] = (self.chorus_flanger_pre_heq_gain >> 2) & 0x7
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX163 Wah // Chorus/Flanger object'''
		json = []
		json.append(('name','FX163'))
		json.append(('description','Wah // Chorus/Flanger'))
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		json.append(['wah_pan',self.wah_pan])
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		json.append(['chorus_flanger_input_trim',self.chorus_flanger_input_trim])
		json.append(['chorus_flanger_pan',self.chorus_flanger_pan])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_pan(self):
		'''(Wah) Pan'''
		return self._wah_pan

	@wah_pan.setter
	def wah_pan(self,value):
		if 0 <= value <= 127:
			self._wah_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_input_trim(self):
		'''(Chorus/Flanger) Input Trim'''
		return self._chorus_flanger_input_trim

	@chorus_flanger_input_trim.setter
	def chorus_flanger_input_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pan(self):
		'''(Chorus/Flanger) Pan'''
		return self._chorus_flanger_pan

	@chorus_flanger_pan.setter
	def chorus_flanger_pan(self,value):
		if 0 <= value <= 127:
			self._chorus_flanger_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

