#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx164.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX164(object):
	category = '''Mono//Mono'''
	description = '''Wah // Phaser'''
	short_description = '''Wah // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX164 Wah // Phaser object'''
		bytes = struct.unpack('<18B',raw[:18])
		self.wah_wet_dry = (bytes[0] >> 1) & 0x7F
		self.wah_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.wah_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = (bytes[8] >> 1) & 0x7F
		self.wah_frequency_bottom = bytes[9] & 0x7F
		self.wah_frequency_top = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		self.wah_sweep_mode = (bytes[10] >> 6) & 0x3
		self.wah_sweep_mode_mod_source = bytes[11] & 0x3F
		self.wah_lfo_frequency = ((bytes[11] & 0xC0) >> 6) | ((bytes[12] & 0x3F) << 2)
		self.wah_resonance = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1F) << 2)
		self.wah_low_pass_filter = (bytes[13] >> 5) & 0x1
		self.phaser_lfo_waveform = (bytes[13] >> 6) & 0x1
		self.phaser_lfo_frequency = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x7F) << 1)
		self.phaser_manual = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x3F) << 1)
		self.phaser_depth = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0x1F) << 2)
		data = ((bytes[16] & 0xE0) >> 5) | ((bytes[17] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX164 Wah // Phaser object into raw data'''
		bytes = [0] * 18
		bytes[0] = (self.wah_wet_dry & 0x7F) << 1
		bytes[1] = (self.wah_wet_dry_mod_source & 0x3F) | ((self.wah_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wah_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.wah_pan << 5) & 0xE0
		bytes[4] = ((self.wah_pan >> 3) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.phaser_wet_dry >> 4) & 0xF) | ((self.phaser_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.phaser_wet_dry_mod_source >> 4) & 0x3) | ((self.phaser_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = (self.phaser_wet_dry_mod_intensity >> 6) & 0x3
		bytes[8] = (self.phaser_pan & 0x7F) << 1
		bytes[9] = (self.wah_frequency_bottom & 0x7F) | ((self.wah_frequency_top << 7) & 0x80)
		bytes[10] = ((self.wah_frequency_top >> 1) & 0x3F) | ((self.wah_sweep_mode & 0x3) << 6)
		bytes[11] = (self.wah_sweep_mode_mod_source & 0x3F) | ((self.wah_lfo_frequency << 6) & 0xC0)
		bytes[12] = ((self.wah_lfo_frequency >> 2) & 0x3F) | ((self.wah_resonance << 6) & 0xC0)
		bytes[13] = ((self.wah_resonance >> 2) & 0x1F) | ((self.wah_low_pass_filter & 0x1) << 5) | ((self.phaser_lfo_waveform & 0x1) << 6) | ((self.phaser_lfo_frequency << 7) & 0x80)
		bytes[14] = ((self.phaser_lfo_frequency >> 1) & 0x7F) | ((self.phaser_manual << 7) & 0x80)
		bytes[15] = ((self.phaser_manual >> 1) & 0x3F) | ((self.phaser_depth << 6) & 0xC0)
		bytes[16] = ((self.phaser_depth >> 2) & 0x1F) | ((self.phaser_resonance << 5) & 0xE0)
		bytes[17] = (self.phaser_resonance >> 3) & 0x1F
		return struct.pack('<18B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX164 Wah // Phaser object'''
		json = []
		json.append(('name','FX164'))
		json.append(('description','Wah // Phaser'))
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		json.append(['wah_pan',self.wah_pan])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_pan(self):
		'''(Wah) Pan'''
		return self._wah_pan

	@wah_pan.setter
	def wah_pan(self,value):
		if 0 <= value <= 127:
			self._wah_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

