#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx165.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX165(object):
	category = '''Mono//Mono'''
	description = '''Wah // Multitap BPM Delay'''
	short_description = '''Wah // BPM Dly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX165 Wah // Multitap BPM Delay object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.wah_wet_dry = (bytes[0] >> 1) & 0x7F
		self.wah_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wah_wet_dry_mod_intensity = data
		self.wah_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.multitap_bpm_delay_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.multitap_bpm_delay_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_wet_dry_mod_intensity = data
		self.multitap_bpm_delay_pan = bytes[8] & 0x7F
		self.wah_frequency_bottom = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)
		self.wah_frequency_top = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.wah_sweep_mode = (bytes[10] >> 5) & 0x3
		self.wah_sweep_mode_mod_source = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x1F) << 1)
		self.wah_lfo_frequency = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x1F) << 3)
		self.wah_resonance = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0xF) << 3)
		self.wah_low_pass_filter = (bytes[13] >> 4) & 0x1
		self.multitap_bpm_delay_bpm = ((bytes[13] & 0xE0) >> 5) | (bytes[14] << 3) | ((bytes[15] & 0xF) << 11)
		self.multitap_bpm_delay_tap1_base_note = (bytes[15] >> 4) & 0xF
		self.multitap_bpm_delay_tap1_times = bytes[16] & 0x3F
		self.multitap_bpm_delay_tap2_base_note = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x3) << 2)
		self.multitap_bpm_delay_tap2_times = (bytes[17] >> 2) & 0x3F
		self.multitap_bpm_delay_tap1_level = bytes[18] & 0x7F
		data = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_tap2_feedback = data
		self.multitap_bpm_delay_high_damp = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0x3F) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX165 Wah // Multitap BPM Delay object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.wah_wet_dry & 0x7F) << 1
		bytes[1] = (self.wah_wet_dry_mod_source & 0x3F) | ((self.wah_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wah_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.wah_pan << 5) & 0xE0
		bytes[4] = ((self.wah_pan >> 3) & 0xF) | ((self.multitap_bpm_delay_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.multitap_bpm_delay_wet_dry >> 4) & 0x7) | ((self.multitap_bpm_delay_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.multitap_bpm_delay_wet_dry_mod_source >> 5) & 0x1) | ((self.multitap_bpm_delay_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.multitap_bpm_delay_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.multitap_bpm_delay_pan & 0x7F) | ((self.wah_frequency_bottom << 7) & 0x80)
		bytes[9] = ((self.wah_frequency_bottom >> 1) & 0x3F) | ((self.wah_frequency_top << 6) & 0xC0)
		bytes[10] = ((self.wah_frequency_top >> 2) & 0x1F) | ((self.wah_sweep_mode & 0x3) << 5) | ((self.wah_sweep_mode_mod_source << 7) & 0x80)
		bytes[11] = ((self.wah_sweep_mode_mod_source >> 1) & 0x1F) | ((self.wah_lfo_frequency << 5) & 0xE0)
		bytes[12] = ((self.wah_lfo_frequency >> 3) & 0x1F) | ((self.wah_resonance << 5) & 0xE0)
		bytes[13] = ((self.wah_resonance >> 3) & 0xF) | ((self.wah_low_pass_filter & 0x1) << 4) | ((self.multitap_bpm_delay_bpm << 5) & 0xE0)
		bytes[14] = (self.multitap_bpm_delay_bpm >> 3) & 0xFF
		bytes[15] = ((self.multitap_bpm_delay_bpm >> 15) & 0xB) | ((self.multitap_bpm_delay_tap1_base_note & 0xF) << 4)
		bytes[16] = (self.multitap_bpm_delay_tap1_times & 0x3F) | ((self.multitap_bpm_delay_tap2_base_note << 6) & 0xC0)
		bytes[17] = ((self.multitap_bpm_delay_tap2_base_note >> 2) & 0x3) | ((self.multitap_bpm_delay_tap2_times & 0x3F) << 2)
		bytes[18] = (self.multitap_bpm_delay_tap1_level & 0x7F) | ((self.multitap_bpm_delay_tap2_feedback << 7) & 0x80)
		bytes[19] = ((self.multitap_bpm_delay_tap2_feedback >> 1) & 0x7F) | ((self.multitap_bpm_delay_high_damp << 7) & 0x80)
		bytes[20] = (self.multitap_bpm_delay_high_damp >> 1) & 0x3F
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX165 Wah // Multitap BPM Delay object'''
		json = []
		json.append(('name','FX165'))
		json.append(('description','Wah // Multitap BPM Delay'))
		json.append(['wah_wet_dry',self.wah_wet_dry])
		json.append(['wah_wet_dry_mod_source',self.wah_wet_dry_mod_source])
		json.append(['wah_wet_dry_mod_intensity',self.wah_wet_dry_mod_intensity])
		json.append(['wah_pan',self.wah_pan])
		json.append(['multitap_bpm_delay_wet_dry',self.multitap_bpm_delay_wet_dry])
		json.append(['multitap_bpm_delay_wet_dry_mod_source',self.multitap_bpm_delay_wet_dry_mod_source])
		json.append(['multitap_bpm_delay_wet_dry_mod_intensity',self.multitap_bpm_delay_wet_dry_mod_intensity])
		json.append(['multitap_bpm_delay_pan',self.multitap_bpm_delay_pan])
		json.append(['wah_frequency_bottom',self.wah_frequency_bottom])
		json.append(['wah_frequency_top',self.wah_frequency_top])
		json.append(['wah_sweep_mode',self.wah_sweep_mode])
		json.append(['wah_sweep_mode_mod_source',self.wah_sweep_mode_mod_source])
		json.append(['wah_lfo_frequency',self.wah_lfo_frequency])
		json.append(['wah_resonance',self.wah_resonance])
		json.append(['wah_low_pass_filter',self.wah_low_pass_filter])
		json.append(['multitap_bpm_delay_bpm',self.multitap_bpm_delay_bpm])
		json.append(['multitap_bpm_delay_bpm_display',self.multitap_bpm_delay_bpm_display])
		json.append(['multitap_bpm_delay_tap1_base_note',self.multitap_bpm_delay_tap1_base_note])
		json.append(['multitap_bpm_delay_tap1_base_note_display',self.multitap_bpm_delay_tap1_base_note_display])
		json.append(['multitap_bpm_delay_tap1_times',self.multitap_bpm_delay_tap1_times])
		json.append(['multitap_bpm_delay_tap2_base_note',self.multitap_bpm_delay_tap2_base_note])
		json.append(['multitap_bpm_delay_tap2_base_note_display',self.multitap_bpm_delay_tap2_base_note_display])
		json.append(['multitap_bpm_delay_tap2_times',self.multitap_bpm_delay_tap2_times])
		json.append(['multitap_bpm_delay_tap1_level',self.multitap_bpm_delay_tap1_level])
		json.append(['multitap_bpm_delay_tap2_feedback',self.multitap_bpm_delay_tap2_feedback])
		json.append(['multitap_bpm_delay_high_damp',self.multitap_bpm_delay_high_damp])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry(self):
		'''(Wah) Wet/Dry'''
		return self._wah_wet_dry

	@wah_wet_dry.setter
	def wah_wet_dry(self,value):
		if 0 <= value <= 100:
			self._wah_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_source(self):
		'''(Wah) Wet/Dry Mod.Source'''
		return self._wah_wet_dry_mod_source

	@wah_wet_dry_mod_source.setter
	def wah_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_wet_dry_mod_intensity(self):
		'''(Wah) Wet/Dry Mod.Intensity'''
		return self._wah_wet_dry_mod_intensity

	@wah_wet_dry_mod_intensity.setter
	def wah_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wah_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_pan(self):
		'''(Wah) Pan'''
		return self._wah_pan

	@wah_pan.setter
	def wah_pan(self,value):
		if 0 <= value <= 127:
			self._wah_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry(self):
		'''(Multitap BPM Delay) Wet/Dry'''
		return self._multitap_bpm_delay_wet_dry

	@multitap_bpm_delay_wet_dry.setter
	def multitap_bpm_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_source(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Source'''
		return self._multitap_bpm_delay_wet_dry_mod_source

	@multitap_bpm_delay_wet_dry_mod_source.setter
	def multitap_bpm_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_bpm_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_intensity(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_bpm_delay_wet_dry_mod_intensity

	@multitap_bpm_delay_wet_dry_mod_intensity.setter
	def multitap_bpm_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_pan(self):
		'''(Multitap BPM Delay) Pan'''
		return self._multitap_bpm_delay_pan

	@multitap_bpm_delay_pan.setter
	def multitap_bpm_delay_pan(self,value):
		if 0 <= value <= 127:
			self._multitap_bpm_delay_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_bottom(self):
		'''(Wah) Frequency Bottom'''
		return self._wah_frequency_bottom

	@wah_frequency_bottom.setter
	def wah_frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_frequency_top(self):
		'''(Wah) Frequency Top'''
		return self._wah_frequency_top

	@wah_frequency_top.setter
	def wah_frequency_top(self,value):
		if 0 <= value <= 100:
			self._wah_frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode(self):
		'''(Wah) Sweep Mode'''
		return self._wah_sweep_mode

	@wah_sweep_mode.setter
	def wah_sweep_mode(self,value):
		if 0 <= value <= 2:
			self._wah_sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_sweep_mode_mod_source(self):
		'''(Wah) Sweep Mode Mod.Source'''
		return self._wah_sweep_mode_mod_source

	@wah_sweep_mode_mod_source.setter
	def wah_sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._wah_sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_lfo_frequency(self):
		'''(Wah) LFO Frequency'''
		return self._wah_lfo_frequency

	@wah_lfo_frequency.setter
	def wah_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._wah_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_resonance(self):
		'''(Wah) Resonance'''
		return self._wah_resonance

	@wah_resonance.setter
	def wah_resonance(self,value):
		if 0 <= value <= 100:
			self._wah_resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wah_low_pass_filter(self):
		'''(Wah) Low Pass Filter'''
		return self._wah_low_pass_filter

	@wah_low_pass_filter.setter
	def wah_low_pass_filter(self,value):
		if 0 <= value <= 1:
			self._wah_low_pass_filter = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_bpm(self):
		'''(Multitap BPM Delay) BPM'''
		return self._multitap_bpm_delay_bpm

	@multitap_bpm_delay_bpm.setter
	def multitap_bpm_delay_bpm(self,value):
		if 3999 <= value <= 30000:
			self._multitap_bpm_delay_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def multitap_bpm_delay_bpm_display(self):
		'''(Multitap BPM Delay) BPM display value'''
		return bpm_decode(self._multitap_bpm_delay_bpm)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_base_note(self):
		'''(Multitap BPM Delay) Tap1 Base Note'''
		return self._multitap_bpm_delay_tap1_base_note

	@multitap_bpm_delay_tap1_base_note.setter
	def multitap_bpm_delay_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap1_base_note_display(self):
		'''(Multitap BPM Delay) Tap1 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_times(self):
		'''(Multitap BPM Delay) Tap1 Times'''
		return self._multitap_bpm_delay_tap1_times

	@multitap_bpm_delay_tap1_times.setter
	def multitap_bpm_delay_tap1_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_base_note(self):
		'''(Multitap BPM Delay) Tap2 Base Note'''
		return self._multitap_bpm_delay_tap2_base_note

	@multitap_bpm_delay_tap2_base_note.setter
	def multitap_bpm_delay_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap2_base_note_display(self):
		'''(Multitap BPM Delay) Tap2 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_times(self):
		'''(Multitap BPM Delay) Tap2 Times'''
		return self._multitap_bpm_delay_tap2_times

	@multitap_bpm_delay_tap2_times.setter
	def multitap_bpm_delay_tap2_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_level(self):
		'''(Multitap BPM Delay) Tap1 Level'''
		return self._multitap_bpm_delay_tap1_level

	@multitap_bpm_delay_tap1_level.setter
	def multitap_bpm_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_feedback(self):
		'''(Multitap BPM Delay) Tap2 Feedback'''
		return self._multitap_bpm_delay_tap2_feedback

	@multitap_bpm_delay_tap2_feedback.setter
	def multitap_bpm_delay_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_high_damp(self):
		'''(Multitap BPM Delay) High Damp'''
		return self._multitap_bpm_delay_high_damp

	@multitap_bpm_delay_high_damp.setter
	def multitap_bpm_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

