#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx166.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX166(object):
	category = '''Mono//Mono'''
	description = '''Chorus/Flanger // Chorus/Flanger'''
	short_description = '''Cho/Fl // Cho/Fl'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX166 Chorus/Flanger // Chorus/Flanger object'''
		bytes = struct.unpack('<21B',raw[:21])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry = data
		self.effect1_wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_input_trim = ((bytes[2] & 0x80) >> 7) | ((bytes[3] & 0x3F) << 1)
		self.effect1_pan = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry = data
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_input_trim = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		self.effect2_pan = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.effect1_pre_delay = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0xF) << 7)
		self.effect1_lfo_waveform = (bytes[10] >> 4) & 0x1
		self.effect1_lfo_frequency = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1F) << 3)
		self.effect1_depth = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.effect1_feedback = data
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.effect1_pre_leq_gain = data
		data = (bytes[14] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.effect1_pre_heq_gain = data
		self.effect2_pre_delay = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0xFF) << 2)
		self.effect2_lfo_waveform = (bytes[16] >> 1) & 0x1
		self.effect2_lfo_frequency = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x3) << 6)
		self.effect2_depth = ((bytes[17] & 0xFC) >> 2) | ((bytes[18] & 0x1) << 6)
		data = ((bytes[18] & 0xFE) >> 1) | ((bytes[19] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_feedback = data
		data = (bytes[19] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.effect2_pre_leq_gain = data
		data = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0x7) << 2)
		if data & 0x10:
			data -= 0x20
		self.effect2_pre_heq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX166 Chorus/Flanger // Chorus/Flanger object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.effect1_wet_dry << 1) & 0xFE
		bytes[1] = ((self.effect1_wet_dry >> 7) & 0x1) | ((self.effect1_wet_dry_mod_source & 0x3F) << 1) | ((self.effect1_wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = ((self.effect1_wet_dry_mod_intensity >> 1) & 0x7F) | ((self.effect1_input_trim << 7) & 0x80)
		bytes[3] = ((self.effect1_input_trim >> 1) & 0x3F) | ((self.effect1_pan << 6) & 0xC0)
		bytes[4] = ((self.effect1_pan >> 2) & 0x1F) | ((self.effect2_wet_dry << 5) & 0xE0)
		bytes[5] = ((self.effect2_wet_dry >> 3) & 0x1F) | ((self.effect2_wet_dry_mod_source << 5) & 0xE0)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 3) & 0x7) | ((self.effect2_wet_dry_mod_intensity << 3) & 0xF8)
		bytes[7] = ((self.effect2_wet_dry_mod_intensity >> 5) & 0x7) | ((self.effect2_input_trim << 3) & 0xF8)
		bytes[8] = ((self.effect2_input_trim >> 5) & 0x3) | ((self.effect2_pan << 2) & 0xFC)
		bytes[9] = ((self.effect2_pan >> 6) & 0x1) | ((self.effect1_pre_delay << 1) & 0xFE)
		bytes[10] = ((self.effect1_pre_delay >> 7) & 0xF) | ((self.effect1_lfo_waveform & 0x1) << 4) | ((self.effect1_lfo_frequency << 5) & 0xE0)
		bytes[11] = ((self.effect1_lfo_frequency >> 3) & 0x1F) | ((self.effect1_depth << 5) & 0xE0)
		bytes[12] = ((self.effect1_depth >> 3) & 0xF) | ((self.effect1_feedback << 4) & 0xF0)
		bytes[13] = ((self.effect1_feedback >> 4) & 0xF) | ((self.effect1_pre_leq_gain << 4) & 0xF0)
		bytes[14] = ((self.effect1_pre_leq_gain >> 4) & 0x1) | ((self.effect1_pre_heq_gain & 0x1F) << 1) | ((self.effect2_pre_delay << 6) & 0xC0)
		bytes[15] = (self.effect2_pre_delay >> 2) & 0xFF
		bytes[16] = ((self.effect2_lfo_waveform & 0x1) << 1) | ((self.effect2_lfo_frequency << 2) & 0xFC)
		bytes[17] = ((self.effect2_lfo_frequency >> 6) & 0x3) | ((self.effect2_depth << 2) & 0xFC)
		bytes[18] = ((self.effect2_depth >> 6) & 0x1) | ((self.effect2_feedback << 1) & 0xFE)
		bytes[19] = ((self.effect2_feedback >> 7) & 0x1) | ((self.effect2_pre_leq_gain & 0x1F) << 1) | ((self.effect2_pre_heq_gain << 6) & 0xC0)
		bytes[20] = (self.effect2_pre_heq_gain >> 2) & 0x7
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX166 Chorus/Flanger // Chorus/Flanger object'''
		json = []
		json.append(('name','FX166'))
		json.append(('description','Chorus/Flanger // Chorus/Flanger'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_input_trim',self.effect1_input_trim])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_input_trim',self.effect2_input_trim])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_pre_delay',self.effect1_pre_delay])
		json.append(['effect1_lfo_waveform',self.effect1_lfo_waveform])
		json.append(['effect1_lfo_frequency',self.effect1_lfo_frequency])
		json.append(['effect1_depth',self.effect1_depth])
		json.append(['effect1_feedback',self.effect1_feedback])
		json.append(['effect1_pre_leq_gain',self.effect1_pre_leq_gain])
		json.append(['effect1_pre_heq_gain',self.effect1_pre_heq_gain])
		json.append(['effect2_pre_delay',self.effect2_pre_delay])
		json.append(['effect2_lfo_waveform',self.effect2_lfo_waveform])
		json.append(['effect2_lfo_frequency',self.effect2_lfo_frequency])
		json.append(['effect2_depth',self.effect2_depth])
		json.append(['effect2_feedback',self.effect2_feedback])
		json.append(['effect2_pre_leq_gain',self.effect2_pre_leq_gain])
		json.append(['effect2_pre_heq_gain',self.effect2_pre_heq_gain])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_input_trim(self):
		'''(Effect1) Input Trim'''
		return self._effect1_input_trim

	@effect1_input_trim.setter
	def effect1_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect1_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_input_trim(self):
		'''(Effect2) Input Trim'''
		return self._effect2_input_trim

	@effect2_input_trim.setter
	def effect2_input_trim(self,value):
		if 0 <= value <= 100:
			self._effect2_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pre_delay(self):
		'''(Effect1) Pre Delay'''
		return self._effect1_pre_delay

	@effect1_pre_delay.setter
	def effect1_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._effect1_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_lfo_waveform(self):
		'''(Effect1) LFO Waveform'''
		return self._effect1_lfo_waveform

	@effect1_lfo_waveform.setter
	def effect1_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._effect1_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_lfo_frequency(self):
		'''(Effect1) LFO Frequency'''
		return self._effect1_lfo_frequency

	@effect1_lfo_frequency.setter
	def effect1_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect1_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_depth(self):
		'''(Effect1) Depth'''
		return self._effect1_depth

	@effect1_depth.setter
	def effect1_depth(self,value):
		if 0 <= value <= 100:
			self._effect1_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_feedback(self):
		'''(Effect1) Feedback'''
		return self._effect1_feedback

	@effect1_feedback.setter
	def effect1_feedback(self,value):
		if -100 <= value <= 100:
			self._effect1_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pre_leq_gain(self):
		'''(Effect1) Pre LEQ Gain'''
		return self._effect1_pre_leq_gain

	@effect1_pre_leq_gain.setter
	def effect1_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._effect1_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pre_heq_gain(self):
		'''(Effect1) Pre HEQ Gain'''
		return self._effect1_pre_heq_gain

	@effect1_pre_heq_gain.setter
	def effect1_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._effect1_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pre_delay(self):
		'''(Effect2) Pre Delay'''
		return self._effect2_pre_delay

	@effect2_pre_delay.setter
	def effect2_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._effect2_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_lfo_waveform(self):
		'''(Effect2) LFO Waveform'''
		return self._effect2_lfo_waveform

	@effect2_lfo_waveform.setter
	def effect2_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._effect2_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_lfo_frequency(self):
		'''(Effect2) LFO Frequency'''
		return self._effect2_lfo_frequency

	@effect2_lfo_frequency.setter
	def effect2_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect2_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_depth(self):
		'''(Effect2) Depth'''
		return self._effect2_depth

	@effect2_depth.setter
	def effect2_depth(self,value):
		if 0 <= value <= 100:
			self._effect2_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_feedback(self):
		'''(Effect2) Feedback'''
		return self._effect2_feedback

	@effect2_feedback.setter
	def effect2_feedback(self,value):
		if -100 <= value <= 100:
			self._effect2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pre_leq_gain(self):
		'''(Effect2) Pre LEQ Gain'''
		return self._effect2_pre_leq_gain

	@effect2_pre_leq_gain.setter
	def effect2_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._effect2_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pre_heq_gain(self):
		'''(Effect2) Pre HEQ Gain'''
		return self._effect2_pre_heq_gain

	@effect2_pre_heq_gain.setter
	def effect2_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._effect2_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

