#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx167.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX167(object):
	category = '''Mono//Mono'''
	description = '''Chorus/Flanger // Phaser'''
	short_description = '''Cho/Fl // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX167 Chorus/Flanger // Phaser object'''
		bytes = struct.unpack('<19B',raw[:19])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry = data
		self.chorus_flanger_wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_wet_dry_mod_intensity = data
		self.chorus_flanger_input_trim = ((bytes[2] & 0x80) >> 7) | ((bytes[3] & 0x3F) << 1)
		self.chorus_flanger_pan = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.chorus_flanger_pre_delay = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0xF) << 7)
		self.chorus_flanger_lfo_waveform = (bytes[10] >> 4) & 0x1
		self.chorus_flanger_lfo_frequency = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1F) << 3)
		self.chorus_flanger_depth = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.chorus_flanger_feedback = data
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_leq_gain = data
		data = (bytes[14] >> 1) & 0x1F
		if data & 0x10:
			data -= 0x20
		self.chorus_flanger_pre_heq_gain = data
		self.phaser_lfo_waveform = (bytes[14] >> 6) & 0x1
		self.phaser_lfo_frequency = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x7F) << 1)
		self.phaser_manual = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x3F) << 1)
		self.phaser_depth = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x1F) << 2)
		data = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX167 Chorus/Flanger // Phaser object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.chorus_flanger_wet_dry << 1) & 0xFE
		bytes[1] = ((self.chorus_flanger_wet_dry >> 7) & 0x1) | ((self.chorus_flanger_wet_dry_mod_source & 0x3F) << 1) | ((self.chorus_flanger_wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = ((self.chorus_flanger_wet_dry_mod_intensity >> 1) & 0x7F) | ((self.chorus_flanger_input_trim << 7) & 0x80)
		bytes[3] = ((self.chorus_flanger_input_trim >> 1) & 0x3F) | ((self.chorus_flanger_pan << 6) & 0xC0)
		bytes[4] = ((self.chorus_flanger_pan >> 2) & 0x1F) | ((self.phaser_wet_dry << 5) & 0xE0)
		bytes[5] = ((self.phaser_wet_dry >> 3) & 0x1F) | ((self.phaser_wet_dry_mod_source << 5) & 0xE0)
		bytes[6] = ((self.phaser_wet_dry_mod_source >> 3) & 0x7) | ((self.phaser_wet_dry_mod_intensity << 3) & 0xF8)
		bytes[7] = (self.phaser_wet_dry_mod_intensity >> 5) & 0x7
		bytes[8] = (self.phaser_pan << 2) & 0xFC
		bytes[9] = ((self.phaser_pan >> 6) & 0x1) | ((self.chorus_flanger_pre_delay << 1) & 0xFE)
		bytes[10] = ((self.chorus_flanger_pre_delay >> 7) & 0xF) | ((self.chorus_flanger_lfo_waveform & 0x1) << 4) | ((self.chorus_flanger_lfo_frequency << 5) & 0xE0)
		bytes[11] = ((self.chorus_flanger_lfo_frequency >> 3) & 0x1F) | ((self.chorus_flanger_depth << 5) & 0xE0)
		bytes[12] = ((self.chorus_flanger_depth >> 3) & 0xF) | ((self.chorus_flanger_feedback << 4) & 0xF0)
		bytes[13] = ((self.chorus_flanger_feedback >> 4) & 0xF) | ((self.chorus_flanger_pre_leq_gain << 4) & 0xF0)
		bytes[14] = ((self.chorus_flanger_pre_leq_gain >> 4) & 0x1) | ((self.chorus_flanger_pre_heq_gain & 0x1F) << 1) | ((self.phaser_lfo_waveform & 0x1) << 6) | ((self.phaser_lfo_frequency << 7) & 0x80)
		bytes[15] = ((self.phaser_lfo_frequency >> 1) & 0x7F) | ((self.phaser_manual << 7) & 0x80)
		bytes[16] = ((self.phaser_manual >> 1) & 0x3F) | ((self.phaser_depth << 6) & 0xC0)
		bytes[17] = ((self.phaser_depth >> 2) & 0x1F) | ((self.phaser_resonance << 5) & 0xE0)
		bytes[18] = (self.phaser_resonance >> 3) & 0x1F
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX167 Chorus/Flanger // Phaser object'''
		json = []
		json.append(('name','FX167'))
		json.append(('description','Chorus/Flanger // Phaser'))
		json.append(['chorus_flanger_wet_dry',self.chorus_flanger_wet_dry])
		json.append(['chorus_flanger_wet_dry_mod_source',self.chorus_flanger_wet_dry_mod_source])
		json.append(['chorus_flanger_wet_dry_mod_intensity',self.chorus_flanger_wet_dry_mod_intensity])
		json.append(['chorus_flanger_input_trim',self.chorus_flanger_input_trim])
		json.append(['chorus_flanger_pan',self.chorus_flanger_pan])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['chorus_flanger_pre_delay',self.chorus_flanger_pre_delay])
		json.append(['chorus_flanger_lfo_waveform',self.chorus_flanger_lfo_waveform])
		json.append(['chorus_flanger_lfo_frequency',self.chorus_flanger_lfo_frequency])
		json.append(['chorus_flanger_depth',self.chorus_flanger_depth])
		json.append(['chorus_flanger_feedback',self.chorus_flanger_feedback])
		json.append(['chorus_flanger_pre_leq_gain',self.chorus_flanger_pre_leq_gain])
		json.append(['chorus_flanger_pre_heq_gain',self.chorus_flanger_pre_heq_gain])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry(self):
		'''(Chorus/Flanger) Wet/Dry'''
		return self._chorus_flanger_wet_dry

	@chorus_flanger_wet_dry.setter
	def chorus_flanger_wet_dry(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_source(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Source'''
		return self._chorus_flanger_wet_dry_mod_source

	@chorus_flanger_wet_dry_mod_source.setter
	def chorus_flanger_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._chorus_flanger_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_wet_dry_mod_intensity(self):
		'''(Chorus/Flanger) Wet/Dry Mod.Intensity'''
		return self._chorus_flanger_wet_dry_mod_intensity

	@chorus_flanger_wet_dry_mod_intensity.setter
	def chorus_flanger_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_input_trim(self):
		'''(Chorus/Flanger) Input Trim'''
		return self._chorus_flanger_input_trim

	@chorus_flanger_input_trim.setter
	def chorus_flanger_input_trim(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pan(self):
		'''(Chorus/Flanger) Pan'''
		return self._chorus_flanger_pan

	@chorus_flanger_pan.setter
	def chorus_flanger_pan(self,value):
		if 0 <= value <= 127:
			self._chorus_flanger_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_delay(self):
		'''(Chorus/Flanger) Pre Delay'''
		return self._chorus_flanger_pre_delay

	@chorus_flanger_pre_delay.setter
	def chorus_flanger_pre_delay(self,value):
		if 0 <= value <= 1415:
			self._chorus_flanger_pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 1415' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_waveform(self):
		'''(Chorus/Flanger) LFO Waveform'''
		return self._chorus_flanger_lfo_waveform

	@chorus_flanger_lfo_waveform.setter
	def chorus_flanger_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._chorus_flanger_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_lfo_frequency(self):
		'''(Chorus/Flanger) LFO Frequency'''
		return self._chorus_flanger_lfo_frequency

	@chorus_flanger_lfo_frequency.setter
	def chorus_flanger_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._chorus_flanger_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_depth(self):
		'''(Chorus/Flanger) Depth'''
		return self._chorus_flanger_depth

	@chorus_flanger_depth.setter
	def chorus_flanger_depth(self,value):
		if 0 <= value <= 100:
			self._chorus_flanger_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_feedback(self):
		'''(Chorus/Flanger) Feedback'''
		return self._chorus_flanger_feedback

	@chorus_flanger_feedback.setter
	def chorus_flanger_feedback(self,value):
		if -100 <= value <= 100:
			self._chorus_flanger_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_leq_gain(self):
		'''(Chorus/Flanger) Pre LEQ Gain'''
		return self._chorus_flanger_pre_leq_gain

	@chorus_flanger_pre_leq_gain.setter
	def chorus_flanger_pre_leq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_leq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def chorus_flanger_pre_heq_gain(self):
		'''(Chorus/Flanger) Pre HEQ Gain'''
		return self._chorus_flanger_pre_heq_gain

	@chorus_flanger_pre_heq_gain.setter
	def chorus_flanger_pre_heq_gain(self,value):
		if -15 <= value <= 15:
			self._chorus_flanger_pre_heq_gain = value
		else:
			raise ValueError('%s not -15 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

