#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx169.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX169(object):
	category = '''Mono//Mono'''
	description = '''Phaser // Phaser'''
	short_description = '''Phaser // Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX169 Phaser // Phaser object'''
		bytes = struct.unpack('<17B',raw[:17])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry = data
		self.effect1_wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_pan = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry = data
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_pan = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.effect1_lfo_waveform = (bytes[9] >> 1) & 0x1
		self.effect1_lfo_frequency = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0x3) << 6)
		self.effect1_manual = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x1) << 6)
		self.effect1_depth = (bytes[11] >> 1) & 0x7F
		data = bytes[12] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.effect1_resonance = data
		self.effect2_lfo_waveform = bytes[13] & 0x1
		self.effect2_lfo_frequency = ((bytes[13] & 0xFE) >> 1) | ((bytes[14] & 0x1) << 7)
		self.effect2_manual = (bytes[14] >> 1) & 0x7F
		self.effect2_depth = bytes[15] & 0x7F
		data = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.effect2_resonance = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX169 Phaser // Phaser object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.effect1_wet_dry << 1) & 0xFE
		bytes[1] = ((self.effect1_wet_dry >> 7) & 0x1) | ((self.effect1_wet_dry_mod_source & 0x3F) << 1) | ((self.effect1_wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.effect1_wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = (self.effect1_pan << 6) & 0xC0
		bytes[4] = ((self.effect1_pan >> 2) & 0x1F) | ((self.effect2_wet_dry << 5) & 0xE0)
		bytes[5] = ((self.effect2_wet_dry >> 3) & 0x1F) | ((self.effect2_wet_dry_mod_source << 5) & 0xE0)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 3) & 0x7) | ((self.effect2_wet_dry_mod_intensity << 3) & 0xF8)
		bytes[7] = (self.effect2_wet_dry_mod_intensity >> 5) & 0x7
		bytes[8] = (self.effect2_pan << 2) & 0xFC
		bytes[9] = ((self.effect2_pan >> 6) & 0x1) | ((self.effect1_lfo_waveform & 0x1) << 1) | ((self.effect1_lfo_frequency << 2) & 0xFC)
		bytes[10] = ((self.effect1_lfo_frequency >> 6) & 0x3) | ((self.effect1_manual << 2) & 0xFC)
		bytes[11] = ((self.effect1_manual >> 6) & 0x1) | ((self.effect1_depth & 0x7F) << 1)
		bytes[12] = self.effect1_resonance & 0xFF
		bytes[13] = (self.effect2_lfo_waveform & 0x1) | ((self.effect2_lfo_frequency << 1) & 0xFE)
		bytes[14] = ((self.effect2_lfo_frequency >> 7) & 0x1) | ((self.effect2_manual & 0x7F) << 1)
		bytes[15] = (self.effect2_depth & 0x7F) | ((self.effect2_resonance << 7) & 0x80)
		bytes[16] = (self.effect2_resonance >> 1) & 0x7F
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX169 Phaser // Phaser object'''
		json = []
		json.append(('name','FX169'))
		json.append(('description','Phaser // Phaser'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_lfo_waveform',self.effect1_lfo_waveform])
		json.append(['effect1_lfo_frequency',self.effect1_lfo_frequency])
		json.append(['effect1_manual',self.effect1_manual])
		json.append(['effect1_depth',self.effect1_depth])
		json.append(['effect1_resonance',self.effect1_resonance])
		json.append(['effect2_lfo_waveform',self.effect2_lfo_waveform])
		json.append(['effect2_lfo_frequency',self.effect2_lfo_frequency])
		json.append(['effect2_manual',self.effect2_manual])
		json.append(['effect2_depth',self.effect2_depth])
		json.append(['effect2_resonance',self.effect2_resonance])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_lfo_waveform(self):
		'''(Effect1) LFO Waveform'''
		return self._effect1_lfo_waveform

	@effect1_lfo_waveform.setter
	def effect1_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._effect1_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_lfo_frequency(self):
		'''(Effect1) LFO Frequency'''
		return self._effect1_lfo_frequency

	@effect1_lfo_frequency.setter
	def effect1_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect1_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_manual(self):
		'''(Effect1) Manual'''
		return self._effect1_manual

	@effect1_manual.setter
	def effect1_manual(self,value):
		if 0 <= value <= 100:
			self._effect1_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_depth(self):
		'''(Effect1) Depth'''
		return self._effect1_depth

	@effect1_depth.setter
	def effect1_depth(self,value):
		if 0 <= value <= 100:
			self._effect1_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_resonance(self):
		'''(Effect1) Resonance'''
		return self._effect1_resonance

	@effect1_resonance.setter
	def effect1_resonance(self,value):
		if -100 <= value <= 100:
			self._effect1_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_lfo_waveform(self):
		'''(Effect2) LFO Waveform'''
		return self._effect2_lfo_waveform

	@effect2_lfo_waveform.setter
	def effect2_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._effect2_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_lfo_frequency(self):
		'''(Effect2) LFO Frequency'''
		return self._effect2_lfo_frequency

	@effect2_lfo_frequency.setter
	def effect2_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._effect2_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_manual(self):
		'''(Effect2) Manual'''
		return self._effect2_manual

	@effect2_manual.setter
	def effect2_manual(self,value):
		if 0 <= value <= 100:
			self._effect2_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_depth(self):
		'''(Effect2) Depth'''
		return self._effect2_depth

	@effect2_depth.setter
	def effect2_depth(self,value):
		if 0 <= value <= 100:
			self._effect2_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_resonance(self):
		'''(Effect2) Resonance'''
		return self._effect2_resonance

	@effect2_resonance.setter
	def effect2_resonance(self,value):
		if -100 <= value <= 100:
			self._effect2_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

