#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx170.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX170(object):
	category = '''Mono//Mono'''
	description = '''Phaser // Multitap BPM Delay'''
	short_description = '''Phaser // BPMDly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX170 Phaser // Multitap BPM Delay object'''
		bytes = struct.unpack('<20B',raw[:20])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry = data
		self.phaser_wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.phaser_wet_dry_mod_intensity = data
		self.phaser_pan = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.multitap_bpm_delay_wet_dry = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.multitap_bpm_delay_wet_dry_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_wet_dry_mod_intensity = data
		self.multitap_bpm_delay_pan = (bytes[8] >> 1) & 0x7F
		self.phaser_lfo_waveform = bytes[9] & 0x1
		self.phaser_lfo_frequency = ((bytes[9] & 0xFE) >> 1) | ((bytes[10] & 0x1) << 7)
		self.phaser_manual = (bytes[10] >> 1) & 0x7F
		self.phaser_depth = bytes[11] & 0x7F
		data = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.phaser_resonance = data
		self.multitap_bpm_delay_bpm = ((bytes[12] & 0x80) >> 7) | (bytes[13] << 1) | ((bytes[14] & 0x3F) << 9)
		self.multitap_bpm_delay_tap1_base_note = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x3) << 2)
		self.multitap_bpm_delay_tap1_times = (bytes[15] >> 2) & 0x3F
		self.multitap_bpm_delay_tap2_base_note = bytes[16] & 0xF
		self.multitap_bpm_delay_tap2_times = ((bytes[16] & 0xF0) >> 4) | ((bytes[17] & 0x3) << 4)
		self.multitap_bpm_delay_tap1_level = ((bytes[17] & 0xFC) >> 2) | ((bytes[18] & 0x1) << 6)
		data = ((bytes[18] & 0xFE) >> 1) | ((bytes[19] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_tap2_feedback = data
		self.multitap_bpm_delay_high_damp = (bytes[19] >> 1) & 0x7F

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX170 Phaser // Multitap BPM Delay object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.phaser_wet_dry << 1) & 0xFE
		bytes[1] = ((self.phaser_wet_dry >> 7) & 0x1) | ((self.phaser_wet_dry_mod_source & 0x3F) << 1) | ((self.phaser_wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.phaser_wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = (self.phaser_pan << 6) & 0xC0
		bytes[4] = ((self.phaser_pan >> 2) & 0x1F) | ((self.multitap_bpm_delay_wet_dry << 5) & 0xE0)
		bytes[5] = ((self.multitap_bpm_delay_wet_dry >> 3) & 0xF) | ((self.multitap_bpm_delay_wet_dry_mod_source << 4) & 0xF0)
		bytes[6] = ((self.multitap_bpm_delay_wet_dry_mod_source >> 4) & 0x3) | ((self.multitap_bpm_delay_wet_dry_mod_intensity << 2) & 0xFC)
		bytes[7] = (self.multitap_bpm_delay_wet_dry_mod_intensity >> 6) & 0x3
		bytes[8] = (self.multitap_bpm_delay_pan & 0x7F) << 1
		bytes[9] = (self.phaser_lfo_waveform & 0x1) | ((self.phaser_lfo_frequency << 1) & 0xFE)
		bytes[10] = ((self.phaser_lfo_frequency >> 7) & 0x1) | ((self.phaser_manual & 0x7F) << 1)
		bytes[11] = (self.phaser_depth & 0x7F) | ((self.phaser_resonance << 7) & 0x80)
		bytes[12] = ((self.phaser_resonance >> 1) & 0x7F) | ((self.multitap_bpm_delay_bpm << 7) & 0x80)
		bytes[13] = (self.multitap_bpm_delay_bpm >> 1) & 0xFF
		bytes[14] = ((self.multitap_bpm_delay_bpm >> 63) & 0x9) | ((self.multitap_bpm_delay_tap1_base_note << 6) & 0xC0)
		bytes[15] = ((self.multitap_bpm_delay_tap1_base_note >> 2) & 0x3) | ((self.multitap_bpm_delay_tap1_times & 0x3F) << 2)
		bytes[16] = (self.multitap_bpm_delay_tap2_base_note & 0xF) | ((self.multitap_bpm_delay_tap2_times << 4) & 0xF0)
		bytes[17] = ((self.multitap_bpm_delay_tap2_times >> 4) & 0x3) | ((self.multitap_bpm_delay_tap1_level << 2) & 0xFC)
		bytes[18] = ((self.multitap_bpm_delay_tap1_level >> 6) & 0x1) | ((self.multitap_bpm_delay_tap2_feedback << 1) & 0xFE)
		bytes[19] = ((self.multitap_bpm_delay_tap2_feedback >> 7) & 0x1) | ((self.multitap_bpm_delay_high_damp & 0x7F) << 1)
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX170 Phaser // Multitap BPM Delay object'''
		json = []
		json.append(('name','FX170'))
		json.append(('description','Phaser // Multitap BPM Delay'))
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['phaser_wet_dry_mod_source',self.phaser_wet_dry_mod_source])
		json.append(['phaser_wet_dry_mod_intensity',self.phaser_wet_dry_mod_intensity])
		json.append(['phaser_pan',self.phaser_pan])
		json.append(['multitap_bpm_delay_wet_dry',self.multitap_bpm_delay_wet_dry])
		json.append(['multitap_bpm_delay_wet_dry_mod_source',self.multitap_bpm_delay_wet_dry_mod_source])
		json.append(['multitap_bpm_delay_wet_dry_mod_intensity',self.multitap_bpm_delay_wet_dry_mod_intensity])
		json.append(['multitap_bpm_delay_pan',self.multitap_bpm_delay_pan])
		json.append(['phaser_lfo_waveform',self.phaser_lfo_waveform])
		json.append(['phaser_lfo_frequency',self.phaser_lfo_frequency])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['multitap_bpm_delay_bpm',self.multitap_bpm_delay_bpm])
		json.append(['multitap_bpm_delay_bpm_display',self.multitap_bpm_delay_bpm_display])
		json.append(['multitap_bpm_delay_tap1_base_note',self.multitap_bpm_delay_tap1_base_note])
		json.append(['multitap_bpm_delay_tap1_base_note_display',self.multitap_bpm_delay_tap1_base_note_display])
		json.append(['multitap_bpm_delay_tap1_times',self.multitap_bpm_delay_tap1_times])
		json.append(['multitap_bpm_delay_tap2_base_note',self.multitap_bpm_delay_tap2_base_note])
		json.append(['multitap_bpm_delay_tap2_base_note_display',self.multitap_bpm_delay_tap2_base_note_display])
		json.append(['multitap_bpm_delay_tap2_times',self.multitap_bpm_delay_tap2_times])
		json.append(['multitap_bpm_delay_tap1_level',self.multitap_bpm_delay_tap1_level])
		json.append(['multitap_bpm_delay_tap2_feedback',self.multitap_bpm_delay_tap2_feedback])
		json.append(['multitap_bpm_delay_high_damp',self.multitap_bpm_delay_high_damp])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''(Phaser) Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_source(self):
		'''(Phaser) Wet/Dry Mod.Source'''
		return self._phaser_wet_dry_mod_source

	@phaser_wet_dry_mod_source.setter
	def phaser_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry_mod_intensity(self):
		'''(Phaser) Wet/Dry Mod.Intensity'''
		return self._phaser_wet_dry_mod_intensity

	@phaser_wet_dry_mod_intensity.setter
	def phaser_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._phaser_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_pan(self):
		'''(Phaser) Pan'''
		return self._phaser_pan

	@phaser_pan.setter
	def phaser_pan(self,value):
		if 0 <= value <= 127:
			self._phaser_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry(self):
		'''(Multitap BPM Delay) Wet/Dry'''
		return self._multitap_bpm_delay_wet_dry

	@multitap_bpm_delay_wet_dry.setter
	def multitap_bpm_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_source(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Source'''
		return self._multitap_bpm_delay_wet_dry_mod_source

	@multitap_bpm_delay_wet_dry_mod_source.setter
	def multitap_bpm_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_bpm_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_intensity(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_bpm_delay_wet_dry_mod_intensity

	@multitap_bpm_delay_wet_dry_mod_intensity.setter
	def multitap_bpm_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_pan(self):
		'''(Multitap BPM Delay) Pan'''
		return self._multitap_bpm_delay_pan

	@multitap_bpm_delay_pan.setter
	def multitap_bpm_delay_pan(self,value):
		if 0 <= value <= 127:
			self._multitap_bpm_delay_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_waveform(self):
		'''(Phaser) LFO Waveform'''
		return self._phaser_lfo_waveform

	@phaser_lfo_waveform.setter
	def phaser_lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._phaser_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_lfo_frequency(self):
		'''(Phaser) LFO Frequency'''
		return self._phaser_lfo_frequency

	@phaser_lfo_frequency.setter
	def phaser_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._phaser_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''(Phaser) Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''(Phaser) Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 100:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''(Phaser) Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -100 <= value <= 100:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_bpm(self):
		'''(Multitap BPM Delay) BPM'''
		return self._multitap_bpm_delay_bpm

	@multitap_bpm_delay_bpm.setter
	def multitap_bpm_delay_bpm(self,value):
		if 3999 <= value <= 30000:
			self._multitap_bpm_delay_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def multitap_bpm_delay_bpm_display(self):
		'''(Multitap BPM Delay) BPM display value'''
		return bpm_decode(self._multitap_bpm_delay_bpm)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_base_note(self):
		'''(Multitap BPM Delay) Tap1 Base Note'''
		return self._multitap_bpm_delay_tap1_base_note

	@multitap_bpm_delay_tap1_base_note.setter
	def multitap_bpm_delay_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap1_base_note_display(self):
		'''(Multitap BPM Delay) Tap1 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_times(self):
		'''(Multitap BPM Delay) Tap1 Times'''
		return self._multitap_bpm_delay_tap1_times

	@multitap_bpm_delay_tap1_times.setter
	def multitap_bpm_delay_tap1_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_base_note(self):
		'''(Multitap BPM Delay) Tap2 Base Note'''
		return self._multitap_bpm_delay_tap2_base_note

	@multitap_bpm_delay_tap2_base_note.setter
	def multitap_bpm_delay_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap2_base_note_display(self):
		'''(Multitap BPM Delay) Tap2 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_times(self):
		'''(Multitap BPM Delay) Tap2 Times'''
		return self._multitap_bpm_delay_tap2_times

	@multitap_bpm_delay_tap2_times.setter
	def multitap_bpm_delay_tap2_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_level(self):
		'''(Multitap BPM Delay) Tap1 Level'''
		return self._multitap_bpm_delay_tap1_level

	@multitap_bpm_delay_tap1_level.setter
	def multitap_bpm_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_feedback(self):
		'''(Multitap BPM Delay) Tap2 Feedback'''
		return self._multitap_bpm_delay_tap2_feedback

	@multitap_bpm_delay_tap2_feedback.setter
	def multitap_bpm_delay_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_high_damp(self):
		'''(Multitap BPM Delay) High Damp'''
		return self._multitap_bpm_delay_high_damp

	@multitap_bpm_delay_high_damp.setter
	def multitap_bpm_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

