#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx171.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX171(object):
	category = '''Mono//Mono'''
	description = '''Multitap BPM Delay // Multitap BPM Delay'''
	short_description = '''BPM Dl // BPM Dl'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX171 Multitap BPM Delay // Multitap BPM Delay object'''
		bytes = struct.unpack('<24B',raw[:24])
		self.effect1_wet_dry = (bytes[0] >> 1) & 0x7F
		self.effect1_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.effect1_wet_dry_mod_intensity = data
		self.effect1_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.effect2_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.effect2_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect2_wet_dry_mod_intensity = data
		self.effect2_pan = bytes[8] & 0x7F
		self.effect1_bpm = ((bytes[8] & 0x80) >> 7) | (bytes[9] << 1) | ((bytes[10] & 0x3F) << 9)
		self.effect1_tap1_base_note = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x3) << 2)
		self.effect1_tap1_times = (bytes[11] >> 2) & 0x3F
		self.effect1_tap2_base_note = bytes[12] & 0xF
		self.effect1_tap2_times = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x3) << 4)
		self.effect1_tap1_level = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		data = ((bytes[14] & 0xFE) >> 1) | ((bytes[15] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.effect1_tap2_feedback = data
		self.effect1_high_damp = (bytes[15] >> 1) & 0x7F
		self.effect2_bpm = (bytes[16] & 0xFF) | ((bytes[17] & 0x7F) << 8)
		self.effect2_tap1_base_note = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x7) << 1)
		self.effect2_tap1_times = ((bytes[18] & 0xF8) >> 3) | ((bytes[19] & 0x1) << 5)
		self.effect2_tap2_base_note = (bytes[19] >> 1) & 0xF
		self.effect2_tap2_times = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x7) << 3)
		self.effect2_tap1_level = ((bytes[20] & 0xF8) >> 3) | ((bytes[21] & 0x3) << 5)
		data = ((bytes[21] & 0xFC) >> 2) | ((bytes[22] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.effect2_tap2_feedback = data
		self.effect2_high_damp = ((bytes[22] & 0xFC) >> 2) | ((bytes[23] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX171 Multitap BPM Delay // Multitap BPM Delay object into raw data'''
		bytes = [0] * 24
		bytes[0] = (self.effect1_wet_dry & 0x7F) << 1
		bytes[1] = (self.effect1_wet_dry_mod_source & 0x3F) | ((self.effect1_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.effect1_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.effect1_pan << 5) & 0xE0
		bytes[4] = ((self.effect1_pan >> 3) & 0xF) | ((self.effect2_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.effect2_wet_dry >> 4) & 0x7) | ((self.effect2_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.effect2_wet_dry_mod_source >> 5) & 0x1) | ((self.effect2_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.effect2_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.effect2_pan & 0x7F) | ((self.effect1_bpm << 7) & 0x80)
		bytes[9] = (self.effect1_bpm >> 1) & 0xFF
		bytes[10] = ((self.effect1_bpm >> 63) & 0x9) | ((self.effect1_tap1_base_note << 6) & 0xC0)
		bytes[11] = ((self.effect1_tap1_base_note >> 2) & 0x3) | ((self.effect1_tap1_times & 0x3F) << 2)
		bytes[12] = (self.effect1_tap2_base_note & 0xF) | ((self.effect1_tap2_times << 4) & 0xF0)
		bytes[13] = ((self.effect1_tap2_times >> 4) & 0x3) | ((self.effect1_tap1_level << 2) & 0xFC)
		bytes[14] = ((self.effect1_tap1_level >> 6) & 0x1) | ((self.effect1_tap2_feedback << 1) & 0xFE)
		bytes[15] = ((self.effect1_tap2_feedback >> 7) & 0x1) | ((self.effect1_high_damp & 0x7F) << 1)
		bytes[16] = self.effect2_bpm & 0xFF
		bytes[17] = ((self.effect2_bpm >> 8) & 0x7F) | ((self.effect2_tap1_base_note << 7) & 0x80)
		bytes[18] = ((self.effect2_tap1_base_note >> 1) & 0x7) | ((self.effect2_tap1_times << 3) & 0xF8)
		bytes[19] = ((self.effect2_tap1_times >> 5) & 0x1) | ((self.effect2_tap2_base_note & 0xF) << 1) | ((self.effect2_tap2_times << 5) & 0xE0)
		bytes[20] = ((self.effect2_tap2_times >> 3) & 0x7) | ((self.effect2_tap1_level << 3) & 0xF8)
		bytes[21] = ((self.effect2_tap1_level >> 5) & 0x3) | ((self.effect2_tap2_feedback << 2) & 0xFC)
		bytes[22] = ((self.effect2_tap2_feedback >> 6) & 0x3) | ((self.effect2_high_damp << 2) & 0xFC)
		bytes[23] = (self.effect2_high_damp >> 6) & 0x1
		return struct.pack('<24B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX171 Multitap BPM Delay // Multitap BPM Delay object'''
		json = []
		json.append(('name','FX171'))
		json.append(('description','Multitap BPM Delay // Multitap BPM Delay'))
		json.append(['effect1_wet_dry',self.effect1_wet_dry])
		json.append(['effect1_wet_dry_mod_source',self.effect1_wet_dry_mod_source])
		json.append(['effect1_wet_dry_mod_intensity',self.effect1_wet_dry_mod_intensity])
		json.append(['effect1_pan',self.effect1_pan])
		json.append(['effect2_wet_dry',self.effect2_wet_dry])
		json.append(['effect2_wet_dry_mod_source',self.effect2_wet_dry_mod_source])
		json.append(['effect2_wet_dry_mod_intensity',self.effect2_wet_dry_mod_intensity])
		json.append(['effect2_pan',self.effect2_pan])
		json.append(['effect1_bpm',self.effect1_bpm])
		json.append(['effect1_bpm_display',self.effect1_bpm_display])
		json.append(['effect1_tap1_base_note',self.effect1_tap1_base_note])
		json.append(['effect1_tap1_base_note_display',self.effect1_tap1_base_note_display])
		json.append(['effect1_tap1_times',self.effect1_tap1_times])
		json.append(['effect1_tap2_base_note',self.effect1_tap2_base_note])
		json.append(['effect1_tap2_base_note_display',self.effect1_tap2_base_note_display])
		json.append(['effect1_tap2_times',self.effect1_tap2_times])
		json.append(['effect1_tap1_level',self.effect1_tap1_level])
		json.append(['effect1_tap2_feedback',self.effect1_tap2_feedback])
		json.append(['effect1_high_damp',self.effect1_high_damp])
		json.append(['effect2_bpm',self.effect2_bpm])
		json.append(['effect2_bpm_display',self.effect2_bpm_display])
		json.append(['effect2_tap1_base_note',self.effect2_tap1_base_note])
		json.append(['effect2_tap1_base_note_display',self.effect2_tap1_base_note_display])
		json.append(['effect2_tap1_times',self.effect2_tap1_times])
		json.append(['effect2_tap2_base_note',self.effect2_tap2_base_note])
		json.append(['effect2_tap2_base_note_display',self.effect2_tap2_base_note_display])
		json.append(['effect2_tap2_times',self.effect2_tap2_times])
		json.append(['effect2_tap1_level',self.effect2_tap1_level])
		json.append(['effect2_tap2_feedback',self.effect2_tap2_feedback])
		json.append(['effect2_high_damp',self.effect2_high_damp])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry(self):
		'''(Effect1) Wet/Dry'''
		return self._effect1_wet_dry

	@effect1_wet_dry.setter
	def effect1_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect1_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_source(self):
		'''(Effect1) Wet/Dry Mod.Source'''
		return self._effect1_wet_dry_mod_source

	@effect1_wet_dry_mod_source.setter
	def effect1_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect1_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_wet_dry_mod_intensity(self):
		'''(Effect1) Wet/Dry Mod.Intensity'''
		return self._effect1_wet_dry_mod_intensity

	@effect1_wet_dry_mod_intensity.setter
	def effect1_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect1_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_pan(self):
		'''(Effect1) Pan'''
		return self._effect1_pan

	@effect1_pan.setter
	def effect1_pan(self,value):
		if 0 <= value <= 127:
			self._effect1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry(self):
		'''(Effect2) Wet/Dry'''
		return self._effect2_wet_dry

	@effect2_wet_dry.setter
	def effect2_wet_dry(self,value):
		if 0 <= value <= 100:
			self._effect2_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_source(self):
		'''(Effect2) Wet/Dry Mod.Source'''
		return self._effect2_wet_dry_mod_source

	@effect2_wet_dry_mod_source.setter
	def effect2_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._effect2_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_wet_dry_mod_intensity(self):
		'''(Effect2) Wet/Dry Mod.Intensity'''
		return self._effect2_wet_dry_mod_intensity

	@effect2_wet_dry_mod_intensity.setter
	def effect2_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._effect2_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_pan(self):
		'''(Effect2) Pan'''
		return self._effect2_pan

	@effect2_pan.setter
	def effect2_pan(self,value):
		if 0 <= value <= 127:
			self._effect2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_bpm(self):
		'''(Effect1) BPM'''
		return self._effect1_bpm

	@effect1_bpm.setter
	def effect1_bpm(self,value):
		if 3999 <= value <= 30000:
			self._effect1_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def effect1_bpm_display(self):
		'''(Effect1) BPM display value'''
		return bpm_decode(self._effect1_bpm)

#-------------------------------------------------------------------------------
	@property
	def effect1_tap1_base_note(self):
		'''(Effect1) Tap1 Base Note'''
		return self._effect1_tap1_base_note

	@effect1_tap1_base_note.setter
	def effect1_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._effect1_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def effect1_tap1_base_note_display(self):
		'''(Effect1) Tap1 Base Note display value'''
		return base_note_dict[self._effect1_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def effect1_tap1_times(self):
		'''(Effect1) Tap1 Times'''
		return self._effect1_tap1_times

	@effect1_tap1_times.setter
	def effect1_tap1_times(self,value):
		if 1 <= value <= 32:
			self._effect1_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_tap2_base_note(self):
		'''(Effect1) Tap2 Base Note'''
		return self._effect1_tap2_base_note

	@effect1_tap2_base_note.setter
	def effect1_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._effect1_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def effect1_tap2_base_note_display(self):
		'''(Effect1) Tap2 Base Note display value'''
		return base_note_dict[self._effect1_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def effect1_tap2_times(self):
		'''(Effect1) Tap2 Times'''
		return self._effect1_tap2_times

	@effect1_tap2_times.setter
	def effect1_tap2_times(self,value):
		if 1 <= value <= 32:
			self._effect1_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_tap1_level(self):
		'''(Effect1) Tap1 Level'''
		return self._effect1_tap1_level

	@effect1_tap1_level.setter
	def effect1_tap1_level(self,value):
		if 0 <= value <= 100:
			self._effect1_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_tap2_feedback(self):
		'''(Effect1) Tap2 Feedback'''
		return self._effect1_tap2_feedback

	@effect1_tap2_feedback.setter
	def effect1_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._effect1_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect1_high_damp(self):
		'''(Effect1) High Damp'''
		return self._effect1_high_damp

	@effect1_high_damp.setter
	def effect1_high_damp(self,value):
		if 0 <= value <= 100:
			self._effect1_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_bpm(self):
		'''(Effect2) BPM'''
		return self._effect2_bpm

	@effect2_bpm.setter
	def effect2_bpm(self,value):
		if 3999 <= value <= 30000:
			self._effect2_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def effect2_bpm_display(self):
		'''(Effect2) BPM display value'''
		return bpm_decode(self._effect2_bpm)

#-------------------------------------------------------------------------------
	@property
	def effect2_tap1_base_note(self):
		'''(Effect2) Tap1 Base Note'''
		return self._effect2_tap1_base_note

	@effect2_tap1_base_note.setter
	def effect2_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._effect2_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def effect2_tap1_base_note_display(self):
		'''(Effect2) Tap1 Base Note display value'''
		return base_note_dict[self._effect2_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def effect2_tap1_times(self):
		'''(Effect2) Tap1 Times'''
		return self._effect2_tap1_times

	@effect2_tap1_times.setter
	def effect2_tap1_times(self,value):
		if 1 <= value <= 32:
			self._effect2_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_tap2_base_note(self):
		'''(Effect2) Tap2 Base Note'''
		return self._effect2_tap2_base_note

	@effect2_tap2_base_note.setter
	def effect2_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._effect2_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def effect2_tap2_base_note_display(self):
		'''(Effect2) Tap2 Base Note display value'''
		return base_note_dict[self._effect2_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def effect2_tap2_times(self):
		'''(Effect2) Tap2 Times'''
		return self._effect2_tap2_times

	@effect2_tap2_times.setter
	def effect2_tap2_times(self,value):
		if 1 <= value <= 32:
			self._effect2_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_tap1_level(self):
		'''(Effect2) Tap1 Level'''
		return self._effect2_tap1_level

	@effect2_tap1_level.setter
	def effect2_tap1_level(self,value):
		if 0 <= value <= 100:
			self._effect2_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_tap2_feedback(self):
		'''(Effect2) Tap2 Feedback'''
		return self._effect2_tap2_feedback

	@effect2_tap2_feedback.setter
	def effect2_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._effect2_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def effect2_high_damp(self):
		'''(Effect2) High Damp'''
		return self._effect2_high_damp

	@effect2_high_damp.setter
	def effect2_high_damp(self,value):
		if 0 <= value <= 100:
			self._effect2_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

