#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx176.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX176(object):
	category = '''Double Size'''
	description = '''Guitar Amp Model + Parametric 4-Band EQ'''
	short_description = '''GuitarAmp + P4EQ'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX176 Guitar Amp Model + Parametric 4-Band EQ object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.guitar_amp_model_amp_type = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1) << 3)
		self.guitar_amp_model_gain = (bytes[4] >> 1) & 0x7F
		self.guitar_amp_model_volume = bytes[5] & 0x7F
		self.guitar_amp_model_volume_mod_source = ((bytes[5] & 0x80) >> 7) | ((bytes[6] & 0x1F) << 1)
		data = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.guitar_amp_model_volume_mod_intensity = data
		self.guitar_amp_model_bass = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0xF) << 3)
		self.guitar_amp_model_middle = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x7) << 4)
		self.guitar_amp_model_treble = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0x3) << 5)
		self.guitar_amp_model_presence_cut = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x1) << 6)
		self.p4eq_eq1_cutoff = (bytes[11] >> 1) & 0x7F
		self.p4eq_eq1_q = bytes[12] & 0x7F
		data = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.p4eq_eq1_gain = data
		self.p4eq_eq2_cutoff = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x3F) << 3)
		self.p4eq_eq2_q = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x1F) << 2)
		data = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.p4eq_eq2_gain = data
		self.p4eq_eq3_cutoff = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x3) << 5)
		self.p4eq_eq3_q = ((bytes[17] & 0xFC) >> 2) | ((bytes[18] & 0x1) << 6)
		data = (bytes[18] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.p4eq_eq3_gain = data
		self.p4eq_eq4_cutoff = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x7F) << 1)
		self.p4eq_eq4_q = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0x3F) << 1)
		data = ((bytes[20] & 0xC0) >> 6) | ((bytes[21] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.p4eq_eq4_gain = data
		self.p4eq_switch = (bytes[21] >> 4) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX176 Guitar Amp Model + Parametric 4-Band EQ object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.guitar_amp_model_amp_type << 5) & 0xE0
		bytes[4] = ((self.guitar_amp_model_amp_type >> 3) & 0x1) | ((self.guitar_amp_model_gain & 0x7F) << 1)
		bytes[5] = (self.guitar_amp_model_volume & 0x7F) | ((self.guitar_amp_model_volume_mod_source << 7) & 0x80)
		bytes[6] = ((self.guitar_amp_model_volume_mod_source >> 1) & 0x1F) | ((self.guitar_amp_model_volume_mod_intensity << 5) & 0xE0)
		bytes[7] = ((self.guitar_amp_model_volume_mod_intensity >> 3) & 0x1F) | ((self.guitar_amp_model_bass << 5) & 0xE0)
		bytes[8] = ((self.guitar_amp_model_bass >> 3) & 0xF) | ((self.guitar_amp_model_middle << 4) & 0xF0)
		bytes[9] = ((self.guitar_amp_model_middle >> 4) & 0x7) | ((self.guitar_amp_model_treble << 3) & 0xF8)
		bytes[10] = ((self.guitar_amp_model_treble >> 5) & 0x3) | ((self.guitar_amp_model_presence_cut << 2) & 0xFC)
		bytes[11] = ((self.guitar_amp_model_presence_cut >> 6) & 0x1) | ((self.p4eq_eq1_cutoff & 0x7F) << 1)
		bytes[12] = (self.p4eq_eq1_q & 0x7F) | ((self.p4eq_eq1_gain << 7) & 0x80)
		bytes[13] = ((self.p4eq_eq1_gain >> 1) & 0x1F) | ((self.p4eq_eq2_cutoff << 5) & 0xE0)
		bytes[14] = ((self.p4eq_eq2_cutoff >> 3) & 0x3F) | ((self.p4eq_eq2_q << 6) & 0xC0)
		bytes[15] = ((self.p4eq_eq2_q >> 2) & 0x1F) | ((self.p4eq_eq2_gain << 5) & 0xE0)
		bytes[16] = ((self.p4eq_eq2_gain >> 3) & 0x7) | ((self.p4eq_eq3_cutoff << 3) & 0xF8)
		bytes[17] = ((self.p4eq_eq3_cutoff >> 5) & 0x3) | ((self.p4eq_eq3_q << 2) & 0xFC)
		bytes[18] = ((self.p4eq_eq3_q >> 6) & 0x1) | ((self.p4eq_eq3_gain & 0x3F) << 1) | ((self.p4eq_eq4_cutoff << 7) & 0x80)
		bytes[19] = ((self.p4eq_eq4_cutoff >> 1) & 0x7F) | ((self.p4eq_eq4_q << 7) & 0x80)
		bytes[20] = ((self.p4eq_eq4_q >> 1) & 0x3F) | ((self.p4eq_eq4_gain << 6) & 0xC0)
		bytes[21] = ((self.p4eq_eq4_gain >> 2) & 0xF) | ((self.p4eq_switch & 0x1) << 4)
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX176 Guitar Amp Model + Parametric 4-Band EQ object'''
		json = []
		json.append(('name','FX176'))
		json.append(('description','Guitar Amp Model + Parametric 4-Band EQ'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['guitar_amp_model_amp_type',self.guitar_amp_model_amp_type])
		json.append(['guitar_amp_model_gain',self.guitar_amp_model_gain])
		json.append(['guitar_amp_model_volume',self.guitar_amp_model_volume])
		json.append(['guitar_amp_model_volume_mod_source',self.guitar_amp_model_volume_mod_source])
		json.append(['guitar_amp_model_volume_mod_intensity',self.guitar_amp_model_volume_mod_intensity])
		json.append(['guitar_amp_model_bass',self.guitar_amp_model_bass])
		json.append(['guitar_amp_model_middle',self.guitar_amp_model_middle])
		json.append(['guitar_amp_model_treble',self.guitar_amp_model_treble])
		json.append(['guitar_amp_model_presence_cut',self.guitar_amp_model_presence_cut])
		json.append(['p4eq_eq1_cutoff',self.p4eq_eq1_cutoff])
		json.append(['p4eq_eq1_cutoff_display',self.p4eq_eq1_cutoff_display])
		json.append(['p4eq_eq1_q',self.p4eq_eq1_q])
		json.append(['p4eq_eq1_gain',self.p4eq_eq1_gain])
		json.append(['p4eq_eq2_cutoff',self.p4eq_eq2_cutoff])
		json.append(['p4eq_eq2_q',self.p4eq_eq2_q])
		json.append(['p4eq_eq2_gain',self.p4eq_eq2_gain])
		json.append(['p4eq_eq3_cutoff',self.p4eq_eq3_cutoff])
		json.append(['p4eq_eq3_q',self.p4eq_eq3_q])
		json.append(['p4eq_eq3_gain',self.p4eq_eq3_gain])
		json.append(['p4eq_eq4_cutoff',self.p4eq_eq4_cutoff])
		json.append(['p4eq_eq4_q',self.p4eq_eq4_q])
		json.append(['p4eq_eq4_gain',self.p4eq_eq4_gain])
		json.append(['p4eq_switch',self.p4eq_switch])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_amp_type(self):
		'''(Guitar Amp Model) Amp Type'''
		return self._guitar_amp_model_amp_type

	@guitar_amp_model_amp_type.setter
	def guitar_amp_model_amp_type(self,value):
		if 0 <= value <= 15:
			self._guitar_amp_model_amp_type = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_gain(self):
		'''(Guitar Amp Model) Gain'''
		return self._guitar_amp_model_gain

	@guitar_amp_model_gain.setter
	def guitar_amp_model_gain(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_gain = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_volume(self):
		'''(Guitar Amp Model) Volume'''
		return self._guitar_amp_model_volume

	@guitar_amp_model_volume.setter
	def guitar_amp_model_volume(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_volume_mod_source(self):
		'''(Guitar Amp Model) Volume Mod.Source'''
		return self._guitar_amp_model_volume_mod_source

	@guitar_amp_model_volume_mod_source.setter
	def guitar_amp_model_volume_mod_source(self,value):
		if 0 <= value <= 37:
			self._guitar_amp_model_volume_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_volume_mod_intensity(self):
		'''(Guitar Amp Model) Volume Mod.Intensity'''
		return self._guitar_amp_model_volume_mod_intensity

	@guitar_amp_model_volume_mod_intensity.setter
	def guitar_amp_model_volume_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._guitar_amp_model_volume_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_bass(self):
		'''(Guitar Amp Model) Bass'''
		return self._guitar_amp_model_bass

	@guitar_amp_model_bass.setter
	def guitar_amp_model_bass(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_bass = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_middle(self):
		'''(Guitar Amp Model) Middle'''
		return self._guitar_amp_model_middle

	@guitar_amp_model_middle.setter
	def guitar_amp_model_middle(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_middle = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_treble(self):
		'''(Guitar Amp Model) Treble'''
		return self._guitar_amp_model_treble

	@guitar_amp_model_treble.setter
	def guitar_amp_model_treble(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_treble = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def guitar_amp_model_presence_cut(self):
		'''(Guitar Amp Model) Presence/Cut'''
		return self._guitar_amp_model_presence_cut

	@guitar_amp_model_presence_cut.setter
	def guitar_amp_model_presence_cut(self,value):
		if 0 <= value <= 100:
			self._guitar_amp_model_presence_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq1_cutoff(self):
		'''(P4EQ) EQ1 Cutoff'''
		return self._p4eq_eq1_cutoff

	@p4eq_eq1_cutoff.setter
	def p4eq_eq1_cutoff(self,value):
		if 0 <= value <= 98:
			self._p4eq_eq1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def p4eq_eq1_cutoff_display(self):
		'''(P4EQ) EQ1 Cutoff display value'''
		dv = 20 + self._p4eq_eq1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq1_q(self):
		'''(P4EQ) EQ1 Q'''
		return self._p4eq_eq1_q

	@p4eq_eq1_q.setter
	def p4eq_eq1_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_eq1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq1_gain(self):
		'''(P4EQ) EQ1 Gain'''
		return self._p4eq_eq1_gain

	@p4eq_eq1_gain.setter
	def p4eq_eq1_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_eq1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq2_cutoff(self):
		'''(P4EQ) EQ2 Cutoff'''
		return self._p4eq_eq2_cutoff

	@p4eq_eq2_cutoff.setter
	def p4eq_eq2_cutoff(self,value):
		if 0 <= value <= 495:
			self._p4eq_eq2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 495' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq2_q(self):
		'''(P4EQ) EQ2 Q'''
		return self._p4eq_eq2_q

	@p4eq_eq2_q.setter
	def p4eq_eq2_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_eq2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq2_gain(self):
		'''(P4EQ) EQ2 Gain'''
		return self._p4eq_eq2_gain

	@p4eq_eq2_gain.setter
	def p4eq_eq2_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_eq2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq3_cutoff(self):
		'''(P4EQ) EQ3 Cutoff'''
		return self._p4eq_eq3_cutoff

	@p4eq_eq3_cutoff.setter
	def p4eq_eq3_cutoff(self,value):
		if 0 <= value <= 97:
			self._p4eq_eq3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq3_q(self):
		'''(P4EQ) EQ3 Q'''
		return self._p4eq_eq3_q

	@p4eq_eq3_q.setter
	def p4eq_eq3_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_eq3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq3_gain(self):
		'''(P4EQ) EQ3 Gain'''
		return self._p4eq_eq3_gain

	@p4eq_eq3_gain.setter
	def p4eq_eq3_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_eq3_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq4_cutoff(self):
		'''(P4EQ) EQ4 Cutoff'''
		return self._p4eq_eq4_cutoff

	@p4eq_eq4_cutoff.setter
	def p4eq_eq4_cutoff(self,value):
		if 0 <= value <= 195:
			self._p4eq_eq4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq4_q(self):
		'''(P4EQ) EQ4 Q'''
		return self._p4eq_eq4_q

	@p4eq_eq4_q.setter
	def p4eq_eq4_q(self,value):
		if 0 <= value <= 95:
			self._p4eq_eq4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_eq4_gain(self):
		'''(P4EQ) EQ4 Gain'''
		return self._p4eq_eq4_gain

	@p4eq_eq4_gain.setter
	def p4eq_eq4_gain(self,value):
		if -18 <= value <= 18:
			self._p4eq_eq4_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def p4eq_switch(self):
		'''(P4EQ) Switch'''
		return self._p4eq_switch

	@p4eq_switch.setter
	def p4eq_switch(self,value):
		if 0 <= value <= 1:
			self._p4eq_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

