#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx182.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX182(object):
	category = '''Double Size'''
	description = '''EP Cabinet/Drive'''
	short_description = '''EP Cabinet/Drive'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX182 EP Cabinet/Drive object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.drive_switch = (bytes[3] >> 5) & 0x1
		self.drive_switch_mod_source = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xF) << 2)
		self.drive_switch_mode = (bytes[4] >> 4) & 0x1
		self.drive = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.drive_mod_source = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.drive_mod_intensity = data
		self.cabinet_type = (bytes[7] >> 2) & 0x3

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX182 EP Cabinet/Drive object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.drive_switch & 0x1) << 5) | ((self.drive_switch_mod_source << 6) & 0xC0)
		bytes[4] = ((self.drive_switch_mod_source >> 2) & 0xF) | ((self.drive_switch_mode & 0x1) << 4) | ((self.drive << 5) & 0xE0)
		bytes[5] = ((self.drive >> 3) & 0xF) | ((self.drive_mod_source << 4) & 0xF0)
		bytes[6] = ((self.drive_mod_source >> 4) & 0x3) | ((self.drive_mod_intensity << 2) & 0xFC)
		bytes[7] = ((self.drive_mod_intensity >> 6) & 0x3) | ((self.cabinet_type & 0x3) << 2)
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX182 EP Cabinet/Drive object'''
		json = []
		json.append(('name','FX182'))
		json.append(('description','EP Cabinet/Drive'))
		json.append(['drive_switch',self.drive_switch])
		json.append(['drive_switch_mod_source',self.drive_switch_mod_source])
		json.append(['drive_switch_mode',self.drive_switch_mode])
		json.append(['drive',self.drive])
		json.append(['drive_mod_source',self.drive_mod_source])
		json.append(['drive_mod_intensity',self.drive_mod_intensity])
		json.append(['cabinet_type',self.cabinet_type])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch(self):
		'''Drive Switch'''
		return self._drive_switch

	@drive_switch.setter
	def drive_switch(self,value):
		if 0 <= value <= 1:
			self._drive_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch_mod_source(self):
		'''Drive Switch Mod.Source'''
		return self._drive_switch_mod_source

	@drive_switch_mod_source.setter
	def drive_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._drive_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch_mode(self):
		'''Drive Switch Mode'''
		return self._drive_switch_mode

	@drive_switch_mode.setter
	def drive_switch_mode(self,value):
		if 0 <= value <= 1:
			self._drive_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def drive(self):
		'''Drive'''
		return self._drive

	@drive.setter
	def drive(self,value):
		if 0 <= value <= 100:
			self._drive = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_mod_source(self):
		'''Drive Mod.Source'''
		return self._drive_mod_source

	@drive_mod_source.setter
	def drive_mod_source(self,value):
		if 0 <= value <= 37:
			self._drive_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_mod_intensity(self):
		'''Drive Mod.Intensity'''
		return self._drive_mod_intensity

	@drive_mod_intensity.setter
	def drive_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._drive_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_type(self):
		'''Cabinet Type'''
		return self._cabinet_type

	@cabinet_type.setter
	def cabinet_type(self,value):
		if 0 <= value <= 2:
			self._cabinet_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

