#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx184.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX184(object):
	category = '''Double Size'''
	description = '''Multitap Chorus/Delay 6Taps'''
	short_description = '''Multitap Cho/Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX184 Multitap Chorus/Delay 6Taps object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.lfo_frequency = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x7) << 3)
		self.tap1_time = ((bytes[4] & 0xF8) >> 3) | ((bytes[5] & 0xF) << 5)
		self.tap1_depth = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x1) << 4)
		self.tap1_status = (bytes[6] >> 1) & 0x3
		self.tap2_time = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0xF) << 5)
		self.tap2_depth = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x1) << 4)
		self.tap2_status = (bytes[8] >> 1) & 0x3
		self.tap3_time = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0xF) << 5)
		self.tap3_depth = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x1) << 4)
		self.tap3_status = (bytes[10] >> 1) & 0x3
		self.tap4_time = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0xF) << 5)
		self.tap4_depth = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x1) << 4)
		self.tap4_status = (bytes[12] >> 1) & 0x3
		self.tap5_time = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0xF) << 5)
		self.tap5_depth = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x1) << 4)
		self.tap5_status = (bytes[14] >> 1) & 0x3
		self.tap6_time = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0xF) << 5)
		self.tap6_depth = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x1) << 4)
		self.tap6_status = (bytes[16] >> 1) & 0x3
		self.panning_preset = (bytes[16] >> 3) & 0x3
		data = ((bytes[16] & 0xE0) >> 5) | ((bytes[17] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.tap1_feedback = data
		self.tap1_feedback_mod_source = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x7) << 3)
		data = ((bytes[18] & 0xF8) >> 3) | ((bytes[19] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.tap1_feedback_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX184 Multitap Chorus/Delay 6Taps object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.lfo_frequency << 5) & 0xE0
		bytes[4] = ((self.lfo_frequency >> 3) & 0x7) | ((self.tap1_time << 3) & 0xF8)
		bytes[5] = ((self.tap1_time >> 5) & 0xF) | ((self.tap1_depth << 4) & 0xF0)
		bytes[6] = ((self.tap1_depth >> 4) & 0x1) | ((self.tap1_status & 0x3) << 1) | ((self.tap2_time << 3) & 0xF8)
		bytes[7] = ((self.tap2_time >> 5) & 0xF) | ((self.tap2_depth << 4) & 0xF0)
		bytes[8] = ((self.tap2_depth >> 4) & 0x1) | ((self.tap2_status & 0x3) << 1) | ((self.tap3_time << 3) & 0xF8)
		bytes[9] = ((self.tap3_time >> 5) & 0xF) | ((self.tap3_depth << 4) & 0xF0)
		bytes[10] = ((self.tap3_depth >> 4) & 0x1) | ((self.tap3_status & 0x3) << 1) | ((self.tap4_time << 3) & 0xF8)
		bytes[11] = ((self.tap4_time >> 5) & 0xF) | ((self.tap4_depth << 4) & 0xF0)
		bytes[12] = ((self.tap4_depth >> 4) & 0x1) | ((self.tap4_status & 0x3) << 1) | ((self.tap5_time << 3) & 0xF8)
		bytes[13] = ((self.tap5_time >> 5) & 0xF) | ((self.tap5_depth << 4) & 0xF0)
		bytes[14] = ((self.tap5_depth >> 4) & 0x1) | ((self.tap5_status & 0x3) << 1) | ((self.tap6_time << 3) & 0xF8)
		bytes[15] = ((self.tap6_time >> 5) & 0xF) | ((self.tap6_depth << 4) & 0xF0)
		bytes[16] = ((self.tap6_depth >> 4) & 0x1) | ((self.tap6_status & 0x3) << 1) | ((self.panning_preset & 0x3) << 3) | ((self.tap1_feedback << 5) & 0xE0)
		bytes[17] = ((self.tap1_feedback >> 3) & 0x1F) | ((self.tap1_feedback_mod_source << 5) & 0xE0)
		bytes[18] = ((self.tap1_feedback_mod_source >> 3) & 0x7) | ((self.tap1_feedback_mod_intensity << 3) & 0xF8)
		bytes[19] = (self.tap1_feedback_mod_intensity >> 5) & 0x7
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX184 Multitap Chorus/Delay 6Taps object'''
		json = []
		json.append(('name','FX184'))
		json.append(('description','Multitap Chorus/Delay 6Taps'))
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['tap1_time',self.tap1_time])
		json.append(['tap2_time',self.tap2_time])
		json.append(['tap3_time',self.tap3_time])
		json.append(['tap4_time',self.tap4_time])
		json.append(['tap5_time',self.tap5_time])
		json.append(['tap6_time',self.tap6_time])
		json.append(['tap1_depth',self.tap1_depth])
		json.append(['tap2_depth',self.tap2_depth])
		json.append(['tap3_depth',self.tap3_depth])
		json.append(['tap4_depth',self.tap4_depth])
		json.append(['tap5_depth',self.tap5_depth])
		json.append(['tap6_depth',self.tap6_depth])
		json.append(['tap1_status',self.tap1_status])
		json.append(['tap2_status',self.tap2_status])
		json.append(['tap3_status',self.tap3_status])
		json.append(['tap4_status',self.tap4_status])
		json.append(['tap5_status',self.tap5_status])
		json.append(['tap6_status',self.tap6_status])
		json.append(['tap1_feedback',self.tap1_feedback])
		json.append(['tap1_feedback_mod_intensity',self.tap1_feedback_mod_intensity])
		json.append(['panning_preset',self.panning_preset])
		json.append(['tap1_feedback_mod_source',self.tap1_feedback_mod_source])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 63:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_time(self):
		'''Tap1 Time'''
		return self._tap1_time

	@tap1_time.setter
	def tap1_time(self,value):
		if 0 <= value <= 155:
			self._tap1_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_depth(self):
		'''Tap1 Depth'''
		return self._tap1_depth

	@tap1_depth.setter
	def tap1_depth(self,value):
		if 0 <= value <= 30:
			self._tap1_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_status(self):
		'''Tap1 Status'''
		return self._tap1_status

	@tap1_status.setter
	def tap1_status(self,value):
		if 0 <= value <= 3:
			self._tap1_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_time(self):
		'''Tap2 Time'''
		return self._tap2_time

	@tap2_time.setter
	def tap2_time(self,value):
		if 0 <= value <= 155:
			self._tap2_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_depth(self):
		'''Tap2 Depth'''
		return self._tap2_depth

	@tap2_depth.setter
	def tap2_depth(self,value):
		if 0 <= value <= 30:
			self._tap2_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_status(self):
		'''Tap2 Status'''
		return self._tap2_status

	@tap2_status.setter
	def tap2_status(self,value):
		if 0 <= value <= 3:
			self._tap2_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_time(self):
		'''Tap3 Time'''
		return self._tap3_time

	@tap3_time.setter
	def tap3_time(self,value):
		if 0 <= value <= 155:
			self._tap3_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_depth(self):
		'''Tap3 Depth'''
		return self._tap3_depth

	@tap3_depth.setter
	def tap3_depth(self,value):
		if 0 <= value <= 30:
			self._tap3_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_status(self):
		'''Tap3 Status'''
		return self._tap3_status

	@tap3_status.setter
	def tap3_status(self,value):
		if 0 <= value <= 3:
			self._tap3_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_time(self):
		'''Tap4 Time'''
		return self._tap4_time

	@tap4_time.setter
	def tap4_time(self,value):
		if 0 <= value <= 155:
			self._tap4_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_depth(self):
		'''Tap4 Depth'''
		return self._tap4_depth

	@tap4_depth.setter
	def tap4_depth(self,value):
		if 0 <= value <= 30:
			self._tap4_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_status(self):
		'''Tap4 Status'''
		return self._tap4_status

	@tap4_status.setter
	def tap4_status(self,value):
		if 0 <= value <= 3:
			self._tap4_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap5_time(self):
		'''Tap5 Time'''
		return self._tap5_time

	@tap5_time.setter
	def tap5_time(self,value):
		if 0 <= value <= 155:
			self._tap5_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap5_depth(self):
		'''Tap5 Depth'''
		return self._tap5_depth

	@tap5_depth.setter
	def tap5_depth(self,value):
		if 0 <= value <= 30:
			self._tap5_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap5_status(self):
		'''Tap5 Status'''
		return self._tap5_status

	@tap5_status.setter
	def tap5_status(self,value):
		if 0 <= value <= 3:
			self._tap5_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap6_time(self):
		'''Tap6 Time'''
		return self._tap6_time

	@tap6_time.setter
	def tap6_time(self,value):
		if 0 <= value <= 155:
			self._tap6_time = value
		else:
			raise ValueError('%s not 0 <= value <= 155' % value)

#-------------------------------------------------------------------------------
	@property
	def tap6_depth(self):
		'''Tap6 Depth'''
		return self._tap6_depth

	@tap6_depth.setter
	def tap6_depth(self,value):
		if 0 <= value <= 30:
			self._tap6_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def tap6_status(self):
		'''Tap6 Status'''
		return self._tap6_status

	@tap6_status.setter
	def tap6_status(self,value):
		if 0 <= value <= 3:
			self._tap6_status = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def panning_preset(self):
		'''Panning Preset'''
		return self._panning_preset

	@panning_preset.setter
	def panning_preset(self,value):
		if 0 <= value <= 3:
			self._panning_preset = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback(self):
		'''Tap1 Feedback'''
		return self._tap1_feedback

	@tap1_feedback.setter
	def tap1_feedback(self,value):
		if -100 <= value <= 100:
			self._tap1_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback_mod_source(self):
		'''Tap1 Feedback Mod.Source'''
		return self._tap1_feedback_mod_source

	@tap1_feedback_mod_source.setter
	def tap1_feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._tap1_feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback_mod_intensity(self):
		'''Tap1 Feedback Mod.Intensity'''
		return self._tap1_feedback_mod_intensity

	@tap1_feedback_mod_intensity.setter
	def tap1_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._tap1_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

