#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx185.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX185(object):
	category = '''Double Size'''
	description = '''Stereo Pitch Shifter'''
	short_description = '''St. Pitch Shifter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX185 Stereo Pitch Shifter object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode = (bytes[3] >> 5) & 0x3
		self.l_r_pitch = (bytes[3] >> 7) & 0x1
		data = bytes[4] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pitch_shift_coarse = data
		self.pitch_shift_mod_source = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0xF) << 2)
		data = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.pitch_shift_coarse_mod_intensity = data
		data = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift_fine = data
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift_fine_mod_intensity = data
		self.l_delay_time = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x7) << 6)
		self.r_delay_time = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0xF) << 5)
		self.feedback_position = (bytes[10] >> 4) & 0x1
		data = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		self.input_trim_mod_source = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x3) << 4)
		data = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		data = ((bytes[14] & 0xFC) >> 2) | ((bytes[15] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.spread = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX185 Stereo Pitch Shifter object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode & 0x3) << 5) | ((self.l_r_pitch & 0x1) << 7)
		bytes[4] = (self.pitch_shift_coarse & 0x3F) | ((self.pitch_shift_mod_source << 6) & 0xC0)
		bytes[5] = ((self.pitch_shift_mod_source >> 2) & 0xF) | ((self.pitch_shift_coarse_mod_intensity << 4) & 0xF0)
		bytes[6] = ((self.pitch_shift_coarse_mod_intensity >> 4) & 0x3) | ((self.pitch_shift_fine << 2) & 0xFC)
		bytes[7] = ((self.pitch_shift_fine >> 6) & 0x3) | ((self.pitch_shift_fine_mod_intensity << 2) & 0xFC)
		bytes[8] = ((self.pitch_shift_fine_mod_intensity >> 6) & 0x3) | ((self.l_delay_time << 2) & 0xFC)
		bytes[9] = ((self.l_delay_time >> 6) & 0x7) | ((self.r_delay_time << 3) & 0xF8)
		bytes[10] = ((self.r_delay_time >> 5) & 0xF) | ((self.feedback_position & 0x1) << 4) | ((self.feedback << 5) & 0xE0)
		bytes[11] = ((self.feedback >> 3) & 0x1F) | ((self.high_damp << 5) & 0xE0)
		bytes[12] = ((self.high_damp >> 3) & 0xF) | ((self.input_trim_mod_source << 4) & 0xF0)
		bytes[13] = ((self.input_trim_mod_source >> 4) & 0x3) | ((self.input_trim_mod_intensity << 2) & 0xFC)
		bytes[14] = ((self.input_trim_mod_intensity >> 6) & 0x3) | ((self.spread << 2) & 0xFC)
		bytes[15] = (self.spread >> 6) & 0x3
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX185 Stereo Pitch Shifter object'''
		json = []
		json.append(('name','FX185'))
		json.append(('description','Stereo Pitch Shifter'))
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['mode',self.mode])
		json.append(['l_r_pitch',self.l_r_pitch])
		json.append(['pitch_shift_coarse',self.pitch_shift_coarse])
		json.append(['pitch_shift_mod_source',self.pitch_shift_mod_source])
		json.append(['pitch_shift_coarse_mod_intensity',self.pitch_shift_coarse_mod_intensity])
		json.append(['pitch_shift_fine',self.pitch_shift_fine])
		json.append(['pitch_shift_fine_mod_intensity',self.pitch_shift_fine_mod_intensity])
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['feedback_position',self.feedback_position])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['spread',self.spread])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 2:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def l_r_pitch(self):
		'''L/R Pitch'''
		return self._l_r_pitch

	@l_r_pitch.setter
	def l_r_pitch(self,value):
		if 0 <= value <= 1:
			self._l_r_pitch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_coarse(self):
		'''Pitch Shift Coarse'''
		return self._pitch_shift_coarse

	@pitch_shift_coarse.setter
	def pitch_shift_coarse(self,value):
		if -24 <= value <= 24:
			self._pitch_shift_coarse = value
		else:
			raise ValueError('%s not -24 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_mod_source(self):
		'''Pitch Shift Mod.Source'''
		return self._pitch_shift_mod_source

	@pitch_shift_mod_source.setter
	def pitch_shift_mod_source(self,value):
		if 0 <= value <= 37:
			self._pitch_shift_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_coarse_mod_intensity(self):
		'''Pitch Shift Coarse Mod.Intensity'''
		return self._pitch_shift_coarse_mod_intensity

	@pitch_shift_coarse_mod_intensity.setter
	def pitch_shift_coarse_mod_intensity(self,value):
		if -24 <= value <= 24:
			self._pitch_shift_coarse_mod_intensity = value
		else:
			raise ValueError('%s not -24 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_fine(self):
		'''Pitch Shift Fine'''
		return self._pitch_shift_fine

	@pitch_shift_fine.setter
	def pitch_shift_fine(self,value):
		if -100 <= value <= 100:
			self._pitch_shift_fine = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_fine_mod_intensity(self):
		'''Pitch Shift Fine Mod.Intensity'''
		return self._pitch_shift_fine_mod_intensity

	@pitch_shift_fine_mod_intensity.setter
	def pitch_shift_fine_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._pitch_shift_fine_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 122:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 122' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 122:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 122' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_position(self):
		'''Feedback Position'''
		return self._feedback_position

	@feedback_position.setter
	def feedback_position(self,value):
		if 0 <= value <= 1:
			self._feedback_position = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -100 <= value <= 100:
			self._spread = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

