#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx187.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX187(object):
	category = '''Double Size'''
	description = '''Rotary Speaker OverDrive'''
	short_description = '''Rotary SpeakerOD'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX187 Rotary Speaker OverDrive object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.drive_switch = (bytes[3] >> 5) & 0x1
		self.drive_switch_mod_source = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xF) << 2)
		self.drive_switch_sw = (bytes[4] >> 4) & 0x1
		self.overdrive_gain = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.overdrive_level = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.overdrive_tone = (bytes[6] >> 3) & 0xF
		self.speaker_simulator = (bytes[6] >> 7) & 0x1
		self.mode_switch = bytes[7] & 0x1
		self.mode_switch_mod_source = (bytes[7] >> 1) & 0x3F
		self.mode_switch_mode = (bytes[7] >> 7) & 0x1
		self.speed_switch = bytes[8] & 0x1
		self.speed_switch_mod_source = (bytes[8] >> 1) & 0x3F
		self.speed_switch_mode = (bytes[8] >> 7) & 0x1
		self.manual_speed_control_mod_source = bytes[9] & 0x3F
		self.horn_acceleration = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x1F) << 2)
		self.horn_ratio = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1F) << 3)
		self.rotor_acceleration = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		self.rotor_ratio = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xF) << 4)
		self.horn_rotor_balance = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0x7) << 4)
		self.mic_distance = ((bytes[14] & 0xF8) >> 3) | ((bytes[15] & 0x3) << 5)
		self.mic_spread = ((bytes[15] & 0xFC) >> 2) | ((bytes[16] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX187 Rotary Speaker OverDrive object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.drive_switch & 0x1) << 5) | ((self.drive_switch_mod_source << 6) & 0xC0)
		bytes[4] = ((self.drive_switch_mod_source >> 2) & 0xF) | ((self.drive_switch_sw & 0x1) << 4) | ((self.overdrive_gain << 5) & 0xE0)
		bytes[5] = ((self.overdrive_gain >> 3) & 0xF) | ((self.overdrive_level << 4) & 0xF0)
		bytes[6] = ((self.overdrive_level >> 4) & 0x7) | ((self.overdrive_tone & 0xF) << 3) | ((self.speaker_simulator & 0x1) << 7)
		bytes[7] = (self.mode_switch & 0x1) | ((self.mode_switch_mod_source & 0x3F) << 1) | ((self.mode_switch_mode & 0x1) << 7)
		bytes[8] = (self.speed_switch & 0x1) | ((self.speed_switch_mod_source & 0x3F) << 1) | ((self.speed_switch_mode & 0x1) << 7)
		bytes[9] = (self.manual_speed_control_mod_source & 0x3F) | ((self.horn_acceleration << 6) & 0xC0)
		bytes[10] = ((self.horn_acceleration >> 2) & 0x1F) | ((self.horn_ratio << 5) & 0xE0)
		bytes[11] = ((self.horn_ratio >> 3) & 0x1F) | ((self.rotor_acceleration << 5) & 0xE0)
		bytes[12] = ((self.rotor_acceleration >> 3) & 0xF) | ((self.rotor_ratio << 4) & 0xF0)
		bytes[13] = ((self.rotor_ratio >> 4) & 0xF) | ((self.horn_rotor_balance << 4) & 0xF0)
		bytes[14] = ((self.horn_rotor_balance >> 4) & 0x7) | ((self.mic_distance << 3) & 0xF8)
		bytes[15] = ((self.mic_distance >> 5) & 0x3) | ((self.mic_spread << 2) & 0xFC)
		bytes[16] = (self.mic_spread >> 6) & 0x1
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX187 Rotary Speaker OverDrive object'''
		json = []
		json.append(('name','FX187'))
		json.append(('description','Rotary Speaker OverDrive'))
		json.append(['drive_switch',self.drive_switch])
		json.append(['drive_switch_mod_source',self.drive_switch_mod_source])
		json.append(['drive_switch_sw',self.drive_switch_sw])
		json.append(['overdrive_gain',self.overdrive_gain])
		json.append(['overdrive_level',self.overdrive_level])
		json.append(['overdrive_tone',self.overdrive_tone])
		json.append(['speaker_simulator',self.speaker_simulator])
		json.append(['mode_switch',self.mode_switch])
		json.append(['mode_switch_mod_source',self.mode_switch_mod_source])
		json.append(['mode_switch_mode',self.mode_switch_mode])
		json.append(['speed_switch',self.speed_switch])
		json.append(['speed_switch_mod_source',self.speed_switch_mod_source])
		json.append(['speed_switch_mode',self.speed_switch_mode])
		json.append(['manual_speed_control_mod_source',self.manual_speed_control_mod_source])
		json.append(['rotor_acceleration',self.rotor_acceleration])
		json.append(['rotor_ratio',self.rotor_ratio])
		json.append(['horn_acceleration',self.horn_acceleration])
		json.append(['horn_ratio',self.horn_ratio])
		json.append(['horn_rotor_balance',self.horn_rotor_balance])
		json.append(['mic_distance',self.mic_distance])
		json.append(['mic_spread',self.mic_spread])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch(self):
		'''Drive Switch'''
		return self._drive_switch

	@drive_switch.setter
	def drive_switch(self,value):
		if 0 <= value <= 1:
			self._drive_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch_mod_source(self):
		'''Drive Switch Mod.Source'''
		return self._drive_switch_mod_source

	@drive_switch_mod_source.setter
	def drive_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._drive_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch_sw(self):
		'''Drive Switch SW'''
		return self._drive_switch_sw

	@drive_switch_sw.setter
	def drive_switch_sw(self,value):
		if 0 <= value <= 1:
			self._drive_switch_sw = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def overdrive_gain(self):
		'''Overdrive Gain'''
		return self._overdrive_gain

	@overdrive_gain.setter
	def overdrive_gain(self,value):
		if 0 <= value <= 100:
			self._overdrive_gain = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def overdrive_level(self):
		'''Overdrive Level'''
		return self._overdrive_level

	@overdrive_level.setter
	def overdrive_level(self,value):
		if 0 <= value <= 100:
			self._overdrive_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def overdrive_tone(self):
		'''Overdrive Tone'''
		return self._overdrive_tone

	@overdrive_tone.setter
	def overdrive_tone(self,value):
		if 0 <= value <= 15:
			self._overdrive_tone = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def speaker_simulator(self):
		'''Speaker Simulator'''
		return self._speaker_simulator

	@speaker_simulator.setter
	def speaker_simulator(self,value):
		if 0 <= value <= 1:
			self._speaker_simulator = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch(self):
		'''Mode Switch'''
		return self._mode_switch

	@mode_switch.setter
	def mode_switch(self,value):
		if 0 <= value <= 1:
			self._mode_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch_mod_source(self):
		'''Mode Switch Mod.Source'''
		return self._mode_switch_mod_source

	@mode_switch_mod_source.setter
	def mode_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._mode_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def mode_switch_mode(self):
		'''Mode Switch Mode'''
		return self._mode_switch_mode

	@mode_switch_mode.setter
	def mode_switch_mode(self,value):
		if 0 <= value <= 1:
			self._mode_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch(self):
		'''Speed Switch'''
		return self._speed_switch

	@speed_switch.setter
	def speed_switch(self,value):
		if 0 <= value <= 1:
			self._speed_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch_mod_source(self):
		'''Speed Switch Mod.Source'''
		return self._speed_switch_mod_source

	@speed_switch_mod_source.setter
	def speed_switch_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_switch_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_switch_mode(self):
		'''Speed Switch Mode'''
		return self._speed_switch_mode

	@speed_switch_mode.setter
	def speed_switch_mode(self,value):
		if 0 <= value <= 1:
			self._speed_switch_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def manual_speed_control_mod_source(self):
		'''Manual Speed Control Mod.Source'''
		return self._manual_speed_control_mod_source

	@manual_speed_control_mod_source.setter
	def manual_speed_control_mod_source(self,value):
		if 0 <= value <= 37:
			self._manual_speed_control_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_acceleration(self):
		'''Horn Acceleration'''
		return self._horn_acceleration

	@horn_acceleration.setter
	def horn_acceleration(self,value):
		if 0 <= value <= 100:
			self._horn_acceleration = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_ratio(self):
		'''Horn Ratio'''
		return self._horn_ratio

	@horn_ratio.setter
	def horn_ratio(self,value):
		if 49 <= value <= 200:
			self._horn_ratio = value
		else:
			raise ValueError('%s not 49 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def rotor_acceleration(self):
		'''Rotor Acceleration'''
		return self._rotor_acceleration

	@rotor_acceleration.setter
	def rotor_acceleration(self,value):
		if 0 <= value <= 100:
			self._rotor_acceleration = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def rotor_ratio(self):
		'''Rotor Ratio'''
		return self._rotor_ratio

	@rotor_ratio.setter
	def rotor_ratio(self,value):
		if 49 <= value <= 200:
			self._rotor_ratio = value
		else:
			raise ValueError('%s not 49 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def horn_rotor_balance(self):
		'''Horn/Rotor Balance'''
		return self._horn_rotor_balance

	@horn_rotor_balance.setter
	def horn_rotor_balance(self,value):
		if 0 <= value <= 100:
			self._horn_rotor_balance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_distance(self):
		'''Mic Distance'''
		return self._mic_distance

	@mic_distance.setter
	def mic_distance(self,value):
		if 0 <= value <= 100:
			self._mic_distance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_spread(self):
		'''Mic Spread'''
		return self._mic_spread

	@mic_spread.setter
	def mic_spread(self,value):
		if 0 <= value <= 100:
			self._mic_spread = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

