#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx189.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX189(object):
	category = '''Double Size'''
	description = '''Stereo/Cross Long Delay'''
	short_description = '''St/Cross Long Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX189 Stereo/Cross Long Delay object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.stereo_or_cross = (bytes[3] >> 5) & 0x1
		self.l_delay_time = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0xFF) << 2)
		self.r_delay_time = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0xFF) << 3)
		data = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.l_feedback = data
		self.feedback_mod_source = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x3) << 4)
		data = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.l_feedback_mod_intensity = data
		data = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.r_feedback = data
		data = ((bytes[11] & 0xFC) >> 2) | ((bytes[12] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.r_feedback_mod_intensity = data
		self.high_damp = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x1) << 6)
		self.low_damp = (bytes[13] >> 1) & 0x7F
		self.input_trim_mod_source = bytes[14] & 0x3F
		data = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		data = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.spread = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX189 Stereo/Cross Long Delay object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.stereo_or_cross & 0x1) << 5) | ((self.l_delay_time << 6) & 0xC0)
		bytes[4] = (self.l_delay_time >> 2) & 0xFF
		bytes[5] = (self.r_delay_time << 5) & 0xE0
		bytes[6] = (self.r_delay_time >> 3) & 0xFF
		bytes[7] = (self.l_feedback << 4) & 0xF0
		bytes[8] = ((self.l_feedback >> 4) & 0xF) | ((self.feedback_mod_source << 4) & 0xF0)
		bytes[9] = ((self.feedback_mod_source >> 4) & 0x3) | ((self.l_feedback_mod_intensity << 2) & 0xFC)
		bytes[10] = ((self.l_feedback_mod_intensity >> 6) & 0x3) | ((self.r_feedback << 2) & 0xFC)
		bytes[11] = ((self.r_feedback >> 6) & 0x3) | ((self.r_feedback_mod_intensity << 2) & 0xFC)
		bytes[12] = ((self.r_feedback_mod_intensity >> 6) & 0x3) | ((self.high_damp << 2) & 0xFC)
		bytes[13] = ((self.high_damp >> 6) & 0x1) | ((self.low_damp & 0x7F) << 1)
		bytes[14] = (self.input_trim_mod_source & 0x3F) | ((self.input_trim_mod_intensity << 6) & 0xC0)
		bytes[15] = ((self.input_trim_mod_intensity >> 2) & 0x3F) | ((self.spread << 6) & 0xC0)
		bytes[16] = (self.spread >> 2) & 0x1F
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX189 Stereo/Cross Long Delay object'''
		json = []
		json.append(('name','FX189'))
		json.append(('description','Stereo/Cross Long Delay'))
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['l_feedback',self.l_feedback])
		json.append(['r_feedback',self.r_feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['l_feedback_mod_intensity',self.l_feedback_mod_intensity])
		json.append(['r_feedback_mod_intensity',self.r_feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['stereo_or_cross',self.stereo_or_cross])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def stereo_or_cross(self):
		'''Stereo Or Cross'''
		return self._stereo_or_cross

	@stereo_or_cross.setter
	def stereo_or_cross(self,value):
		if 0 <= value <= 1:
			self._stereo_or_cross = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 27300:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 27300' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 27300:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 27300' % value)

#-------------------------------------------------------------------------------
	@property
	def l_feedback(self):
		'''L Feedback'''
		return self._l_feedback

	@l_feedback.setter
	def l_feedback(self,value):
		if -100 <= value <= 100:
			self._l_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def l_feedback_mod_intensity(self):
		'''L Feedback Mod.Intensity'''
		return self._l_feedback_mod_intensity

	@l_feedback_mod_intensity.setter
	def l_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._l_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_feedback(self):
		'''R Feedback'''
		return self._r_feedback

	@r_feedback.setter
	def r_feedback(self,value):
		if -100 <= value <= 100:
			self._r_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_feedback_mod_intensity(self):
		'''R Feedback Mod.Intensity'''
		return self._r_feedback_mod_intensity

	@r_feedback_mod_intensity.setter
	def r_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._r_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -50 <= value <= 50:
			self._spread = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

