#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx193.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX193(object):
	category = '''Double Size'''
	description = '''Early Reflections Hi Dens'''
	short_description = '''Early Reflections'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX193 Early Reflections Hi Dens object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.type = (bytes[3] >> 5) & 0x3
		self.er_time = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x7F) << 1)
		self.pre_delay = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x7F) << 1)
		data = ((bytes[5] & 0x80) >> 7) | ((bytes[6] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.pre_leq_gain = data
		data = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.pre_heq_gain = data
		self.pre_leq_fc = (bytes[7] >> 3) & 0x1
		self.pre_heq_fc = (bytes[7] >> 4) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX193 Early Reflections Hi Dens object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.type & 0x3) << 5) | ((self.er_time << 7) & 0x80)
		bytes[4] = ((self.er_time >> 1) & 0x7F) | ((self.pre_delay << 7) & 0x80)
		bytes[5] = ((self.pre_delay >> 1) & 0x7F) | ((self.pre_leq_gain << 7) & 0x80)
		bytes[6] = ((self.pre_leq_gain >> 1) & 0x1F) | ((self.pre_heq_gain << 5) & 0xE0)
		bytes[7] = ((self.pre_heq_gain >> 3) & 0x7) | ((self.pre_leq_fc & 0x1) << 3) | ((self.pre_heq_fc & 0x1) << 4)
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX193 Early Reflections Hi Dens object'''
		json = []
		json.append(('name','FX193'))
		json.append(('description','Early Reflections Hi Dens'))
		json.append(['type',self.type])
		json.append(['er_time',self.er_time])
		json.append(['pre_delay',self.pre_delay])
		json.append(['input_trim',self.input_trim])
		json.append(['pre_leq_gain',self.pre_leq_gain])
		json.append(['pre_heq_gain',self.pre_heq_gain])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['pre_leq_fc',self.pre_leq_fc])
		json.append(['pre_heq_fc',self.pre_heq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 3:
			self._type = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def er_time(self):
		'''ER Time'''
		return self._er_time

	@er_time.setter
	def er_time(self,value):
		if 10 <= value <= 138:
			self._er_time = value
		else:
			raise ValueError('%s not 10 <= value <= 138' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_delay(self):
		'''Pre Delay'''
		return self._pre_delay

	@pre_delay.setter
	def pre_delay(self,value):
		if 0 <= value <= 100:
			self._pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_gain(self):
		'''Pre LEQ Gain'''
		return self._pre_leq_gain

	@pre_leq_gain.setter
	def pre_leq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_leq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_gain(self):
		'''Pre HEQ Gain'''
		return self._pre_heq_gain

	@pre_heq_gain.setter
	def pre_heq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_heq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_fc(self):
		'''Pre LEQ Fc'''
		return self._pre_leq_fc

	@pre_leq_fc.setter
	def pre_leq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_leq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_fc(self):
		'''Pre HEQ Fc'''
		return self._pre_heq_fc

	@pre_heq_fc.setter
	def pre_heq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_heq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

