#!@PYTHON@
# -*- coding: utf-8 -*-
#
# table10.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

from song import Song
import struct
import sys
import table1
import table2
import table6
import table7
import table9
import table10
import table11
import table12

_debug = False

#===============================================================================
class Chunk(object):
	def __init__(self,tag,chunklen,version,formt,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.formt = formt
		self.flags = flags
			
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,level=0):
		self.offset = offset
		self.level = level
		
		offset += 12
		if self.formt == 0x00:
			if self.tag == 'CBK1':
				self.record_count,self.record_length,self.record_id = struct.unpack('>3I',data[offset:offset+12])
				offset += 12
				self.data = []
				for i in xrange(self.record_count):
					child = table2.CombinationParameters(data[offset:offset+self.record_length])
					self.data.append(child)
					offset += self.record_length
			elif self.tag == 'DBK1':
				self.record_count,self.record_length,self.record_id = struct.unpack('>3I',data[offset:offset+12])
				offset += 12
				self.data = []
				for i in xrange(self.record_count):
					child = table6.DrumKit(data[offset:offset+self.record_length])
					self.data.append(child)
					offset += self.record_length
			elif self.tag == 'DIV1':
				self.data = struct.unpack('>%dH' % (self.chunklen/2),data[offset:offset+self.chunklen])
			elif self.tag == 'INI1':
				self.data = []
				self.record_count, = struct.unpack('>I',data[offset:offset+4])
				offset += 4
				for i in xrange(self.record_count):
					child = INI1Entry(data[offset:offset+52])
					self.data.append(child)
					offset += 52
			elif self.tag == 'INI2':
				self.data = []
				self.record_count, = struct.unpack('>I',data[offset:offset+4])
				offset += 4
				for i in xrange(self.record_count):
					child = INI2Entry(data[offset:offset+64])
					self.data.append(child)
					offset += 64
			elif self.tag == 'PBK1':
				self.record_count,self.record_length,self.record_id = struct.unpack('>3I',data[offset:offset+12])
				offset += 12
				self.data = []
				for i in xrange(self.record_count):
					child = table1.ProgramParameters(data[offset:offset+self.record_length])
					self.data.append(child)
					offset += self.record_length
			else:
				self.data = data[offset:offset+self.chunklen]
		elif self.formt == 0x01:
			self.record_count,self.record_length,self.record_id = struct.unpack('>3I',data[offset:offset+12])
		elif self.formt == 0x04:
			self._children = []
			end_offset = offset+self.chunklen
#			print 'end_offset: 0x%08X' % end_offset
			while offset < end_offset:
				tag,length,version,formt,flags = struct.unpack('>4sIH2B',data[offset:offset+12])
				child = Chunk(tag,length,version,formt,flags)
				child.unpack(data,offset,level+1)
				self._children.append(child)
				offset += length + 12

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		if self.formt == 0x00:
			if self.tag == 'CBK1':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d %d %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,self.record_count,self.record_length,self.record_id))
				for d in self.data:
					fo.write('\t\t\t%s%s\n' % ('\t'*self.level,d.combination_name))
			elif self.tag == 'DBK1':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d %d %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,self.record_count,self.record_length,self.record_id))
				for d in self.data:
					fo.write('\t\t\t%s%s\n' % ('\t'*self.level,d.drumkit_name))
			elif self.tag == 'DIV1':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,len(self.data)))
				fo.write('\t\t\t%s%s\n' % ('\t'*self.level,self.data))
			elif self.tag == 'INI1':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,len(self.data)))
				for d in self.data:
					fo.write('\t\t\t%s%s\n' % ('\t'*self.level,d))
			elif self.tag == 'INI2':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,len(self.data)))
				for d in self.data:
					fo.write('\t\t\t%s%s\n' % ('\t'*self.level,d))
			elif self.tag == 'PBK1':
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d %d %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,self.record_count,self.record_length,self.record_id))
				for d in self.data:
					fo.write('\t\t\t%s%s\n' % ('\t'*self.level,d.program_name))
			else:
				fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,len(self.data)))
		elif self.formt == 0x01:
			fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X %d %d %d\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags,self.record_count,self.record_length,self.record_id))
		elif self.formt == 0x04:
			fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags))
			for child in self._children:
				child.display(fo)
		else:
			fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%02X 0x%02X\n' % (self.offset,'\t'*self.level,self.tag,self.chunklen,self.version,self.formt,self.flags))

#===============================================================================
class KORG(Chunk):
	'''Root of PCG file'''
	def __init__(self,product=0x95,filetype=1,majver=2,minver=0):
		self.tag = 'KORG'
		self.product = product
		self.filetype = filetype
		self.majver = majver
		self.minver = minver
		self.byte8 = 0
		self.byte9 = 1 if filetype == 0 else 0
		self.bytea = 0
		self.byteb = 0
		self.bytec = 0
		self.byted = 0
		self.bytee = 0
		self.bytef = 0
	
#-------------------------------------------------------------------------------
	def load(self,filename_or_stream):
		if isinstance(filename_or_stream,basestring):
			fo = open(filename_or_stream,'rb')
			do_close = True
		else:
			fo = filename_or_stream
			do_close = False
		data = fo.read()
		if do_close:
			fo.close
		self.unpack(data)

#-------------------------------------------------------------------------------
	def unpack(self,data):
		v = list(struct.unpack('<4s12B',data[:16]))
	#	print v
		self.tag = v.pop(0)
		self.product = v.pop(0)
		self.filetype = v.pop(0)
		self.majver = v.pop(0)
		self.minver = v.pop(0)
		self.byte8 = v.pop(0)
		self.byte9 = v.pop(0)
		self.bytea = v.pop(0)
		self.byteb = v.pop(0)
		self.bytec = v.pop(0)
		self.byted = v.pop(0)
		self.bytee = v.pop(0)
		self.bytef = v.pop(0)
		offset = 16
		if self.filetype == 1:
			v = list(struct.unpack('>10H',data[offset:offset+20]))
			offset += 20
	#		print v
			self.sha0 = v.pop(0)
			self.sha1 = v.pop(0)
			self.sha2 = v.pop(0)
			self.sha3 = v.pop(0)
			self.sha4 = v.pop(0)
			self.sha5 = v.pop(0)
			self.sha6 = v.pop(0)
			self.sha7 = v.pop(0)
			self.sha8 = v.pop(0)
			self.shb_size = v.pop(0)
	#		print self.shb_size
			v = list(struct.unpack('>%dI'%(self.shb_size/4),data[offset:offset+self.shb_size]))
	#		print v
			offset += self.shb_size
			self.shb0 = v.pop(0)
			self.song_count = v.pop(0)
			self.shb2 = v.pop(0)
			self.shb3 = v.pop(0)
			self.cuelist_count = v.pop(0)
			self.cuelist_size = v.pop(0)
			self.shb6 = v.pop(0)
			self.shb7 = v.pop(0)
			self.shb8 = v.pop(0)
			self.shb9 = v.pop(0)
			self.shba = v.pop(0)
			# a list of song numbers and the lengths of the track events (individual and master)
			self.song_index = [(v.pop(0),v.pop(0)) for i in xrange(self.song_count)]
			self.cuelist_index = [v.pop(0) for i in xrange(self.cuelist_count)]
			if self.shb0 != 0x00000000:
				print 'shb0: 0x%08X' % self.shb0
			if self.shb2 != 0x00000000:
				print 'shb2: 0x%08X' % self.shb2
			if self.shb3 != 0x00000000:
				print 'shb3: 0x%08X' % self.shb3
			if self.cuelist_count != 20:
				print 'cuelist_count: 0x%08X' % self.cuelist_count
			if self.cuelist_size != 0x000001AC:
				print 'cuelist_size: 0x%08X' % self.cuelist_size
			if self.shb6 != 0x00000008:
				print 'shb6: 0x%08X' % self.shb6
			if self.shb7 != 0x00100001:
				print 'shb7: 0x%08X' % self.shb7
			if self.shb9 != 0x08080808:
				print 'shb9: 0x%08X' % self.shb9
			if self.shba != 0x08000000:
				print 'shba: 0x%08X' % self.shba
	#		print ['%d:0x%08X' % index for index in self.song_index]
			if self.cuelist_index != [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]:
				print 'cuelist_index',self.cuelist_index
			print 'offset: 0x%08X' % offset
		tag,length,version,formt,flags = struct.unpack('>4sIH2B',data[offset:offset+12])
		self.root = Chunk(tag,length,version,formt,flags)
		self.root.unpack(data,offset,0)
		self.root.display()
		
#-------------------------------------------------------------------------------
	def pack(self):
		v = list()
		v.append(self.tag)
		v.append(self.product)
		v.append(self.filetype)
		v.append(self.majver)
		v.append(self.minver)
		v.append(self.byte8)
		v.append(self.byte9)
		v.append(self.bytea)
		v.append(self.byteb)
		v.append(self.bytec)
		v.append(self.byted)
		v.append(self.bytee)
		v.append(self.bytef)
		return struct.pack('<4s12B',*v)

#===============================================================================
class INI1Entry(object):
	def __init__(self,data):
		if data is None:
			pass
		else:
			self.unpack(data)

#-------------------------------------------------------------------------------
	def __str__(self):
		return '%s 0x%05X %2d 0x%08X 0x%08X %3d %5d 0x%05X %-20s' % (self.tag,\
		self.bankno,self.u2,self.u3,self.bank_offset,self.record_count,self.record_length,\
		self.u7,self.bank_name)

#-------------------------------------------------------------------------------
	def unpack(self,data):
		v = list(struct.unpack('>4s2I5I20s',data))
		self.tag = v.pop(0)
		self.bankno = v.pop(0)
		self.u2 = v.pop(0)
		self.u3 = v.pop(0)
		self.bank_offset = v.pop(0)
		self.record_count = v.pop(0)
		self.record_length = v.pop(0)
		self.u7 = v.pop(0)
		self.bank_name = v.pop(0).replace('\0','')
		pass

#===============================================================================
class INI2Entry(object):
	def __init__(self,data):
		if data is None:
			pass
		else:
			self.unpack(data)

#-------------------------------------------------------------------------------
	def __str__(self):
		return '%s 0x%05X %2d 0x%08X 0x%08X %3d %5d 0x%05X %-20s 0x%04X 0x%02X 0x%02X %d %d' % (self.tag,\
		self.bankno,self.u2,self.u3,self.bank_offset,self.record_count,self.record_length,\
		self.u7,self.bank_name,self.version,self.u10,self.u11,self.u12,self.u13)

#-------------------------------------------------------------------------------
	def unpack(self,data):
#		ctag,s1,l1,o1,bank_offset,n,l,s2,name,d1,d2,d3 = struct.unpack('>4s2I5I20s2H2I',data)
		v = list(struct.unpack('>4s2I5I20sH2B2I',data))
		self.tag = v.pop(0)
		self.bankno = v.pop(0)
		self.u2 = v.pop(0)
		self.u3 = v.pop(0)
		self.bank_offset = v.pop(0)
		self.record_count = v.pop(0)
		self.record_length = v.pop(0)
		self.u7 = v.pop(0)
		self.bank_name = v.pop(0).replace('\0','')
		self.version = v.pop(0)
		self.u10 = v.pop(0)
		self.u11 = v.pop(0)
		self.u12 = v.pop(0)
		self.u13 = v.pop(0)
		pass


#===============================================================================
class PCG1(Chunk):
	'''Root of PCG file'''
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.div1 = None
		self.ini2 = None
		self.ini1 = None
		self.prg1 = None
		self.cmb1 = None
		self.dkt1 = None
		self.arp1 = None
		self.glb1 = None
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
#		if _debug: print '%s %d 0x%x' % (self.tag,self.chunklen,self.chunkother)
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'DIV1':
				self.div1 = DIV1(tag,length,flags)
				self.div1.unpack(cdata,offset,indent+'\t')
			elif tag == 'INI2':
				self.ini2 = INI2(tag,length,flags)
				self.ini2.unpack(cdata,offset,indent+'\t')
			elif tag == 'INI1':
				self.ini1 = INI1(tag,length,flags)
				self.ini1.unpack(cdata,offset,indent+'\t')
			elif tag == 'PRG1':
				self.prg1 = None
				self.prg1 = PRG1(tag,length,flags)
				self.prg1.unpack(cdata,offset,indent+'\t')
			elif tag == 'CMB1':
				self.cmb1 = CMB1(tag,length,flags)
				self.cmb1.unpack(cdata,offset,indent+'\t')
			elif tag == 'DKT1':
				self.dkt1 = DKT1(tag,length,flags)
				self.dkt1.unpack(cdata,offset,indent+'\t')
			elif tag == 'ARP1':
				self.arp1 = ARP1(tag,length,flags)
				self.arp1.unpack(cdata,offset,indent+'\t')
			elif tag == 'GLB1':
				self.glb1 = GLB1(tag,length,flags)
				self.glb1.unpack(cdata,offset,indent+'\t')
			else:
				raise ValueError('Unrecognized tag %s' % tag)
#			self._children.append(globals()[tag](tag,length,flags,cdata,offset,indent+'\t'))
			offset += length

#===============================================================================
class SNG1(object):
#-------------------------------------------------------------------------------
	def __init__(self,tag='SNG1',chunklen=0,version=0,flags=0x400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.sdk1 = None
		self.pdx1 = None
		self.cue1 = None
		self.sgs1 = None
		self._children = []
		pass

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.data = str(data)
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
			if tag == 'SDK1':
				self.sdk1 = obj = SDK1(tag,length,version,flags)
			elif tag == 'PDX1':
				self.pdx1 = obj = PDX1(tag,length,version,flags)
			elif tag == 'CUE1':
				self.cue1 = obj = CUE1(tag,length,version,flags)
			elif tag == 'SGS1':
				self.sgs1 = obj = SGS1(tag,length,version,flags)
			else:
				obj = None
			if obj:
				obj.unpack(cdata,offset,indent+'\t')
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x 0x%x' % (tag,length,version,flags))
			offset += length
		pass

#-------------------------------------------------------------------------------
	def pack(self):
		header = struct.pack('>4s2I',self.tag,self.chunklen,self.flags)
		sdk1 = self.sdk1.pack() if self.sdk1 else ''
		pdx1 = self.pdx1.pack() if self.pdx1 else ''
		cue1 = self.cue1.pack() if self.cue1 else ''
		sgs1 = self.sgs1.pack() if self.sgs1 else ''
		parts = [header,sdk1,pdx1,cue1,sgs1]
		return ''.join(parts)
		
#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
#		if self.sdk1:
#			self.sdk1.display()
#		if self.pdx1:
#			self.pdx1.display()
#		if self.cue1:
#			self.cue1.display()
		for child in self._children:
			child.display()

#===============================================================================
class ABK1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r,blocklen,b = struct.unpack('>3I',data[0:12])
#		print '%d %d 0x%X' % (r,blocklen,b)
		data = data[12:]
#		return
#		print '0x%08X %s%s 0x%08X 0x%08X %d %d 0x%08X' % (offset,indent,tag,length,flags,r,blocklen,b)
		offset += 24
		count = 0
		for i in xrange(r):
			cdata = data[:blocklen]
			data = data[blocklen:]
			name = cdata[:24].strip()
#			obj = table4.ArpeggioPatternParameters(cdata)
#			print '0x%08X %s%-24s' % (offset,indent+'\t',name)
			offset += blocklen

#===============================================================================
class ARP1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
#			self._children.append(globals()[tag](tag,length,flags,cdata,offset,indent+'\t'))
			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'ABK1':
				obj = ABK1(tag,length,flags)
				obj.unpack(cdata,offset,indent+'\t')
			offset += length

#===============================================================================
class BMT1(object):
	def __init__(self,tag='BMT1',chunklen=2640,version=0x0030,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.number = 1
		self.length = 2628
		self.songno = None
		self.song_timbre_set = None

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._songs[index]
	
#-------------------------------------------------------------------------------
	def calcsize(self):
		self.length = self.song_timbre_set.calcsize()
#		print self.tag,self.chunklen, 12 + self.length
		self.chunklen = 12 + self.length
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.number,self.length,self.songno = struct.unpack('>3I',data[:12])
#		print self.number,self.length,self.songno
		self.song_timbre_set = table10.SongTimbreSet(data[12:])

#-------------------------------------------------------------------------------
	def pack(self):
		self.calcsize()
		header = struct.pack('>4s5I',self.tag,self.chunklen,self.flags,self.number,self.length,self.songno)
		parts = [header,self.song_timbre_set.pack()]
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.songno))

#===============================================================================
class CBK1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r,blocklen,b = struct.unpack('>3I',data[0:12])
#		print '%d %d 0x%X' % (r,blocklen,b)
		data = data[12:]
		offset += 24
		count = 0
		for i in xrange(r):
			cdata = data[:blocklen]
			data = data[blocklen:]
			name = cdata[:24].strip()
			self._children.append(table2.CombinationParameters(cdata))
#			print cdata[:24].strip()
			offset += blocklen

#===============================================================================
class CMB1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
#		print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.cbk1 = None
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'CBK1':
				self.cbk1 = CBK1(tag,length,flags)
				self.cbk1.unpack(cdata,offset,indent+'\t')
#			self._children.append(globals()[tag](tag,length,flags,cdata,offset,indent+'\t'))
#			print ['%02X' % ord(d) for d in cdata[:20]]
#			print struct.unpack('>III24s',
			offset += length

#===============================================================================
# A list of cue lists
class CUE1(object):
	def __init__(self,tag='CUE1',chunklen=0x217C,version=0x0000,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.entries = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self.entries)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self.entries[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.entry_count,self.entry_length,self.index = struct.unpack('>3I',data[:12])
		self.raw = data[12:]
#		print '%d %d 0x%X' % (number,blocklen,index)
#		if _debug: print '%s %d 0x%x %d %d %d' % (self.tag,self.chunklen,self.chunkother,number,length,index)
		pos = 12
		for i in xrange(self.entry_count):
			obj = table9.CueListData(data[pos:pos+self.entry_length])
			self.entries.append(obj)
			pos += self.entry_length

#-------------------------------------------------------------------------------
	def pack(self):
		header = struct.pack('>4s5I',self.tag,self.chunklen,self.flags,self.entry_count,self.entry_length,self.index)
		parts = [header,self.raw]
#		for entry in self.entries:
#			parts.append(entry.pack())
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.entry_count,self.entry_length,self.index))

#===============================================================================
class DBK1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r,blocklen,b = struct.unpack('>3I',data[0:12])
#		print r,blocklen,b,self.chunklen,r*blocklen
#		print '%d %d 0x%X' % (r,blocklen,b)
		data = data[12:]
#		print '0x%08X %s%s 0x%08X 0x%08X %d %d 0x%08X' % (offset,indent,tag,length,flags,r,blocklen,b)
		offset += 24
		count = 0
		for i in xrange(r):
			cdata = data[:blocklen]
			data = data[blocklen:]
			name = cdata[:24].strip()
#			obj = table6.(cdata)
#			print '0x%08X %s%-24s' % (offset,indent+'\t',name)
			offset += blocklen

#===============================================================================
class DIV1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._data = None

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self._data = struct.unpack('>26H',data)
#		print '%s%s 0x%08X 0x%08X %d %s' % (indent,tag,length,c,c,['%04X' % c for c in self._data])
#		print '0x%08X %s%s 0x%08X 0x%08X %d' % (offset,indent,tag,length,c,c)

#===============================================================================
class DKT1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'DBK1':
				obj = DBK1(tag,length,flags)
				obj.unpack(cdata,offset,indent+'\t')
#			self._children.append(globals()[tag](tag,length,flags,cdata,offset,indent+'\t'))
			offset += length

#===============================================================================
# Master track
class EVM1(object):
	def __init__(self,tag='EVM1',chunklen=0,version=0x0000,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.number = 0
		self.length = 8
		self.index = 0
		self.event_list = []
	
#-------------------------------------------------------------------------------
	def calcsize(self):
		self.length = 8
		self.chunklen = 12 + len(self.event_list) * self.length
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.number,self.length,self.index = struct.unpack('>3I',data[:12])
		self.offset = offset
		self.indent = indent
#		print self.number,self.length,self.index
		self.event_list = []
		data = data[12:]
		for i in xrange(self.number):
			self.event_list.append(data[i*8:(i+1)*8])

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.index))

#===============================================================================
class GLB1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
#		print ['%02X' % ord(d) for d in data[:512]]
#		for i in xrange(8,9224,72):
#			print data[i:i+24]
#			print '%s 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X' % struct.unpack('>24s12I',data[i:i+72])
#		obj = table3.GlobalParameters(data)
		pass
		

#===============================================================================
class INI1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r, = struct.unpack('>I',data[0:4])
		data = data[4:]
		offset += 16
		count = 0
		for i in xrange(r):
			ctag,s1,l1,o1,o2,n,l,s2,name = struct.unpack('>4s2I5I20s',data[i*52:(i+1)*52])
			name = name.replace('\0','')
			self._children.append((ctag,s1,l1,o1,o2,n,l,s2,name))
			offset += 52
#			print '0x%08X %s%s 0x%08X 0x%08X 0x%08X 0x%08X %6d %6d %6d %-20s' % (offset,indent+'\t',ctag,s1,l1,o1,o2,n,l,s2,name)

#===============================================================================
class INI2(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r, = struct.unpack('>I',data[0:4])
		data = data[4:]
		offset += 16
		for i in xrange(r):
			ctag,s1,l1,o1,bank_offset,n,l,s2,name,d1,d2,d3 = struct.unpack('>4s2I5I20s3I',data[i*64:(i+1)*64])
			name = name.replace('\0','')
			child = (ctag,s1,l1,o1,bank_offset,n,l,s2,name,d1,d2,d3)
			self._children.append(child)
			offset += 64
#			print '0x%08X %s%s 0x%08X 0x%08X 0x%08X 0x%08X %6d %6d %6d %-20s 0x%08X %6d %6d' % (offset,indent+'\t',ctag,s1,l1,o1,bank_offset,n,l,s2,name,d1,d2,d3)

#===============================================================================
class MDT1(object):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.tma1 = None
		self._children = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset=offset
		self.indent = indent
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
		#	print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			offset += 12
			if tag == 'MTK1':
				obj = self.mtk1 = MTK1(tag,length,version,flags)
			elif tag == 'TMA1':
				obj = self.tma1 = TMA1(tag,length,version,flags)
			else:
				obj = None
			if obj:
				obj.unpack(cdata,offset,indent+'\t')
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x 0x%x' % (tag,length,version,flags))
			offset += length
		
#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self._children:
			child.display(fo)

#===============================================================================
# MIDI track
class MTE1(object):#TrackEventList):
	def __init__(self,tag='MTE1',chunklen=0,version=0x0000,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.number = 0
		self.length = 8
		self.index = 0
		self.event_list = []
	
#-------------------------------------------------------------------------------
	def calcsize(self):
		self.length = 8
		self.chunklen = 12 + len(self.event_list) * self.length
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.number,self.length,self.index = struct.unpack('>3I',data[:12])
		self.offset = offset
		self.indent = indent
#		print self.number,self.length,self.index
		self.event_list = []
		data = data[12:]
		for i in xrange(self.number):
			self.event_list.append(data[i*8:(i+1)*8])

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.index))
#		fo.write('%d %s %d %d %d %d\n' % (self.offset,self.tag,self.chunklen,self.number,self.length,self.index))

#===============================================================================
class MTK1(object):
	def __init__(self,tag='MTK1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		if _debug: print '%s %d 0x%x' % (self.tag,self.chunklen,self.chunkother)
		pos = 0
		lim = len(data)
		offset += 12
		while pos < lim:
			tag,chunklen,chunkother = struct.unpack('>4sII',data[pos:pos+12])
			if tag == 'MTE1':
				obj = MTE1(tag,chunklen,chunkother)
			else:
				obj = None
			if obj:
				obj.unpack(data[pos+12:pos+chunklen+12],offset,indent+'\t')
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x' % (tag,chunklen,chunkother))
			pos += chunklen	+ 12
			offset += chunklen	+ 12
		pass

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self._children:
			child.display(fo)

#===============================================================================
class PBK1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		r,blocklen,self.index = struct.unpack('>3I',data[0:12])
#		print '%d %d 0x%X' % (r,blocklen,self.index)
		data = data[12:]
		return
		bank_name = bank_dict[offset+24][8]
#		print '0x%08X %s%s 0x%08X 0x%08X %d %d %d : %s' % (offset,indent,tag,length,flags,r,blocklen,b,bank_name)
		offset += 24
		count = 0
		for i in xrange(r):
			program = ProgramParameters(data[:blocklen])
			data = data[blocklen:]
			name = program.program_name.strip()
#			name = name.replace('\0','')
			self._children.append(program)
			print '0x%08X %s%-24s : %3d %3d %3d %3d %3d %3d %3d %3d' % (offset,indent+'\t',name,program.ifx1.effect_type,program.ifx2.effect_type,program.ifx3.effect_type,program.ifx4.effect_type,program.ifx5.effect_type,program.mfx1.effect_type,program.mfx2.effect_type,program.tfx.effect_type)
			offset += blocklen
	def __getitem__(self,index):
		return self._children[index]

#===============================================================================
class PDK1(object):
	'''A chunk containing a list of drum track parameter objects'''
	def __init__(self,tag='PDK1',chunklen=0x274,version=0x0000,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.patterns = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self.patterns)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self.patterns[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.number,self.length,self.index = struct.unpack('>3I',data[:12])
		self.patterns = []
		pos = 12
		for i in xrange(self.number):
			self.patterns.append(data[pos:pos+self.length])
#			self.patterns.append(DrumTrackPattern(data[pos:pos+length]))
			pos += self.length
		if _debug:
			for dtp in self:
				print '\t%s' % str(dtp)

#-------------------------------------------------------------------------------
	@property
	def patterns(self):
		return self._patterns
	
	@patterns.setter
	def patterns(self,value):
		self._patterns = value
	
#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.index))
		for child in self._patterns:
#			child.display(fo)
			print '%-24s 0x%08X' % struct.unpack('<24sI',child)
#			print '%-24s %d' % struct.unpack('<24sI',child)

#===============================================================================
class PDX1(object):
	def __init__(self,tag='PDX1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		pos = 0
		lim = len(data)
		while pos < lim:
			tag,length,version,flags = struct.unpack('>4sI2H',data[pos:pos+12])
			if tag == 'PDK1':
				self.pdk1 = obj = PDK1(tag,length,version,flags)
			else:
				obj = None
			if obj:
				obj.unpack(data[pos+12:pos+length+12],offset,indent+'\t')
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x 0x%x' % (tag,length,version,flags))
			pos += length	+ 12
		pass

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self._children:
			child.display(fo)

#===============================================================================
# Pattern events
class PEV1(object):#PatternEventList):
	def __init__(self,tag='PEV1',chunklen=0,version=0x0000,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.number,self.length,self.index = struct.unpack('>3I',data[:12])
#		PatternEventList.unpack(self,data[12:])
		if _debug:
			for event in self:
				print '\t%s' % str(event)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.index))
#		fo.write('%d %s %d %d %d %d\n' % (self.offset,self.tag,self.chunklen,self.number,self.length,self.index))

#===============================================================================
class PRG1(Chunk):
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
#		print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
		count = 0
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			count += 1
			offset += 12
#			self._children.append(globals()[tag](tag,length,flags,cdata,offset,indent+'\t'))
			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'PBK1':
				obj = PBK1(tag,length,flags)
				obj.unpack(cdata,offset,indent+'\t')
			offset += length
#		program = self._children[0][0]
#		json = program.toJSON()
#		print_json(('pbk',json))

	def __getitem__(self,index):
		return self._children[index]

#===============================================================================
class PTN1(object):
	def __init__(self,tag='PTN1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		pos = 0
		offset += 12
		lim = len(data)
		while pos < lim:
			tag,length,version,flags = struct.unpack('>4sI2H',data[pos:pos+12])
			if tag == 'PEV1':
				obj = PEV1(tag,length,version,flags)
			else:
				obj = None
			if obj:
				obj.unpack(data[pos+12:pos+length+12],offset,indent+'\t')
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x 0x%x' % (tag,length,version,flags))
			pos += length	+ 12
			offset += length	+ 12
		pass

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self._children:
			child.display(fo)

#===============================================================================
class SDK1(object):
	'''Directory of Song SDT1 chunks'''
	def __init__(self,tag,chunklen,version,flags):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.entries = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self.entries)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self.entries[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.entry_count,self.entry_length,self.index = struct.unpack('>3I',data[:12])
#		print nsongs,length,index
#		if _debug: print '%s %d 0x%x %d %d %d' % (self.tag,self.chunklen,self.chunkother,number,length,index)
		offset += 12
		data = data[12:]
		for entry in xrange(self.entry_count):
			cdata = data[:self.entry_length]
			data = data[self.entry_length:]
			obj = SDK1Entry(cdata)
#			obj.unpack(cdata)
			self.entries.append(obj)
			if _debug: print '0x%08X %sSDK1.entry %3d %s' % (offset,indent,entry,obj)
			offset += self.entry_length

#-------------------------------------------------------------------------------
	def pack(self):
		header = struct.pack('>4s5I',self.tag,self.chunklen,self.flags,self.entry_count,self.entry_length,self.index)
		parts = [header]
		for entry in self.entries:
			parts.append(entry.pack())
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.entry_count,self.entry_length,self.index))
		offset = self.offset+24
		indent = self.indent+'\t'
		for i,entry in enumerate(self.entries):
			fo.write('0x%08X %sSDK1.entry %3d %s\n' % (offset,indent,i,entry))
			
#===============================================================================
# RCC2014.SNG
#0 SDK1Entry SATBRL Template          0x2378 0x0000 0x0210 0x0000 0x0000 0x0000 0x0000 0x260c 0x23e4 0x23e4
#1 SDK1Entry Lonesome Valley          0x5120 0x2d98 0x0220 0x0000 0x0000 0x0000 0x0000 0x49fc 0x5240 0x5240
#2 SDK1Entry Didn't My Lord Deliver D 0x5fa0 0x3b68 0x02d0 0x0000 0x0000 0x0000 0x0000 0x9c48 0x6090 0x6090
#3 SDK1Entry The Glory of Love        0x92e8 0x6f58 0x0228 0x0000 0x0000 0x0000 0x0000 0xfce4 0x9420 0x9420
#4 SDK1Entry Beatles Love Songs       0x7e50 0x59a0 0x0348 0x0000 0x0000 0x0000 0x0000 0x19110 0x7f58 0x7f58
#5 SDK1Entry Blowin in the Wind/Ameri 0x8118 0x5ba0 0x0410 0x0000 0x0000 0x0000 0x0000 0x21074 0x8220 0x8220
#6 SDK1Entry California Dreamin'      0xee18 0xc6e0 0x05d0 0x0000 0x0000 0x0000 0x0000 0x292a0 0xef68 0xef68
#7 SDK1Entry Ticket to Ride           0x7bb0 0x5790 0x02b8 0x0000 0x0000 0x0000 0x0000 0x38214 0x7cd0 0x7cd0
#8 SDK1Entry Bridge Over Troubled Wat 0xaa40 0x8568 0x0370 0x0000 0x0000 0x0000 0x0000 0x3fef0 0xab48 0xab48
#9 SDK1Entry Scarboro/Sound Silence   0xabd8 0x8558 0x0518 0x0000 0x0000 0x0000 0x0000 0x4aa44 0xace0 0xace0
#10 SDK1Entry All My Trials            0x58a8 0x3438 0x0308 0x0000 0x0000 0x0000 0x0000 0x55730 0x59b0 0x59b0
#11 SDK1Entry California Dreamin'      0xebf8 0xc4c0 0x05d0 0x0000 0x0000 0x0000 0x0000 0x5b0ec 0xed48 0xed48
#12 SDK1Entry Bridge Over Troubled Wat 0x9720 0x7248 0x0370 0x0000 0x0000 0x0000 0x0000 0x69e40 0x9828 0x9828

# A = Total Memory required = Control Data (5924) + Timbre Set (2628) + Events (B) + Master Events (C)
# B = Event list Length
# C = Master track event list Length
# D = 
# E = 
# F = Pattern event list length, sum of (PEV1.chunklen-12)
# G = Count of PEV1 chunks
# H = offset to the contents of an SDT1 chunk
# I = length of the contents of an SDT1 chunk
# J = 
#	128SONGS.SNG                             A      B      C      D      E      F      G      H      I      J
#  0 SDK1Entry NEW SONG                 0x2378 0x0000 0x0210 0x0000 0x0000 0x0000 0x0000 0x4664 0x23e4 0x23e4
#  1 SDK1Entry NEW SONG                 0x2180 0x0000 0x0018 0x0000 0x0000 0x0000 0x0000 0x6a54 0x21ec 0x21ec
class SDK1Entry(object):
	'''SNG File Directory Entry'''
	def __init__(self,raw):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return 'SDK1Entry %s 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%x 0x%x 0x%x' % (self.name,self.v1,self.v2,self.event_list_length,self.v4,self.v5,self.v6,self.v7,self.data_offset,self.data_length,self.v10)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		v = list(struct.unpack('<24s10I',raw))
		self.name = v.pop(0)
		self.v1 = v.pop(0)
		self.v2 = v.pop(0)
		self.event_list_length= v.pop(0)
		self.v4 = v.pop(0)
		self.v5 = v.pop(0)
		self.v6 = v.pop(0)
		self.v7 = v.pop(0)
		self.data_offset = v.pop(0)
		self.data_length = v.pop(0)
		self.v10=  v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		v = []
		v.append(self.name)
		v.append(self.v1)
		v.append(self.v2)
		v.append(self.event_list_length)
		v.append(self.v4)
		v.append(self.v5)
		v.append(self.v6)
		v.append(self.v7)
		v.append(self.data_offset)
		v.append(self.data_length)
		v.append(self.v10)
		return struct.pack('<24s10I',*v)

#-------------------------------------------------------------------------------
	@property
	def name(self):
		return self._name

	@name.setter
	def name(self,value):
		self._name = (value+'                        ')[:24]

#===============================================================================
class SDT1(object):
	def __init__(self,tag='SDT1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self._children = []
		self.spr1 = None
		self.bmt1 = None
		self.trk1 = None
		self.ptn1 = None

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def calcsize(self):
		spr1_size = 0 if self.spr1 == None else self.spr1.calcsize()
		bmt1_size = 0 if self.bmt1 == None else self.bmt1.calcsize()
		trk1_size = 0 if self.trk1 == None else self.trk1.calcsize()
		ptn1_size = 0 if self.ptn1 == None else self.ptn1.calcsize()
#		print self.tag,self.chunklen,spr1_size + bmt1_size + trk1_size + ptn1_size
		self.chunklen = spr1_size + bmt1_size + trk1_size + ptn1_size
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
#		self.raw = data
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
			offset += 12
			if tag == 'SPR1':
				self.spr1 = obj = SPR1(tag,length,version,flags)
			elif tag == 'BMT1':
				self.bmt1 = obj = BMT1(tag,length,version,flags)
			elif tag == 'TRK1':
				self.trk1 = obj = TRK1(tag,length,version,flags)
			elif tag == 'PTN1':
				self.ptn1 = obj = PTN1(tag,length,version,flags)
			else:
				obj = None
			if obj:
				obj.unpack(cdata,offset,indent+'\t')
				self._children.append(obj)
			offset += length

#-------------------------------------------------------------------------------
	def pack(self):
		self.calcsize()
		header = struct.pack('>4s2I',self.tag,self.chunklen,self.flags)
		parts = [header]
	#	print self.spr1.chunklen
		if self.spr1 is not None:
			parts.append(self.spr1.pack())
		if self.bmt1 is not None:
			parts.append(self.bmt1.pack())
		if self.trk1 is not None:
			parts.append(self.trk1.pack())
		if self.ptn1 is not None:
			parts.append(self.ptn1.pack())
#		for entry in self.entries:
#			parts.append(entry.pack())
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self._children:
			child.display(fo)

#-------------------------------------------------------------------------------
	@property
	def song_no(self):
		if self.spr1 is not None:
			return self.spr1.songno
		if self.bmt1 is not None:
			return self.bmt1.songno

	@song_no.setter
	def song_no(self,value):
		if self.spr1 is not None:
			self.spr1.songno = value
		if self.bmt1 is not None:
			self.bmt1.songno = value

#===============================================================================
class SGS1(object):
	def __init__(self,tag='SGS1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
#		self._children = []
		self.sdt1 = []
		self.songs = {}
		pass

#-------------------------------------------------------------------------------
#	def __iter__(self):
#		return iter(self._children)

#-------------------------------------------------------------------------------
#	def __getitem__(self,index):
#		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
#			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			offset += 12
			if tag == 'SDT1':
				obj = SDT1(tag,length,version,flags)
				obj.unpack(cdata,offset,indent+'\t')
				self.sdt1.append(obj)
				song_no = obj.song_no
				song = Song()
				song.number = song_no
				self.songs[song_no] = song
				song.control_data = obj.spr1.song_control_data
				song.timbre_set = obj.bmt1.song_timbre_set
#				obj.trk1.mdt1.tma1.evm1.display()
#				print len(obj.trk1.mdt1.tma1.evm1.event_list)
				for event in obj.trk1.mdt1.tma1.evm1.event_list:
#					print struct.unpack('<8B',event)
					song.append_event(event)
#				print len(obj.trk1.mdt1.tma1.evm1.event_list)
			offset += length


#-------------------------------------------------------------------------------
	def pack(self):
		header = struct.pack('>4s2I',self.tag,self.chunklen,self.flags)
		parts = [header]
		for entry in self.sdt1:
			parts.append(entry.pack())
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		for child in self.sdt1:
			child.display(fo)

#===============================================================================
class SPR1(object):
	'''Chunk for a SongControlData structure'''
	def __init__(self,tag='SPR1',chunklen=5924,version=0x0300,flags=0x0100):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.number = 1
		self.length = 5924
		self.songno = None
		self.song_control_data = None

#-------------------------------------------------------------------------------
	def calcsize(self):
		self.length = self.song_control_data.calcsize()
#		print self.tag,self.chunklen, 12 + self.length
		self.chunklen = 12 + self.length
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.number,self.length,self.songno = struct.unpack('>3I',data[:12])
		self.song_control_data = table11.SongControlData(data[12:])

#-------------------------------------------------------------------------------
	def pack(self):
		self.length = self.song_control_data.calcsize()
		self.chunklen = 12 + self.length
		header = struct.pack('>4s5I',self.tag,self.chunklen,self.flags,self.number,self.length,self.songno)
		parts = [header,self.song_control_data.pack()]
		return ''.join(parts)

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X %d %d %d\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags,self.number,self.length,self.songno))

#===============================================================================
class TMA1(object):
	def __init__(self,tag='TMA1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
#		self._children = []

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		offset += 12
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
#			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			if tag == 'EVM1':
				obj = EVM1(tag,length,version,flags)
				obj.unpack(cdata,offset,indent+'\t')
				self.evm1 = obj
#				obj.display()
#				self._children.append(obj)
			offset += length + 12

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		self.evm1.display(fo)

#===============================================================================
class TRK1(object):
	def __init__(self,tag='TRK1',chunklen=0,version=0x0000,flags=0x0400):
		self.tag = tag
		self.chunklen = chunklen
		self.version = version
		self.flags = flags
		self.mdt1 = None
#		self.song_control_data = None
#		self._children = []

#-------------------------------------------------------------------------------
	def calcsize(self):
		self.length = len(self.raw)
#		print self.tag,self.chunklen, 12 + self.length
		self.chunklen = self.length
		return 12 + self.chunklen

#-------------------------------------------------------------------------------
	def unpack(self,data,offset,indent=''):
		self.offset = offset
		self.indent = indent
		self.raw = data#[12:]
		while data:
			tag,length,version,flags = struct.unpack('>4sI2H',data[0:12])
			cdata = data[12:12+length]
			data = data[12+length:]
#			print '0x%08X %s%s 0x%08X 0x%08X' % (offset,indent,tag,length,flags)
			offset += 12
			if tag == 'MDT1':
				self.mdt1 = MDT1(tag,length,version,flags)
				self.mdt1.unpack(cdata,offset,indent+'\t')
			offset += length
			
#-------------------------------------------------------------------------------
	def pack(self):
		self.calcsize()
		header = struct.pack('>4s2I',self.tag,self.chunklen,self.flags)
#		parts = [header,self.mdt1.pack()]
		parts = [header,self.raw]
		return ''.join(parts)

#		number,length,self.songno = struct.unpack('>3I',data[:12])
#		print number,length,self.songno
#		self.song_control_data = table11.SongControlData(data[12:])

#-------------------------------------------------------------------------------
	def display(self,fo=sys.stdout):
		fo.write('0x%08X %s%s 0x%08X 0x%04X 0x%04X\n' % (self.offset,self.indent,self.tag,self.chunklen,self.version,self.flags))
		self.mdt1.display(fo)

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._children)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._children[index]
	
#-------------------------------------------------------------------------------
	def unpackx(self,data,offset,indent=''):
		if _debug: print '%s %d 0x%x' % (self.tag,self.chunklen,self.chunkother)
		pos = 0
		lim = len(data)
		while pos < lim:
			tag,chunklen,chunkother = struct.unpack('>4sII',data[pos:pos+12])
			if tag == 'MDT1':
				obj = MDT1(tag,chunklen,chunkother)
			else:
				obj = None
			if obj:
				obj.unpack(data[pos+12:pos+chunklen+12])
				self._children.append(obj)
			else:
				raise TypeError('%s %d 0x%x' % (tag,chunklen,chunkother))
			pos += chunklen	+ 12
		pass


