#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import time
import multiprocessing
import struct
import midi
from modeparam import modeparam_dict
from table1 import ProgramParameters
import sysex
from Queue import Empty
import datetime
import table10
import table11
import table12

#===============================================================================
class Bank(object):
	def __init__(self):
		self._elements = [None for i in xrange(128)]
	
	def __setitem__(self,index,value):
		self._elements[index] = value
	
	def __getitem__(self,index):
		return self._elements[index]

#===============================================================================
class Song(object):
	def __init__(self):
		self._timbre_set = None
		self._control = None
		self._events = []
	
#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self._events)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self._events[index]
	
#===============================================================================
class StatusManager(multiprocessing.Process):
	def __init__(self,namespace,command_queue,response_queue):
		multiprocessing.Process.__init__(self)
		self._namespace = namespace
		self._command_queue = command_queue
		self._response_queue = response_queue
		self._running = False
		self._clock_count = 0
		self._programs = [Bank() for i in xrange(6)]
		self._combinations = [Bank() for i in xrange(6)]
		self._songs = [Song() for i in xrange(128)]
		self._drum_kits = [Bank() for i in xrange(2)]
		
#-------------------------------------------------------------------------------
#  def start(self):
#    print "Start Status Manager"

#-------------------------------------------------------------------------------
	def do_universal_nonrealtime(self,data):
		try:
			sub_id1 = data[3]
			sub_id2 = data[4]
			if sub_id1 == 0x6:
				if sub_id2 == 0x02:
					return sysex.InquiryReply(data)
		except IndexError: pass

#-------------------------------------------------------------------------------
	def do_universal_realtime(self,data):
		pass

#-------------------------------------------------------------------------------
	def do_result_report(self,data):
		rr = sysex.ResultReport()
		rr.code = data[7]
		return rr

#-------------------------------------------------------------------------------
	def do_parameter_change(self,data):
		value = (data[12]<<28)|(data[13]<<21)|(data[14]<<14)|(data[15]<<7)|data[16]
		try:
			mp = modeparam_dict[data[7]][data[9]][data[11]]
			data = sysex.ParameterChange(data[2] & 0x0F,data[7],data[9],data[11],value,mp['offset'],mp['bit'],mp['bits'],parameter=mp['parameter'],rng=mp['range'])
		except:
			data = sysex.ParameterChange(data[2] & 0x0F,data[7],data[9],data[11],value)
		return data
#		qin.put(csts_dump)

#-------------------------------------------------------------------------------
	def do_mode_change(self,data):
		return sysex.ModeChange(data[2] & 0x0F,data[7])

#-------------------------------------------------------------------------------
	def do_drum_kit_parameter_change(self,data):
		return None

#-------------------------------------------------------------------------------
	def do_arpeggio_pattern_parameter_change(self,data):
		return None

#-------------------------------------------------------------------------------
	def do_parameter_dump_request(self,data):
		obj = data[7]
		print 'request object',obj
		if obj == 0:  # Program TABLE 1
			bank = data[8]
			prog = data[10]
			pass
		elif obj == 0x01:  # Combination TABLE 2
			bank = data[8]
			comb = data[10]
			pass
		elif obj == 0x02:  # Song Timbre Set TABLE 10
			song = data[10]
			pass
		elif obj == 0x03:  # Global Data Dump TABLE 3
			pass
		elif obj == 0x04:  # Drum Kit Parameter Dump TABLE 6
			bank = data[8]
			kit = data[10]
			pass
		elif obj == 0x08:  # Song Control Data TABLE 11
			song = data[10]
			pass
		elif obj == 0x09:  # Song Event Data TABLE 12
			song = data[10]
			self._songs[song]._events = []
			fo = open('table12_%d.json' % song,'wb')
			fo.close()

#-------------------------------------------------------------------------------
	def do_parameter_dump(self,data):
		obj = data[7]
		sdata = struct.pack('>6B',*data[6:12])
		filename = 'dump/%02X/%s.sysex' % (obj,datetime.datetime.utcnow().isoformat())
		fo = open(filename,'wb')
		fo.write(sdata)
		try:
			mdata = sysex.convert_7_bit_to_8_bit(list(data[12:-1]))
		except Exception as exc:
			print exc
			mdata = ''.join([chr(d) for d in data[12:]])
		fo.write(mdata)
		fo.close()
		print 'object %02X saved to %s' % (obj,filename)
		if obj == 0:  # Program TABLE 1
			bank = data[8]
			prog = data[10]
			self._programs[bank][prog] = mdata
			return [ord(d) for d in self._programs[bank][prog]]
		elif obj == 0x01:  # Combination TABLE 2
			bank = data[8]
			comb = data[10]
			self._combinations[bank][comb] = mdata
			return [ord(d) for d in self._combinations[bank][comb]]
		elif obj == 0x02:  # Song Timbre Set TABLE 10
			song = data[10]
			try:
				self._songs[song]._timbre_set = table10.SongTimbreSet(mdata)
			except Exception as exc:
				print exc
				self._songs[song]._timbre_set = mdata
#			fo = open('table10_%d.json' % song,'wb')
#			fo.write(json.dumps(self._songs[song]._timbre_set.toJSON(),sort_keys=True,indent=2, separators=(',', ': ')))
#			fo.close()
#			return [ord(d) for d in self._songs[song]._timbre_set]
#			print json.dumps(self._songs[song]._timbre_set.toJSON(),sort_keys=True,indent=2, separators=(',', ': '))
		elif obj == 0x03:  # Global Data Dump TABLE 3
			self._global_data = mdata
			return [ord(d) for d in self._global_data]
		elif obj == 0x04:  # Drum Kit Parameter Dump TABLE 6
			bank = data[8]>>6
			kit = data[10]
			self._drum_kits[bank][kit] = mdata
#			return [ord(d) for d in self._drum_kits[bank][kit]]
		elif obj == 0x08:  # Song Control Data TABLE 11
			song = data[10]
			try:
				self._songs[song]._control = table11.SongControlData(mdata)
			except Exception as exc:
				print exc
				self._songs[song]._control = mdata
#			fo = open('table11_%d.json' % song,'wb')
#			fo.write(json.dumps(self._songs[song]._control.toJSON(),sort_keys=True,indent=2, separators=(',', ': ')))
#			fo.close()
		elif obj == 0x09:  # Song Event Data TABLE 12
			msgno = data[8]
			song = data[10]
			if msgno == 0:
				self._songs[song]._events = []
			try:
				song_events = table12.TrackEventList(mdata)
				self._songs[song]._events += song_events.events
			except Exception as exc:
				print exc
#			self._songs[song]._events = table12.SongControlData(tdata)
#			fo = open('table12_%d.json' % song,'wb')
#			events = self._songs[song]._events
#			evjson = []
#			for i,e in enumerate(events):
#				evjson.append((i,events[i].toJSON()))
#				fo.write('\t(%d,%s),\n' % (i,repr(events[i].toJSON())))
			
#			fo.write(json.dumps(evjson,sort_keys=True,indent=2, separators=(',', ': ')))
#			fo.close()
#			print [ord(d) for d in tdata[5196:5204]]
#			return [ord(d) for d in self._songs[song]._control]
		elif obj == 0x0B:  # Drum Track Pattern Data TABLE 7
			pass
		elif obj == 0x0C:  # Drum Track Pattern Event Data TABLE 8
			pass
		elif obj == 0x0D:  # Cuelist Data TABLE 9
			cuelist = data[10]
			print len(mdata)
		elif obj == 0x0F:  # Program Bank Parameter TABLE 1
			pass
		elif obj == 0x11:  # Combination Bank Parameter TABLE 2 
			pass
		elif obj == 0x12:  # Drum Kit Bank Parameter TABLE 6
			pass
		elif obj == 0x16:  # Arpeggio Pattern Parameter TABLE 4
			pass
		elif obj == 0x17:  # Arpeggio Pattern All Parameter TABLE 4
			pass
		
#-------------------------------------------------------------------------------
	def do_current_object_dump(self,data):
		obj = data[7]
#		print '\nobject',obj
		if obj == 0:  # Program TABLE 1
			self._current_program = sysex.convert_7_bit_to_8_bit(list(data[9:-1]))
			return [ord(d) for d in self._current_program]
		elif obj == 1:  # Combination TABLE 2
			self._current_combination = sysex.convert_7_bit_to_8_bit(list(data[9:-1]))
			return [ord(d) for d in self._current_combination]
		elif obj == 2:  # Song Timbre Set TABLE 10
			self._current_timbre_set = table10.SongTimbreSet(sysex.convert_7_bit_to_8_bit(list(data[9:-1])))
#			return [ord(d) for d in self._current_timbre_set]
			print json.dumps(self._current_timbre_set.toJSON(),sort_keys=True,indent=2, separators=(',', ': '))

#-------------------------------------------------------------------------------
	def do_sysex(self,data):
#		print data
		category = data[1]
		if category != 0x42:
			if category == 0x7E:
				return self.do_universal_nonrealtime(data)
			if category == 0x7F:
				return self.do_universal_realtime(data)
			return None

		try:
			function = data[6]
		except IndexError:
			return None

#		print 'function 0x%X' % function
		if function == 0x24:
			return self.do_result_report(data)
		elif function == 0x41:
			return self.do_parameter_change(data)
		elif function == 0x4E:
			return self.do_mode_change(data)
		elif function == 0x53:
			return self.do_drum_kit_parameter_change(data)
		elif function == 0x54:
			return self.do_arpeggio_pattern_parameter_change(data)
		elif function == 0x72:
			self.do_parameter_dump_request(data)
			return ''
		elif function == 0x73:
			return self.do_parameter_dump(data)
		elif function == 0x74:
			return ''
		elif function == 0x75:
			return self.do_current_object_dump(data)

#-------------------------------------------------------------------------------
	def run(self):
		print 'Start Status Manager'
		self._running = True
		while self._running:
			try:
				data = self._response_queue.get_nowait()
				if data is None: return
				status = data[0]
				if status < 0x90:
					chnl = (status & 0x0F)+1
					data = midi.NoteOff(self._clock_count,chnl,data[1],data[2])
				elif status < 0xA0:
					chnl = (status & 0x0F)+1
					data = midi.NoteOn(self._clock_count,chnl,data[1],data[2])
				elif status < 0xB0:
					chnl = (status & 0x0F)+1
					data = midi.PolyKeyPressure(self._clock_count,chnl,data[1],data[2])
				elif status < 0xC0:
					chnl = (status & 0x0F)+1
					data = midi.ControlChange(self._clock_count,chnl,data[1],data[2])
				elif status < 0xD0:
					chnl = (status & 0x0F)+1
					data = midi.ProgramChange(self._clock_count,chnl,data[1])
				elif status < 0xE0:
					chnl = (status & 0x0F)+1
					data = midi.ChannelKeyPressure(self._clock_count,chnl,data[1])
				elif status < 0xF0:
					chnl = (status & 0x0F)+1
					data = midi.PitchBendChange(self._clock_count,chnl,data[1]|(data[2]<<7))
				elif status == 0xF2:
					data = midi.SongPositionPointer(data[1]|(data[2]<<7))
				elif status == 0xF3:
					data = midi.SongSelect(data[1])
				elif status == 0xF8:
	#				data = midi.TimingClock()
					self._clock_count += 1
					data = None
				elif status == 0xFA:
					data = midi.Start()
					self._clock_count = 0
				elif status == 0xFB:
					data = midi.Continue()
				elif status == 0xFC:
					data = midi.Stop()
				elif status == 0xFE:
					data = midi.ActiveSensing()
				elif status == 0xF0:
					data = self.do_sysex(data)
				if data is not None:
					print data
			except Empty:
				pass
			time.sleep(0.01)

#-------------------------------------------------------------------------------
	def clear_clock(self):
		self._clock_count = 0

#-------------------------------------------------------------------------------
	@property
	def song(self):
		return self._songs


