#!@PYTHON@
# -*- coding: utf-8 -*-
#
# table10.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

import struct

tempo_mode_dict = {0:'Auto', 1:'Manual'}

off_on_dict = {0:'Off',1:'On'}

#===============================================================================
# Table 9
class CueListData(object):
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __iter__(self):
		return iter(self.steps)

#-------------------------------------------------------------------------------
	def __getitem__(self,index):
		return self.steps[index]
	
#-------------------------------------------------------------------------------
	def __str__(self):
		return '%s %d 0x%x 0x%x' % (self.name,self.tempo,self.temp_mode,self._reserved0027)

#-------------------------------------------------------------------------------
	def calcsize(self):
		return 28 + 4 * len(self.steps)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		v = list(struct.unpack('>24sHBB4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s',raw))
		self.cuelist_name = v.pop(0)
		self.tempo = v.pop(0)
		self.tempo_mode = v.pop(0)
		self._reserved0027 = v.pop(0)
		self.steps = [CueListStep(s) for s in v]

#-------------------------------------------------------------------------------
	def pack(self):
		v = []
		v.append(self.cuelist_name)
		v.append(self.tempo)
		v.append(self.temp_mode)
		v.append(self._reserved0027)
		for step in self.steps:
			v.append(step.pack())
		return struct.pack('>24sHBB4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s',*v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a CueListData Cue List Data object'''
		json = []
		json.append(('name','CueListData'))
		json.append(('description','Cue List Data'))
		json.append(['tempo',self.tempo])
		json.append(['tempo_mode',self.tempo_mode])
		json.append(['tempo_mode_display',self.tempo_mode_display])
		json.append(['steps',[step.toJSON() for step in self.steps]])
		return json

#-------------------------------------------------------------------------------
	@property
	def cuelist_name(self):
		'''Cuelist Name'''
		return self._cuelist_name

	@cuelist_name.setter
	def cuelist_name(self,value):
		self._cuelist_name = value

#-------------------------------------------------------------------------------
	@property
	def tempo(self):
		'''Tempo'''
		return self._tempo

	@tempo.setter
	def tempo(self,value):
		if 4000 <= value <= 30000 or True: #FIXME
			self._tempo = value
		else:
			raise ValueError('%s not 4000 <= value <= 30000' % value)

#-------------------------------------------------------------------------------
	@property
	def tempo_mode(self):
		'''Tempo Mode'''
		return self._tempo_mode

	@tempo_mode.setter
	def tempo_mode(self,value):
		self._tempo_mode = value

	@property
	def tempo_mode_display(self):
		'''Tempo Mode'''
		return tempo_mode_dict[self._tempo_mode]

#===============================================================================
# Table 9
class CueListStep(object):
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return '0x%x 0x%x %d 0x%x 0x%x' % (self.song_no,self.repeat,self.load_fx,self.u1,self.u2)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		v = list(struct.unpack('>BBBB',raw))
		self.song_no = v.pop(0)
		byte29 = v.pop(0)
		self.repeat = byte29 & 0x7F
		self.load_fx = byte29 >> 7
		self._reserved30 = v.pop(0)
		self._reserved31 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		v = []
		v.append(self.song_no)
		byte29 = self.repeat & 0x7F
		byte29 |= (self.load_fx & 1) << 7
		v.append(byte29)
		v.append(self._reserved30)
		v.append(self._reserved31)
		return struct.pack('>BBBB',*v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a CueListStep Cue List Step object'''
		json = []
		json.append(('name','CueListStep'))
		json.append(('description','Cue List Step'))
		json.append(['song_no',self.song_no])
		json.append(['repeat',self.repeat])
		json.append(['load_fx',self.load_fx])
		json.append(['load_fx_display',self.load_fx_display])
		if self._reserved30:
			json.append(['_reserved30',self._reserved30])
		if self._reserved31:
			json.append(['_reserved31',self._reserved31])
		return json

#-------------------------------------------------------------------------------
	@property
	def song_no(self):
		'''Song No'''
		return self._song_no

	@song_no.setter
	def song_no(self,value):
		self._song_no = value

#-------------------------------------------------------------------------------
	@property
	def repeat(self):
		'''Repeat'''
		return self._repeat

	@repeat.setter
	def repeat(self,value):
		self._repeat = value

#-------------------------------------------------------------------------------
	@property
	def load_fx(self):
		'''Load FX'''
		return self._load_fx

	@load_fx.setter
	def load_fx(self,value):
		self._load_fx = value

	@property
	def load_fx_display(self):
		'''Load FX'''
		return off_on_dict[self._load_fx]


