#!@PYTHON@
# -*- coding: utf-8 -*-
#
# table1.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

import struct
from fx import *

__all__ = ['ProgramParameters']

#===============================================================================
# Table 1
class ProgramParameters(object):
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		pass
		
#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a ProgramParameters object'''
		v = list(struct.unpack('<24s76s76s76s76s76s68s68s4B72s8s8s2BH156s20s24s100s68s440s440s',raw)) # 1884 
#		v = list(struct.unpack('<24s76s76s76s76s76s68s68s4B68s2BH8s8s2BH156s20s24s100s68s440s440s',raw)) # 1884 
		self.program_name = v.pop(0)
#		print self.program_name
		self.ifx1 = v.pop(0)
		self.ifx2 = v.pop(0)
		self.ifx3 = v.pop(0)
		self.ifx4 = v.pop(0)
		self.ifx5 = v.pop(0)
		self.mfx1 = v.pop(0)
		self.mfx2 = v.pop(0)
		self.mfx1_return_level = v.pop(0)
		self.mfx2_return_level = v.pop(0)
		byte542 = v.pop(0)
		self.mfx_chain_direction = byte542 & 0x01
		self.mfx_chain_on_off = (byte542 >> 1) & 0x01
		self.mfx_chain_level = v.pop(0)
		self.tfx = v.pop(0)
#		self.master_volume = v.pop(0)
#		self._reserved613 = v.pop(0)
#		self._reserved614 = v.pop(0)
		self.lfo1 = v.pop(0)
		self.lfo2 = v.pop(0)
		self.control_assign = v.pop(0)
		self._reserved633 = v.pop(0)
		self._reserved634 = v.pop(0)
		self.drum = v.pop(0)
		self.arpeggiator = v.pop(0)
		self.drum_timbre = v.pop(0)
		self.common = v.pop(0)
		self.pcm_osc = v.pop(0)
		self.pcm_osc1 = v.pop(0)
		self.pcm_osc2 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a ProgramParameters object into raw data'''
		v = list()
		v.append(self.program_name)
		v.append(self.ifx1.pack())
		v.append(self.ifx2.pack())
		v.append(self.ifx3.pack())
		v.append(self.ifx4.pack())
		v.append(self.ifx5.pack())
		v.append(self.mfx1.pack())
		v.append(self.mfx2.pack())
		v.append(self.mfx1_return_level)
		v.append(self.mfx2_return_level)
		byte542 = 0
		byte542 |= self.mfx_chain_direction & 0x01
		byte542 |= (self.mfx_chain_on_off & 0x01) << 1
		v.append(byte542)
		v.append(self.mfx_chain_level)
		v.append(self.tfx.pack())
		v.append(self.master_volume)
		v.append(self._reserved613)
		v.append(self._reserved614)
		v.append(self.lfo1.pack())
		v.append(self.lfo2.pack())
		v.append(self.control_assign)
		v.append(self._reserved633)
		v.append(self._reserved634)
		v.append(self.drum.pack())
		v.append(self.arpeggiator.pack())
		v.append(self.drum_timbre.pack())
		v.append(self.common.pack())
		v.append(self.pcm_osc.pack())
		v.append(self.pcm_osc1.pack())
		v.append(self.pcm_osc2.pack())
		return struct.pack('>24s76s76s76s76s76s68s68s4B68s2BH8s8s2BH156s20s24s100s68s440s440s',*v) # 1884 

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ProgramParameters object'''
		json = []
		json.append(['program_name',self.program_name])
#		print self.program_name
		json.append(['ifx1',self.ifx1.toJSON()])
		json.append(['ifx2',self.ifx2.toJSON()])
		json.append(['ifx3',self.ifx3.toJSON()])
		json.append(['ifx4',self.ifx4.toJSON()])
		json.append(['ifx5',self.ifx5.toJSON()])
		json.append(['mfx1',self.mfx1.toJSON()])
		json.append(['mfx2',self.mfx2.toJSON()])
		json.append(['mfx1_return_level',self.mfx1_return_level])
		json.append(['mfx2_return_level',self.mfx2_return_level])
		json.append(['mfx_chain_direction]',self.mfx_chain_direction])
		json.append(['mfx_chain_on_off',self.mfx_chain_on_off])
		json.append(['mfx_chain_on_off',self.mfx_chain_level])
		json.append(['tfx',self.tfx.toJSON()])
#		self.master_volume = v.pop(0)
#		self._reserved613 = v.pop(0)
#		self._reserved614 = v.pop(0)
		json.append(['lfo1',self.lfo1.toJSON()])
		json.append(['lfo2',self.lfo2.toJSON()])
		json.append(['control_assign',self.control_assign])
		json.append(['_reserved633',self._reserved633])
		json.append(['_reserved634',self._reserved634])
		json.append(['drum',self.drum.toJSON()])
		json.append(['arpeggiator',self.arpeggiator.toJSON()])
		json.append(['drum_timbre',self.drum_timbre.toJSON()])
		json.append(['common',self.common.toJSON()])
		json.append(['pcm_osc',self.pcm_osc.toJSON()])
		json.append(['pcm_osc1',self.pcm_osc1.toJSON()])
		json.append(['pcm_osc2',self.pcm_osc2.toJSON()])
		return json

#-------------------------------------------------------------------------------
	@property
	def program_name(self):
		'''Progam Name'''
		return self._program_name

	@program_name.setter
	def program_name(self,value):
		self._program_name = value

#-------------------------------------------------------------------------------
	@property
	def ifx1(self):
		'''Insert Effect 1 Parameters'''
		return self._ifx1

	@ifx1.setter
	def ifx1(self,value):
		if isinstance(value,InsertEffectParameters):
			self._ifx1 = value
		elif isinstance(value,basestring):
			self._ifx1 = InsertEffectParameters(value)
		else:
			raise TypeError('value must of type InsertEffectParameters')
		self._ifx1.context = 'ifx1'

#-------------------------------------------------------------------------------
	@property
	def ifx2(self):
		'''Insert Effect 2 Parameters'''
		return self._ifx2

	@ifx2.setter
	def ifx2(self,value):
		if isinstance(value,InsertEffectParameters):
			self._ifx2 = value
		elif isinstance(value,basestring):
			self._ifx2 = InsertEffectParameters(value)
		else:
			raise TypeError('value must of type InsertEffectParameters')
		self._ifx2.context = 'ifx2'

#-------------------------------------------------------------------------------
	@property
	def ifx3(self):
		'''Insert Effect 3 Parameters'''
		return self._ifx3

	@ifx3.setter
	def ifx3(self,value):
		if isinstance(value,InsertEffectParameters):
			self._ifx3 = value
		elif isinstance(value,basestring):
			self._ifx3 = InsertEffectParameters(value)
		else:
			raise TypeError('value must of type InsertEffectParameters')
		self._ifx3.context = 'ifx3'

#-------------------------------------------------------------------------------
	@property
	def ifx4(self):
		'''Insert Effect 4 Parameters'''
		return self._ifx4

	@ifx4.setter
	def ifx4(self,value):
		if isinstance(value,InsertEffectParameters):
			self._ifx4 = value
		elif isinstance(value,basestring):
			self._ifx4 = InsertEffectParameters(value)
		else:
			raise TypeError('value must of type InsertEffectParameters')
		self._ifx4.context = 'ifx4'

#-------------------------------------------------------------------------------
	@property
	def ifx5(self):
		'''Insert Effect 5 Parameters'''
		return self._ifx5

	@ifx5.setter
	def ifx5(self,value):
		if isinstance(value,InsertEffectParameters):
			self._ifx5 = value
		elif isinstance(value,basestring):
			self._ifx5 = InsertEffectParameters(value)
		else:
			raise TypeError('value must of type InsertEffectParameters')
		self._ifx5.context = 'ifx5'

#-------------------------------------------------------------------------------
	@property
	def mfx1(self):
		'''Master Effect 1 Parameters'''
		return self._mfx1

	@mfx1.setter
	def mfx1(self,value):
		if isinstance(value,MasterEffectParameters):
			self._mfx1 = value
		elif isinstance(value,basestring):
			self._mfx1 = MasterEffectParameters(value)
		else:
			raise TypeError('value must of type MasterEffectParameters')
		self._mfx1.context = 'mfx1'

#-------------------------------------------------------------------------------
	@property
	def mfx2(self):
		'''Master Effect 2 Parameters'''
		return self._mfx2

	@mfx2.setter
	def mfx2(self,value):
		if isinstance(value,MasterEffectParameters):
			self._mfx2 = value
		elif isinstance(value,basestring):
			self._mfx2 = MasterEffectParameters(value)
		else:
			raise TypeError('value must of type MasterEffectParameters')
		self._mfx2.context = 'mfx2'

#-------------------------------------------------------------------------------
	@property
	def mfx1_return_level(self):
		'''MFX1 Return Level'''
		return self._mfx1_return_level

	@mfx1_return_level.setter
	def mfx1_return_level(self,value):
		if 0 <= value <= 127:
			self._mfx1_return_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def mfx2_return_level(self):
		'''MFX2 Return Level'''
		return self._mfx2_return_level

	@mfx2_return_level.setter
	def mfx2_return_level(self,value):
		if 0 <= value <= 127:
			self._mfx2_return_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_direction(self):
		'''MFX Chain Direction'''
		return self._mfx_chain_direction

	@mfx_chain_direction.setter
	def mfx_chain_direction(self,value):
		if 0 <= value <= 1:
			self._mfx_chain_direction = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_on_off(self):
		'''MFX Chain On/Off'''
		return self._mfx_chain_on_off

	@mfx_chain_on_off.setter
	def mfx_chain_on_off(self,value):
		if 0 <= value <= 1:
			self._mfx_chain_on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_level(self):
		'''MFX Chain Level'''
		return self._mfx_chain_level

	@mfx_chain_level.setter
	def mfx_chain_level(self,value):
		if 0 <= value <= 127:
			self._mfx_chain_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def tfx(self):
		'''Total Effect Parameters'''
		return self._tfx

	@tfx.setter
	def tfx(self,value):
		if isinstance(value,TotalEffectParameters):
			self._tfx = value
		elif isinstance(value,basestring):
			self._tfx = TotalEffectParameters(value)
		else:
			raise TypeError('value must of type TotalEffectParameters')
		self._tfx.context = 'tfx'

#-------------------------------------------------------------------------------
	@property
	def master_volume(self):
		'''Master Volume'''
		return self._master_volume

	@master_volume.setter
	def master_volume(self,value):
		if 0 <= value <= 127:
			self._master_volume = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def lfo1(self):
		'''Effect LFO1 Parameters'''
		return self._lfo1

	@lfo1.setter
	def lfo1(self,value):
		if isinstance(value,EffectLFOParameters):
			self._lfo1 = value
		elif isinstance(value,basestring):
			self._lfo1 = EffectLFOParameters(value)
		else:
			raise TypeError('value must of type EffectLFOParameters')

#-------------------------------------------------------------------------------
	@property
	def lfo2(self):
		'''Effect LFO2 Parameters'''
		return self._lfo2

	@lfo2.setter
	def lfo2(self,value):
		if isinstance(value,EffectLFOParameters):
			self._lfo2 = value
		elif isinstance(value,basestring):
			self._lfo2 = EffectLFOParameters(value)
		else:
			raise TypeError('value must of type EffectLFOParameters')

#-------------------------------------------------------------------------------
	@property
	def control_assign(self):
		''''''
		return self._control_assign

	@control_assign.setter
	def control_assign(self,value):
#FIXME		if 0 <= value <= 2:
		self._control_assign = value
#		else:
#			raise ValueError('%s not 0 <= value <= 2' % value)
#			 print 'control_assign not 0 <= value[%d] <= 2' % value

#-------------------------------------------------------------------------------
	@property
	def drum(self):
		'''Drum Track Parameters'''
		return self._drum

	@drum.setter
	def drum(self,value):
		if isinstance(value,DrumTrackParameters):
			self._drum = value
		elif isinstance(value,basestring):
			self._drum = DrumTrackParameters(value)
		else:
			raise TypeError('value must of type DrumTrackParameters')

#-------------------------------------------------------------------------------
	@property
	def arpeggiator(self):
		'''Arpeggiator Parameters'''
		return self._arpeggiator

	@arpeggiator.setter
	def arpeggiator(self,value):
		if isinstance(value,ArpeggiatorParameters):
			self._arpeggiator = value
		elif isinstance(value,basestring):
			self._arpeggiator = ArpeggiatorParameters(value)
		else:
			raise TypeError('value must of type ArpeggiatorParameters')

#-------------------------------------------------------------------------------
	@property
	def drum_timbre(self):
		'''Drum Track Timbre Parameters'''
		return self._drum_timbre

	@drum_timbre.setter
	def drum_timbre(self,value):
		if isinstance(value,DrumTrackTimbreParameters):
			self._drum_timbre = value
		elif isinstance(value,basestring):
			self._drum_timbre = DrumTrackTimbreParameters(value)
		else:
			raise TypeError('value must of type DrumTrackTimbreParameters')

#-------------------------------------------------------------------------------
	@property
	def common(self):
		'''Program Common Parameters'''
		return self._common

	@common.setter
	def common(self,value):
		if isinstance(value,ProgramCommonParameters):
			self._common = value
		elif isinstance(value,basestring):
			self._common = ProgramCommonParameters(value)
		else:
			raise TypeError('value must of type ProgramCommonParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc(self):
		'''PCM OSC Common Parameters'''
		return self._pcm_osc

	@pcm_osc.setter
	def pcm_osc(self,value):
		if isinstance(value,PCMOSCCommonParameters):
			self._pcm_osc = value
		elif isinstance(value,basestring):
			self._pcm_osc = PCMOSCCommonParameters(value)
		else:
			raise TypeError('value must of type PCMOSCCommonParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc1(self):
		'''PCM OSC1 Storage Parameters'''
		return self._pcm_osc1

	@pcm_osc1.setter
	def pcm_osc1(self,value):
		if isinstance(value,PCMOSCStorageParameters):
			self._pcm_osc1 = value
		elif isinstance(value,basestring):
			self._pcm_osc1 = PCMOSCStorageParameters(value)
		else:
			raise TypeError('value must of type PCMOSCStorageParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc2(self):
		'''PCM OSC2 Storage Parameters'''
		return self._pcm_osc2

	@pcm_osc2.setter
	def pcm_osc2(self,value):
		if isinstance(value,PCMOSCStorageParameters):
			self._pcm_osc2 = value
		elif isinstance(value,basestring):
			self._pcm_osc2 = PCMOSCStorageParameters(value)
		else:
			raise TypeError('value must of type PCMOSCStorageParameters')

#===============================================================================
# Table 1 24-99 x 5
class InsertEffectParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<64s8BI',raw)) # 76 
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte65 = v.pop(0)
		self.channel = byte65 & 0x3F
		self.on_off = (byte65 >> 6) & 0x01
		self.chain = (byte65 >> 7) & 0x01
		self.chain_to = v.pop(0)
		self._reserved67 = v.pop(0)
		self.pan = v.pop(0)
		byte69 = v.pop(0)
		self.bus_select_1 = byte69 & 0x0F
		self.bus_select_2 = (byte69 >> 4) & 0x03
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved72 = v.pop(0)
		if self.effect_type > 0:
			fxcls = globals()['FX%03d' % self.effect_type]
			self.param_struct = fxcls(self.param_struct)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffectParameters object into raw data'''
		v = list()
		v.append(self.param_struct)
		v.append(self.effect_type)
		byte65 = 0
		byte65 |= self.channel & 0x3F
		byte65 |= (self.on_off & 0x01) << 6
		byte65 |= (self.chain & 0x01) << 7
		v.append(byte65)
		v.append(self.chain_to)
		v.append(self._reserved67)
		v.append(self.pan)
		byte69 = 0
		byte69 |= self.bus_select_1 & 0x0F
		byte69 |= (self.bus_select_2 & 0x03) << 4
		v.append(byte69)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved72)
		return struct.pack('>64s8BI',*v) # 76

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffectParameters object'''
		json = []
		json.append(['effect_type',self.effect_type])
		if self.effect_type == 0:
			json.append(['effect_type_display',None])
		else:
			json.append(['effect_type_display',self.param_struct.description])
		json.append(['channel',self.channel])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display','On' if self.on_off else 'Off'])
		json.append(['chain',self.chain])
		json.append(['chain_display','On' if self.chain else 'Off'])
		json.append(['chain_to',self.chain_to])
		if self.context in ('ifx1','ifx2','ifx3'):
			json.append(['chain_to_display','IFX%d' % self.chain_to])
			
		json.append(['_reserved67',self._reserved67])
		json.append(['pan',self.pan])
		json.append(['bus_select_1',self.bus_select_1])
		json.append(['bus_select_1_display','L/R' if self.bus_select_1==0 else 'Off'])
		json.append(['bus_select_2',self.bus_select_2])
		json.append(['bus_select_2_display','Off' if self.bus_select_2==0 else 'FX Ctrl1' if self.bus_select_2==1 else 'FX Ctrl2'])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		json.append(['_reserved72',self._reserved72])
		if self.effect_type != 0:
			json.append(['param',self.param_struct.toJSON()])
		return json

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''Insert Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
		else:
			raise ValueError('not 0 <= value <= 193')

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		self._channel = value

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def chain(self):
		'''Chain'''
		return self._chain

	@chain.setter
	def chain(self,value):
		if 0 <= value <= 1:
			self._chain = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def chain_to(self):
		'''Chain To'''
		return self._chain_to

	@chain_to.setter
	def chain_to(self,value):
		if 0 <= value <= 6:
			self._chain_to = value
		else:
			raise ValueError('0x%X not 0 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def bus_select_1(self):
		'''Bus Select'''
		return self._bus_select_1

	@bus_select_1.setter
	def bus_select_1(self,value):
		if 0 <= value <= 7:
			self._bus_select_1 = value
		else:
			raise ValueError('not 0 <= value <= 7')

#-------------------------------------------------------------------------------
	@property
	def bus_select_2(self):
		'''Bus Select'''
		return self._bus_select_2

	@bus_select_2.setter
	def bus_select_2(self,value):
		if 0 <= value <= 2:
			self._bus_select_2 = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#===============================================================================
# Table 1 404-471 x 2
class MasterEffectParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<64s2BH',raw)) # 68 
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte65 = v.pop(0)
		self.on_off = (byte65 >> 6) & 0x01
		self._reserved66 = v.pop(0)
		if self.effect_type > 0:
			fxcls = globals()['FX%03d' % self.effect_type]
			self.param_struct = fxcls(self.param_struct)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.param_struct)
		v.append(self.effect_type)
		byte65 = 0
		byte65 |= (self.on_off & 0x01) << 6
		v.append(byte65)
		v.append(self._reserved66)
		return struct.pack('>64s2BH',*v) # 68

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''Master Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
		else:
			raise ValueError('not 0 <= value <= 193')

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#===============================================================================
# Table 1 544-615
class TotalEffectParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
#		print len(raw)
		v = list(struct.unpack('<64s2BH2BH',raw)) # 72 
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte65 = v.pop(0)
		self.on_off = (byte65 >> 6) & 0x01
		self._reserved610 = v.pop(0)
		self.master_volume = v.pop(0)
		self._reserved613 = v.pop(0)
		self._reserved614 = v.pop(0)
		if self.effect_type > 0:
			fxcls = globals()['FX%03d' % self.effect_type]
			self.param_struct = fxcls(self.param_struct)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.param_struct)
		v.append(self.effect_type)
		byte65 = 0
		byte65 |= (self.on_off & 0x01) << 6
		v.append(byte65)
		v.append(self._reserved610)
		v.append(self.master_volume)
		v.append(self._reserved613)
		v.append(self._reserved614)
		return struct.pack('>64s2BH2BH',*v) # 72

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''Total Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
		else:
			raise ValueError('not 0 <= value <= 193')

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def master_volume(self):
		'''Master Volume'''
		return self._master_volume

	@master_volume.setter
	def master_volume(self,value):
		if 0 <= value <= 127:
			self._master_volume = value
		else:
			raise ValueError('not 0 <= value <= 127')

#===============================================================================
# Table 1 616-623 x 2
class EffectLFOParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<4BH2B',raw)) # 8 
		self.frequency = v.pop(0)
		self.flag = v.pop(0)
		self.dmod_src = v.pop(0)
		self.midi_channel = v.pop(0)
		self.bpm = v.pop(0)
		byte6 = v.pop(0)
		self.midi_sync_base_note = byte6 & 0x0F
		self.midi_sync_on_off = (byte6 >> 7) & 0x01
		self.midi_sync_time = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.frequency)
		v.append(self.flag)
		v.append(self.dmod_src)
		v.append(self.midi_channel)
		v.append(self.bpm)
		byte6 = 0
		byte6 |= self.midi_sync_base_note & 0x0F
		byte6 |= (self.midi_sync_on_off & 0x01) << 7
		v.append(byte6)
		v.append(self.midi_sync_time)
		return struct.pack('>4BH2B',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def frequency(self):
		'''Frequency'''
		return self._frequency

	@frequency.setter
	def frequency(self,value):
		if 1 <= value <= 230:
			self._frequency = value
		else:
			raise ValueError('not 1 <= value <= 230')

#-------------------------------------------------------------------------------
	@property
	def flag(self):
		'''Sync Flag'''
		return self._flag

	@flag.setter
	def flag(self,value):
		if 0 <= value <= 1:
			self._flag = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def dmod_src(self):
		'''DMod Src'''
		return self._dmod_src

	@dmod_src.setter
	def dmod_src(self,value):
		if 0 <= value <= 37:
			self._dmod_src = value
		else:
			raise ValueError('not 0 <= value <= 37')

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
		else:
			raise ValueError('not 0 <= value <= 16')

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 7530:
			self._bpm = value
		else:
			raise ValueError('not 3999 <= value[%d] <= 7530' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_sync_base_note(self):
		'''MIDI Sync Base Note'''
		return self._midi_sync_base_note

	@midi_sync_base_note.setter
	def midi_sync_base_note(self,value):
		if 0 <= value <= 9:
			self._midi_sync_base_note = value
		else:
			raise ValueError('not 0 <= value <= 9')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_on_off(self):
		'''MIDI Sync On/Off'''
		return self._midi_sync_on_off

	@midi_sync_on_off.setter
	def midi_sync_on_off(self,value):
		if 0 <= value <= 1:
			self._midi_sync_on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_time(self):
		'''MIDI Sync Time'''
		return self._midi_sync_time

	@midi_sync_time.setter
	def midi_sync_time(self,value):
		if 1 <= value <= 32:
			self._midi_sync_time = value
		else:
			raise ValueError('not 1 <= value <= 32')

#===============================================================================
# Table 1 636-791
class DrumTrackParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<H6Bb2B145s',raw)) # 156 
		self.pattern_number = v.pop(0)
		self.pattern_bank = v.pop(0)
		byte3 = v.pop(0)
		self.button_mode = byte3 & 0x01
		self.latch_mode = (byte3 >> 1) & 0x03
		self.sync_parameter = (byte3 >> 3) & 0x01
		self.drum_track_on = (byte3 >> 4) & 0x01
		self.trigger_bottom_key = v.pop(0)
		self.trigger_top_key = v.pop(0)
		self.velocity_bottom = v.pop(0)
		self.velocity_top = v.pop(0)
		self.shift = v.pop(0)
		self.output_channel = v.pop(0)
		self.input_channel = v.pop(0)
		self._reserved11 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.pattern_number)
		v.append(self.pattern_bank)
		byte3 = 0
		byte3 |= self.button_mode & 0x01
		byte3 |= (self.latch_mode & 0x03) << 1
		byte3 |= (self.sync_parameter & 0x01) << 3
		byte3 |= (self.drum_track_on & 0x01) << 4
		v.append(byte3)
		v.append(self.trigger_bottom_key)
		v.append(self.trigger_top_key)
		v.append(self.velocity_bottom)
		v.append(self.velocity_top)
		v.append(self.shift)
		v.append(self.output_channel)
		v.append(self.input_channel)
		v.append(self._reserved11)
		return struct.pack('>H6Bb2B145s',*v) # 156

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def pattern_number(self):
		'''Pattern Num'''
		return self._pattern_number

	@pattern_number.setter
	def pattern_number(self,value):
		if 0 <= value <= 999:
			self._pattern_number = value
		else:
			raise ValueError('not 0 <= value <= 999')

#-------------------------------------------------------------------------------
	@property
	def pattern_bank(self):
		'''Pattern Bank'''
		return self._pattern_bank

	@pattern_bank.setter
	def pattern_bank(self,value):
		if 0 <= value <= 1:
			self._pattern_bank = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def button_mode(self):
		'''Button Mode'''
		return self._button_mode

	@button_mode.setter
	def button_mode(self,value):
		if 0 <= value <= 1:
			self._button_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def latch_mode(self):
		'''Latch Mode'''
		return self._latch_mode

	@latch_mode.setter
	def latch_mode(self,value):
		if 0 <= value <= 1:
			self._latch_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def sync_parameter(self):
		'''Sync Parameter'''
		return self._sync_parameter

	@sync_parameter.setter
	def sync_parameter(self,value):
		if 0 <= value <= 1:
			self._sync_parameter = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def drum_track_on(self):
		'''Drum Track On'''
		return self._drum_track_on

	@drum_track_on.setter
	def drum_track_on(self,value):
		if 0 <= value <= 1:
			self._drum_track_on = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def trigger_bottom_key(self):
		'''Trigger Bottom Key'''
		return self._trigger_bottom_key

	@trigger_bottom_key.setter
	def trigger_bottom_key(self,value):
		if 0 <= value <= 127:
			self._trigger_bottom_key = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def trigger_top_key(self):
		'''Trigger Top Key'''
		return self._trigger_top_key

	@trigger_top_key.setter
	def trigger_top_key(self,value):
		if 0 <= value <= 127:
			self._trigger_top_key = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def velocity_bottom(self):
		'''Velocity Bottom'''
		return self._velocity_bottom

	@velocity_bottom.setter
	def velocity_bottom(self,value):
		if 0 <= value <= 127:
			self._velocity_bottom = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def velocity_top(self):
		'''Velocity Top'''
		return self._velocity_top

	@velocity_top.setter
	def velocity_top(self,value):
		if 0 <= value <= 127:
			self._velocity_top = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def shift(self):
		'''Shift'''
		return self._shift

	@shift.setter
	def shift(self,value):
		if -24 <= value <= 24:
			self._shift = value
		else:
			raise ValueError('not -24 <= value <= 24')

#-------------------------------------------------------------------------------
	@property
	def output_channel(self):
		'''Output Channel'''
		return self._output_channel

	@output_channel.setter
	def output_channel(self,value):
		if 0 <= value <= 16:
			self._output_channel = value
		else:
			raise ValueError('not 0 <= value <= 16')

#-------------------------------------------------------------------------------
	@property
	def input_channel(self):
		'''Input Channel'''
		return self._input_channel

	@input_channel.setter
	def input_channel(self,value):
		if 0 <= value <= 16:
			self._input_channel = value
		else:
			raise ValueError('not 0 <= value <= 16')

#===============================================================================
# Table 1 792-811
class ArpeggiatorParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
#		print len(raw)
		v = list(struct.unpack('<H2B4bH5Bb4B',raw))
		self.tempo = v.pop(0)
		self.switch = v.pop(0)
		self._reserved795 = v.pop(0)
		self.gate_control = v.pop(0)
		self.velocity_control = v.pop(0)
		self.swing_control = v.pop(0)
		self.step_control = v.pop(0)
		self.pattern_number = v.pop(0)
		byte802 = v.pop(0)
		self.octave = byte802 & 3
		self.resolution = (byte802 >> 2) & 7
		byte803 = v.pop(0)
		self.sort = byte803 & 1
		self.latch = (byte803 >> 1) & 1
		self.key_sync = (byte803 >> 2) & 1
		self.keyboard = (byte803 >> 3) & 1
		self.gate = v.pop(0)
		self.velocity = v.pop(0)
		self.swing = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		self._reserved811 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		v = list()
		v.append(self.tempo)
		v.append(self.switch)
		v.append(self._reserved795)
		v.append(self.gate_control)
		v.append(self.velocity_control)
		v.append(self.swing_control)
		v.append(self.step_control)
		v.append(self.pattern_number)
		byte802 = self.octave & 3
		byte802 |= (self.resolution & 7) << 2
		v.append(byte802)
		byte803 = self.sort & 1
		byte803 |= (self.latch & 1) << 1
		byte803 |= (self.key_sync & 1) << 2
		byte803 |= (self.keyboard & 1) << 3
		v.append(byte803)
		v.append(self.gate)
		v.append(self.velocity)
		v.append(self.swing)
		v.append(self.top_key)
		v.append(self.bottom_key)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		v.append(self._reserved811)
		return struct.pack('>H2B4bH5Bb4B',v)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def tempo(self):
		'''Tempo'''
		return self._tempo

	@tempo.setter
	def tempo(self,value):
		if 4000 <= value <= 30000:
			self._tempo = value
		else:
			raise ValueError('not 4000 <= value <= 30000')

#-------------------------------------------------------------------------------
	@property
	def switch(self):
		'''Switch'''
		return self._switch

	@switch.setter
	def switch(self,value):
		if 0 <= value <= 1:
			self._switch = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def gate_control(self):
		'''Gate Control'''
		return self._gate_control

	@gate_control.setter
	def gate_control(self,value):
		if -64 <= value <= 64:
			self._gate_control = value
		else:
			raise ValueError('not -64 <= value <= 64')

#-------------------------------------------------------------------------------
	@property
	def velocity_control(self):
		'''Velocity Control'''
		return self._velocity_control

	@velocity_control.setter
	def velocity_control(self,value):
		if -64 <= value <= 64:
			self._velocity_control = value
		else:
			raise ValueError('not -64 <= value <= 64')

#-------------------------------------------------------------------------------
	@property
	def swing_control(self):
		'''Swing Control'''
		return self._swing_control

	@swing_control.setter
	def swing_control(self,value):
		if -64 <= value <= 64:
			self._swing_control = value
		else:
			raise ValueError('not -64 <= value <= 64')

#-------------------------------------------------------------------------------
	@property
	def step_control(self):
		'''Step Control'''
		return self._step_control

	@step_control.setter
	def step_control(self,value):
		if -64 <= value <= 64:
			self._step_control = value
		else:
			raise ValueError('not -64 <= value <= 64')

#-------------------------------------------------------------------------------
	@property
	def pattern_number(self):
		'''Pattern Number'''
		return self._pattern_number

	@pattern_number.setter
	def pattern_number(self,value):
		if 0 <= value <= 1032:
			self._pattern_number = value
		else:
			raise ValueError('not 0 <= value <= 1032')

#-------------------------------------------------------------------------------
	@property
	def octave(self):
		'''Octave'''
		return self._octave

	@octave.setter
	def octave(self,value):
		if 0 <= value <= 3:
			self._octave = value
		else:
			raise ValueError('not 0 <= value <= 3')

#-------------------------------------------------------------------------------
	@property
	def resolution(self):
		'''Resolution'''
		return self._resolution

	@resolution.setter
	def resolution(self,value):
		if 0 <= value <= 7:
			self._resolution = value
		else:
			raise ValueError('not 0 <= value <= 7')

#-------------------------------------------------------------------------------
	@property
	def sort(self):
		'''Sort'''
		return self._sort

	@sort.setter
	def sort(self,value):
		if 0 <= value <= 1:
			self._sort = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def latch(self):
		'''Latch'''
		return self._latch

	@latch.setter
	def latch(self,value):
		if 0 <= value <= 1:
			self._latch = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def key_sync(self):
		'''Key Sync'''
		return self._key_sync

	@key_sync.setter
	def key_sync(self,value):
		if 0 <= value <= 1:
			self._key_sync = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def keyboard(self):
		'''Keyboard'''
		return self._keyboard

	@keyboard.setter
	def keyboard(self,value):
		if 0 <= value <= 1:
			self._keyboard = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def gate(self):
		'''Gate'''
		return self._gate

	@gate.setter
	def gate(self,value):
		if 0 <= value <= 101:
			self._gate = value
		else:
			raise ValueError('not 0 <= value <= 101')

#-------------------------------------------------------------------------------
	@property
	def velocity(self):
		'''Velocity'''
		return self._velocity

	@velocity.setter
	def velocity(self,value):
		if 1 <= value <= 129:
			self._velocity = value
		else:
			raise ValueError('not 1 <= value <= 129')

#-------------------------------------------------------------------------------
	@property
	def swing(self):
		'''Swing'''
		return self._swing

	@swing.setter
	def swing(self,value):
#FIXME		if -100 <= value <= 100:
		self._swing = value
#		else:
#			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 1 <= value <= 127:
			self._top_velocity = value
		else:
			raise ValueError('not 1 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 1 <= value <= 127:
			self._bottom_velocity = value
		else:
			raise ValueError('not 1 <= value <= 127')

#===============================================================================
# Table 1 812-835
class DrumTrackTimbreParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<3B3bHh8BH4B',raw)) # 24 
		byte0 = v.pop(0)
		self.trim = byte0 & 0x7F
		self.trim_status = (byte0 >> 7) & 0x01
		self._reserved813 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self.reserved818 = v.pop(0)
		self.detune = v.pop(0)
		self.dkit_patch_ifx1 = v.pop(0)
		self.dkit_patch_ifx2 = v.pop(0)
		self.dkit_patch_ifx3 = v.pop(0)
		self.dkit_patch_ifx4 = v.pop(0)
		self.dkit_patch_ifx5 = v.pop(0)
		byte15 = v.pop(0)
		self.bus_select = byte15 & 0x3F
		self.fx_control_bus = (byte15 >> 6) & 0x03
		self._reserved828 = v.pop(0)
		byte17 = v.pop(0)
		self.auto_load_progs_eq = byte17 & 0x01
		self.timbre_mute = (byte17 >> 1) & 0x01
		self.bank_prog_number = v.pop(0)
		self.volume = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self._reserved835 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		byte0 = 0
		byte0 |= self.trim & 0x7F
		byte0 = (self.trim_status & 0x01) << 7
		v.append(byte0)
		v.append(self._reserved813)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self.reserved818)
		v.append(self.detune)
		v.append(self.dkit_patch_ifx1)
		v.append(self.dkit_patch_ifx2)
		v.append(self.dkit_patch_ifx3)
		v.append(self.dkit_patch_ifx4)
		v.append(self.dkit_patch_ifx5)
		byte15 = 0
		byte15 |= self.bus_select & 0x3F
		byte15 = (self.fx_control_bus & 0x03) << 6
		v.append(byte15)
		v.append(self._reserved828)
		byte17 = 0
		byte17 |= self.auto_load_progs_eq & 0x01
		byte17 = (self.timbre_mute & 0x01) << 1
		v.append(byte17)
		v.append(self.bank_prog_number)
		v.append(self.volume)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self._reserved835)
		return struct.pack('>3B3bHh8BH4B',*v) # 24

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def trim(self):
		'''Trim'''
		return self._trim

	@trim.setter
	def trim(self,value):
		if 0 <= value <= 153:
			self._trim = value
		else:
			raise ValueError('not 0 <= value <= 153')

#-------------------------------------------------------------------------------
	@property
	def trim_status(self):
		'''Trim Normal/Bypass'''
		return self._trim_status

	@trim_status.setter
	def trim_status(self,value):
		if 0 <= value <= 1:
			self._trim_status = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''MidFreq'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
		else:
			raise ValueError('not 0 <= value <= 165')

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		''''''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		''''''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		''''''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def reserved818(self):
		'''None'''
		return self._reserved818

	@reserved818.setter
	def reserved818(self,value):
		self._reserved818 = value

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		''''''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
		else:
			raise ValueError('not -1200 <= value <= 1200')

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx1(self):
		'''Dkit Patch[0]'''
		return self._dkit_patch_ifx1

	@dkit_patch_ifx1.setter
	def dkit_patch_ifx1(self,value):
		if 0 <= value <= 12:
			self._dkit_patch_ifx1 = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx2(self):
		'''Dkit Patch[1]'''
		return self._dkit_patch_ifx2

	@dkit_patch_ifx2.setter
	def dkit_patch_ifx2(self,value):
		if 0 <= value <= 12:
			self._dkit_patch_ifx2 = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx3(self):
		'''Dkit Patch[2]'''
		return self._dkit_patch_ifx3

	@dkit_patch_ifx3.setter
	def dkit_patch_ifx3(self,value):
		if 0 <= value <= 12:
			self._dkit_patch_ifx3 = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx4(self):
		'''Dkit Patch[3]'''
		return self._dkit_patch_ifx4

	@dkit_patch_ifx4.setter
	def dkit_patch_ifx4(self,value):
		if 0 <= value <= 12:
			self._dkit_patch_ifx4 = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx5(self):
		'''Dkit Patch[4]'''
		return self._dkit_patch_ifx5

	@dkit_patch_ifx5.setter
	def dkit_patch_ifx5(self,value):
		if 0 <= value <= 12:
			self._dkit_patch_ifx5 = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if 0 <= value <= 13:
			self._bus_select = value
		else:
			raise ValueError('not 0 <= value <= 13')

#-------------------------------------------------------------------------------
	@property
	def fx_control_bus(self):
		'''FX Control Bus'''
		return self._fx_control_bus

	@fx_control_bus.setter
	def fx_control_bus(self,value):
		if 0 <= value <= 2:
			self._fx_control_bus = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def auto_load_progs_eq(self):
		'''Auto Load Prog's EQ'''
		return self._auto_load_progs_eq

	@auto_load_progs_eq.setter
	def auto_load_progs_eq(self,value):
		if 0 <= value <= 1:
			self._auto_load_progs_eq = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def timbre_mute(self):
		'''Timbre Mute'''
		return self._timbre_mute

	@timbre_mute.setter
	def timbre_mute(self,value):
		if 0 <= value <= 1:
			self._timbre_mute = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def bank_prog_number(self):
		'''Bank Prog Number'''
		return self._bank_prog_number

	@bank_prog_number.setter
	def bank_prog_number(self,value):
		if 0 <= value <= 2047:
			self._bank_prog_number = value
		else:
			raise ValueError('not 0 <= value <= 2047')

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#===============================================================================
# Table 1 836-935
class ProgramCommonParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<8BH13B3bH4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s6BH',raw)) # 100 
		byte0 = v.pop(0)
		self.osc_mode = byte0 & 0x07
		self.osc_mode_favorite = (byte0 >> 5) & 0x01
		self.osc_mode_hold = (byte0 >> 6) & 0x01
		self.osc_mode_poly_mono = (byte0 >> 7) & 0x01
		byte1 = v.pop(0)
		self.poly_single_trigger = byte1 & 0x01
		self.poly_poly_legato = (byte1 >> 1) & 0x01
		self.mono_priority = (byte1 >> 2) & 0x03
		self.mono_mono_legato = (byte1 >> 4) & 0x01
		self.mono_mode = (byte1 >> 5) & 0x01
		self.enable_half_damper = (byte1 >> 6) & 0x01
		byte2 = v.pop(0)
		self.number_of_voices = byte2 & 0x07
		self.unison = (byte2 >> 7) & 0x01
		self.detune = v.pop(0)
		self.thickness = v.pop(0)
		self.hold_bottom = v.pop(0)
		self.hold_top = v.pop(0)
		byte7 = v.pop(0)
		self.bus_select = byte7 & 0x1F
		self.fx_control_bus = (byte7 >> 5) & 0x03
		self.use_dkit_setting = (byte7 >> 7) & 0x01
		self._reserved844 = v.pop(0)
		byte10 = v.pop(0)
		self.main_category = byte10 & 0x1F
		self.sub_category = (byte10 >> 5) & 0x07
		self.scale_type = v.pop(0)
		self.scale_key = v.pop(0)
		self.scale_random = v.pop(0)
		byte14 = v.pop(0)
		self.sw1_assignable_sw = byte14 & 0x3F
		self.sw1_mode = (byte14 >> 6) & 0x01
		self.sw1_on_off = (byte14 >> 7) & 0x01
		byte15 = v.pop(0)
		self.sw2_assignable_sw = byte15 & 0x3F
		self.sw2_mode = (byte15 >> 6) & 0x01
		self.sw2_on_off = (byte15 >> 7) & 0x01
		self.rt_knob_1_assign = v.pop(0)
		self.rt_knob_2_assign = v.pop(0)
		self.rt_knob_3_assign = v.pop(0)
		self.rt_knob_4_assign = v.pop(0)
		byte20 = v.pop(0)
		self.trim = byte20 & 0x7F
		self.trim_status = (byte20 >> 7) & 0x01
		self.reserved813 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved862 = v.pop(0)
		self.tone_adjust_slider1 = v.pop(0)
		self.tone_adjust_slider2 = v.pop(0)
		self.tone_adjust_slider3 = v.pop(0)
		self.tone_adjust_slider4 = v.pop(0)
		self.tone_adjust_slider5 = v.pop(0)
		self.tone_adjust_slider6 = v.pop(0)
		self.tone_adjust_slider7 = v.pop(0)
		self.tone_adjust_slider8 = v.pop(0)
		self.tone_adjust_switch1 = v.pop(0)
		self.tone_adjust_switch2 = v.pop(0)
		self.tone_adjust_switch3 = v.pop(0)
		self.tone_adjust_switch4 = v.pop(0)
		self.tone_adjust_switch5 = v.pop(0)
		self.tone_adjust_switch6 = v.pop(0)
		self.tone_adjust_switch7 = v.pop(0)
		self.tone_adjust_switch8 = v.pop(0)
		self.key_zone_low1 = v.pop(0)
		self.key_zone_high1 = v.pop(0)
		self.key_zone_low2 = v.pop(0)
		self.key_zone_high2 = v.pop(0)
		byte96 = v.pop(0)
		self.osc1_patch_level = byte96 & 0x7F
		self.osc1_mute = (byte96 >> 7) & 0x01
		byte97 = v.pop(0)
		self.osc2_patch_level = byte97 & 0x7F
		self.osc2_mute = (byte97 >> 7) & 0x01
		self._reserved934 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		byte0 = 0
		byte0 |= self.osc_mode & 0x07
		byte0 = (self.osc_mode_favorite & 0x01) << 5
		byte0 = (self.osc_mode_hold & 0x01) << 6
		byte0 = (self.osc_mode_poly_mono & 0x01) << 7
		byte1 = 0
		byte1 |= self.poly_single_trigger & 0x01
		byte1 = (self.poly_poly_legato & 0x01) << 1
		byte1 = (self.mono_priority & 0x03) << 2
		byte1 = (self.mono_mono_legato & 0x01) << 4
		byte1 = (self.mono_mode & 0x01) << 5
		byte1 = (self.enable_half_damper & 0x01) << 6
		byte2 = 0
		byte2 |= self.number_of_voices & 0x07
		byte2 = (self.unison & 0x01) << 7
		v.append(byte2)
		v.append(self.detune)
		v.append(self.thickness)
		v.append(self.hold_bottom)
		v.append(self.hold_top)
		byte7 = 0
		byte7 |= self.bus_select & 0x1F
		byte7 = (self.fx_control_bus & 0x03) << 5
		byte7 = (self.use_dkit_setting & 0x01) << 7
		v.append(byte7)
		v.append(self._reserved844)
		byte10 = 0
		byte10 |= self.main_category & 0x1F
		byte10 = (self.sub_category & 0x07) << 5
		v.append(byte10)
		v.append(self.scale_type)
		v.append(self.scale_key)
		v.append(self.scale_random)
		byte14 = 0
		byte14 |= self.sw1_assignable_sw & 0x3F
		byte14 = (self.sw1_mode & 0x01) << 6
		byte14 = (self.sw1_on_off & 0x01) << 7
		byte15 = 0
		byte15 |= self.sw2_assignable_sw & 0x3F
		byte15 = (self.sw2_mode & 0x01) << 6
		byte15 = (self.sw2_on_off & 0x01) << 7
		v.append(byte15)
		v.append(self.rt_knob_1_assign)
		v.append(self.rt_knob_2_assign)
		v.append(self.rt_knob_3_assign)
		v.append(self.rt_knob_4_assign)
		byte20 = 0
		byte20 |= self.trim & 0x7F
		byte20 = (self.trim_status & 0x01) << 7
		v.append(byte20)
		v.append(self.reserved813)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved862)
		v.append(self.tone_adjust_slider1.pack())
		v.append(self.tone_adjust_slider2.pack())
		v.append(self.tone_adjust_slider3.pack())
		v.append(self.tone_adjust_slider4.pack())
		v.append(self.tone_adjust_slider5.pack())
		v.append(self.tone_adjust_slider6.pack())
		v.append(self.tone_adjust_slider7.pack())
		v.append(self.tone_adjust_slider8.pack())
		v.append(self.tone_adjust_switch1.pack())
		v.append(self.tone_adjust_switch2.pack())
		v.append(self.tone_adjust_switch3.pack())
		v.append(self.tone_adjust_switch4.pack())
		v.append(self.tone_adjust_switch5.pack())
		v.append(self.tone_adjust_switch6.pack())
		v.append(self.tone_adjust_switch7.pack())
		v.append(self.tone_adjust_switch8.pack())
		v.append(self.key_zone_low1)
		v.append(self.key_zone_high1)
		v.append(self.key_zone_low2)
		v.append(self.key_zone_high2)
		byte96 = 0
		byte96 |= self.osc1_patch_level & 0x7F
		byte96 = (self.osc1_mute & 0x01) << 7
		byte97 = 0
		byte97 |= self.osc2_patch_level & 0x7F
		byte97 = (self.osc2_mute & 0x01) << 7
		v.append(byte97)
		v.append(self._reserved934)
		return struct.pack('>8BH13B3bH4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s4s6BH',*v) # 100

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def osc_mode(self):
		''''''
		return self._osc_mode

	@osc_mode.setter
	def osc_mode(self,value):
		if 0 <= value <= 5:
			self._osc_mode = value
		else:
			raise ValueError('not 0 <= value <= 5')

#-------------------------------------------------------------------------------
	@property
	def osc_mode_favorite(self):
		''''''
		return self._osc_mode_favorite

	@osc_mode_favorite.setter
	def osc_mode_favorite(self,value):
		if 0 <= value <= 1:
			self._osc_mode_favorite = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def osc_mode_hold(self):
		''''''
		return self._osc_mode_hold

	@osc_mode_hold.setter
	def osc_mode_hold(self,value):
		if 0 <= value <= 1:
			self._osc_mode_hold = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def osc_mode_poly_mono(self):
		''''''
		return self._osc_mode_poly_mono

	@osc_mode_poly_mono.setter
	def osc_mode_poly_mono(self,value):
		if 0 <= value <= 1:
			self._osc_mode_poly_mono = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def poly_single_trigger(self):
		'''(Poly)Single Trigger'''
		return self._poly_single_trigger

	@poly_single_trigger.setter
	def poly_single_trigger(self,value):
		if 0 <= value <= 1:
			self._poly_single_trigger = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def poly_poly_legato(self):
		'''(Poly)Poly Legato'''
		return self._poly_poly_legato

	@poly_poly_legato.setter
	def poly_poly_legato(self,value):
		if 0 <= value <= 1:
			self._poly_poly_legato = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def mono_priority(self):
		'''(Mono)Priority'''
		return self._mono_priority

	@mono_priority.setter
	def mono_priority(self,value):
		if 0 <= value <= 2:
			self._mono_priority = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def mono_mono_legato(self):
		'''(Mono)Mono Legato'''
		return self._mono_mono_legato

	@mono_mono_legato.setter
	def mono_mono_legato(self,value):
		if 0 <= value <= 1:
			self._mono_mono_legato = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def mono_mode(self):
		'''(Mono)Mode'''
		return self._mono_mode

	@mono_mode.setter
	def mono_mode(self,value):
		if 0 <= value <= 1:
			self._mono_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def enable_half_damper(self):
		'''Enable Half Damper'''
		return self._enable_half_damper

	@enable_half_damper.setter
	def enable_half_damper(self,value):
		if 0 <= value <= 1:
			self._enable_half_damper = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def number_of_voices(self):
		'''Number Of Voices'''
		return self._number_of_voices

	@number_of_voices.setter
	def number_of_voices(self,value):
		if 0 <= value <= 6:
			self._number_of_voices = value
		else:
			raise ValueError('not 0 <= value <= 6')

#-------------------------------------------------------------------------------
	@property
	def unison(self):
		'''Unison'''
		return self._unison

	@unison.setter
	def unison(self,value):
		if 0 <= value <= 1:
			self._unison = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if 0 <= value <= 99:
			self._detune = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def thickness(self):
		'''Thickness'''
		return self._thickness

	@thickness.setter
	def thickness(self,value):
		if 0 <= value <= 9:
			self._thickness = value
		else:
			raise ValueError('not 0 <= value <= 9')

#-------------------------------------------------------------------------------
	@property
	def hold_bottom(self):
		'''Hold Bottom'''
		return self._hold_bottom

	@hold_bottom.setter
	def hold_bottom(self,value):
		if 0 <= value <= 127:
			self._hold_bottom = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def hold_top(self):
		'''Hold Top'''
		return self._hold_top

	@hold_top.setter
	def hold_top(self,value):
		if 0 <= value <= 127:
			self._hold_top = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if 0 <= value <= 12:
			self._bus_select = value
		else:
			raise ValueError('not 0 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def fx_control_bus(self):
		''''''
		return self._fx_control_bus

	@fx_control_bus.setter
	def fx_control_bus(self,value):
		if 0 <= value <= 2:
			self._fx_control_bus = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def use_dkit_setting(self):
		'''Use DKit Setting'''
		return self._use_dkit_setting

	@use_dkit_setting.setter
	def use_dkit_setting(self,value):
		if 0 <= value <= 1:
			self._use_dkit_setting = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def main_category(self):
		'''Main Category '''
		return self._main_category

	@main_category.setter
	def main_category(self,value):
		if 0 <= value <= 17:
			self._main_category = value
		else:
			raise ValueError('not 0 <= value <= 17')

#-------------------------------------------------------------------------------
	@property
	def sub_category(self):
		'''Sub Category'''
		return self._sub_category

	@sub_category.setter
	def sub_category(self,value):
		if 0 <= value <= 7:
			self._sub_category = value
		else:
			raise ValueError('not 0 <= value <= 7')

#-------------------------------------------------------------------------------
	@property
	def scale_type(self):
		'''Scale Type'''
		return self._scale_type

	@scale_type.setter
	def scale_type(self,value):
		if 0 <= value <= 15:
			self._scale_type = value
		else:
			raise ValueError('not 0 <= value <= 15')

#-------------------------------------------------------------------------------
	@property
	def scale_key(self):
		'''Scale Key'''
		return self._scale_key

	@scale_key.setter
	def scale_key(self,value):
		if 0 <= value <= 11:
			self._scale_key = value
		else:
			raise ValueError('not 0 <= value <= 11')

#-------------------------------------------------------------------------------
	@property
	def scale_random(self):
		'''Scale Random'''
		return self._scale_random

	@scale_random.setter
	def scale_random(self,value):
		if 0 <= value <= 7:
			self._scale_random = value
		else:
			raise ValueError('not 0 <= value <= 7')

#-------------------------------------------------------------------------------
	@property
	def sw1_assignable_sw(self):
		'''SW1 Assignable SW'''
		return self._sw1_assignable_sw

	@sw1_assignable_sw.setter
	def sw1_assignable_sw(self,value):
		if 0 <= value <= 14:
			self._sw1_assignable_sw = value
		else:
			raise ValueError('not 0 <= value <= 14')

#-------------------------------------------------------------------------------
	@property
	def sw1_mode(self):
		'''SW1 Mode'''
		return self._sw1_mode

	@sw1_mode.setter
	def sw1_mode(self,value):
		if 0 <= value <= 1:
			self._sw1_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def sw1_on_off(self):
		'''SW1 On/Off'''
		return self._sw1_on_off

	@sw1_on_off.setter
	def sw1_on_off(self,value):
		if 0 <= value <= 1:
			self._sw1_on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def sw2_assignable_sw(self):
		'''SW2 Assignable SW'''
		return self._sw2_assignable_sw

	@sw2_assignable_sw.setter
	def sw2_assignable_sw(self,value):
		if 0 <= value <= 14:
			self._sw2_assignable_sw = value
		else:
			raise ValueError('not 0 <= value <= 14')

#-------------------------------------------------------------------------------
	@property
	def sw2_mode(self):
		'''SW2 Mode'''
		return self._sw2_mode

	@sw2_mode.setter
	def sw2_mode(self,value):
		if 0 <= value <= 1:
			self._sw2_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def sw2_on_off(self):
		'''SW2 On/Off'''
		return self._sw2_on_off

	@sw2_on_off.setter
	def sw2_on_off(self,value):
		if 0 <= value <= 1:
			self._sw2_on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def rt_knob_1_assign(self):
		'''RT Knob-1 Assign'''
		return self._rt_knob_1_assign

	@rt_knob_1_assign.setter
	def rt_knob_1_assign(self,value):
		if 0 <= value <= 142:
			self._rt_knob_1_assign = value
		else:
			raise ValueError('not 0 <= value <= 142')

#-------------------------------------------------------------------------------
	@property
	def rt_knob_2_assign(self):
		'''RT Knob-2 Assign'''
		return self._rt_knob_2_assign

	@rt_knob_2_assign.setter
	def rt_knob_2_assign(self,value):
		if 0 <= value <= 142:
			self._rt_knob_2_assign = value
		else:
			raise ValueError('not 0 <= value <= 142')

#-------------------------------------------------------------------------------
	@property
	def rt_knob_3_assign(self):
		'''RT Knob-3 Assign'''
		return self._rt_knob_3_assign

	@rt_knob_3_assign.setter
	def rt_knob_3_assign(self,value):
		if 0 <= value <= 142:
			self._rt_knob_3_assign = value
		else:
			raise ValueError('not 0 <= value <= 142')

#-------------------------------------------------------------------------------
	@property
	def rt_knob_4_assign(self):
		'''RT Knob-4 Assign'''
		return self._rt_knob_4_assign

	@rt_knob_4_assign.setter
	def rt_knob_4_assign(self,value):
		if 0 <= value <= 142:
			self._rt_knob_4_assign = value
		else:
			raise ValueError('not 0 <= value <= 142')

#-------------------------------------------------------------------------------
	@property
	def trim(self):
		'''Trim'''
		return self._trim

	@trim.setter
	def trim(self,value):
		if 0 <= value <= 153:
			self._trim = value
		else:
			raise ValueError('not 0 <= value <= 153')

#-------------------------------------------------------------------------------
	@property
	def trim_status(self):
		'''Trim Normal/Bypass'''
		return self._trim_status

	@trim_status.setter
	def trim_status(self,value):
		if 0 <= value <= 1:
			self._trim_status = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def reserved813(self):
		'''None'''
		return self._reserved813

	@reserved813.setter
	def reserved813(self,value):
		self._reserved813 = value

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''MidFreq'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
		else:
			raise ValueError('not 0 <= value <= 165')

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''LowGain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''MidGain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''HighGain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
		else:
			raise ValueError('not -36 <= value <= 36')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider1(self):
		'''Tone Adjust Slider 1 Parameters'''
		return self._tone_adjust_slider1

	@tone_adjust_slider1.setter
	def tone_adjust_slider1(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider1 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider1 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider2(self):
		'''Tone Adjust Slider 2 Parameters'''
		return self._tone_adjust_slider2

	@tone_adjust_slider2.setter
	def tone_adjust_slider2(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider2 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider2 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider3(self):
		'''Tone Adjust Slider 3 Parameters'''
		return self._tone_adjust_slider3

	@tone_adjust_slider3.setter
	def tone_adjust_slider3(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider3 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider3 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider4(self):
		'''Tone Adjust Slider 4 Parameters'''
		return self._tone_adjust_slider4

	@tone_adjust_slider4.setter
	def tone_adjust_slider4(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider4 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider4 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider5(self):
		'''Tone Adjust Slider 5 Parameters'''
		return self._tone_adjust_slider5

	@tone_adjust_slider5.setter
	def tone_adjust_slider5(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider5 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider5 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider6(self):
		'''Tone Adjust Slider 6 Parameters'''
		return self._tone_adjust_slider6

	@tone_adjust_slider6.setter
	def tone_adjust_slider6(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider6 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider6 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider7(self):
		'''Tone Adjust Slider 7 Parameters'''
		return self._tone_adjust_slider7

	@tone_adjust_slider7.setter
	def tone_adjust_slider7(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider7 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider7 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_slider8(self):
		'''Tone Adjust Slider 8 Parameters'''
		return self._tone_adjust_slider8

	@tone_adjust_slider8.setter
	def tone_adjust_slider8(self,value):
		if isinstance(value,ToneAdjustSliderParameters):
			self._tone_adjust_slider8 = value
		elif isinstance(value,basestring):
			self._tone_adjust_slider8 = ToneAdjustSliderParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSliderParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch1(self):
		'''Tone Adjust Switch 1 Parameters'''
		return self._tone_adjust_switch1

	@tone_adjust_switch1.setter
	def tone_adjust_switch1(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch1 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch1 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch2(self):
		'''Tone Adjust Switch 2 Parameters'''
		return self._tone_adjust_switch2

	@tone_adjust_switch2.setter
	def tone_adjust_switch2(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch2 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch2 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch3(self):
		'''Tone Adjust Switch 3 Parameters'''
		return self._tone_adjust_switch3

	@tone_adjust_switch3.setter
	def tone_adjust_switch3(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch3 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch3 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch4(self):
		'''Tone Adjust Switch 4 Parameters'''
		return self._tone_adjust_switch4

	@tone_adjust_switch4.setter
	def tone_adjust_switch4(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch4 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch4 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch5(self):
		'''Tone Adjust Switch 5 Parameters'''
		return self._tone_adjust_switch5

	@tone_adjust_switch5.setter
	def tone_adjust_switch5(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch5 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch5 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch6(self):
		'''Tone Adjust Switch 6 Parameters'''
		return self._tone_adjust_switch6

	@tone_adjust_switch6.setter
	def tone_adjust_switch6(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch6 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch6 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch7(self):
		'''Tone Adjust Switch 7 Parameters'''
		return self._tone_adjust_switch7

	@tone_adjust_switch7.setter
	def tone_adjust_switch7(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch7 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch7 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def tone_adjust_switch8(self):
		'''Tone Adjust Switch 8 Parameters'''
		return self._tone_adjust_switch8

	@tone_adjust_switch8.setter
	def tone_adjust_switch8(self,value):
		if isinstance(value,ToneAdjustSwitchParameters):
			self._tone_adjust_switch8 = value
		elif isinstance(value,basestring):
			self._tone_adjust_switch8 = ToneAdjustSwitchParameters(value)
		else:
			raise TypeError('value must of type ToneAdjustSwitchParameters')

#-------------------------------------------------------------------------------
	@property
	def key_zone_low1(self):
		'''Key Zone Low1'''
		return self._key_zone_low1

	@key_zone_low1.setter
	def key_zone_low1(self,value):
		if 0 <= value <= 127:
			self._key_zone_low1 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_zone_high1(self):
		'''Key Zone High1'''
		return self._key_zone_high1

	@key_zone_high1.setter
	def key_zone_high1(self,value):
		if 0 <= value <= 127:
			self._key_zone_high1 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_zone_low2(self):
		'''Key Zone Low2'''
		return self._key_zone_low2

	@key_zone_low2.setter
	def key_zone_low2(self,value):
		if 0 <= value <= 127:
			self._key_zone_low2 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_zone_high2(self):
		'''Key Zone High2'''
		return self._key_zone_high2

	@key_zone_high2.setter
	def key_zone_high2(self,value):
		if 0 <= value <= 127:
			self._key_zone_high2 = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def osc1_patch_level(self):
		'''OSC1 Patch Level'''
		return self._osc1_patch_level

	@osc1_patch_level.setter
	def osc1_patch_level(self,value):
		if 0 <= value <= 127:
			self._osc1_patch_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def osc1_mute(self):
		'''OSC1 Mute'''
		return self._osc1_mute

	@osc1_mute.setter
	def osc1_mute(self,value):
		if 0 <= value <= 1:
			self._osc1_mute = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def osc2_patch_level(self):
		'''OSC2 Patch Level'''
		return self._osc2_patch_level

	@osc2_patch_level.setter
	def osc2_patch_level(self,value):
		if 0 <= value <= 127:
			self._osc2_patch_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def osc2_mute(self):
		'''OSC2 Mute'''
		return self._osc2_mute

	@osc2_mute.setter
	def osc2_mute(self,value):
		if 0 <= value <= 1:
			self._osc2_mute = value
		else:
			raise ValueError('not 0 <= value <= 1')

#===============================================================================
# Table 1 864-867 x 8
class ToneAdjustSliderParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return '%d 0x%x %d' % (self.assign,self._reserved865,self.value)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<2Bh',raw)) # 4 
		self.assign = v.pop(0)
		self._reserved865 = v.pop(0)
		self.value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.assign)
		v.append(self._reserved865)
		v.append(self.value)
		return struct.pack('>2Bh',*v) # 4

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def assign(self):
		'''Assign'''
		return self._assign

	@assign.setter
	def assign(self,value):
		if 0 <= value <= 77:
			self._assign = value
		else:
			raise ValueError('not 0 <= value <= 77')

#-------------------------------------------------------------------------------
	@property
	def value(self):
		'''Value'''
		return self._value

	@value.setter
	def value(self,value):
		if -16383 <= value <= 16382:
			self._value = value
		else:
			raise ValueError('not -16383 <= value <= 16382')

#===============================================================================
# Table 1 896-899 x 8
class ToneAdjustSwitchParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def __str__(self):
		return '%d %d %d' % (self.assign,self.on_off,self.value)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<2Bh',raw)) # 4 
		byte0 = v.pop(0)
		if byte0 <= 34:
			self.assign = byte0
		elif 48 <= byte0 <= 70:
			self.assign = byte0-13
		elif 176 <= byte0 <= 198:
			self.assign = byte0-141
		self.on_off = v.pop(0)
		self.value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.assign)
		v.append(self.on_off)
		v.append(self.value)
		return struct.pack('>2Bh',*v) # 4

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def assign(self):
		'''Assign'''
		return self._assign

	@assign.setter
	def assign(self,value):
		if 0 <= value <= 77:
			self._assign = value
		else:
			raise ValueError('not 0 <= value[%d] <= 77' % value)

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def value(self):
		'''Value'''
		return self._value

	@value.setter
	def value(self,value):
		if -16383 <= value <= 16382:
			self._value = value
		else:
			raise ValueError('not -16383 <= value <= 16382')

#===============================================================================
# Table 1 936-1003
class PCMOSCCommonParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<bBbBb2Bb5B3bB7bBbHBbH2BHBbHBbH3B3b2B8s8s',raw)) # 68 
		self.start_level = v.pop(0)
		self.attack_time = v.pop(0)
		self.attack_level = v.pop(0)
		self.decay_time = v.pop(0)
		self.break_level = v.pop(0)
		self.slope_time = v.pop(0)
		self.release_time = v.pop(0)
		self.release_level = v.pop(0)
		self.attack_eg_curve = v.pop(0)
		self.delay_eg_curve = v.pop(0)
		self.slope_eg_curve = v.pop(0)
		self.release_eg_curve = v.pop(0)
		self.ams1_level = v.pop(0)
		self.ams1_start_level = v.pop(0)
		self.ams1_attack_level = v.pop(0)
		self.ams1_break_level = v.pop(0)
		self.ams2_level = v.pop(0)
		self.ams2_start_level = v.pop(0)
		self.ams2_attack_level = v.pop(0)
		self.ams2_break_level = v.pop(0)
		self.ams_time = v.pop(0)
		self.ams_attack_time = v.pop(0)
		self.ams_decay_time = v.pop(0)
		self.ams_slope_time = v.pop(0)
		self.reset_source = v.pop(0)
		self.reset_level = v.pop(0)
		self._reserved962 = v.pop(0)
		self.ams_to_shape_source = v.pop(0)
		self.ams_to_shape_level = v.pop(0)
		self._reserved966 = v.pop(0)
		byte32 = v.pop(0)
		self.base_note = byte32 & 0x0F
		self.midi_sync_off = (byte32 >> 7) & 0x01
		self.midi_sync_time = v.pop(0)
		self._reserved970 = v.pop(0)
		self.ams_to_time_source = v.pop(0)
		self.ams_to_time_intensity = v.pop(0)
		self._reserved974 = v.pop(0)
		self.ams_to_ams_time_source = v.pop(0)
		self.ams_to_ams_time_ = v.pop(0)
		self._reserved978 = v.pop(0)
		byte44 = v.pop(0)
		self.sync_wave_form = byte44 & 0x3F
		self.sync_wave_form_stop = (byte44 >> 6) & 0x01
		self.frequency = v.pop(0)
		self._reserved982 = v.pop(0)
		self.start_phase = v.pop(0)
		self.shape = v.pop(0)
		self.offset = v.pop(0)
		self.reset_ams = v.pop(0)
		self._reserved987 = v.pop(0)
		self.KeyTrack1 = v.pop(0)
		self.KeyTrack2 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.start_level)
		v.append(self.attack_time)
		v.append(self.attack_level)
		v.append(self.decay_time)
		v.append(self.break_level)
		v.append(self.slope_time)
		v.append(self.release_time)
		v.append(self.release_level)
		v.append(self.attack_eg_curve)
		v.append(self.delay_eg_curve)
		v.append(self.slope_eg_curve)
		v.append(self.release_eg_curve)
		v.append(self.ams1_level)
		v.append(self.ams1_start_level)
		v.append(self.ams1_attack_level)
		v.append(self.ams1_break_level)
		v.append(self.ams2_level)
		v.append(self.ams2_start_level)
		v.append(self.ams2_attack_level)
		v.append(self.ams2_break_level)
		v.append(self.ams_time)
		v.append(self.ams_attack_time)
		v.append(self.ams_decay_time)
		v.append(self.ams_slope_time)
		v.append(self.reset_source)
		v.append(self.reset_level)
		v.append(self._reserved962)
		v.append(self.ams_to_shape_source)
		v.append(self.ams_to_shape_level)
		v.append(self._reserved966)
		byte32 = 0
		byte32 |= self.base_note & 0x0F
		byte32 = (self.midi_sync_off & 0x01) << 7
		v.append(byte32)
		v.append(self.midi_sync_time)
		v.append(self._reserved970)
		v.append(self.ams_to_time_source)
		v.append(self.ams_to_time_intensity)
		v.append(self._reserved974)
		v.append(self.ams_to_ams_time_source)
		v.append(self.ams_to_ams_time_)
		v.append(self._reserved978)
		byte44 = 0
		byte44 |= self.sync_wave_form & 0x3F
		byte44 = (self.sync_wave_form_stop & 0x01) << 6
		v.append(byte44)
		v.append(self.frequency)
		v.append(self._reserved982)
		v.append(self.start_phase)
		v.append(self.shape)
		v.append(self.offset)
		v.append(self.reset_ams)
		v.append(self._reserved987)
		v.append(self.KeyTrack1.pack())
		v.append(self.KeyTrack2.pack())
		return struct.pack('>bBbBb2Bb5B3bB7bBbHBbH2BHBbHBbH3B3b2B8s8s',*v) # 68

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def start_level(self):
		'''Start Level'''
		return self._start_level

	@start_level.setter
	def start_level(self,value):
		if -99 <= value <= 99:
			self._start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_time(self):
		'''Attack Time'''
		return self._attack_time

	@attack_time.setter
	def attack_time(self,value):
		if 0 <= value <= 99:
			self._attack_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_level(self):
		'''Attack Level'''
		return self._attack_level

	@attack_level.setter
	def attack_level(self,value):
		if -99 <= value <= 99:
			self._attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def decay_time(self):
		'''Decay Time'''
		return self._decay_time

	@decay_time.setter
	def decay_time(self,value):
		if 0 <= value <= 99:
			self._decay_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def break_level(self):
		'''Break Level'''
		return self._break_level

	@break_level.setter
	def break_level(self,value):
		if -99 <= value <= 99:
			self._break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def slope_time(self):
		'''Slope Time'''
		return self._slope_time

	@slope_time.setter
	def slope_time(self,value):
		if 0 <= value <= 99:
			self._slope_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def release_time(self):
		'''Release Time'''
		return self._release_time

	@release_time.setter
	def release_time(self,value):
		if 0 <= value <= 99:
			self._release_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def release_level(self):
		'''Release Level'''
		return self._release_level

	@release_level.setter
	def release_level(self,value):
		if -99 <= value <= 99:
			self._release_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_eg_curve(self):
		'''Attack EG Curve'''
		return self._attack_eg_curve

	@attack_eg_curve.setter
	def attack_eg_curve(self,value):
		if 0 <= value <= 10:
			self._attack_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def delay_eg_curve(self):
		'''Delay EG Curve'''
		return self._delay_eg_curve

	@delay_eg_curve.setter
	def delay_eg_curve(self,value):
		if 0 <= value <= 10:
			self._delay_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def slope_eg_curve(self):
		'''Slope EG Curve'''
		return self._slope_eg_curve

	@slope_eg_curve.setter
	def slope_eg_curve(self,value):
		if 0 <= value <= 10:
			self._slope_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def release_eg_curve(self):
		'''Release EG Curve'''
		return self._release_eg_curve

	@release_eg_curve.setter
	def release_eg_curve(self,value):
		if 0 <= value <= 10:
			self._release_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def ams1_level(self):
		'''AMS1 Level'''
		return self._ams1_level

	@ams1_level.setter
	def ams1_level(self,value):
		if 0 <= value <= 50:
			self._ams1_level = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams1_start_level(self):
		'''AMS1 Start Level'''
		return self._ams1_start_level

	@ams1_start_level.setter
	def ams1_start_level(self,value):
		if -99 <= value <= 99:
			self._ams1_start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams1_attack_level(self):
		'''AMS1 Attack Level'''
		return self._ams1_attack_level

	@ams1_attack_level.setter
	def ams1_attack_level(self,value):
		if -99 <= value <= 99:
			self._ams1_attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams1_break_level(self):
		'''AMS1 Break Level'''
		return self._ams1_break_level

	@ams1_break_level.setter
	def ams1_break_level(self,value):
		if -99 <= value <= 99:
			self._ams1_break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams2_level(self):
		'''AMS2 Level'''
		return self._ams2_level

	@ams2_level.setter
	def ams2_level(self,value):
		if 0 <= value <= 50:
			self._ams2_level = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams2_start_level(self):
		'''AMS2 Start Level'''
		return self._ams2_start_level

	@ams2_start_level.setter
	def ams2_start_level(self,value):
		if -99 <= value <= 99:
			self._ams2_start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams2_attack_level(self):
		'''AMS2 Attack Level'''
		return self._ams2_attack_level

	@ams2_attack_level.setter
	def ams2_attack_level(self,value):
		if -99 <= value <= 99:
			self._ams2_attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams2_break_level(self):
		'''AMS2 Break Level'''
		return self._ams2_break_level

	@ams2_break_level.setter
	def ams2_break_level(self,value):
		if -99 <= value <= 99:
			self._ams2_break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_time(self):
		'''AMS Time'''
		return self._ams_time

	@ams_time.setter
	def ams_time(self,value):
		if -99 <= value <= 99:
			self._ams_time = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_attack_time(self):
		'''AMS Attack Time'''
		return self._ams_attack_time

	@ams_attack_time.setter
	def ams_attack_time(self,value):
		if -99 <= value <= 99:
			self._ams_attack_time = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_decay_time(self):
		''' Level'''
		return self._ams_decay_time

	@ams_decay_time.setter
	def ams_decay_time(self,value):
		if -99 <= value <= 99:
			self._ams_decay_time = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_slope_time(self):
		''' Level'''
		return self._ams_slope_time

	@ams_slope_time.setter
	def ams_slope_time(self,value):
		if -99 <= value <= 99:
			self._ams_slope_time = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_source(self):
		'''Reset Source'''
		return self._reset_source

	@reset_source.setter
	def reset_source(self,value):
		if 0 <= value <= 50:
			self._reset_source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def reset_level(self):
		'''Reset Threshold '''
		return self._reset_level

	@reset_level.setter
	def reset_level(self,value):
		if -99 <= value <= 99:
			self._reset_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_shape_source(self):
		'''AMSToShape Source'''
		return self._ams_to_shape_source

	@ams_to_shape_source.setter
	def ams_to_shape_source(self,value):
		if 0 <= value <= 50:
			self._ams_to_shape_source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams_to_shape_level(self):
		'''AMSToShape Intensity'''
		return self._ams_to_shape_level

	@ams_to_shape_level.setter
	def ams_to_shape_level(self,value):
		if -99 <= value <= 99:
			self._ams_to_shape_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def base_note(self):
		'''Base Note'''
		return self._base_note

	@base_note.setter
	def base_note(self,value):
		if 0 <= value <= 9:
			self._base_note = value
		else:
			raise ValueError('not 0 <= value <= 9')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_off(self):
		'''MIDI Sync Off'''
		return self._midi_sync_off

	@midi_sync_off.setter
	def midi_sync_off(self,value):
		if 0 <= value <= 1:
			self._midi_sync_off = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_time(self):
		'''MIDI Sync Time'''
		return self._midi_sync_time

	@midi_sync_time.setter
	def midi_sync_time(self,value):
		if 0 <= value <= 31:
			self._midi_sync_time = value
		else:
			raise ValueError('not 0 <= value <= 31')

#-------------------------------------------------------------------------------
	@property
	def ams_to_time_source(self):
		'''AMSToTime Source'''
		return self._ams_to_time_source

	@ams_to_time_source.setter
	def ams_to_time_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_time_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_time_intensity(self):
		'''AMSToTime Intensity'''
		return self._ams_to_time_intensity

	@ams_to_time_intensity.setter
	def ams_to_time_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_time_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_ams_time_source(self):
		'''AMSToAMSTime Source'''
		return self._ams_to_ams_time_source

	@ams_to_ams_time_source.setter
	def ams_to_ams_time_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_ams_time_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_ams_time_(self):
		'''AMSToAMSTime Intensity'''
		return self._ams_to_ams_time_

	@ams_to_ams_time_.setter
	def ams_to_ams_time_(self,value):
		if -99 <= value <= 99:
			self._ams_to_ams_time_ = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def sync_wave_form(self):
		'''Sync Wave Form'''
		return self._sync_wave_form

	@sync_wave_form.setter
	def sync_wave_form(self,value):
		if 0 <= value <= 17:
			self._sync_wave_form = value
		else:
			raise ValueError('not 0 <= value <= 17')

#-------------------------------------------------------------------------------
	@property
	def sync_wave_form_stop(self):
		'''Stop'''
		return self._sync_wave_form_stop

	@sync_wave_form_stop.setter
	def sync_wave_form_stop(self,value):
		if 0 <= value <= 1:
			self._sync_wave_form_stop = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def frequency(self):
		'''Frequency'''
		return self._frequency

	@frequency.setter
	def frequency(self,value):
		if 0 <= value <= 99:
			self._frequency = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def start_phase(self):
		'''Start Phase'''
		return self._start_phase

	@start_phase.setter
	def start_phase(self,value):
		if -36 <= value <= 37:
			self._start_phase = value
		else:
			raise ValueError('not -36 <= value <= 37')

#-------------------------------------------------------------------------------
	@property
	def shape(self):
		'''Shape'''
		return self._shape

	@shape.setter
	def shape(self,value):
		if -99 <= value <= 99:
			self._shape = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def offset(self):
		'''Offset'''
		return self._offset

	@offset.setter
	def offset(self,value):
		if -99 <= value <= 99:
			self._offset = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_ams(self):
		'''Reset AMS'''
		return self._reset_ams

	@reset_ams.setter
	def reset_ams(self,value):
		if 0 <= value <= 50:
			self._reset_ams = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def KeyTrack1(self):
		'''PCM OSC KEY TRACK 1 PARAMETERS'''
		return self._KeyTrack1

	@KeyTrack1.setter
	def KeyTrack1(self,value):
		if isinstance(value,PCMOSCKeyTrackParameters):
			self._KeyTrack1 = value
		elif isinstance(value,basestring):
			self._KeyTrack1 = PCMOSCKeyTrackParameters(value)
		else:
			raise TypeError('value must of type PCMOSCKeyTrackParameters')

#-------------------------------------------------------------------------------
	@property
	def KeyTrack2(self):
		'''PCM OSC KEY TRACK 2 PARAMETERS'''
		return self._KeyTrack2

	@KeyTrack2.setter
	def KeyTrack2(self,value):
		if isinstance(value,PCMOSCKeyTrackParameters):
			self._KeyTrack2 = value
		elif isinstance(value,basestring):
			self._KeyTrack2 = PCMOSCKeyTrackParameters(value)
		else:
			raise TypeError('value must of type PCMOSCKeyTrackParameters')

#===============================================================================
# Table 1 988-995 x 2
class PCMOSCKeyTrackParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<3B4bB',raw)) # 8 
		self.key_low = v.pop(0)
		self.key_center = v.pop(0)
		self.key_high = v.pop(0)
		self.ramp_low = v.pop(0)
		self.ramp_center = v.pop(0)
		self.ramp_center_high = v.pop(0)
		self.ramp_high_top = v.pop(0)
		self._reserved995 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.key_low)
		v.append(self.key_center)
		v.append(self.key_high)
		v.append(self.ramp_low)
		v.append(self.ramp_center)
		v.append(self.ramp_center_high)
		v.append(self.ramp_high_top)
		v.append(self._reserved995)
		return struct.pack('>3B4bB',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def key_low(self):
		'''Key Low'''
		return self._key_low

	@key_low.setter
	def key_low(self,value):
		if 0 <= value <= 127:
			self._key_low = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_center(self):
		'''Key Center'''
		return self._key_center

	@key_center.setter
	def key_center(self,value):
		if 0 <= value <= 127:
			self._key_center = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_high(self):
		'''Key High'''
		return self._key_high

	@key_high.setter
	def key_high(self,value):
		if 0 <= value <= 127:
			self._key_high = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def ramp_low(self):
		'''Ramp Low'''
		return self._ramp_low

	@ramp_low.setter
	def ramp_low(self,value):
		if -100 <= value <= 100:
			self._ramp_low = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center(self):
		'''Ramp Center'''
		return self._ramp_center

	@ramp_center.setter
	def ramp_center(self,value):
		if -100 <= value <= 100:
			self._ramp_center = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center_high(self):
		'''Ramp Center High'''
		return self._ramp_center_high

	@ramp_center_high.setter
	def ramp_center_high(self,value):
		if -100 <= value <= 100:
			self._ramp_center_high = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_high_top(self):
		'''Ramp High Top'''
		return self._ramp_high_top

	@ramp_high_top.setter
	def ramp_high_top(self,value):
		if -100 <= value <= 100:
			self._ramp_high_top = value
		else:
			raise ValueError('not -100 <= value <= 100')

#===============================================================================
# Table 1 1004-1443 x 2
class PCMOSCStorageParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<8s8s8s8s8s8s8s8s4s4s4s4s4s4s4s4B28s28s32s8s36s36s36s8s12s20s36s8s8s24s24s',raw)) # 440 
		self.multisample1 = v.pop(0)
		self.multisample2 = v.pop(0)
		self.multisample3 = v.pop(0)
		self.multisample4 = v.pop(0)
		self.multisample5 = v.pop(0)
		self.multisample6 = v.pop(0)
		self.multisample7 = v.pop(0)
		self.multisample8 = v.pop(0)
		self.multisample_velocity_control1 = v.pop(0)
		self.multisample_velocity_control2 = v.pop(0)
		self.multisample_velocity_control3 = v.pop(0)
		self.multisample_velocity_control4 = v.pop(0)
		self.multisample_velocity_control5 = v.pop(0)
		self.multisample_velocity_control6 = v.pop(0)
		self.multisample_velocity_control7 = v.pop(0)
		byte92 = v.pop(0)
		self.note_on_ctrl_delay = byte92 & 0x7F
		self.note_on_ctrl_mode = (byte92 >> 7) & 0x01
		self.velocity_zone_bottom = v.pop(0)
		self.velocity_zone_top = v.pop(0)
		self._reserved1099 = v.pop(0)
		self.pcm_osc_lfo1 = v.pop(0)
		self.pcm_osc_lfo2 = v.pop(0)
		self.pcm_osc_pitch = v.pop(0)
		self.pcm_osc_filter = v.pop(0)
		self.pcm_osc_filter_block1 = v.pop(0)
		self.pcm_osc_filter_block2 = v.pop(0)
		self.pcm_osc_filter_eg = v.pop(0)
		self.pcm_osc_filter_key_track = v.pop(0)
		self.pcm_osc_drive = v.pop(0)
		self.pcm_osc_amp = v.pop(0)
		self.pcm_osc_amp_eg = v.pop(0)
		self.pcm_osc_amp_key_track = v.pop(0)
		self.pcm_osc_output = v.pop(0)
		self.ams_mixer1 = v.pop(0)
		self.ams_mixer2 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.multisample1.pack())
		v.append(self.multisample2.pack())
		v.append(self.multisample3.pack())
		v.append(self.multisample4.pack())
		v.append(self.multisample5.pack())
		v.append(self.multisample6.pack())
		v.append(self.multisample7.pack())
		v.append(self.multisample8.pack())
		v.append(self.multisample_velocity_control1.pack())
		v.append(self.multisample_velocity_control2.pack())
		v.append(self.multisample_velocity_control3.pack())
		v.append(self.multisample_velocity_control4.pack())
		v.append(self.multisample_velocity_control5.pack())
		v.append(self.multisample_velocity_control6.pack())
		v.append(self.multisample_velocity_control7.pack())
		byte92 = 0
		byte92 |= self.note_on_ctrl_delay & 0x7F
		byte92 |= (self.note_on_ctrl_mode & 0x01) << 7
		v.append(byte92)
		v.append(self.velocity_zone_bottom)
		v.append(self.velocity_zone_top)
		v.append(self._reserved1099)
		v.append(self.pcm_osc_lfo1.pack())
		v.append(self.pcm_osc_lfo2.pack())
		v.append(self.pcm_osc_pitch.pack())
		v.append(self.pcm_osc_filter.pack())
		v.append(self.pcm_osc_filter_block1.pack())
		v.append(self.pcm_osc_filter_block2.pack())
		v.append(self.pcm_osc_filter_eg.pack())
		v.append(self.pcm_osc_filter_key_track.pack())
		v.append(self.pcm_osc_drive.pack())
		v.append(self.pcm_osc_amp.pack())
		v.append(self.pcm_osc_amp_eg.pack())
		v.append(self.pcm_osc_amp_key_track.pack())
		v.append(self.pcm_osc_output.pack())
		v.append(self.ams_mixer1.pack())
		v.append(self.ams_mixer2.pack())
		return struct.pack('>8s8s8s8s8s8s8s8s4s4s4s4s4s4s4s4B28s28s32s8s36s36s36s8s12s20s36s8s8s24s24s',*v) # 440

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def multisample1(self):
		'''Multisample 1'''
		return self._multisample1

	@multisample1.setter
	def multisample1(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample1 = value
		elif isinstance(value,basestring):
			self._multisample1 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample2(self):
		'''Multisample 2'''
		return self._multisample2

	@multisample2.setter
	def multisample2(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample2 = value
		elif isinstance(value,basestring):
			self._multisample2 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample3(self):
		'''Multisample 3'''
		return self._multisample3

	@multisample3.setter
	def multisample3(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample3 = value
		elif isinstance(value,basestring):
			self._multisample3 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample4(self):
		'''Multisample 4'''
		return self._multisample4

	@multisample4.setter
	def multisample4(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample4 = value
		elif isinstance(value,basestring):
			self._multisample4 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample5(self):
		'''Multisample 5'''
		return self._multisample5

	@multisample5.setter
	def multisample5(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample5 = value
		elif isinstance(value,basestring):
			self._multisample5 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample6(self):
		'''Multisample 6'''
		return self._multisample6

	@multisample6.setter
	def multisample6(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample6 = value
		elif isinstance(value,basestring):
			self._multisample6 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample7(self):
		'''Multisample 7'''
		return self._multisample7

	@multisample7.setter
	def multisample7(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample7 = value
		elif isinstance(value,basestring):
			self._multisample7 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample8(self):
		'''Multisample 8'''
		return self._multisample8

	@multisample8.setter
	def multisample8(self,value):
		if isinstance(value,PCMOSCMultisampleParameters):
			self._multisample8 = value
		elif isinstance(value,basestring):
			self._multisample8 = PCMOSCMultisampleParameters(value)
		else:
			raise TypeError('value must of type PCMOSCMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control1(self):
		'''Multisample Velocity Control 1'''
		return self._multisample_velocity_control1

	@multisample_velocity_control1.setter
	def multisample_velocity_control1(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control1 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control1 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control2(self):
		'''Multisample Velocity Control 2'''
		return self._multisample_velocity_control2

	@multisample_velocity_control2.setter
	def multisample_velocity_control2(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control2 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control2 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control3(self):
		'''Multisample Velocity Control 3'''
		return self._multisample_velocity_control3

	@multisample_velocity_control3.setter
	def multisample_velocity_control3(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control3 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control3 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control4(self):
		'''Multisample Velocity Control 4'''
		return self._multisample_velocity_control4

	@multisample_velocity_control4.setter
	def multisample_velocity_control4(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control4 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control4 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control5(self):
		'''Multisample Velocity Control 5'''
		return self._multisample_velocity_control5

	@multisample_velocity_control5.setter
	def multisample_velocity_control5(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control5 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control5 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control6(self):
		'''Multisample Velocity Control 6'''
		return self._multisample_velocity_control6

	@multisample_velocity_control6.setter
	def multisample_velocity_control6(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control6 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control6 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def multisample_velocity_control7(self):
		'''Multisample Velocity Control 7'''
		return self._multisample_velocity_control7

	@multisample_velocity_control7.setter
	def multisample_velocity_control7(self,value):
		if isinstance(value,VelocityControlMultisampleParameters):
			self._multisample_velocity_control7 = value
		elif isinstance(value,basestring):
			self._multisample_velocity_control7 = VelocityControlMultisampleParameters(value)
		else:
			raise TypeError('value must of type VelocityControlMultisampleParameters')

#-------------------------------------------------------------------------------
	@property
	def note_on_ctrl_delay(self):
		'''Note-On Ctrl Delay'''
		return self._note_on_ctrl_delay

	@note_on_ctrl_delay.setter
	def note_on_ctrl_delay(self,value):
		if 0 <= value <= 97:
			self._note_on_ctrl_delay = value
		else:
			raise ValueError('not 0 <= value <= 97')

#-------------------------------------------------------------------------------
	@property
	def note_on_ctrl_mode(self):
		'''Note-On Ctrl Mode'''
		return self._note_on_ctrl_mode

	@note_on_ctrl_mode.setter
	def note_on_ctrl_mode(self,value):
		if 0 <= value <= 1:
			self._note_on_ctrl_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def velocity_zone_bottom(self):
		'''Velocity Zone Bottom'''
		return self._velocity_zone_bottom

	@velocity_zone_bottom.setter
	def velocity_zone_bottom(self,value):
		if 1 <= value <= 127:
			self._velocity_zone_bottom = value
		else:
			raise ValueError('not 1 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def velocity_zone_top(self):
		'''Velocity Zone Top'''
		return self._velocity_zone_top

	@velocity_zone_top.setter
	def velocity_zone_top(self,value):
		if 1 <= value <= 127:
			self._velocity_zone_top = value
		else:
			raise ValueError('not 1 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_lfo1(self):
		'''PCM OSC LFO Parameters 1'''
		return self._pcm_osc_lfo1

	@pcm_osc_lfo1.setter
	def pcm_osc_lfo1(self,value):
		if isinstance(value,PCMOSCLFOParameters):
			self._pcm_osc_lfo1 = value
		elif isinstance(value,basestring):
			self._pcm_osc_lfo1 = PCMOSCLFOParameters(value)
		else:
			raise TypeError('value must of type PCMOSCLFOParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_lfo2(self):
		'''PCM OSC LFO Parameters 1'''
		return self._pcm_osc_lfo2

	@pcm_osc_lfo2.setter
	def pcm_osc_lfo2(self,value):
		if isinstance(value,PCMOSCLFOParameters):
			self._pcm_osc_lfo2 = value
		elif isinstance(value,basestring):
			self._pcm_osc_lfo2 = PCMOSCLFOParameters(value)
		else:
			raise TypeError('value must of type PCMOSCLFOParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_pitch(self):
		'''PCM OSC LFO Parameters 1'''
		return self._pcm_osc_pitch

	@pcm_osc_pitch.setter
	def pcm_osc_pitch(self,value):
		if isinstance(value,PCMOSCPitchParameters):
			self._pcm_osc_pitch = value
		elif isinstance(value,basestring):
			self._pcm_osc_pitch = PCMOSCPitchParameters(value)
		else:
			raise TypeError('value must of type PCMOSCPitchParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_filter(self):
		'''PCM OSC Filter Parameters'''
		return self._pcm_osc_filter

	@pcm_osc_filter.setter
	def pcm_osc_filter(self,value):
		if isinstance(value,PCMOSCFilterParameters):
			self._pcm_osc_filter = value
		elif isinstance(value,basestring):
			self._pcm_osc_filter = PCMOSCFilterParameters(value)
		else:
			raise TypeError('value must of type PCMOSCFilterParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_filter_block1(self):
		'''PCM OSC Filter Block 1 Parameters'''
		return self._pcm_osc_filter_block1

	@pcm_osc_filter_block1.setter
	def pcm_osc_filter_block1(self,value):
		if isinstance(value,PCMOSCFilterBlockParameters):
			self._pcm_osc_filter_block1 = value
		elif isinstance(value,basestring):
			self._pcm_osc_filter_block1 = PCMOSCFilterBlockParameters(value)
		else:
			raise TypeError('value must of type PCMOSCFilterBlockParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_filter_block2(self):
		'''PCM OSC Filter Block 2 Parameters'''
		return self._pcm_osc_filter_block2

	@pcm_osc_filter_block2.setter
	def pcm_osc_filter_block2(self,value):
		if isinstance(value,PCMOSCFilterBlockParameters):
			self._pcm_osc_filter_block2 = value
		elif isinstance(value,basestring):
			self._pcm_osc_filter_block2 = PCMOSCFilterBlockParameters(value)
		else:
			raise TypeError('value must of type PCMOSCFilterBlockParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_filter_eg(self):
		'''PCM OSC Filter EG Parameters'''
		return self._pcm_osc_filter_eg

	@pcm_osc_filter_eg.setter
	def pcm_osc_filter_eg(self,value):
		if isinstance(value,PCMOSCFilterEGParameters):
			self._pcm_osc_filter_eg = value
		elif isinstance(value,basestring):
			self._pcm_osc_filter_eg = PCMOSCFilterEGParameters(value)
		else:
			raise TypeError('value must of type PCMOSCFilterEGParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_filter_key_track(self):
		'''PCM OSC Filter Key Track Parameters'''
		return self._pcm_osc_filter_key_track

	@pcm_osc_filter_key_track.setter
	def pcm_osc_filter_key_track(self,value):
		if isinstance(value,PCMOSCFilterKeyTrackParameters):
			self._pcm_osc_filter_key_track = value
		elif isinstance(value,basestring):
			self._pcm_osc_filter_key_track = PCMOSCFilterKeyTrackParameters(value)
		else:
			raise TypeError('value must of type PCMOSCFilterKeyTrackParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_drive(self):
		'''PCM OSC Drive Parameters'''
		return self._pcm_osc_drive

	@pcm_osc_drive.setter
	def pcm_osc_drive(self,value):
		if isinstance(value,PCMOSCDriveParameters):
			self._pcm_osc_drive = value
		elif isinstance(value,basestring):
			self._pcm_osc_drive = PCMOSCDriveParameters(value)
		else:
			raise TypeError('value must of type PCMOSCDriveParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_amp(self):
		'''PCM OSC Amp Parameters'''
		return self._pcm_osc_amp

	@pcm_osc_amp.setter
	def pcm_osc_amp(self,value):
		if isinstance(value,PCMOSCAmpParameters):
			self._pcm_osc_amp = value
		elif isinstance(value,basestring):
			self._pcm_osc_amp = PCMOSCAmpParameters(value)
		else:
			raise TypeError('value must of type PCMOSCAmpParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_amp_eg(self):
		'''PCM OSC Amp EG Parameters'''
		return self._pcm_osc_amp_eg

	@pcm_osc_amp_eg.setter
	def pcm_osc_amp_eg(self,value):
		if isinstance(value,PCMOSCAmpEGParameters):
			self._pcm_osc_amp_eg = value
		elif isinstance(value,basestring):
			self._pcm_osc_amp_eg = PCMOSCAmpEGParameters(value)
		else:
			raise TypeError('value must of type PCMOSCAmpEGParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_amp_key_track(self):
		'''PCM OSC Amp Key Track Parameters'''
		return self._pcm_osc_amp_key_track

	@pcm_osc_amp_key_track.setter
	def pcm_osc_amp_key_track(self,value):
		if isinstance(value,PCMOSCAmpKeyTrackParameters):
			self._pcm_osc_amp_key_track = value
		elif isinstance(value,basestring):
			self._pcm_osc_amp_key_track = PCMOSCAmpKeyTrackParameters(value)
		else:
			raise TypeError('value must of type PCMOSCAmpKeyTrackParameters')

#-------------------------------------------------------------------------------
	@property
	def pcm_osc_output(self):
		'''PCM OSC Output Parameters'''
		return self._pcm_osc_output

	@pcm_osc_output.setter
	def pcm_osc_output(self,value):
		if isinstance(value,PCMOSCOutputParameters):
			self._pcm_osc_output = value
		elif isinstance(value,basestring):
			self._pcm_osc_output = PCMOSCOutputParameters(value)
		else:
			raise TypeError('value must of type PCMOSCOutputParameters')

#-------------------------------------------------------------------------------
	@property
	def ams_mixer1(self):
		'''AMS Mixer 1 Parameters'''
		return self._ams_mixer1

	@ams_mixer1.setter
	def ams_mixer1(self,value):
		if isinstance(value,AMSMixerParameters):
			self._ams_mixer1 = value
		elif isinstance(value,basestring):
			self._ams_mixer1 = AMSMixerParameters(value)
		else:
			raise TypeError('value must of type AMSMixerParameters')

#-------------------------------------------------------------------------------
	@property
	def ams_mixer2(self):
		'''AMS Mixer 2 Parameters'''
		return self._ams_mixer2

	@ams_mixer2.setter
	def ams_mixer2(self,value):
		if isinstance(value,AMSMixerParameters):
			self._ams_mixer2 = value
		elif isinstance(value,basestring):
			self._ams_mixer2 = AMSMixerParameters(value)
		else:
			raise TypeError('value must of type AMSMixerParameters')

#===============================================================================
# Table 1 1004-1011 x 8
class PCMOSCMultisampleParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<2BH2BH',raw)) # 8 
		self.type = v.pop(0)
		self.bank = v.pop(0)
		self.number = v.pop(0)
		self.level = v.pop(0)
		byte5 = v.pop(0)
		self.offset = byte5 & 0x0F
		self.direction = (byte5 >> 7) & 0x01
		self._reserved1010 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.type)
		v.append(self.bank)
		v.append(self.number)
		v.append(self.level)
		byte5 = 0
		byte5 |= self.offset & 0x0F
		byte5 = (self.direction & 0x01) << 7
		v.append(byte5)
		v.append(self._reserved1010)
		return struct.pack('>2BH2BH',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 1:
			self._type = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def bank(self):
		'''Bank'''
		return self._bank

	@bank.setter
	def bank(self,value):
		if 0 <= value <= 3:
			self._bank = value
		else:
			raise ValueError('not 0 <= value <= 3')

#-------------------------------------------------------------------------------
	@property
	def number(self):
		'''Number'''
		return self._number

	@number.setter
	def number(self,value):
#FIXME		if 0 <= value <= 1999:
		self._number = value
#		else:
#			raise ValueError('not 0 <= value <= 1999')

#-------------------------------------------------------------------------------
	@property
	def level(self):
		'''Level'''
		return self._level

	@level.setter
	def level(self,value):
		if 0 <= value <= 127:
			self._level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def offset(self):
		'''Offset'''
		return self._offset

	@offset.setter
	def offset(self,value):
		if 0 <= value <= 8:
			self._offset = value
		else:
			raise ValueError('not 0 <= value <= 8')

#-------------------------------------------------------------------------------
	@property
	def direction(self):
		'''Direction Forward/Reverse'''
		return self._direction

	@direction.setter
	def direction(self,value):
		if 0 <= value <= 1:
			self._direction = value
		else:
			raise ValueError('not 0 <= value <= 1')

#===============================================================================
# Table 1 1068-1071 x 2
class VelocityControlMultisampleParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<4B',raw)) # 4 
		self.bottom_velocity = v.pop(0)
		self.xfade_range = v.pop(0)
		self.xfade_curve = v.pop(0)
		self._reserved1071 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.bottom_velocity)
		v.append(self.xfade_range)
		v.append(self.xfade_curve)
		v.append(self._reserved1071)
		return struct.pack('>4B',*v) # 4

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def xfade_range(self):
		'''XFade Range'''
		return self._xfade_range

	@xfade_range.setter
	def xfade_range(self,value):
		if 0 <= value <= 127:
			self._xfade_range = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def xfade_curve(self):
		'''XFade Curve'''
		return self._xfade_curve

	@xfade_curve.setter
	def xfade_curve(self,value):
		if 0 <= value <= 2:
			self._xfade_curve = value
		else:
			raise ValueError('not 0 <= value <= 2')

#===============================================================================
# Table 1 1100-1127 x 2
class PCMOSCLFOParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<3B3b3BbH2BHBbHBbHBbH',raw)) # 28 
		byte0 = v.pop(0)
		self.waveform = byte0 & 0x3F
		self.stop = (byte0 >> 6) & 0x01
		self.key_sync_f = (byte0 >> 7) & 0x01
		self.frequency = v.pop(0)
		self._reserved1102 = v.pop(0)
		self.start_phase = v.pop(0)
		self.shape = v.pop(0)
		self.offset = v.pop(0)
		self.delay = v.pop(0)
		self.fade = v.pop(0)
		self.source = v.pop(0)
		self.intensity = v.pop(0)
		self._reserved1110 = v.pop(0)
		byte12 = v.pop(0)
		self.midi_sync_base_note = byte12 & 0x0F
		self.midi_tempo_sync = (byte12 >> 7) & 0x01
		self.midi_sync_time = v.pop(0)
		self._reserved1114 = v.pop(0)
		self.ams1_source = v.pop(0)
		self.ams1_intensity = v.pop(0)
		self._reserved1118 = v.pop(0)
		self.int_mod_ams_source = v.pop(0)
		self.int_mod_ams_intensity = v.pop(0)
		self._reserved1121 = v.pop(0)
		self.ams2_source = v.pop(0)
		self.ams2_intensity = v.pop(0)
		self._reserved1126 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		byte0 = 0
		byte0 |= self.waveform & 0x3F
		byte0 |= (self.stop & 0x01) << 6
		byte0 |= (self.key_sync_f & 0x01) << 7
		v.append(byte0)
		v.append(self.frequency)
		v.append(self._reserved1102)
		v.append(self.start_phase)
		v.append(self.shape)
		v.append(self.offset)
		v.append(self.delay)
		v.append(self.fade)
		v.append(self.source)
		v.append(self.intensity)
		v.append(self._reserved1110)
		byte12 = 0
		byte12 |= self.midi_sync_base_note & 0x0F
		byte12 |= (self.midi_tempo_sync & 0x01) << 7
		v.append(byte12)
		v.append(self.midi_sync_time)
		v.append(self._reserved1114)
		v.append(self.ams1_source)
		v.append(self.ams1_intensity)
		v.append(self._reserved1118)
		v.append(self.int_mod_ams_source)
		v.append(self.int_mod_ams_intensity)
		v.append(self._reserved1121)
		v.append(self.ams2_source)
		v.append(self.ams2_intensity)
		v.append(self._reserved1126)
		return struct.pack('>3B3b3BbH2BHBbHBbHBbH',*v) # 28

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def waveform(self):
		'''Waveform'''
		return self._waveform

	@waveform.setter
	def waveform(self,value):
#FIXME		if 0 <= value <= 15:
		self._waveform = value
#		else:
#			raise ValueError('not 0 <= value <= 15')

#-------------------------------------------------------------------------------
	@property
	def stop(self):
		'''Stop'''
		return self._stop

	@stop.setter
	def stop(self,value):
		if 0 <= value <= 1:
			self._stop = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def key_sync_f(self):
		'''Key Sync.f'''
		return self._key_sync_f

	@key_sync_f.setter
	def key_sync_f(self,value):
		if 0 <= value <= 1:
			self._key_sync_f = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def frequency(self):
		'''Frequency'''
		return self._frequency

	@frequency.setter
	def frequency(self,value):
		if 0 <= value <= 99:
			self._frequency = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def start_phase(self):
		'''Start Phase'''
		return self._start_phase

	@start_phase.setter
	def start_phase(self,value):
		if -36 <= value <= 37:
			self._start_phase = value
		else:
			raise ValueError('not -36 <= value <= 37')

#-------------------------------------------------------------------------------
	@property
	def shape(self):
		'''Shape'''
		return self._shape

	@shape.setter
	def shape(self,value):
		if -99 <= value <= 99:
			self._shape = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def offset(self):
		'''Offset'''
		return self._offset

	@offset.setter
	def offset(self,value):
		if -99 <= value <= 99:
			self._offset = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def delay(self):
		'''Delay'''
		return self._delay

	@delay.setter
	def delay(self,value):
		if 0 <= value <= 99:
			self._delay = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def fade(self):
		'''Fade'''
		return self._fade

	@fade.setter
	def fade(self,value):
		if 0 <= value <= 99:
			self._fade = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def source(self):
		'''Source'''
		return self._source

	@source.setter
	def source(self,value):
		if 0 <= value <= 50:
			self._source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def intensity(self):
		'''Intensity'''
		return self._intensity

	@intensity.setter
	def intensity(self,value):
		if -99 <= value <= 99:
			self._intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_base_note(self):
		'''MIDISync Base Note'''
		return self._midi_sync_base_note

	@midi_sync_base_note.setter
	def midi_sync_base_note(self,value):
		if 0 <= value <= 9:
			self._midi_sync_base_note = value
		else:
			raise ValueError('not 0 <= value <= 9')

#-------------------------------------------------------------------------------
	@property
	def midi_tempo_sync(self):
		'''MIDI/Tempo Sync'''
		return self._midi_tempo_sync

	@midi_tempo_sync.setter
	def midi_tempo_sync(self,value):
		if 0 <= value <= 1:
			self._midi_tempo_sync = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def midi_sync_time(self):
		'''MIDISync Time'''
		return self._midi_sync_time

	@midi_sync_time.setter
	def midi_sync_time(self,value):
		if 0 <= value <= 31:
			self._midi_sync_time = value
		else:
			raise ValueError('not 0 <= value <= 31')

#-------------------------------------------------------------------------------
	@property
	def ams1_source(self):
		'''AMS1 Source'''
		return self._ams1_source

	@ams1_source.setter
	def ams1_source(self,value):
		if 0 <= value <= 50:
			self._ams1_source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams1_intensity(self):
		'''AMS1 Intensity'''
		return self._ams1_intensity

	@ams1_intensity.setter
	def ams1_intensity(self,value):
		if -99 <= value <= 99:
			self._ams1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def int_mod_ams_source(self):
		'''IntModAMS Source'''
		return self._int_mod_ams_source

	@int_mod_ams_source.setter
	def int_mod_ams_source(self,value):
		if 0 <= value <= 50:
			self._int_mod_ams_source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def int_mod_ams_intensity(self):
		'''IntModAMS Intensity'''
		return self._int_mod_ams_intensity

	@int_mod_ams_intensity.setter
	def int_mod_ams_intensity(self,value):
		if -99 <= value <= 99:
			self._int_mod_ams_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams2_source(self):
		'''AMS2 Source'''
		return self._ams2_source

	@ams2_source.setter
	def ams2_source(self,value):
		if 0 <= value <= 50:
			self._ams2_source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams2_intensity(self):
		'''AMS2 Intensity'''
		return self._ams2_intensity

	@ams2_intensity.setter
	def ams2_intensity(self,value):
		if -99 <= value <= 99:
			self._ams2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
# Table 1 1156-1187
class PCMOSCPitchParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<BbHBbHBbHBbH2Bh4b2B6b',raw)) # 32 
		self.ams_src_to_pitch = v.pop(0)
		self.ams_int_to_pitch = v.pop(0)
		self._reserved1158 = v.pop(0)
		self.ams_src_to_pitch_eg = v.pop(0)
		self.ams_int_to_pitch_eg = v.pop(0)
		self._reserved1162 = v.pop(0)
		self.ams_src_to_lfo1 = v.pop(0)
		self.ams_int_to_lfo1 = v.pop(0)
		self._reserved1166 = v.pop(0)
		self.ams_src_to_lfo2 = v.pop(0)
		self.ams_int_to_lfo2 = v.pop(0)
		self._reserved1170 = v.pop(0)
		byte16 = v.pop(0)
		if byte16 < 128:
			self.octave = byte16
		else:
			self.octave = byte16 - 256
		byte17 = v.pop(0)
		if byte17 < 128:
			self.transpose = byte17
		else:
			self.transpose = byte17	- 256
		self.tune = v.pop(0)
		self.note_track_to_pitch = v.pop(0)
		self.pitch_eg_intensity = v.pop(0)
		self.lfo1_intensity = v.pop(0)
		self.lfo2_intensity = v.pop(0)
		byte24 = v.pop(0)
		self.portamento_enable = byte24 & 0x01
		self.portamento_fingered = (byte24 >> 1) & 0x01
		self.portamento_mode = (byte24 >> 2) & 0x01
		self.portamento_time = v.pop(0)
		self.jsx_plus_to_pitch = v.pop(0)
		self.jsx_minus_to_pitch = v.pop(0)
		self.ribbon_x_to_pitch = v.pop(0)
		self.offset_in_hz = v.pop(0)
		self.lfo1_jsy_plus_intensity = v.pop(0)
		self.lfo2_jsy_plus_intensity = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.ams_src_to_pitch)
		v.append(self.ams_int_to_pitch)
		v.append(self._reserved1158)
		v.append(self.ams_src_to_pitch_eg)
		v.append(self.ams_int_to_pitch_eg)
		v.append(self._reserved1162)
		v.append(self.ams_src_to_lfo1)
		v.append(self.ams_int_to_lfo1)
		v.append(self._reserved1166)
		v.append(self.ams_src_to_lfo2)
		v.append(self.ams_int_to_lfo2)
		v.append(self._reserved1170)
		v.append(self.octave)
		v.append(self.transpose)
		v.append(self.tune)
		v.append(self.note_track_to_pitch)
		v.append(self.pitch_eg_intensity)
		v.append(self.lfo1_intensity)
		v.append(self.lfo2_intensity)
		byte24 = 0
		byte24 |= self.portamento_enable & 0x01
		byte24 |= (self.portamento_fingered & 0x01) << 1
		byte24 |= (self.portamento_mode & 0x01) << 2
		v.append(byte24)
		v.append(self.portamento_time)
		v.append(self.jsx_plus_to_pitch)
		v.append(self.jsx_minus_to_pitch)
		v.append(self.ribbon_x_to_pitch)
		v.append(self.offset_in_hz)
		v.append(self.lfo1_jsy_plus_intensity)
		v.append(self.lfo2_jsy_plus_intensity)
		return struct.pack('>BbHBbHBbHBbH2Bh4b2B6b',*v) # 32

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_pitch(self):
		'''AMSSrc To Pitch'''
		return self._ams_src_to_pitch

	@ams_src_to_pitch.setter
	def ams_src_to_pitch(self,value):
		if 0 <= value <= 50:
			self._ams_src_to_pitch = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_pitch(self):
		'''AMSInt To Pitch'''
		return self._ams_int_to_pitch

	@ams_int_to_pitch.setter
	def ams_int_to_pitch(self,value):
		if -115 <= value <= 115:
			self._ams_int_to_pitch = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_pitch_eg(self):
		'''AMSSrc To Pitch EG'''
		return self._ams_src_to_pitch_eg

	@ams_src_to_pitch_eg.setter
	def ams_src_to_pitch_eg(self,value):
		if 0 <= value <= 50:
			self._ams_src_to_pitch_eg = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_pitch_eg(self):
		'''AMSInt To Pitch EG'''
		return self._ams_int_to_pitch_eg

	@ams_int_to_pitch_eg.setter
	def ams_int_to_pitch_eg(self,value):
		if -115 <= value <= 115:
			self._ams_int_to_pitch_eg = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_lfo1(self):
		'''AMSSrc To LFO1'''
		return self._ams_src_to_lfo1

	@ams_src_to_lfo1.setter
	def ams_src_to_lfo1(self,value):
		if 0 <= value <= 50:
			self._ams_src_to_lfo1 = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_lfo1(self):
		'''AMSInt To LFO1'''
		return self._ams_int_to_lfo1

	@ams_int_to_lfo1.setter
	def ams_int_to_lfo1(self,value):
		if -115 <= value <= 115:
			self._ams_int_to_lfo1 = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_lfo2(self):
		'''AMSSrc To LFO2'''
		return self._ams_src_to_lfo2

	@ams_src_to_lfo2.setter
	def ams_src_to_lfo2(self,value):
		if 0 <= value <= 50:
			self._ams_src_to_lfo2 = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_lfo2(self):
		'''AMSInt To LFO2'''
		return self._ams_int_to_lfo2

	@ams_int_to_lfo2.setter
	def ams_int_to_lfo2(self,value):
		if -115 <= value <= 115:
			self._ams_int_to_lfo2 = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def octave(self):
		'''Octave'''
		return self._octave

	@octave.setter
	def octave(self,value):
		if -2 <= value <= 1:
			self._octave = value
		else:
			raise ValueError('%s not -2 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -12 <= value <= 12:
			self._transpose = value
		else:
			raise ValueError('%s not -12 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def tune(self):
		'''Tune'''
		return self._tune

	@tune.setter
	def tune(self,value):
		if -1200 <= value <= 1200:
			self._tune = value
		else:
			raise ValueError('not -1200 <= value <= 1200')

#-------------------------------------------------------------------------------
	@property
	def note_track_to_pitch(self):
		'''Note Track To Pitch'''
		return self._note_track_to_pitch

	@note_track_to_pitch.setter
	def note_track_to_pitch(self,value):
		if -10 <= value <= 20:
			self._note_track_to_pitch = value
		else:
			raise ValueError('not -10 <= value <= 20')

#-------------------------------------------------------------------------------
	@property
	def pitch_eg_intensity(self):
		'''Pitch EG Intensity'''
		return self._pitch_eg_intensity

	@pitch_eg_intensity.setter
	def pitch_eg_intensity(self,value):
		if -115 <= value <= 115:
			self._pitch_eg_intensity = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def lfo1_intensity(self):
		'''LFO1 Intensity'''
		return self._lfo1_intensity

	@lfo1_intensity.setter
	def lfo1_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo1_intensity = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def lfo2_intensity(self):
		'''LFO2 Intensity'''
		return self._lfo2_intensity

	@lfo2_intensity.setter
	def lfo2_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo2_intensity = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def portamento_enable(self):
		'''Portamento Enable'''
		return self._portamento_enable

	@portamento_enable.setter
	def portamento_enable(self,value):
		if 0 <= value <= 1:
			self._portamento_enable = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def portamento_fingered(self):
		'''Portamento Fingered'''
		return self._portamento_fingered

	@portamento_fingered.setter
	def portamento_fingered(self,value):
		if 0 <= value <= 1:
			self._portamento_fingered = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def portamento_mode(self):
		'''Portamento Mode'''
		return self._portamento_mode

	@portamento_mode.setter
	def portamento_mode(self,value):
		if 0 <= value <= 1:
			self._portamento_mode = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if 0 <= value <= 127:
			self._portamento_time = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def jsx_plus_to_pitch(self):
		'''JSXPlus To Pitch'''
		return self._jsx_plus_to_pitch

	@jsx_plus_to_pitch.setter
	def jsx_plus_to_pitch(self,value):
		if -60 <= value <= 12:
			self._jsx_plus_to_pitch = value
		else:
			raise ValueError('not -60 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def jsx_minus_to_pitch(self):
		'''JSXMinus To Pitch'''
		return self._jsx_minus_to_pitch

	@jsx_minus_to_pitch.setter
	def jsx_minus_to_pitch(self,value):
		if -60 <= value <= 12:
			self._jsx_minus_to_pitch = value
		else:
			raise ValueError('not -60 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def ribbon_x_to_pitch(self):
		'''RibbonX To Pitch'''
		return self._ribbon_x_to_pitch

	@ribbon_x_to_pitch.setter
	def ribbon_x_to_pitch(self,value):
		if -12 <= value <= 12:
			self._ribbon_x_to_pitch = value
		else:
			raise ValueError('not -12 <= value <= 12')

#-------------------------------------------------------------------------------
	@property
	def offset_in_hz(self):
		'''Offset IN Hz'''
		return self._offset_in_hz

	@offset_in_hz.setter
	def offset_in_hz(self,value):
		if -100 <= value <= 100:
			self._offset_in_hz = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def lfo1_jsy_plus_intensity(self):
		'''LFO1 JSY+ Intensity'''
		return self._lfo1_jsy_plus_intensity

	@lfo1_jsy_plus_intensity.setter
	def lfo1_jsy_plus_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo1_jsy_plus_intensity = value
		else:
			raise ValueError('not -115 <= value <= 115')

#-------------------------------------------------------------------------------
	@property
	def lfo2_jsy_plus_intensity(self):
		'''LFO2 JSY+ Intensity'''
		return self._lfo2_jsy_plus_intensity

	@lfo2_jsy_plus_intensity.setter
	def lfo2_jsy_plus_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo2_jsy_plus_intensity = value
		else:
			raise ValueError('not -115 <= value <= 115')

#===============================================================================
# Table 1 1188-1195
class PCMOSCFilterParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<B4bBH',raw)) # 8 
		self.routing = v.pop(0)
		self.filter_eg_intensity = v.pop(0)
		self.lfo1_intensity = v.pop(0)
		self.lfo1_intensity = v.pop(0)
		self.common_lfo_intensity = v.pop(0)
		self._reserved1193 = v.pop(0)
		self._reserved1194 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.routing)
		v.append(self.filter_eg_intensity)
		v.append(self.lfo1_intensity)
		v.append(self.lfo1_intensity)
		v.append(self.common_lfo_intensity)
		v.append(self._reserved1193)
		v.append(self._reserved1194)
		return struct.pack('>B4bBH',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 3:
			self._routing = value
		else:
			raise ValueError('not 0 <= value <= 3')

#-------------------------------------------------------------------------------
	@property
	def filter_eg_intensity(self):
		'''Filter EG Intensity'''
		return self._filter_eg_intensity

	@filter_eg_intensity.setter
	def filter_eg_intensity(self,value):
		if -100 <= value <= 100:
			self._filter_eg_intensity = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def lfo1_intensity(self):
		'''LFO1 Intensity'''
		return self._lfo1_intensity

	@lfo1_intensity.setter
	def lfo1_intensity(self,value):
		if -100 <= value <= 100:
			self._lfo1_intensity = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def lfo1_intensity(self):
		'''LFO2 Intensity'''
		return self._lfo1_intensity

	@lfo1_intensity.setter
	def lfo1_intensity(self,value):
		if -100 <= value <= 100:
			self._lfo1_intensity = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def common_lfo_intensity(self):
		'''Common LFO Intensity'''
		return self._common_lfo_intensity

	@common_lfo_intensity.setter
	def common_lfo_intensity(self,value):
		if -100 <= value <= 100:
			self._common_lfo_intensity = value
		else:
			raise ValueError('not -100 <= value <= 100')

#===============================================================================
# Table 1 1196-1231 x 2
class PCMOSCFilterBlockParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<BbHBbHBbHBbH5B13bH',raw)) # 36 
		self.output_level_source = v.pop(0)
		self.output_level_intensity = v.pop(0)
		self._reserved1198 = v.pop(0)
		self.resonance_source = v.pop(0)
		self.resonance_intensity = v.pop(0)
		self._reserved1202 = v.pop(0)
		self.cutoff_1_source = v.pop(0)
		self.cutoff_1_intensity = v.pop(0)
		self._reserved1206 = v.pop(0)
		self.cut_off_2_source = v.pop(0)
		self.cut_off_2_intensity = v.pop(0)
		self._reserved1210 = v.pop(0)
		byte16 = v.pop(0)
		self.type_ = byte16 & 0x07
		self.bypass = (byte16 >> 7) & 0x01
		self.trim = v.pop(0)
		self.output_level = v.pop(0)
		self.cut_off = v.pop(0)
		self.resonance = v.pop(0)
		self.key_track_to_cut_off = v.pop(0)
		self.filter_eg_intensity = v.pop(0)
		self.velocity_to_filter_eg_int = v.pop(0)
		self.ams_int_to_filter_eg_int = v.pop(0)
		self.lfo1_intensity = v.pop(0)
		self.lfo2_intensity = v.pop(0)
		self.common_lfo_intensity = v.pop(0)
		self.jsy_lfo1_intensity = v.pop(0)
		self.jsy_lfo2_intensity = v.pop(0)
		self.jsy_common_lfo_intensity = v.pop(0)
		self.ams_int_lfo1_intensity = v.pop(0)
		self.ams_int_lfo2_intensity = v.pop(0)
		self.ams_int_lfo_intensity = v.pop(0)
		self._reserved1230 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.output_level_source)
		v.append(self.output_level_intensity)
		v.append(self._reserved1198)
		v.append(self.resonance_source)
		v.append(self.resonance_intensity)
		v.append(self._reserved1202)
		v.append(self.cutoff_1_source)
		v.append(self.cutoff_1_intensity)
		v.append(self._reserved1206)
		v.append(self.cut_off_2_source)
		v.append(self.cut_off_2_intensity)
		v.append(self._reserved1210)
		byte16 = 0
		byte16 |= self.type_ & 0x07
		byte16 |= (self.bypass & 0x01) << 7
		v.append(byte16)
		v.append(self.trim)
		v.append(self.output_level)
		v.append(self.cut_off)
		v.append(self.resonance)
		v.append(self.key_track_to_cut_off)
		v.append(self.filter_eg_intensity)
		v.append(self.velocity_to_filter_eg_int)
		v.append(self.ams_int_to_filter_eg_int)
		v.append(self.lfo1_intensity)
		v.append(self.lfo2_intensity)
		v.append(self.common_lfo_intensity)
		v.append(self.jsy_lfo1_intensity)
		v.append(self.jsy_lfo2_intensity)
		v.append(self.jsy_common_lfo_intensity)
		v.append(self.ams_int_lfo1_intensity)
		v.append(self.ams_int_lfo2_intensity)
		v.append(self.ams_int_lfo_intensity)
		v.append(self._reserved1230)
		return struct.pack('>BbHBbHBbHBbH5B13bH',*v) # 36

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def output_level_source(self):
		'''Output Level Source'''
		return self._output_level_source

	@output_level_source.setter
	def output_level_source(self,value):
		if 0 <= value <= 99:
			self._output_level_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def output_level_intensity(self):
		'''Output Level Intensity'''
		return self._output_level_intensity

	@output_level_intensity.setter
	def output_level_intensity(self,value):
		if -99 <= value <= 99:
			self._output_level_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def resonance_source(self):
		'''Resonance Source'''
		return self._resonance_source

	@resonance_source.setter
	def resonance_source(self,value):
		if 0 <= value <= 99:
			self._resonance_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def resonance_intensity(self):
		'''Resonance Intensity'''
		return self._resonance_intensity

	@resonance_intensity.setter
	def resonance_intensity(self,value):
		if -99 <= value <= 99:
			self._resonance_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def cutoff_1_source(self):
		'''CutOff 1 Source'''
		return self._cutoff_1_source

	@cutoff_1_source.setter
	def cutoff_1_source(self,value):
		if 0 <= value <= 99:
			self._cutoff_1_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def cutoff_1_intensity(self):
		'''CutOff 1 Intensity'''
		return self._cutoff_1_intensity

	@cutoff_1_intensity.setter
	def cutoff_1_intensity(self,value):
		if -99 <= value <= 99:
			self._cutoff_1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def cut_off_2_source(self):
		'''CutOff 2 Source'''
		return self._cut_off_2_source

	@cut_off_2_source.setter
	def cut_off_2_source(self,value):
		if 0 <= value <= 99:
			self._cut_off_2_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def cut_off_2_intensity(self):
		'''CutOff 2 Intensity'''
		return self._cut_off_2_intensity

	@cut_off_2_intensity.setter
	def cut_off_2_intensity(self,value):
		if -99 <= value <= 99:
			self._cut_off_2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def type_(self):
		'''Type'''
		return self._type_

	@type_.setter
	def type_(self,value):
		if 0 <= value <= 3:
			self._type_ = value
		else:
			raise ValueError('not 0 <= value <= 3')

#-------------------------------------------------------------------------------
	@property
	def bypass(self):
		'''Bypass'''
		return self._bypass

	@bypass.setter
	def bypass(self,value):
		if 0 <= value <= 1:
			self._bypass = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def trim(self):
		'''Trim'''
		return self._trim

	@trim.setter
	def trim(self,value):
		if 0 <= value <= 99:
			self._trim = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 99:
			self._output_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def cut_off(self):
		'''CutOff'''
		return self._cut_off

	@cut_off.setter
	def cut_off(self,value):
		if 0 <= value <= 99:
			self._cut_off = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if 0 <= value <= 99:
			self._resonance = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def key_track_to_cut_off(self):
		'''Key Track To CutOff'''
		return self._key_track_to_cut_off

	@key_track_to_cut_off.setter
	def key_track_to_cut_off(self,value):
		if -99 <= value <= 99:
			self._key_track_to_cut_off = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def filter_eg_intensity(self):
		'''Filter EG Intensity'''
		return self._filter_eg_intensity

	@filter_eg_intensity.setter
	def filter_eg_intensity(self,value):
		if -99 <= value <= 99:
			self._filter_eg_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def velocity_to_filter_eg_int(self):
		'''VelToFilterEG Int.'''
		return self._velocity_to_filter_eg_int

	@velocity_to_filter_eg_int.setter
	def velocity_to_filter_eg_int(self,value):
		if -99 <= value <= 99:
			self._velocity_to_filter_eg_int = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_filter_eg_int(self):
		'''AMS Int To Filter EG Int.'''
		return self._ams_int_to_filter_eg_int

	@ams_int_to_filter_eg_int.setter
	def ams_int_to_filter_eg_int(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_filter_eg_int = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def lfo1_intensity(self):
		'''LFO1 Intensity'''
		return self._lfo1_intensity

	@lfo1_intensity.setter
	def lfo1_intensity(self,value):
		if -99 <= value <= 99:
			self._lfo1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def lfo2_intensity(self):
		'''LFO2 Intensity'''
		return self._lfo2_intensity

	@lfo2_intensity.setter
	def lfo2_intensity(self,value):
		if -99 <= value <= 99:
			self._lfo2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def common_lfo_intensity(self):
		'''Common LFO Intensity'''
		return self._common_lfo_intensity

	@common_lfo_intensity.setter
	def common_lfo_intensity(self,value):
		if -99 <= value <= 99:
			self._common_lfo_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def jsy_lfo1_intensity(self):
		'''JS-Y LFO1 Intensity'''
		return self._jsy_lfo1_intensity

	@jsy_lfo1_intensity.setter
	def jsy_lfo1_intensity(self,value):
		if -99 <= value <= 99:
			self._jsy_lfo1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def jsy_lfo2_intensity(self):
		'''JS-Y LFO2 Intensity'''
		return self._jsy_lfo2_intensity

	@jsy_lfo2_intensity.setter
	def jsy_lfo2_intensity(self,value):
		if -99 <= value <= 99:
			self._jsy_lfo2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def jsy_common_lfo_intensity(self):
		'''JS-Y Common LFO Int.'''
		return self._jsy_common_lfo_intensity

	@jsy_common_lfo_intensity.setter
	def jsy_common_lfo_intensity(self,value):
		if -99 <= value <= 99:
			self._jsy_common_lfo_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_lfo1_intensity(self):
		'''AMSInt LFO1 Intensity'''
		return self._ams_int_lfo1_intensity

	@ams_int_lfo1_intensity.setter
	def ams_int_lfo1_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_int_lfo1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_lfo2_intensity(self):
		'''AMSInt LFO2 Intensity'''
		return self._ams_int_lfo2_intensity

	@ams_int_lfo2_intensity.setter
	def ams_int_lfo2_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_int_lfo2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_lfo_intensity(self):
		'''AMSInt CommonLFO Int.'''
		return self._ams_int_lfo_intensity

	@ams_int_lfo_intensity.setter
	def ams_int_lfo_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_int_lfo_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
# Table 1 1268-1303
class PCMOSCFilterEGParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<4BbBbBbBbBbB3bB4bB4bB4bBbH',raw)) # 36 
		self.attack_eg_curve = v.pop(0)
		self.decay_eg_curve = v.pop(0)
		self.slope_eg_curve = v.pop(0)
		self.release_eg_curve = v.pop(0)
		self.start_level = v.pop(0)
		self.attack_time = v.pop(0)
		self.attack_level = v.pop(0)
		self.decay_time = v.pop(0)
		self.break_level = v.pop(0)
		self.slope_time = v.pop(0)
		self.sustain_level = v.pop(0)
		self.release_time = v.pop(0)
		self.release_level = v.pop(0)
		self.ams_src_to_level = v.pop(0)
		self.ams_int_to_start_level = v.pop(0)
		self.ams_int_to_attack_level = v.pop(0)
		self.ams_int_to_break_level = v.pop(0)
		self.ams_src_to_time1 = v.pop(0)
		self.ams_int_to_attack_time1 = v.pop(0)
		self.ams_int_to_decay_time1 = v.pop(0)
		self.ams_int_to_slope_time1 = v.pop(0)
		self.ams_int_to_release_time1 = v.pop(0)
		self.ams_src_to_time2 = v.pop(0)
		self.ams_int_to_attack_time2 = v.pop(0)
		self.ams_int_to_decay_time2 = v.pop(0)
		self.ams_int_to_slope_time2 = v.pop(0)
		self.ams_int_to_release_time2 = v.pop(0)
		self.ams_src_to_time3 = v.pop(0)
		self.ams_int_to_attack_time3 = v.pop(0)
		self.ams_int_to_decay_time3 = v.pop(0)
		self.ams_int_to_slope_time3 = v.pop(0)
		self.ams_int_to_release_time3 = v.pop(0)
		self.reset_source = v.pop(0)
		self.reset_threshold = v.pop(0)
		self._reserved1302 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.attack_eg_curve)
		v.append(self.decay_eg_curve)
		v.append(self.slope_eg_curve)
		v.append(self.release_eg_curve)
		v.append(self.start_level)
		v.append(self.attack_time)
		v.append(self.attack_level)
		v.append(self.decay_time)
		v.append(self.break_level)
		v.append(self.slope_time)
		v.append(self.sustain_level)
		v.append(self.release_time)
		v.append(self.release_level)
		v.append(self.ams_src_to_level)
		v.append(self.ams_int_to_start_level)
		v.append(self.ams_int_to_attack_level)
		v.append(self.ams_int_to_break_level)
		v.append(self.ams_src_to_time1)
		v.append(self.ams_int_to_attack_time1)
		v.append(self.ams_int_to_decay_time1)
		v.append(self.ams_int_to_slope_time1)
		v.append(self.ams_int_to_release_time1)
		v.append(self.ams_src_to_time2)
		v.append(self.ams_int_to_attack_time2)
		v.append(self.ams_int_to_decay_time2)
		v.append(self.ams_int_to_slope_time2)
		v.append(self.ams_int_to_release_time2)
		v.append(self.ams_src_to_time3)
		v.append(self.ams_int_to_attack_time3)
		v.append(self.ams_int_to_decay_time3)
		v.append(self.ams_int_to_slope_time3)
		v.append(self.ams_int_to_release_time3)
		v.append(self.reset_source)
		v.append(self.reset_threshold)
		v.append(self._reserved1302)
		return struct.pack('>4BbBbBbBbBbB3bB4bB4bB4bBbH',*v) # 36

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def attack_eg_curve(self):
		'''Attack EG Curve'''
		return self._attack_eg_curve

	@attack_eg_curve.setter
	def attack_eg_curve(self,value):
		if 0 <= value <= 10:
			self._attack_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def decay_eg_curve(self):
		'''Decay EG Curve'''
		return self._decay_eg_curve

	@decay_eg_curve.setter
	def decay_eg_curve(self,value):
		if 0 <= value <= 10:
			self._decay_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def slope_eg_curve(self):
		'''Slope EG Curve'''
		return self._slope_eg_curve

	@slope_eg_curve.setter
	def slope_eg_curve(self,value):
		if 0 <= value <= 10:
			self._slope_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def release_eg_curve(self):
		'''Release EG Curve'''
		return self._release_eg_curve

	@release_eg_curve.setter
	def release_eg_curve(self,value):
		if 0 <= value <= 10:
			self._release_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def start_level(self):
		'''Start Level'''
		return self._start_level

	@start_level.setter
	def start_level(self,value):
		if -99 <= value <= 99:
			self._start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_time(self):
		'''Attack Time'''
		return self._attack_time

	@attack_time.setter
	def attack_time(self,value):
		if 0 <= value <= 99:
			self._attack_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_level(self):
		'''Attack Level'''
		return self._attack_level

	@attack_level.setter
	def attack_level(self,value):
		if -99 <= value <= 99:
			self._attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def decay_time(self):
		'''Decay Time'''
		return self._decay_time

	@decay_time.setter
	def decay_time(self,value):
		if 0 <= value <= 99:
			self._decay_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def break_level(self):
		'''Break Level'''
		return self._break_level

	@break_level.setter
	def break_level(self,value):
		if -99 <= value <= 99:
			self._break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def slope_time(self):
		'''Slope Time'''
		return self._slope_time

	@slope_time.setter
	def slope_time(self,value):
		if 0 <= value <= 99:
			self._slope_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def sustain_level(self):
		'''Sustain Level'''
		return self._sustain_level

	@sustain_level.setter
	def sustain_level(self,value):
		if -99 <= value <= 99:
			self._sustain_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def release_time(self):
		'''Release Time'''
		return self._release_time

	@release_time.setter
	def release_time(self,value):
		if 0 <= value <= 99:
			self._release_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def release_level(self):
		'''Release Level'''
		return self._release_level

	@release_level.setter
	def release_level(self,value):
		if -99 <= value <= 99:
			self._release_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_level(self):
		'''AMS Src To Level'''
		return self._ams_src_to_level

	@ams_src_to_level.setter
	def ams_src_to_level(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_start_level(self):
		'''AMS Int To Start Level'''
		return self._ams_int_to_start_level

	@ams_int_to_start_level.setter
	def ams_int_to_start_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_level(self):
		'''AMS Int To Attack Level'''
		return self._ams_int_to_attack_level

	@ams_int_to_attack_level.setter
	def ams_int_to_attack_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_break_level(self):
		'''AMS Int To Break Level'''
		return self._ams_int_to_break_level

	@ams_int_to_break_level.setter
	def ams_int_to_break_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time1(self):
		'''AMS Src To Time 1'''
		return self._ams_src_to_time1

	@ams_src_to_time1.setter
	def ams_src_to_time1(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time1 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time1(self):
		'''AMS Int To Attack Time 1'''
		return self._ams_int_to_attack_time1

	@ams_int_to_attack_time1.setter
	def ams_int_to_attack_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time1(self):
		'''AMS Int To Decay Time 1'''
		return self._ams_int_to_decay_time1

	@ams_int_to_decay_time1.setter
	def ams_int_to_decay_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time1(self):
		'''AMS Int To Slope Time 1'''
		return self._ams_int_to_slope_time1

	@ams_int_to_slope_time1.setter
	def ams_int_to_slope_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time1(self):
		'''AMS Int To Release Time 1'''
		return self._ams_int_to_release_time1

	@ams_int_to_release_time1.setter
	def ams_int_to_release_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time2(self):
		'''AMS Src To Time 2'''
		return self._ams_src_to_time2

	@ams_src_to_time2.setter
	def ams_src_to_time2(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time2 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time2(self):
		'''AMS Int To Attack Time 2'''
		return self._ams_int_to_attack_time2

	@ams_int_to_attack_time2.setter
	def ams_int_to_attack_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time2(self):
		'''AMS Int To Decay Time 2'''
		return self._ams_int_to_decay_time2

	@ams_int_to_decay_time2.setter
	def ams_int_to_decay_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time2(self):
		'''AMS Int To Slope Time 2'''
		return self._ams_int_to_slope_time2

	@ams_int_to_slope_time2.setter
	def ams_int_to_slope_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time2(self):
		'''AMS Int To Release Time 2'''
		return self._ams_int_to_release_time2

	@ams_int_to_release_time2.setter
	def ams_int_to_release_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time3(self):
		'''AMS Src To Time 3'''
		return self._ams_src_to_time3

	@ams_src_to_time3.setter
	def ams_src_to_time3(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time3 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time3(self):
		'''AMS Int To Attack Time 3'''
		return self._ams_int_to_attack_time3

	@ams_int_to_attack_time3.setter
	def ams_int_to_attack_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time3(self):
		'''AMS Int To Decay Time 3'''
		return self._ams_int_to_decay_time3

	@ams_int_to_decay_time3.setter
	def ams_int_to_decay_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time3(self):
		'''AMS Int To Slope Time 3'''
		return self._ams_int_to_slope_time3

	@ams_int_to_slope_time3.setter
	def ams_int_to_slope_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time3(self):
		'''AMS Int To Release Time 3'''
		return self._ams_int_to_release_time3

	@ams_int_to_release_time3.setter
	def ams_int_to_release_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_source(self):
		'''Reset Source'''
		return self._reset_source

	@reset_source.setter
	def reset_source(self,value):
		if 0 <= value <= 99:
			self._reset_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_threshold(self):
		'''Reset Threshold'''
		return self._reset_threshold

	@reset_threshold.setter
	def reset_threshold(self,value):
		if -99 <= value <= 99:
			self._reset_threshold = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
# Table 1 1304-1311
class PCMOSCFilterKeyTrackParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<3B4bB',raw)) # 8 
		self.key_low = v.pop(0)
		self.key_center = v.pop(0)
		self.key_high = v.pop(0)
		self.ramp_low = v.pop(0)
		self.ramp_center = v.pop(0)
		self.ramp_center_high = v.pop(0)
		self.ramp_high_top = v.pop(0)
		self._reserved1311 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.key_low)
		v.append(self.key_center)
		v.append(self.key_high)
		v.append(self.ramp_low)
		v.append(self.ramp_center)
		v.append(self.ramp_center_high)
		v.append(self.ramp_high_top)
		v.append(self._reserved1311)
		return struct.pack('>3B4bB',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def key_low(self):
		'''Key Low'''
		return self._key_low

	@key_low.setter
	def key_low(self,value):
		if 0 <= value <= 127:
			self._key_low = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_center(self):
		'''Key Center'''
		return self._key_center

	@key_center.setter
	def key_center(self,value):
		if 0 <= value <= 127:
			self._key_center = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_high(self):
		'''Key High'''
		return self._key_high

	@key_high.setter
	def key_high(self,value):
		if 0 <= value <= 127:
			self._key_high = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def ramp_low(self):
		'''Ramp Low'''
		return self._ramp_low

	@ramp_low.setter
	def ramp_low(self,value):
		if -100 <= value <= 100:
			self._ramp_low = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center(self):
		'''Ramp Center'''
		return self._ramp_center

	@ramp_center.setter
	def ramp_center(self,value):
		if -100 <= value <= 100:
			self._ramp_center = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center_high(self):
		'''Ramp Center High'''
		return self._ramp_center_high

	@ramp_center_high.setter
	def ramp_center_high(self,value):
		if -100 <= value <= 100:
			self._ramp_center_high = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_high_top(self):
		'''Ramp High Top'''
		return self._ramp_high_top

	@ramp_high_top.setter
	def ramp_high_top(self,value):
		if -100 <= value <= 100:
			self._ramp_high_top = value
		else:
			raise ValueError('not -100 <= value <= 100')

#===============================================================================
# Table 1 1312-1323
class PCMOSCDriveParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<BbHBbH2BH',raw)) # 12 
		self.ams_to_drive_source = v.pop(0)
		self.ams_to_drive_intensity = v.pop(0)
		self._reserved1314 = v.pop(0)
		self.ams_to_boost_source = v.pop(0)
		self.ams_to_boost_intensity = v.pop(0)
		self._reserved1318 = v.pop(0)
		byte8 = v.pop(0)
		self.drive = byte8 & 0x7F
		self.bypass = (byte8 >> 7) & 0x01
		self.boost = v.pop(0)
		self._reserved1322 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.ams_to_drive_source)
		v.append(self.ams_to_drive_intensity)
		v.append(self._reserved1314)
		v.append(self.ams_to_boost_source)
		v.append(self.ams_to_boost_intensity)
		v.append(self._reserved1318)
		byte8 = 0
		byte8 |= self.Drive & 0x7F
		byte8 |= (self.Bypass & 0x01) << 7
		v.append(byte8)
		v.append(self.Boost)
		v.append(self._reserved1322)
		return struct.pack('>BbHBbH2BH',*v) # 12

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def ams_to_drive_source(self):
		'''AMSTo Drive Source'''
		return self._ams_to_drive_source

	@ams_to_drive_source.setter
	def ams_to_drive_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_drive_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_drive_intensity(self):
		'''AMSToDrive Intensity'''
		return self._ams_to_drive_intensity

	@ams_to_drive_intensity.setter
	def ams_to_drive_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_drive_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_boost_source(self):
		'''AMSTo Boost Source'''
		return self._ams_to_boost_source

	@ams_to_boost_source.setter
	def ams_to_boost_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_boost_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_boost_intensity(self):
		'''AMSTo Boost Intensity'''
		return self._ams_to_boost_intensity

	@ams_to_boost_intensity.setter
	def ams_to_boost_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_boost_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def Drive(self):
		'''Drive'''
		return self._Drive

	@Drive.setter
	def Drive(self,value):
		if 0 <= value <= 99:
			self._Drive = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def Bypass(self):
		'''Bypass'''
		return self._Bypass

	@Bypass.setter
	def Bypass(self,value):
		if 0 <= value <= 1:
			self._Bypass = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def Boost(self):
		'''Boost'''
		return self._Boost

	@Boost.setter
	def Boost(self,value):
		if 0 <= value <= 99:
			self._Boost = value
		else:
			raise ValueError('not 0 <= value <= 99')

#===============================================================================
# Table 1 1324-1343
class PCMOSCAmpParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<BbHBbH2bHBbHBbH',raw)) # 20 
		self.osc_amp_level = v.pop(0)
		self.velocity_to_amp_eg = v.pop(0)
		self._reserved1326 = v.pop(0)
		self.ams_to_amp_eg_source = v.pop(0)
		self.ams_to_amp_eg_intensity = v.pop(0)
		self._reserved1330 = v.pop(0)
		self.lfo1_intensity = v.pop(0)
		self.lfo2_intensity = v.pop(0)
		self._reserved1334 = v.pop(0)
		self.ams_to_lfo1_source = v.pop(0)
		self.ams_to_lfo1_intensity = v.pop(0)
		self._reserved1338 = v.pop(0)
		self.ams_to_lfo2_source = v.pop(0)
		self.ams_to_lfo2_intensity = v.pop(0)
		self._reserved1342 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.osc_amp_level)
		v.append(self.velocity_to_amp_eg)
		v.append(self._reserved1326)
		v.append(self.ams_to_amp_eg_source)
		v.append(self.ams_to_amp_eg_intensity)
		v.append(self._reserved1330)
		v.append(self.lfo1_intensity)
		v.append(self.lfo2_intensity)
		v.append(self._reserved1334)
		v.append(self.ams_to_lfo1_source)
		v.append(self.ams_to_lfo1_intensity)
		v.append(self._reserved1338)
		v.append(self.ams_to_lfo2_source)
		v.append(self.ams_to_lfo2_intensity)
		v.append(self._reserved1342)
		return struct.pack('>BbHBbH2bHBbHBbH',*v) # 20

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def osc_amp_level(self):
		'''OSC AMP Level'''
		return self._osc_amp_level

	@osc_amp_level.setter
	def osc_amp_level(self,value):
		if 0 <= value <= 127:
			self._osc_amp_level = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def velocity_to_amp_eg(self):
		'''Velocity To Amp EG'''
		return self._velocity_to_amp_eg

	@velocity_to_amp_eg.setter
	def velocity_to_amp_eg(self,value):
		if -99 <= value <= 99:
			self._velocity_to_amp_eg = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_amp_eg_source(self):
		'''AMS To Amp EG Source'''
		return self._ams_to_amp_eg_source

	@ams_to_amp_eg_source.setter
	def ams_to_amp_eg_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_amp_eg_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_amp_eg_intensity(self):
		'''AMS To Amp EG Intensity'''
		return self._ams_to_amp_eg_intensity

	@ams_to_amp_eg_intensity.setter
	def ams_to_amp_eg_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_amp_eg_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def lfo1_intensity(self):
		'''LFO1 Intensity'''
		return self._lfo1_intensity

	@lfo1_intensity.setter
	def lfo1_intensity(self,value):
		if -99 <= value <= 99:
			self._lfo1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def lfo2_intensity(self):
		'''LFO2 Intensity'''
		return self._lfo2_intensity

	@lfo2_intensity.setter
	def lfo2_intensity(self,value):
		if -99 <= value <= 99:
			self._lfo2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_lfo1_source(self):
		'''AMS To LFO1 Source'''
		return self._ams_to_lfo1_source

	@ams_to_lfo1_source.setter
	def ams_to_lfo1_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_lfo1_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_lfo1_intensity(self):
		'''AMS To LFO1 Intensity'''
		return self._ams_to_lfo1_intensity

	@ams_to_lfo1_intensity.setter
	def ams_to_lfo1_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_lfo1_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_lfo2_source(self):
		'''AMS To LFO2 Source'''
		return self._ams_to_lfo2_source

	@ams_to_lfo2_source.setter
	def ams_to_lfo2_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_lfo2_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_lfo2_intensity(self):
		'''AMS To LFO2 Intensity'''
		return self._ams_to_lfo2_intensity

	@ams_to_lfo2_intensity.setter
	def ams_to_lfo2_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_lfo2_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
# Table 1 1344-1379
class PCMOSCAmpEGParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<6Bb6B3bB4bB4bB4bBbBH',raw)) # 36 
		self.start_level = v.pop(0)
		self.attack_time = v.pop(0)
		self.attack_level = v.pop(0)
		self.decay_time = v.pop(0)
		self.break_level = v.pop(0)
		self.slope_time = v.pop(0)
		self.sustain_level = v.pop(0)
		self.release_time = v.pop(0)
		self.attack_eg_curve = v.pop(0)
		self.decay_eg_curve = v.pop(0)
		self.slope_eg_curve = v.pop(0)
		self.release_eg_curve = v.pop(0)
		self.ams_src_to_level = v.pop(0)
		self.ams_int_to_start_level = v.pop(0)
		self.ams_int_to_attack_level = v.pop(0)
		self.ams_int_to_break_level = v.pop(0)
		self.ams_src_to_time1 = v.pop(0)
		self.ams_int_to_attack_time1 = v.pop(0)
		self.ams_int_to_decay_time1 = v.pop(0)
		self.ams_int_to_slope_time1 = v.pop(0)
		self.ams_int_to_release_time1 = v.pop(0)
		self.ams_src_to_time2 = v.pop(0)
		self.ams_int_to_attack_time2 = v.pop(0)
		self.ams_int_to_decay_time2 = v.pop(0)
		self.ams_int_to_slope_time2 = v.pop(0)
		self.ams_int_to_release_time2 = v.pop(0)
		self.ams_src_to_time3 = v.pop(0)
		self.ams_int_to_attack_time3 = v.pop(0)
		self.ams_int_to_decay_time3 = v.pop(0)
		self.ams_int_to_slope_time3 = v.pop(0)
		self.ams_int_to_release_time3 = v.pop(0)
		self.reset_source = v.pop(0)
		self.reset_threshold = v.pop(0)
		self._reserved1377 = v.pop(0)
		self._reserved1378 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.start_level)
		v.append(self.attack_time)
		v.append(self.attack_level)
		v.append(self.decay_time)
		v.append(self.break_level)
		v.append(self.slope_time)
		v.append(self.sustain_level)
		v.append(self.release_time)
		v.append(self.attack_eg_curve)
		v.append(self.decay_eg_curve)
		v.append(self.slope_eg_curve)
		v.append(self.release_eg_curve)
		v.append(self.ams_src_to_level)
		v.append(self.ams_int_to_start_level)
		v.append(self.ams_int_to_attack_level)
		v.append(self.ams_int_to_break_level)
		v.append(self.ams_src_to_time1)
		v.append(self.ams_int_to_attack_time1)
		v.append(self.ams_int_to_decay_time1)
		v.append(self.ams_int_to_slope_time1)
		v.append(self.ams_int_to_release_time1)
		v.append(self.ams_src_to_time2)
		v.append(self.ams_int_to_attack_time2)
		v.append(self.ams_int_to_decay_time2)
		v.append(self.ams_int_to_slope_time2)
		v.append(self.ams_int_to_release_time2)
		v.append(self.ams_src_to_time3)
		v.append(self.ams_int_to_attack_time3)
		v.append(self.ams_int_to_decay_time3)
		v.append(self.ams_int_to_slope_time3)
		v.append(self.ams_int_to_release_time3)
		v.append(self.reset_source)
		v.append(self.reset_threshold)
		v.append(self._reserved1377)
		v.append(self._reserved1378)
		return struct.pack('>6Bb6B3bB4bB4bB4bBbBH',*v) # 36

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def start_level(self):
		'''Start Level'''
		return self._start_level

	@start_level.setter
	def start_level(self,value):
		if 0 <= value <= 99:
			self._start_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_time(self):
		'''Attack Time'''
		return self._attack_time

	@attack_time.setter
	def attack_time(self,value):
		if 0 <= value <= 99:
			self._attack_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_level(self):
		'''Attack Level'''
		return self._attack_level

	@attack_level.setter
	def attack_level(self,value):
		if 0 <= value <= 99:
			self._attack_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def decay_time(self):
		'''Decay Time'''
		return self._decay_time

	@decay_time.setter
	def decay_time(self,value):
		if 0 <= value <= 99:
			self._decay_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def break_level(self):
		'''Break Level'''
		return self._break_level

	@break_level.setter
	def break_level(self,value):
		if 0 <= value <= 99:
			self._break_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def slope_time(self):
		'''Slope Time'''
		return self._slope_time

	@slope_time.setter
	def slope_time(self,value):
		if 0 <= value <= 99:
			self._slope_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def sustain_level(self):
		'''Sustain Level'''
		return self._sustain_level

	@sustain_level.setter
	def sustain_level(self,value):
		if -99 <= value <= 99:
			self._sustain_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def release_time(self):
		'''Release Time'''
		return self._release_time

	@release_time.setter
	def release_time(self,value):
		if 0 <= value <= 99:
			self._release_time = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def attack_eg_curve(self):
		'''Attack EG Curve'''
		return self._attack_eg_curve

	@attack_eg_curve.setter
	def attack_eg_curve(self,value):
		if 0 <= value <= 10:
			self._attack_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def decay_eg_curve(self):
		'''Decay EG Curve'''
		return self._decay_eg_curve

	@decay_eg_curve.setter
	def decay_eg_curve(self,value):
		if 0 <= value <= 10:
			self._decay_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def slope_eg_curve(self):
		'''Slope EG Curve'''
		return self._slope_eg_curve

	@slope_eg_curve.setter
	def slope_eg_curve(self,value):
		if 0 <= value <= 10:
			self._slope_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def release_eg_curve(self):
		'''Release EG Curve'''
		return self._release_eg_curve

	@release_eg_curve.setter
	def release_eg_curve(self,value):
		if 0 <= value <= 10:
			self._release_eg_curve = value
		else:
			raise ValueError('not 0 <= value <= 10')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_level(self):
		'''AMS Src To Level'''
		return self._ams_src_to_level

	@ams_src_to_level.setter
	def ams_src_to_level(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_level = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_start_level(self):
		'''AMS Int To Start Level'''
		return self._ams_int_to_start_level

	@ams_int_to_start_level.setter
	def ams_int_to_start_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_start_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_level(self):
		'''AMS Int To Attack Level'''
		return self._ams_int_to_attack_level

	@ams_int_to_attack_level.setter
	def ams_int_to_attack_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_break_level(self):
		'''AMS Int To Break Level'''
		return self._ams_int_to_break_level

	@ams_int_to_break_level.setter
	def ams_int_to_break_level(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_break_level = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time1(self):
		'''AMS Src To Time 1'''
		return self._ams_src_to_time1

	@ams_src_to_time1.setter
	def ams_src_to_time1(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time1 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time1(self):
		'''AMS Int To Attack Time 1'''
		return self._ams_int_to_attack_time1

	@ams_int_to_attack_time1.setter
	def ams_int_to_attack_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time1(self):
		'''AMS Int To Decay Time 1'''
		return self._ams_int_to_decay_time1

	@ams_int_to_decay_time1.setter
	def ams_int_to_decay_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time1(self):
		'''AMS Int To Slope Time 1'''
		return self._ams_int_to_slope_time1

	@ams_int_to_slope_time1.setter
	def ams_int_to_slope_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time1(self):
		'''AMS Int To Release Time 1'''
		return self._ams_int_to_release_time1

	@ams_int_to_release_time1.setter
	def ams_int_to_release_time1(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time1 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time2(self):
		'''AMS Src To Time 2'''
		return self._ams_src_to_time2

	@ams_src_to_time2.setter
	def ams_src_to_time2(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time2 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time2(self):
		'''AMS Int To Attack Time 2'''
		return self._ams_int_to_attack_time2

	@ams_int_to_attack_time2.setter
	def ams_int_to_attack_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time2(self):
		'''AMS Int To Decay Time 2'''
		return self._ams_int_to_decay_time2

	@ams_int_to_decay_time2.setter
	def ams_int_to_decay_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time2(self):
		'''AMS Int To Slope Time 2'''
		return self._ams_int_to_slope_time2

	@ams_int_to_slope_time2.setter
	def ams_int_to_slope_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time2(self):
		'''AMS Int To Release Time 2'''
		return self._ams_int_to_release_time2

	@ams_int_to_release_time2.setter
	def ams_int_to_release_time2(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time2 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_src_to_time3(self):
		'''AMS Src To Time 3'''
		return self._ams_src_to_time3

	@ams_src_to_time3.setter
	def ams_src_to_time3(self,value):
		if 0 <= value <= 99:
			self._ams_src_to_time3 = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_attack_time3(self):
		'''AMS Int To Attack Time 3'''
		return self._ams_int_to_attack_time3

	@ams_int_to_attack_time3.setter
	def ams_int_to_attack_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_attack_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_decay_time3(self):
		'''AMS Int To Decay Time 3'''
		return self._ams_int_to_decay_time3

	@ams_int_to_decay_time3.setter
	def ams_int_to_decay_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_decay_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_slope_time3(self):
		'''AMS Int To Slope Time 3'''
		return self._ams_int_to_slope_time3

	@ams_int_to_slope_time3.setter
	def ams_int_to_slope_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_slope_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_int_to_release_time3(self):
		'''AMS Int To Release Time 3'''
		return self._ams_int_to_release_time3

	@ams_int_to_release_time3.setter
	def ams_int_to_release_time3(self,value):
		if -99 <= value <= 99:
			self._ams_int_to_release_time3 = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_source(self):
		'''Reset Source'''
		return self._reset_source

	@reset_source.setter
	def reset_source(self,value):
		if 0 <= value <= 99:
			self._reset_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def reset_threshold(self):
		'''Reset Threshold'''
		return self._reset_threshold

	@reset_threshold.setter
	def reset_threshold(self,value):
		if -99 <= value <= 99:
			self._reset_threshold = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
# Table 1 1380-1387
class PCMOSCAmpKeyTrackParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<3B4bB',raw)) # 8 
		self.key_low = v.pop(0)
		self.key_center = v.pop(0)
		self.key_high = v.pop(0)
		self.ramp_low = v.pop(0)
		self.ramp_center = v.pop(0)
		self.ramp_center_high = v.pop(0)
		self.ramp_high_top = v.pop(0)
		self._reserved1387 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.key_low)
		v.append(self.key_center)
		v.append(self.key_high)
		v.append(self.ramp_low)
		v.append(self.ramp_center)
		v.append(self.ramp_center_high)
		v.append(self.ramp_high_top)
		v.append(self._reserved1387)
		return struct.pack('>3B4bB',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def key_low(self):
		'''Key Low'''
		return self._key_low

	@key_low.setter
	def key_low(self,value):
		if 0 <= value <= 127:
			self._key_low = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_center(self):
		'''Key Center'''
		return self._key_center

	@key_center.setter
	def key_center(self,value):
		if 0 <= value <= 127:
			self._key_center = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def key_high(self):
		'''Key High'''
		return self._key_high

	@key_high.setter
	def key_high(self,value):
		if 0 <= value <= 127:
			self._key_high = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def ramp_low(self):
		'''Ramp Low'''
		return self._ramp_low

	@ramp_low.setter
	def ramp_low(self,value):
		if -100 <= value <= 100:
			self._ramp_low = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center(self):
		'''Ramp Center'''
		return self._ramp_center

	@ramp_center.setter
	def ramp_center(self,value):
		if -100 <= value <= 100:
			self._ramp_center = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_center_high(self):
		'''Ramp Center High'''
		return self._ramp_center_high

	@ramp_center_high.setter
	def ramp_center_high(self,value):
		if -100 <= value <= 100:
			self._ramp_center_high = value
		else:
			raise ValueError('not -100 <= value <= 100')

#-------------------------------------------------------------------------------
	@property
	def ramp_high_top(self):
		'''Ramp High Top'''
		return self._ramp_high_top

	@ramp_high_top.setter
	def ramp_high_top(self,value):
		if -100 <= value <= 100:
			self._ramp_high_top = value
		else:
			raise ValueError('not -100 <= value <= 100')

#===============================================================================
# Table 1 1388-1395
class PCMOSCOutputParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<BbH4B',raw)) # 8 
		self.ams_to_pan_source = v.pop(0)
		self.ams_to_pan_intensity = v.pop(0)
		self._reserved1390 = v.pop(0)
		self._reserved1392 = v.pop(0)
		byte5 = v.pop(0)
		self.pan = byte5 & 0x7F
		self.use_dkit_setting = (byte5 >> 7) & 0x01
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.ams_to_pan_source)
		v.append(self.ams_to_pan_intensity)
		v.append(self._reserved1390)
		v.append(self._reserved1392)
		byte5 = 0
		byte5 |= self.pan & 0x7F
		byte5 |= (self.use_dkit_setting & 0x01) << 7
		v.append(byte5)
		v.append(self.send1)
		v.append(self.send2)
		return struct.pack('>BbH4B',*v) # 8

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def ams_to_pan_source(self):
		'''AMS To Pan Source'''
		return self._ams_to_pan_source

	@ams_to_pan_source.setter
	def ams_to_pan_source(self,value):
		if 0 <= value <= 99:
			self._ams_to_pan_source = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def ams_to_pan_intensity(self):
		'''AMS To Pan Intensity'''
		return self._ams_to_pan_intensity

	@ams_to_pan_intensity.setter
	def ams_to_pan_intensity(self,value):
		if -99 <= value <= 99:
			self._ams_to_pan_intensity = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def use_dkit_setting(self):
		'''Use DKit Setting'''
		return self._use_dkit_setting

	@use_dkit_setting.setter
	def use_dkit_setting(self,value):
		if -99 <= value <= 99:
			self._use_dkit_setting = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
# Table 1 1396-1419 x 2
class AMSMixerParameters(object):
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
#	def __str__(self):
#		return '%d %d 0x%x' % (self.effect_type,self.on_off,self.reserved470)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('<2BbBbB2h2BbBH2B3bBH',raw)) # 24 
		self.type = v.pop(0)
		self.source_a = v.pop(0)
		self.amount_a = v.pop(0)
		self.source_b = v.pop(0)
		self.amount_b = v.pop(0)
		self.number_of_steps = v.pop(0)
		self.amount_a_for_offset = v.pop(0)
		self.offset = v.pop(0)
		self.attack_a = v.pop(0)
		self.decay_a = v.pop(0)
		self.shape = v.pop(0)
		self.type_for_shape = v.pop(0)
		self._reserved1410 = v.pop(0)
		byte16 = v.pop(0)
		self.cntl_at_note_on_only = byte16 & 0x01
		self.below_threshold = (byte16 >> 1) & 0x01
		self.at_above_threshold = (byte16 >> 2) & 0x01
		self.source = v.pop(0)
		self.threshold = v.pop(0)
		self.velow_fixed_value = v.pop(0)
		self.at_above_fixed_value = v.pop(0)
		self._reserved1417 = v.pop(0)
		self._reserved1418 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self.type)
		v.append(self.source_a)
		v.append(self.amount_a)
		v.append(self.source_b)
		v.append(self.amount_b)
		v.append(self.number_of_steps)
		v.append(self.amount_a_for_offset)
		v.append(self.offset)
		v.append(self.attack_a)
		v.append(self.decay_a)
		v.append(self.shape)
		v.append(self.type_for_shape)
		v.append(self._reserved1410)
		byte16 = 0
		byte16 |= self.cntl_at_note_on_only & 0x01
		byte16 |= (self.below_threshold & 0x01) << 1
		byte16 |= (self.at_above_threshold & 0x01) << 2
		v.append(byte16)
		v.append(self.source)
		v.append(self.threshold)
		v.append(self.velow_fixed_value)
		v.append(self.at_above_fixed_value)
		v.append(self._reserved1417)
		v.append(self._reserved1418)
		return struct.pack('>2BbBbB2h2BbBH2B3bBH',*v) # 24

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a  object'''
		json = []
		return json

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 6:
			self._type = value
		else:
			raise ValueError('not 0 <= value <= 6')

#-------------------------------------------------------------------------------
	@property
	def source_a(self):
		'''Source A'''
		return self._source_a

	@source_a.setter
	def source_a(self,value):
		if 0 <= value <= 50:
			self._source_a = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def amount_a(self):
		'''Amount A'''
		return self._amount_a

	@amount_a.setter
	def amount_a(self,value):
		if -99 <= value <= 99:
			self._amount_a = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def source_b(self):
		'''Source B'''
		return self._source_b

	@source_b.setter
	def source_b(self,value):
		if 0 <= value <= 50:
			self._source_b = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def amount_b(self):
		'''Amount B'''
		return self._amount_b

	@amount_b.setter
	def amount_b(self,value):
		if -99 <= value <= 99:
			self._amount_b = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def number_of_steps(self):
		'''Number Of Steps'''
		return self._number_of_steps

	@number_of_steps.setter
	def number_of_steps(self,value):
		if 2 <= value <= 32:
			self._number_of_steps = value
		else:
			raise ValueError('not 2 <= value <= 32')

#-------------------------------------------------------------------------------
	@property
	def amount_a_for_offset(self):
		'''Amount A For Offset'''
		return self._amount_a_for_offset

	@amount_a_for_offset.setter
	def amount_a_for_offset(self,value):
		if -199 <= value <= 199:
			self._amount_a_for_offset = value
		else:
			raise ValueError('not -199 <= value <= 199')

#-------------------------------------------------------------------------------
	@property
	def offset(self):
		'''Offset'''
		return self._offset

	@offset.setter
	def offset(self,value):
		if -199 <= value <= 199:
			self._offset = value
		else:
			raise ValueError('not -199 <= value <= 199')

#-------------------------------------------------------------------------------
	@property
	def attack_a(self):
		'''Attack A'''
		return self._attack_a

	@attack_a.setter
	def attack_a(self,value):
		if 0 <= value <= 99:
			self._attack_a = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def decay_a(self):
		'''Decay A'''
		return self._decay_a

	@decay_a.setter
	def decay_a(self,value):
		if 0 <= value <= 99:
			self._decay_a = value
		else:
			raise ValueError('not 0 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def shape(self):
		'''Shape'''
		return self._shape

	@shape.setter
	def shape(self,value):
		if -99 <= value <= 99:
			self._shape = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def type_for_shape(self):
		'''Type For Shape'''
		return self._type_for_shape

	@type_for_shape.setter
	def type_for_shape(self,value):
		if 0 <= value <= 1:
			self._type_for_shape = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def cntl_at_note_on_only(self):
		'''Cntl At Note On Only'''
		return self._cntl_at_note_on_only

	@cntl_at_note_on_only.setter
	def cntl_at_note_on_only(self,value):
		if 0 <= value <= 1:
			self._cntl_at_note_on_only = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def below_threshold(self):
		'''Below Threshold'''
		return self._below_threshold

	@below_threshold.setter
	def below_threshold(self,value):
		if 0 <= value <= 1:
			self._below_threshold = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def at_above_threshold(self):
		'''At & Above Threshold'''
		return self._at_above_threshold

	@at_above_threshold.setter
	def at_above_threshold(self,value):
		if 0 <= value <= 1:
			self._at_above_threshold = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def source(self):
		'''Source'''
		return self._source

	@source.setter
	def source(self,value):
		if 0 <= value <= 50:
			self._source = value
		else:
			raise ValueError('not 0 <= value <= 50')

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if -99 <= value <= 99:
			self._threshold = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def velow_fixed_value(self):
		'''Velow Fixed Value'''
		return self._velow_fixed_value

	@velow_fixed_value.setter
	def velow_fixed_value(self,value):
		if -99 <= value <= 99:
			self._velow_fixed_value = value
		else:
			raise ValueError('not -99 <= value <= 99')

#-------------------------------------------------------------------------------
	@property
	def at_above_fixed_value(self):
		'''At & Above Fixed Value'''
		return self._at_above_fixed_value

	@at_above_fixed_value.setter
	def at_above_fixed_value(self,value):
		if -99 <= value <= 99:
			self._at_above_fixed_value = value
		else:
			raise ValueError('not -99 <= value <= 99')

#===============================================================================
if __name__ == '__main__':
	test_data = [
		75, 82, 79, 77, 69, 32, 71, 114, 97, 110, 100, 32,
		80, 105, 97, 110, 111, 32, 32, 32, 32, 32, 32, 32,
		200, 0, 64, 119, 194, 0, 17, 211, 97, 96, 48, 102,
		10, 192, 220, 129, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 7, 16, 2, 0, 64, 0, 127, 127,
		0, 0, 0, 0, 200, 18, 153, 152, 12, 0, 184, 72,
		23, 0, 0, 0, 0, 64, 240, 1, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 16, 3, 0,
		64, 0, 127, 127, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 16, 4, 0, 64, 0, 127, 127, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 16, 5, 0, 64, 0, 127, 127,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0,
		64, 0, 127, 127, 0, 0, 0, 0, 0, 38, 25, 50,
		9, 0, 70, 202, 240, 249, 176, 64, 174, 237, 4, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		29, 80, 0, 0, 20, 20, 69, 80, 226, 1, 0, 0,
		60, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 84, 80, 0, 0,
		90, 90, 0, 127, 200, 0, 0, 57, 248, 254, 97, 68,
		1, 96, 0, 120, 5, 128, 98, 225, 1, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 5, 80, 0, 0,
		127, 0, 0, 0, 25, 0, 0, 16, 159, 15, 0, 4,
		25, 0, 0, 16, 159, 15, 0, 4, 9, 0, 0, 0,
		41, 0, 0, 11, 0, 127, 1, 127, 0, 16, 16, 0,
		60, 117, 0, 0, 63, 117, 0, 0, 67, 117, 0, 0,
		70, 117, 0, 0, 255, 117, 0, 0, 255, 117, 0, 0,
		255, 117, 0, 0, 255, 117, 0, 0, 0, 16, 0, 0,
		62, 117, 0, 0, 65, 117, 0, 0, 69, 117, 0, 0,
		72, 117, 0, 0, 255, 117, 0, 0, 255, 117, 0, 0,
		255, 117, 0, 0, 255, 117, 0, 0, 0, 16, 0, 0,
		65, 117, 0, 0, 68, 117, 0, 0, 72, 117, 0, 0,
		75, 117, 0, 0, 255, 117, 0, 0, 255, 117, 0, 0,
		255, 117, 0, 0, 255, 117, 0, 0, 0, 16, 0, 0,
		67, 117, 0, 0, 70, 117, 0, 0, 74, 117, 0, 0,
		77, 117, 0, 0, 255, 117, 0, 0, 255, 117, 0, 0,
		255, 117, 0, 0, 255, 117, 0, 0, 0, 16, 0, 0,
		148, 42, 0, 0, 0, 0, 0, 0, 93, 0, 5, 24,
		101, 87, 0, 127, 0, 127, 1, 0, 99, 0, 97, 0,
		6, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
		0, 0, 17, 0, 90, 0, 10, 0, 65, 72, 2, 0,
		0, 92, 127, 0, 0, 0, 0, 0, 0, 0, 3, 4,
		16, 2, 3, 4, 99, 0, 83, 16, 4, 2, 0, 0,
		48, 0, 0, 0, 65, 0, 0, 0, 63, 0, 0, 0,
		58, 0, 0, 0, 5, 0, 0, 0, 6, 0, 0, 0,
		7, 0, 0, 0, 8, 0, 0, 0, 52, 0, 12, 0,
		180, 0, 7, 0, 51, 0, 0, 0, 179, 0, 10, 0,
		53, 0, 255, 255, 1, 0, 10, 0, 2, 0, 40, 0,
		3, 0, 10, 0, 0, 127, 0, 127, 127, 127, 1, 2,
		157, 30, 0, 30, 0, 30, 39, 0, 0, 0, 0, 0,
		38, 0, 0, 0, 14, 0, 0, 0, 14, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 132, 1, 0, 0,
		19, 0, 0, 0, 0, 0, 0, 0, 3, 39, 0, 0,
		0, 0, 0, 0, 41, 55, 67, 45, 40, 0, 0, 0,
		41, 60, 84, 206, 206, 50, 50, 0, 1, 3, 7, 0,
		127, 0, 0, 0, 1, 3, 6, 0, 120, 0, 0, 0,
		1, 3, 5, 0, 102, 0, 0, 0, 1, 3, 4, 0,
		87, 0, 0, 0, 1, 3, 3, 0, 76, 0, 0, 0,
		1, 3, 2, 0, 70, 0, 0, 0, 1, 3, 1, 0,
		67, 0, 0, 0, 1, 3, 0, 0, 67, 0, 0, 0,
		114, 0, 0, 0, 100, 0, 0, 0, 86, 0, 0, 0,
		72, 0, 0, 0, 58, 0, 0, 0, 44, 0, 0, 0,
		30, 0, 0, 0, 0, 1, 127, 0, 131, 60, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 132, 1, 0, 0,
		13, 0, 0, 0, 0, 0, 0, 0, 19, 0, 0, 0,
		0, 68, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		6, 1, 0, 0, 13, 0, 0, 0, 0, 0, 0, 0,
		19, 0, 0, 0, 17, 0, 0, 0, 20, 0, 0, 0,
		8, 0, 0, 0, 17, 0, 0, 0, 0, 0, 0, 0,
		10, 0, 0, 0, 0, 23, 0, 0, 0, 0, 0, 0,
		0, 24, 8, 17, 6, 0, 0, 0, 14, 0, 0, 0,
		20, 0, 0, 0, 20, 254, 0, 0, 17, 0, 0, 0,
		0, 99, 99, 2, 0, 50, 35, 70, 12, 0, 0, 0,
		0, 35, 0, 0, 0, 0, 0, 0, 14, 0, 0, 0,
		14, 0, 0, 0, 18, 0, 0, 0, 17, 0, 0, 0,
		0, 99, 99, 99, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 4, 4, 4, 4,
		99, 43, 99, 71, 82, 99, 1, 90, 255, 14, 0, 0,
		0, 14, 14, 3, 0, 70, 13, 0, 239, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 41, 55, 91, 45,
		40, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 127, 70, 0, 0, 17, 0, 0, 0,
		0, 0, 0, 0, 19, 73, 0, 0, 20, 0, 0, 0,
		0, 0, 99, 40, 99, 40, 99, 42, 0, 6, 6, 10,
		14, 0, 0, 0, 9, 0, 0, 0, 40, 15, 0, 0,
		0, 9, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		41, 60, 84, 16, 8, 255, 254, 0, 13, 8, 0, 0,
		0, 64, 127, 127, 2, 14, 0, 27, 99, 2, 132, 0,
		172, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 2, 14, 99, 26, 99, 2, 132, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 1, 3, 8, 0, 24, 0, 0, 0,
		0, 1, 1, 0, 119, 0, 0, 0, 0, 1, 0, 0,
		119, 0, 0, 0, 0, 0, 0, 0, 127, 0, 0, 0,
		0, 0, 0, 0, 127, 0, 0, 0, 0, 0, 0, 0,
		127, 0, 0, 0, 0, 0, 0, 0, 127, 0, 0, 0,
		0, 0, 0, 0, 127, 0, 0, 0, 1, 0, 0, 0,
		1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0,
		1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0,
		128, 1, 127, 0, 131, 60, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 132, 1, 0, 0, 13, 0, 0, 0,
		0, 0, 0, 0, 19, 0, 0, 0, 0, 68, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 6, 1, 0, 0,
		13, 0, 0, 0, 0, 0, 0, 0, 19, 0, 0, 0,
		17, 0, 0, 0, 20, 0, 0, 0, 8, 0, 0, 0,
		17, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
		0, 23, 0, 0, 0, 0, 0, 0, 0, 24, 8, 17,
		6, 0, 0, 0, 14, 0, 0, 0, 20, 0, 0, 0,
		20, 254, 0, 0, 17, 0, 0, 0, 0, 99, 99, 2,
		0, 50, 35, 70, 12, 0, 0, 0, 0, 35, 0, 0,
		0, 0, 0, 0, 14, 0, 0, 0, 14, 0, 0, 0,
		18, 0, 0, 0, 17, 0, 0, 0, 0, 99, 99, 99,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 4, 4, 4, 4, 99, 43, 99, 71,
		82, 99, 1, 90, 255, 14, 0, 0, 0, 14, 14, 3,
		0, 70, 13, 0, 239, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 41, 55, 91, 45, 40, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		127, 70, 0, 0, 17, 0, 0, 0, 0, 0, 0, 0,
		19, 73, 0, 0, 20, 0, 0, 0, 0, 0, 99, 40,
		99, 40, 99, 40, 0, 6, 6, 10, 14, 0, 0, 0,
		9, 0, 0, 0, 40, 15, 0, 0, 0, 9, 1, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 41, 60, 84, 16,
		8, 255, 254, 0, 13, 8, 0, 0, 0, 64, 127, 127,
		2, 14, 0, 27, 99, 2, 132, 0, 172, 255, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		2, 14, 99, 26, 99, 2, 132, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
	print 'program name',''.join([chr(d) for d in test_data[0:24]])
	if test_data[88] > 0:
		print 'ifx1',(test_data[88],test_data[92],tuple(test_data[24:88]))
	if test_data[164] > 0:
		print 'ifx2',(test_data[164],tuple(test_data[100:164]))
	if test_data[240] > 0:
		print 'ifx3',(test_data[240],tuple(test_data[176:240]))
	if test_data[316] > 0:
		print 'ifx4',(test_data[316],tuple(test_data[252:316]))
	if test_data[392] > 0:
		print 'ifx5',(test_data[392],tuple(test_data[328:392]))
	if test_data[468] > 0:
		print 'mfx1',(test_data[468],tuple(test_data[404:468]))
	if test_data[536] > 0:
		print 'mfx2',(test_data[536],tuple(test_data[472:536]))
	if test_data[608] > 0:
		print 'tfx',(test_data[608],tuple(test_data[544:608]))
	print 'tone_adjust_switch1',test_data[896:900]
	print 'tone_adjust_switch2',test_data[900:904]
	print 'tone_adjust_switch3',test_data[904:908]
	print 'tone_adjust_switch4',test_data[908:912]
	print 'tone_adjust_switch5',test_data[912:916]
	print 'tone_adjust_switch6',test_data[916:920]
	print 'tone_adjust_switch7',test_data[920:924]
	print 'tone_adjust_switch8',test_data[924:928]
	raw = ''.join([chr(d) for d in test_data])
#	obj = ProgramParameters(raw)
