#!@PYTHON@
# -*- coding: utf-8 -*-
#
# table10.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
#from modsrc import module_sources

mfx_chain_direction_dict = {0:'MFX1->MFX2',1:'MFX2->MFX1'}

off_on_dict = {0:'Off',1:'On'}

control_assign_dict = {0:'Tone Control',1:'User Control',2:'ARP'}

ifx_channel_dict = {0:'Ch01',1:'Ch02',2:'Ch03',3:'Ch04',4:'Ch05',5:'Ch06',6:'Ch07',7:'Ch08',8:'Ch09',9:'Ch10',10:'Ch11',11:'Ch12',12:'Ch13',13:'Ch14',14:'Ch15',15:'Ch16',16:'G ch',17:'All-R'}

chain1_to_dict = {0:'IF2',1:'IFX3',2:'IFX4',3:'IFX5'}

bus_select_dict = {0:'L/R',7:'Off'}

fx_control_dict = {0:'Off',1:'FX Ctrl1',2:'FX Ctrl2'}

chain2_to_dict = {0:'IF3',1:'IFX4',2:'IFX5'}

chain3_to_dict = {0:'IF4',1:'IFX5'}

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

pattern_bank_dict = {0:'Preset',1:'User'}

button_mode_dict = {0:'Start Immediately',1:'Wait KBD Trigger'}

trigger_sync_dict = {0:'Off',1:'SEQ'}

sw_assignable_sw_dict = {0:'Off',1:'SW1 Mod. (CC#80)',2:'Porta.SW (CC#65)',3:'Octave Down',4:'Octave Up',5:'JS X Lock',6:'JS Y Lock',7:'JS+Y Lock',8:'JS-Y Lock'}

sw_mode_dict = {0:'Toggle',1:'Momentary'}

midi_status_dict = {0:'Off',1:'Int',2:'Ext',3:'Ext2'}

delay_mode_dict = {0:'Time',1:'Tempo'}

dkit_patch_ifx_dict = {0:'L/R',1:'IFX1',2:'IFX2',3:'IFX3',4:'IFX4',5:'IFX5',12:'Off'}

tbus_select_dict = {0:'DKit',1:'L/R',2:'IFX1',3:'IFX2',4:'IFX3',5:'IFX4',6:'IFX5',13:'Off'}

force_osc_mode_dict = {0:'Prog',1:'Poly',2:'Mono',3:'Mono Legato'}

osc_select_dict = {0:'Both',1:'OSC1',2:'OSC2'}

arpeggiator_assign_dict = {0:'Off',1:'A',2:'B'}

tone_adjust_p2s = {
		0: 0,
		1: 1,
		2: 2,
		3: 3,
		4: 4,
		5: 5,
		6: 6,
		7: 7,
		8: 8,
		9: 9,
		10: 10,
		11: 11,
		12: 12,
		13: 13,
		14: 14,
		15: 15,
		16: 16,
		17: 17,
		18: 18,
		19: 19,
		20: 20,
		21: 21,
		22: 22,
		23: 23,
		24: 24,
		25: 25,
		26: 26,
		27: 27,
		28: 28,
		29: 29,
		30: 30,
		31: 31,
		32: 32,
		33: 33,
		34: 34,
		35: 48,
		36: 49,
		37: 50,
		38: 51,
		39: 52,
		40: 53,
		41: 54,
		42: 55,
		43: 56,
		44: 57,
		45: 58,
		46: 59,
		47: 60,
		48: 61,
		49: 62,
		50: 63,
		51: 64,
		52: 65,
		53: 66,
		54: 67,
		55: 68,
		56: 69,
		57: 70,
		58: 179,
		59: 180,
		60: 181,
		61: 182,
		62: 183,
		63: 184,
		64: 185,
		65: 186,
		66: 187,
		67: 188,
		68: 189,
		69: 190,
		70: 191,
		71: 192,
		72: 193,
		73: 194,
		74: 195,
		75: 196,
		76: 197,
		77: 198,
}

tone_adjust_s2p = {
		0: 0,
		1: 1,
		2: 2,
		3: 3,
		4: 4,
		5: 5,
		6: 6,
		7: 7,
		8: 8,
		9: 9,
		10: 10,
		11: 11,
		12: 12,
		13: 13,
		14: 14,
		15: 15,
		16: 16,
		17: 17,
		18: 18,
		19: 19,
		20: 20,
		21: 21,
		22: 22,
		23: 23,
		24: 24,
		25: 25,
		26: 26,
		27: 27,
		28: 28,
		29: 29,
		30: 30,
		31: 31,
		32: 32,
		33: 33,
		34: 34,
		48: 35,
		49: 36,
		50: 37,
		51: 38,
		52: 39,
		53: 40,
		54: 41,
		55: 42,
		56: 43,
		57: 44,
		58: 45,
		59: 46,
		60: 47,
		61: 48,
		62: 49,
		63: 50,
		64: 51,
		65: 52,
		66: 53,
		67: 54,
		68: 55,
		69: 56,
		70: 57,
		179: 58,
		180: 59,
		181: 60,
		182: 61,
		183: 62,
		184: 63,
		185: 64,
		186: 65,
		187: 66,
		188: 67,
		189: 68,
		190: 69,
		191: 70,
		192: 71,
		193: 72,
		194: 73,
		195: 74,
		196: 75,
		197: 76,
		198: 77,
}

tone_adjust_dict = {
		0: 'Off',
		1: 'Filter Cutoff',
		2: 'Filter Resonance',
		3: 'Filter EG Intensity',
		4: 'Amp Velocity Intensity',
		5: 'F/A EG Attack Time',
		6: 'F/A EG Decay Time',
		7: 'F/A EG Sustain Level',
		8: 'F/A EG Release Time',
		9: 'Filter EG Attack Time',
		10: 'Filter EG Decay Time',
		11: 'Filter EG Sustain Level',
		12: 'Filter EG Release Time',
		13: 'Amp EG Attack Time',
		14: 'Amp EG Decay Time',
		15: 'Amp EG Sustain Level',
		16: 'Amp EG Release Time',
		17: 'Pitch EG Attack Time',
		18: 'Pitch EG Decay Time',
		19: 'Pitch EG Sustain Level (N/A)',
		20: 'Pitch EG Release Time',
		21: 'Pitch LFO1 Intensity',
		22: 'LFO1 Speed',
		23: 'LFO1 Fade',
		24: 'LFO1 Delay',
		25: 'LFO1 Stop',
		26: 'LFO2 Speed',
		27: 'LFO2 Fade',
		28: 'LFO2 Delay',
		29: 'LFO2 Stop',
		30: 'Common LFO Speed',
		31: 'Unison',
		32: 'Number Of Voices',
		33: 'Detune',
		34: 'Thickness',
		35: '[OSC1&2] Pitch Stretch',
		36: '[OSC1&2] Hold',
		37: '[OSC1&2] Reverse',
		38: '[OSC1] Tune',
		39: '[OSC1] Transpose',
		40: '[OSC1] MS/DKit Select',
		41: '[OSC1] MS Bank',
		42: '[OSC1] MS/DKit Min #',
		43: '[OSC1] MS/DKit Max #',
		44: '[OSC1] Start Offset',
		45: '[OSC1] Drive',
		46: '[OSC1] Low Boost',
		47: '[OSC1] Pitch Slope',
		48: '[OSC1] LFO1 Waveform',
		49: '[OSC1] LFO2 Waveform',
		50: '[OSC1] Amp LFO1 Int',
		51: '[OSC1] Amp LFO2 Int',
		52: '[OSC1] Filtr LFO1 Int A',
		53: '[OSC1] Filtr LFO1 Int B',
		54: '[OSC1] Filtr LFO2 Int A',
		55: '[OSC1] Filtr LFO2 Int B',
		56: '[OSC1] Pitch LFO1 AMS Int',
		57: '[OSC1] Pitch LFO2 AMS Int',
		58: '[OSC2] Tune',
		59: '[OSC2] Transpose',
		60: '[OSC2] MS/DKit Select',
		61: '[OSC2] MS Bank',
		62: '[OSC2] MS/DKit Min #',
		63: '[OSC2] MS/DKit Max #',
		64: '[OSC2] Start Offset',
		65: '[OSC2] Drive',
		66: '[OSC2] Low Boost',
		67: '[OSC2] Pitch Slope',
		68: '[OSC2] LFO1 Waveform',
		69: '[OSC2] LFO2 Waveform',
		70: '[OSC2] Amp LFO1 Int',
		71: '[OSC2] Amp LFO2 Int',
		72: '[OSC2] Filtr LFO1 Int A',
		73: '[OSC2] Filtr LFO1 Int B',
		74: '[OSC2] Filtr LFO2 Int A',
		75: '[OSC2] Filtr LFO2 Int B',
		76: '[OSC2] Pitch LFO1 AMS Int',
		77: '[OSC2] Pitch LFO2 AMS Int',
}

tone_adjust_dict_s = dict([(k,tone_adjust_dict[tone_adjust_s2p[k]]) for k in tone_adjust_s2p.keys()])

#===============================================================================
class TableBase(object):
#-------------------------------------------------------------------------------
	def __init__(self):
		self._listeners = []

#-------------------------------------------------------------------------------
	def add_listener(self,listener):
		self._listeners.append(listener)

#-------------------------------------------------------------------------------
	def remove_listener(self,listener):
		self._listeners.remove(listener)

#-------------------------------------------------------------------------------
	def notify_listeners(self,paramid,subid,value):
		for listener in self._listeners:
			listener.parameter_changed(4,paramid,subid,value)

#===============================================================================
class CombinationParameters(TableBase):
	description = '''Combination Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a CombinationParameters Combination Parameters object'''
		v = list(struct.unpack('<24s76s76s76s76s76s68s68s4B72s8s8s4B12s144B8s12s12s12s112s112s112s112s112s112s112s112s112s112s112s112s112s112s112s112s',raw[:2628]))
		self.combination_name = v.pop(0)
		self.ifx1 = v.pop(0)
		self.ifx2 = v.pop(0)
		self.ifx3 = v.pop(0)
		self.ifx4 = v.pop(0)
		self.ifx5 = v.pop(0)
		self.mfx1 = v.pop(0)
		self.mfx2 = v.pop(0)
		self.mfx1_return_level = v.pop(0)
		self.mfx2_return_level = v.pop(0)
		byte542 = v.pop(0)
		self.mfx_chain_direction = byte542 & 0x1
		self.mfx_chain_on_off = (byte542 >> 1) & 0x1
		self._reserved0542_7 = (byte542 >> 2) & 0x3F
		self.mfx_chain_level = v.pop(0)
		self.tfx = v.pop(0)
		self.lfo1 = v.pop(0)
		self.lfo2 = v.pop(0)
		self.control_assign = v.pop(0)
		self.selected_track = v.pop(0)
		self._reserved0634 = v.pop(0)
		self._reserved0635 = v.pop(0)
		self.drum = v.pop(0)
		self._reserved0648 = v.pop(0)
		self._reserved0649 = v.pop(0)
		self._reserved0650 = v.pop(0)
		self._reserved0651 = v.pop(0)
		self._reserved0652 = v.pop(0)
		self._reserved0653 = v.pop(0)
		self._reserved0654 = v.pop(0)
		self._reserved0655 = v.pop(0)
		self._reserved0656 = v.pop(0)
		self._reserved0657 = v.pop(0)
		self._reserved0658 = v.pop(0)
		self._reserved0659 = v.pop(0)
		self._reserved0660 = v.pop(0)
		self._reserved0661 = v.pop(0)
		self._reserved0662 = v.pop(0)
		self._reserved0663 = v.pop(0)
		self._reserved0664 = v.pop(0)
		self._reserved0665 = v.pop(0)
		self._reserved0666 = v.pop(0)
		self._reserved0667 = v.pop(0)
		self._reserved0668 = v.pop(0)
		self._reserved0669 = v.pop(0)
		self._reserved0670 = v.pop(0)
		self._reserved0671 = v.pop(0)
		self._reserved0672 = v.pop(0)
		self._reserved0673 = v.pop(0)
		self._reserved0674 = v.pop(0)
		self._reserved0675 = v.pop(0)
		self._reserved0676 = v.pop(0)
		self._reserved0677 = v.pop(0)
		self._reserved0678 = v.pop(0)
		self._reserved0679 = v.pop(0)
		self._reserved0680 = v.pop(0)
		self._reserved0681 = v.pop(0)
		self._reserved0682 = v.pop(0)
		self._reserved0683 = v.pop(0)
		self._reserved0684 = v.pop(0)
		self._reserved0685 = v.pop(0)
		self._reserved0686 = v.pop(0)
		self._reserved0687 = v.pop(0)
		self._reserved0688 = v.pop(0)
		self._reserved0689 = v.pop(0)
		self._reserved0690 = v.pop(0)
		self._reserved0691 = v.pop(0)
		self._reserved0692 = v.pop(0)
		self._reserved0693 = v.pop(0)
		self._reserved0694 = v.pop(0)
		self._reserved0695 = v.pop(0)
		self._reserved0696 = v.pop(0)
		self._reserved0697 = v.pop(0)
		self._reserved0698 = v.pop(0)
		self._reserved0699 = v.pop(0)
		self._reserved0700 = v.pop(0)
		self._reserved0701 = v.pop(0)
		self._reserved0702 = v.pop(0)
		self._reserved0703 = v.pop(0)
		self._reserved0704 = v.pop(0)
		self._reserved0705 = v.pop(0)
		self._reserved0706 = v.pop(0)
		self._reserved0707 = v.pop(0)
		self._reserved0708 = v.pop(0)
		self._reserved0709 = v.pop(0)
		self._reserved0710 = v.pop(0)
		self._reserved0711 = v.pop(0)
		self._reserved0712 = v.pop(0)
		self._reserved0713 = v.pop(0)
		self._reserved0714 = v.pop(0)
		self._reserved0715 = v.pop(0)
		self._reserved0716 = v.pop(0)
		self._reserved0717 = v.pop(0)
		self._reserved0718 = v.pop(0)
		self._reserved0719 = v.pop(0)
		self._reserved0720 = v.pop(0)
		self._reserved0721 = v.pop(0)
		self._reserved0722 = v.pop(0)
		self._reserved0723 = v.pop(0)
		self._reserved0724 = v.pop(0)
		self._reserved0725 = v.pop(0)
		self._reserved0726 = v.pop(0)
		self._reserved0727 = v.pop(0)
		self._reserved0728 = v.pop(0)
		self._reserved0729 = v.pop(0)
		self._reserved0730 = v.pop(0)
		self._reserved0731 = v.pop(0)
		self._reserved0732 = v.pop(0)
		self._reserved0733 = v.pop(0)
		self._reserved0734 = v.pop(0)
		self._reserved0735 = v.pop(0)
		self._reserved0736 = v.pop(0)
		self._reserved0737 = v.pop(0)
		self._reserved0738 = v.pop(0)
		self._reserved0739 = v.pop(0)
		self._reserved0740 = v.pop(0)
		self._reserved0741 = v.pop(0)
		self._reserved0742 = v.pop(0)
		self._reserved0743 = v.pop(0)
		self._reserved0744 = v.pop(0)
		self._reserved0745 = v.pop(0)
		self._reserved0746 = v.pop(0)
		self._reserved0747 = v.pop(0)
		self._reserved0748 = v.pop(0)
		self._reserved0749 = v.pop(0)
		self._reserved0750 = v.pop(0)
		self._reserved0751 = v.pop(0)
		self._reserved0752 = v.pop(0)
		self._reserved0753 = v.pop(0)
		self._reserved0754 = v.pop(0)
		self._reserved0755 = v.pop(0)
		self._reserved0756 = v.pop(0)
		self._reserved0757 = v.pop(0)
		self._reserved0758 = v.pop(0)
		self._reserved0759 = v.pop(0)
		self._reserved0760 = v.pop(0)
		self._reserved0761 = v.pop(0)
		self._reserved0762 = v.pop(0)
		self._reserved0763 = v.pop(0)
		self._reserved0764 = v.pop(0)
		self._reserved0765 = v.pop(0)
		self._reserved0766 = v.pop(0)
		self._reserved0767 = v.pop(0)
		self._reserved0768 = v.pop(0)
		self._reserved0769 = v.pop(0)
		self._reserved0770 = v.pop(0)
		self._reserved0771 = v.pop(0)
		self._reserved0772 = v.pop(0)
		self._reserved0773 = v.pop(0)
		self._reserved0774 = v.pop(0)
		self._reserved0775 = v.pop(0)
		self._reserved0776 = v.pop(0)
		self._reserved0777 = v.pop(0)
		self._reserved0778 = v.pop(0)
		self._reserved0779 = v.pop(0)
		self._reserved0780 = v.pop(0)
		self._reserved0781 = v.pop(0)
		self._reserved0782 = v.pop(0)
		self._reserved0783 = v.pop(0)
		self._reserved0784 = v.pop(0)
		self._reserved0785 = v.pop(0)
		self._reserved0786 = v.pop(0)
		self._reserved0787 = v.pop(0)
		self._reserved0788 = v.pop(0)
		self._reserved0789 = v.pop(0)
		self._reserved0790 = v.pop(0)
		self._reserved0791 = v.pop(0)
		self.arp = v.pop(0)
		self.arpa = v.pop(0)
		self.arpb = v.pop(0)
		self.common = v.pop(0)
		self.trk1 = v.pop(0)
		self.trk2 = v.pop(0)
		self.trk3 = v.pop(0)
		self.trk4 = v.pop(0)
		self.trk5 = v.pop(0)
		self.trk6 = v.pop(0)
		self.trk7 = v.pop(0)
		self.trk8 = v.pop(0)
		self.trk9 = v.pop(0)
		self.trk10 = v.pop(0)
		self.trk11 = v.pop(0)
		self.trk12 = v.pop(0)
		self.trk13 = v.pop(0)
		self.trk14 = v.pop(0)
		self.trk15 = v.pop(0)
		self.trk16 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a CombinationParameters Combination Parameters object into raw data'''
		v = []
		v.append(self.ifx1.pack())
		v.append(self.ifx2.pack())
		v.append(self.ifx3.pack())
		v.append(self.ifx4.pack())
		v.append(self.ifx5.pack())
		v.append(self.mfx1.pack())
		v.append(self.mfx2.pack())
		v.append(self.mfx1_return_level)
		v.append(self.mfx2_return_level)
		byte542 = 0
		byte542 |= self.mfx_chain_direction & 0x1
		byte542 |= (self.mfx_chain_on_off & 0x1) << 1
		byte542 |= (self._reserved0542_7 & 0x3F) << 2
		v.append(byte542)
		v.append(self.mfx_chain_level)
		v.append(self.tfx.pack())
		v.append(self.lfo1.pack())
		v.append(self.lfo2.pack())
		v.append(self.control_assign)
		v.append(self.selected_track)
		v.append(self._reserved0634)
		v.append(self._reserved0635)
		v.append(self.drum.pack())
		v.append(self._reserved0648)
		v.append(self._reserved0649)
		v.append(self._reserved0650)
		v.append(self._reserved0651)
		v.append(self._reserved0652)
		v.append(self._reserved0653)
		v.append(self._reserved0654)
		v.append(self._reserved0655)
		v.append(self._reserved0656)
		v.append(self._reserved0657)
		v.append(self._reserved0658)
		v.append(self._reserved0659)
		v.append(self._reserved0660)
		v.append(self._reserved0661)
		v.append(self._reserved0662)
		v.append(self._reserved0663)
		v.append(self._reserved0664)
		v.append(self._reserved0665)
		v.append(self._reserved0666)
		v.append(self._reserved0667)
		v.append(self._reserved0668)
		v.append(self._reserved0669)
		v.append(self._reserved0670)
		v.append(self._reserved0671)
		v.append(self._reserved0672)
		v.append(self._reserved0673)
		v.append(self._reserved0674)
		v.append(self._reserved0675)
		v.append(self._reserved0676)
		v.append(self._reserved0677)
		v.append(self._reserved0678)
		v.append(self._reserved0679)
		v.append(self._reserved0680)
		v.append(self._reserved0681)
		v.append(self._reserved0682)
		v.append(self._reserved0683)
		v.append(self._reserved0684)
		v.append(self._reserved0685)
		v.append(self._reserved0686)
		v.append(self._reserved0687)
		v.append(self._reserved0688)
		v.append(self._reserved0689)
		v.append(self._reserved0690)
		v.append(self._reserved0691)
		v.append(self._reserved0692)
		v.append(self._reserved0693)
		v.append(self._reserved0694)
		v.append(self._reserved0695)
		v.append(self._reserved0696)
		v.append(self._reserved0697)
		v.append(self._reserved0698)
		v.append(self._reserved0699)
		v.append(self._reserved0700)
		v.append(self._reserved0701)
		v.append(self._reserved0702)
		v.append(self._reserved0703)
		v.append(self._reserved0704)
		v.append(self._reserved0705)
		v.append(self._reserved0706)
		v.append(self._reserved0707)
		v.append(self._reserved0708)
		v.append(self._reserved0709)
		v.append(self._reserved0710)
		v.append(self._reserved0711)
		v.append(self._reserved0712)
		v.append(self._reserved0713)
		v.append(self._reserved0714)
		v.append(self._reserved0715)
		v.append(self._reserved0716)
		v.append(self._reserved0717)
		v.append(self._reserved0718)
		v.append(self._reserved0719)
		v.append(self._reserved0720)
		v.append(self._reserved0721)
		v.append(self._reserved0722)
		v.append(self._reserved0723)
		v.append(self._reserved0724)
		v.append(self._reserved0725)
		v.append(self._reserved0726)
		v.append(self._reserved0727)
		v.append(self._reserved0728)
		v.append(self._reserved0729)
		v.append(self._reserved0730)
		v.append(self._reserved0731)
		v.append(self._reserved0732)
		v.append(self._reserved0733)
		v.append(self._reserved0734)
		v.append(self._reserved0735)
		v.append(self._reserved0736)
		v.append(self._reserved0737)
		v.append(self._reserved0738)
		v.append(self._reserved0739)
		v.append(self._reserved0740)
		v.append(self._reserved0741)
		v.append(self._reserved0742)
		v.append(self._reserved0743)
		v.append(self._reserved0744)
		v.append(self._reserved0745)
		v.append(self._reserved0746)
		v.append(self._reserved0747)
		v.append(self._reserved0748)
		v.append(self._reserved0749)
		v.append(self._reserved0750)
		v.append(self._reserved0751)
		v.append(self._reserved0752)
		v.append(self._reserved0753)
		v.append(self._reserved0754)
		v.append(self._reserved0755)
		v.append(self._reserved0756)
		v.append(self._reserved0757)
		v.append(self._reserved0758)
		v.append(self._reserved0759)
		v.append(self._reserved0760)
		v.append(self._reserved0761)
		v.append(self._reserved0762)
		v.append(self._reserved0763)
		v.append(self._reserved0764)
		v.append(self._reserved0765)
		v.append(self._reserved0766)
		v.append(self._reserved0767)
		v.append(self._reserved0768)
		v.append(self._reserved0769)
		v.append(self._reserved0770)
		v.append(self._reserved0771)
		v.append(self._reserved0772)
		v.append(self._reserved0773)
		v.append(self._reserved0774)
		v.append(self._reserved0775)
		v.append(self._reserved0776)
		v.append(self._reserved0777)
		v.append(self._reserved0778)
		v.append(self._reserved0779)
		v.append(self._reserved0780)
		v.append(self._reserved0781)
		v.append(self._reserved0782)
		v.append(self._reserved0783)
		v.append(self._reserved0784)
		v.append(self._reserved0785)
		v.append(self._reserved0786)
		v.append(self._reserved0787)
		v.append(self._reserved0788)
		v.append(self._reserved0789)
		v.append(self._reserved0790)
		v.append(self._reserved0791)
		v.append(self.arp.pack())
		v.append(self.arpa.pack())
		v.append(self.arpb.pack())
		v.append(self.common.pack())
		v.append(self.trk1.pack())
		v.append(self.trk2.pack())
		v.append(self.trk3.pack())
		v.append(self.trk4.pack())
		v.append(self.trk5.pack())
		v.append(self.trk6.pack())
		v.append(self.trk7.pack())
		v.append(self.trk8.pack())
		v.append(self.trk9.pack())
		v.append(self.trk10.pack())
		v.append(self.trk11.pack())
		v.append(self.trk12.pack())
		v.append(self.trk13.pack())
		v.append(self.trk14.pack())
		v.append(self.trk15.pack())
		v.append(self.trk16.pack())
		return struct.pack('<24s76s76s76s76s76s68s68s4B72s8s8s4B12s144B8s12s12s12s112s112s112s112s112s112s112s112s112s112s112s112s112s112s112s112s',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a CombinationParameters Combination Parameters object'''
		json = []
		json.append(('name','CombinationParameters'))
		json.append(('description','Combination Parameters'))
		json.append(['combination_name',self.combination_name])
		json.append(['ifx1',self.ifx1.toJSON()])
		json.append(['ifx2',self.ifx2.toJSON()])
		json.append(['ifx3',self.ifx3.toJSON()])
		json.append(['ifx4',self.ifx4.toJSON()])
		json.append(['ifx5',self.ifx5.toJSON()])
		json.append(['mfx1',self.mfx1.toJSON()])
		json.append(['mfx2',self.mfx2.toJSON()])
		json.append(['mfx1_return_level',self.mfx1_return_level])
		json.append(['mfx2_return_level',self.mfx2_return_level])
		json.append(['mfx_chain_direction',self.mfx_chain_direction])
		json.append(['mfx_chain_direction_display',self.mfx_chain_direction_display])
		json.append(['mfx_chain_on_off',self.mfx_chain_on_off])
		json.append(['mfx_chain_on_off_display',self.mfx_chain_on_off_display])
		json.append(['mfx_chain_level',self.mfx_chain_level])
		json.append(['tfx',self.tfx.toJSON()])
		json.append(['lfo1',self.lfo1.toJSON()])
		json.append(['lfo2',self.lfo2.toJSON()])
		json.append(['control_assign',self.control_assign])
		json.append(['control_assign_display',self.control_assign_display])
		json.append(['selected_track',self.selected_track])
		json.append(['drum',self.drum.toJSON()])
		json.append(['arp',self.arp.toJSON()])
		json.append(['arpa',self.arpa.toJSON()])
		json.append(['arpb',self.arpb.toJSON()])
		json.append(['common',self.common.toJSON()])
		json.append(['trk1',self.trk1.toJSON()])
		json.append(['trk2',self.trk2.toJSON()])
		json.append(['trk3',self.trk3.toJSON()])
		json.append(['trk4',self.trk4.toJSON()])
		json.append(['trk5',self.trk5.toJSON()])
		json.append(['trk6',self.trk6.toJSON()])
		json.append(['trk7',self.trk7.toJSON()])
		json.append(['trk8',self.trk8.toJSON()])
		json.append(['trk9',self.trk9.toJSON()])
		json.append(['trk10',self.trk10.toJSON()])
		json.append(['trk11',self.trk11.toJSON()])
		json.append(['trk12',self.trk12.toJSON()])
		json.append(['trk13',self.trk13.toJSON()])
		json.append(['trk14',self.trk14.toJSON()])
		json.append(['trk15',self.trk15.toJSON()])
		json.append(['trk16',self.trk16.toJSON()])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def combination_name(self):
		'''Combination Name'''
		return self._combination_name

	@combination_name.setter
	def combination_name(self,value):
		self._combination_name = value

#-------------------------------------------------------------------------------
	@property
	def ifx1(self):
		'''IFX1 Effect Parameters'''
		return self._ifx1

	@ifx1.setter
	def ifx1(self,value):
		if isinstance(value,InsertEffect1Parameters):
			self._ifx1 = value
		elif isinstance(value,basestring):
			self._ifx1 = InsertEffect1Parameters(value)
		else:
			raise TypeError('value must of type InsertEffect1Parameters')

#-------------------------------------------------------------------------------
	@property
	def ifx2(self):
		'''IFX2 Effect Parameters'''
		return self._ifx2

	@ifx2.setter
	def ifx2(self,value):
		if isinstance(value,InsertEffect2Parameters):
			self._ifx2 = value
		elif isinstance(value,basestring):
			self._ifx2 = InsertEffect2Parameters(value)
		else:
			raise TypeError('value must of type InsertEffect2Parameters')

#-------------------------------------------------------------------------------
	@property
	def ifx3(self):
		'''IFX3 Effect Parameters'''
		return self._ifx3

	@ifx3.setter
	def ifx3(self,value):
		if isinstance(value,InsertEffect3Parameters):
			self._ifx3 = value
		elif isinstance(value,basestring):
			self._ifx3 = InsertEffect3Parameters(value)
		else:
			raise TypeError('value must of type InsertEffect3Parameters')

#-------------------------------------------------------------------------------
	@property
	def ifx4(self):
		'''IFX4 Effect Parameters'''
		return self._ifx4

	@ifx4.setter
	def ifx4(self,value):
		if isinstance(value,InsertEffect4Parameters):
			self._ifx4 = value
		elif isinstance(value,basestring):
			self._ifx4 = InsertEffect4Parameters(value)
		else:
			raise TypeError('value must of type InsertEffect4Parameters')

#-------------------------------------------------------------------------------
	@property
	def ifx5(self):
		'''IFX5 Effect Parameters'''
		return self._ifx5

	@ifx5.setter
	def ifx5(self,value):
		if isinstance(value,InsertEffect5Parameters):
			self._ifx5 = value
		elif isinstance(value,basestring):
			self._ifx5 = InsertEffect5Parameters(value)
		else:
			raise TypeError('value must of type InsertEffect5Parameters')

#-------------------------------------------------------------------------------
	@property
	def mfx1(self):
		'''MFX1 Effect Parameters'''
		return self._mfx1

	@mfx1.setter
	def mfx1(self,value):
		if isinstance(value,MasterEffect1Parameters):
			self._mfx1 = value
		elif isinstance(value,basestring):
			self._mfx1 = MasterEffect1Parameters(value)
		else:
			raise TypeError('value must of type MasterEffect1Parameters')

#-------------------------------------------------------------------------------
	@property
	def mfx2(self):
		'''MFX2 Effect Parameters'''
		return self._mfx2

	@mfx2.setter
	def mfx2(self,value):
		if isinstance(value,MasterEffect2Parameters):
			self._mfx2 = value
		elif isinstance(value,basestring):
			self._mfx2 = MasterEffect2Parameters(value)
		else:
			raise TypeError('value must of type MasterEffect2Parameters')

#-------------------------------------------------------------------------------
	@property
	def mfx1_return_level(self):
		'''MFX1 Return Level'''
		return self._mfx1_return_level

	@mfx1_return_level.setter
	def mfx1_return_level(self,value):
		if 0 <= value <= 127:
			self._mfx1_return_level = value
			self.notify_listeners(0x32,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def mfx2_return_level(self):
		'''MFX2 Return Level'''
		return self._mfx2_return_level

	@mfx2_return_level.setter
	def mfx2_return_level(self,value):
		if 0 <= value <= 127:
			self._mfx2_return_level = value
			self.notify_listeners(0x32,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_direction(self):
		'''MFX Chain Direction'''
		return self._mfx_chain_direction

	@mfx_chain_direction.setter
	def mfx_chain_direction(self,value):
		if 0 <= value <= 1:
			self._mfx_chain_direction = value
			self.notify_listeners(0x32,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def mfx_chain_direction_display(self):
		'''MFX Chain Direction display value'''
		return mfx_chain_direction_dict[self._mfx_chain_direction]

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_on_off(self):
		'''MFX Chain On/Off'''
		return self._mfx_chain_on_off

	@mfx_chain_on_off.setter
	def mfx_chain_on_off(self,value):
		if 0 <= value <= 1:
			self._mfx_chain_on_off = value
			self.notify_listeners(0x32,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def mfx_chain_on_off_display(self):
		'''MFX Chain On/Off display value'''
		return off_on_dict[self._mfx_chain_on_off]

#-------------------------------------------------------------------------------
	@property
	def mfx_chain_level(self):
		'''MFX Chain Level'''
		return self._mfx_chain_level

	@mfx_chain_level.setter
	def mfx_chain_level(self,value):
		if 0 <= value <= 127:
			self._mfx_chain_level = value
			self.notify_listeners(0x32,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def tfx(self):
		'''TFX Effect Parameters'''
		return self._tfx

	@tfx.setter
	def tfx(self,value):
		if isinstance(value,TotalEffectParameters):
			self._tfx = value
		elif isinstance(value,basestring):
			self._tfx = TotalEffectParameters(value)
		else:
			raise TypeError('value must of type TotalEffectParameters')

#-------------------------------------------------------------------------------
	@property
	def lfo1(self):
		'''LFO1 Parameters'''
		return self._lfo1

	@lfo1.setter
	def lfo1(self,value):
		if isinstance(value,LFO1Parameters):
			self._lfo1 = value
		elif isinstance(value,basestring):
			self._lfo1 = LFO1Parameters(value)
		else:
			raise TypeError('value must of type LFO1Parameters')

#-------------------------------------------------------------------------------
	@property
	def lfo2(self):
		'''LFO2 Parameters'''
		return self._lfo2

	@lfo2.setter
	def lfo2(self,value):
		if isinstance(value,LFO2Parameters):
			self._lfo2 = value
		elif isinstance(value,basestring):
			self._lfo2 = LFO2Parameters(value)
		else:
			raise TypeError('value must of type LFO2Parameters')

#-------------------------------------------------------------------------------
	@property
	def control_assign(self):
		'''Control Assign'''
		return self._control_assign

	@control_assign.setter
	def control_assign(self,value):
		if 0 <= value <= 2 or True: #FIXME
			self._control_assign = value
			self.notify_listeners(0x1,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def control_assign_display(self):
		'''Control Assign display value'''
		return control_assign_dict[self._control_assign]

#-------------------------------------------------------------------------------
	@property
	def selected_track(self):
		'''Selected Timbre'''
		return self._selected_track

	@selected_track.setter
	def selected_track(self,value):
		if 0 <= value <= 15:
			self._selected_track = value
			self.notify_listeners(0x1,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def drum(self):
		'''Drum Track Parameters'''
		return self._drum

	@drum.setter
	def drum(self,value):
		if isinstance(value,DrumTrackParameters):
			self._drum = value
		elif isinstance(value,basestring):
			self._drum = DrumTrackParameters(value)
		else:
			raise TypeError('value must of type DrumTrackParameters')

#-------------------------------------------------------------------------------
	@property
	def arp(self):
		'''Arpeggiator Parameters'''
		return self._arp

	@arp.setter
	def arp(self,value):
		if isinstance(value,ArpeggiatorParameters):
			self._arp = value
		elif isinstance(value,basestring):
			self._arp = ArpeggiatorParameters(value)
		else:
			raise TypeError('value must of type ArpeggiatorParameters')

#-------------------------------------------------------------------------------
	@property
	def arpa(self):
		'''Arpeggiator A Parameters'''
		return self._arpa

	@arpa.setter
	def arpa(self,value):
		if isinstance(value,ArpeggiatorAParameters):
			self._arpa = value
		elif isinstance(value,basestring):
			self._arpa = ArpeggiatorAParameters(value)
		else:
			raise TypeError('value must of type ArpeggiatorAParameters')

#-------------------------------------------------------------------------------
	@property
	def arpb(self):
		'''Arpeggiator B Parameters'''
		return self._arpb

	@arpb.setter
	def arpb(self,value):
		if isinstance(value,ArpeggiatorBParameters):
			self._arpb = value
		elif isinstance(value,basestring):
			self._arpb = ArpeggiatorBParameters(value)
		else:
			raise TypeError('value must of type ArpeggiatorBParameters')

#-------------------------------------------------------------------------------
	@property
	def common(self):
		'''Common Parameters'''
		return self._common

	@common.setter
	def common(self,value):
		if isinstance(value,CommonParameters):
			self._common = value
		elif isinstance(value,basestring):
			self._common = CommonParameters(value)
		else:
			raise TypeError('value must of type CommonParameters')

#-------------------------------------------------------------------------------
	@property
	def trk1(self):
		'''Track 1 Parameters'''
		return self._trk1

	@trk1.setter
	def trk1(self,value):
		if isinstance(value,Track1Parameters):
			self._trk1 = value
		elif isinstance(value,basestring):
			self._trk1 = Track1Parameters(value)
		else:
			raise TypeError('value must of type Track1Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk2(self):
		'''Track 2 Parameters'''
		return self._trk2

	@trk2.setter
	def trk2(self,value):
		if isinstance(value,Track2Parameters):
			self._trk2 = value
		elif isinstance(value,basestring):
			self._trk2 = Track2Parameters(value)
		else:
			raise TypeError('value must of type Track2Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk3(self):
		'''Track 3 Parameters'''
		return self._trk3

	@trk3.setter
	def trk3(self,value):
		if isinstance(value,Track3Parameters):
			self._trk3 = value
		elif isinstance(value,basestring):
			self._trk3 = Track3Parameters(value)
		else:
			raise TypeError('value must of type Track3Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk4(self):
		'''Track 4 Parameters'''
		return self._trk4

	@trk4.setter
	def trk4(self,value):
		if isinstance(value,Track4Parameters):
			self._trk4 = value
		elif isinstance(value,basestring):
			self._trk4 = Track4Parameters(value)
		else:
			raise TypeError('value must of type Track4Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk5(self):
		'''Track 5 Parameters'''
		return self._trk5

	@trk5.setter
	def trk5(self,value):
		if isinstance(value,Track5Parameters):
			self._trk5 = value
		elif isinstance(value,basestring):
			self._trk5 = Track5Parameters(value)
		else:
			raise TypeError('value must of type Track5Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk6(self):
		'''Track 6 Parameters'''
		return self._trk6

	@trk6.setter
	def trk6(self,value):
		if isinstance(value,Track6Parameters):
			self._trk6 = value
		elif isinstance(value,basestring):
			self._trk6 = Track6Parameters(value)
		else:
			raise TypeError('value must of type Track6Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk7(self):
		'''Track 7 Parameters'''
		return self._trk7

	@trk7.setter
	def trk7(self,value):
		if isinstance(value,Track7Parameters):
			self._trk7 = value
		elif isinstance(value,basestring):
			self._trk7 = Track7Parameters(value)
		else:
			raise TypeError('value must of type Track7Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk8(self):
		'''Track 8 Parameters'''
		return self._trk8

	@trk8.setter
	def trk8(self,value):
		if isinstance(value,Track8Parameters):
			self._trk8 = value
		elif isinstance(value,basestring):
			self._trk8 = Track8Parameters(value)
		else:
			raise TypeError('value must of type Track8Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk9(self):
		'''Track 9 Parameters'''
		return self._trk9

	@trk9.setter
	def trk9(self,value):
		if isinstance(value,Track9Parameters):
			self._trk9 = value
		elif isinstance(value,basestring):
			self._trk9 = Track9Parameters(value)
		else:
			raise TypeError('value must of type Track9Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk10(self):
		'''Track 10 Parameters'''
		return self._trk10

	@trk10.setter
	def trk10(self,value):
		if isinstance(value,Track10Parameters):
			self._trk10 = value
		elif isinstance(value,basestring):
			self._trk10 = Track10Parameters(value)
		else:
			raise TypeError('value must of type Track10Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk11(self):
		'''Track 11 Parameters'''
		return self._trk11

	@trk11.setter
	def trk11(self,value):
		if isinstance(value,Track11Parameters):
			self._trk11 = value
		elif isinstance(value,basestring):
			self._trk11 = Track11Parameters(value)
		else:
			raise TypeError('value must of type Track11Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk12(self):
		'''Track 12 Parameters'''
		return self._trk12

	@trk12.setter
	def trk12(self,value):
		if isinstance(value,Track12Parameters):
			self._trk12 = value
		elif isinstance(value,basestring):
			self._trk12 = Track12Parameters(value)
		else:
			raise TypeError('value must of type Track12Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk13(self):
		'''Track 13 Parameters'''
		return self._trk13

	@trk13.setter
	def trk13(self,value):
		if isinstance(value,Track13Parameters):
			self._trk13 = value
		elif isinstance(value,basestring):
			self._trk13 = Track13Parameters(value)
		else:
			raise TypeError('value must of type Track13Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk14(self):
		'''Track 14 Parameters'''
		return self._trk14

	@trk14.setter
	def trk14(self,value):
		if isinstance(value,Track14Parameters):
			self._trk14 = value
		elif isinstance(value,basestring):
			self._trk14 = Track14Parameters(value)
		else:
			raise TypeError('value must of type Track14Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk15(self):
		'''Track 15 Parameters'''
		return self._trk15

	@trk15.setter
	def trk15(self,value):
		if isinstance(value,Track15Parameters):
			self._trk15 = value
		elif isinstance(value,basestring):
			self._trk15 = Track15Parameters(value)
		else:
			raise TypeError('value must of type Track15Parameters')

#-------------------------------------------------------------------------------
	@property
	def trk16(self):
		'''Track 16 Parameters'''
		return self._trk16

	@trk16.setter
	def trk16(self,value):
		if isinstance(value,Track16Parameters):
			self._trk16 = value
		elif isinstance(value,basestring):
			self._trk16 = Track16Parameters(value)
		else:
			raise TypeError('value must of type Track16Parameters')

#===============================================================================
class InsertEffect1Parameters(TableBase):
	description = '''Insert Effect 1 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a InsertEffect1Parameters Insert Effect 1 Parameters object'''
		v = list(struct.unpack('<64s12B',raw[:76]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte89 = v.pop(0)
		self.channel = byte89 & 0x3F
		self.on_off = (byte89 >> 6) & 0x1
		self.chain = (byte89 >> 7) & 0x1
		self.chain_to = v.pop(0)
		self._reserved0091 = v.pop(0)
		self.pan = v.pop(0)
		byte93 = v.pop(0)
		self.bus_select = byte93 & 0xF
		self.fx_control = (byte93 >> 4) & 0x3
		self._reserved0093_7 = (byte93 >> 6) & 0x3
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved0096 = v.pop(0)
		self._reserved0097 = v.pop(0)
		self._reserved0098 = v.pop(0)
		self._reserved0099 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffect1Parameters Insert Effect 1 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte89 = 0
		byte89 |= self.channel & 0x3F
		byte89 |= (self.on_off & 0x1) << 6
		byte89 |= (self.chain & 0x1) << 7
		v.append(byte89)
		v.append(self.chain_to)
		v.append(self._reserved0091)
		v.append(self.pan)
		byte93 = 0
		byte93 |= self.bus_select & 0xF
		byte93 |= (self.fx_control & 0x3) << 4
		byte93 |= (self._reserved0093_7 & 0x3) << 6
		v.append(byte93)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved0096)
		v.append(self._reserved0097)
		v.append(self._reserved0098)
		v.append(self._reserved0099)
		return struct.pack('<64s12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffect1Parameters Insert Effect 1 Parameters object'''
		json = []
		json.append(('name','InsertEffect1Parameters'))
		json.append(('description','Insert Effect 1 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['channel',self.channel])
		json.append(['channel_display',self.channel_display])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['chain',self.chain])
		json.append(['chain_display',self.chain_display])
		json.append(['chain_to',self.chain_to])
		json.append(['chain_to_display',self.chain_to_display])
		json.append(['pan',self.pan])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_control',self.fx_control])
		json.append(['fx_control_display',self.fx_control_display])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''IFX1 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''IFX1 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
			self.notify_listeners(0x2C,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 193' % value)

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''IFX1 Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		if 0 <= value <= 17:
			self._channel = value
			self.notify_listeners(0x2C,0x2F,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

	@property
	def channel_display(self):
		'''IFX1 Channel display value'''
		return ifx_channel_dict[self._channel]

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''IFX1 Off/On'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x2C,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''IFX1 Off/On display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def chain(self):
		'''IFX1 Chain'''
		return self._chain

	@chain.setter
	def chain(self,value):
		if 0 <= value <= 1:
			self._chain = value
			self.notify_listeners(0x2C,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def chain_display(self):
		'''IFX1 Chain display value'''
		return off_on_dict[self._chain]

#-------------------------------------------------------------------------------
	@property
	def chain_to(self):
		'''IFX1 Chain To'''
		return self._chain_to

	@chain_to.setter
	def chain_to(self,value):
		if 2 <= value <= 5:
			self._chain_to = value
			self.notify_listeners(0x2C,0x2C,value)
		else:
			raise ValueError('%s not 2 <= value <= 5' % value)

	@property
	def chain_to_display(self):
		'''IFX1 Chain To display value'''
		return chain1_to_dict[self._chain_to]

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''IFX1 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x2C,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''IFX1 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 7):
			self._bus_select = value
			self.notify_listeners(0x2C,0xF,value)
		else:
			raise ValueError('%s value not in (0, 7)' % value)

	@property
	def bus_select_display(self):
		'''IFX1 Bus Select display value'''
		return bus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_control(self):
		'''IFX1 FX Control'''
		return self._fx_control

	@fx_control.setter
	def fx_control(self,value):
		if 0 <= value <= 2:
			self._fx_control = value
			self.notify_listeners(0x2C,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_control_display(self):
		'''IFX1 FX Control display value'''
		return fx_control_dict[self._fx_control]

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''IFX1 Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
			self.notify_listeners(0x2C,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''IFX1 Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
			self.notify_listeners(0x2C,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class InsertEffect2Parameters(TableBase):
	description = '''Insert Effect 2 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a InsertEffect2Parameters Insert Effect 2 Parameters object'''
		v = list(struct.unpack('<64s12B',raw[:76]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte165 = v.pop(0)
		self.channel = byte165 & 0x3F
		self.on_off = (byte165 >> 6) & 0x1
		self.chain = (byte165 >> 7) & 0x1
		self.chain_to = v.pop(0)
		self._reserved0167 = v.pop(0)
		self.pan = v.pop(0)
		byte169 = v.pop(0)
		self.bus_select = byte169 & 0xF
		self.fx_control = (byte169 >> 4) & 0x3
		self._reserved0169_7 = (byte169 >> 6) & 0x3
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved0172 = v.pop(0)
		self._reserved0173 = v.pop(0)
		self._reserved0174 = v.pop(0)
		self._reserved0175 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffect2Parameters Insert Effect 2 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte165 = 0
		byte165 |= self.channel & 0x3F
		byte165 |= (self.on_off & 0x1) << 6
		byte165 |= (self.chain & 0x1) << 7
		v.append(byte165)
		v.append(self.chain_to)
		v.append(self._reserved0167)
		v.append(self.pan)
		byte169 = 0
		byte169 |= self.bus_select & 0xF
		byte169 |= (self.fx_control & 0x3) << 4
		byte169 |= (self._reserved0169_7 & 0x3) << 6
		v.append(byte169)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved0172)
		v.append(self._reserved0173)
		v.append(self._reserved0174)
		v.append(self._reserved0175)
		return struct.pack('<64s12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffect2Parameters Insert Effect 2 Parameters object'''
		json = []
		json.append(('name','InsertEffect2Parameters'))
		json.append(('description','Insert Effect 2 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['channel',self.channel])
		json.append(['channel_display',self.channel_display])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['chain',self.chain])
		json.append(['chain_display',self.chain_display])
		json.append(['chain_to',self.chain_to])
		json.append(['chain_to_display',self.chain_to_display])
		json.append(['pan',self.pan])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_control',self.fx_control])
		json.append(['fx_control_display',self.fx_control_display])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''IFX2 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''IFX2 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
			self.notify_listeners(0x2C,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 193' % value)

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''IFX2 Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		if 0 <= value <= 17:
			self._channel = value
			self.notify_listeners(0x2C,0x30,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

	@property
	def channel_display(self):
		'''IFX2 Channel display value'''
		return ifx_channel_dict[self._channel]

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''IFX2 Off/On'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x2C,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''IFX2 Off/On display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def chain(self):
		'''IFX2 Chain'''
		return self._chain

	@chain.setter
	def chain(self,value):
		if 0 <= value <= 1:
			self._chain = value
			self.notify_listeners(0x2C,0x29,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def chain_display(self):
		'''IFX2 Chain display value'''
		return off_on_dict[self._chain]

#-------------------------------------------------------------------------------
	@property
	def chain_to(self):
		'''IFX2 Chain To'''
		return self._chain_to

	@chain_to.setter
	def chain_to(self,value):
		if 3 <= value <= 5:
			self._chain_to = value
			self.notify_listeners(0x2C,0x2D,value)
		else:
			raise ValueError('%s 3 <= value <= 5' % value)

	@property
	def chain_to_display(self):
		'''IFX2 Chain To display value'''
		return chain2_to_dict[self._chain_to]

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''IFX2 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x2C,0xB,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''IFX2 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 7):
			self._bus_select = value
			self.notify_listeners(0x2C,0x10,value)
		else:
			raise ValueError('%s value not in (0, 7)' % value)

	@property
	def bus_select_display(self):
		'''IFX2 Bus Select display value'''
		return bus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_control(self):
		'''IFX2 FX Control'''
		return self._fx_control

	@fx_control.setter
	def fx_control(self,value):
		if 0 <= value <= 2:
			self._fx_control = value
			self.notify_listeners(0x2C,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_control_display(self):
		'''IFX2 FX Control display value'''
		return fx_control_dict[self._fx_control]

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''IFX2 Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
			self.notify_listeners(0x2C,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''IFX2 Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
			self.notify_listeners(0x2C,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class InsertEffect3Parameters(TableBase):
	description = '''Insert Effect 3 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a InsertEffect3Parameters Insert Effect 3 Parameters object'''
		v = list(struct.unpack('<64s12B',raw[:76]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte241 = v.pop(0)
		self.channel = byte241 & 0x3F
		self.on_off = (byte241 >> 6) & 0x1
		self.chain = (byte241 >> 7) & 0x1
		self.chain_to = v.pop(0)
		self._reserved0243 = v.pop(0)
		self.pan = v.pop(0)
		byte245 = v.pop(0)
		self.bus_select = byte245 & 0xF
		self.fx_control = (byte245 >> 4) & 0x3
		self._reserved0245_7 = (byte245 >> 6) & 0x3
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved0248 = v.pop(0)
		self._reserved0249 = v.pop(0)
		self._reserved0250 = v.pop(0)
		self._reserved0251 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffect3Parameters Insert Effect 3 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte241 = 0
		byte241 |= self.channel & 0x3F
		byte241 |= (self.on_off & 0x1) << 6
		byte241 |= (self.chain & 0x1) << 7
		v.append(byte241)
		v.append(self.chain_to)
		v.append(self._reserved0243)
		v.append(self.pan)
		byte245 = 0
		byte245 |= self.bus_select & 0xF
		byte245 |= (self.fx_control & 0x3) << 4
		byte245 |= (self._reserved0245_7 & 0x3) << 6
		v.append(byte245)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved0248)
		v.append(self._reserved0249)
		v.append(self._reserved0250)
		v.append(self._reserved0251)
		return struct.pack('<64s12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffect3Parameters Insert Effect 3 Parameters object'''
		json = []
		json.append(('name','InsertEffect3Parameters'))
		json.append(('description','Insert Effect 3 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['channel',self.channel])
		json.append(['channel_display',self.channel_display])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['chain',self.chain])
		json.append(['chain_display',self.chain_display])
		json.append(['chain_to',self.chain_to])
		json.append(['chain_to_display',self.chain_to_display])
		json.append(['pan',self.pan])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_control',self.fx_control])
		json.append(['fx_control_display',self.fx_control_display])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''IFX3 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''IFX3 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
			self.notify_listeners(0x2C,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 193' % value)

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''IFX3 Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		if 0 <= value <= 17:
			self._channel = value
			self.notify_listeners(0x2C,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

	@property
	def channel_display(self):
		'''IFX3 Channel display value'''
		return ifx_channel_dict[self._channel]

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''IFX3 Off/On'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x2C,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''IFX3 Off/On display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def chain(self):
		'''IFX3 Chain'''
		return self._chain

	@chain.setter
	def chain(self,value):
		if 0 <= value <= 1:
			self._chain = value
			self.notify_listeners(0x2C,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def chain_display(self):
		'''IFX3 Chain display value'''
		return off_on_dict[self._chain]

#-------------------------------------------------------------------------------
	@property
	def chain_to(self):
		'''IFX3 Chain To'''
		return self._chain_to

	@chain_to.setter
	def chain_to(self,value):
		if 5 <= value <= 5 or True: #FIXME
			self._chain_to = value
			self.notify_listeners(0x2C,0x2E,value)
		else:
			raise ValueError('%s not 4 <= value <= 4' % value)

	@property
	def chain_to_display(self):
		'''IFX3 Chain To display value'''
		return chain3_to_dict[self._chain_to]

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''IFX3 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x2C,0xC,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''IFX3 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 7):
			self._bus_select = value
			self.notify_listeners(0x2C,0x11,value)
		else:
			raise ValueError('%s value not in (0, 7)' % value)

	@property
	def bus_select_display(self):
		'''IFX3 Bus Select display value'''
		return bus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_control(self):
		'''IFX3 FX Control'''
		return self._fx_control

	@fx_control.setter
	def fx_control(self,value):
		if 0 <= value <= 2:
			self._fx_control = value
			self.notify_listeners(0x2C,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_control_display(self):
		'''IFX3 FX Control display value'''
		return fx_control_dict[self._fx_control]

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''IFX3 Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
			self.notify_listeners(0x2C,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''IFX3 Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
			self.notify_listeners(0x2C,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class InsertEffect4Parameters(TableBase):
	description = '''Insert Effect 4 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a InsertEffect4Parameters Insert Effect 4 Parameters object'''
		v = list(struct.unpack('<64s12B',raw[:76]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte317 = v.pop(0)
		self.channel = byte317 & 0x3F
		self.on_off = (byte317 >> 6) & 0x1
		self.chain = (byte317 >> 7) & 0x1
		self._reserved0318 = v.pop(0)
		self._reserved0319 = v.pop(0)
		self.pan = v.pop(0)
		byte321 = v.pop(0)
		self.bus_select = byte321 & 0xF
		self.fx_control = (byte321 >> 4) & 0x3
		self._reserved0321_7 = (byte321 >> 6) & 0x3
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved0324 = v.pop(0)
		self._reserved0325 = v.pop(0)
		self._reserved0326 = v.pop(0)
		self._reserved0327 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffect4Parameters Insert Effect 4 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte317 = 0
		byte317 |= self.channel & 0x3F
		byte317 |= (self.on_off & 0x1) << 6
		byte317 |= (self.chain & 0x1) << 7
		v.append(byte317)
		v.append(self._reserved0318)
		v.append(self._reserved0319)
		v.append(self.pan)
		byte321 = 0
		byte321 |= self.bus_select & 0xF
		byte321 |= (self.fx_control & 0x3) << 4
		byte321 |= (self._reserved0321_7 & 0x3) << 6
		v.append(byte321)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved0324)
		v.append(self._reserved0325)
		v.append(self._reserved0326)
		v.append(self._reserved0327)
		return struct.pack('<64s12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffect4Parameters Insert Effect 4 Parameters object'''
		json = []
		json.append(('name','InsertEffect4Parameters'))
		json.append(('description','Insert Effect 4 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['channel',self.channel])
		json.append(['channel_display',self.channel_display])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['chain',self.chain])
		json.append(['chain_display',self.chain_display])
		json.append(['pan',self.pan])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_control',self.fx_control])
		json.append(['fx_control_display',self.fx_control_display])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''IFX4 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''IFX4 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
			self.notify_listeners(0x2C,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 193' % value)

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''IFX4 Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		if 0 <= value <= 17:
			self._channel = value
			self.notify_listeners(0x2C,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

	@property
	def channel_display(self):
		'''IFX4 Channel display value'''
		return ifx_channel_dict[self._channel]

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''IFX4 Off/On'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x2C,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''IFX4 Off/On display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def chain(self):
		'''IFX4 Chain'''
		return self._chain

	@chain.setter
	def chain(self,value):
		if 0 <= value <= 1:
			self._chain = value
			self.notify_listeners(0x2C,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def chain_display(self):
		'''IFX4 Chain display value'''
		return off_on_dict[self._chain]

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''IFX4 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x2C,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''IFX4 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 7):
			self._bus_select = value
			self.notify_listeners(0x2C,0x12,value)
		else:
			raise ValueError('%s value not in (0, 7)' % value)

	@property
	def bus_select_display(self):
		'''IFX4 Bus Select display value'''
		return bus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_control(self):
		'''IFX4 FX Control'''
		return self._fx_control

	@fx_control.setter
	def fx_control(self,value):
		if 0 <= value <= 2:
			self._fx_control = value
			self.notify_listeners(0x2C,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_control_display(self):
		'''IFX4 FX Control display value'''
		return fx_control_dict[self._fx_control]

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''IFX4 Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
			self.notify_listeners(0x2C,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''IFX4 Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
			self.notify_listeners(0x2C,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class InsertEffect5Parameters(TableBase):
	description = '''Insert Effect 5 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a InsertEffect5Parameters Insert Effect 5 Parameters object'''
		v = list(struct.unpack('<64s12B',raw[:76]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte393 = v.pop(0)
		self.channel = byte393 & 0x3F
		self.on_off = (byte393 >> 6) & 0x1
		self._reserved0393_7 = (byte393 >> 7) & 0x1
		self._reserved0394 = v.pop(0)
		self._reserved0395 = v.pop(0)
		self.pan = v.pop(0)
		byte397 = v.pop(0)
		self.bus_select = byte397 & 0xF
		self.fx_control = (byte397 >> 4) & 0x3
		self._reserved0397_7 = (byte397 >> 6) & 0x3
		self.send1level = v.pop(0)
		self.send2level = v.pop(0)
		self._reserved0400 = v.pop(0)
		self._reserved0401 = v.pop(0)
		self._reserved0402 = v.pop(0)
		self._reserved0403 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a InsertEffect5Parameters Insert Effect 5 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte393 = 0
		byte393 |= self.channel & 0x3F
		byte393 |= (self.on_off & 0x1) << 6
		byte393 |= (self._reserved0393_7 & 0x1) << 7
		v.append(byte393)
		v.append(self._reserved0394)
		v.append(self._reserved0395)
		v.append(self.pan)
		byte397 = 0
		byte397 |= self.bus_select & 0xF
		byte397 |= (self.fx_control & 0x3) << 4
		byte397 |= (self._reserved0397_7 & 0x3) << 6
		v.append(byte397)
		v.append(self.send1level)
		v.append(self.send2level)
		v.append(self._reserved0400)
		v.append(self._reserved0401)
		v.append(self._reserved0402)
		v.append(self._reserved0403)
		return struct.pack('<64s12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a InsertEffect5Parameters Insert Effect 5 Parameters object'''
		json = []
		json.append(('name','InsertEffect5Parameters'))
		json.append(('description','Insert Effect 5 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['channel',self.channel])
		json.append(['channel_display',self.channel_display])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['pan',self.pan])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_control',self.fx_control])
		json.append(['fx_control_display',self.fx_control_display])
		json.append(['send1level',self.send1level])
		json.append(['send2level',self.send2level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''IFX5 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''IFX5 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 171:
			self._effect_type = value
			self.notify_listeners(0x2C,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 171' % value)

#-------------------------------------------------------------------------------
	@property
	def channel(self):
		'''IFX5 Channel'''
		return self._channel

	@channel.setter
	def channel(self,value):
		if 0 <= value <= 17:
			self._channel = value
			self.notify_listeners(0x2C,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

	@property
	def channel_display(self):
		'''IFX5 Channel display value'''
		return ifx_channel_dict[self._channel]

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''IFX5 Off/On'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x2C,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''IFX5 Off/On display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''IFX5 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x2C,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''IFX5 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 7):
			self._bus_select = value
			self.notify_listeners(0x2C,0x13,value)
		else:
			raise ValueError('%s value not in (0, 7)' % value)

	@property
	def bus_select_display(self):
		'''IFX5 Bus Select display value'''
		return bus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_control(self):
		'''IFX5 FX Control'''
		return self._fx_control

	@fx_control.setter
	def fx_control(self,value):
		if 0 <= value <= 2:
			self._fx_control = value
			self.notify_listeners(0x2C,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_control_display(self):
		'''IFX5 FX Control display value'''
		return fx_control_dict[self._fx_control]

#-------------------------------------------------------------------------------
	@property
	def send1level(self):
		'''IFX5 Send 1 Level'''
		return self._send1level

	@send1level.setter
	def send1level(self,value):
		if 0 <= value <= 127:
			self._send1level = value
			self.notify_listeners(0x2C,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2level(self):
		'''IFX5 Send 2 Level'''
		return self._send2level

	@send2level.setter
	def send2level(self,value):
		if 0 <= value <= 127:
			self._send2level = value
			self.notify_listeners(0x2C,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class MasterEffect1Parameters(TableBase):
	description = '''Master Effect 1 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a MasterEffect1Parameters Master Effect 1 Parameters object'''
		v = list(struct.unpack('<64s4B',raw[:68]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte469 = v.pop(0)
		self._reserved0469_5 = byte469 & 0x3F
		self.on_off = (byte469 >> 6) & 0x1
		self._reserved0469_7 = (byte469 >> 7) & 0x1
		self._reserved0470 = v.pop(0)
		self._reserved0471 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a MasterEffect1Parameters Master Effect 1 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte469 = 0
		byte469 |= self._reserved0469_5 & 0x3F
		byte469 |= (self.on_off & 0x1) << 6
		byte469 |= (self._reserved0469_7 & 0x1) << 7
		v.append(byte469)
		v.append(self._reserved0470)
		v.append(self._reserved0471)
		return struct.pack('<64s4B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a MasterEffect1Parameters Master Effect 1 Parameters object'''
		json = []
		json.append(('name','MasterEffect1Parameters'))
		json.append(('description','Master Effect 1 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''MFX1 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''MFX1 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 193:
			self._effect_type = value
			self.notify_listeners(0x32,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 193' % value)

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''MFX1 On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x32,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''MFX1 On/Off display value'''
		return off_on_dict[self._on_off]

#===============================================================================
class MasterEffect2Parameters(TableBase):
	description = '''Master Effect 2 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a MasterEffect2Parameters Master Effect 2 Parameters object'''
		v = list(struct.unpack('<64s4B',raw[:68]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte537 = v.pop(0)
		self._reserved0537_5 = byte537 & 0x3F
		self.on_off = (byte537 >> 6) & 0x1
		self._reserved0537_7 = (byte537 >> 7) & 0x1
		self._reserved0538 = v.pop(0)
		self._reserved0539 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a MasterEffect2Parameters Master Effect 2 Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte537 = 0
		byte537 |= self._reserved0537_5 & 0x3F
		byte537 |= (self.on_off & 0x1) << 6
		byte537 |= (self._reserved0537_7 & 0x1) << 7
		v.append(byte537)
		v.append(self._reserved0538)
		v.append(self._reserved0539)
		return struct.pack('<64s4B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a MasterEffect2Parameters Master Effect 2 Parameters object'''
		json = []
		json.append(('name','MasterEffect2Parameters'))
		json.append(('description','Master Effect 2 Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''MFX2 Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''MFX2 Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 171:
			self._effect_type = value
			self.notify_listeners(0x32,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 171' % value)

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''MFX2 On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x32,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''MFX2 On/Off display value'''
		return off_on_dict[self._on_off]

#===============================================================================
class TotalEffectParameters(TableBase):
	description = '''Total Effect Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a TotalEffectParameters Total Effect Parameters object'''
		v = list(struct.unpack('<64s8B',raw[:72]))
		self.param_struct = v.pop(0)
		self.effect_type = v.pop(0)
		byte609 = v.pop(0)
		self._reserved0609_5 = byte609 & 0x3F
		self.on_off = (byte609 >> 6) & 0x1
		self._reserved0609_7 = (byte609 >> 7) & 0x1
		self._reserved0610 = v.pop(0)
		self._reserved0611 = v.pop(0)
		self.master_volume = v.pop(0)
		self._reserved0613 = v.pop(0)
		self._reserved0614 = v.pop(0)
		self._reserved0615 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a TotalEffectParameters Total Effect Parameters object into raw data'''
		v = []
		v.append(self.effect_type)
		byte609 = 0
		byte609 |= self._reserved0609_5 & 0x3F
		byte609 |= (self.on_off & 0x1) << 6
		byte609 |= (self._reserved0609_7 & 0x1) << 7
		v.append(byte609)
		v.append(self._reserved0610)
		v.append(self._reserved0611)
		v.append(self.master_volume)
		v.append(self._reserved0613)
		v.append(self._reserved0614)
		v.append(self._reserved0615)
		return struct.pack('<64s8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a TotalEffectParameters Total Effect Parameters object'''
		json = []
		json.append(('name','TotalEffectParameters'))
		json.append(('description','Total Effect Parameters'))
		json.append(['param_struct',self.param_struct])
		json.append(['effect_type',self.effect_type])
		json.append(['on_off',self.on_off])
		json.append(['on_off_display',self.on_off_display])
		json.append(['master_volume',self.master_volume])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def param_struct(self):
		'''TFX Effect Parameter Structure'''
		return self._param_struct

	@param_struct.setter
	def param_struct(self,value):
		self._param_struct = value

#-------------------------------------------------------------------------------
	@property
	def effect_type(self):
		'''TFX Effect Type'''
		return self._effect_type

	@effect_type.setter
	def effect_type(self,value):
		if 0 <= value <= 171:
			self._effect_type = value
			self.notify_listeners(0x35,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 171' % value)

#-------------------------------------------------------------------------------
	@property
	def on_off(self):
		'''TFX On/Off'''
		return self._on_off

	@on_off.setter
	def on_off(self,value):
		if 0 <= value <= 1:
			self._on_off = value
			self.notify_listeners(0x35,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def on_off_display(self):
		'''TFX On/Off display value'''
		return off_on_dict[self._on_off]

#-------------------------------------------------------------------------------
	@property
	def master_volume(self):
		'''TFX Master Level'''
		return self._master_volume

	@master_volume.setter
	def master_volume(self,value):
		if 0 <= value <= 127:
			self._master_volume = value
			self.notify_listeners(0x35,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#===============================================================================
class LFO1Parameters(TableBase):
	description = '''LFO1 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a LFO1Parameters LFO1 Parameters object'''
		v = list(struct.unpack('<4BH2B',raw[:8]))
		self.frequency = v.pop(0)
		self.flag = v.pop(0)
		self.dmod_src = v.pop(0)
		self.midi_channel = v.pop(0)
		self.bpm = v.pop(0)
		byte622 = v.pop(0)
		self.midi_sync_base_note = byte622 & 0xF
		self._reserved0622_6 = (byte622 >> 4) & 0x7
		self.midi_sync_off_on = (byte622 >> 7) & 0x1
		self.midi_sync_time = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a LFO1Parameters LFO1 Parameters object into raw data'''
		v = []
		v.append(self.frequency)
		v.append(self.flag)
		v.append(self.dmod_src)
		v.append(self.midi_channel)
		v.append(self.bpm)
		byte622 = 0
		byte622 |= self.midi_sync_base_note & 0xF
		byte622 |= (self._reserved0622_6 & 0x7) << 4
		byte622 |= (self.midi_sync_off_on & 0x1) << 7
		v.append(byte622)
		v.append(self.midi_sync_time)
		return struct.pack('<4BH2B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a LFO1Parameters LFO1 Parameters object'''
		json = []
		json.append(('name','LFO1Parameters'))
		json.append(('description','LFO1 Parameters'))
		json.append(['frequency',self.frequency])
		json.append(['flag',self.flag])
		json.append(['flag_display',self.flag_display])
		json.append(['dmod_src',self.dmod_src])
		json.append(['midi_channel',self.midi_channel])
		json.append(['bpm',self.bpm])
		json.append(['midi_sync_base_note',self.midi_sync_base_note])
		json.append(['midi_sync_base_note_display',self.midi_sync_base_note_display])
		json.append(['midi_sync_off_on',self.midi_sync_off_on])
		json.append(['midi_sync_off_on_display',self.midi_sync_off_on_display])
		json.append(['midi_sync_time',self.midi_sync_time])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def frequency(self):
		'''LFO1 Frequency'''
		return self._frequency

	@frequency.setter
	def frequency(self,value):
		if 0 <= value <= 230:
			self._frequency = value
			self.notify_listeners(0x6,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def flag(self):
		'''LFO1 Sync'''
		return self._flag

	@flag.setter
	def flag(self,value):
		if 0 <= value <= 1 or True: #FIXME
			self._flag = value
			self.notify_listeners(0x6,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def flag_display(self):
		'''LFO1 Sync display value'''
		return off_on_dict[self._flag]

#-------------------------------------------------------------------------------
	@property
	def dmod_src(self):
		'''LFO1 DMod Src'''
		return self._dmod_src

	@dmod_src.setter
	def dmod_src(self,value):
		if 0 <= value <= 37:
			self._dmod_src = value
			self.notify_listeners(0x6,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''LFO1 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x6,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''LFO1 BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
			self.notify_listeners(0x6,0x4,value)
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_sync_base_note(self):
		'''LFO1 MIDI Sync Base Note'''
		return self._midi_sync_base_note

	@midi_sync_base_note.setter
	def midi_sync_base_note(self,value):
		if 0 <= value <= 9:
			self._midi_sync_base_note = value
			self.notify_listeners(0x6,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def midi_sync_base_note_display(self):
		'''LFO1 MIDI Sync Base Note display value'''
		return base_note_dict[self._midi_sync_base_note]

#-------------------------------------------------------------------------------
	@property
	def midi_sync_off_on(self):
		'''LFO1 MIDI Sync'''
		return self._midi_sync_off_on

	@midi_sync_off_on.setter
	def midi_sync_off_on(self,value):
		if 0 <= value <= 1:
			self._midi_sync_off_on = value
			self.notify_listeners(0x6,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def midi_sync_off_on_display(self):
		'''LFO1 MIDI Sync display value'''
		return off_on_dict[self._midi_sync_off_on]

#-------------------------------------------------------------------------------
	@property
	def midi_sync_time(self):
		'''LFO1 MIDI Sync Time'''
		return self._midi_sync_time

	@midi_sync_time.setter
	def midi_sync_time(self,value):
		if 1 <= value <= 32:
			self._midi_sync_time = value
			self.notify_listeners(0x6,0x7,value)
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#===============================================================================
class LFO2Parameters(TableBase):
	description = '''LFO2 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a LFO2Parameters LFO2 Parameters object'''
		v = list(struct.unpack('<4BH2B',raw[:8]))
		self.frequency = v.pop(0)
		self.flag = v.pop(0)
		self.dmod_src = v.pop(0)
		self.midi_channel = v.pop(0)
		self.bpm = v.pop(0)
		byte630 = v.pop(0)
		self.midi_sync_base_note = byte630 & 0xF
		self._reserved0630_6 = (byte630 >> 4) & 0x7
		self.midi_sync_off_on = (byte630 >> 7) & 0x1
		self.midi_sync_time = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a LFO2Parameters LFO2 Parameters object into raw data'''
		v = []
		v.append(self.frequency)
		v.append(self.flag)
		v.append(self.dmod_src)
		v.append(self.midi_channel)
		v.append(self.bpm)
		byte630 = 0
		byte630 |= self.midi_sync_base_note & 0xF
		byte630 |= (self._reserved0630_6 & 0x7) << 4
		byte630 |= (self.midi_sync_off_on & 0x1) << 7
		v.append(byte630)
		v.append(self.midi_sync_time)
		return struct.pack('<4BH2B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a LFO2Parameters LFO2 Parameters object'''
		json = []
		json.append(('name','LFO2Parameters'))
		json.append(('description','LFO2 Parameters'))
		json.append(['frequency',self.frequency])
		json.append(['flag',self.flag])
		json.append(['flag_display',self.flag_display])
		json.append(['dmod_src',self.dmod_src])
		json.append(['midi_channel',self.midi_channel])
		json.append(['bpm',self.bpm])
		json.append(['midi_sync_base_note',self.midi_sync_base_note])
		json.append(['midi_sync_base_note_display',self.midi_sync_base_note_display])
		json.append(['midi_sync_off_on',self.midi_sync_off_on])
		json.append(['midi_sync_off_on_display',self.midi_sync_off_on_display])
		json.append(['midi_sync_time',self.midi_sync_time])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def frequency(self):
		'''LFO2 Frequency'''
		return self._frequency

	@frequency.setter
	def frequency(self,value):
		if 0 <= value <= 230:
			self._frequency = value
			self.notify_listeners(0x7,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def flag(self):
		'''LFO2 Sync'''
		return self._flag

	@flag.setter
	def flag(self,value):
		if 0 <= value <= 1 or True: #FIXME
			self._flag = value
			self.notify_listeners(0x7,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def flag_display(self):
		'''LFO2 Sync display value'''
		return off_on_dict[self._flag]

#-------------------------------------------------------------------------------
	@property
	def dmod_src(self):
		'''LFO2 DMod Src'''
		return self._dmod_src

	@dmod_src.setter
	def dmod_src(self,value):
		if 0 <= value <= 37:
			self._dmod_src = value
			self.notify_listeners(0x7,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''LFO2 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x7,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''LFO2 BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
			self.notify_listeners(0x7,0x4,value)
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_sync_base_note(self):
		'''LFO2 MIDI Sync Base Note'''
		return self._midi_sync_base_note

	@midi_sync_base_note.setter
	def midi_sync_base_note(self,value):
		if 0 <= value <= 9:
			self._midi_sync_base_note = value
			self.notify_listeners(0x7,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def midi_sync_base_note_display(self):
		'''LFO2 MIDI Sync Base Note display value'''
		return base_note_dict[self._midi_sync_base_note]

#-------------------------------------------------------------------------------
	@property
	def midi_sync_off_on(self):
		'''LFO2 MIDI Sync'''
		return self._midi_sync_off_on

	@midi_sync_off_on.setter
	def midi_sync_off_on(self,value):
		if 0 <= value <= 1:
			self._midi_sync_off_on = value
			self.notify_listeners(0x7,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def midi_sync_off_on_display(self):
		'''LFO2 MIDI Sync display value'''
		return off_on_dict[self._midi_sync_off_on]

#-------------------------------------------------------------------------------
	@property
	def midi_sync_time(self):
		'''LFO2 MIDI Sync Time'''
		return self._midi_sync_time

	@midi_sync_time.setter
	def midi_sync_time(self,value):
		if 1 <= value <= 32:
			self._midi_sync_time = value
			self.notify_listeners(0x7,0x7,value)
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#===============================================================================
class DrumTrackParameters(TableBase):
	description = '''Drum Track Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a DrumTrackParameters Drum Track Parameters object'''
		v = list(struct.unpack('<H6Bb3B',raw[:12]))
		self.pattern_number = v.pop(0)
		self.pattern_bank = v.pop(0)
		byte639 = v.pop(0)
		self.button_mode = byte639 & 0x1
		self.latch_mode = (byte639 >> 1) & 0x3
		self.trigger_sync = (byte639 >> 3) & 0x1
		self.drum_track_on = (byte639 >> 4) & 0x1
		self._reserved0639_7 = (byte639 >> 5) & 0x7
		self.trigger_bottom_key = v.pop(0)
		self.trigger_top_key = v.pop(0)
		self.velocity_bottom = v.pop(0)
		self.velocity_top = v.pop(0)
		self.shift = v.pop(0)
		self.output_channel = v.pop(0)
		self.input_channel = v.pop(0)
		self._reserved0647 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a DrumTrackParameters Drum Track Parameters object into raw data'''
		v = []
		v.append(self.pattern_number)
		v.append(self.pattern_bank)
		byte639 = 0
		byte639 |= self.button_mode & 0x1
		byte639 |= (self.latch_mode & 0x3) << 1
		byte639 |= (self.trigger_sync & 0x1) << 3
		byte639 |= (self.drum_track_on & 0x1) << 4
		byte639 |= (self._reserved0639_7 & 0x7) << 5
		v.append(byte639)
		v.append(self.trigger_bottom_key)
		v.append(self.trigger_top_key)
		v.append(self.velocity_bottom)
		v.append(self.velocity_top)
		v.append(self.shift)
		v.append(self.output_channel)
		v.append(self.input_channel)
		v.append(self._reserved0647)
		return struct.pack('<H6Bb3B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a DrumTrackParameters Drum Track Parameters object'''
		json = []
		json.append(('name','DrumTrackParameters'))
		json.append(('description','Drum Track Parameters'))
		json.append(['pattern_number',self.pattern_number])
		json.append(['pattern_bank',self.pattern_bank])
		json.append(['pattern_bank_display',self.pattern_bank_display])
		json.append(['button_mode',self.button_mode])
		json.append(['button_mode_display',self.button_mode_display])
		json.append(['latch_mode',self.latch_mode])
		json.append(['latch_mode_display',self.latch_mode_display])
		json.append(['trigger_sync',self.trigger_sync])
		json.append(['trigger_sync_display',self.trigger_sync_display])
		json.append(['drum_track_on',self.drum_track_on])
		json.append(['drum_track_on_display',self.drum_track_on_display])
		json.append(['trigger_bottom_key',self.trigger_bottom_key])
		json.append(['trigger_top_key',self.trigger_top_key])
		json.append(['velocity_bottom',self.velocity_bottom])
		json.append(['velocity_top',self.velocity_top])
		json.append(['shift',self.shift])
		json.append(['output_channel',self.output_channel])
		json.append(['input_channel',self.input_channel])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def pattern_number(self):
		'''Drum Track Pattern'''
		return self._pattern_number

	@pattern_number.setter
	def pattern_number(self,value):
		if 0 <= value <= 127 or True: #FIXME
			self._pattern_number = value
			self.notify_listeners(0xB,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def pattern_bank(self):
		'''Pattern Bank'''
		return self._pattern_bank

	@pattern_bank.setter
	def pattern_bank(self,value):
		if 0 <= value <= 1:
			self._pattern_bank = value
			self.notify_listeners(0xB,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def pattern_bank_display(self):
		'''Pattern Bank display value'''
		return pattern_bank_dict[self._pattern_bank]

#-------------------------------------------------------------------------------
	@property
	def button_mode(self):
		'''Trigger Mode'''
		return self._button_mode

	@button_mode.setter
	def button_mode(self,value):
		if 0 <= value <= 1:
			self._button_mode = value
			self.notify_listeners(0xB,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def button_mode_display(self):
		'''Trigger Mode display value'''
		return button_mode_dict[self._button_mode]

#-------------------------------------------------------------------------------
	@property
	def latch_mode(self):
		'''Trigger Latch'''
		return self._latch_mode

	@latch_mode.setter
	def latch_mode(self,value):
		if 0 <= value <= 1:
			self._latch_mode = value
			self.notify_listeners(0xB,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def latch_mode_display(self):
		'''Trigger Latch display value'''
		return off_on_dict[self._latch_mode]

#-------------------------------------------------------------------------------
	@property
	def trigger_sync(self):
		'''Trigger Sync'''
		return self._trigger_sync

	@trigger_sync.setter
	def trigger_sync(self,value):
		if 0 <= value <= 1:
			self._trigger_sync = value
			self.notify_listeners(0xB,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trigger_sync_display(self):
		'''Trigger Sync display value'''
		return trigger_sync_dict[self._trigger_sync]

#-------------------------------------------------------------------------------
	@property
	def drum_track_on(self):
		'''Drum Track On'''
		return self._drum_track_on

	@drum_track_on.setter
	def drum_track_on(self,value):
		if 0 <= value <= 1:
			self._drum_track_on = value
			self.notify_listeners(0xB,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def drum_track_on_display(self):
		'''Drum Track On display value'''
		return off_on_dict[self._drum_track_on]

#-------------------------------------------------------------------------------
	@property
	def trigger_bottom_key(self):
		'''Zone Keyboard Bottom'''
		return self._trigger_bottom_key

	@trigger_bottom_key.setter
	def trigger_bottom_key(self,value):
		if 0 <= value <= 127:
			self._trigger_bottom_key = value
			self.notify_listeners(0xB,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trigger_top_key(self):
		'''Zone Keyboard Top'''
		return self._trigger_top_key

	@trigger_top_key.setter
	def trigger_top_key(self,value):
		if 0 <= value <= 127:
			self._trigger_top_key = value
			self.notify_listeners(0xB,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def velocity_bottom(self):
		'''Zone Velocity Bottom'''
		return self._velocity_bottom

	@velocity_bottom.setter
	def velocity_bottom(self,value):
		if 1 <= value <= 127 or True: #FIXME
			self._velocity_bottom = value
			self.notify_listeners(0xB,0x7,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def velocity_top(self):
		'''Zone Velocity Top'''
		return self._velocity_top

	@velocity_top.setter
	def velocity_top(self,value):
		if 1 <= value <= 127:
			self._velocity_top = value
			self.notify_listeners(0xB,0x8,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def shift(self):
		'''Shift'''
		return self._shift

	@shift.setter
	def shift(self,value):
		if -24 <= value <= 24:
			self._shift = value
			self.notify_listeners(0xB,0x9,value)
		else:
			raise ValueError('%s not -24 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def output_channel(self):
		'''Output Channel'''
		return self._output_channel

	@output_channel.setter
	def output_channel(self,value):
		if 0 <= value <= 16:
			self._output_channel = value
			self.notify_listeners(0xB,0xB,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def input_channel(self):
		'''Input Channel'''
		return self._input_channel

	@input_channel.setter
	def input_channel(self,value):
		if 0 <= value <= 16:
			self._input_channel = value
			self.notify_listeners(0xB,0xC,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#===============================================================================
class ArpeggiatorParameters(TableBase):
	description = '''Arpeggiator Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a ArpeggiatorParameters Arpeggiator Parameters object'''
		v = list(struct.unpack('<H2B4b',raw[:8]))
		self.tempo = v.pop(0)
		byte794 = v.pop(0)
		self.switch = byte794 & 0x1
		self.run_a = (byte794 >> 1) & 0x1
		self.run_b = (byte794 >> 2) & 0x1
		self._reserved0794_7 = (byte794 >> 3) & 0x1F
		self._reserved0795 = v.pop(0)
		self.gate_control = v.pop(0)
		self.velocity_control = v.pop(0)
		self.swing_control = v.pop(0)
		self.step_control = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a ArpeggiatorParameters Arpeggiator Parameters object into raw data'''
		v = []
		v.append(self.tempo)
		byte794 = 0
		byte794 |= self.switch & 0x1
		byte794 |= (self.run_a & 0x1) << 1
		byte794 |= (self.run_b & 0x1) << 2
		byte794 |= (self._reserved0794_7 & 0x1F) << 3
		v.append(byte794)
		v.append(self._reserved0795)
		v.append(self.gate_control)
		v.append(self.velocity_control)
		v.append(self.swing_control)
		v.append(self.step_control)
		return struct.pack('<H2B4b',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ArpeggiatorParameters Arpeggiator Parameters object'''
		json = []
		json.append(('name','ArpeggiatorParameters'))
		json.append(('description','Arpeggiator Parameters'))
		json.append(['tempo',self.tempo])
		json.append(['switch',self.switch])
		json.append(['switch_display',self.switch_display])
		json.append(['run_a',self.run_a])
		json.append(['run_a_display',self.run_a_display])
		json.append(['run_b',self.run_b])
		json.append(['run_b_display',self.run_b_display])
		json.append(['gate_control',self.gate_control])
		json.append(['velocity_control',self.velocity_control])
		json.append(['swing_control',self.swing_control])
		json.append(['step_control',self.step_control])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def tempo(self):
		'''Tempo'''
		return self._tempo

	@tempo.setter
	def tempo(self,value):
		if 4000 <= value <= 30000:
			self._tempo = value
			self.notify_listeners(0x8,0x0,value)
		else:
			raise ValueError('%s not 4000 <= value <= 30000' % value)

#-------------------------------------------------------------------------------
	@property
	def switch(self):
		'''ARP Switch'''
		return self._switch

	@switch.setter
	def switch(self,value):
		if 0 <= value <= 1:
			self._switch = value
			self.notify_listeners(0x8,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def switch_display(self):
		'''ARP Switch display value'''
		return off_on_dict[self._switch]

#-------------------------------------------------------------------------------
	@property
	def run_a(self):
		'''ARP Run A'''
		return self._run_a

	@run_a.setter
	def run_a(self,value):
		if 0 <= value <= 1:
			self._run_a = value
			self.notify_listeners(0x8,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def run_a_display(self):
		'''ARP Run A display value'''
		return off_on_dict[self._run_a]

#-------------------------------------------------------------------------------
	@property
	def run_b(self):
		'''ARP Run B'''
		return self._run_b

	@run_b.setter
	def run_b(self,value):
		if 0 <= value <= 1:
			self._run_b = value
			self.notify_listeners(0x8,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def run_b_display(self):
		'''ARP Run B display value'''
		return off_on_dict[self._run_b]

#-------------------------------------------------------------------------------
	@property
	def gate_control(self):
		'''ARP Gate'''
		return self._gate_control

	@gate_control.setter
	def gate_control(self,value):
		if -64 <= value <= 63:
			self._gate_control = value
			self.notify_listeners(0x8,0x4,value)
		else:
			raise ValueError('%s not -64 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def velocity_control(self):
		'''ARP Velocity'''
		return self._velocity_control

	@velocity_control.setter
	def velocity_control(self,value):
		if -64 <= value <= 63:
			self._velocity_control = value
			self.notify_listeners(0x8,0x5,value)
		else:
			raise ValueError('%s not -64 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def swing_control(self):
		'''ARP Swing'''
		return self._swing_control

	@swing_control.setter
	def swing_control(self,value):
		if -64 <= value <= 63:
			self._swing_control = value
			self.notify_listeners(0x8,0x6,value)
		else:
			raise ValueError('%s not -64 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def step_control(self):
		'''ARP Step'''
		return self._step_control

	@step_control.setter
	def step_control(self,value):
		if -64 <= value <= 63:
			self._step_control = value
			self.notify_listeners(0x8,0x7,value)
		else:
			raise ValueError('%s not -64 <= value <= 63' % value)

#===============================================================================
class ArpeggiatorAParameters(TableBase):
	description = '''Arpeggiator A Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a ArpeggiatorAParameters Arpeggiator A Parameters object'''
		v = list(struct.unpack('<H4Bb5B',raw[:12]))
		self.pattern_number = v.pop(0)
		byte802 = v.pop(0)
		self.octave = byte802 & 0x3
		self.resolution = (byte802 >> 2) & 0x7
		self._reserved0802_7 = (byte802 >> 5) & 0x7
		byte803 = v.pop(0)
		self.sort = byte803 & 0x1
		self.latch = (byte803 >> 1) & 0x1
		self.key_sync = (byte803 >> 2) & 0x1
		self.keyboard = (byte803 >> 3) & 0x1
		self._reserved0803_7 = (byte803 >> 4) & 0xF
		self.gate = v.pop(0)
		self.velocity = v.pop(0)
		self.swing = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		self._reserved0811 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a ArpeggiatorAParameters Arpeggiator A Parameters object into raw data'''
		v = []
		v.append(self.pattern_number)
		byte802 = 0
		byte802 |= self.octave & 0x3
		byte802 |= (self.resolution & 0x7) << 2
		byte802 |= (self._reserved0802_7 & 0x7) << 5
		v.append(byte802)
		byte803 = 0
		byte803 |= self.sort & 0x1
		byte803 |= (self.latch & 0x1) << 1
		byte803 |= (self.key_sync & 0x1) << 2
		byte803 |= (self.keyboard & 0x1) << 3
		byte803 |= (self._reserved0803_7 & 0xF) << 4
		v.append(byte803)
		v.append(self.gate)
		v.append(self.velocity)
		v.append(self.swing)
		v.append(self.top_key)
		v.append(self.bottom_key)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		v.append(self._reserved0811)
		return struct.pack('<H4Bb5B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ArpeggiatorAParameters Arpeggiator A Parameters object'''
		json = []
		json.append(('name','ArpeggiatorAParameters'))
		json.append(('description','Arpeggiator A Parameters'))
		json.append(['pattern_number',self.pattern_number])
		json.append(['octave',self.octave])
		json.append(['resolution',self.resolution])
		json.append(['sort',self.sort])
		json.append(['sort_display',self.sort_display])
		json.append(['latch',self.latch])
		json.append(['latch_display',self.latch_display])
		json.append(['key_sync',self.key_sync])
		json.append(['key_sync_display',self.key_sync_display])
		json.append(['keyboard',self.keyboard])
		json.append(['keyboard_display',self.keyboard_display])
		json.append(['gate',self.gate])
		json.append(['velocity',self.velocity])
		json.append(['swing',self.swing])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def pattern_number(self):
		'''Arpeggiator A Pattern'''
		return self._pattern_number

	@pattern_number.setter
	def pattern_number(self,value):
		if 0 <= value <= 1032:
			self._pattern_number = value
			self.notify_listeners(0x8,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 1032' % value)

#-------------------------------------------------------------------------------
	@property
	def octave(self):
		'''Arpeggiator A Octave'''
		return self._octave

	@octave.setter
	def octave(self,value):
		if 0 <= value <= 4:
			self._octave = value
			self.notify_listeners(0x8,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def resolution(self):
		'''Arpeggiator A Resolution'''
		return self._resolution

	@resolution.setter
	def resolution(self,value):
		if 0 <= value <= 7:
			self._resolution = value
			self.notify_listeners(0x8,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 7' % value)

#-------------------------------------------------------------------------------
	@property
	def sort(self):
		'''Arpeggiator A Sort'''
		return self._sort

	@sort.setter
	def sort(self,value):
		if 0 <= value <= 1:
			self._sort = value
			self.notify_listeners(0x8,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sort_display(self):
		'''Arpeggiator A Sort display value'''
		return off_on_dict[self._sort]

#-------------------------------------------------------------------------------
	@property
	def latch(self):
		'''Arpeggiator A Latch'''
		return self._latch

	@latch.setter
	def latch(self,value):
		if 0 <= value <= 1:
			self._latch = value
			self.notify_listeners(0x8,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def latch_display(self):
		'''Arpeggiator A Latch display value'''
		return off_on_dict[self._latch]

#-------------------------------------------------------------------------------
	@property
	def key_sync(self):
		'''Arpeggiator A Key Sync'''
		return self._key_sync

	@key_sync.setter
	def key_sync(self,value):
		if 0 <= value <= 1:
			self._key_sync = value
			self.notify_listeners(0x8,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def key_sync_display(self):
		'''Arpeggiator A Key Sync display value'''
		return off_on_dict[self._key_sync]

#-------------------------------------------------------------------------------
	@property
	def keyboard(self):
		'''Arpeggiator A Keyboard'''
		return self._keyboard

	@keyboard.setter
	def keyboard(self,value):
		if 0 <= value <= 1:
			self._keyboard = value
			self.notify_listeners(0x8,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def keyboard_display(self):
		'''Arpeggiator A Keyboard display value'''
		return off_on_dict[self._keyboard]

#-------------------------------------------------------------------------------
	@property
	def gate(self):
		'''Arpeggiator A Gate'''
		return self._gate

	@gate.setter
	def gate(self,value):
		if 0 <= value <= 100 or True: #FIXME
			self._gate = value
			self.notify_listeners(0x8,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def velocity(self):
		'''Arpeggiator A Velocity'''
		return self._velocity

	@velocity.setter
	def velocity(self,value):
		if 1 <= value <= 129:
			self._velocity = value
			self.notify_listeners(0x8,0x4,value)
		else:
			raise ValueError('%s not 1 <= value <= 129' % value)

#-------------------------------------------------------------------------------
	@property
	def swing(self):
		'''Arpeggiator A Swing'''
		return self._swing

	@swing.setter
	def swing(self,value):
		if -100 <= value <= 1100:
			self._swing = value
			self.notify_listeners(0x8,0x5,value)
		else:
			raise ValueError('%s not -100 <= value <= 1100' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Arpeggiator A Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x8,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Arpeggiator A Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x8,0xB,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Arpeggiator A Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 1 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x8,0xC,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Arpeggiator A Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 1 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x8,0xD,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#===============================================================================
class ArpeggiatorBParameters(TableBase):
	description = '''Arpeggiator B Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a ArpeggiatorBParameters Arpeggiator B Parameters object'''
		v = list(struct.unpack('<H4Bb5B',raw[:12]))
		self.pattern_number = v.pop(0)
		byte814 = v.pop(0)
		self.octave = byte814 & 0x3
		self.resolution = (byte814 >> 2) & 0x7
		self._reserved0814_7 = (byte814 >> 5) & 0x7
		byte815 = v.pop(0)
		self.sort = byte815 & 0x1
		self.latch = (byte815 >> 1) & 0x1
		self.key_sync = (byte815 >> 2) & 0x1
		self.keyboard = (byte815 >> 3) & 0x1
		self._reserved0815_7 = (byte815 >> 4) & 0xF
		self.gate = v.pop(0)
		self.velocity = v.pop(0)
		self.swing = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		self._reserved0823 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a ArpeggiatorBParameters Arpeggiator B Parameters object into raw data'''
		v = []
		v.append(self.pattern_number)
		byte814 = 0
		byte814 |= self.octave & 0x3
		byte814 |= (self.resolution & 0x7) << 2
		byte814 |= (self._reserved0814_7 & 0x7) << 5
		v.append(byte814)
		byte815 = 0
		byte815 |= self.sort & 0x1
		byte815 |= (self.latch & 0x1) << 1
		byte815 |= (self.key_sync & 0x1) << 2
		byte815 |= (self.keyboard & 0x1) << 3
		byte815 |= (self._reserved0815_7 & 0xF) << 4
		v.append(byte815)
		v.append(self.gate)
		v.append(self.velocity)
		v.append(self.swing)
		v.append(self.top_key)
		v.append(self.bottom_key)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		v.append(self._reserved0823)
		return struct.pack('<H4Bb5B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a ArpeggiatorBParameters Arpeggiator B Parameters object'''
		json = []
		json.append(('name','ArpeggiatorBParameters'))
		json.append(('description','Arpeggiator B Parameters'))
		json.append(['pattern_number',self.pattern_number])
		json.append(['octave',self.octave])
		json.append(['resolution',self.resolution])
		json.append(['sort',self.sort])
		json.append(['sort_display',self.sort_display])
		json.append(['latch',self.latch])
		json.append(['latch_display',self.latch_display])
		json.append(['key_sync',self.key_sync])
		json.append(['key_sync_display',self.key_sync_display])
		json.append(['keyboard',self.keyboard])
		json.append(['keyboard_display',self.keyboard_display])
		json.append(['gate',self.gate])
		json.append(['velocity',self.velocity])
		json.append(['swing',self.swing])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def pattern_number(self):
		'''Arpeggiator B Pattern'''
		return self._pattern_number

	@pattern_number.setter
	def pattern_number(self,value):
		if 0 <= value <= 1032:
			self._pattern_number = value
			self.notify_listeners(0xA,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 1032' % value)

#-------------------------------------------------------------------------------
	@property
	def octave(self):
		'''Arpeggiator B Octave'''
		return self._octave

	@octave.setter
	def octave(self,value):
		if 0 <= value <= 4:
			self._octave = value
			self.notify_listeners(0xA,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def resolution(self):
		'''Arpeggiator B Resolution'''
		return self._resolution

	@resolution.setter
	def resolution(self,value):
		if 0 <= value <= 7:
			self._resolution = value
			self.notify_listeners(0xA,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 7' % value)

#-------------------------------------------------------------------------------
	@property
	def sort(self):
		'''Arpeggiator B Sort'''
		return self._sort

	@sort.setter
	def sort(self,value):
		if 0 <= value <= 1:
			self._sort = value
			self.notify_listeners(0xA,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sort_display(self):
		'''Arpeggiator B Sort display value'''
		return off_on_dict[self._sort]

#-------------------------------------------------------------------------------
	@property
	def latch(self):
		'''Arpeggiator B Latch'''
		return self._latch

	@latch.setter
	def latch(self,value):
		if 0 <= value <= 1:
			self._latch = value
			self.notify_listeners(0xA,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def latch_display(self):
		'''Arpeggiator B Latch display value'''
		return off_on_dict[self._latch]

#-------------------------------------------------------------------------------
	@property
	def key_sync(self):
		'''Arpeggiator B Key Sync'''
		return self._key_sync

	@key_sync.setter
	def key_sync(self,value):
		if 0 <= value <= 1:
			self._key_sync = value
			self.notify_listeners(0xA,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def key_sync_display(self):
		'''Arpeggiator B Key Sync display value'''
		return off_on_dict[self._key_sync]

#-------------------------------------------------------------------------------
	@property
	def keyboard(self):
		'''Arpeggiator B Keyboard'''
		return self._keyboard

	@keyboard.setter
	def keyboard(self,value):
		if 0 <= value <= 1:
			self._keyboard = value
			self.notify_listeners(0xA,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def keyboard_display(self):
		'''Arpeggiator B Keyboard display value'''
		return off_on_dict[self._keyboard]

#-------------------------------------------------------------------------------
	@property
	def gate(self):
		'''Arpeggiator B Gate'''
		return self._gate

	@gate.setter
	def gate(self,value):
		if 0 <= value <= 100 or True: #FIXME
			self._gate = value
			self.notify_listeners(0xA,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def velocity(self):
		'''Arpeggiator B Velocity'''
		return self._velocity

	@velocity.setter
	def velocity(self,value):
		if 1 <= value <= 129:
			self._velocity = value
			self.notify_listeners(0xA,0x4,value)
		else:
			raise ValueError('%s not 1 <= value <= 129' % value)

#-------------------------------------------------------------------------------
	@property
	def swing(self):
		'''Arpeggiator B Swing'''
		return self._swing

	@swing.setter
	def swing(self,value):
		if -100 <= value <= 1100:
			self._swing = value
			self.notify_listeners(0xA,0x5,value)
		else:
			raise ValueError('%s not -100 <= value <= 1100' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Arpeggiator B Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0xA,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Arpeggiator B Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0xA,0xB,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Arpeggiator B Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 1 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0xA,0xC,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Arpeggiator B Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 1 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0xA,0xD,value)
		else:
			raise ValueError('%s not 1 <= value <= 127' % value)

#===============================================================================
class CommonParameters(TableBase):
	description = '''Common Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a CommonParameters Common Parameters object'''
		v = list(struct.unpack('<12B',raw[:12]))
		byte824 = v.pop(0)
		self.category = byte824 & 0x1F
		self.sub_category = (byte824 >> 5) & 0x7
		self.favorite = v.pop(0)
		byte826 = v.pop(0)
		self.auto_load_tone_adjust = byte826 & 0x1
		self._reserved0826_7 = (byte826 >> 1) & 0x7F
		self.scale_type = v.pop(0)
		self.scale_key = v.pop(0)
		self.scale_random = v.pop(0)
		byte830 = v.pop(0)
		self.sw1_assignable_sw = byte830 & 0x3F
		self.sw1_mode = (byte830 >> 6) & 0x1
		self.sw1_off_on = (byte830 >> 7) & 0x1
		byte831 = v.pop(0)
		self.sw2_assignable_sw = byte831 & 0x3F
		self.sw2_mode = (byte831 >> 6) & 0x1
		self.sw2_off_on = (byte831 >> 7) & 0x1
		self.rt_knob_1 = v.pop(0)
		self.rt_knob_2 = v.pop(0)
		self.rt_knob_3 = v.pop(0)
		self.rt_knob_4 = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a CommonParameters Common Parameters object into raw data'''
		v = []
		byte824 = 0
		byte824 |= self.category & 0x1F
		byte824 |= (self.sub_category & 0x7) << 5
		v.append(byte824)
		v.append(self.favorite)
		byte826 = 0
		byte826 |= self.auto_load_tone_adjust & 0x1
		byte826 |= (self._reserved0826_7 & 0x7F) << 1
		v.append(byte826)
		v.append(self.scale_type)
		v.append(self.scale_key)
		v.append(self.scale_random)
		byte830 = 0
		byte830 |= self.sw1_assignable_sw & 0x3F
		byte830 |= (self.sw1_mode & 0x1) << 6
		byte830 |= (self.sw1_off_on & 0x1) << 7
		v.append(byte830)
		byte831 = 0
		byte831 |= self.sw2_assignable_sw & 0x3F
		byte831 |= (self.sw2_mode & 0x1) << 6
		byte831 |= (self.sw2_off_on & 0x1) << 7
		v.append(byte831)
		v.append(self.rt_knob_1)
		v.append(self.rt_knob_2)
		v.append(self.rt_knob_3)
		v.append(self.rt_knob_4)
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a CommonParameters Common Parameters object'''
		json = []
		json.append(('name','CommonParameters'))
		json.append(('description','Common Parameters'))
		json.append(['category',self.category])
		json.append(['sub_category',self.sub_category])
		json.append(['favorite',self.favorite])
		json.append(['auto_load_tone_adjust',self.auto_load_tone_adjust])
		json.append(['auto_load_tone_adjust_display',self.auto_load_tone_adjust_display])
		json.append(['scale_type',self.scale_type])
		json.append(['scale_key',self.scale_key])
		json.append(['scale_random',self.scale_random])
		json.append(['sw1_assignable_sw',self.sw1_assignable_sw])
		json.append(['sw1_assignable_sw_display',self.sw1_assignable_sw_display])
		json.append(['sw1_mode',self.sw1_mode])
		json.append(['sw1_mode_display',self.sw1_mode_display])
		json.append(['sw1_off_on',self.sw1_off_on])
		json.append(['sw1_off_on_display',self.sw1_off_on_display])
		json.append(['sw2_assignable_sw',self.sw2_assignable_sw])
		json.append(['sw2_assignable_sw_display',self.sw2_assignable_sw_display])
		json.append(['sw2_mode',self.sw2_mode])
		json.append(['sw2_mode_display',self.sw2_mode_display])
		json.append(['sw2_off_on',self.sw2_off_on])
		json.append(['sw2_off_on_display',self.sw2_off_on_display])
		json.append(['rt_knob_1',self.rt_knob_1])
		json.append(['rt_knob_2',self.rt_knob_2])
		json.append(['rt_knob_3',self.rt_knob_3])
		json.append(['rt_knob_4',self.rt_knob_4])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def category(self):
		'''Category'''
		return self._category

	@category.setter
	def category(self,value):
		if 0 <= value <= 17:
			self._category = value
			self.notify_listeners(0x1,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 17' % value)

#-------------------------------------------------------------------------------
	@property
	def sub_category(self):
		'''Sub Category'''
		return self._sub_category

	@sub_category.setter
	def sub_category(self,value):
		if 0 <= value <= 7:
			self._sub_category = value
			self.notify_listeners(0x1,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 7' % value)

#-------------------------------------------------------------------------------
	@property
	def favorite(self):
		'''Favorite'''
		return self._favorite

	@favorite.setter
	def favorite(self,value):
		if 0 <= value <= 1:
			self._favorite = value
			self.notify_listeners(0x1,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def auto_load_tone_adjust(self):
		'''Auto Load Tone Adjust'''
		return self._auto_load_tone_adjust

	@auto_load_tone_adjust.setter
	def auto_load_tone_adjust(self,value):
		if 0 <= value <= 1:
			self._auto_load_tone_adjust = value
			self.notify_listeners(0x1,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def auto_load_tone_adjust_display(self):
		'''Auto Load Tone Adjust display value'''
		return off_on_dict[self._auto_load_tone_adjust]

#-------------------------------------------------------------------------------
	@property
	def scale_type(self):
		'''Scale Type'''
		return self._scale_type

	@scale_type.setter
	def scale_type(self,value):
		if 0 <= value <= 26:
			self._scale_type = value
			self.notify_listeners(0x1,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 26' % value)

#-------------------------------------------------------------------------------
	@property
	def scale_key(self):
		'''Scale Key'''
		return self._scale_key

	@scale_key.setter
	def scale_key(self,value):
		if 0 <= value <= 11:
			self._scale_key = value
			self.notify_listeners(0x1,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 11' % value)

#-------------------------------------------------------------------------------
	@property
	def scale_random(self):
		'''Scale Random'''
		return self._scale_random

	@scale_random.setter
	def scale_random(self,value):
		if 0 <= value <= 7:
			self._scale_random = value
			self.notify_listeners(0x1,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 7' % value)

#-------------------------------------------------------------------------------
	@property
	def sw1_assignable_sw(self):
		'''Switch 1 Assign'''
		return self._sw1_assignable_sw

	@sw1_assignable_sw.setter
	def sw1_assignable_sw(self,value):
		if 0 <= value <= 8:
			self._sw1_assignable_sw = value
			self.notify_listeners(0x1,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 8' % value)

	@property
	def sw1_assignable_sw_display(self):
		'''Switch 1 Assign display value'''
		return sw_assignable_sw_dict[self._sw1_assignable_sw]

#-------------------------------------------------------------------------------
	@property
	def sw1_mode(self):
		'''Switch 1 Mode'''
		return self._sw1_mode

	@sw1_mode.setter
	def sw1_mode(self,value):
		if 0 <= value <= 1:
			self._sw1_mode = value
			self.notify_listeners(0x1,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sw1_mode_display(self):
		'''Switch 1 Mode display value'''
		return sw_mode_dict[self._sw1_mode]

#-------------------------------------------------------------------------------
	@property
	def sw1_off_on(self):
		'''Switch 1 On/Off'''
		return self._sw1_off_on

	@sw1_off_on.setter
	def sw1_off_on(self,value):
		if 0 <= value <= 1:
			self._sw1_off_on = value
			self.notify_listeners(0x1,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sw1_off_on_display(self):
		'''Switch 1 On/Off display value'''
		return off_on_dict[self._sw1_off_on]

#-------------------------------------------------------------------------------
	@property
	def sw2_assignable_sw(self):
		'''Switch 2 Assign'''
		return self._sw2_assignable_sw

	@sw2_assignable_sw.setter
	def sw2_assignable_sw(self,value):
		if 0 <= value <= 8:
			self._sw2_assignable_sw = value
			self.notify_listeners(0x1,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 8' % value)

	@property
	def sw2_assignable_sw_display(self):
		'''Switch 2 Assign display value'''
		return sw_assignable_sw_dict[self._sw2_assignable_sw]

#-------------------------------------------------------------------------------
	@property
	def sw2_mode(self):
		'''Switch 2 Mode'''
		return self._sw2_mode

	@sw2_mode.setter
	def sw2_mode(self,value):
		if 0 <= value <= 1:
			self._sw2_mode = value
			self.notify_listeners(0x1,0xB,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sw2_mode_display(self):
		'''Switch 2 Mode display value'''
		return sw_mode_dict[self._sw2_mode]

#-------------------------------------------------------------------------------
	@property
	def sw2_off_on(self):
		'''Switch 2 On/Off'''
		return self._sw2_off_on

	@sw2_off_on.setter
	def sw2_off_on(self,value):
		if 0 <= value <= 1:
			self._sw2_off_on = value
			self.notify_listeners(0x1,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def sw2_off_on_display(self):
		'''Switch 2 On/Off display value'''
		return off_on_dict[self._sw2_off_on]

#-------------------------------------------------------------------------------
	@property
	def rt_knob_1(self):
		'''RT Knob-1 Assign'''
		return self._rt_knob_1

	@rt_knob_1.setter
	def rt_knob_1(self,value):
		if 0 <= value <= 142:
			self._rt_knob_1 = value
			self.notify_listeners(0x1,0xC,value)
		else:
			raise ValueError('%s not 0 <= value <= 142' % value)

#-------------------------------------------------------------------------------
	@property
	def rt_knob_2(self):
		'''RT Knob-2 Assign'''
		return self._rt_knob_2

	@rt_knob_2.setter
	def rt_knob_2(self,value):
		if 0 <= value <= 142:
			self._rt_knob_2 = value
			self.notify_listeners(0x1,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 142' % value)

#-------------------------------------------------------------------------------
	@property
	def rt_knob_3(self):
		'''RT Knob-3 Assign'''
		return self._rt_knob_3

	@rt_knob_3.setter
	def rt_knob_3(self,value):
		if 0 <= value <= 142:
			self._rt_knob_3 = value
			self.notify_listeners(0x1,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 142' % value)

#-------------------------------------------------------------------------------
	@property
	def rt_knob_4(self):
		'''RT Knob-4 Assign'''
		return self._rt_knob_4

	@rt_knob_4.setter
	def rt_knob_4(self,value):
		if 0 <= value <= 142:
			self._rt_knob_4 = value
			self.notify_listeners(0x1,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 142' % value)

#===============================================================================
class Track1Parameters(TableBase):
	description = '''Track 1 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track1Parameters Track 1 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte838 = v.pop(0)
		self.midi_channel = byte838 & 0x1F
		self.midi_status = (byte838 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved0847 = v.pop(0)
		byte848 = v.pop(0)
		self.delay_tempo_base = byte848 & 0xF
		self._reserved0848_6 = (byte848 >> 4) & 0x7
		self.delay_mode = (byte848 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte858 = v.pop(0)
		self.bus_select = byte858 & 0x1F
		self.fx_bus_select = (byte858 >> 5) & 0x3
		self._reserved0858_7 = (byte858 >> 7) & 0x1
		self._reserved0859 = v.pop(0)
		self._reserved0860 = v.pop(0)
		byte861 = v.pop(0)
		self.enable_program_change = byte861 & 0x1
		self.enable_after_touch = (byte861 >> 1) & 0x1
		self.enable_damper = (byte861 >> 2) & 0x1
		self.enable_portamento = (byte861 >> 3) & 0x1
		self.enable_joystick_x = (byte861 >> 4) & 0x1
		self.enable_joystick_plus = (byte861 >> 5) & 0x1
		self.enable_joystick_minus = (byte861 >> 6) & 0x1
		self.enable_ribbon = (byte861 >> 7) & 0x1
		byte862 = v.pop(0)
		self.enable_rt_knob_cutoff = byte862 & 0x1
		self.enable_rt_knob_resonance = (byte862 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte862 >> 2) & 0x1
		self.enable_rt_knob_release = (byte862 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte862 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte862 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte862 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte862 >> 7) & 0x1
		byte863 = v.pop(0)
		self.enable_sw1 = byte863 & 0x1
		self.enable_sw2 = (byte863 >> 1) & 0x1
		self.enable_foot_switch = (byte863 >> 2) & 0x1
		self.enable_foot_pedal = (byte863 >> 3) & 0x1
		self.enable_other_control_change = (byte863 >> 4) & 0x1
		self.enable_use_prg_scales = (byte863 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte863 >> 6) & 0x1
		self.enable_timbre_mute = (byte863 >> 7) & 0x1
		byte864 = v.pop(0)
		self.force_osc_mode = byte864 & 0x3
		self.osc_select = (byte864 >> 2) & 0x3
		self._reserved0864_4 = (byte864 >> 4) & 0x1
		self.arpeggiator_assign = (byte864 >> 5) & 0x3
		self.enable_waveform_sysex = (byte864 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte868 = v.pop(0)
		self.top_slope = byte868 & 0xF
		self.bottom_slope = (byte868 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte871 = v.pop(0)
		self.top_velocity_slope = byte871 & 0xF
		self.bottom_velocity_slope = (byte871 >> 4) & 0xF
		byte872 = v.pop(0)
		self.trim_value = byte872 & 0x7F
		self.trim_bypass = (byte872 >> 7) & 0x1
		self._reserved0873 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved0878 = v.pop(0)
		self._reserved0879 = v.pop(0)
		self._reserved0880 = v.pop(0)
		self._reserved0881 = v.pop(0)
		self._reserved0882 = v.pop(0)
		self._reserved0883 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved0885 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved0889 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved0893 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved0897 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved0901 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved0905 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved0909 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved0913 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track1Parameters Track 1 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte838 = 0
		byte838 |= self.midi_channel & 0x1F
		byte838 |= (self.midi_status & 0x7) << 5
		v.append(byte838)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved0847)
		byte848 = 0
		byte848 |= self.delay_tempo_base & 0xF
		byte848 |= (self._reserved0848_6 & 0x7) << 4
		byte848 |= (self.delay_mode & 0x1) << 7
		v.append(byte848)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte858 = 0
		byte858 |= self.bus_select & 0x1F
		byte858 |= (self.fx_bus_select & 0x3) << 5
		byte858 |= (self._reserved0858_7 & 0x1) << 7
		v.append(byte858)
		v.append(self._reserved0859)
		v.append(self._reserved0860)
		byte861 = 0
		byte861 |= self.enable_program_change & 0x1
		byte861 |= (self.enable_after_touch & 0x1) << 1
		byte861 |= (self.enable_damper & 0x1) << 2
		byte861 |= (self.enable_portamento & 0x1) << 3
		byte861 |= (self.enable_joystick_x & 0x1) << 4
		byte861 |= (self.enable_joystick_plus & 0x1) << 5
		byte861 |= (self.enable_joystick_minus & 0x1) << 6
		byte861 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte861)
		byte862 = 0
		byte862 |= self.enable_rt_knob_cutoff & 0x1
		byte862 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte862 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte862 |= (self.enable_rt_knob_release & 0x1) << 3
		byte862 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte862 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte862 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte862 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte862)
		byte863 = 0
		byte863 |= self.enable_sw1 & 0x1
		byte863 |= (self.enable_sw2 & 0x1) << 1
		byte863 |= (self.enable_foot_switch & 0x1) << 2
		byte863 |= (self.enable_foot_pedal & 0x1) << 3
		byte863 |= (self.enable_other_control_change & 0x1) << 4
		byte863 |= (self.enable_use_prg_scales & 0x1) << 5
		byte863 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte863 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte863)
		byte864 = 0
		byte864 |= self.force_osc_mode & 0x3
		byte864 |= (self.osc_select & 0x3) << 2
		byte864 |= (self._reserved0864_4 & 0x1) << 4
		byte864 |= (self.arpeggiator_assign & 0x3) << 5
		byte864 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte864)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte868 = 0
		byte868 |= self.top_slope & 0xF
		byte868 |= (self.bottom_slope & 0xF) << 4
		v.append(byte868)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte871 = 0
		byte871 |= self.top_velocity_slope & 0xF
		byte871 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte871)
		byte872 = 0
		byte872 |= self.trim_value & 0x7F
		byte872 |= (self.trim_bypass & 0x1) << 7
		v.append(byte872)
		v.append(self._reserved0873)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved0878)
		v.append(self._reserved0879)
		v.append(self._reserved0880)
		v.append(self._reserved0881)
		v.append(self._reserved0882)
		v.append(self._reserved0883)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved0885)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved0889)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved0893)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved0897)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved0901)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved0905)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved0909)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved0913)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track1Parameters Track 1 Parameters object'''
		json = []
		json.append(('name','Track1Parameters'))
		json.append(('description','Track 1 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk1 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0xC,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk1 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0xC,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk1 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0xC,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk1 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk1 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0xC,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk1 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0xC,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk1 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0xC,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk1 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0xC,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk1 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0xC,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk1 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0xC,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk1 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0xC,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk1 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0xC,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk1 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk1 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0xC,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk1 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk1 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0xC,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk1 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0xC,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk1 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0xC,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk1 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0xC,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk1 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0xC,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk1 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk1 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0xC,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk1 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk1 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0xC,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk1 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk1 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0xC,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk1 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk1 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0xC,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk1 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk1 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0xC,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk1 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk1 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0xC,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk1 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk1 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0xC,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk1 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk1 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0xC,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk1 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk1 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0xC,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk1 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk1 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0xC,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk1 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk1 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0xC,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk1 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk1 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0xC,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk1 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk1 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0xC,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk1 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk1 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0xC,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk1 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk1 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0xC,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk1 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk1 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0xC,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk1 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk1 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0xC,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk1 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk1 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0xC,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk1 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk1 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0xC,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk1 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk1 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0xC,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk1 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk1 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0xC,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk1 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk1 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0xC,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk1 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk1 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0xC,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk1 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk1 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0xC,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk1 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk1 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0xC,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk1 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk1 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0xC,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk1 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk1 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0xC,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk1 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk1 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0xC,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk1 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk1 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0xC,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk1 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk1 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0xC,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk1 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk1 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0xC,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk1 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk1 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0xC,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk1 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk1 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0xC,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk1 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk1 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0xC,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk1 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk1 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0xC,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk1 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0xC,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk1 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0xC,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk1 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0xC,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk1 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0xC,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk1 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0xC,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk1 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0xC,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk1 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0xC,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk1 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0xC,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk1 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0xC,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk1 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0xC,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk1 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk1 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0xC,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk1 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0xC,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk1 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0xC,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk1 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0xC,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk1 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x1C,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk1 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk1 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x1C,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk1 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x1C,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk1 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk1 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x1C,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk1 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x1C,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk1 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk1 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x1C,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk1 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x1C,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk1 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk1 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x1C,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk1 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x1C,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk1 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk1 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x1C,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk1 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x1C,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk1 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk1 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x1C,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk1 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x1C,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk1 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk1 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x1C,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk1 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x1C,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk1 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk1 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x1C,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk1 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x1C,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk1 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk1 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x1C,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk1 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk1 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x1C,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk1 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x1C,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk1 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk1 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x1C,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk1 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk1 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x1C,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk1 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x1C,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk1 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk1 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x1C,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk1 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk1 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x1C,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk1 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x1C,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk1 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk1 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x1C,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk1 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk1 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x1C,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk1 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x1C,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk1 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk1 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x1C,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk1 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk1 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x1C,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk1 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x1C,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk1 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk1 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x1C,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk1 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk1 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x1C,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk1 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x1C,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk1 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk1 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x1C,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk1 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk1 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x1C,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk1 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x1C,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk1 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk1 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk1 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x1C,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk1 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk1 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x1C,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track2Parameters(TableBase):
	description = '''Track 2 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track2Parameters Track 2 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte950 = v.pop(0)
		self.midi_channel = byte950 & 0x1F
		self.midi_status = (byte950 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved0959 = v.pop(0)
		byte960 = v.pop(0)
		self.delay_tempo_base = byte960 & 0xF
		self._reserved0960_6 = (byte960 >> 4) & 0x7
		self.delay_mode = (byte960 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte970 = v.pop(0)
		self.bus_select = byte970 & 0x1F
		self.fx_bus_select = (byte970 >> 5) & 0x3
		self._reserved0970_7 = (byte970 >> 7) & 0x1
		self._reserved0971 = v.pop(0)
		self._reserved0972 = v.pop(0)
		byte973 = v.pop(0)
		self.enable_program_change = byte973 & 0x1
		self.enable_after_touch = (byte973 >> 1) & 0x1
		self.enable_damper = (byte973 >> 2) & 0x1
		self.enable_portamento = (byte973 >> 3) & 0x1
		self.enable_joystick_x = (byte973 >> 4) & 0x1
		self.enable_joystick_plus = (byte973 >> 5) & 0x1
		self.enable_joystick_minus = (byte973 >> 6) & 0x1
		self.enable_ribbon = (byte973 >> 7) & 0x1
		byte974 = v.pop(0)
		self.enable_rt_knob_cutoff = byte974 & 0x1
		self.enable_rt_knob_resonance = (byte974 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte974 >> 2) & 0x1
		self.enable_rt_knob_release = (byte974 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte974 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte974 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte974 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte974 >> 7) & 0x1
		byte975 = v.pop(0)
		self.enable_sw1 = byte975 & 0x1
		self.enable_sw2 = (byte975 >> 1) & 0x1
		self.enable_foot_switch = (byte975 >> 2) & 0x1
		self.enable_foot_pedal = (byte975 >> 3) & 0x1
		self.enable_other_control_change = (byte975 >> 4) & 0x1
		self.enable_use_prg_scales = (byte975 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte975 >> 6) & 0x1
		self.enable_timbre_mute = (byte975 >> 7) & 0x1
		byte976 = v.pop(0)
		self.force_osc_mode = byte976 & 0x3
		self.osc_select = (byte976 >> 2) & 0x3
		self._reserved0976_4 = (byte976 >> 4) & 0x1
		self.arpeggiator_assign = (byte976 >> 5) & 0x3
		self.enable_waveform_sysex = (byte976 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte980 = v.pop(0)
		self.top_slope = byte980 & 0xF
		self.bottom_slope = (byte980 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte983 = v.pop(0)
		self.top_velocity_slope = byte983 & 0xF
		self.bottom_velocity_slope = (byte983 >> 4) & 0xF
		byte984 = v.pop(0)
		self.trim_value = byte984 & 0x7F
		self.trim_bypass = (byte984 >> 7) & 0x1
		self._reserved0985 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved0990 = v.pop(0)
		self._reserved0991 = v.pop(0)
		self._reserved0992 = v.pop(0)
		self._reserved0993 = v.pop(0)
		self._reserved0994 = v.pop(0)
		self._reserved0995 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved0997 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1001 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1005 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1009 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1013 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1017 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1021 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1025 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track2Parameters Track 2 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte950 = 0
		byte950 |= self.midi_channel & 0x1F
		byte950 |= (self.midi_status & 0x7) << 5
		v.append(byte950)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved0959)
		byte960 = 0
		byte960 |= self.delay_tempo_base & 0xF
		byte960 |= (self._reserved0960_6 & 0x7) << 4
		byte960 |= (self.delay_mode & 0x1) << 7
		v.append(byte960)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte970 = 0
		byte970 |= self.bus_select & 0x1F
		byte970 |= (self.fx_bus_select & 0x3) << 5
		byte970 |= (self._reserved0970_7 & 0x1) << 7
		v.append(byte970)
		v.append(self._reserved0971)
		v.append(self._reserved0972)
		byte973 = 0
		byte973 |= self.enable_program_change & 0x1
		byte973 |= (self.enable_after_touch & 0x1) << 1
		byte973 |= (self.enable_damper & 0x1) << 2
		byte973 |= (self.enable_portamento & 0x1) << 3
		byte973 |= (self.enable_joystick_x & 0x1) << 4
		byte973 |= (self.enable_joystick_plus & 0x1) << 5
		byte973 |= (self.enable_joystick_minus & 0x1) << 6
		byte973 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte973)
		byte974 = 0
		byte974 |= self.enable_rt_knob_cutoff & 0x1
		byte974 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte974 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte974 |= (self.enable_rt_knob_release & 0x1) << 3
		byte974 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte974 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte974 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte974 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte974)
		byte975 = 0
		byte975 |= self.enable_sw1 & 0x1
		byte975 |= (self.enable_sw2 & 0x1) << 1
		byte975 |= (self.enable_foot_switch & 0x1) << 2
		byte975 |= (self.enable_foot_pedal & 0x1) << 3
		byte975 |= (self.enable_other_control_change & 0x1) << 4
		byte975 |= (self.enable_use_prg_scales & 0x1) << 5
		byte975 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte975 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte975)
		byte976 = 0
		byte976 |= self.force_osc_mode & 0x3
		byte976 |= (self.osc_select & 0x3) << 2
		byte976 |= (self._reserved0976_4 & 0x1) << 4
		byte976 |= (self.arpeggiator_assign & 0x3) << 5
		byte976 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte976)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte980 = 0
		byte980 |= self.top_slope & 0xF
		byte980 |= (self.bottom_slope & 0xF) << 4
		v.append(byte980)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte983 = 0
		byte983 |= self.top_velocity_slope & 0xF
		byte983 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte983)
		byte984 = 0
		byte984 |= self.trim_value & 0x7F
		byte984 |= (self.trim_bypass & 0x1) << 7
		v.append(byte984)
		v.append(self._reserved0985)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved0990)
		v.append(self._reserved0991)
		v.append(self._reserved0992)
		v.append(self._reserved0993)
		v.append(self._reserved0994)
		v.append(self._reserved0995)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved0997)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1001)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1005)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1009)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1013)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1017)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1021)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1025)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track2Parameters Track 2 Parameters object'''
		json = []
		json.append(('name','Track2Parameters'))
		json.append(('description','Track 2 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk2 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0xD,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk2 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0xD,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk2 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0xD,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk2 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk2 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0xD,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk2 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0xD,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk2 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0xD,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk2 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0xD,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk2 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0xD,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk2 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0xD,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk2 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0xD,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk2 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0xD,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk2 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk2 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0xD,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk2 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk2 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0xD,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk2 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0xD,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk2 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0xD,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk2 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0xD,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk2 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0xD,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk2 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk2 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0xD,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk2 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk2 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0xD,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk2 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk2 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0xD,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk2 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk2 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0xD,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk2 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk2 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0xD,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk2 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk2 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0xD,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk2 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk2 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0xD,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk2 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk2 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0xD,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk2 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk2 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0xD,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk2 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk2 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0xD,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk2 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk2 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0xD,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk2 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk2 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0xD,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk2 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk2 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0xD,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk2 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk2 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0xD,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk2 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk2 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0xD,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk2 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk2 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0xD,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk2 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk2 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0xD,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk2 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk2 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0xD,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk2 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk2 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0xD,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk2 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk2 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0xD,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk2 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk2 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0xD,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk2 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk2 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0xD,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk2 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk2 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0xD,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk2 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk2 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0xD,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk2 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk2 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0xD,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk2 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk2 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0xD,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk2 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk2 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0xD,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk2 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk2 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0xD,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk2 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk2 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0xD,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk2 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk2 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0xD,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk2 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk2 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0xD,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk2 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk2 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0xD,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk2 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk2 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0xD,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk2 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk2 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0xD,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk2 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk2 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0xD,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk2 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0xD,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk2 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0xD,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk2 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0xD,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk2 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0xD,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk2 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0xD,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk2 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0xD,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk2 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0xD,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk2 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0xD,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk2 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0xD,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk2 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0xD,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk2 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk2 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0xD,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk2 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0xD,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk2 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0xD,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk2 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0xD,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk2 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x1D,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk2 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk2 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x1D,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk2 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x1D,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk2 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk2 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x1D,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk2 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x1D,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk2 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk2 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x1D,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk2 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x1D,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk2 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk2 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x1D,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk2 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x1D,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk2 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk2 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x1D,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk2 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x1D,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk2 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk2 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x1D,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk2 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x1D,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk2 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk2 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x1D,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk2 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x1D,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk2 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk2 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x1D,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk2 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x1D,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk2 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk2 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x1D,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk2 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk2 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x1D,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk2 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x1D,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk2 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk2 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x1D,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk2 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk2 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x1D,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk2 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x1D,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk2 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk2 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x1D,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk2 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk2 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x1D,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk2 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x1D,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk2 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk2 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x1D,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk2 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk2 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x1D,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk2 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x1D,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk2 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk2 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x1D,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk2 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk2 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x1D,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk2 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x1D,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk2 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk2 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x1D,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk2 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk2 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x1D,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk2 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x1D,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk2 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk2 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x1D,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk2 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk2 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x1D,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk2 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x1D,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk2 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk2 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk2 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x1D,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk2 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk2 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x1D,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track3Parameters(TableBase):
	description = '''Track 3 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track3Parameters Track 3 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1062 = v.pop(0)
		self.midi_channel = byte1062 & 0x1F
		self.midi_status = (byte1062 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1071 = v.pop(0)
		byte1072 = v.pop(0)
		self.delay_tempo_base = byte1072 & 0xF
		self._reserved1072_6 = (byte1072 >> 4) & 0x7
		self.delay_mode = (byte1072 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1082 = v.pop(0)
		self.bus_select = byte1082 & 0x1F
		self.fx_bus_select = (byte1082 >> 5) & 0x3
		self._reserved1082_7 = (byte1082 >> 7) & 0x1
		self._reserved1083 = v.pop(0)
		self._reserved1084 = v.pop(0)
		byte1085 = v.pop(0)
		self.enable_program_change = byte1085 & 0x1
		self.enable_after_touch = (byte1085 >> 1) & 0x1
		self.enable_damper = (byte1085 >> 2) & 0x1
		self.enable_portamento = (byte1085 >> 3) & 0x1
		self.enable_joystick_x = (byte1085 >> 4) & 0x1
		self.enable_joystick_plus = (byte1085 >> 5) & 0x1
		self.enable_joystick_minus = (byte1085 >> 6) & 0x1
		self.enable_ribbon = (byte1085 >> 7) & 0x1
		byte1086 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1086 & 0x1
		self.enable_rt_knob_resonance = (byte1086 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1086 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1086 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1086 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1086 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1086 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1086 >> 7) & 0x1
		byte1087 = v.pop(0)
		self.enable_sw1 = byte1087 & 0x1
		self.enable_sw2 = (byte1087 >> 1) & 0x1
		self.enable_foot_switch = (byte1087 >> 2) & 0x1
		self.enable_foot_pedal = (byte1087 >> 3) & 0x1
		self.enable_other_control_change = (byte1087 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1087 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1087 >> 6) & 0x1
		self.enable_timbre_mute = (byte1087 >> 7) & 0x1
		byte1088 = v.pop(0)
		self.force_osc_mode = byte1088 & 0x3
		self.osc_select = (byte1088 >> 2) & 0x3
		self._reserved1088_4 = (byte1088 >> 4) & 0x1
		self.arpeggiator_assign = (byte1088 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1088 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1092 = v.pop(0)
		self.top_slope = byte1092 & 0xF
		self.bottom_slope = (byte1092 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1095 = v.pop(0)
		self.top_velocity_slope = byte1095 & 0xF
		self.bottom_velocity_slope = (byte1095 >> 4) & 0xF
		byte1096 = v.pop(0)
		self.trim_value = byte1096 & 0x7F
		self.trim_bypass = (byte1096 >> 7) & 0x1
		self._reserved1097 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1102 = v.pop(0)
		self._reserved1103 = v.pop(0)
		self._reserved1104 = v.pop(0)
		self._reserved1105 = v.pop(0)
		self._reserved1106 = v.pop(0)
		self._reserved1107 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1109 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1113 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1117 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1121 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1125 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1129 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1133 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1137 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track3Parameters Track 3 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1062 = 0
		byte1062 |= self.midi_channel & 0x1F
		byte1062 |= (self.midi_status & 0x7) << 5
		v.append(byte1062)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1071)
		byte1072 = 0
		byte1072 |= self.delay_tempo_base & 0xF
		byte1072 |= (self._reserved1072_6 & 0x7) << 4
		byte1072 |= (self.delay_mode & 0x1) << 7
		v.append(byte1072)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1082 = 0
		byte1082 |= self.bus_select & 0x1F
		byte1082 |= (self.fx_bus_select & 0x3) << 5
		byte1082 |= (self._reserved1082_7 & 0x1) << 7
		v.append(byte1082)
		v.append(self._reserved1083)
		v.append(self._reserved1084)
		byte1085 = 0
		byte1085 |= self.enable_program_change & 0x1
		byte1085 |= (self.enable_after_touch & 0x1) << 1
		byte1085 |= (self.enable_damper & 0x1) << 2
		byte1085 |= (self.enable_portamento & 0x1) << 3
		byte1085 |= (self.enable_joystick_x & 0x1) << 4
		byte1085 |= (self.enable_joystick_plus & 0x1) << 5
		byte1085 |= (self.enable_joystick_minus & 0x1) << 6
		byte1085 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1085)
		byte1086 = 0
		byte1086 |= self.enable_rt_knob_cutoff & 0x1
		byte1086 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1086 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1086 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1086 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1086 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1086 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1086 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1086)
		byte1087 = 0
		byte1087 |= self.enable_sw1 & 0x1
		byte1087 |= (self.enable_sw2 & 0x1) << 1
		byte1087 |= (self.enable_foot_switch & 0x1) << 2
		byte1087 |= (self.enable_foot_pedal & 0x1) << 3
		byte1087 |= (self.enable_other_control_change & 0x1) << 4
		byte1087 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1087 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1087 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1087)
		byte1088 = 0
		byte1088 |= self.force_osc_mode & 0x3
		byte1088 |= (self.osc_select & 0x3) << 2
		byte1088 |= (self._reserved1088_4 & 0x1) << 4
		byte1088 |= (self.arpeggiator_assign & 0x3) << 5
		byte1088 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1088)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1092 = 0
		byte1092 |= self.top_slope & 0xF
		byte1092 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1092)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1095 = 0
		byte1095 |= self.top_velocity_slope & 0xF
		byte1095 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1095)
		byte1096 = 0
		byte1096 |= self.trim_value & 0x7F
		byte1096 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1096)
		v.append(self._reserved1097)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1102)
		v.append(self._reserved1103)
		v.append(self._reserved1104)
		v.append(self._reserved1105)
		v.append(self._reserved1106)
		v.append(self._reserved1107)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1109)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1113)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1117)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1121)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1125)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1129)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1133)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1137)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track3Parameters Track 3 Parameters object'''
		json = []
		json.append(('name','Track3Parameters'))
		json.append(('description','Track 3 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk3 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0xE,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk3 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0xE,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk3 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0xE,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk3 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk3 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0xE,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk3 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0xE,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk3 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0xE,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk3 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0xE,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk3 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0xE,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk3 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0xE,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk3 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0xE,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk3 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0xE,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk3 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk3 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0xE,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk3 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk3 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0xE,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk3 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0xE,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk3 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0xE,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk3 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0xE,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk3 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0xE,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk3 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk3 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0xE,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk3 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk3 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0xE,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk3 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk3 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0xE,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk3 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk3 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0xE,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk3 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk3 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0xE,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk3 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk3 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0xE,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk3 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk3 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0xE,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk3 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk3 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0xE,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk3 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk3 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0xE,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk3 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk3 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0xE,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk3 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk3 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0xE,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk3 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk3 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0xE,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk3 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk3 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0xE,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk3 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk3 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0xE,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk3 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk3 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0xE,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk3 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk3 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0xE,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk3 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk3 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0xE,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk3 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk3 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0xE,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk3 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk3 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0xE,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk3 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk3 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0xE,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk3 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk3 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0xE,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk3 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk3 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0xE,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk3 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk3 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0xE,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk3 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk3 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0xE,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk3 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk3 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0xE,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk3 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk3 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0xE,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk3 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk3 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0xE,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk3 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk3 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0xE,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk3 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk3 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0xE,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk3 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk3 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0xE,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk3 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk3 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0xE,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk3 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk3 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0xE,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk3 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk3 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0xE,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk3 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk3 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0xE,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk3 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk3 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0xE,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk3 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0xE,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk3 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0xE,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk3 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0xE,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk3 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0xE,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk3 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0xE,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk3 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0xE,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk3 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0xE,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk3 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0xE,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk3 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0xE,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk3 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0xE,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk3 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk3 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0xE,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk3 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0xE,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk3 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0xE,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk3 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0xE,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk3 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x1E,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk3 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk3 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x1E,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk3 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x1E,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk3 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk3 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x1E,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk3 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x1E,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk3 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk3 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x1E,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk3 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x1E,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk3 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk3 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x1E,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk3 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x1E,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk3 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk3 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x1E,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk3 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x1E,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk3 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk3 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x1E,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk3 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x1E,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk3 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk3 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x1E,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk3 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x1E,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk3 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk3 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x1E,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk3 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x1E,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk3 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk3 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x1E,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk3 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk3 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x1E,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk3 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x1E,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk3 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk3 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x1E,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk3 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk3 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x1E,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk3 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x1E,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk3 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk3 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x1E,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk3 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk3 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x1E,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk3 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x1E,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk3 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk3 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x1E,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk3 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk3 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x1E,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk3 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x1E,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk3 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk3 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x1E,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk3 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk3 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x1E,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk3 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x1E,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk3 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk3 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x1E,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk3 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk3 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x1E,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk3 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x1E,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk3 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk3 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x1E,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk3 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk3 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x1E,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk3 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x1E,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk3 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk3 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk3 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x1E,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk3 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk3 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x1E,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track4Parameters(TableBase):
	description = '''Track 4 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track4Parameters Track 4 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1174 = v.pop(0)
		self.midi_channel = byte1174 & 0x1F
		self.midi_status = (byte1174 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1183 = v.pop(0)
		byte1184 = v.pop(0)
		self.delay_tempo_base = byte1184 & 0xF
		self._reserved1184_6 = (byte1184 >> 4) & 0x7
		self.delay_mode = (byte1184 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1194 = v.pop(0)
		self.bus_select = byte1194 & 0x1F
		self.fx_bus_select = (byte1194 >> 5) & 0x3
		self._reserved1194_7 = (byte1194 >> 7) & 0x1
		self._reserved1195 = v.pop(0)
		self._reserved1196 = v.pop(0)
		byte1197 = v.pop(0)
		self.enable_program_change = byte1197 & 0x1
		self.enable_after_touch = (byte1197 >> 1) & 0x1
		self.enable_damper = (byte1197 >> 2) & 0x1
		self.enable_portamento = (byte1197 >> 3) & 0x1
		self.enable_joystick_x = (byte1197 >> 4) & 0x1
		self.enable_joystick_plus = (byte1197 >> 5) & 0x1
		self.enable_joystick_minus = (byte1197 >> 6) & 0x1
		self.enable_ribbon = (byte1197 >> 7) & 0x1
		byte1198 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1198 & 0x1
		self.enable_rt_knob_resonance = (byte1198 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1198 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1198 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1198 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1198 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1198 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1198 >> 7) & 0x1
		byte1199 = v.pop(0)
		self.enable_sw1 = byte1199 & 0x1
		self.enable_sw2 = (byte1199 >> 1) & 0x1
		self.enable_foot_switch = (byte1199 >> 2) & 0x1
		self.enable_foot_pedal = (byte1199 >> 3) & 0x1
		self.enable_other_control_change = (byte1199 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1199 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1199 >> 6) & 0x1
		self.enable_timbre_mute = (byte1199 >> 7) & 0x1
		byte1200 = v.pop(0)
		self.force_osc_mode = byte1200 & 0x3
		self.osc_select = (byte1200 >> 2) & 0x3
		self._reserved1200_4 = (byte1200 >> 4) & 0x1
		self.arpeggiator_assign = (byte1200 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1200 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1204 = v.pop(0)
		self.top_slope = byte1204 & 0xF
		self.bottom_slope = (byte1204 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1207 = v.pop(0)
		self.top_velocity_slope = byte1207 & 0xF
		self.bottom_velocity_slope = (byte1207 >> 4) & 0xF
		byte1208 = v.pop(0)
		self.trim_value = byte1208 & 0x7F
		self.trim_bypass = (byte1208 >> 7) & 0x1
		self._reserved1209 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1214 = v.pop(0)
		self._reserved1215 = v.pop(0)
		self._reserved1216 = v.pop(0)
		self._reserved1217 = v.pop(0)
		self._reserved1218 = v.pop(0)
		self._reserved1219 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1221 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1225 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1229 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1233 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1237 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1241 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1245 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1249 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track4Parameters Track 4 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1174 = 0
		byte1174 |= self.midi_channel & 0x1F
		byte1174 |= (self.midi_status & 0x7) << 5
		v.append(byte1174)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1183)
		byte1184 = 0
		byte1184 |= self.delay_tempo_base & 0xF
		byte1184 |= (self._reserved1184_6 & 0x7) << 4
		byte1184 |= (self.delay_mode & 0x1) << 7
		v.append(byte1184)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1194 = 0
		byte1194 |= self.bus_select & 0x1F
		byte1194 |= (self.fx_bus_select & 0x3) << 5
		byte1194 |= (self._reserved1194_7 & 0x1) << 7
		v.append(byte1194)
		v.append(self._reserved1195)
		v.append(self._reserved1196)
		byte1197 = 0
		byte1197 |= self.enable_program_change & 0x1
		byte1197 |= (self.enable_after_touch & 0x1) << 1
		byte1197 |= (self.enable_damper & 0x1) << 2
		byte1197 |= (self.enable_portamento & 0x1) << 3
		byte1197 |= (self.enable_joystick_x & 0x1) << 4
		byte1197 |= (self.enable_joystick_plus & 0x1) << 5
		byte1197 |= (self.enable_joystick_minus & 0x1) << 6
		byte1197 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1197)
		byte1198 = 0
		byte1198 |= self.enable_rt_knob_cutoff & 0x1
		byte1198 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1198 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1198 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1198 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1198 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1198 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1198 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1198)
		byte1199 = 0
		byte1199 |= self.enable_sw1 & 0x1
		byte1199 |= (self.enable_sw2 & 0x1) << 1
		byte1199 |= (self.enable_foot_switch & 0x1) << 2
		byte1199 |= (self.enable_foot_pedal & 0x1) << 3
		byte1199 |= (self.enable_other_control_change & 0x1) << 4
		byte1199 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1199 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1199 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1199)
		byte1200 = 0
		byte1200 |= self.force_osc_mode & 0x3
		byte1200 |= (self.osc_select & 0x3) << 2
		byte1200 |= (self._reserved1200_4 & 0x1) << 4
		byte1200 |= (self.arpeggiator_assign & 0x3) << 5
		byte1200 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1200)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1204 = 0
		byte1204 |= self.top_slope & 0xF
		byte1204 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1204)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1207 = 0
		byte1207 |= self.top_velocity_slope & 0xF
		byte1207 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1207)
		byte1208 = 0
		byte1208 |= self.trim_value & 0x7F
		byte1208 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1208)
		v.append(self._reserved1209)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1214)
		v.append(self._reserved1215)
		v.append(self._reserved1216)
		v.append(self._reserved1217)
		v.append(self._reserved1218)
		v.append(self._reserved1219)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1221)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1225)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1229)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1233)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1237)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1241)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1245)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1249)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track4Parameters Track 4 Parameters object'''
		json = []
		json.append(('name','Track4Parameters'))
		json.append(('description','Track 4 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk4 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0xF,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk4 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0xF,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk4 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0xF,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk4 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk4 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0xF,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk4 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0xF,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk4 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0xF,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk4 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0xF,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk4 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0xF,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk4 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0xF,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk4 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0xF,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk4 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0xF,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk4 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk4 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0xF,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk4 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk4 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0xF,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk4 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0xF,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk4 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0xF,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk4 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0xF,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk4 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0xF,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk4 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk4 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0xF,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk4 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk4 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0xF,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk4 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk4 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0xF,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk4 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk4 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0xF,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk4 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk4 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0xF,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk4 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk4 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0xF,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk4 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk4 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0xF,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk4 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk4 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0xF,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk4 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk4 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0xF,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk4 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk4 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0xF,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk4 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk4 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0xF,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk4 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk4 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0xF,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk4 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk4 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0xF,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk4 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk4 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0xF,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk4 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk4 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0xF,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk4 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk4 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0xF,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk4 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk4 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0xF,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk4 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk4 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0xF,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk4 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk4 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0xF,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk4 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk4 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0xF,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk4 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk4 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0xF,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk4 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk4 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0xF,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk4 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk4 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0xF,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk4 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk4 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0xF,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk4 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk4 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0xF,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk4 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk4 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0xF,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk4 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk4 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0xF,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk4 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk4 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0xF,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk4 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk4 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0xF,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk4 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk4 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0xF,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk4 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk4 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0xF,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk4 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk4 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0xF,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk4 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk4 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0xF,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk4 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk4 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0xF,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk4 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk4 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0xF,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk4 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0xF,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk4 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0xF,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk4 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0xF,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk4 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0xF,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk4 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0xF,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk4 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0xF,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk4 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0xF,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk4 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0xF,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk4 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0xF,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk4 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0xF,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk4 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk4 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0xF,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk4 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0xF,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk4 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0xF,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk4 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0xF,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk4 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x1F,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk4 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk4 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x1F,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk4 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x1F,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk4 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk4 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x1F,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk4 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x1F,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk4 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk4 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x1F,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk4 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x1F,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk4 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk4 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x1F,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk4 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x1F,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk4 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk4 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x1F,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk4 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x1F,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk4 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk4 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x1F,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk4 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x1F,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk4 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk4 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x1F,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk4 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x1F,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk4 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk4 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x1F,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk4 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x1F,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk4 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk4 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x1F,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk4 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk4 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x1F,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk4 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x1F,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk4 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk4 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x1F,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk4 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk4 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x1F,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk4 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x1F,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk4 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk4 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x1F,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk4 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk4 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x1F,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk4 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x1F,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk4 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk4 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x1F,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk4 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk4 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x1F,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk4 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x1F,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk4 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk4 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x1F,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk4 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk4 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x1F,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk4 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x1F,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk4 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk4 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x1F,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk4 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk4 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x1F,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk4 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x1F,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk4 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk4 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x1F,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk4 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk4 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x1F,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk4 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x1F,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk4 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk4 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk4 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x1F,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk4 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk4 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x1F,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track5Parameters(TableBase):
	description = '''Track 5 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track5Parameters Track 5 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1286 = v.pop(0)
		self.midi_channel = byte1286 & 0x1F
		self.midi_status = (byte1286 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1295 = v.pop(0)
		byte1296 = v.pop(0)
		self.delay_tempo_base = byte1296 & 0xF
		self._reserved1296_6 = (byte1296 >> 4) & 0x7
		self.delay_mode = (byte1296 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1306 = v.pop(0)
		self.bus_select = byte1306 & 0x1F
		self.fx_bus_select = (byte1306 >> 5) & 0x3
		self._reserved1306_7 = (byte1306 >> 7) & 0x1
		self._reserved1307 = v.pop(0)
		self._reserved1308 = v.pop(0)
		byte1309 = v.pop(0)
		self.enable_program_change = byte1309 & 0x1
		self.enable_after_touch = (byte1309 >> 1) & 0x1
		self.enable_damper = (byte1309 >> 2) & 0x1
		self.enable_portamento = (byte1309 >> 3) & 0x1
		self.enable_joystick_x = (byte1309 >> 4) & 0x1
		self.enable_joystick_plus = (byte1309 >> 5) & 0x1
		self.enable_joystick_minus = (byte1309 >> 6) & 0x1
		self.enable_ribbon = (byte1309 >> 7) & 0x1
		byte1310 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1310 & 0x1
		self.enable_rt_knob_resonance = (byte1310 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1310 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1310 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1310 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1310 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1310 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1310 >> 7) & 0x1
		byte1311 = v.pop(0)
		self.enable_sw1 = byte1311 & 0x1
		self.enable_sw2 = (byte1311 >> 1) & 0x1
		self.enable_foot_switch = (byte1311 >> 2) & 0x1
		self.enable_foot_pedal = (byte1311 >> 3) & 0x1
		self.enable_other_control_change = (byte1311 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1311 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1311 >> 6) & 0x1
		self.enable_timbre_mute = (byte1311 >> 7) & 0x1
		byte1312 = v.pop(0)
		self.force_osc_mode = byte1312 & 0x3
		self.osc_select = (byte1312 >> 2) & 0x3
		self._reserved1312_4 = (byte1312 >> 4) & 0x1
		self.arpeggiator_assign = (byte1312 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1312 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1316 = v.pop(0)
		self.top_slope = byte1316 & 0xF
		self.bottom_slope = (byte1316 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1319 = v.pop(0)
		self.top_velocity_slope = byte1319 & 0xF
		self.bottom_velocity_slope = (byte1319 >> 4) & 0xF
		byte1320 = v.pop(0)
		self.trim_value = byte1320 & 0x7F
		self.trim_bypass = (byte1320 >> 7) & 0x1
		self._reserved1321 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1326 = v.pop(0)
		self._reserved1327 = v.pop(0)
		self._reserved1328 = v.pop(0)
		self._reserved1329 = v.pop(0)
		self._reserved1330 = v.pop(0)
		self._reserved1331 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1333 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1337 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1341 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1345 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1349 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1353 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1357 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1361 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track5Parameters Track 5 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1286 = 0
		byte1286 |= self.midi_channel & 0x1F
		byte1286 |= (self.midi_status & 0x7) << 5
		v.append(byte1286)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1295)
		byte1296 = 0
		byte1296 |= self.delay_tempo_base & 0xF
		byte1296 |= (self._reserved1296_6 & 0x7) << 4
		byte1296 |= (self.delay_mode & 0x1) << 7
		v.append(byte1296)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1306 = 0
		byte1306 |= self.bus_select & 0x1F
		byte1306 |= (self.fx_bus_select & 0x3) << 5
		byte1306 |= (self._reserved1306_7 & 0x1) << 7
		v.append(byte1306)
		v.append(self._reserved1307)
		v.append(self._reserved1308)
		byte1309 = 0
		byte1309 |= self.enable_program_change & 0x1
		byte1309 |= (self.enable_after_touch & 0x1) << 1
		byte1309 |= (self.enable_damper & 0x1) << 2
		byte1309 |= (self.enable_portamento & 0x1) << 3
		byte1309 |= (self.enable_joystick_x & 0x1) << 4
		byte1309 |= (self.enable_joystick_plus & 0x1) << 5
		byte1309 |= (self.enable_joystick_minus & 0x1) << 6
		byte1309 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1309)
		byte1310 = 0
		byte1310 |= self.enable_rt_knob_cutoff & 0x1
		byte1310 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1310 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1310 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1310 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1310 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1310 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1310 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1310)
		byte1311 = 0
		byte1311 |= self.enable_sw1 & 0x1
		byte1311 |= (self.enable_sw2 & 0x1) << 1
		byte1311 |= (self.enable_foot_switch & 0x1) << 2
		byte1311 |= (self.enable_foot_pedal & 0x1) << 3
		byte1311 |= (self.enable_other_control_change & 0x1) << 4
		byte1311 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1311 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1311 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1311)
		byte1312 = 0
		byte1312 |= self.force_osc_mode & 0x3
		byte1312 |= (self.osc_select & 0x3) << 2
		byte1312 |= (self._reserved1312_4 & 0x1) << 4
		byte1312 |= (self.arpeggiator_assign & 0x3) << 5
		byte1312 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1312)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1316 = 0
		byte1316 |= self.top_slope & 0xF
		byte1316 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1316)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1319 = 0
		byte1319 |= self.top_velocity_slope & 0xF
		byte1319 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1319)
		byte1320 = 0
		byte1320 |= self.trim_value & 0x7F
		byte1320 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1320)
		v.append(self._reserved1321)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1326)
		v.append(self._reserved1327)
		v.append(self._reserved1328)
		v.append(self._reserved1329)
		v.append(self._reserved1330)
		v.append(self._reserved1331)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1333)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1337)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1341)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1345)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1349)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1353)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1357)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1361)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track5Parameters Track 5 Parameters object'''
		json = []
		json.append(('name','Track5Parameters'))
		json.append(('description','Track 5 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk5 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x10,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk5 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x10,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk5 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x10,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk5 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk5 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x10,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk5 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x10,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk5 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x10,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk5 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x10,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk5 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x10,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk5 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x10,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk5 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x10,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk5 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x10,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk5 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk5 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x10,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk5 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk5 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x10,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk5 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x10,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk5 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x10,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk5 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x10,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk5 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x10,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk5 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk5 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x10,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk5 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk5 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x10,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk5 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk5 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x10,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk5 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk5 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x10,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk5 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk5 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x10,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk5 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk5 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x10,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk5 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk5 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x10,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk5 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk5 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x10,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk5 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk5 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x10,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk5 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk5 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x10,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk5 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk5 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x10,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk5 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk5 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x10,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk5 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk5 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x10,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk5 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk5 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x10,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk5 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk5 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x10,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk5 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk5 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x10,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk5 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk5 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x10,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk5 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk5 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x10,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk5 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk5 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x10,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk5 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk5 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x10,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk5 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk5 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x10,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk5 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk5 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x10,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk5 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk5 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x10,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk5 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk5 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x10,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk5 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk5 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x10,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk5 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk5 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x10,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk5 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk5 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x10,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk5 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk5 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x10,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk5 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk5 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x10,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk5 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk5 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x10,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk5 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk5 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x10,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk5 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk5 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x10,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk5 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk5 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x10,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk5 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk5 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x10,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk5 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk5 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x10,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk5 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x10,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk5 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x10,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk5 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x10,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk5 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x10,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk5 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x10,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk5 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x10,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk5 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x10,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk5 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x10,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk5 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x10,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk5 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x10,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk5 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk5 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x10,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk5 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x10,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk5 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x10,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk5 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x10,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk5 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x20,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk5 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk5 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x20,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk5 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x20,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk5 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk5 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x20,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk5 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x20,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk5 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk5 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x20,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk5 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x20,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk5 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk5 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x20,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk5 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x20,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk5 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk5 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x20,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk5 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x20,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk5 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk5 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x20,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk5 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x20,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk5 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk5 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x20,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk5 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x20,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk5 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk5 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x20,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk5 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x20,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk5 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk5 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x20,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk5 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk5 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x20,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk5 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x20,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk5 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk5 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x20,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk5 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk5 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x20,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk5 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x20,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk5 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk5 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x20,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk5 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk5 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x20,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk5 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x20,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk5 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk5 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x20,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk5 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk5 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x20,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk5 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x20,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk5 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk5 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x20,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk5 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk5 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x20,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk5 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x20,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk5 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk5 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x20,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk5 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk5 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x20,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk5 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x20,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk5 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk5 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x20,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk5 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk5 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x20,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk5 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x20,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk5 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk5 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk5 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x20,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk5 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk5 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x20,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track6Parameters(TableBase):
	description = '''Track 6 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track6Parameters Track 6 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1398 = v.pop(0)
		self.midi_channel = byte1398 & 0x1F
		self.midi_status = (byte1398 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1407 = v.pop(0)
		byte1408 = v.pop(0)
		self.delay_tempo_base = byte1408 & 0xF
		self._reserved1408_6 = (byte1408 >> 4) & 0x7
		self.delay_mode = (byte1408 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1418 = v.pop(0)
		self.bus_select = byte1418 & 0x1F
		self.fx_bus_select = (byte1418 >> 5) & 0x3
		self._reserved1418_7 = (byte1418 >> 7) & 0x1
		self._reserved1419 = v.pop(0)
		self._reserved1420 = v.pop(0)
		byte1421 = v.pop(0)
		self.enable_program_change = byte1421 & 0x1
		self.enable_after_touch = (byte1421 >> 1) & 0x1
		self.enable_damper = (byte1421 >> 2) & 0x1
		self.enable_portamento = (byte1421 >> 3) & 0x1
		self.enable_joystick_x = (byte1421 >> 4) & 0x1
		self.enable_joystick_plus = (byte1421 >> 5) & 0x1
		self.enable_joystick_minus = (byte1421 >> 6) & 0x1
		self.enable_ribbon = (byte1421 >> 7) & 0x1
		byte1422 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1422 & 0x1
		self.enable_rt_knob_resonance = (byte1422 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1422 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1422 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1422 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1422 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1422 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1422 >> 7) & 0x1
		byte1423 = v.pop(0)
		self.enable_sw1 = byte1423 & 0x1
		self.enable_sw2 = (byte1423 >> 1) & 0x1
		self.enable_foot_switch = (byte1423 >> 2) & 0x1
		self.enable_foot_pedal = (byte1423 >> 3) & 0x1
		self.enable_other_control_change = (byte1423 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1423 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1423 >> 6) & 0x1
		self.enable_timbre_mute = (byte1423 >> 7) & 0x1
		byte1424 = v.pop(0)
		self.force_osc_mode = byte1424 & 0x3
		self.osc_select = (byte1424 >> 2) & 0x3
		self._reserved1424_4 = (byte1424 >> 4) & 0x1
		self.arpeggiator_assign = (byte1424 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1424 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1428 = v.pop(0)
		self.top_slope = byte1428 & 0xF
		self.bottom_slope = (byte1428 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1431 = v.pop(0)
		self.top_velocity_slope = byte1431 & 0xF
		self.bottom_velocity_slope = (byte1431 >> 4) & 0xF
		byte1432 = v.pop(0)
		self.trim_value = byte1432 & 0x7F
		self.trim_bypass = (byte1432 >> 7) & 0x1
		self._reserved1433 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1438 = v.pop(0)
		self._reserved1439 = v.pop(0)
		self._reserved1440 = v.pop(0)
		self._reserved1441 = v.pop(0)
		self._reserved1442 = v.pop(0)
		self._reserved1443 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1445 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1449 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1453 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1457 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1461 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1465 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1469 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1473 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track6Parameters Track 6 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1398 = 0
		byte1398 |= self.midi_channel & 0x1F
		byte1398 |= (self.midi_status & 0x7) << 5
		v.append(byte1398)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1407)
		byte1408 = 0
		byte1408 |= self.delay_tempo_base & 0xF
		byte1408 |= (self._reserved1408_6 & 0x7) << 4
		byte1408 |= (self.delay_mode & 0x1) << 7
		v.append(byte1408)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1418 = 0
		byte1418 |= self.bus_select & 0x1F
		byte1418 |= (self.fx_bus_select & 0x3) << 5
		byte1418 |= (self._reserved1418_7 & 0x1) << 7
		v.append(byte1418)
		v.append(self._reserved1419)
		v.append(self._reserved1420)
		byte1421 = 0
		byte1421 |= self.enable_program_change & 0x1
		byte1421 |= (self.enable_after_touch & 0x1) << 1
		byte1421 |= (self.enable_damper & 0x1) << 2
		byte1421 |= (self.enable_portamento & 0x1) << 3
		byte1421 |= (self.enable_joystick_x & 0x1) << 4
		byte1421 |= (self.enable_joystick_plus & 0x1) << 5
		byte1421 |= (self.enable_joystick_minus & 0x1) << 6
		byte1421 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1421)
		byte1422 = 0
		byte1422 |= self.enable_rt_knob_cutoff & 0x1
		byte1422 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1422 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1422 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1422 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1422 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1422 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1422 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1422)
		byte1423 = 0
		byte1423 |= self.enable_sw1 & 0x1
		byte1423 |= (self.enable_sw2 & 0x1) << 1
		byte1423 |= (self.enable_foot_switch & 0x1) << 2
		byte1423 |= (self.enable_foot_pedal & 0x1) << 3
		byte1423 |= (self.enable_other_control_change & 0x1) << 4
		byte1423 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1423 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1423 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1423)
		byte1424 = 0
		byte1424 |= self.force_osc_mode & 0x3
		byte1424 |= (self.osc_select & 0x3) << 2
		byte1424 |= (self._reserved1424_4 & 0x1) << 4
		byte1424 |= (self.arpeggiator_assign & 0x3) << 5
		byte1424 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1424)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1428 = 0
		byte1428 |= self.top_slope & 0xF
		byte1428 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1428)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1431 = 0
		byte1431 |= self.top_velocity_slope & 0xF
		byte1431 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1431)
		byte1432 = 0
		byte1432 |= self.trim_value & 0x7F
		byte1432 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1432)
		v.append(self._reserved1433)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1438)
		v.append(self._reserved1439)
		v.append(self._reserved1440)
		v.append(self._reserved1441)
		v.append(self._reserved1442)
		v.append(self._reserved1443)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1445)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1449)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1453)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1457)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1461)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1465)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1469)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1473)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track6Parameters Track 6 Parameters object'''
		json = []
		json.append(('name','Track6Parameters'))
		json.append(('description','Track 6 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk6 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x11,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk6 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x11,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk6 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x11,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk6 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk6 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x11,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk6 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x11,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk6 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x11,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk6 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x11,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk6 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x11,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk6 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x11,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk6 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x11,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk6 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x11,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk6 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk6 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x11,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk6 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk6 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x11,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk6 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x11,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk6 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x11,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk6 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x11,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk6 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x11,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk6 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk6 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x11,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk6 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk6 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x11,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk6 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk6 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x11,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk6 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk6 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x11,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk6 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk6 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x11,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk6 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk6 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x11,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk6 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk6 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x11,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk6 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk6 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x11,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk6 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk6 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x11,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk6 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk6 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x11,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk6 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk6 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x11,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk6 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk6 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x11,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk6 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk6 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x11,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk6 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk6 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x11,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk6 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk6 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x11,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk6 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk6 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x11,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk6 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk6 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x11,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk6 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk6 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x11,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk6 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk6 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x11,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk6 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk6 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x11,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk6 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk6 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x11,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk6 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk6 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x11,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk6 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk6 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x11,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk6 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk6 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x11,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk6 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk6 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x11,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk6 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk6 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x11,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk6 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk6 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x11,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk6 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk6 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x11,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk6 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk6 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x11,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk6 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk6 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x11,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk6 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk6 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x11,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk6 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk6 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x11,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk6 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk6 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x11,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk6 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk6 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x11,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk6 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk6 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x11,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk6 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x11,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk6 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x11,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk6 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x11,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk6 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x11,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk6 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x11,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk6 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x11,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk6 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x11,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk6 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x11,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk6 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x11,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk6 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x11,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk6 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk6 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x11,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk6 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x11,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk6 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x11,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk6 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x11,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk6 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x21,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk6 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk6 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x21,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk6 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x21,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk6 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk6 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x21,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk6 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x21,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk6 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk6 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x21,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk6 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x21,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk6 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk6 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x21,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk6 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x21,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk6 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk6 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x21,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk6 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x21,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk6 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk6 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x21,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk6 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x21,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk6 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk6 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x21,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk6 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x21,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk6 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk6 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x21,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk6 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x21,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk6 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk6 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x21,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk6 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk6 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x21,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk6 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x21,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk6 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk6 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x21,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk6 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk6 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x21,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk6 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x21,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk6 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk6 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x21,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk6 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk6 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x21,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk6 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x21,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk6 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk6 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x21,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk6 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk6 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x21,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk6 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x21,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk6 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk6 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x21,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk6 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk6 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x21,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk6 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x21,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk6 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk6 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x21,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk6 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk6 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x21,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk6 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x21,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk6 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk6 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x21,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk6 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk6 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x21,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk6 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x21,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk6 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk6 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk6 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x21,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk6 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk6 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x21,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track7Parameters(TableBase):
	description = '''Track 7 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track7Parameters Track 7 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1510 = v.pop(0)
		self.midi_channel = byte1510 & 0x1F
		self.midi_status = (byte1510 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1519 = v.pop(0)
		byte1520 = v.pop(0)
		self.delay_tempo_base = byte1520 & 0xF
		self._reserved1520_6 = (byte1520 >> 4) & 0x7
		self.delay_mode = (byte1520 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1530 = v.pop(0)
		self.bus_select = byte1530 & 0x1F
		self.fx_bus_select = (byte1530 >> 5) & 0x3
		self._reserved1530_7 = (byte1530 >> 7) & 0x1
		self._reserved1531 = v.pop(0)
		self._reserved1532 = v.pop(0)
		byte1533 = v.pop(0)
		self.enable_program_change = byte1533 & 0x1
		self.enable_after_touch = (byte1533 >> 1) & 0x1
		self.enable_damper = (byte1533 >> 2) & 0x1
		self.enable_portamento = (byte1533 >> 3) & 0x1
		self.enable_joystick_x = (byte1533 >> 4) & 0x1
		self.enable_joystick_plus = (byte1533 >> 5) & 0x1
		self.enable_joystick_minus = (byte1533 >> 6) & 0x1
		self.enable_ribbon = (byte1533 >> 7) & 0x1
		byte1534 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1534 & 0x1
		self.enable_rt_knob_resonance = (byte1534 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1534 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1534 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1534 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1534 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1534 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1534 >> 7) & 0x1
		byte1535 = v.pop(0)
		self.enable_sw1 = byte1535 & 0x1
		self.enable_sw2 = (byte1535 >> 1) & 0x1
		self.enable_foot_switch = (byte1535 >> 2) & 0x1
		self.enable_foot_pedal = (byte1535 >> 3) & 0x1
		self.enable_other_control_change = (byte1535 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1535 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1535 >> 6) & 0x1
		self.enable_timbre_mute = (byte1535 >> 7) & 0x1
		byte1536 = v.pop(0)
		self.force_osc_mode = byte1536 & 0x3
		self.osc_select = (byte1536 >> 2) & 0x3
		self._reserved1536_4 = (byte1536 >> 4) & 0x1
		self.arpeggiator_assign = (byte1536 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1536 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1540 = v.pop(0)
		self.top_slope = byte1540 & 0xF
		self.bottom_slope = (byte1540 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1543 = v.pop(0)
		self.top_velocity_slope = byte1543 & 0xF
		self.bottom_velocity_slope = (byte1543 >> 4) & 0xF
		byte1544 = v.pop(0)
		self.trim_value = byte1544 & 0x7F
		self.trim_bypass = (byte1544 >> 7) & 0x1
		self._reserved1545 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1550 = v.pop(0)
		self._reserved1551 = v.pop(0)
		self._reserved1552 = v.pop(0)
		self._reserved1553 = v.pop(0)
		self._reserved1554 = v.pop(0)
		self._reserved1555 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1557 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1561 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1565 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1569 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1573 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1577 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1581 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1585 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track7Parameters Track 7 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1510 = 0
		byte1510 |= self.midi_channel & 0x1F
		byte1510 |= (self.midi_status & 0x7) << 5
		v.append(byte1510)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1519)
		byte1520 = 0
		byte1520 |= self.delay_tempo_base & 0xF
		byte1520 |= (self._reserved1520_6 & 0x7) << 4
		byte1520 |= (self.delay_mode & 0x1) << 7
		v.append(byte1520)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1530 = 0
		byte1530 |= self.bus_select & 0x1F
		byte1530 |= (self.fx_bus_select & 0x3) << 5
		byte1530 |= (self._reserved1530_7 & 0x1) << 7
		v.append(byte1530)
		v.append(self._reserved1531)
		v.append(self._reserved1532)
		byte1533 = 0
		byte1533 |= self.enable_program_change & 0x1
		byte1533 |= (self.enable_after_touch & 0x1) << 1
		byte1533 |= (self.enable_damper & 0x1) << 2
		byte1533 |= (self.enable_portamento & 0x1) << 3
		byte1533 |= (self.enable_joystick_x & 0x1) << 4
		byte1533 |= (self.enable_joystick_plus & 0x1) << 5
		byte1533 |= (self.enable_joystick_minus & 0x1) << 6
		byte1533 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1533)
		byte1534 = 0
		byte1534 |= self.enable_rt_knob_cutoff & 0x1
		byte1534 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1534 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1534 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1534 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1534 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1534 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1534 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1534)
		byte1535 = 0
		byte1535 |= self.enable_sw1 & 0x1
		byte1535 |= (self.enable_sw2 & 0x1) << 1
		byte1535 |= (self.enable_foot_switch & 0x1) << 2
		byte1535 |= (self.enable_foot_pedal & 0x1) << 3
		byte1535 |= (self.enable_other_control_change & 0x1) << 4
		byte1535 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1535 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1535 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1535)
		byte1536 = 0
		byte1536 |= self.force_osc_mode & 0x3
		byte1536 |= (self.osc_select & 0x3) << 2
		byte1536 |= (self._reserved1536_4 & 0x1) << 4
		byte1536 |= (self.arpeggiator_assign & 0x3) << 5
		byte1536 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1536)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1540 = 0
		byte1540 |= self.top_slope & 0xF
		byte1540 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1540)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1543 = 0
		byte1543 |= self.top_velocity_slope & 0xF
		byte1543 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1543)
		byte1544 = 0
		byte1544 |= self.trim_value & 0x7F
		byte1544 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1544)
		v.append(self._reserved1545)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1550)
		v.append(self._reserved1551)
		v.append(self._reserved1552)
		v.append(self._reserved1553)
		v.append(self._reserved1554)
		v.append(self._reserved1555)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1557)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1561)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1565)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1569)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1573)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1577)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1581)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1585)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track7Parameters Track 7 Parameters object'''
		json = []
		json.append(('name','Track7Parameters'))
		json.append(('description','Track 7 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk7 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x12,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk7 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x12,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk7 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x12,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk7 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk7 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x12,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk7 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x12,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk7 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x12,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk7 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x12,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk7 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x12,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk7 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x12,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk7 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x12,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk7 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x12,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk7 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk7 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x12,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk7 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk7 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x12,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk7 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x12,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk7 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x12,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk7 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x12,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk7 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x12,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk7 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk7 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x12,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk7 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk7 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x12,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk7 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk7 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x12,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk7 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk7 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x12,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk7 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk7 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x12,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk7 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk7 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x12,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk7 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk7 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x12,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk7 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk7 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x12,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk7 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk7 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x12,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk7 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk7 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x12,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk7 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk7 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x12,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk7 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk7 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x12,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk7 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk7 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x12,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk7 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk7 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x12,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk7 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk7 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x12,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk7 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk7 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x12,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk7 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk7 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x12,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk7 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk7 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x12,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk7 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk7 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x12,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk7 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk7 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x12,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk7 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk7 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x12,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk7 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk7 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x12,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk7 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk7 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x12,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk7 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk7 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x12,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk7 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk7 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x12,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk7 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk7 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x12,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk7 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk7 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x12,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk7 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk7 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x12,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk7 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk7 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x12,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk7 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk7 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x12,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk7 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk7 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x12,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk7 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk7 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x12,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk7 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk7 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x12,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk7 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk7 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x12,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk7 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk7 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x12,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk7 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x12,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk7 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x12,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk7 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x12,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk7 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x12,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk7 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x12,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk7 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x12,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk7 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x12,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk7 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x12,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk7 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x12,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk7 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x12,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk7 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk7 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x12,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk7 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x12,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk7 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x12,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk7 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x12,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk7 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x22,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk7 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk7 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x22,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk7 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x22,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk7 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk7 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x22,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk7 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x22,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk7 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk7 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x22,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk7 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x22,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk7 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk7 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x22,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk7 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x22,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk7 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk7 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x22,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk7 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x22,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk7 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk7 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x22,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk7 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x22,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk7 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk7 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x22,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk7 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x22,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk7 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk7 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x22,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk7 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x22,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk7 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk7 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x22,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk7 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk7 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x22,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk7 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x22,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk7 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk7 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x22,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk7 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk7 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x22,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk7 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x22,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk7 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk7 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x22,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk7 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk7 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x22,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk7 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x22,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk7 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk7 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x22,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk7 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk7 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x22,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk7 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x22,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk7 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk7 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x22,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk7 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk7 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x22,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk7 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x22,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk7 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk7 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x22,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk7 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk7 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x22,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk7 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x22,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk7 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk7 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x22,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk7 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk7 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x22,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk7 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x22,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk7 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk7 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk7 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x22,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk7 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk7 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x22,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track8Parameters(TableBase):
	description = '''Track 8 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track8Parameters Track 8 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1622 = v.pop(0)
		self.midi_channel = byte1622 & 0x1F
		self.midi_status = (byte1622 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1631 = v.pop(0)
		byte1632 = v.pop(0)
		self.delay_tempo_base = byte1632 & 0xF
		self._reserved1632_6 = (byte1632 >> 4) & 0x7
		self.delay_mode = (byte1632 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1642 = v.pop(0)
		self.bus_select = byte1642 & 0x1F
		self.fx_bus_select = (byte1642 >> 5) & 0x3
		self._reserved1642_7 = (byte1642 >> 7) & 0x1
		self._reserved1643 = v.pop(0)
		self._reserved1644 = v.pop(0)
		byte1645 = v.pop(0)
		self.enable_program_change = byte1645 & 0x1
		self.enable_after_touch = (byte1645 >> 1) & 0x1
		self.enable_damper = (byte1645 >> 2) & 0x1
		self.enable_portamento = (byte1645 >> 3) & 0x1
		self.enable_joystick_x = (byte1645 >> 4) & 0x1
		self.enable_joystick_plus = (byte1645 >> 5) & 0x1
		self.enable_joystick_minus = (byte1645 >> 6) & 0x1
		self.enable_ribbon = (byte1645 >> 7) & 0x1
		byte1646 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1646 & 0x1
		self.enable_rt_knob_resonance = (byte1646 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1646 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1646 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1646 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1646 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1646 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1646 >> 7) & 0x1
		byte1647 = v.pop(0)
		self.enable_sw1 = byte1647 & 0x1
		self.enable_sw2 = (byte1647 >> 1) & 0x1
		self.enable_foot_switch = (byte1647 >> 2) & 0x1
		self.enable_foot_pedal = (byte1647 >> 3) & 0x1
		self.enable_other_control_change = (byte1647 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1647 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1647 >> 6) & 0x1
		self.enable_timbre_mute = (byte1647 >> 7) & 0x1
		byte1648 = v.pop(0)
		self.force_osc_mode = byte1648 & 0x3
		self.osc_select = (byte1648 >> 2) & 0x3
		self._reserved1648_4 = (byte1648 >> 4) & 0x1
		self.arpeggiator_assign = (byte1648 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1648 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1652 = v.pop(0)
		self.top_slope = byte1652 & 0xF
		self.bottom_slope = (byte1652 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1655 = v.pop(0)
		self.top_velocity_slope = byte1655 & 0xF
		self.bottom_velocity_slope = (byte1655 >> 4) & 0xF
		byte1656 = v.pop(0)
		self.trim_value = byte1656 & 0x7F
		self.trim_bypass = (byte1656 >> 7) & 0x1
		self._reserved1657 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1662 = v.pop(0)
		self._reserved1663 = v.pop(0)
		self._reserved1664 = v.pop(0)
		self._reserved1665 = v.pop(0)
		self._reserved1666 = v.pop(0)
		self._reserved1667 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1669 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1673 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1677 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1681 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1685 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1689 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1693 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1697 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track8Parameters Track 8 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1622 = 0
		byte1622 |= self.midi_channel & 0x1F
		byte1622 |= (self.midi_status & 0x7) << 5
		v.append(byte1622)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1631)
		byte1632 = 0
		byte1632 |= self.delay_tempo_base & 0xF
		byte1632 |= (self._reserved1632_6 & 0x7) << 4
		byte1632 |= (self.delay_mode & 0x1) << 7
		v.append(byte1632)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1642 = 0
		byte1642 |= self.bus_select & 0x1F
		byte1642 |= (self.fx_bus_select & 0x3) << 5
		byte1642 |= (self._reserved1642_7 & 0x1) << 7
		v.append(byte1642)
		v.append(self._reserved1643)
		v.append(self._reserved1644)
		byte1645 = 0
		byte1645 |= self.enable_program_change & 0x1
		byte1645 |= (self.enable_after_touch & 0x1) << 1
		byte1645 |= (self.enable_damper & 0x1) << 2
		byte1645 |= (self.enable_portamento & 0x1) << 3
		byte1645 |= (self.enable_joystick_x & 0x1) << 4
		byte1645 |= (self.enable_joystick_plus & 0x1) << 5
		byte1645 |= (self.enable_joystick_minus & 0x1) << 6
		byte1645 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1645)
		byte1646 = 0
		byte1646 |= self.enable_rt_knob_cutoff & 0x1
		byte1646 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1646 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1646 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1646 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1646 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1646 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1646 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1646)
		byte1647 = 0
		byte1647 |= self.enable_sw1 & 0x1
		byte1647 |= (self.enable_sw2 & 0x1) << 1
		byte1647 |= (self.enable_foot_switch & 0x1) << 2
		byte1647 |= (self.enable_foot_pedal & 0x1) << 3
		byte1647 |= (self.enable_other_control_change & 0x1) << 4
		byte1647 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1647 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1647 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1647)
		byte1648 = 0
		byte1648 |= self.force_osc_mode & 0x3
		byte1648 |= (self.osc_select & 0x3) << 2
		byte1648 |= (self._reserved1648_4 & 0x1) << 4
		byte1648 |= (self.arpeggiator_assign & 0x3) << 5
		byte1648 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1648)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1652 = 0
		byte1652 |= self.top_slope & 0xF
		byte1652 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1652)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1655 = 0
		byte1655 |= self.top_velocity_slope & 0xF
		byte1655 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1655)
		byte1656 = 0
		byte1656 |= self.trim_value & 0x7F
		byte1656 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1656)
		v.append(self._reserved1657)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1662)
		v.append(self._reserved1663)
		v.append(self._reserved1664)
		v.append(self._reserved1665)
		v.append(self._reserved1666)
		v.append(self._reserved1667)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1669)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1673)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1677)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1681)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1685)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1689)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1693)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1697)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track8Parameters Track 8 Parameters object'''
		json = []
		json.append(('name','Track8Parameters'))
		json.append(('description','Track 8 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk8 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x13,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk8 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x13,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk8 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x13,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk8 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk8 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x13,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk8 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x13,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk8 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x13,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk8 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x13,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk8 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x13,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk8 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x13,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk8 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x13,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk8 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x13,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk8 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk8 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x13,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk8 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk8 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x13,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk8 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x13,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk8 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x13,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk8 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x13,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk8 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x13,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk8 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk8 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x13,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk8 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk8 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x13,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk8 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk8 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x13,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk8 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk8 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x13,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk8 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk8 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x13,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk8 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk8 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x13,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk8 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk8 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x13,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk8 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk8 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x13,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk8 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk8 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x13,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk8 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk8 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x13,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk8 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk8 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x13,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk8 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk8 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x13,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk8 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk8 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x13,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk8 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk8 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x13,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk8 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk8 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x13,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk8 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk8 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x13,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk8 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk8 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x13,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk8 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk8 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x13,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk8 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk8 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x13,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk8 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk8 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x13,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk8 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk8 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x13,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk8 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk8 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x13,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk8 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk8 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x13,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk8 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk8 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x13,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk8 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk8 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x13,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk8 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk8 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x13,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk8 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk8 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x13,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk8 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk8 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x13,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk8 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk8 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x13,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk8 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk8 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x13,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk8 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk8 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x13,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk8 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk8 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x13,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk8 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk8 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x13,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk8 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk8 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x13,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk8 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk8 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x13,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk8 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x13,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk8 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x13,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk8 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x13,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk8 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x13,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk8 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x13,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk8 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x13,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk8 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x13,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk8 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x13,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk8 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x13,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk8 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x13,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk8 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk8 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x13,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk8 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x13,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk8 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x13,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk8 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x13,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk8 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x23,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk8 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk8 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x23,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk8 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x23,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk8 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk8 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x23,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk8 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x23,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk8 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk8 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x23,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk8 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x23,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk8 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk8 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x23,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk8 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x23,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk8 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk8 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x23,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk8 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x23,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk8 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk8 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x23,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk8 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x23,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk8 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk8 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x23,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk8 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x23,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk8 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk8 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x23,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk8 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x23,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk8 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk8 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x23,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk8 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk8 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x23,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk8 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x23,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk8 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk8 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x23,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk8 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk8 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x23,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk8 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x23,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk8 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk8 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x23,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk8 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk8 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x23,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk8 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x23,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk8 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk8 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x23,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk8 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk8 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x23,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk8 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x23,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk8 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk8 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x23,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk8 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk8 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x23,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk8 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x23,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk8 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk8 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x23,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk8 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk8 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x23,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk8 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x23,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk8 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk8 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x23,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk8 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk8 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x23,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk8 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x23,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk8 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk8 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk8 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x23,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk8 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk8 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x23,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track9Parameters(TableBase):
	description = '''Track 9 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track9Parameters Track 9 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1734 = v.pop(0)
		self.midi_channel = byte1734 & 0x1F
		self.midi_status = (byte1734 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1743 = v.pop(0)
		byte1744 = v.pop(0)
		self.delay_tempo_base = byte1744 & 0xF
		self._reserved1744_6 = (byte1744 >> 4) & 0x7
		self.delay_mode = (byte1744 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1754 = v.pop(0)
		self.bus_select = byte1754 & 0x1F
		self.fx_bus_select = (byte1754 >> 5) & 0x3
		self._reserved1754_7 = (byte1754 >> 7) & 0x1
		self._reserved1755 = v.pop(0)
		self._reserved1756 = v.pop(0)
		byte1757 = v.pop(0)
		self.enable_program_change = byte1757 & 0x1
		self.enable_after_touch = (byte1757 >> 1) & 0x1
		self.enable_damper = (byte1757 >> 2) & 0x1
		self.enable_portamento = (byte1757 >> 3) & 0x1
		self.enable_joystick_x = (byte1757 >> 4) & 0x1
		self.enable_joystick_plus = (byte1757 >> 5) & 0x1
		self.enable_joystick_minus = (byte1757 >> 6) & 0x1
		self.enable_ribbon = (byte1757 >> 7) & 0x1
		byte1758 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1758 & 0x1
		self.enable_rt_knob_resonance = (byte1758 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1758 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1758 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1758 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1758 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1758 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1758 >> 7) & 0x1
		byte1759 = v.pop(0)
		self.enable_sw1 = byte1759 & 0x1
		self.enable_sw2 = (byte1759 >> 1) & 0x1
		self.enable_foot_switch = (byte1759 >> 2) & 0x1
		self.enable_foot_pedal = (byte1759 >> 3) & 0x1
		self.enable_other_control_change = (byte1759 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1759 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1759 >> 6) & 0x1
		self.enable_timbre_mute = (byte1759 >> 7) & 0x1
		byte1760 = v.pop(0)
		self.force_osc_mode = byte1760 & 0x3
		self.osc_select = (byte1760 >> 2) & 0x3
		self._reserved1760_4 = (byte1760 >> 4) & 0x1
		self.arpeggiator_assign = (byte1760 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1760 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1764 = v.pop(0)
		self.top_slope = byte1764 & 0xF
		self.bottom_slope = (byte1764 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1767 = v.pop(0)
		self.top_velocity_slope = byte1767 & 0xF
		self.bottom_velocity_slope = (byte1767 >> 4) & 0xF
		byte1768 = v.pop(0)
		self.trim_value = byte1768 & 0x7F
		self.trim_bypass = (byte1768 >> 7) & 0x1
		self._reserved1769 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1774 = v.pop(0)
		self._reserved1775 = v.pop(0)
		self._reserved1776 = v.pop(0)
		self._reserved1777 = v.pop(0)
		self._reserved1778 = v.pop(0)
		self._reserved1779 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1781 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1785 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1789 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1793 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1797 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1801 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1805 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1809 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track9Parameters Track 9 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1734 = 0
		byte1734 |= self.midi_channel & 0x1F
		byte1734 |= (self.midi_status & 0x7) << 5
		v.append(byte1734)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1743)
		byte1744 = 0
		byte1744 |= self.delay_tempo_base & 0xF
		byte1744 |= (self._reserved1744_6 & 0x7) << 4
		byte1744 |= (self.delay_mode & 0x1) << 7
		v.append(byte1744)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1754 = 0
		byte1754 |= self.bus_select & 0x1F
		byte1754 |= (self.fx_bus_select & 0x3) << 5
		byte1754 |= (self._reserved1754_7 & 0x1) << 7
		v.append(byte1754)
		v.append(self._reserved1755)
		v.append(self._reserved1756)
		byte1757 = 0
		byte1757 |= self.enable_program_change & 0x1
		byte1757 |= (self.enable_after_touch & 0x1) << 1
		byte1757 |= (self.enable_damper & 0x1) << 2
		byte1757 |= (self.enable_portamento & 0x1) << 3
		byte1757 |= (self.enable_joystick_x & 0x1) << 4
		byte1757 |= (self.enable_joystick_plus & 0x1) << 5
		byte1757 |= (self.enable_joystick_minus & 0x1) << 6
		byte1757 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1757)
		byte1758 = 0
		byte1758 |= self.enable_rt_knob_cutoff & 0x1
		byte1758 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1758 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1758 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1758 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1758 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1758 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1758 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1758)
		byte1759 = 0
		byte1759 |= self.enable_sw1 & 0x1
		byte1759 |= (self.enable_sw2 & 0x1) << 1
		byte1759 |= (self.enable_foot_switch & 0x1) << 2
		byte1759 |= (self.enable_foot_pedal & 0x1) << 3
		byte1759 |= (self.enable_other_control_change & 0x1) << 4
		byte1759 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1759 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1759 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1759)
		byte1760 = 0
		byte1760 |= self.force_osc_mode & 0x3
		byte1760 |= (self.osc_select & 0x3) << 2
		byte1760 |= (self._reserved1760_4 & 0x1) << 4
		byte1760 |= (self.arpeggiator_assign & 0x3) << 5
		byte1760 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1760)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1764 = 0
		byte1764 |= self.top_slope & 0xF
		byte1764 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1764)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1767 = 0
		byte1767 |= self.top_velocity_slope & 0xF
		byte1767 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1767)
		byte1768 = 0
		byte1768 |= self.trim_value & 0x7F
		byte1768 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1768)
		v.append(self._reserved1769)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1774)
		v.append(self._reserved1775)
		v.append(self._reserved1776)
		v.append(self._reserved1777)
		v.append(self._reserved1778)
		v.append(self._reserved1779)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1781)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1785)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1789)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1793)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1797)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1801)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1805)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1809)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track9Parameters Track 9 Parameters object'''
		json = []
		json.append(('name','Track9Parameters'))
		json.append(('description','Track 9 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk9 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x14,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk9 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x14,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk9 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x14,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk9 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk9 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x14,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk9 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x14,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk9 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x14,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk9 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x14,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk9 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x14,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk9 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x14,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk9 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x14,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk9 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x14,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk9 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk9 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x14,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk9 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk9 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x14,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk9 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x14,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk9 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x14,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk9 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x14,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk9 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x14,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk9 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk9 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x14,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk9 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk9 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x14,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk9 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk9 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x14,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk9 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk9 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x14,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk9 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk9 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x14,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk9 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk9 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x14,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk9 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk9 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x14,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk9 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk9 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x14,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk9 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk9 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x14,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk9 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk9 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x14,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk9 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk9 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x14,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk9 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk9 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x14,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk9 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk9 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x14,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk9 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk9 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x14,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk9 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk9 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x14,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk9 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk9 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x14,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk9 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk9 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x14,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk9 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk9 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x14,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk9 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk9 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x14,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk9 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk9 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x14,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk9 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk9 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x14,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk9 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk9 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x14,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk9 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk9 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x14,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk9 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk9 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x14,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk9 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk9 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x14,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk9 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk9 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x14,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk9 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk9 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x14,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk9 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk9 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x14,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk9 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk9 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x14,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk9 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk9 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x14,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk9 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk9 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x14,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk9 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk9 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x14,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk9 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk9 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x14,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk9 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk9 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x14,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk9 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk9 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x14,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk9 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x14,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk9 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x14,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk9 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x14,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk9 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x14,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk9 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x14,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk9 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x14,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk9 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x14,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk9 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x14,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk9 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x14,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk9 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x14,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk9 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk9 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x14,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk9 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x14,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk9 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x14,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk9 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x14,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk9 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x24,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk9 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk9 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x24,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk9 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x24,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk9 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk9 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x24,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk9 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x24,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk9 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk9 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x24,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk9 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x24,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk9 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk9 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x24,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk9 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x24,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk9 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk9 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x24,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk9 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x24,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk9 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk9 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x24,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk9 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x24,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk9 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk9 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x24,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk9 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x24,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk9 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk9 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x24,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk9 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x24,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk9 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk9 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x24,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk9 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk9 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x24,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk9 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x24,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk9 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk9 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x24,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk9 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk9 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x24,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk9 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x24,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk9 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk9 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x24,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk9 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk9 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x24,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk9 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x24,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk9 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk9 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x24,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk9 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk9 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x24,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk9 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x24,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk9 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk9 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x24,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk9 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk9 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x24,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk9 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x24,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk9 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk9 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x24,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk9 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk9 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x24,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk9 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x24,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk9 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk9 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x24,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk9 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk9 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x24,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk9 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x24,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk9 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk9 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk9 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x24,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk9 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk9 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x24,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track10Parameters(TableBase):
	description = '''Track 10 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track10Parameters Track 10 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1846 = v.pop(0)
		self.midi_channel = byte1846 & 0x1F
		self.midi_status = (byte1846 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1855 = v.pop(0)
		byte1856 = v.pop(0)
		self.delay_tempo_base = byte1856 & 0xF
		self._reserved1856_6 = (byte1856 >> 4) & 0x7
		self.delay_mode = (byte1856 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1866 = v.pop(0)
		self.bus_select = byte1866 & 0x1F
		self.fx_bus_select = (byte1866 >> 5) & 0x3
		self._reserved1866_7 = (byte1866 >> 7) & 0x1
		self._reserved1867 = v.pop(0)
		self._reserved1868 = v.pop(0)
		byte1869 = v.pop(0)
		self.enable_program_change = byte1869 & 0x1
		self.enable_after_touch = (byte1869 >> 1) & 0x1
		self.enable_damper = (byte1869 >> 2) & 0x1
		self.enable_portamento = (byte1869 >> 3) & 0x1
		self.enable_joystick_x = (byte1869 >> 4) & 0x1
		self.enable_joystick_plus = (byte1869 >> 5) & 0x1
		self.enable_joystick_minus = (byte1869 >> 6) & 0x1
		self.enable_ribbon = (byte1869 >> 7) & 0x1
		byte1870 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1870 & 0x1
		self.enable_rt_knob_resonance = (byte1870 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1870 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1870 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1870 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1870 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1870 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1870 >> 7) & 0x1
		byte1871 = v.pop(0)
		self.enable_sw1 = byte1871 & 0x1
		self.enable_sw2 = (byte1871 >> 1) & 0x1
		self.enable_foot_switch = (byte1871 >> 2) & 0x1
		self.enable_foot_pedal = (byte1871 >> 3) & 0x1
		self.enable_other_control_change = (byte1871 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1871 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1871 >> 6) & 0x1
		self.enable_timbre_mute = (byte1871 >> 7) & 0x1
		byte1872 = v.pop(0)
		self.force_osc_mode = byte1872 & 0x3
		self.osc_select = (byte1872 >> 2) & 0x3
		self._reserved1872_4 = (byte1872 >> 4) & 0x1
		self.arpeggiator_assign = (byte1872 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1872 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1876 = v.pop(0)
		self.top_slope = byte1876 & 0xF
		self.bottom_slope = (byte1876 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1879 = v.pop(0)
		self.top_velocity_slope = byte1879 & 0xF
		self.bottom_velocity_slope = (byte1879 >> 4) & 0xF
		byte1880 = v.pop(0)
		self.trim_value = byte1880 & 0x7F
		self.trim_bypass = (byte1880 >> 7) & 0x1
		self._reserved1881 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1886 = v.pop(0)
		self._reserved1887 = v.pop(0)
		self._reserved1888 = v.pop(0)
		self._reserved1889 = v.pop(0)
		self._reserved1890 = v.pop(0)
		self._reserved1891 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved1893 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved1897 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved1901 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved1905 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved1909 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved1913 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved1917 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved1921 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track10Parameters Track 10 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1846 = 0
		byte1846 |= self.midi_channel & 0x1F
		byte1846 |= (self.midi_status & 0x7) << 5
		v.append(byte1846)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1855)
		byte1856 = 0
		byte1856 |= self.delay_tempo_base & 0xF
		byte1856 |= (self._reserved1856_6 & 0x7) << 4
		byte1856 |= (self.delay_mode & 0x1) << 7
		v.append(byte1856)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1866 = 0
		byte1866 |= self.bus_select & 0x1F
		byte1866 |= (self.fx_bus_select & 0x3) << 5
		byte1866 |= (self._reserved1866_7 & 0x1) << 7
		v.append(byte1866)
		v.append(self._reserved1867)
		v.append(self._reserved1868)
		byte1869 = 0
		byte1869 |= self.enable_program_change & 0x1
		byte1869 |= (self.enable_after_touch & 0x1) << 1
		byte1869 |= (self.enable_damper & 0x1) << 2
		byte1869 |= (self.enable_portamento & 0x1) << 3
		byte1869 |= (self.enable_joystick_x & 0x1) << 4
		byte1869 |= (self.enable_joystick_plus & 0x1) << 5
		byte1869 |= (self.enable_joystick_minus & 0x1) << 6
		byte1869 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1869)
		byte1870 = 0
		byte1870 |= self.enable_rt_knob_cutoff & 0x1
		byte1870 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1870 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1870 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1870 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1870 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1870 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1870 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1870)
		byte1871 = 0
		byte1871 |= self.enable_sw1 & 0x1
		byte1871 |= (self.enable_sw2 & 0x1) << 1
		byte1871 |= (self.enable_foot_switch & 0x1) << 2
		byte1871 |= (self.enable_foot_pedal & 0x1) << 3
		byte1871 |= (self.enable_other_control_change & 0x1) << 4
		byte1871 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1871 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1871 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1871)
		byte1872 = 0
		byte1872 |= self.force_osc_mode & 0x3
		byte1872 |= (self.osc_select & 0x3) << 2
		byte1872 |= (self._reserved1872_4 & 0x1) << 4
		byte1872 |= (self.arpeggiator_assign & 0x3) << 5
		byte1872 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1872)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1876 = 0
		byte1876 |= self.top_slope & 0xF
		byte1876 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1876)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1879 = 0
		byte1879 |= self.top_velocity_slope & 0xF
		byte1879 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1879)
		byte1880 = 0
		byte1880 |= self.trim_value & 0x7F
		byte1880 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1880)
		v.append(self._reserved1881)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1886)
		v.append(self._reserved1887)
		v.append(self._reserved1888)
		v.append(self._reserved1889)
		v.append(self._reserved1890)
		v.append(self._reserved1891)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved1893)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved1897)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved1901)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved1905)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved1909)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved1913)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved1917)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved1921)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track10Parameters Track 10 Parameters object'''
		json = []
		json.append(('name','Track10Parameters'))
		json.append(('description','Track 10 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk10 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x15,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk10 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x15,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk10 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x15,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk10 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk10 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x15,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk10 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x15,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk10 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x15,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk10 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x15,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk10 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x15,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk10 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x15,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk10 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x15,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk10 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x15,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk10 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk10 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x15,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk10 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk10 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x15,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk10 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x15,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk10 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x15,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk10 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x15,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk10 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x15,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk10 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk10 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x15,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk10 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk10 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x15,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk10 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk10 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x15,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk10 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk10 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x15,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk10 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk10 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x15,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk10 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk10 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x15,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk10 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk10 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x15,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk10 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk10 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x15,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk10 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk10 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x15,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk10 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk10 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x15,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk10 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk10 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x15,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk10 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk10 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x15,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk10 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk10 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x15,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk10 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk10 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x15,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk10 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk10 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x15,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk10 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk10 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x15,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk10 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk10 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x15,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk10 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk10 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x15,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk10 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk10 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x15,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk10 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk10 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x15,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk10 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk10 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x15,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk10 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk10 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x15,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk10 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk10 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x15,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk10 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk10 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x15,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk10 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk10 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x15,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk10 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk10 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x15,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk10 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk10 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x15,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk10 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk10 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x15,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk10 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk10 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x15,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk10 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk10 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x15,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk10 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk10 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x15,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk10 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk10 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x15,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk10 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk10 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x15,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk10 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk10 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x15,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk10 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk10 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x15,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk10 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x15,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk10 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x15,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk10 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x15,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk10 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x15,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk10 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x15,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk10 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x15,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk10 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x15,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk10 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x15,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk10 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x15,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk10 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x15,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk10 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk10 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x15,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk10 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x15,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk10 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x15,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk10 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x15,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk10 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x25,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk10 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk10 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x25,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk10 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x25,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk10 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk10 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x25,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk10 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x25,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk10 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk10 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x25,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk10 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x25,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk10 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk10 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x25,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk10 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x25,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk10 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk10 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x25,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk10 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x25,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk10 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk10 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x25,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk10 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x25,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk10 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk10 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x25,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk10 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x25,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk10 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk10 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x25,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk10 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x25,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk10 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk10 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x25,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk10 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk10 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x25,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk10 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x25,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk10 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk10 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x25,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk10 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk10 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x25,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk10 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x25,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk10 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk10 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x25,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk10 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk10 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x25,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk10 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x25,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk10 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk10 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x25,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk10 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk10 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x25,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk10 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x25,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk10 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk10 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x25,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk10 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk10 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x25,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk10 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x25,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk10 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk10 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x25,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk10 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk10 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x25,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk10 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x25,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk10 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk10 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x25,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk10 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk10 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x25,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk10 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x25,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk10 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk10 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk10 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x25,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk10 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk10 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x25,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track11Parameters(TableBase):
	description = '''Track 11 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track11Parameters Track 11 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte1958 = v.pop(0)
		self.midi_channel = byte1958 & 0x1F
		self.midi_status = (byte1958 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved1967 = v.pop(0)
		byte1968 = v.pop(0)
		self.delay_tempo_base = byte1968 & 0xF
		self._reserved1968_6 = (byte1968 >> 4) & 0x7
		self.delay_mode = (byte1968 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte1978 = v.pop(0)
		self.bus_select = byte1978 & 0x1F
		self.fx_bus_select = (byte1978 >> 5) & 0x3
		self._reserved1978_7 = (byte1978 >> 7) & 0x1
		self._reserved1979 = v.pop(0)
		self._reserved1980 = v.pop(0)
		byte1981 = v.pop(0)
		self.enable_program_change = byte1981 & 0x1
		self.enable_after_touch = (byte1981 >> 1) & 0x1
		self.enable_damper = (byte1981 >> 2) & 0x1
		self.enable_portamento = (byte1981 >> 3) & 0x1
		self.enable_joystick_x = (byte1981 >> 4) & 0x1
		self.enable_joystick_plus = (byte1981 >> 5) & 0x1
		self.enable_joystick_minus = (byte1981 >> 6) & 0x1
		self.enable_ribbon = (byte1981 >> 7) & 0x1
		byte1982 = v.pop(0)
		self.enable_rt_knob_cutoff = byte1982 & 0x1
		self.enable_rt_knob_resonance = (byte1982 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte1982 >> 2) & 0x1
		self.enable_rt_knob_release = (byte1982 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte1982 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte1982 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte1982 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte1982 >> 7) & 0x1
		byte1983 = v.pop(0)
		self.enable_sw1 = byte1983 & 0x1
		self.enable_sw2 = (byte1983 >> 1) & 0x1
		self.enable_foot_switch = (byte1983 >> 2) & 0x1
		self.enable_foot_pedal = (byte1983 >> 3) & 0x1
		self.enable_other_control_change = (byte1983 >> 4) & 0x1
		self.enable_use_prg_scales = (byte1983 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte1983 >> 6) & 0x1
		self.enable_timbre_mute = (byte1983 >> 7) & 0x1
		byte1984 = v.pop(0)
		self.force_osc_mode = byte1984 & 0x3
		self.osc_select = (byte1984 >> 2) & 0x3
		self._reserved1984_4 = (byte1984 >> 4) & 0x1
		self.arpeggiator_assign = (byte1984 >> 5) & 0x3
		self.enable_waveform_sysex = (byte1984 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte1988 = v.pop(0)
		self.top_slope = byte1988 & 0xF
		self.bottom_slope = (byte1988 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte1991 = v.pop(0)
		self.top_velocity_slope = byte1991 & 0xF
		self.bottom_velocity_slope = (byte1991 >> 4) & 0xF
		byte1992 = v.pop(0)
		self.trim_value = byte1992 & 0x7F
		self.trim_bypass = (byte1992 >> 7) & 0x1
		self._reserved1993 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved1998 = v.pop(0)
		self._reserved1999 = v.pop(0)
		self._reserved2000 = v.pop(0)
		self._reserved2001 = v.pop(0)
		self._reserved2002 = v.pop(0)
		self._reserved2003 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2005 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2009 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2013 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2017 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2021 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2025 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2029 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2033 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track11Parameters Track 11 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte1958 = 0
		byte1958 |= self.midi_channel & 0x1F
		byte1958 |= (self.midi_status & 0x7) << 5
		v.append(byte1958)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved1967)
		byte1968 = 0
		byte1968 |= self.delay_tempo_base & 0xF
		byte1968 |= (self._reserved1968_6 & 0x7) << 4
		byte1968 |= (self.delay_mode & 0x1) << 7
		v.append(byte1968)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte1978 = 0
		byte1978 |= self.bus_select & 0x1F
		byte1978 |= (self.fx_bus_select & 0x3) << 5
		byte1978 |= (self._reserved1978_7 & 0x1) << 7
		v.append(byte1978)
		v.append(self._reserved1979)
		v.append(self._reserved1980)
		byte1981 = 0
		byte1981 |= self.enable_program_change & 0x1
		byte1981 |= (self.enable_after_touch & 0x1) << 1
		byte1981 |= (self.enable_damper & 0x1) << 2
		byte1981 |= (self.enable_portamento & 0x1) << 3
		byte1981 |= (self.enable_joystick_x & 0x1) << 4
		byte1981 |= (self.enable_joystick_plus & 0x1) << 5
		byte1981 |= (self.enable_joystick_minus & 0x1) << 6
		byte1981 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte1981)
		byte1982 = 0
		byte1982 |= self.enable_rt_knob_cutoff & 0x1
		byte1982 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte1982 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte1982 |= (self.enable_rt_knob_release & 0x1) << 3
		byte1982 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte1982 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte1982 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte1982 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte1982)
		byte1983 = 0
		byte1983 |= self.enable_sw1 & 0x1
		byte1983 |= (self.enable_sw2 & 0x1) << 1
		byte1983 |= (self.enable_foot_switch & 0x1) << 2
		byte1983 |= (self.enable_foot_pedal & 0x1) << 3
		byte1983 |= (self.enable_other_control_change & 0x1) << 4
		byte1983 |= (self.enable_use_prg_scales & 0x1) << 5
		byte1983 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte1983 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte1983)
		byte1984 = 0
		byte1984 |= self.force_osc_mode & 0x3
		byte1984 |= (self.osc_select & 0x3) << 2
		byte1984 |= (self._reserved1984_4 & 0x1) << 4
		byte1984 |= (self.arpeggiator_assign & 0x3) << 5
		byte1984 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte1984)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte1988 = 0
		byte1988 |= self.top_slope & 0xF
		byte1988 |= (self.bottom_slope & 0xF) << 4
		v.append(byte1988)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte1991 = 0
		byte1991 |= self.top_velocity_slope & 0xF
		byte1991 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte1991)
		byte1992 = 0
		byte1992 |= self.trim_value & 0x7F
		byte1992 |= (self.trim_bypass & 0x1) << 7
		v.append(byte1992)
		v.append(self._reserved1993)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved1998)
		v.append(self._reserved1999)
		v.append(self._reserved2000)
		v.append(self._reserved2001)
		v.append(self._reserved2002)
		v.append(self._reserved2003)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2005)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2009)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2013)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2017)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2021)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2025)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2029)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2033)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track11Parameters Track 11 Parameters object'''
		json = []
		json.append(('name','Track11Parameters'))
		json.append(('description','Track 11 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk11 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x16,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk11 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x16,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk11 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x16,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk11 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk11 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x16,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk11 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x16,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk11 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x16,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk11 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x16,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk11 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x16,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk11 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x16,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk11 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x16,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk11 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x16,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk11 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk11 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x16,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk11 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk11 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x16,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk11 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x16,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk11 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x16,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk11 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x16,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk11 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x16,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk11 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk11 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x16,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk11 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk11 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x16,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk11 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk11 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x16,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk11 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk11 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x16,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk11 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk11 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x16,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk11 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk11 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x16,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk11 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk11 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x16,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk11 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk11 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x16,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk11 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk11 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x16,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk11 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk11 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x16,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk11 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk11 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x16,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk11 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk11 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x16,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk11 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk11 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x16,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk11 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk11 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x16,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk11 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk11 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x16,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk11 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk11 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x16,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk11 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk11 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x16,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk11 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk11 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x16,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk11 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk11 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x16,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk11 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk11 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x16,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk11 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk11 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x16,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk11 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk11 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x16,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk11 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk11 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x16,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk11 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk11 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x16,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk11 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk11 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x16,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk11 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk11 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x16,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk11 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk11 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x16,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk11 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk11 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x16,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk11 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk11 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x16,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk11 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk11 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x16,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk11 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk11 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x16,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk11 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk11 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x16,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk11 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk11 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x16,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk11 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk11 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x16,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk11 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk11 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x16,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk11 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x16,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk11 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x16,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk11 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x16,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk11 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x16,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk11 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x16,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk11 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x16,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk11 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x16,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk11 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x16,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk11 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x16,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk11 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x16,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk11 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk11 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x16,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk11 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x16,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk11 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x16,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk11 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x16,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk11 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x26,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk11 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk11 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x26,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk11 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x26,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk11 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk11 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x26,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk11 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x26,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk11 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk11 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x26,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk11 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x26,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk11 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk11 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x26,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk11 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x26,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk11 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk11 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x26,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk11 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x26,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk11 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk11 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x26,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk11 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x26,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk11 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk11 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x26,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk11 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x26,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk11 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk11 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x26,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk11 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x26,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk11 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk11 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x26,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk11 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk11 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x26,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk11 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x26,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk11 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk11 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x26,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk11 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk11 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x26,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk11 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x26,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk11 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk11 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x26,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk11 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk11 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x26,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk11 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x26,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk11 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk11 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x26,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk11 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk11 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x26,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk11 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x26,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk11 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk11 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x26,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk11 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk11 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x26,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk11 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x26,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk11 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk11 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x26,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk11 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk11 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x26,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk11 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x26,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk11 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk11 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x26,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk11 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk11 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x26,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk11 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x26,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk11 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk11 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk11 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x26,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk11 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk11 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x26,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track12Parameters(TableBase):
	description = '''Track 12 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track12Parameters Track 12 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte2070 = v.pop(0)
		self.midi_channel = byte2070 & 0x1F
		self.midi_status = (byte2070 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved2079 = v.pop(0)
		byte2080 = v.pop(0)
		self.delay_tempo_base = byte2080 & 0xF
		self._reserved2080_6 = (byte2080 >> 4) & 0x7
		self.delay_mode = (byte2080 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte2090 = v.pop(0)
		self.bus_select = byte2090 & 0x1F
		self.fx_bus_select = (byte2090 >> 5) & 0x3
		self._reserved2090_7 = (byte2090 >> 7) & 0x1
		self._reserved2091 = v.pop(0)
		self._reserved2092 = v.pop(0)
		byte2093 = v.pop(0)
		self.enable_program_change = byte2093 & 0x1
		self.enable_after_touch = (byte2093 >> 1) & 0x1
		self.enable_damper = (byte2093 >> 2) & 0x1
		self.enable_portamento = (byte2093 >> 3) & 0x1
		self.enable_joystick_x = (byte2093 >> 4) & 0x1
		self.enable_joystick_plus = (byte2093 >> 5) & 0x1
		self.enable_joystick_minus = (byte2093 >> 6) & 0x1
		self.enable_ribbon = (byte2093 >> 7) & 0x1
		byte2094 = v.pop(0)
		self.enable_rt_knob_cutoff = byte2094 & 0x1
		self.enable_rt_knob_resonance = (byte2094 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte2094 >> 2) & 0x1
		self.enable_rt_knob_release = (byte2094 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte2094 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte2094 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte2094 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte2094 >> 7) & 0x1
		byte2095 = v.pop(0)
		self.enable_sw1 = byte2095 & 0x1
		self.enable_sw2 = (byte2095 >> 1) & 0x1
		self.enable_foot_switch = (byte2095 >> 2) & 0x1
		self.enable_foot_pedal = (byte2095 >> 3) & 0x1
		self.enable_other_control_change = (byte2095 >> 4) & 0x1
		self.enable_use_prg_scales = (byte2095 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte2095 >> 6) & 0x1
		self.enable_timbre_mute = (byte2095 >> 7) & 0x1
		byte2096 = v.pop(0)
		self.force_osc_mode = byte2096 & 0x3
		self.osc_select = (byte2096 >> 2) & 0x3
		self._reserved2096_4 = (byte2096 >> 4) & 0x1
		self.arpeggiator_assign = (byte2096 >> 5) & 0x3
		self.enable_waveform_sysex = (byte2096 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte2100 = v.pop(0)
		self.top_slope = byte2100 & 0xF
		self.bottom_slope = (byte2100 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte2103 = v.pop(0)
		self.top_velocity_slope = byte2103 & 0xF
		self.bottom_velocity_slope = (byte2103 >> 4) & 0xF
		byte2104 = v.pop(0)
		self.trim_value = byte2104 & 0x7F
		self.trim_bypass = (byte2104 >> 7) & 0x1
		self._reserved2105 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved2110 = v.pop(0)
		self._reserved2111 = v.pop(0)
		self._reserved2112 = v.pop(0)
		self._reserved2113 = v.pop(0)
		self._reserved2114 = v.pop(0)
		self._reserved2115 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2117 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2121 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2125 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2129 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2133 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2137 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2141 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2145 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track12Parameters Track 12 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte2070 = 0
		byte2070 |= self.midi_channel & 0x1F
		byte2070 |= (self.midi_status & 0x7) << 5
		v.append(byte2070)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved2079)
		byte2080 = 0
		byte2080 |= self.delay_tempo_base & 0xF
		byte2080 |= (self._reserved2080_6 & 0x7) << 4
		byte2080 |= (self.delay_mode & 0x1) << 7
		v.append(byte2080)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte2090 = 0
		byte2090 |= self.bus_select & 0x1F
		byte2090 |= (self.fx_bus_select & 0x3) << 5
		byte2090 |= (self._reserved2090_7 & 0x1) << 7
		v.append(byte2090)
		v.append(self._reserved2091)
		v.append(self._reserved2092)
		byte2093 = 0
		byte2093 |= self.enable_program_change & 0x1
		byte2093 |= (self.enable_after_touch & 0x1) << 1
		byte2093 |= (self.enable_damper & 0x1) << 2
		byte2093 |= (self.enable_portamento & 0x1) << 3
		byte2093 |= (self.enable_joystick_x & 0x1) << 4
		byte2093 |= (self.enable_joystick_plus & 0x1) << 5
		byte2093 |= (self.enable_joystick_minus & 0x1) << 6
		byte2093 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte2093)
		byte2094 = 0
		byte2094 |= self.enable_rt_knob_cutoff & 0x1
		byte2094 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte2094 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte2094 |= (self.enable_rt_knob_release & 0x1) << 3
		byte2094 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte2094 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte2094 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte2094 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte2094)
		byte2095 = 0
		byte2095 |= self.enable_sw1 & 0x1
		byte2095 |= (self.enable_sw2 & 0x1) << 1
		byte2095 |= (self.enable_foot_switch & 0x1) << 2
		byte2095 |= (self.enable_foot_pedal & 0x1) << 3
		byte2095 |= (self.enable_other_control_change & 0x1) << 4
		byte2095 |= (self.enable_use_prg_scales & 0x1) << 5
		byte2095 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte2095 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte2095)
		byte2096 = 0
		byte2096 |= self.force_osc_mode & 0x3
		byte2096 |= (self.osc_select & 0x3) << 2
		byte2096 |= (self._reserved2096_4 & 0x1) << 4
		byte2096 |= (self.arpeggiator_assign & 0x3) << 5
		byte2096 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte2096)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte2100 = 0
		byte2100 |= self.top_slope & 0xF
		byte2100 |= (self.bottom_slope & 0xF) << 4
		v.append(byte2100)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte2103 = 0
		byte2103 |= self.top_velocity_slope & 0xF
		byte2103 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte2103)
		byte2104 = 0
		byte2104 |= self.trim_value & 0x7F
		byte2104 |= (self.trim_bypass & 0x1) << 7
		v.append(byte2104)
		v.append(self._reserved2105)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved2110)
		v.append(self._reserved2111)
		v.append(self._reserved2112)
		v.append(self._reserved2113)
		v.append(self._reserved2114)
		v.append(self._reserved2115)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2117)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2121)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2125)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2129)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2133)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2137)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2141)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2145)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track12Parameters Track 12 Parameters object'''
		json = []
		json.append(('name','Track12Parameters'))
		json.append(('description','Track 12 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk12 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x17,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk12 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x17,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk12 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x17,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk12 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk12 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x17,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk12 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x17,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk12 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x17,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk12 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x17,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk12 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x17,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk12 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x17,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk12 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x17,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk12 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x17,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk12 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk12 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x17,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk12 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk12 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x17,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk12 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x17,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk12 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x17,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk12 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x17,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk12 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x17,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk12 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk12 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x17,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk12 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk12 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x17,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk12 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk12 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x17,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk12 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk12 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x17,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk12 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk12 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x17,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk12 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk12 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x17,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk12 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk12 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x17,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk12 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk12 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x17,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk12 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk12 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x17,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk12 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk12 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x17,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk12 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk12 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x17,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk12 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk12 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x17,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk12 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk12 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x17,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk12 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk12 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x17,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk12 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk12 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x17,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk12 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk12 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x17,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk12 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk12 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x17,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk12 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk12 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x17,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk12 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk12 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x17,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk12 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk12 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x17,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk12 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk12 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x17,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk12 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk12 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x17,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk12 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk12 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x17,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk12 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk12 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x17,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk12 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk12 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x17,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk12 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk12 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x17,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk12 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk12 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x17,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk12 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk12 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x17,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk12 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk12 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x17,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk12 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk12 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x17,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk12 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk12 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x17,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk12 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk12 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x17,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk12 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk12 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x17,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk12 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk12 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x17,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk12 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk12 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x17,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk12 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x17,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk12 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x17,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk12 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x17,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk12 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x17,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk12 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x17,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk12 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x17,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk12 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x17,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk12 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x17,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk12 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x17,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk12 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x17,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk12 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk12 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x17,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk12 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x17,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk12 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x17,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk12 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x17,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk12 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x27,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk12 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk12 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x27,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk12 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x27,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk12 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk12 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x27,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk12 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x27,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk12 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk12 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x27,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk12 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x27,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk12 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk12 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x27,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk12 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x27,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk12 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk12 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x27,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk12 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x27,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk12 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk12 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x27,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk12 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x27,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk12 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk12 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x27,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk12 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x27,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk12 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk12 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x27,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk12 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x27,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk12 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk12 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x27,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk12 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk12 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x27,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk12 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x27,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk12 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk12 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x27,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk12 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk12 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x27,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk12 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x27,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk12 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk12 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x27,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk12 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk12 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x27,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk12 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x27,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk12 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk12 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x27,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk12 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk12 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x27,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk12 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x27,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk12 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk12 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x27,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk12 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk12 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x27,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk12 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x27,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk12 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk12 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x27,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk12 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk12 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x27,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk12 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x27,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk12 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk12 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x27,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk12 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk12 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x27,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk12 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x27,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk12 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk12 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk12 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x27,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk12 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk12 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x27,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track13Parameters(TableBase):
	description = '''Track 13 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track13Parameters Track 13 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte2182 = v.pop(0)
		self.midi_channel = byte2182 & 0x1F
		self.midi_status = (byte2182 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved2191 = v.pop(0)
		byte2192 = v.pop(0)
		self.delay_tempo_base = byte2192 & 0xF
		self._reserved2192_6 = (byte2192 >> 4) & 0x7
		self.delay_mode = (byte2192 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte2202 = v.pop(0)
		self.bus_select = byte2202 & 0x1F
		self.fx_bus_select = (byte2202 >> 5) & 0x3
		self._reserved2202_7 = (byte2202 >> 7) & 0x1
		self._reserved2203 = v.pop(0)
		self._reserved2204 = v.pop(0)
		byte2205 = v.pop(0)
		self.enable_program_change = byte2205 & 0x1
		self.enable_after_touch = (byte2205 >> 1) & 0x1
		self.enable_damper = (byte2205 >> 2) & 0x1
		self.enable_portamento = (byte2205 >> 3) & 0x1
		self.enable_joystick_x = (byte2205 >> 4) & 0x1
		self.enable_joystick_plus = (byte2205 >> 5) & 0x1
		self.enable_joystick_minus = (byte2205 >> 6) & 0x1
		self.enable_ribbon = (byte2205 >> 7) & 0x1
		byte2206 = v.pop(0)
		self.enable_rt_knob_cutoff = byte2206 & 0x1
		self.enable_rt_knob_resonance = (byte2206 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte2206 >> 2) & 0x1
		self.enable_rt_knob_release = (byte2206 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte2206 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte2206 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte2206 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte2206 >> 7) & 0x1
		byte2207 = v.pop(0)
		self.enable_sw1 = byte2207 & 0x1
		self.enable_sw2 = (byte2207 >> 1) & 0x1
		self.enable_foot_switch = (byte2207 >> 2) & 0x1
		self.enable_foot_pedal = (byte2207 >> 3) & 0x1
		self.enable_other_control_change = (byte2207 >> 4) & 0x1
		self.enable_use_prg_scales = (byte2207 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte2207 >> 6) & 0x1
		self.enable_timbre_mute = (byte2207 >> 7) & 0x1
		byte2208 = v.pop(0)
		self.force_osc_mode = byte2208 & 0x3
		self.osc_select = (byte2208 >> 2) & 0x3
		self._reserved2208_4 = (byte2208 >> 4) & 0x1
		self.arpeggiator_assign = (byte2208 >> 5) & 0x3
		self.enable_waveform_sysex = (byte2208 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte2212 = v.pop(0)
		self.top_slope = byte2212 & 0xF
		self.bottom_slope = (byte2212 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte2215 = v.pop(0)
		self.top_velocity_slope = byte2215 & 0xF
		self.bottom_velocity_slope = (byte2215 >> 4) & 0xF
		byte2216 = v.pop(0)
		self.trim_value = byte2216 & 0x7F
		self.trim_bypass = (byte2216 >> 7) & 0x1
		self._reserved2217 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved2222 = v.pop(0)
		self._reserved2223 = v.pop(0)
		self._reserved2224 = v.pop(0)
		self._reserved2225 = v.pop(0)
		self._reserved2226 = v.pop(0)
		self._reserved2227 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2229 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2233 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2237 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2241 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2245 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2249 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2253 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2257 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track13Parameters Track 13 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte2182 = 0
		byte2182 |= self.midi_channel & 0x1F
		byte2182 |= (self.midi_status & 0x7) << 5
		v.append(byte2182)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved2191)
		byte2192 = 0
		byte2192 |= self.delay_tempo_base & 0xF
		byte2192 |= (self._reserved2192_6 & 0x7) << 4
		byte2192 |= (self.delay_mode & 0x1) << 7
		v.append(byte2192)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte2202 = 0
		byte2202 |= self.bus_select & 0x1F
		byte2202 |= (self.fx_bus_select & 0x3) << 5
		byte2202 |= (self._reserved2202_7 & 0x1) << 7
		v.append(byte2202)
		v.append(self._reserved2203)
		v.append(self._reserved2204)
		byte2205 = 0
		byte2205 |= self.enable_program_change & 0x1
		byte2205 |= (self.enable_after_touch & 0x1) << 1
		byte2205 |= (self.enable_damper & 0x1) << 2
		byte2205 |= (self.enable_portamento & 0x1) << 3
		byte2205 |= (self.enable_joystick_x & 0x1) << 4
		byte2205 |= (self.enable_joystick_plus & 0x1) << 5
		byte2205 |= (self.enable_joystick_minus & 0x1) << 6
		byte2205 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte2205)
		byte2206 = 0
		byte2206 |= self.enable_rt_knob_cutoff & 0x1
		byte2206 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte2206 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte2206 |= (self.enable_rt_knob_release & 0x1) << 3
		byte2206 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte2206 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte2206 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte2206 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte2206)
		byte2207 = 0
		byte2207 |= self.enable_sw1 & 0x1
		byte2207 |= (self.enable_sw2 & 0x1) << 1
		byte2207 |= (self.enable_foot_switch & 0x1) << 2
		byte2207 |= (self.enable_foot_pedal & 0x1) << 3
		byte2207 |= (self.enable_other_control_change & 0x1) << 4
		byte2207 |= (self.enable_use_prg_scales & 0x1) << 5
		byte2207 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte2207 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte2207)
		byte2208 = 0
		byte2208 |= self.force_osc_mode & 0x3
		byte2208 |= (self.osc_select & 0x3) << 2
		byte2208 |= (self._reserved2208_4 & 0x1) << 4
		byte2208 |= (self.arpeggiator_assign & 0x3) << 5
		byte2208 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte2208)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte2212 = 0
		byte2212 |= self.top_slope & 0xF
		byte2212 |= (self.bottom_slope & 0xF) << 4
		v.append(byte2212)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte2215 = 0
		byte2215 |= self.top_velocity_slope & 0xF
		byte2215 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte2215)
		byte2216 = 0
		byte2216 |= self.trim_value & 0x7F
		byte2216 |= (self.trim_bypass & 0x1) << 7
		v.append(byte2216)
		v.append(self._reserved2217)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved2222)
		v.append(self._reserved2223)
		v.append(self._reserved2224)
		v.append(self._reserved2225)
		v.append(self._reserved2226)
		v.append(self._reserved2227)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2229)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2233)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2237)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2241)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2245)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2249)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2253)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2257)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track13Parameters Track 13 Parameters object'''
		json = []
		json.append(('name','Track13Parameters'))
		json.append(('description','Track 13 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk13 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x18,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk13 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x18,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk13 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x18,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk13 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk13 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x18,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk13 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x18,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk13 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x18,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk13 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x18,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk13 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x18,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk13 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x18,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk13 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x18,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk13 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x18,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk13 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk13 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x18,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk13 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk13 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x18,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk13 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x18,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk13 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x18,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk13 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x18,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk13 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x18,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk13 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk13 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x18,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk13 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk13 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x18,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk13 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk13 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x18,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk13 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk13 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x18,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk13 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk13 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x18,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk13 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk13 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x18,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk13 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk13 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x18,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk13 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk13 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x18,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk13 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk13 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x18,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk13 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk13 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x18,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk13 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk13 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x18,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk13 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk13 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x18,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk13 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk13 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x18,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk13 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk13 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x18,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk13 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk13 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x18,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk13 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk13 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x18,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk13 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk13 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x18,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk13 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk13 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x18,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk13 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk13 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x18,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk13 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk13 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x18,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk13 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk13 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x18,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk13 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk13 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x18,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk13 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk13 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x18,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk13 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk13 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x18,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk13 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk13 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x18,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk13 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk13 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x18,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk13 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk13 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x18,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk13 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk13 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x18,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk13 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk13 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x18,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk13 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk13 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x18,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk13 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk13 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x18,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk13 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk13 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x18,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk13 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk13 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x18,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk13 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk13 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x18,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk13 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk13 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x18,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk13 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x18,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk13 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x18,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk13 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x18,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk13 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x18,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk13 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x18,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk13 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x18,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk13 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x18,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk13 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x18,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk13 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x18,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk13 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x18,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk13 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk13 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x18,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk13 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x18,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk13 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x18,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk13 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x18,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk13 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x28,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk13 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk13 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x28,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk13 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x28,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk13 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk13 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x28,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk13 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x28,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk13 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk13 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x28,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk13 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x28,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk13 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk13 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x28,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk13 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x28,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk13 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk13 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x28,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk13 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x28,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk13 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk13 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x28,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk13 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x28,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk13 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk13 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x28,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk13 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x28,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk13 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk13 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x28,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk13 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x28,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk13 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk13 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x28,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk13 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk13 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x28,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk13 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x28,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk13 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk13 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x28,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk13 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk13 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x28,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk13 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x28,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk13 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk13 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x28,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk13 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk13 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x28,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk13 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x28,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk13 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk13 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x28,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk13 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk13 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x28,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk13 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x28,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk13 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk13 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x28,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk13 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk13 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x28,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk13 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x28,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk13 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk13 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x28,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk13 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk13 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x28,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk13 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x28,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk13 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk13 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x28,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk13 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk13 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x28,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk13 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x28,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk13 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk13 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk13 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x28,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk13 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk13 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x28,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track14Parameters(TableBase):
	description = '''Track 14 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track14Parameters Track 14 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte2294 = v.pop(0)
		self.midi_channel = byte2294 & 0x1F
		self.midi_status = (byte2294 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved2303 = v.pop(0)
		byte2304 = v.pop(0)
		self.delay_tempo_base = byte2304 & 0xF
		self._reserved2304_6 = (byte2304 >> 4) & 0x7
		self.delay_mode = (byte2304 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte2314 = v.pop(0)
		self.bus_select = byte2314 & 0x1F
		self.fx_bus_select = (byte2314 >> 5) & 0x3
		self._reserved2314_7 = (byte2314 >> 7) & 0x1
		self._reserved2315 = v.pop(0)
		self._reserved2316 = v.pop(0)
		byte2317 = v.pop(0)
		self.enable_program_change = byte2317 & 0x1
		self.enable_after_touch = (byte2317 >> 1) & 0x1
		self.enable_damper = (byte2317 >> 2) & 0x1
		self.enable_portamento = (byte2317 >> 3) & 0x1
		self.enable_joystick_x = (byte2317 >> 4) & 0x1
		self.enable_joystick_plus = (byte2317 >> 5) & 0x1
		self.enable_joystick_minus = (byte2317 >> 6) & 0x1
		self.enable_ribbon = (byte2317 >> 7) & 0x1
		byte2318 = v.pop(0)
		self.enable_rt_knob_cutoff = byte2318 & 0x1
		self.enable_rt_knob_resonance = (byte2318 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte2318 >> 2) & 0x1
		self.enable_rt_knob_release = (byte2318 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte2318 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte2318 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte2318 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte2318 >> 7) & 0x1
		byte2319 = v.pop(0)
		self.enable_sw1 = byte2319 & 0x1
		self.enable_sw2 = (byte2319 >> 1) & 0x1
		self.enable_foot_switch = (byte2319 >> 2) & 0x1
		self.enable_foot_pedal = (byte2319 >> 3) & 0x1
		self.enable_other_control_change = (byte2319 >> 4) & 0x1
		self.enable_use_prg_scales = (byte2319 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte2319 >> 6) & 0x1
		self.enable_timbre_mute = (byte2319 >> 7) & 0x1
		byte2320 = v.pop(0)
		self.force_osc_mode = byte2320 & 0x3
		self.osc_select = (byte2320 >> 2) & 0x3
		self._reserved2320_4 = (byte2320 >> 4) & 0x1
		self.arpeggiator_assign = (byte2320 >> 5) & 0x3
		self.enable_waveform_sysex = (byte2320 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte2324 = v.pop(0)
		self.top_slope = byte2324 & 0xF
		self.bottom_slope = (byte2324 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte2327 = v.pop(0)
		self.top_velocity_slope = byte2327 & 0xF
		self.bottom_velocity_slope = (byte2327 >> 4) & 0xF
		byte2328 = v.pop(0)
		self.trim_value = byte2328 & 0x7F
		self.trim_bypass = (byte2328 >> 7) & 0x1
		self._reserved2329 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved2334 = v.pop(0)
		self._reserved2335 = v.pop(0)
		self._reserved2336 = v.pop(0)
		self._reserved2337 = v.pop(0)
		self._reserved2338 = v.pop(0)
		self._reserved2339 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2341 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2345 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2349 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2353 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2357 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2361 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2365 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2369 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track14Parameters Track 14 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte2294 = 0
		byte2294 |= self.midi_channel & 0x1F
		byte2294 |= (self.midi_status & 0x7) << 5
		v.append(byte2294)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved2303)
		byte2304 = 0
		byte2304 |= self.delay_tempo_base & 0xF
		byte2304 |= (self._reserved2304_6 & 0x7) << 4
		byte2304 |= (self.delay_mode & 0x1) << 7
		v.append(byte2304)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte2314 = 0
		byte2314 |= self.bus_select & 0x1F
		byte2314 |= (self.fx_bus_select & 0x3) << 5
		byte2314 |= (self._reserved2314_7 & 0x1) << 7
		v.append(byte2314)
		v.append(self._reserved2315)
		v.append(self._reserved2316)
		byte2317 = 0
		byte2317 |= self.enable_program_change & 0x1
		byte2317 |= (self.enable_after_touch & 0x1) << 1
		byte2317 |= (self.enable_damper & 0x1) << 2
		byte2317 |= (self.enable_portamento & 0x1) << 3
		byte2317 |= (self.enable_joystick_x & 0x1) << 4
		byte2317 |= (self.enable_joystick_plus & 0x1) << 5
		byte2317 |= (self.enable_joystick_minus & 0x1) << 6
		byte2317 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte2317)
		byte2318 = 0
		byte2318 |= self.enable_rt_knob_cutoff & 0x1
		byte2318 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte2318 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte2318 |= (self.enable_rt_knob_release & 0x1) << 3
		byte2318 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte2318 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte2318 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte2318 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte2318)
		byte2319 = 0
		byte2319 |= self.enable_sw1 & 0x1
		byte2319 |= (self.enable_sw2 & 0x1) << 1
		byte2319 |= (self.enable_foot_switch & 0x1) << 2
		byte2319 |= (self.enable_foot_pedal & 0x1) << 3
		byte2319 |= (self.enable_other_control_change & 0x1) << 4
		byte2319 |= (self.enable_use_prg_scales & 0x1) << 5
		byte2319 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte2319 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte2319)
		byte2320 = 0
		byte2320 |= self.force_osc_mode & 0x3
		byte2320 |= (self.osc_select & 0x3) << 2
		byte2320 |= (self._reserved2320_4 & 0x1) << 4
		byte2320 |= (self.arpeggiator_assign & 0x3) << 5
		byte2320 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte2320)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte2324 = 0
		byte2324 |= self.top_slope & 0xF
		byte2324 |= (self.bottom_slope & 0xF) << 4
		v.append(byte2324)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte2327 = 0
		byte2327 |= self.top_velocity_slope & 0xF
		byte2327 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte2327)
		byte2328 = 0
		byte2328 |= self.trim_value & 0x7F
		byte2328 |= (self.trim_bypass & 0x1) << 7
		v.append(byte2328)
		v.append(self._reserved2329)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved2334)
		v.append(self._reserved2335)
		v.append(self._reserved2336)
		v.append(self._reserved2337)
		v.append(self._reserved2338)
		v.append(self._reserved2339)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2341)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2345)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2349)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2353)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2357)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2361)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2365)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2369)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track14Parameters Track 14 Parameters object'''
		json = []
		json.append(('name','Track14Parameters'))
		json.append(('description','Track 14 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk14 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x19,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk14 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x19,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk14 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x19,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk14 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk14 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x19,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk14 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x19,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk14 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x19,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk14 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x19,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk14 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x19,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk14 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x19,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk14 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x19,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk14 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x19,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk14 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk14 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x19,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk14 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk14 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x19,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk14 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x19,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk14 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x19,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk14 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x19,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk14 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x19,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk14 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk14 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x19,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk14 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk14 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x19,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk14 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk14 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x19,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk14 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk14 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x19,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk14 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk14 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x19,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk14 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk14 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x19,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk14 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk14 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x19,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk14 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk14 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x19,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk14 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk14 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x19,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk14 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk14 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x19,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk14 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk14 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x19,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk14 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk14 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x19,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk14 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk14 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x19,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk14 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk14 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x19,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk14 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk14 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x19,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk14 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk14 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x19,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk14 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk14 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x19,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk14 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk14 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x19,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk14 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk14 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x19,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk14 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk14 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x19,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk14 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk14 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x19,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk14 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk14 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x19,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk14 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk14 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x19,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk14 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk14 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x19,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk14 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk14 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x19,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk14 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk14 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x19,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk14 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk14 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x19,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk14 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk14 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x19,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk14 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk14 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x19,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk14 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk14 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x19,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk14 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk14 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x19,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk14 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk14 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x19,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk14 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk14 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x19,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk14 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk14 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x19,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk14 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk14 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x19,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk14 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x19,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk14 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x19,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk14 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x19,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk14 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x19,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk14 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x19,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk14 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x19,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk14 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x19,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk14 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x19,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk14 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x19,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk14 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x19,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk14 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk14 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x19,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk14 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x19,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk14 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x19,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk14 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x19,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk14 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x29,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk14 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk14 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x29,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk14 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x29,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk14 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk14 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x29,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk14 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x29,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk14 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk14 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x29,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk14 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x29,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk14 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk14 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x29,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk14 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x29,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk14 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk14 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x29,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk14 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x29,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk14 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk14 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x29,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk14 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x29,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk14 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk14 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x29,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk14 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x29,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk14 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk14 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x29,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk14 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x29,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk14 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk14 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x29,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk14 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk14 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x29,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk14 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x29,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk14 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk14 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x29,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk14 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk14 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x29,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk14 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x29,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk14 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk14 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x29,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk14 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk14 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x29,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk14 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x29,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk14 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk14 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x29,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk14 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk14 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x29,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk14 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x29,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk14 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk14 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x29,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk14 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk14 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x29,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk14 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x29,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk14 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk14 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x29,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk14 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk14 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x29,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk14 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x29,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk14 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk14 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x29,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk14 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk14 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x29,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk14 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x29,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk14 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk14 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk14 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x29,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk14 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk14 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x29,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track15Parameters(TableBase):
	description = '''Track 15 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track15Parameters Track 15 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte2406 = v.pop(0)
		self.midi_channel = byte2406 & 0x1F
		self.midi_status = (byte2406 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved2415 = v.pop(0)
		byte2416 = v.pop(0)
		self.delay_tempo_base = byte2416 & 0xF
		self._reserved2416_6 = (byte2416 >> 4) & 0x7
		self.delay_mode = (byte2416 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte2426 = v.pop(0)
		self.bus_select = byte2426 & 0x1F
		self.fx_bus_select = (byte2426 >> 5) & 0x3
		self._reserved2426_7 = (byte2426 >> 7) & 0x1
		self._reserved2427 = v.pop(0)
		self._reserved2428 = v.pop(0)
		byte2429 = v.pop(0)
		self.enable_program_change = byte2429 & 0x1
		self.enable_after_touch = (byte2429 >> 1) & 0x1
		self.enable_damper = (byte2429 >> 2) & 0x1
		self.enable_portamento = (byte2429 >> 3) & 0x1
		self.enable_joystick_x = (byte2429 >> 4) & 0x1
		self.enable_joystick_plus = (byte2429 >> 5) & 0x1
		self.enable_joystick_minus = (byte2429 >> 6) & 0x1
		self.enable_ribbon = (byte2429 >> 7) & 0x1
		byte2430 = v.pop(0)
		self.enable_rt_knob_cutoff = byte2430 & 0x1
		self.enable_rt_knob_resonance = (byte2430 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte2430 >> 2) & 0x1
		self.enable_rt_knob_release = (byte2430 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte2430 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte2430 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte2430 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte2430 >> 7) & 0x1
		byte2431 = v.pop(0)
		self.enable_sw1 = byte2431 & 0x1
		self.enable_sw2 = (byte2431 >> 1) & 0x1
		self.enable_foot_switch = (byte2431 >> 2) & 0x1
		self.enable_foot_pedal = (byte2431 >> 3) & 0x1
		self.enable_other_control_change = (byte2431 >> 4) & 0x1
		self.enable_use_prg_scales = (byte2431 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte2431 >> 6) & 0x1
		self.enable_timbre_mute = (byte2431 >> 7) & 0x1
		byte2432 = v.pop(0)
		self.force_osc_mode = byte2432 & 0x3
		self.osc_select = (byte2432 >> 2) & 0x3
		self._reserved2432_4 = (byte2432 >> 4) & 0x1
		self.arpeggiator_assign = (byte2432 >> 5) & 0x3
		self.enable_waveform_sysex = (byte2432 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte2436 = v.pop(0)
		self.top_slope = byte2436 & 0xF
		self.bottom_slope = (byte2436 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte2439 = v.pop(0)
		self.top_velocity_slope = byte2439 & 0xF
		self.bottom_velocity_slope = (byte2439 >> 4) & 0xF
		byte2440 = v.pop(0)
		self.trim_value = byte2440 & 0x7F
		self.trim_bypass = (byte2440 >> 7) & 0x1
		self._reserved2441 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved2446 = v.pop(0)
		self._reserved2447 = v.pop(0)
		self._reserved2448 = v.pop(0)
		self._reserved2449 = v.pop(0)
		self._reserved2450 = v.pop(0)
		self._reserved2451 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2453 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2457 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2461 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2465 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2469 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2473 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2477 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2481 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track15Parameters Track 15 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte2406 = 0
		byte2406 |= self.midi_channel & 0x1F
		byte2406 |= (self.midi_status & 0x7) << 5
		v.append(byte2406)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved2415)
		byte2416 = 0
		byte2416 |= self.delay_tempo_base & 0xF
		byte2416 |= (self._reserved2416_6 & 0x7) << 4
		byte2416 |= (self.delay_mode & 0x1) << 7
		v.append(byte2416)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte2426 = 0
		byte2426 |= self.bus_select & 0x1F
		byte2426 |= (self.fx_bus_select & 0x3) << 5
		byte2426 |= (self._reserved2426_7 & 0x1) << 7
		v.append(byte2426)
		v.append(self._reserved2427)
		v.append(self._reserved2428)
		byte2429 = 0
		byte2429 |= self.enable_program_change & 0x1
		byte2429 |= (self.enable_after_touch & 0x1) << 1
		byte2429 |= (self.enable_damper & 0x1) << 2
		byte2429 |= (self.enable_portamento & 0x1) << 3
		byte2429 |= (self.enable_joystick_x & 0x1) << 4
		byte2429 |= (self.enable_joystick_plus & 0x1) << 5
		byte2429 |= (self.enable_joystick_minus & 0x1) << 6
		byte2429 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte2429)
		byte2430 = 0
		byte2430 |= self.enable_rt_knob_cutoff & 0x1
		byte2430 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte2430 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte2430 |= (self.enable_rt_knob_release & 0x1) << 3
		byte2430 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte2430 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte2430 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte2430 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte2430)
		byte2431 = 0
		byte2431 |= self.enable_sw1 & 0x1
		byte2431 |= (self.enable_sw2 & 0x1) << 1
		byte2431 |= (self.enable_foot_switch & 0x1) << 2
		byte2431 |= (self.enable_foot_pedal & 0x1) << 3
		byte2431 |= (self.enable_other_control_change & 0x1) << 4
		byte2431 |= (self.enable_use_prg_scales & 0x1) << 5
		byte2431 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte2431 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte2431)
		byte2432 = 0
		byte2432 |= self.force_osc_mode & 0x3
		byte2432 |= (self.osc_select & 0x3) << 2
		byte2432 |= (self._reserved2432_4 & 0x1) << 4
		byte2432 |= (self.arpeggiator_assign & 0x3) << 5
		byte2432 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte2432)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte2436 = 0
		byte2436 |= self.top_slope & 0xF
		byte2436 |= (self.bottom_slope & 0xF) << 4
		v.append(byte2436)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte2439 = 0
		byte2439 |= self.top_velocity_slope & 0xF
		byte2439 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte2439)
		byte2440 = 0
		byte2440 |= self.trim_value & 0x7F
		byte2440 |= (self.trim_bypass & 0x1) << 7
		v.append(byte2440)
		v.append(self._reserved2441)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved2446)
		v.append(self._reserved2447)
		v.append(self._reserved2448)
		v.append(self._reserved2449)
		v.append(self._reserved2450)
		v.append(self._reserved2451)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2453)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2457)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2461)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2465)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2469)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2473)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2477)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2481)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track15Parameters Track 15 Parameters object'''
		json = []
		json.append(('name','Track15Parameters'))
		json.append(('description','Track 15 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk15 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x1A,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk15 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x1A,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk15 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x1A,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk15 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk15 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x1A,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk15 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x1A,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk15 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x1A,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk15 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x1A,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk15 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x1A,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk15 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x1A,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk15 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x1A,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk15 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x1A,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk15 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk15 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x1A,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk15 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk15 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x1A,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk15 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x1A,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk15 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x1A,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk15 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x1A,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk15 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x1A,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk15 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk15 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x1A,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk15 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk15 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x1A,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk15 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk15 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x1A,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk15 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk15 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x1A,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk15 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk15 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x1A,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk15 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk15 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x1A,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk15 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk15 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x1A,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk15 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk15 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x1A,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk15 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk15 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x1A,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk15 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk15 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x1A,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk15 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk15 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x1A,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk15 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk15 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x1A,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk15 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk15 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x1A,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk15 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk15 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x1A,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk15 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk15 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x1A,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk15 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk15 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x1A,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk15 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk15 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x1A,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk15 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk15 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x1A,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk15 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk15 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x1A,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk15 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk15 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x1A,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk15 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk15 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x1A,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk15 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk15 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x1A,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk15 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk15 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x1A,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk15 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk15 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x1A,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk15 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk15 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x1A,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk15 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk15 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x1A,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk15 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk15 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x1A,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk15 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk15 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x1A,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk15 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk15 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x1A,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk15 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk15 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x1A,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk15 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk15 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x1A,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk15 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk15 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x1A,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk15 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk15 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x1A,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk15 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk15 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x1A,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk15 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk15 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x1A,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk15 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x1A,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk15 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x1A,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk15 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x1A,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk15 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x1A,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk15 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x1A,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk15 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x1A,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk15 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x1A,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk15 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x1A,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk15 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x1A,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk15 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x1A,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk15 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk15 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x1A,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk15 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x1A,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk15 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x1A,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk15 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x1A,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk15 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x2A,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk15 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk15 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x2A,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk15 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x2A,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk15 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk15 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x2A,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk15 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x2A,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk15 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk15 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x2A,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk15 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x2A,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk15 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk15 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x2A,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk15 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x2A,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk15 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk15 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x2A,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk15 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x2A,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk15 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk15 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x2A,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk15 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x2A,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk15 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk15 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x2A,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk15 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x2A,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk15 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk15 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x2A,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk15 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x2A,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk15 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk15 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x2A,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk15 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk15 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x2A,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk15 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x2A,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk15 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk15 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x2A,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk15 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk15 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x2A,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk15 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x2A,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk15 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk15 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x2A,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk15 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk15 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x2A,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk15 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x2A,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk15 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk15 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x2A,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk15 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk15 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x2A,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk15 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x2A,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk15 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk15 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x2A,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk15 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk15 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x2A,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk15 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x2A,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk15 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk15 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x2A,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk15 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk15 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x2A,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk15 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x2A,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk15 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk15 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x2A,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk15 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk15 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x2A,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk15 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x2A,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk15 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk15 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk15 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x2A,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk15 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk15 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x2A,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#===============================================================================
class Track16Parameters(TableBase):
	description = '''Track 16 Parameters'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		TableBase.__init__(self)
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a Track16Parameters Track 16 Parameters object'''
		v = list(struct.unpack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',raw[:112]))
		self.program_bank_number = v.pop(0)
		byte2518 = v.pop(0)
		self.midi_channel = byte2518 & 0x1F
		self.midi_status = (byte2518 >> 5) & 0x7
		self.ext_bank_msb = v.pop(0)
		self.ext_bank_lsb = v.pop(0)
		self.volume = v.pop(0)
		self.bend_range = v.pop(0)
		self.transpose = v.pop(0)
		self.detune = v.pop(0)
		self.delay_time = v.pop(0)
		self._reserved2527 = v.pop(0)
		byte2528 = v.pop(0)
		self.delay_tempo_base = byte2528 & 0xF
		self._reserved2528_6 = (byte2528 >> 4) & 0x7
		self.delay_mode = (byte2528 >> 7) & 0x1
		self.delay_tempo_times = v.pop(0)
		self.pan = v.pop(0)
		self.send1 = v.pop(0)
		self.send2 = v.pop(0)
		self.dkit_patch_ifx_1 = v.pop(0)
		self.dkit_patch_ifx_2 = v.pop(0)
		self.dkit_patch_ifx_3 = v.pop(0)
		self.dkit_patch_ifx_4 = v.pop(0)
		self.dkit_patch_ifx_5 = v.pop(0)
		byte2538 = v.pop(0)
		self.bus_select = byte2538 & 0x1F
		self.fx_bus_select = (byte2538 >> 5) & 0x3
		self._reserved2538_7 = (byte2538 >> 7) & 0x1
		self._reserved2539 = v.pop(0)
		self._reserved2540 = v.pop(0)
		byte2541 = v.pop(0)
		self.enable_program_change = byte2541 & 0x1
		self.enable_after_touch = (byte2541 >> 1) & 0x1
		self.enable_damper = (byte2541 >> 2) & 0x1
		self.enable_portamento = (byte2541 >> 3) & 0x1
		self.enable_joystick_x = (byte2541 >> 4) & 0x1
		self.enable_joystick_plus = (byte2541 >> 5) & 0x1
		self.enable_joystick_minus = (byte2541 >> 6) & 0x1
		self.enable_ribbon = (byte2541 >> 7) & 0x1
		byte2542 = v.pop(0)
		self.enable_rt_knob_cutoff = byte2542 & 0x1
		self.enable_rt_knob_resonance = (byte2542 >> 1) & 0x1
		self.enable_rt_knob_eg_int = (byte2542 >> 2) & 0x1
		self.enable_rt_knob_release = (byte2542 >> 3) & 0x1
		self.enable_rt_knob_user1 = (byte2542 >> 4) & 0x1
		self.enable_rt_knob_user2 = (byte2542 >> 5) & 0x1
		self.enable_rt_knob_user3 = (byte2542 >> 6) & 0x1
		self.enable_rt_knob_user4 = (byte2542 >> 7) & 0x1
		byte2543 = v.pop(0)
		self.enable_sw1 = byte2543 & 0x1
		self.enable_sw2 = (byte2543 >> 1) & 0x1
		self.enable_foot_switch = (byte2543 >> 2) & 0x1
		self.enable_foot_pedal = (byte2543 >> 3) & 0x1
		self.enable_other_control_change = (byte2543 >> 4) & 0x1
		self.enable_use_prg_scales = (byte2543 >> 5) & 0x1
		self.enable_auto_load_prg_eq = (byte2543 >> 6) & 0x1
		self.enable_timbre_mute = (byte2543 >> 7) & 0x1
		byte2544 = v.pop(0)
		self.force_osc_mode = byte2544 & 0x3
		self.osc_select = (byte2544 >> 2) & 0x3
		self._reserved2544_4 = (byte2544 >> 4) & 0x1
		self.arpeggiator_assign = (byte2544 >> 5) & 0x3
		self.enable_waveform_sysex = (byte2544 >> 7) & 0x1
		self.portamento_time = v.pop(0)
		self.top_key = v.pop(0)
		self.bottom_key = v.pop(0)
		byte2548 = v.pop(0)
		self.top_slope = byte2548 & 0xF
		self.bottom_slope = (byte2548 >> 4) & 0xF
		self.top_velocity = v.pop(0)
		self.bottom_velocity = v.pop(0)
		byte2551 = v.pop(0)
		self.top_velocity_slope = byte2551 & 0xF
		self.bottom_velocity_slope = (byte2551 >> 4) & 0xF
		byte2552 = v.pop(0)
		self.trim_value = byte2552 & 0x7F
		self.trim_bypass = (byte2552 >> 7) & 0x1
		self._reserved2553 = v.pop(0)
		self.mid_freq = v.pop(0)
		self.low_gain = v.pop(0)
		self.mid_gain = v.pop(0)
		self.high_gain = v.pop(0)
		self._reserved2558 = v.pop(0)
		self._reserved2559 = v.pop(0)
		self._reserved2560 = v.pop(0)
		self._reserved2561 = v.pop(0)
		self._reserved2562 = v.pop(0)
		self._reserved2563 = v.pop(0)
		self.tone_adj_slider1_assign = v.pop(0)
		self._reserved2565 = v.pop(0)
		self.tone_adj_slider1_value = v.pop(0)
		self.tone_adj_slider2_assign = v.pop(0)
		self._reserved2569 = v.pop(0)
		self.tone_adj_slider2_value = v.pop(0)
		self.tone_adj_slider3_assign = v.pop(0)
		self._reserved2573 = v.pop(0)
		self.tone_adj_slider3_value = v.pop(0)
		self.tone_adj_slider4_assign = v.pop(0)
		self._reserved2577 = v.pop(0)
		self.tone_adj_slider4_value = v.pop(0)
		self.tone_adj_slider5_assign = v.pop(0)
		self._reserved2581 = v.pop(0)
		self.tone_adj_slider5_value = v.pop(0)
		self.tone_adj_slider6_assign = v.pop(0)
		self._reserved2585 = v.pop(0)
		self.tone_adj_slider6_value = v.pop(0)
		self.tone_adj_slider7_assign = v.pop(0)
		self._reserved2589 = v.pop(0)
		self.tone_adj_slider7_value = v.pop(0)
		self.tone_adj_slider8_assign = v.pop(0)
		self._reserved2593 = v.pop(0)
		self.tone_adj_slider8_value = v.pop(0)
		self.tone_adj_switch1_assign = v.pop(0)
		self.tone_adj_switch1_off_on = v.pop(0)
		self.tone_adj_switch1_value = v.pop(0)
		self.tone_adj_switch2_assign = v.pop(0)
		self.tone_adj_switch2_off_on = v.pop(0)
		self.tone_adj_switch2_value = v.pop(0)
		self.tone_adj_switch3_assign = v.pop(0)
		self.tone_adj_switch3_off_on = v.pop(0)
		self.tone_adj_switch3_value = v.pop(0)
		self.tone_adj_switch4_assign = v.pop(0)
		self.tone_adj_switch4_off_on = v.pop(0)
		self.tone_adj_switch4_value = v.pop(0)
		self.tone_adj_switch5_assign = v.pop(0)
		self.tone_adj_switch5_off_on = v.pop(0)
		self.tone_adj_switch5_value = v.pop(0)
		self.tone_adj_switch6_assign = v.pop(0)
		self.tone_adj_switch6_off_on = v.pop(0)
		self.tone_adj_switch6_value = v.pop(0)
		self.tone_adj_switch7_assign = v.pop(0)
		self.tone_adj_switch7_off_on = v.pop(0)
		self.tone_adj_switch7_value = v.pop(0)
		self.tone_adj_switch8_assign = v.pop(0)
		self.tone_adj_switch8_off_on = v.pop(0)
		self.tone_adj_switch8_value = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a Track16Parameters Track 16 Parameters object into raw data'''
		v = []
		v.append(self.program_bank_number)
		byte2518 = 0
		byte2518 |= self.midi_channel & 0x1F
		byte2518 |= (self.midi_status & 0x7) << 5
		v.append(byte2518)
		v.append(self.ext_bank_msb)
		v.append(self.ext_bank_lsb)
		v.append(self.volume)
		v.append(self.bend_range)
		v.append(self.transpose)
		v.append(self.detune)
		v.append(self.delay_time)
		v.append(self._reserved2527)
		byte2528 = 0
		byte2528 |= self.delay_tempo_base & 0xF
		byte2528 |= (self._reserved2528_6 & 0x7) << 4
		byte2528 |= (self.delay_mode & 0x1) << 7
		v.append(byte2528)
		v.append(self.delay_tempo_times)
		v.append(self.pan)
		v.append(self.send1)
		v.append(self.send2)
		v.append(self.dkit_patch_ifx_1)
		v.append(self.dkit_patch_ifx_2)
		v.append(self.dkit_patch_ifx_3)
		v.append(self.dkit_patch_ifx_4)
		v.append(self.dkit_patch_ifx_5)
		byte2538 = 0
		byte2538 |= self.bus_select & 0x1F
		byte2538 |= (self.fx_bus_select & 0x3) << 5
		byte2538 |= (self._reserved2538_7 & 0x1) << 7
		v.append(byte2538)
		v.append(self._reserved2539)
		v.append(self._reserved2540)
		byte2541 = 0
		byte2541 |= self.enable_program_change & 0x1
		byte2541 |= (self.enable_after_touch & 0x1) << 1
		byte2541 |= (self.enable_damper & 0x1) << 2
		byte2541 |= (self.enable_portamento & 0x1) << 3
		byte2541 |= (self.enable_joystick_x & 0x1) << 4
		byte2541 |= (self.enable_joystick_plus & 0x1) << 5
		byte2541 |= (self.enable_joystick_minus & 0x1) << 6
		byte2541 |= (self.enable_ribbon & 0x1) << 7
		v.append(byte2541)
		byte2542 = 0
		byte2542 |= self.enable_rt_knob_cutoff & 0x1
		byte2542 |= (self.enable_rt_knob_resonance & 0x1) << 1
		byte2542 |= (self.enable_rt_knob_eg_int & 0x1) << 2
		byte2542 |= (self.enable_rt_knob_release & 0x1) << 3
		byte2542 |= (self.enable_rt_knob_user1 & 0x1) << 4
		byte2542 |= (self.enable_rt_knob_user2 & 0x1) << 5
		byte2542 |= (self.enable_rt_knob_user3 & 0x1) << 6
		byte2542 |= (self.enable_rt_knob_user4 & 0x1) << 7
		v.append(byte2542)
		byte2543 = 0
		byte2543 |= self.enable_sw1 & 0x1
		byte2543 |= (self.enable_sw2 & 0x1) << 1
		byte2543 |= (self.enable_foot_switch & 0x1) << 2
		byte2543 |= (self.enable_foot_pedal & 0x1) << 3
		byte2543 |= (self.enable_other_control_change & 0x1) << 4
		byte2543 |= (self.enable_use_prg_scales & 0x1) << 5
		byte2543 |= (self.enable_auto_load_prg_eq & 0x1) << 6
		byte2543 |= (self.enable_timbre_mute & 0x1) << 7
		v.append(byte2543)
		byte2544 = 0
		byte2544 |= self.force_osc_mode & 0x3
		byte2544 |= (self.osc_select & 0x3) << 2
		byte2544 |= (self._reserved2544_4 & 0x1) << 4
		byte2544 |= (self.arpeggiator_assign & 0x3) << 5
		byte2544 |= (self.enable_waveform_sysex & 0x1) << 7
		v.append(byte2544)
		v.append(self.portamento_time)
		v.append(self.top_key)
		v.append(self.bottom_key)
		byte2548 = 0
		byte2548 |= self.top_slope & 0xF
		byte2548 |= (self.bottom_slope & 0xF) << 4
		v.append(byte2548)
		v.append(self.top_velocity)
		v.append(self.bottom_velocity)
		byte2551 = 0
		byte2551 |= self.top_velocity_slope & 0xF
		byte2551 |= (self.bottom_velocity_slope & 0xF) << 4
		v.append(byte2551)
		byte2552 = 0
		byte2552 |= self.trim_value & 0x7F
		byte2552 |= (self.trim_bypass & 0x1) << 7
		v.append(byte2552)
		v.append(self._reserved2553)
		v.append(self.mid_freq)
		v.append(self.low_gain)
		v.append(self.mid_gain)
		v.append(self.high_gain)
		v.append(self._reserved2558)
		v.append(self._reserved2559)
		v.append(self._reserved2560)
		v.append(self._reserved2561)
		v.append(self._reserved2562)
		v.append(self._reserved2563)
		v.append(self.tone_adj_slider1_assign)
		v.append(self._reserved2565)
		v.append(self.tone_adj_slider1_value)
		v.append(self.tone_adj_slider2_assign)
		v.append(self._reserved2569)
		v.append(self.tone_adj_slider2_value)
		v.append(self.tone_adj_slider3_assign)
		v.append(self._reserved2573)
		v.append(self.tone_adj_slider3_value)
		v.append(self.tone_adj_slider4_assign)
		v.append(self._reserved2577)
		v.append(self.tone_adj_slider4_value)
		v.append(self.tone_adj_slider5_assign)
		v.append(self._reserved2581)
		v.append(self.tone_adj_slider5_value)
		v.append(self.tone_adj_slider6_assign)
		v.append(self._reserved2585)
		v.append(self.tone_adj_slider6_value)
		v.append(self.tone_adj_slider7_assign)
		v.append(self._reserved2589)
		v.append(self.tone_adj_slider7_value)
		v.append(self.tone_adj_slider8_assign)
		v.append(self._reserved2593)
		v.append(self.tone_adj_slider8_value)
		v.append(self.tone_adj_switch1_assign)
		v.append(self.tone_adj_switch1_off_on)
		v.append(self.tone_adj_switch1_value)
		v.append(self.tone_adj_switch2_assign)
		v.append(self.tone_adj_switch2_off_on)
		v.append(self.tone_adj_switch2_value)
		v.append(self.tone_adj_switch3_assign)
		v.append(self.tone_adj_switch3_off_on)
		v.append(self.tone_adj_switch3_value)
		v.append(self.tone_adj_switch4_assign)
		v.append(self.tone_adj_switch4_off_on)
		v.append(self.tone_adj_switch4_value)
		v.append(self.tone_adj_switch5_assign)
		v.append(self.tone_adj_switch5_off_on)
		v.append(self.tone_adj_switch5_value)
		v.append(self.tone_adj_switch6_assign)
		v.append(self.tone_adj_switch6_off_on)
		v.append(self.tone_adj_switch6_value)
		v.append(self.tone_adj_switch7_assign)
		v.append(self.tone_adj_switch7_off_on)
		v.append(self.tone_adj_switch7_value)
		v.append(self.tone_adj_switch8_assign)
		v.append(self.tone_adj_switch8_off_on)
		v.append(self.tone_adj_switch8_value)
		return struct.pack('<H4B2bh19Bb9B3b8Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh2Bh',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a Track16Parameters Track 16 Parameters object'''
		json = []
		json.append(('name','Track16Parameters'))
		json.append(('description','Track 16 Parameters'))
		json.append(['program_bank_number',self.program_bank_number])
		json.append(['midi_channel',self.midi_channel])
		json.append(['midi_status',self.midi_status])
		json.append(['midi_status_display',self.midi_status_display])
		json.append(['ext_bank_msb',self.ext_bank_msb])
		json.append(['ext_bank_lsb',self.ext_bank_lsb])
		json.append(['volume',self.volume])
		json.append(['bend_range',self.bend_range])
		json.append(['transpose',self.transpose])
		json.append(['detune',self.detune])
		json.append(['delay_time',self.delay_time])
		json.append(['delay_tempo_base',self.delay_tempo_base])
		json.append(['delay_tempo_base_display',self.delay_tempo_base_display])
		json.append(['delay_mode',self.delay_mode])
		json.append(['delay_mode_display',self.delay_mode_display])
		json.append(['delay_tempo_times',self.delay_tempo_times])
		json.append(['pan',self.pan])
		json.append(['send1',self.send1])
		json.append(['send2',self.send2])
		json.append(['dkit_patch_ifx_1',self.dkit_patch_ifx_1])
		json.append(['dkit_patch_ifx_1_display',self.dkit_patch_ifx_1_display])
		json.append(['dkit_patch_ifx_2',self.dkit_patch_ifx_2])
		json.append(['dkit_patch_ifx_2_display',self.dkit_patch_ifx_2_display])
		json.append(['dkit_patch_ifx_3',self.dkit_patch_ifx_3])
		json.append(['dkit_patch_ifx_3_display',self.dkit_patch_ifx_3_display])
		json.append(['dkit_patch_ifx_4',self.dkit_patch_ifx_4])
		json.append(['dkit_patch_ifx_4_display',self.dkit_patch_ifx_4_display])
		json.append(['dkit_patch_ifx_5',self.dkit_patch_ifx_5])
		json.append(['dkit_patch_ifx_5_display',self.dkit_patch_ifx_5_display])
		json.append(['bus_select',self.bus_select])
		json.append(['bus_select_display',self.bus_select_display])
		json.append(['fx_bus_select',self.fx_bus_select])
		json.append(['fx_bus_select_display',self.fx_bus_select_display])
		json.append(['enable_program_change',self.enable_program_change])
		json.append(['enable_program_change_display',self.enable_program_change_display])
		json.append(['enable_after_touch',self.enable_after_touch])
		json.append(['enable_after_touch_display',self.enable_after_touch_display])
		json.append(['enable_damper',self.enable_damper])
		json.append(['enable_damper_display',self.enable_damper_display])
		json.append(['enable_portamento',self.enable_portamento])
		json.append(['enable_portamento_display',self.enable_portamento_display])
		json.append(['enable_joystick_x',self.enable_joystick_x])
		json.append(['enable_joystick_x_display',self.enable_joystick_x_display])
		json.append(['enable_joystick_plus',self.enable_joystick_plus])
		json.append(['enable_joystick_plus_display',self.enable_joystick_plus_display])
		json.append(['enable_joystick_minus',self.enable_joystick_minus])
		json.append(['enable_joystick_minus_display',self.enable_joystick_minus_display])
		json.append(['enable_ribbon',self.enable_ribbon])
		json.append(['enable_ribbon_display',self.enable_ribbon_display])
		json.append(['enable_rt_knob_cutoff',self.enable_rt_knob_cutoff])
		json.append(['enable_rt_knob_cutoff_display',self.enable_rt_knob_cutoff_display])
		json.append(['enable_rt_knob_resonance',self.enable_rt_knob_resonance])
		json.append(['enable_rt_knob_resonance_display',self.enable_rt_knob_resonance_display])
		json.append(['enable_rt_knob_eg_int',self.enable_rt_knob_eg_int])
		json.append(['enable_rt_knob_eg_int_display',self.enable_rt_knob_eg_int_display])
		json.append(['enable_rt_knob_release',self.enable_rt_knob_release])
		json.append(['enable_rt_knob_release_display',self.enable_rt_knob_release_display])
		json.append(['enable_rt_knob_user1',self.enable_rt_knob_user1])
		json.append(['enable_rt_knob_user1_display',self.enable_rt_knob_user1_display])
		json.append(['enable_rt_knob_user2',self.enable_rt_knob_user2])
		json.append(['enable_rt_knob_user2_display',self.enable_rt_knob_user2_display])
		json.append(['enable_rt_knob_user3',self.enable_rt_knob_user3])
		json.append(['enable_rt_knob_user3_display',self.enable_rt_knob_user3_display])
		json.append(['enable_rt_knob_user4',self.enable_rt_knob_user4])
		json.append(['enable_rt_knob_user4_display',self.enable_rt_knob_user4_display])
		json.append(['enable_sw1',self.enable_sw1])
		json.append(['enable_sw1_display',self.enable_sw1_display])
		json.append(['enable_sw2',self.enable_sw2])
		json.append(['enable_sw2_display',self.enable_sw2_display])
		json.append(['enable_foot_switch',self.enable_foot_switch])
		json.append(['enable_foot_switch_display',self.enable_foot_switch_display])
		json.append(['enable_foot_pedal',self.enable_foot_pedal])
		json.append(['enable_foot_pedal_display',self.enable_foot_pedal_display])
		json.append(['enable_other_control_change',self.enable_other_control_change])
		json.append(['enable_other_control_change_display',self.enable_other_control_change_display])
		json.append(['enable_use_prg_scales',self.enable_use_prg_scales])
		json.append(['enable_use_prg_scales_display',self.enable_use_prg_scales_display])
		json.append(['enable_auto_load_prg_eq',self.enable_auto_load_prg_eq])
		json.append(['enable_auto_load_prg_eq_display',self.enable_auto_load_prg_eq_display])
		json.append(['enable_timbre_mute',self.enable_timbre_mute])
		json.append(['enable_timbre_mute_display',self.enable_timbre_mute_display])
		json.append(['force_osc_mode',self.force_osc_mode])
		json.append(['force_osc_mode_display',self.force_osc_mode_display])
		json.append(['osc_select',self.osc_select])
		json.append(['osc_select_display',self.osc_select_display])
		json.append(['arpeggiator_assign',self.arpeggiator_assign])
		json.append(['arpeggiator_assign_display',self.arpeggiator_assign_display])
		json.append(['enable_waveform_sysex',self.enable_waveform_sysex])
		json.append(['enable_waveform_sysex_display',self.enable_waveform_sysex_display])
		json.append(['portamento_time',self.portamento_time])
		json.append(['top_key',self.top_key])
		json.append(['bottom_key',self.bottom_key])
		json.append(['top_slope',self.top_slope])
		json.append(['bottom_slope',self.bottom_slope])
		json.append(['top_velocity',self.top_velocity])
		json.append(['bottom_velocity',self.bottom_velocity])
		json.append(['top_velocity_slope',self.top_velocity_slope])
		json.append(['bottom_velocity_slope',self.bottom_velocity_slope])
		json.append(['trim_value',self.trim_value])
		json.append(['trim_bypass',self.trim_bypass])
		json.append(['trim_bypass_display',self.trim_bypass_display])
		json.append(['mid_freq',self.mid_freq])
		json.append(['low_gain',self.low_gain])
		json.append(['mid_gain',self.mid_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['tone_adj_slider1_assign',self.tone_adj_slider1_assign])
		json.append(['tone_adj_slider1_value',self.tone_adj_slider1_value])
		json.append(['tone_adj_slider2_assign',self.tone_adj_slider2_assign])
		json.append(['tone_adj_slider2_value',self.tone_adj_slider2_value])
		json.append(['tone_adj_slider3_assign',self.tone_adj_slider3_assign])
		json.append(['tone_adj_slider3_value',self.tone_adj_slider3_value])
		json.append(['tone_adj_slider4_assign',self.tone_adj_slider4_assign])
		json.append(['tone_adj_slider4_value',self.tone_adj_slider4_value])
		json.append(['tone_adj_slider5_assign',self.tone_adj_slider5_assign])
		json.append(['tone_adj_slider5_value',self.tone_adj_slider5_value])
		json.append(['tone_adj_slider6_assign',self.tone_adj_slider6_assign])
		json.append(['tone_adj_slider6_value',self.tone_adj_slider6_value])
		json.append(['tone_adj_slider7_assign',self.tone_adj_slider7_assign])
		json.append(['tone_adj_slider7_value',self.tone_adj_slider7_value])
		json.append(['tone_adj_slider8_assign',self.tone_adj_slider8_assign])
		json.append(['tone_adj_slider8_value',self.tone_adj_slider8_value])
		json.append(['tone_adj_switch1_assign',self.tone_adj_switch1_assign])
		json.append(['tone_adj_switch1_off_on',self.tone_adj_switch1_off_on])
		json.append(['tone_adj_switch1_off_on_display',self.tone_adj_switch1_off_on_display])
		json.append(['tone_adj_switch1_value',self.tone_adj_switch1_value])
		json.append(['tone_adj_switch2_assign',self.tone_adj_switch2_assign])
		json.append(['tone_adj_switch2_off_on',self.tone_adj_switch2_off_on])
		json.append(['tone_adj_switch2_off_on_display',self.tone_adj_switch2_off_on_display])
		json.append(['tone_adj_switch2_value',self.tone_adj_switch2_value])
		json.append(['tone_adj_switch3_assign',self.tone_adj_switch3_assign])
		json.append(['tone_adj_switch3_off_on',self.tone_adj_switch3_off_on])
		json.append(['tone_adj_switch3_off_on_display',self.tone_adj_switch3_off_on_display])
		json.append(['tone_adj_switch3_value',self.tone_adj_switch3_value])
		json.append(['tone_adj_switch4_assign',self.tone_adj_switch4_assign])
		json.append(['tone_adj_switch4_off_on',self.tone_adj_switch4_off_on])
		json.append(['tone_adj_switch4_off_on_display',self.tone_adj_switch4_off_on_display])
		json.append(['tone_adj_switch4_value',self.tone_adj_switch4_value])
		json.append(['tone_adj_switch5_assign',self.tone_adj_switch5_assign])
		json.append(['tone_adj_switch5_off_on',self.tone_adj_switch5_off_on])
		json.append(['tone_adj_switch5_off_on_display',self.tone_adj_switch5_off_on_display])
		json.append(['tone_adj_switch5_value',self.tone_adj_switch5_value])
		json.append(['tone_adj_switch6_assign',self.tone_adj_switch6_assign])
		json.append(['tone_adj_switch6_off_on',self.tone_adj_switch6_off_on])
		json.append(['tone_adj_switch6_off_on_display',self.tone_adj_switch6_off_on_display])
		json.append(['tone_adj_switch6_value',self.tone_adj_switch6_value])
		json.append(['tone_adj_switch7_assign',self.tone_adj_switch7_assign])
		json.append(['tone_adj_switch7_off_on',self.tone_adj_switch7_off_on])
		json.append(['tone_adj_switch7_off_on_display',self.tone_adj_switch7_off_on_display])
		json.append(['tone_adj_switch7_value',self.tone_adj_switch7_value])
		json.append(['tone_adj_switch8_assign',self.tone_adj_switch8_assign])
		json.append(['tone_adj_switch8_off_on',self.tone_adj_switch8_off_on])
		json.append(['tone_adj_switch8_off_on_display',self.tone_adj_switch8_off_on_display])
		json.append(['tone_adj_switch8_value',self.tone_adj_switch8_value])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def program_bank_number(self):
		'''Tk16 Bank No & Program No'''
		return self._program_bank_number

	@program_bank_number.setter
	def program_bank_number(self,value):
		if 0 <= value <= 2175 or True: #FIXME
			self._program_bank_number = value
			self.notify_listeners(0x1B,0x0,value)
		else:
			raise ValueError('%s not 0 <= value <= 2175' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_channel(self):
		'''Tk16 MIDI Channel'''
		return self._midi_channel

	@midi_channel.setter
	def midi_channel(self,value):
		if 0 <= value <= 16:
			self._midi_channel = value
			self.notify_listeners(0x1B,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def midi_status(self):
		'''Tk16 Status'''
		return self._midi_status

	@midi_status.setter
	def midi_status(self,value):
		if 0 <= value <= 3:
			self._midi_status = value
			self.notify_listeners(0x1B,0x3,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def midi_status_display(self):
		'''Tk16 Status display value'''
		return midi_status_dict[self._midi_status]

#-------------------------------------------------------------------------------
	@property
	def ext_bank_msb(self):
		'''Tk16 Ext Bank MSB'''
		return self._ext_bank_msb

	@ext_bank_msb.setter
	def ext_bank_msb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_msb = value
			self.notify_listeners(0x1B,0x5,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def ext_bank_lsb(self):
		'''Tk16 Ext Bank LSB'''
		return self._ext_bank_lsb

	@ext_bank_lsb.setter
	def ext_bank_lsb(self,value):
		if 0 <= value <= 127:
			self._ext_bank_lsb = value
			self.notify_listeners(0x1B,0x6,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Tk16 Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 127:
			self._volume = value
			self.notify_listeners(0x1B,0x2,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bend_range(self):
		'''Tk16 Bend Range'''
		return self._bend_range

	@bend_range.setter
	def bend_range(self,value):
		if -25 <= value <= 24:
			self._bend_range = value
			self.notify_listeners(0x1B,0xD,value)
		else:
			raise ValueError('%s not -25 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def transpose(self):
		'''Tk16 Transpose'''
		return self._transpose

	@transpose.setter
	def transpose(self,value):
		if -60 <= value <= 60:
			self._transpose = value
			self.notify_listeners(0x1B,0xB,value)
		else:
			raise ValueError('%s not -60 <= value <= 60' % value)

#-------------------------------------------------------------------------------
	@property
	def detune(self):
		'''Tk16 Detune'''
		return self._detune

	@detune.setter
	def detune(self,value):
		if -1200 <= value <= 1200:
			self._detune = value
			self.notify_listeners(0x1B,0xC,value)
		else:
			raise ValueError('%s not -1200 <= value <= 1200' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Tk16 Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 97:
			self._delay_time = value
			self.notify_listeners(0x1B,0xE,value)
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_base(self):
		'''Tk16 Delay Tempo Base'''
		return self._delay_tempo_base

	@delay_tempo_base.setter
	def delay_tempo_base(self,value):
		if 0 <= value <= 9:
			self._delay_tempo_base = value
			self.notify_listeners(0x1B,0xF,value)
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_tempo_base_display(self):
		'''Tk16 Delay Tempo Base display value'''
		return base_note_dict[self._delay_tempo_base]

#-------------------------------------------------------------------------------
	@property
	def delay_mode(self):
		'''Tk16 Delay Mode'''
		return self._delay_mode

	@delay_mode.setter
	def delay_mode(self,value):
		if 0 <= value <= 1:
			self._delay_mode = value
			self.notify_listeners(0x1B,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def delay_mode_display(self):
		'''Tk16 Delay Mode display value'''
		return delay_mode_dict[self._delay_mode]

#-------------------------------------------------------------------------------
	@property
	def delay_tempo_times(self):
		'''Tk16 Delay Tempo Times'''
		return self._delay_tempo_times

	@delay_tempo_times.setter
	def delay_tempo_times(self,value):
		if 0 <= value <= 32:
			self._delay_tempo_times = value
			self.notify_listeners(0x1B,0x11,value)
		else:
			raise ValueError('%s not 0 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Tk16 Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if 0 <= value <= 127:
			self._pan = value
			self.notify_listeners(0x1B,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send1(self):
		'''Tk16 Send1'''
		return self._send1

	@send1.setter
	def send1(self,value):
		if 0 <= value <= 127:
			self._send1 = value
			self.notify_listeners(0x1B,0x3B,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def send2(self):
		'''Tk16 Send2'''
		return self._send2

	@send2.setter
	def send2(self,value):
		if 0 <= value <= 127:
			self._send2 = value
			self.notify_listeners(0x1B,0x3C,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_1(self):
		'''Tk16 DKit Patch IFX 1'''
		return self._dkit_patch_ifx_1

	@dkit_patch_ifx_1.setter
	def dkit_patch_ifx_1(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_1 = value
			self.notify_listeners(0x1B,0x3D,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_1_display(self):
		'''Tk16 DKit Patch IFX 1 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_1]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_2(self):
		'''Tk16 DKit Patch IFX 2'''
		return self._dkit_patch_ifx_2

	@dkit_patch_ifx_2.setter
	def dkit_patch_ifx_2(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_2 = value
			self.notify_listeners(0x1B,0x3E,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_2_display(self):
		'''Tk16 DKit Patch IFX 2 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_2]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_3(self):
		'''Tk16 DKit Patch IFX 3'''
		return self._dkit_patch_ifx_3

	@dkit_patch_ifx_3.setter
	def dkit_patch_ifx_3(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_3 = value
			self.notify_listeners(0x1B,0x3F,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_3_display(self):
		'''Tk16 DKit Patch IFX 3 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_3]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_4(self):
		'''Tk16 DKit Patch IFX 4'''
		return self._dkit_patch_ifx_4

	@dkit_patch_ifx_4.setter
	def dkit_patch_ifx_4(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_4 = value
			self.notify_listeners(0x1B,0x40,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_4_display(self):
		'''Tk16 DKit Patch IFX 4 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_4]

#-------------------------------------------------------------------------------
	@property
	def dkit_patch_ifx_5(self):
		'''Tk16 DKit Patch IFX 5'''
		return self._dkit_patch_ifx_5

	@dkit_patch_ifx_5.setter
	def dkit_patch_ifx_5(self,value):
		if value in (0, 1, 2, 3, 4, 5, 12):
			self._dkit_patch_ifx_5 = value
			self.notify_listeners(0x1B,0x41,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 12)' % value)

	@property
	def dkit_patch_ifx_5_display(self):
		'''Tk16 DKit Patch IFX 5 display value'''
		return dkit_patch_ifx_dict[self._dkit_patch_ifx_5]

#-------------------------------------------------------------------------------
	@property
	def bus_select(self):
		'''Tk16 Bus Select'''
		return self._bus_select

	@bus_select.setter
	def bus_select(self,value):
		if value in (0, 1, 2, 3, 4, 5, 6, 13):
			self._bus_select = value
			self.notify_listeners(0x1B,0x39,value)
		else:
			raise ValueError('%s value not in (0, 1, 2, 3, 4, 5, 6, 13)' % value)

	@property
	def bus_select_display(self):
		'''Tk16 Bus Select display value'''
		return tbus_select_dict[self._bus_select]

#-------------------------------------------------------------------------------
	@property
	def fx_bus_select(self):
		'''Tk16 FX Control Bus'''
		return self._fx_bus_select

	@fx_bus_select.setter
	def fx_bus_select(self,value):
		if 0 <= value <= 2:
			self._fx_bus_select = value
			self.notify_listeners(0x1B,0x3A,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def fx_bus_select_display(self):
		'''Tk16 FX Control Bus display value'''
		return fx_control_dict[self._fx_bus_select]

#-------------------------------------------------------------------------------
	@property
	def enable_program_change(self):
		'''Tk16 Enable Program Change'''
		return self._enable_program_change

	@enable_program_change.setter
	def enable_program_change(self,value):
		if 0 <= value <= 1:
			self._enable_program_change = value
			self.notify_listeners(0x1B,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_program_change_display(self):
		'''Tk16 Enable Program Change display value'''
		return off_on_dict[self._enable_program_change]

#-------------------------------------------------------------------------------
	@property
	def enable_after_touch(self):
		'''Tk16 Enable After Touch'''
		return self._enable_after_touch

	@enable_after_touch.setter
	def enable_after_touch(self,value):
		if 0 <= value <= 1:
			self._enable_after_touch = value
			self.notify_listeners(0x1B,0x14,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_after_touch_display(self):
		'''Tk16 Enable After Touch display value'''
		return off_on_dict[self._enable_after_touch]

#-------------------------------------------------------------------------------
	@property
	def enable_damper(self):
		'''Tk16 Enable Damper'''
		return self._enable_damper

	@enable_damper.setter
	def enable_damper(self,value):
		if 0 <= value <= 1:
			self._enable_damper = value
			self.notify_listeners(0x1B,0x15,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_damper_display(self):
		'''Tk16 Enable Damper display value'''
		return off_on_dict[self._enable_damper]

#-------------------------------------------------------------------------------
	@property
	def enable_portamento(self):
		'''Tk16 Enable Portamento SW'''
		return self._enable_portamento

	@enable_portamento.setter
	def enable_portamento(self,value):
		if 0 <= value <= 1:
			self._enable_portamento = value
			self.notify_listeners(0x1B,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_portamento_display(self):
		'''Tk16 Enable Portamento SW display value'''
		return off_on_dict[self._enable_portamento]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_x(self):
		'''Tk16 Enable JS X as AMS'''
		return self._enable_joystick_x

	@enable_joystick_x.setter
	def enable_joystick_x(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_x = value
			self.notify_listeners(0x1B,0x17,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_x_display(self):
		'''Tk16 Enable JS X as AMS display value'''
		return off_on_dict[self._enable_joystick_x]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_plus(self):
		'''Tk16 Enable JS+y'''
		return self._enable_joystick_plus

	@enable_joystick_plus.setter
	def enable_joystick_plus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_plus = value
			self.notify_listeners(0x1B,0x18,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_plus_display(self):
		'''Tk16 Enable JS+y display value'''
		return off_on_dict[self._enable_joystick_plus]

#-------------------------------------------------------------------------------
	@property
	def enable_joystick_minus(self):
		'''Tk16 Enable JS-y'''
		return self._enable_joystick_minus

	@enable_joystick_minus.setter
	def enable_joystick_minus(self,value):
		if 0 <= value <= 1:
			self._enable_joystick_minus = value
			self.notify_listeners(0x1B,0x19,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_joystick_minus_display(self):
		'''Tk16 Enable JS-y display value'''
		return off_on_dict[self._enable_joystick_minus]

#-------------------------------------------------------------------------------
	@property
	def enable_ribbon(self):
		'''Tk16 Enable Ribbon CC#16'''
		return self._enable_ribbon

	@enable_ribbon.setter
	def enable_ribbon(self,value):
		if 0 <= value <= 1:
			self._enable_ribbon = value
			self.notify_listeners(0x1B,0x1A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_ribbon_display(self):
		'''Tk16 Enable Ribbon CC#16 display value'''
		return off_on_dict[self._enable_ribbon]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_cutoff(self):
		'''Tk16 Enable Realtime Control-CUTOFF'''
		return self._enable_rt_knob_cutoff

	@enable_rt_knob_cutoff.setter
	def enable_rt_knob_cutoff(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_cutoff = value
			self.notify_listeners(0x1B,0x1B,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_cutoff_display(self):
		'''Tk16 Enable Realtime Control-CUTOFF display value'''
		return off_on_dict[self._enable_rt_knob_cutoff]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_resonance(self):
		'''Tk16 Enable Realtime Control-RESONANCE'''
		return self._enable_rt_knob_resonance

	@enable_rt_knob_resonance.setter
	def enable_rt_knob_resonance(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_resonance = value
			self.notify_listeners(0x1B,0x1C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_resonance_display(self):
		'''Tk16 Enable Realtime Control-RESONANCE display value'''
		return off_on_dict[self._enable_rt_knob_resonance]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_eg_int(self):
		'''Tk16 Enable Realtime Control-EG INT'''
		return self._enable_rt_knob_eg_int

	@enable_rt_knob_eg_int.setter
	def enable_rt_knob_eg_int(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_eg_int = value
			self.notify_listeners(0x1B,0x1D,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_eg_int_display(self):
		'''Tk16 Enable Realtime Control-EG INT display value'''
		return off_on_dict[self._enable_rt_knob_eg_int]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_release(self):
		'''Tk16 Enable Realtime Control-RELEASE'''
		return self._enable_rt_knob_release

	@enable_rt_knob_release.setter
	def enable_rt_knob_release(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_release = value
			self.notify_listeners(0x1B,0x1E,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_release_display(self):
		'''Tk16 Enable Realtime Control-RELEASE display value'''
		return off_on_dict[self._enable_rt_knob_release]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user1(self):
		'''Tk16 Enable Realtime Control-USER[1]'''
		return self._enable_rt_knob_user1

	@enable_rt_knob_user1.setter
	def enable_rt_knob_user1(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user1 = value
			self.notify_listeners(0x1B,0x1F,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user1_display(self):
		'''Tk16 Enable Realtime Control-USER[1] display value'''
		return off_on_dict[self._enable_rt_knob_user1]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user2(self):
		'''Tk16 Enable Realtime Control-USER[2]'''
		return self._enable_rt_knob_user2

	@enable_rt_knob_user2.setter
	def enable_rt_knob_user2(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user2 = value
			self.notify_listeners(0x1B,0x20,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user2_display(self):
		'''Tk16 Enable Realtime Control-USER[2] display value'''
		return off_on_dict[self._enable_rt_knob_user2]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user3(self):
		'''Tk16 Enable Realtime Control-USER[3]'''
		return self._enable_rt_knob_user3

	@enable_rt_knob_user3.setter
	def enable_rt_knob_user3(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user3 = value
			self.notify_listeners(0x1B,0x21,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user3_display(self):
		'''Tk16 Enable Realtime Control-USER[3] display value'''
		return off_on_dict[self._enable_rt_knob_user3]

#-------------------------------------------------------------------------------
	@property
	def enable_rt_knob_user4(self):
		'''Tk16 Enable Realtime Control-USER[4]'''
		return self._enable_rt_knob_user4

	@enable_rt_knob_user4.setter
	def enable_rt_knob_user4(self,value):
		if 0 <= value <= 1:
			self._enable_rt_knob_user4 = value
			self.notify_listeners(0x1B,0x22,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_rt_knob_user4_display(self):
		'''Tk16 Enable Realtime Control-USER[4] display value'''
		return off_on_dict[self._enable_rt_knob_user4]

#-------------------------------------------------------------------------------
	@property
	def enable_sw1(self):
		'''Tk16 Enable SW1'''
		return self._enable_sw1

	@enable_sw1.setter
	def enable_sw1(self,value):
		if 0 <= value <= 1:
			self._enable_sw1 = value
			self.notify_listeners(0x1B,0x23,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw1_display(self):
		'''Tk16 Enable SW1 display value'''
		return off_on_dict[self._enable_sw1]

#-------------------------------------------------------------------------------
	@property
	def enable_sw2(self):
		'''Tk16 Enable SW2'''
		return self._enable_sw2

	@enable_sw2.setter
	def enable_sw2(self,value):
		if 0 <= value <= 1:
			self._enable_sw2 = value
			self.notify_listeners(0x1B,0x24,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_sw2_display(self):
		'''Tk16 Enable SW2 display value'''
		return off_on_dict[self._enable_sw2]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_switch(self):
		'''Tk16 Enable Foot Switch'''
		return self._enable_foot_switch

	@enable_foot_switch.setter
	def enable_foot_switch(self,value):
		if 0 <= value <= 1:
			self._enable_foot_switch = value
			self.notify_listeners(0x1B,0x25,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_switch_display(self):
		'''Tk16 Enable Foot Switch display value'''
		return off_on_dict[self._enable_foot_switch]

#-------------------------------------------------------------------------------
	@property
	def enable_foot_pedal(self):
		'''Tk16 Enable Foot Pedal'''
		return self._enable_foot_pedal

	@enable_foot_pedal.setter
	def enable_foot_pedal(self,value):
		if 0 <= value <= 1:
			self._enable_foot_pedal = value
			self.notify_listeners(0x1B,0x26,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_foot_pedal_display(self):
		'''Tk16 Enable Foot Pedal display value'''
		return off_on_dict[self._enable_foot_pedal]

#-------------------------------------------------------------------------------
	@property
	def enable_other_control_change(self):
		'''Tk16 Enable Other Control Change'''
		return self._enable_other_control_change

	@enable_other_control_change.setter
	def enable_other_control_change(self,value):
		if 0 <= value <= 1:
			self._enable_other_control_change = value
			self.notify_listeners(0x1B,0x27,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_other_control_change_display(self):
		'''Tk16 Enable Other Control Change display value'''
		return off_on_dict[self._enable_other_control_change]

#-------------------------------------------------------------------------------
	@property
	def enable_use_prg_scales(self):
		'''Tk16 Use Program's Scale'''
		return self._enable_use_prg_scales

	@enable_use_prg_scales.setter
	def enable_use_prg_scales(self,value):
		if 0 <= value <= 1:
			self._enable_use_prg_scales = value
			self.notify_listeners(0x1B,0x12,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_use_prg_scales_display(self):
		'''Tk16 Use Program's Scale display value'''
		return off_on_dict[self._enable_use_prg_scales]

#-------------------------------------------------------------------------------
	@property
	def enable_auto_load_prg_eq(self):
		'''Tk16 Auto Load Program EQ'''
		return self._enable_auto_load_prg_eq

	@enable_auto_load_prg_eq.setter
	def enable_auto_load_prg_eq(self,value):
		if 0 <= value <= 1:
			self._enable_auto_load_prg_eq = value
			self.notify_listeners(0x1B,0x2A,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_auto_load_prg_eq_display(self):
		'''Tk16 Auto Load Program EQ display value'''
		return off_on_dict[self._enable_auto_load_prg_eq]

#-------------------------------------------------------------------------------
	@property
	def enable_timbre_mute(self):
		'''Tk16 Enable Timbre Mute'''
		return self._enable_timbre_mute

	@enable_timbre_mute.setter
	def enable_timbre_mute(self,value):
		if 0 <= value <= 1:
			self._enable_timbre_mute = value
			self.notify_listeners(0x1B,0x28,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_timbre_mute_display(self):
		'''Tk16 Enable Timbre Mute display value'''
		return off_on_dict[self._enable_timbre_mute]

#-------------------------------------------------------------------------------
	@property
	def force_osc_mode(self):
		'''Tk16 Force OSC Mode'''
		return self._force_osc_mode

	@force_osc_mode.setter
	def force_osc_mode(self,value):
		if 0 <= value <= 3:
			self._force_osc_mode = value
			self.notify_listeners(0x1B,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

	@property
	def force_osc_mode_display(self):
		'''Tk16 Force OSC Mode display value'''
		return force_osc_mode_dict[self._force_osc_mode]

#-------------------------------------------------------------------------------
	@property
	def osc_select(self):
		'''Tk16 OSC Select'''
		return self._osc_select

	@osc_select.setter
	def osc_select(self,value):
		if 0 <= value <= 2:
			self._osc_select = value
			self.notify_listeners(0x1B,0x8,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def osc_select_display(self):
		'''Tk16 OSC Select display value'''
		return osc_select_dict[self._osc_select]

#-------------------------------------------------------------------------------
	@property
	def arpeggiator_assign(self):
		'''Tk16 Arpeggiator Assign'''
		return self._arpeggiator_assign

	@arpeggiator_assign.setter
	def arpeggiator_assign(self,value):
		if 0 <= value <= 2:
			self._arpeggiator_assign = value
			self.notify_listeners(0x1B,0x9,value)
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def arpeggiator_assign_display(self):
		'''Tk16 Arpeggiator Assign display value'''
		return arpeggiator_assign_dict[self._arpeggiator_assign]

#-------------------------------------------------------------------------------
	@property
	def enable_waveform_sysex(self):
		'''Tk16 Enable Waveform SysEx'''
		return self._enable_waveform_sysex

	@enable_waveform_sysex.setter
	def enable_waveform_sysex(self,value):
		if 0 <= value <= 1:
			self._enable_waveform_sysex = value
			self.notify_listeners(0x1B,0x42,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def enable_waveform_sysex_display(self):
		'''Tk16 Enable Waveform SysEx display value'''
		return off_on_dict[self._enable_waveform_sysex]

#-------------------------------------------------------------------------------
	@property
	def portamento_time(self):
		'''Tk16 Portamento Time'''
		return self._portamento_time

	@portamento_time.setter
	def portamento_time(self,value):
		if -1 <= value <= 127:
			self._portamento_time = value
			self.notify_listeners(0x1B,0xA,value)
		else:
			raise ValueError('%s not -1 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_key(self):
		'''Tk16 Top Key'''
		return self._top_key

	@top_key.setter
	def top_key(self,value):
		if 0 <= value <= 127:
			self._top_key = value
			self.notify_listeners(0x1B,0x31,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_key(self):
		'''Tk16 Bottom Key'''
		return self._bottom_key

	@bottom_key.setter
	def bottom_key(self,value):
		if 0 <= value <= 127:
			self._bottom_key = value
			self.notify_listeners(0x1B,0x34,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_slope(self):
		'''Tk16 Top Slope'''
		return self._top_slope

	@top_slope.setter
	def top_slope(self,value):
		if 0 <= value <= 15:
			self._top_slope = value
			self.notify_listeners(0x1B,0x32,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_slope(self):
		'''Tk16 Bottom Slope'''
		return self._bottom_slope

	@bottom_slope.setter
	def bottom_slope(self,value):
		if 0 <= value <= 15:
			self._bottom_slope = value
			self.notify_listeners(0x1B,0x33,value)
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity(self):
		'''Tk16 Top Velocity'''
		return self._top_velocity

	@top_velocity.setter
	def top_velocity(self,value):
		if 0 <= value <= 127:
			self._top_velocity = value
			self.notify_listeners(0x1B,0x35,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity(self):
		'''Tk16 Bottom Velocity'''
		return self._bottom_velocity

	@bottom_velocity.setter
	def bottom_velocity(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity = value
			self.notify_listeners(0x1B,0x38,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def top_velocity_slope(self):
		'''Tk16 Top Velocity Slope'''
		return self._top_velocity_slope

	@top_velocity_slope.setter
	def top_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._top_velocity_slope = value
			self.notify_listeners(0x1B,0x36,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def bottom_velocity_slope(self):
		'''Tk16 Bottom Velocity Slope'''
		return self._bottom_velocity_slope

	@bottom_velocity_slope.setter
	def bottom_velocity_slope(self,value):
		if 0 <= value <= 127:
			self._bottom_velocity_slope = value
			self.notify_listeners(0x1B,0x37,value)
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_value(self):
		'''Tk16 Input Trim'''
		return self._trim_value

	@trim_value.setter
	def trim_value(self,value):
		if 0 <= value <= 99:
			self._trim_value = value
			self.notify_listeners(0x1B,0x2B,value)
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def trim_bypass(self):
		'''Tk16 Trim Bypass'''
		return self._trim_bypass

	@trim_bypass.setter
	def trim_bypass(self,value):
		if 0 <= value <= 1:
			self._trim_bypass = value
			self.notify_listeners(0x1B,0x2C,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def trim_bypass_display(self):
		'''Tk16 Trim Bypass display value'''
		return off_on_dict[self._trim_bypass]

#-------------------------------------------------------------------------------
	@property
	def mid_freq(self):
		'''Tk16 Mid Frequency'''
		return self._mid_freq

	@mid_freq.setter
	def mid_freq(self,value):
		if 0 <= value <= 165:
			self._mid_freq = value
			self.notify_listeners(0x1B,0x2D,value)
		else:
			raise ValueError('%s not 0 <= value <= 165' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Tk16 Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -36 <= value <= 36:
			self._low_gain = value
			self.notify_listeners(0x1B,0x2E,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Tk16 Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -36 <= value <= 36:
			self._mid_gain = value
			self.notify_listeners(0x1B,0x2F,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''Tk16 High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -36 <= value <= 36:
			self._high_gain = value
			self.notify_listeners(0x1B,0x30,value)
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_assign(self):
		'''Tk16 Tone Adjust Slider 1 Assign'''
		return self._tone_adj_slider1_assign

	@tone_adj_slider1_assign.setter
	def tone_adj_slider1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider1_assign = value
			self.notify_listeners(0x2B,0x18,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 1 Assign')

	@property
	def tone_adj_slider1_assign_display(self):
		'''Tk16 Tone Adjust Slider 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider1_value(self):
		'''Tk16 Tone Adjust Slider 1 Value'''
		return self._tone_adj_slider1_value

	@tone_adj_slider1_value.setter
	def tone_adj_slider1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider1_value = value
			self.notify_listeners(0x2B,0x19,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_assign(self):
		'''Tk16 Tone Adjust Slider 2 Assign'''
		return self._tone_adj_slider2_assign

	@tone_adj_slider2_assign.setter
	def tone_adj_slider2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider2_assign = value
			self.notify_listeners(0x2B,0x1A,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 2 Assign')

	@property
	def tone_adj_slider2_assign_display(self):
		'''Tk16 Tone Adjust Slider 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider2_value(self):
		'''Tk16 Tone Adjust Slider 2 Value'''
		return self._tone_adj_slider2_value

	@tone_adj_slider2_value.setter
	def tone_adj_slider2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider2_value = value
			self.notify_listeners(0x2B,0x1B,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_assign(self):
		'''Tk16 Tone Adjust Slider 3 Assign'''
		return self._tone_adj_slider3_assign

	@tone_adj_slider3_assign.setter
	def tone_adj_slider3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider3_assign = value
			self.notify_listeners(0x2B,0x1C,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 3 Assign')

	@property
	def tone_adj_slider3_assign_display(self):
		'''Tk16 Tone Adjust Slider 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider3_value(self):
		'''Tk16 Tone Adjust Slider 3 Value'''
		return self._tone_adj_slider3_value

	@tone_adj_slider3_value.setter
	def tone_adj_slider3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider3_value = value
			self.notify_listeners(0x2B,0x1D,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_assign(self):
		'''Tk16 Tone Adjust Slider 4 Assign'''
		return self._tone_adj_slider4_assign

	@tone_adj_slider4_assign.setter
	def tone_adj_slider4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider4_assign = value
			self.notify_listeners(0x2B,0x1E,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 4 Assign')

	@property
	def tone_adj_slider4_assign_display(self):
		'''Tk16 Tone Adjust Slider 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider4_value(self):
		'''Tk16 Tone Adjust Slider 4 Value'''
		return self._tone_adj_slider4_value

	@tone_adj_slider4_value.setter
	def tone_adj_slider4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider4_value = value
			self.notify_listeners(0x2B,0x1F,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_assign(self):
		'''Tk16 Tone Adjust Slider 5 Assign'''
		return self._tone_adj_slider5_assign

	@tone_adj_slider5_assign.setter
	def tone_adj_slider5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider5_assign = value
			self.notify_listeners(0x2B,0x20,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 5 Assign')

	@property
	def tone_adj_slider5_assign_display(self):
		'''Tk16 Tone Adjust Slider 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider5_value(self):
		'''Tk16 Tone Adjust Slider 5 Value'''
		return self._tone_adj_slider5_value

	@tone_adj_slider5_value.setter
	def tone_adj_slider5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider5_value = value
			self.notify_listeners(0x2B,0x21,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_assign(self):
		'''Tk16 Tone Adjust Slider 6 Assign'''
		return self._tone_adj_slider6_assign

	@tone_adj_slider6_assign.setter
	def tone_adj_slider6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider6_assign = value
			self.notify_listeners(0x2B,0x22,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 6 Assign')

	@property
	def tone_adj_slider6_assign_display(self):
		'''Tk16 Tone Adjust Slider 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider6_value(self):
		'''Tk16 Tone Adjust Slider 6 Value'''
		return self._tone_adj_slider6_value

	@tone_adj_slider6_value.setter
	def tone_adj_slider6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider6_value = value
			self.notify_listeners(0x2B,0x23,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_assign(self):
		'''Tk16 Tone Adjust Slider 7 Assign'''
		return self._tone_adj_slider7_assign

	@tone_adj_slider7_assign.setter
	def tone_adj_slider7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider7_assign = value
			self.notify_listeners(0x2B,0x24,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 7 Assign')

	@property
	def tone_adj_slider7_assign_display(self):
		'''Tk16 Tone Adjust Slider 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider7_value(self):
		'''Tk16 Tone Adjust Slider 7 Value'''
		return self._tone_adj_slider7_value

	@tone_adj_slider7_value.setter
	def tone_adj_slider7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider7_value = value
			self.notify_listeners(0x2B,0x25,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_assign(self):
		'''Tk16 Tone Adjust Slider 8 Assign'''
		return self._tone_adj_slider8_assign

	@tone_adj_slider8_assign.setter
	def tone_adj_slider8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_slider8_assign = value
			self.notify_listeners(0x2B,0x26,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Slider 8 Assign')

	@property
	def tone_adj_slider8_assign_display(self):
		'''Tk16 Tone Adjust Slider 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_slider8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_slider8_value(self):
		'''Tk16 Tone Adjust Slider 8 Value'''
		return self._tone_adj_slider8_value

	@tone_adj_slider8_value.setter
	def tone_adj_slider8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_slider8_value = value
			self.notify_listeners(0x2B,0x27,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_assign(self):
		'''Tk16 Tone Adjust Switch 1 Assign'''
		return self._tone_adj_switch1_assign

	@tone_adj_switch1_assign.setter
	def tone_adj_switch1_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch1_assign = value
			self.notify_listeners(0x2B,0x0,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 1 Assign')

	@property
	def tone_adj_switch1_assign_display(self):
		'''Tk16 Tone Adjust Switch 1 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch1_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_off_on(self):
		'''Tk16 Tone Adjust Switch 1 Off/On'''
		return self._tone_adj_switch1_off_on

	@tone_adj_switch1_off_on.setter
	def tone_adj_switch1_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch1_off_on = value
			self.notify_listeners(0x2B,0x1,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch1_off_on_display(self):
		'''Tk16 Tone Adjust Switch 1 Off/On display value'''
		return off_on_dict[self._tone_adj_switch1_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch1_value(self):
		'''Tk16 Tone Adjust Switch 1 Value'''
		return self._tone_adj_switch1_value

	@tone_adj_switch1_value.setter
	def tone_adj_switch1_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch1_value = value
			self.notify_listeners(0x2B,0x2,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_assign(self):
		'''Tk16 Tone Adjust Switch 2 Assign'''
		return self._tone_adj_switch2_assign

	@tone_adj_switch2_assign.setter
	def tone_adj_switch2_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch2_assign = value
			self.notify_listeners(0x2B,0x3,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 2 Assign')

	@property
	def tone_adj_switch2_assign_display(self):
		'''Tk16 Tone Adjust Switch 2 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch2_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_off_on(self):
		'''Tk16 Tone Adjust Switch 2 Off/On'''
		return self._tone_adj_switch2_off_on

	@tone_adj_switch2_off_on.setter
	def tone_adj_switch2_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch2_off_on = value
			self.notify_listeners(0x2B,0x4,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch2_off_on_display(self):
		'''Tk16 Tone Adjust Switch 2 Off/On display value'''
		return off_on_dict[self._tone_adj_switch2_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch2_value(self):
		'''Tk16 Tone Adjust Switch 2 Value'''
		return self._tone_adj_switch2_value

	@tone_adj_switch2_value.setter
	def tone_adj_switch2_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch2_value = value
			self.notify_listeners(0x2B,0x5,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_assign(self):
		'''Tk16 Tone Adjust Switch 3 Assign'''
		return self._tone_adj_switch3_assign

	@tone_adj_switch3_assign.setter
	def tone_adj_switch3_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch3_assign = value
			self.notify_listeners(0x2B,0x6,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 3 Assign')

	@property
	def tone_adj_switch3_assign_display(self):
		'''Tk16 Tone Adjust Switch 3 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch3_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_off_on(self):
		'''Tk16 Tone Adjust Switch 3 Off/On'''
		return self._tone_adj_switch3_off_on

	@tone_adj_switch3_off_on.setter
	def tone_adj_switch3_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch3_off_on = value
			self.notify_listeners(0x2B,0x7,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch3_off_on_display(self):
		'''Tk16 Tone Adjust Switch 3 Off/On display value'''
		return off_on_dict[self._tone_adj_switch3_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch3_value(self):
		'''Tk16 Tone Adjust Switch 3 Value'''
		return self._tone_adj_switch3_value

	@tone_adj_switch3_value.setter
	def tone_adj_switch3_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch3_value = value
			self.notify_listeners(0x2B,0x8,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_assign(self):
		'''Tk16 Tone Adjust Switch 4 Assign'''
		return self._tone_adj_switch4_assign

	@tone_adj_switch4_assign.setter
	def tone_adj_switch4_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch4_assign = value
			self.notify_listeners(0x2B,0x9,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 4 Assign')

	@property
	def tone_adj_switch4_assign_display(self):
		'''Tk16 Tone Adjust Switch 4 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch4_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_off_on(self):
		'''Tk16 Tone Adjust Switch 4 Off/On'''
		return self._tone_adj_switch4_off_on

	@tone_adj_switch4_off_on.setter
	def tone_adj_switch4_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch4_off_on = value
			self.notify_listeners(0x2B,0xA,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch4_off_on_display(self):
		'''Tk16 Tone Adjust Switch 4 Off/On display value'''
		return off_on_dict[self._tone_adj_switch4_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch4_value(self):
		'''Tk16 Tone Adjust Switch 4 Value'''
		return self._tone_adj_switch4_value

	@tone_adj_switch4_value.setter
	def tone_adj_switch4_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch4_value = value
			self.notify_listeners(0x2B,0xB,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_assign(self):
		'''Tk16 Tone Adjust Switch 5 Assign'''
		return self._tone_adj_switch5_assign

	@tone_adj_switch5_assign.setter
	def tone_adj_switch5_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch5_assign = value
			self.notify_listeners(0x2B,0xC,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 5 Assign')

	@property
	def tone_adj_switch5_assign_display(self):
		'''Tk16 Tone Adjust Switch 5 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch5_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_off_on(self):
		'''Tk16 Tone Adjust Switch 5 Off/On'''
		return self._tone_adj_switch5_off_on

	@tone_adj_switch5_off_on.setter
	def tone_adj_switch5_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch5_off_on = value
			self.notify_listeners(0x2B,0xD,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch5_off_on_display(self):
		'''Tk16 Tone Adjust Switch 5 Off/On display value'''
		return off_on_dict[self._tone_adj_switch5_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch5_value(self):
		'''Tk16 Tone Adjust Switch 5 Value'''
		return self._tone_adj_switch5_value

	@tone_adj_switch5_value.setter
	def tone_adj_switch5_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch5_value = value
			self.notify_listeners(0x2B,0xE,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_assign(self):
		'''Tk16 Tone Adjust Switch 6 Assign'''
		return self._tone_adj_switch6_assign

	@tone_adj_switch6_assign.setter
	def tone_adj_switch6_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch6_assign = value
			self.notify_listeners(0x2B,0xF,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 6 Assign')

	@property
	def tone_adj_switch6_assign_display(self):
		'''Tk16 Tone Adjust Switch 6 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch6_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_off_on(self):
		'''Tk16 Tone Adjust Switch 6 Off/On'''
		return self._tone_adj_switch6_off_on

	@tone_adj_switch6_off_on.setter
	def tone_adj_switch6_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch6_off_on = value
			self.notify_listeners(0x2B,0x10,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch6_off_on_display(self):
		'''Tk16 Tone Adjust Switch 6 Off/On display value'''
		return off_on_dict[self._tone_adj_switch6_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch6_value(self):
		'''Tk16 Tone Adjust Switch 6 Value'''
		return self._tone_adj_switch6_value

	@tone_adj_switch6_value.setter
	def tone_adj_switch6_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch6_value = value
			self.notify_listeners(0x2B,0x11,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_assign(self):
		'''Tk16 Tone Adjust Switch 7 Assign'''
		return self._tone_adj_switch7_assign

	@tone_adj_switch7_assign.setter
	def tone_adj_switch7_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch7_assign = value
			self.notify_listeners(0x2B,0x12,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 7 Assign')

	@property
	def tone_adj_switch7_assign_display(self):
		'''Tk16 Tone Adjust Switch 7 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch7_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_off_on(self):
		'''Tk16 Tone Adjust Switch 7 Off/On'''
		return self._tone_adj_switch7_off_on

	@tone_adj_switch7_off_on.setter
	def tone_adj_switch7_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch7_off_on = value
			self.notify_listeners(0x2B,0x13,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch7_off_on_display(self):
		'''Tk16 Tone Adjust Switch 7 Off/On display value'''
		return off_on_dict[self._tone_adj_switch7_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch7_value(self):
		'''Tk16 Tone Adjust Switch 7 Value'''
		return self._tone_adj_switch7_value

	@tone_adj_switch7_value.setter
	def tone_adj_switch7_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch7_value = value
			self.notify_listeners(0x2B,0x14,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_assign(self):
		'''Tk16 Tone Adjust Switch 8 Assign'''
		return self._tone_adj_switch8_assign

	@tone_adj_switch8_assign.setter
	def tone_adj_switch8_assign(self,value):
		if value in tone_adjust_s2p:
			self._tone_adj_switch8_assign = value
			self.notify_listeners(0x2B,0x15,tone_adjust_s2p[value])
		else:
			raise ValueError('%s not valid for Tk16 Tone Adjust Switch 8 Assign')

	@property
	def tone_adj_switch8_assign_display(self):
		'''Tk16 Tone Adjust Switch 8 Assign display value'''
		return tone_adjust_dict_s[self._tone_adj_switch8_assign]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_off_on(self):
		'''Tk16 Tone Adjust Switch 8 Off/On'''
		return self._tone_adj_switch8_off_on

	@tone_adj_switch8_off_on.setter
	def tone_adj_switch8_off_on(self,value):
		if 0 <= value <= 1:
			self._tone_adj_switch8_off_on = value
			self.notify_listeners(0x2B,0x16,value)
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def tone_adj_switch8_off_on_display(self):
		'''Tk16 Tone Adjust Switch 8 Off/On display value'''
		return off_on_dict[self._tone_adj_switch8_off_on]

#-------------------------------------------------------------------------------
	@property
	def tone_adj_switch8_value(self):
		'''Tk16 Tone Adjust Switch 8 Value'''
		return self._tone_adj_switch8_value

	@tone_adj_switch8_value.setter
	def tone_adj_switch8_value(self,value):
		if -16383 <= value <= 16382:
			self._tone_adj_switch8_value = value
			self.notify_listeners(0x2B,0x17,value)
		else:
			raise ValueError('%s not -16383 <= value <= 16382' % value)

